//	-------------------------------------------------------------------------
//
//	BSP.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is a program to display BSPLINE-based map generation
//					techniques.
//
//	-------------------------------------------------------------------------

// include program definition

#include "bsp.hpp"
#include "graphics.hpp"
#include "palettes.hpp"
#include "palfile.hpp"

// here #include all relevant bits and pieces!!!

#include "bspdefn.hpp"		// program-specific definitions

#include "proc.hpp"			// processor object
#include "options.hpp"		// program options window
#include "info.hpp"			// information window
#include "view.hpp"			// viewing & manipulation window
#include "d3.hpp"			// 3-d representation display window

#include "buttons.hpp"

//	-------------------------------------------------------------------------
//
//	FUNC:		BSP::Constructor()
//	DESC:		Allocates memory for map storage & since the program only
//				uses a single palette, sets this also.
//
//	-------------------------------------------------------------------------
BSP::BSP(char* init_string,int options)
	: Program("Bezier Spline Generation",init_string,options)
	{
	DBG(460,"START OF BSP::CONSTRUCTOR");

	map = grCreateVirtual(0,0,grMaxx,grMaxy,true);
	map3D = grCreateVirtual(0,0,grMaxx,grMaxy,true);

	orig_pal = grCreatePalette(256);
	grGetPalette(orig_pal,0,255);
/*
	grPalette* tmp_pal = grCreatePalette(256);
	grGetPalette(tmp_pal,0,255);
	int c;
	for (c=0;c<16;c++)
		{
		grSetPaletteEntry(tmp_pal,c+16,	 c*4,c*4, c*4);
		grSetPaletteEntry(tmp_pal,c+32,	 c*4,0,	  0);
		grSetPaletteEntry(tmp_pal,c+48,	 0,	 c*4, 0);
		grSetPaletteEntry(tmp_pal,c+64,	 0,	 0,	  c*4);
		grSetPaletteEntry(tmp_pal,c+80,	 0,	 c*4, c*4);
		grSetPaletteEntry(tmp_pal,c+96,	 c*4,0,	  c*4);
		grSetPaletteEntry(tmp_pal,c+112, c*4,c*4, 0);
		grSetPaletteEntry(tmp_pal,c+128, c*4,c*1, c*1);
		grSetPaletteEntry(tmp_pal,c+144, c*1,c*4, c*1);
		grSetPaletteEntry(tmp_pal,c+160, c*1,c*1, c*4);
		grSetPaletteEntry(tmp_pal,c+176, c*4,c*2, c*2);
		grSetPaletteEntry(tmp_pal,c+208, c*2,c*4, c*2);
		grSetPaletteEntry(tmp_pal,c+224, c*2,c*2, c*4);
		grSetPaletteEntry(tmp_pal,c+240, c*4,c*2, c*1);
		}

	for (c=0;c<16;c++)
		{
		grSetPaletteEntry(tmp_pal,c+32,0,c*2,0);
		grSetPaletteEntry(tmp_pal,c+48,0,c*2+16,0);
		grSetPaletteEntry(tmp_pal,c+64,c*4,c*2,c*1);
		grSetPaletteEntry(tmp_pal,c+80,c*3+12,c*3+12,c*3+12);
		grSetPaletteEntry(tmp_pal,c+96,16-(c*3+12),16-(c*3+12),16-(c*3+12));
		}
	grSetPalette(tmp_pal,0,255);

	grDestroyPalette(tmp_pal);
*/
	// setup anything required _only_ for the mainloop
	// ie. palettes, fonts, resourcefiles, etc.
	PALFile* palfile = new PALFile("RESOURCE\\MAINLOOP.PAL");
	grPalette* pal = palfile->LoadPalette();
	if (pal) grSetPalette(pal,0,255);

	grDestroyPalette(pal);
	delete palfile;

	DBG(470,"END OF BSP::CONSTRUCTOR");
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		BSP::Destructor()
//	DESC:		Cleans up after BSP::Constructor()
//
//	-------------------------------------------------------------------------
BSP::~BSP()
	{
	grSetPalette(orig_pal,0,255);
	grDestroyPalette(orig_pal);

	grDestroyVirtual(map3D);
	grDestroyVirtual(map);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		MainLoop()
//	DESC:       This is the main loop of the program.
//
//	-------------------------------------------------------------------------
void BSP::MainLoop()
	{
	// set up our timer to work at 100 times a second
	timer_ChangeInterruptRate(timer_100Hz);

	// create all objects used in the program
	// NOTE:: for optimum use of this library, objects should be created
	//		  such that the first handler is the most used, the last the
	//		  least used....

	DBG(470,"Creating EVENT HANDLERS"); delay(200);

	EVENTMGR->AddEventHandler(new Processor());
	EVENTMGR->AddEventHandler(new OptionsWindow());
	EVENTMGR->AddEventHandler(new InfoWindow());
	EVENTMGR->AddEventHandler(new ViewWindow());
	EVENTMGR->AddEventHandler(new D3Window());

	// clear the screen
	grClearScreen(grBLACK);

	// draw all the relevant handlers
	EVENTMGR->DrawAllHandlers();

	// tell the event manager to execute using the handlers!!!
	EVENTMGR->Execute();

	// delete the objects now that we have finished with them
	EVENTMGR->DelAllEventHandlers();
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		main()
//	DESC:		This is the function called at program execution.
//				At the moment it simply creates the program, executes it,
//				then deletes it, exiting normally
//
//	-------------------------------------------------------------------------
void main(int argc,char** argv)
	{
	// create our program
	BSP* theProgram = new BSP(argv[0],prg_InstallGUI|prg_InstallSystemFont);

	// run it's intro, then setup, then mainloop, then finale
	theProgram->Execute();

	// and clear up after ourselves
	delete theProgram;
	}
