//	-------------------------------------------------------------------------
//
//	PROC.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the Processor.
//
//	-------------------------------------------------------------------------

#include "proc.hpp"
#include "eventmgr.hpp"
#include "stddefns.hpp"

#include "mouse.hpp"
#include "keyboard.hpp"

#include "bspdefn.hpp"


#include "stdio.h"
#include "conio.h"

Processor::Processor()
	: BackgroundTask(&evhno_Processor,0)
	{
	}

#define grVertical		1
#define grHorizontal	2

grPoint ControlPoints[40005];	// heaps of room + small error margin??

void grDrawBSpline3D(grVirtualBuffer* virt,grPoint* ControlPoints,
					  int max_width,int num_ctrl_pts,
					  int pos,int mode)

// draws a bspline into a virtual buffer in either _vertical_ or _horizontal_
// mode, assuming that the calculated values are stored in the z-plane.

// note: this is done using a 2-D spline curve projected onto a 3D map.
//
// params:	max_width 	- the maximum spline length
//			num_ctrl_pts- the number of control points along the length
// 			pos 		- the x or y position to store the data
//			mode		- iff "vertical", will use 'pos' to store the
//						  data going down the map.
//						  iff "horizontal", will use 'pos' to store the
//						  data going left-to-right on the map.
//

	{
//	--- draw the Bezier-Spline-Curve

	float n = max_width / num_ctrl_pts;
	float i;
	float t,delta;
	float OneMinusT,OneMinusTSquared,OneMinusTCubed;
	float TSquared,TCubed,Val1,Val2;

	delta = 1.0 / n;

	float x = ControlPoints[0].x;
	float y = ControlPoints[0].y;

	int k = 0;
	for (int m=3;m<num_ctrl_pts;m++)
		{
		grPoint Pminus3 = ControlPoints[k];
		grPoint Pminus2 = ControlPoints[k+1];
		grPoint Pminus1 = ControlPoints[k+2];
		grPoint P = ControlPoints[k+3];
		k++;

		for (i=1;i<=n;i++)
			{
			t = i * delta;

			TSquared  = (t * t);
			TCubed	  = (TSquared * t);

			OneMinusT        = (1.0 - t);
			OneMinusTSquared = (OneMinusT * OneMinusT);
			OneMinusTCubed   = (OneMinusTSquared * OneMinusT);
			Val1			 = (3*TCubed - 6*TSquared+4);
			Val2			 = (-3*TCubed + 3*TSquared + 3*t + 1);

			x = (Pminus3.x * OneMinusTCubed
				  + Pminus2.x * Val1
				  + Pminus1.x * Val2
				  + P.x * TCubed) / 6;

			y = (Pminus3.y * OneMinusTCubed
				  + Pminus2.y * Val1
				  + Pminus1.y * Val2
				  + P.y * TCubed) / 6;


			if (virt)
				{
				if (mode==grVertical)
					grPutPixelVirtual(virt,pos,x,y);
				else
					grPutPixelVirtual(virt,x,pos,y);
				}
			}
		}
	}
/*
void Processor::GenerateBaseTerrain(grVirtualBuffer* map,
									int map_width,int map_height,
									int xdist,int ydist)

// Generates base terrain onto the map....
// ie. areas containing plains, grasslands, deserts & swamps

	{
	}

void Processor::GenerateOverlaidTerrain(grVirtualBuffer* map,
										int map_width,int map_height,
										int xdist,int ydist)


// generates other terrain onto the map....
// ie. woods, forests, shrub, mangroves, hills & mountains
	{
	}
*/
void Processor::Generate3DMap()
	{
	int ctrl_pt_width = d3_width / ctrl_pts_x;
	int ctrl_pt_height = d3_height / ctrl_pts_y;

	int x,y,z,i,j;

	if (map)
		{
		grClearVirtual(map,grBLACK);
		grClearVirtual(map3D,grBLACK);

		// create a grid of b-spline control points to represent a
		// large area of plains/grasslands/etc. (with random height)
		int index;
		for (x=0;x<ctrl_pts_x;x++)
			{
			index = x * ctrl_pts_y;
			for (y=0;y<ctrl_pts_y;y++)
				{
				ControlPoints[index+y].x = y * ctrl_pt_height;
				ControlPoints[index+y].y = random(plains_height) + 48;	// +48 color offset
																		// also below
				}
			}

		// for each of the mountain ranges
		for (i=0;i<num_mountain_ranges;i++)
			{
			DBG(470,"CREATING RANDOM MOUNTAIN RANGE %d",i);

			// choose a random starting location (control point)
			x = random(ctrl_pts_x);
			y = random(ctrl_pts_y);

			// figure out an approximate height for the mountain
			int last_height = 48 + plains_height + random(mountains_height);

			// for each element of the mountain range
			for (j=0;j<mountain_range_size;j++)
				{
				// choose a random point nearby.... (+-1 for both x,y)
				int tmpx=x,tmpy=y;
				do
					{
					x += random(3) - 1;
					y += random(3) - 1;
					}
				while ((x==tmpx)&&(y=tmpy));

				// bounds-check (not perfect but close...)
				if (x<0) x=0;
				if (y<0) y=0;
				if (x>=ctrl_pts_x) x = ctrl_pts_x-1;
				if (y>=ctrl_pts_y) y = ctrl_pts_y-1;

				// and modify ctrl pts around it...
				ControlPoints[x*ctrl_pts_y+y].x = y * ctrl_pt_height;
				ControlPoints[x*ctrl_pts_y+y].y = last_height + random(plains_height+1) - plains_height/2;

				// save the last generated height
				last_height = ControlPoints[x*ctrl_pts_y+y].y;
				}
			}

		// draw a number of splines from the top to the bottom of the map
		// at fixed intervals.
		for (x=0;x<ctrl_pts_x;x++)
			{
			DBG(470,"Creating vertical bspline at  x = %d",x);
			grDrawBSpline3D(map,&ControlPoints[x*ctrl_pts_y],d3_height,ctrl_pts_y,x*ctrl_pt_width,grVertical);

			if (x%5==0)
				{
				grPutImage(d3_left,d3_top,d3_right,d3_bottom,map);
				grBlitRegion(d3_left,d3_top,d3_right,d3_bottom);
				}

			// skip-out on keypress as this can take a while.....
			if (keyboard_IsHit()) { keyboard_Flush(); return; }
			}

		// for each row of the map, use the control points (created above)
		// to draw a bspline....
		for (y=0;y<d3_height;y++)
			{
			DBG(470,"Creating horizontal bspline at  y = %d",y);
			for (x=0;x<ctrl_pts_x;x++)
				{
				ControlPoints[x].x = x * ctrl_pt_width;
				ControlPoints[x].y = grGetPixelVirtual(map,x*ctrl_pt_width,y);
				}

			grDrawBSpline3D(map,ControlPoints,d3_width,ctrl_pts_x,y,grHorizontal);

			if (y%20==0)
				{
				grPutImage(d3_left,d3_top,d3_right,d3_bottom,map);
				grBlitRegion(d3_left,d3_top,d3_right,d3_bottom);
				}

			if (keyboard_IsHit()) { keyboard_Flush(); return; }
			}

		// use 'painters algorithm' for _reasonable_ hidden surface removal
		for (y=0;y<d3_height;y++)
			{
			DBG(470,"Drawing 3D representation row # %d",y);
			for (x=0;x<d3_width;x++)
				{
				z = grGetPixelVirtual(map,x,y) - 48;
				grPutPixelVirtual(map3D,x+(d3_bottom-y)/2,d3_top+y-(z/2),z+48);
				}
			if (keyboard_IsHit()) { keyboard_Flush(); return; }
			}

		}
	}

void Processor::HandleEvent(Event &e)
	{
	switch (e.what)
		{
		case ev_command:	switch (e.command.event)
								{
								case cmdGenerate:	// generate the map
													Generate3DMap();
													// update the appropriate display window
													if (display_mode==evhno_3DWindow) EVENTMGR->GenCommandEvent(evhno_3DWindow,cmdRedraw,NULL);
													else EVENTMGR->GenCommandEvent(evhno_ViewWindow,cmdRedraw,NULL);
													// and clear the event
													EVENTMGR->ClearEvent(e);
													break;
								}
		}
	}
