//	-------------------------------------------------------------------------
//
//	OPTIONS.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the Options Window
//
//	-------------------------------------------------------------------------

#include "options.hpp"
#include "eventmgr.hpp"
#include "timer.hpp"
#include "keydefns.hpp"
#include "buttons.hpp"
#include "mouse.hpp"

#include "bspdefn.hpp"


#define cmdDisplayMessage		500000


OptionsWindow::OptionsWindow()
	: OnScreenHandler(d3_right+1,0,grMaxx,grMaxy,grDARKGRAY,bs_DBLRAISED,bs_NONE,
					&evhno_OptionsWindow,0,1)
	{
	int x = grSizeOfBorder[active_bs] + 4;
	int y = grSizeOfBorder[active_bs] + 4;
	int w = 25;

	// adjustment of number of vertical control points
	AddAccessory(new TextButton(x,y,      "Ver-",0,cmdDecVertPts));
	AddAccessory(new TextButton(x+w,y,    "Ver+",0,cmdIncVertPts));

	// adjustment of number of horizontal control points
	AddAccessory(new TextButton(x,y+20,   "Hor-",0,cmdDecHorizPts));
	AddAccessory(new TextButton(x+w,y+20, "Hor+",0,cmdIncHorizPts));

	// adjustment of heights
	AddAccessory(new TextButton(x,y+40,   "Pln-",0,cmdDecPlainsHeight));
	AddAccessory(new TextButton(x+w,y+40, "Pln+",0,cmdIncPlainsHeight));

	AddAccessory(new TextButton(x,y+60,   "Rol-",0,cmdDecRollingHeight));
	AddAccessory(new TextButton(x+w,y+60, "Rol+",0,cmdIncRollingHeight));

	AddAccessory(new TextButton(x,y+80,   "Hil-",0,cmdDecHillsHeight));
	AddAccessory(new TextButton(x+w,y+80, "Hil+",0,cmdIncHillsHeight));

	AddAccessory(new TextButton(x,y+100,  "Mtn-",0,cmdDecMaximumHeight));
	AddAccessory(new TextButton(x+w,y+100,"Mtn+",0,cmdIncMaximumHeight));

	// adjustment of number of ranges
	AddAccessory(new TextButton(x,y+120,  "Rolling -",0,cmdDecNumRolling));
	AddAccessory(new TextButton(x,y+140,  "Rolling +",0,cmdIncNumRolling));

	AddAccessory(new TextButton(x,y+160,  "Hills   -",0,cmdDecNumHills));
	AddAccessory(new TextButton(x,y+180,  "Hills   +",0,cmdIncNumHills));

	AddAccessory(new TextButton(x,y+200,  "Mountain-",0,cmdDecNumMountains));
	AddAccessory(new TextButton(x,y+220,  "Mountain+",0,cmdIncNumMountains));

	// adjustment of the sizes of each ranges
	AddAccessory(new TextButton(x,y+240,  "Roll Rng-",0,cmdDecRollingRangeSize));
	AddAccessory(new TextButton(x,y+260,  "Roll Rng+",0,cmdIncRollingRangeSize));

	AddAccessory(new TextButton(x,y+280,  "Hill Rng-",0,cmdDecHillRangeSize));
	AddAccessory(new TextButton(x,y+300,  "Hill Rng+",0,cmdIncHillRangeSize));

	AddAccessory(new TextButton(x,y+320,  "MountRng-",0,cmdDecMountainRangeSize));
	AddAccessory(new TextButton(x,y+340,  "MountRng+",0,cmdIncMountainRangeSize));

	// display options
	AddAccessory(new TextButton(x,y+380,"Show View",0,cmdShowView));
	AddAccessory(new TextButton(x,y+400,"Show 3d  ",0,cmdShow3D));

	// generation options
	AddAccessory(new TextButton(x,y+420,"GENERATE ",0,cmdGenerate));

	// quit button (refer HandleEvent() below....)
	AddAccessory(new TextButton(x,y+440,"Exit     ",0,cmdExitProgram));

	// create a simple test button!!!!
//	AddAccessory(new UpArrowButton(x,y+400,0,cmdDisplayMessage));
	}

void OptionsWindow::Draw()
	{
	OnScreenHandler::Draw();
//	grBlitRegion(r.left,r.top,r.right,r.bottom);
	}

void OptionsWindow::HandleEvent(Event &e)
	{
	// pre-process the mouse to set our increment
	int incr = 1;
	if (e.what==ev_mouse)
		{
		if (e.mouse.event==mb_rightup) incr = 5;
		if (e.mouse.event==mb_middleup) incr = 50;
		}

	// do all the _boring_ event processing
	OnScreenHandler::HandleEvent(e);

	// check the response from the buttons
	bool handled = false;
	if (e.what==ev_command)
		{
		switch (e.command.event)
			{
			// simple buttons
			case cmdIncVertPts:		ctrl_pts_y += incr;
									if (ctrl_pts_y>d3_height) ctrl_pts_y = d3_height;
									if (ctrl_pts_y>200) ctrl_pts_y = 200;
									handled = true;
									break;
			case cmdDecVertPts:		ctrl_pts_y -= incr;
									if (ctrl_pts_y<4) ctrl_pts_y = 4;
									handled = true;
									break;
			case cmdIncHorizPts:	ctrl_pts_x += incr;
									if (ctrl_pts_x>d3_width) ctrl_pts_x = d3_width;
									if (ctrl_pts_x>200) ctrl_pts_x = 200;
									handled = true;
									break;
			case cmdDecHorizPts:	ctrl_pts_x -= incr;
									if (ctrl_pts_x<4) ctrl_pts_x = 4;
									handled = true;
									break;
/*			case cmdDecNumLakes:
									min_rolling_height += incr;
									if (min_rolling_height>255) min_rolling_height = 255;
									handled = true;
									break;
			case cmdIncNumLakes:
									min_rolling_height -= incr;
									if (min_rolling_height<2) min_rolling_height = 2;
									handled = true;
									break;
			case cmdDecNumSw:
									min_hills_height += incr;
									if (min_hills_height>255) min_hills_height = 255;
									handled = true;
									break;
			case cmdDecRollingHeight:
									min_hills_height -= incr;
									if (min_hills_height<2) min_hills_height = 2;
									handled = true;
									break;
			case cmdIncHillsHeight:
									min_mountains_height += incr;
									if (min_mountains_height>255) min_mountains_height = 255;
									handled = true;
									break;
			case cmdDecHillsHeight:
									min_mountains_height -= incr;
									if (min_mountains_height<2) min_mountains_height = 2;
									handled = true;
									break;
			case cmdIncMaximumHeight:
									maximum_height += incr;
									if (maximum_height>255) maximum_height = 255;
									handled = true;
									break;
			case cmdDecMaximumHeight:
									maximum_height -= incr;
									if (maximum_height<2) maximum_height = 2;
									handled = true;
									break;
			case cmdIncNumRolling:
									num_rolling += incr;
									if (num_rolling>255) num_rolling = 255;
									handled = true;
									break;
			case cmdDecNumRolling:
									num_rolling -= incr;
									if (num_rolling<2) num_rolling = 2;
									handled = true;
									break;
*/			case cmdIncNumHills:
									num_hills += incr;
									if (num_hills>255) num_hills = 255;
									handled = true;
									break;
			case cmdDecNumHills :
									num_hills -= incr;
									if (num_hills<2) num_hills = 2;
									handled = true;
									break;
			case cmdIncNumMountains:
									num_mountains += incr;
									if (num_mountains>255) num_mountains = 255;
									handled = true;
									break;
			case cmdDecNumMountains:
									num_mountains -= incr;
									if (num_mountains<2) num_mountains = 2;
									handled = true;
									break;
/*			case cmdIncRollingRangeSize:
									rolling_spread += incr;
									if (rolling_spread>400) rolling_spread = 400;
									handled = true;
									break;
			case cmdDecRollingRangeSize:
									rolling_spread -= incr;
									if (rolling_spread<1) rolling_spread = 1;
									handled = true;
									break;
			case cmdIncHillRangeSize:
									hill_spread += incr;
									if (hill_spread>400) hill_spread = 400;
									handled = true;
									break;
			case cmdDecHillRangeSize:
									hill_spread -= incr;
									if (hill_spread<1) hill_spread = 1;
									handled = true;
									break;
			case cmdIncMountainRangeSize:
									mountain_spread += incr;
									if (mountain_spread>400) mountain_spread = 400;
									handled = true;
									break;
			case cmdDecMountainRangeSize:
									mountain_spread -= incr;
									if (mountain_spread<1) mountain_spread = 1;
									handled = true;
									break;
*/
			// display buttons
			case cmdShowView:		if (display_mode==evhno_3DWindow) EVENTMGR->GenCommandEvent(evhno_3DWindow,cmdClose,NULL);
									EVENTMGR->GenCommandEvent(evhno_ViewWindow,cmdOpen,NULL);
									display_mode = evhno_ViewWindow;
									EVENTMGR->GenCommandEvent(evhno_InfoWindow,cmdRedraw,NULL);
									EVENTMGR->ClearEvent(e);
									break;
			case cmdShow3D:			if (display_mode!=evhno_3DWindow) EVENTMGR->GenCommandEvent(evhno_ViewWindow,cmdClose,NULL);
									EVENTMGR->GenCommandEvent(evhno_3DWindow,cmdOpen,NULL);
									display_mode = evhno_3DWindow;
									EVENTMGR->GenCommandEvent(evhno_InfoWindow,cmdRedraw,NULL);
									EVENTMGR->ClearEvent(e);
									break;

			// and the generate button
			case cmdGenerate:		e.where = evhno_Processor;
									//EVENTMGR->ModEvent(e,evhno_Processor,cmdGenerate,NULL);
									return;

			case cmdDisplayMessage:
									DBG(470,"YOU JUST CLICKED THE TEST BUTTON");
									EVENTMGR->ClearEvent(e);
									break;

			// note: cmdExitProgram is handled internally, so nothing should
			//		 need to be done with it....
			//
			// note: the EVENTMGR will _IMMEDIATELY_ exit the currently
			//		 Execute()'ing function (ie. Intro,MainLoop,Finale) as
			//		 soon as it detects event:
			//
			//		 	[ev_command]   e.command.event == cmdExitProgram
			//
			}
		}

	if (handled)
		{
		EVENTMGR->GenCommandEvent(evhno_InfoWindow,cmdRefresh,NULL);
		EVENTMGR->ClearEvent(e);
		}

	if (e.what==ev_mouse)
		if (e.mouse.event==mb_leftup)
		{
		Refresh();
		EVENTMGR->ClearEvent(e);
		}
	}