//	-------------------------------------------------------------------------
//
//	PROC.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the Processor.
//
//	-------------------------------------------------------------------------

#include "proc.hpp"
#include "eventmgr.hpp"
#include "stddefns.hpp"

#include "mouse.hpp"
#include "keyboard.hpp"

#include "bspdefn.hpp"


#include "dos.h"
#include "stdio.h"
#include "conio.h"

#define __FIXED

// --------------------------------------------------
// implementing a fixed point number system
// --------------------------------------------------
#ifndef fp
#define fp long
#define FixedPointShiftAmount 10

#define FPToInteger(FP)		((FP) >> FixedPointShiftAmount)
#define IntegerToFP(Int)	((Int) << FixedPointShiftAmount)
fp FPMultFP(fp a,fp b) { return (fp)(((long)a * (long)b) >> FixedPointShiftAmount); }
fp FPDivFP(fp a,fp b) { return (fp)(((long)a << FixedPointShiftAmount) / (long)b); }
#endif
// --------------------------------------------------

Processor::Processor()
	: BackgroundTask(&evhno_Processor,0)
	{
	}

#define grVertical		1
#define grHorizontal	2

grPoint ControlPoints[40005];	// heaps of room + small error margin??

#ifdef __FIXED
//---------------------------------------------------------------------------
// draws a bspline into a virtual buffer in either _vertical_ or _horizontal_
// mode, assuming that the calculated values are stored in the z-plane.

// note: this is done using a 2-D spline curve projected onto a 3D map.
//
// params:	max_width 	- the maximum spline length
//			num_ctrl_pts- the number of control points along the length
// 			pos 		- the x or y position to store the data
//			mode		- iff "vertical", will use 'pos' to store the
//						  data going down the map.
//						  iff "horizontal", will use 'pos' to store the
//						  data going left-to-right on the map.
//
void grDrawBSpline3D(grVirtualBuffer* virt,grPoint* ControlPoints,
					  int max_width,int num_ctrl_pts,
					  int pos,int mode)
	{
//	--- draw the Bezier-Spline-Curve

	int i,k,m,x,y;

	fp fpt;
	fp fpTSquared,fpTCubed,fpOneMinusT,fpOneMinusTSquared,fpOneMinusTCubed;
	fp fpVal1,fpVal2;

	fp fpn = FPDivFP(IntegerToFP(max_width),IntegerToFP(num_ctrl_pts));
	fp fpdelta = FPDivFP(IntegerToFP(1),fpn);

	float n = max_width / num_ctrl_pts;
	float delta = 1.0 / n;

	k = 0;
	for (m=3;m<num_ctrl_pts;m++)
		{
		grPoint Pminus3 = ControlPoints[k];
		grPoint Pminus2 = ControlPoints[k+1];
		grPoint Pminus1 = ControlPoints[k+2];
		grPoint P = ControlPoints[k+3];
		k++;

		for (i=1;i<=n;i++)
//		for (i=1;i<=FPToInteger(fpn);i++)
			{
//			float t = i * delta;
//			fpt = IntegerToFP((int)(t*1000.0));
//			fpt /= 1000;
			fpt = i * fpdelta;

			fpTSquared  = FPMultFP(fpt,fpt);
			fpTCubed	= FPMultFP(fpTSquared,fpt);

			fpOneMinusT        = IntegerToFP(1) - fpt;
			fpOneMinusTSquared = FPMultFP(fpOneMinusT,fpOneMinusT);
			fpOneMinusTCubed   = FPMultFP(fpOneMinusTSquared,fpOneMinusT);

			fpVal1			 = 3*fpTCubed +IntegerToFP(4) -6*fpTSquared ;
			fpVal2			 = 3*fpTSquared +3*fpt +IntegerToFP(1) -3*fpTCubed;

			fp fp1 = Pminus3.x*fpOneMinusTCubed;
			fp fp2 = Pminus2.x*fpVal1;
			fp fp3 = Pminus1.x*fpVal2;
			fp fp4 = P.x*fpTCubed;

			fp fp5 = (fp1/6) + (fp2/6) + (fp3/6) + (fp4/6);

			x = FPToInteger(fp5);

/*			x = FPToInteger((
					Pminus3.x*fpOneMinusTCubed +
					Pminus2.x*fpVal1 +
					Pminus1.x*fpVal2 +
					P.x*fpTCubed)/6);


			y = FPToInteger((
					Pminus3.y*fpOneMinusTCubed +
					Pminus2.y*fpVal1 +
					Pminus1.y*fpVal2 +
					P.y*fpTCubed)/6);
*/

			fp1 = Pminus3.y*fpOneMinusTCubed;
			fp2 = Pminus2.y*fpVal1;
			fp3 = Pminus1.y*fpVal2;
			fp4 = P.y*fpTCubed;

			fp5 = (fp1/6) + (fp2/6) + (fp3/6) + (fp4/6);

			y = FPToInteger(fp5);

			if (mode==grVertical)	grPutPixelVirtual(virt,pos,x,y);
			else					grPutPixelVirtual(virt,x,pos,y);
			}
		}
	}

#else

//---------------------------------------------------------------------------
// draws a bspline into a virtual buffer in either _vertical_ or _horizontal_
// mode, assuming that the calculated values are stored in the z-plane.

// note: this is done using a 2-D spline curve projected onto a 3D map.
//
// params:	max_width 	- the maximum spline length
//			num_ctrl_pts- the number of control points along the length
// 			pos 		- the x or y position to store the data
//			mode		- iff "vertical", will use 'pos' to store the
//						  data going down the map.
//						  iff "horizontal", will use 'pos' to store the
//						  data going left-to-right on the map.
//
void grDrawBSpline3D(grVirtualBuffer* virt,grPoint* ControlPoints,
					  int max_width,int num_ctrl_pts,
					  int pos,int mode)
	{
	if (!virt) return;

//	--- draw the Bezier-Spline-Curve

	float n = max_width / num_ctrl_pts;
	float i;
	float t,delta;
	float OneMinusT,OneMinusTSquared,OneMinusTCubed;
	float TSquared,TCubed,Val1,Val2;

	delta = 1.0 / n;

	float x = ControlPoints[0].x;
	float y = ControlPoints[0].y;

	int k = 0;
	for (int m=3;m<num_ctrl_pts;m++)
		{
		grPoint Pminus3 = ControlPoints[k];
		grPoint Pminus2 = ControlPoints[k+1];
		grPoint Pminus1 = ControlPoints[k+2];
		grPoint P = ControlPoints[k+3];
		k++;

		for (i=1;i<=n;i++)
			{
			t = i * delta;

			TSquared  = (t * t);
			TCubed	  = (TSquared * t);

			OneMinusT        = (1.0 - t);
			OneMinusTSquared = (OneMinusT * OneMinusT);
			OneMinusTCubed   = (OneMinusTSquared * OneMinusT);
			Val1			 = (3*TCubed - 6*TSquared+4);
			Val2			 = (-3*TCubed + 3*TSquared + 3*t + 1);

			x = (Pminus3.x * OneMinusTCubed
				  + Pminus2.x * Val1
				  + Pminus1.x * Val2
				  + P.x * TCubed) / 6;

			y = (Pminus3.y * OneMinusTCubed
				  + Pminus2.y * Val1
				  + Pminus1.y * Val2
				  + P.y * TCubed) / 6;


			if (mode==grVertical) 	grPutPixelVirtual(virt,pos,x,y);
			else					grPutPixelVirtual(virt,x,pos,y);
			}
		}
	}
#endif

//---------------------------------------------------------------------------
void Processor::Generate3DMap()
	{
	int ctrl_pt_width = d3_width / ctrl_pts_x;
	int ctrl_pt_height = d3_height / ctrl_pts_y;

	int x,y,xx[MAX_RANGE_LENGTH],yy[MAX_RANGE_LENGTH],z,i,j;

	if (map)
		{
		grClearVirtual(map,grBLACK);
		grClearVirtual(map3D,grBLACK);

		// generate control points
		GenerateBase(grass_max-5);

		for (i=0;i<num_plains;i++)
			{
			x = random(ctrl_pts_x);
			y = random(ctrl_pts_y);
			GenerateBumps(x,y,spread_plains,variation_plains,grass_max,plains_max);
			}

		for (i=0;i<num_grass;i++)
			{
			x = random(ctrl_pts_x);
			y = random(ctrl_pts_y);
			GenerateBumps(x,y,spread_grass,variation_grass,swamp_max,grass_max);
			}

		for (i=0;i<num_swamp;i++)
			{
			x = random(ctrl_pts_x);
			y = random(ctrl_pts_y);
			GenerateBumps(x,y,spread_swamp,variation_swamp,sea_max,swamp_max);
			}

		for (i=0;i<num_hills;i++)
			{
			x = random(ctrl_pts_x);
			y = random(ctrl_pts_y);
			GenerateBumps(x,y,spread_hills,variation_hills,plains_max,hills_max);
			}

		for (i=0;i<num_mountains;i++)
			{
			x = random(ctrl_pts_x);
			y = random(ctrl_pts_y);
			int direction = random(8);

			for (j=0;j<range_length;j++)
				{
				if (j==0)
					{
					xx[j] = x;
					yy[j] = y;
					}
				else
					{
					switch (direction)
						{
						case 0:
							xx[j] = xx[j-1] + (random(3)-1)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] - random(3)*range_length/ctrl_pt_height;
							break;
						case 1:
							xx[j] = xx[j-1] + random(3)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] - random(3)*range_length/ctrl_pt_height;
							break;
						case 2:
							xx[j] = xx[j-1] + random(3)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] + (random(3)-1)*range_length/ctrl_pt_height;
							break;
						case 3:
							xx[j] = xx[j-1] + random(3)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] + random(3)*range_length/ctrl_pt_height;
							break;
						case 4:
							xx[j] = xx[j-1] + (random(3)-1)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] + random(3)*range_length/ctrl_pt_height;
							break;
						case 5:
							xx[j] = xx[j-1] - random(3)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] + random(3)*range_length/ctrl_pt_height;
							break;
						case 6:
							xx[j] = xx[j-1] - random(3)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] + (random(3)-1)*range_length/ctrl_pt_height;
							break;
						case 7:
							xx[j] = xx[j-1] - random(3)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] - random(3)*range_length/ctrl_pt_height;
							break;
						}
					}
				// for debugging
				grPutPixel(xx[j]*ctrl_pt_width,yy[j]*ctrl_pt_height,mountains_max);
				grBlitRegion(xx[j]*ctrl_pt_width,yy[j]*ctrl_pt_height,xx[j]*ctrl_pt_width,yy[j]*ctrl_pt_height);
				delay(50);

				GenerateBumps(xx[j],yy[j],spread_mountains,variation_mountains,hills_max,mountains_max);
				}
			}

		for (i=0;i<num_lakes;i++)
			{
			x = random(ctrl_pts_x);
			y = random(ctrl_pts_y);
			GenerateBumps(x,y,spread_lakes,variation_lakes,minimum_height,sea_max);
			}

		// draw a number of splines from the top to the bottom of the map
		// at fixed intervals.
		for (x=0;x<ctrl_pts_x;x++)
			{
			grDrawBSpline3D(map,&ControlPoints[x*ctrl_pts_y],d3_height,ctrl_pts_y,x*ctrl_pt_width,grVertical);

			if (x%5==0)
				{
			DBG(470,"Creating vertical bspline at  x = %d",x);
//				grPutImage(d3_left,d3_top,d3_left+x*ctrl_pt_width,d3_bottom,map);
//				grBlitRegion(d3_left,d3_top,d3_left+x*ctrl_pt_width,d3_bottom);
				}

			// skip-out on keypress as this can take a while.....
			if (keyboard_IsHit()) { keyboard_Flush(); return; }
			}

		// for each row of the map, use the control points (created above)
		// to draw a bspline....
		for (y=0;y<d3_height;y++)
			{
			for (x=0;x<ctrl_pts_x;x++)
				{
				ControlPoints[x].x = x * ctrl_pt_width;
				ControlPoints[x].y = grGetPixelVirtual(map,x*ctrl_pt_width,y);
				}

			grDrawBSpline3D(map,ControlPoints,d3_width,ctrl_pts_x,y,grHorizontal);

			if (y%20==0)
				{
			DBG(470,"Creating horizontal bspline at  y = %d",y);
//				grPutImage(d3_left,d3_top,d3_right,d3_top+y,map);
//				grBlitRegion(d3_left,d3_top,d3_right,grMaxy); //d3_top+y);

//				delay(2000);
				}

			if (keyboard_IsHit()) { keyboard_Flush(); return; }
			}

		// use 'painters algorithm' for _reasonable_ hidden surface removal
		for (y=0;y<d3_height;y++)
			{
//			DBG(470,"Drawing 3D representation row # %d",y);
			for (x=0;x<d3_width;x++)
				{
				z = grGetPixelVirtual(map,x,y) - minimum_height;

				if (z>sea_max-minimum_height)
					grPutPixelVirtual(map3D,x+(d3_bottom-y)/2,d3_top+y-(z/2),z+minimum_height);
				else
					grPutPixelVirtual(map3D,x+(d3_bottom-y)/2,d3_top+y-((sea_max-minimum_height)/2),z+minimum_height);
				}
			if (keyboard_IsHit()) { keyboard_Flush(); return; }
			}

		}
	}

//---------------------------------------------------------------------------
void Processor::GenerateBase(int base_height)
	{
	int x,y,z,i,j;
	int ctrl_pt_width = d3_width / ctrl_pts_x;
	int ctrl_pt_height = d3_height / ctrl_pts_y;

	// create a grid of b-spline control points to represent a
	// large area of plains/grasslands/etc. (with random height)
	int index;
	for (x=0;x<ctrl_pts_x;x++)
		{
		index = x * ctrl_pts_y;
		for (y=0;y<ctrl_pts_y;y++)
			{
			ControlPoints[index+y].x = y * ctrl_pt_height;
			ControlPoints[index+y].y = base_height;
			}
		}
	}

//---------------------------------------------------------------------------
void Processor::GenerateBumps(int x, int y, int spread,int variation,int min,int max)
	{
	int z,i,j;
	int ctrl_pt_width = d3_width / ctrl_pts_x;
	int ctrl_pt_height = d3_height / ctrl_pts_y;

	// figure out an approximate height for the mountain
	int last_height = random(max-min) + min;

	// for each element of the mountain range
	for (j=0;j<spread;j++)
		{
		// choose a random point nearby.... (+-1 for both x,y)
		int tmpx=x,tmpy=y;
		int chose,count=0;
		bool finished = false;
		do
			{
			count++;
			x += random(3) - 1;
			y += random(3) - 1;
			chose = grGetPixelVirtual(map,x,y);

			finished = ((x!=tmpx) || (y!=tmpy));
			finished &= ((chose<min) || (chose>max));
			}
		while ((count<10) && !(finished));

		// bounds-check (not perfect but close...)
		if (x<0) x=0;
		if (y<0) y=0;
		if (x>=ctrl_pts_x) x = ctrl_pts_x-1;
		if (y>=ctrl_pts_y) y = ctrl_pts_y-1;

		// and modify ctrl pts around it...
		ControlPoints[x*ctrl_pts_y+y].x = y * ctrl_pt_height;

		int change;
		do
			{
			change = random(variation) - variation/2;
			}
		while ((last_height+change<=min) && (last_height+change>max));

		// and copy it back
		ControlPoints[x*ctrl_pts_y+y].y = last_height+change;
		}
	}

//---------------------------------------------------------------------------
void Processor::HandleEvent(Event &e)
	{
	switch (e.what)
		{
		case ev_command:	switch (e.command.event)
								{
								case cmdGenerate:	// generate the map
													Generate3DMap();
													// update the appropriate display window
													if (display_mode==evhno_3DWindow) EVENTMGR->GenCommandEvent(evhno_3DWindow,cmdRefresh,NULL);
													else EVENTMGR->GenCommandEvent(evhno_ViewWindow,cmdRefresh,NULL);
													// and clear the event
													EVENTMGR->ClearEvent(e);
													break;
								}
		}
	}

/*
//---------------------------------------------------------------------------
void Processor::GenerateBaseTerrain(grVirtualBuffer* map,
									int map_width,int map_height,
									int xdist,int ydist)

// Generates base terrain onto the map....
// ie. areas containing plains, grasslands, deserts & swamps

	{
	}

void Processor::GenerateOverlaidTerrain(grVirtualBuffer* map,
										int map_width,int map_height,
										int xdist,int ydist)


// generates other terrain onto the map....
// ie. woods, forests, shrub, mangroves, hills & mountains
	{
	}
//---------------------------------------------------------------------------
*/
