//	-------------------------------------------------------------------------
//
//	VIEW.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the Viewing Window
//
//	-------------------------------------------------------------------------

#include "view.hpp"
#include "eventmgr.hpp"
#include "timer.hpp"
#include "keydefns.hpp"
#include "buttons.hpp"
#include "mouse.hpp"

#include "bspdefn.hpp"

ViewWindow::ViewWindow()
	: OnScreenHandler(d3_left,d3_top,d3_right,d3_bottom,grWINDOW_BG,bs_DBLRAISED,bs_NONE,
					&evhno_ViewWindow,0,1)
	{
	x_pos = d3_width/2;
	y_pos = d3_height/2;

	display_mode = evhno_ViewWindow;	// set the display mode to this window
	isOnScreen = true;					// and make sure we're displayed (refer MainLoop)

	}

void ViewWindow::Draw()
	{
	grClearScreen(grBLACK);

	// draw the top-down view of the map
	grPutImage(r.left,r.top,r.right,r.bottom,map);

	// draw the cursor hairs
	grDrawVLine(r.left+x_pos,r.top,r.bottom-50,grLIGHTRED);
	grDrawHLine(r.top+y_pos,r.left,r.right-50,grLIGHTRED);

	// draw the horizontal cross-section
	for (int x=r.left;x<r.right-50;x++)
		{
		int color = grGetPixelVirtual(map,x-r.left,y_pos) - minimum_height; //	color-offset
		grPutPixel(x,grMaxy-color,color+minimum_height);
		}

	// draw the vertical cross-section
	for (int y=r.top;y<r.bottom-50;y++)
		{
		int color = grGetPixelVirtual(map,x_pos,y-r.top) - minimum_height;
		grPutPixel(grMaxx-color,y,color+minimum_height);
		}

	grBlitScreen();//r.left,r.top,r.right,r.bottom);
	}

void ViewWindow::HandleEvent(Event& e)
	{
	OnScreenHandler::HandleEvent(e);

	switch (e.what)
		{
		case ev_mouse:		if (e.mouse.event==mb_leftup)
								{
								x_pos = e.mouse.where.x - d3_left;
								y_pos = e.mouse.where.y - d3_top;

								if (x_pos<0) x_pos = 0;
								if (x_pos>d3_width-50) x_pos = d3_width-50;
								if (y_pos<0) y_pos = 0;
								if (y_pos>d3_height-50) y_pos = d3_height-50;

								Refresh();

								EVENTMGR->ClearEvent(e);
								}
							break;
		}
	}