//	-------------------------------------------------------------------------
//
//	ACCESSRY.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains defines for the Accessory base class,
//					and some simple derived classes - Button, Bar & Box.
//					Also functions for installing accessory bitmaps, etc.
//					eg. InstallGUI() and ExitGUI()
//
//	-------------------------------------------------------------------------

#ifndef __accessry_hpp
#define __accessry_hpp

#include "graphics.hpp"
#include "event.hpp"

//	-------------------------------------------------------------------------
//
//	CLASS: 		Accessory
//	DESC: 		base class for all objects contained inside an event handler
//      		Examples for a simple gui include Buttons, Bars, Boxes, etc.
//
//	-------------------------------------------------------------------------

#define ac_NoOption				0x00000000		// no option (0 suffices)

class Accessory
	{
	protected:
		int 	LMBreturnEvent;  	// commandevent returned on LMB press/rel
		int		RMBreturnEvent;		// commandevent returned on RMB press/rel

		int 	selected_options; 	// the options set for this item

	public:

		// global variables.... :-(

		bool 	isSelected;			// the accessory is _currently_ down
		bool	isActive;			// the accessory is active ["on"]

		grRect	r;					// global screen coords of the accessory
		grRect	loc_r;				// local coords of the accessory

		Accessory(int x1,int y1,int x2,int y2,		// the local coordinates
				  int options,						// the options for this accessory
				  int ret_event);					// the return event (lmb==rmb)

		Accessory(int x1,int y1,int x2,int y2,		// the local coordinates
				  int options,						// the options for this accessory
				  int lmb_event,int rmb_event);		// the return events

		virtual ~Accessory();

		// adjusts global screen coordinates of accessory.
		virtual	void 	AdjustCoords(int dx,int dy);

		// a quick check to see whether or not the mouse is inside 'r'
		virtual bool 	MouseInside(grPoint& p);

		// sets the command events to be returned
		virtual void 	SetEvents(int lmb_event,int rmb_event);

		// selects the accessory
		virtual void 	Select();

		// deselects the accessory
		virtual void 	DeSelect();

		// open the accessory
		virtual void 	Activate();

		// close the accessory
		virtual void 	DeActivate();

		// --- DRAW FUNCTIONS

		// this function draws the accessory normally
		virtual void 	Draw();

		// this function draws the accessory into a specified virtual buffer
		virtual	void	Draw(grVirtualBuffer* vb);

		// this function Draw()'s then blits
		virtual void 	Refresh();

		// --- OTHER FUNCTIONS

		// handle any events required by the accessory!
		virtual void 	HandleEvent(Event &e);
	};

#endif