//	-------------------------------------------------------------------------
//
//	BUTTONS.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains code for Accessory-derived classes
//					such as TextButton and PictureButton. See GUI.CPP
//					TextButton & PictureButton derived objects such as
//					OKButton, UpArrowButton, etc.
//
//	-------------------------------------------------------------------------

#include "buttons.hpp"
#include "keyboard.hpp"
#include "keydefns.hpp"
#include "mouse.hpp"
#include "eventmgr.hpp"

#include "resource.hpp"

#include "stdlib.h"
#include "dos.h"

//	-------------------------------------------------------------------------
//
//	BUTTON FUNCTIONS
//
//	-------------------------------------------------------------------------
void 	Button::HandleEvent(Event &e)
	{
//	DBG(400,"IN BUTTON:: HANDLEEVENT()");

	if (!isActive) return;

	if (selected_options & ac_ToggleMode) { HandleToggleEvent(e); return; }

	switch (e.what)
		{
		case	ev_mouse:

			switch (e.mouse.event)
				{
				case	mb_leftdn:
				case	mb_rightdn:

					if (!isSelected) Select();
					EVENTMGR->ClearEvent(e);
					return;

				case	mb_leftauto:
				case	mb_rightauto:

					if (!isSelected) Select();
					if (selected_options&ac_AutoMode)
						{
						int evt;

						if (e.mouse.event==mb_leftauto) evt = LMBreturnEvent;
						else evt = RMBreturnEvent;

						EVENTMGR->ModCommandEvent(e,e.where,evt,NULL);
						return;
						}
					EVENTMGR->ClearEvent(e);
					return;

				case 	mb_leftup:
				case	mb_rightup:

					if (isSelected) DeSelect();
					// don't bother responding for auto events
//					if (selected_options&ac_AutoMode) ;
//					else
						{
						int evt;

						if (e.mouse.event==mb_leftup) evt = LMBreturnEvent;
						else evt = RMBreturnEvent;

						EVENTMGR->ModCommandEvent(e,e.where,evt,NULL);
						return;
						}
//					EVENTMGR->ClearEvent(e);
//					return;
				}
			break;

		case	ev_keyboard:

			switch (e.keyboard.event)
				{
				case	ENTER_DN:
				case	SPACE_DN:

					if (!isSelected) Select();
					if (selected_options&ac_AutoMode)
						{
						int evt;

						if (e.keyboard.event==ENTER_DN) evt = LMBreturnEvent;
						else evt = RMBreturnEvent;

						EVENTMGR->ModCommandEvent(e,e.where,evt,NULL);
						return;
						}
					EVENTMGR->ClearEvent(e);
					break;

				case	SPACE_UP:
				case	ENTER_UP:

					if (isSelected) DeSelect();
					if (selected_options&ac_AutoMode)	;
					else
						{
						int evt;

						if (e.keyboard.event==ENTER_DN) evt = LMBreturnEvent;
						else evt = RMBreturnEvent;

						EVENTMGR->ModCommandEvent(e,e.where,evt,NULL);
						return;
						}
					EVENTMGR->ClearEvent(e);
					return;

				default:

//						DBG(410,"UNKNOWN KEY");
						break;
				}
			break;
		}
	}

void 	Button::HandleToggleEvent(Event &e)
	{
//	DBG(400,"IN BUTTON:: HANDLE_TOGGLE_EVENT()");

	switch (e.what)
		{
		case	ev_mouse:

			switch (e.mouse.event)
				{
				case 	mb_leftup:

					if (isSelected) Accessory::DeSelect(); else Accessory::Select();
					EVENTMGR->ModCommandEvent(e,e.where,LMBreturnEvent,NULL);
					return;

				case	mb_rightup:

					if (isSelected) Accessory::DeSelect(); else Accessory::Select();
					EVENTMGR->ModCommandEvent(e,e.where,RMBreturnEvent,NULL);
					return;

				default:

					EVENTMGR->ClearEvent(e);
					break;

				}
			break;

		case	ev_keyboard:

			switch (e.keyboard.event)
				{
				case	ENTER_UP:

					if (isSelected) Accessory::DeSelect(); else Accessory::Select();
					EVENTMGR->ModCommandEvent(e,e.where,LMBreturnEvent,NULL);
					return;

				case	SPACE_UP:

					if (isSelected) Accessory::DeSelect(); else Accessory::Select();
					EVENTMGR->ModCommandEvent(e,e.where,RMBreturnEvent,NULL);
					return;

				case	ENTER_DN:
				case	SPACE_DN:

					EVENTMGR->ClearEvent(e);
					break;

				default:

//						DBG(410,"UNKNOWN KEY");
						break;
				}
			break;

		}
	}

void	Button::Select()
	{
	if (selected_options&ac_ToggleMode) ;
	else Accessory::Select();
	}

void	Button::DeSelect()
	{
	if (selected_options&ac_ToggleMode) ;
	else Accessory::DeSelect();
	}

void	Button::Toggle()
	{
	if (selected_options&ac_ToggleMode)
		if (isActive)
			{
			isSelected = !isSelected;
			Refresh();
			}
	}

//	-------------------------------------------------------------------------
//
//	TEXTBUTTON FUNCTIONS
//
//	-------------------------------------------------------------------------
TextButton::TextButton(int x1,int y1,
					int font_id,char* text,
					int options,int ret_event)
		: Button(x1,y1,x1,y1,options,ret_event,ret_event)
	{
	text_ptr = text;
	font = font_id;

	active_text 	= grDARKGRAY;
	inactive_text 	= grLIGHTGRAY;
	active_bg 		= grLIGHTGRAY;
	inactive_bg		= grDARKGRAY;
	active_bs		= bs_RAISED;
	inactive_bs		= bs_LOWERED;

	int save_font = grGetFont();

	grSetFont(font_id);

	int border_size = grSizeOfBorder[active_bs];

	r.right 	= r.left + 2*border_size + grFontWidth(text)+4;
	r.bottom 	= r.top  + 2*border_size + grFontHeight()+4,

	xofs = border_size + 2;
	yofs = border_size + 2;

	grSetFont(save_font);
	}


TextButton::TextButton(int x1,int y1,
					int font_id,char* text,
					int options,int lmb_event,int rmb_event)
		: Button(x1,y1,x1,y1,options,lmb_event,rmb_event)
	{
	text_ptr = text;
	font = font_id;

	active_text 	= grDARKGRAY;
	inactive_text 	= grLIGHTGRAY;
	active_bg 		= grLIGHTGRAY;
	inactive_bg		= grDARKGRAY;
	active_bs		= bs_RAISED;
	inactive_bs		= bs_LOWERED;

	int save_font = grGetFont();

	grSetFont(font_id);

	int border_size = grSizeOfBorder[active_bs];

	r.right 	= r.left + 2*border_size + grFontWidth(text)+4;
	r.bottom 	= r.top  + 2*border_size + grFontHeight()+4,

	xofs = border_size + 2;
	yofs = border_size + 2;

	grSetFont(save_font);
	}


TextButton::TextButton(int x1,int y1,
					int font_id,char* text,
					char active_txt,char active_bkg,int active_border,
					char inactive_txt,char inactive_bkg,int inactive_border,
					int options,int ret_event)
		: Button(x1,y1,x1,y1,options,ret_event,ret_event)
	{
	text_ptr = text;
	font = font_id;

	active_text 	= active_txt;
	inactive_text 	= inactive_txt;
	active_bg 		= active_bkg;
	inactive_bg		= inactive_bkg;
	active_bs		= active_border;
	inactive_bs		= inactive_border;

	int save_font = grGetFont();

	grSetFont(font_id);

	int border_size = grSizeOfBorder[active_bs];

	r.right 	= r.left + 2*border_size + grFontWidth(text)+4;
	r.bottom 	= r.top  + 2*border_size + grFontHeight()+4,

	xofs = border_size + 2;
	yofs = border_size + 2;

	grSetFont(save_font);
	}

TextButton::TextButton(int x1,int y1,
					int font_id,char* text,
					char active_txt,char active_bkg,int active_border,
					char inactive_txt,char inactive_bkg,int inactive_border,
					int options,int lmb_event,int rmb_event)
		: Button(x1,y1,x1,y1,options,lmb_event,rmb_event)
	{
	text_ptr = text;
	font = font_id;

	active_text 	= active_txt;
	inactive_text 	= inactive_txt;
	active_bg 		= active_bkg;
	inactive_bg		= inactive_bkg;
	active_bs		= active_border;
	inactive_bs		= inactive_border;

	int save_font = grGetFont();

	grSetFont(font_id);

	int border_size = grSizeOfBorder[active_bs];

	r.right 	= r.left + 2*border_size + grFontWidth(text)+4;
	r.bottom 	= r.top  + 2*border_size + grFontHeight()+4,

	xofs = border_size + 2;
	yofs = border_size + 2;

	grSetFont(save_font);
	}


void 	TextButton::Draw()
	{
	int save_font = grGetFont();
	grSetFont(font);

	if (isActive)
		{
		if (isSelected)
			{
			grDrawWindow(r.left,r.top,r.right,r.bottom,active_bg,active_bs);
			grDraw2CString(r.left+xofs,r.top+yofs,active_text,"%s",text_ptr);
			}
		else
			{
			grDrawWindow(r.left,r.top,r.right,r.bottom,inactive_bg,inactive_bs);
			grDraw2CString(r.left+xofs,r.top+yofs,inactive_text,"%s",text_ptr);
			}
		}
	else
		{
		grDrawWindow(r.left,r.top,r.right,r.bottom,active_bg,active_bs);
		grDraw2CString(r.left+xofs,r.top+yofs,active_text,"%s",text_ptr);
		grDrawShadedRect(r.left,r.top,r.right,r.bottom,inactive_bg,2);
		}
	grSetFont(save_font);
	}


void 	TextButton::Draw(grVirtualBuffer* vb)
	{
	if (!vb) return;

	int save_font = grGetFont();
	grSetFont(font);

	if (isActive)
		{
		if (isSelected)
			{
			grDrawWindowVirtual(vb,loc_r.left,loc_r.top,loc_r.right,loc_r.bottom,active_bg,active_bs);
			grDraw2CStringVirtual(vb,loc_r.left+xofs,loc_r.top+yofs,active_text,"%s",text_ptr);
			}
		else
			{
			grDrawWindowVirtual(vb,loc_r.left,loc_r.top,loc_r.right,loc_r.bottom,inactive_bg,inactive_bs);
			grDraw2CStringVirtual(vb,loc_r.left+xofs,loc_r.top+yofs,inactive_text,"%s",text_ptr);
			}
		}
	else
		{
		grDrawWindowVirtual(vb,loc_r.left,loc_r.top,loc_r.right,loc_r.bottom,active_bg,active_bs);
		grDraw2CStringVirtual(vb,loc_r.left+xofs,loc_r.top+yofs,active_text,"%s",text_ptr);
		grDrawShadedRectVirtual(vb,loc_r.left,loc_r.top,loc_r.right,loc_r.bottom,inactive_bg,2);
		}
	grSetFont(save_font);
	}

//	-------------------------------------------------------------------------
//
//	PICTUREBUTTON FUNCTIONS
//
//	-------------------------------------------------------------------------
// assuming no transparency
PictureButton::PictureButton(int x1,int y1,
						grImage* active_image,
						grImage* inactive_image,
						int options,
						int ret_event)
		: Button(x1,y1,x1,y1,options,ret_event,ret_event)
	{
	active_pic = active_image;
	inactive_pic = inactive_image;

	r.right = r.left + active_pic->hdr.width-1;
	r.bottom = r.top + active_pic->hdr.height-1;

	trans = -1;
	}

PictureButton::PictureButton(int x1,int y1,
						grImage* active_image,
						grImage* inactive_image,
						int options,
						int lmb_event,int rmb_event)
		: Button(x1,y1,x1,y1,options,lmb_event,rmb_event)
	{
	active_pic = active_image;
	inactive_pic = inactive_image;

	r.right = r.left + active_pic->hdr.width-1;
	r.bottom = r.top + active_pic->hdr.height-1;

	trans = -1;
	}

// assumes transparency
PictureButton::PictureButton(int x1,int y1,
						int transparent_color,
						grImage* active_image,
						grImage* inactive_image,
						int options,
						int ret_event)
		: Button(x1,y1,x1,y1,options,ret_event,ret_event)
	{
	active_pic = active_image;
	inactive_pic = inactive_image;

	r.right = r.left + active_pic->hdr.width-1;
	r.bottom = r.top + active_pic->hdr.height-1;

	trans = transparent_color;
	}

PictureButton::PictureButton(int x1,int y1,
						int transparent_color,
						grImage* active_image,
						grImage* inactive_image,
						int options,
						int lmb_event,int rmb_event)
		: Button(x1,y1,x1,y1,options,lmb_event,rmb_event)
	{
	active_pic = active_image;
	inactive_pic = inactive_image;

	r.right = r.left + active_pic->hdr.width-1;
	r.bottom = r.top + active_pic->hdr.height-1;

	trans = transparent_color;
	}


void 	PictureButton::Draw()
	{
	if (isActive)
		{
		if (isSelected)
			{
			if (trans==-1) grPutImage(r.left,r.top,r.right,r.bottom,active_pic);
			else grOverlayImage(r.left,r.top,r.right,r.bottom,trans,active_pic);
			}
		else
			{
			if (trans==-1) grPutImage(r.left,r.top,r.right,r.bottom,inactive_pic);
			else grOverlayImage(r.left,r.top,r.right,r.bottom,trans,inactive_pic);
			}
		}
	else
		{
		if (trans==-1) grPutImage(r.left,r.top,r.right,r.bottom,active_pic);
		else grOverlayImage(r.left,r.top,r.right,r.bottom,trans,active_pic);

		grDrawShadedRect(r.left,r.top,r.right,r.bottom,grDARKGRAY,2);
		}
	}


void 	PictureButton::Draw(grVirtualBuffer* vb)
	{
	if (!vb) return;

	if (isActive)
		{
		if (isSelected)
			{
			if (trans==-1) 	grPutImageVirtual(vb,loc_r.left,loc_r.top,
							   loc_r.right,loc_r.bottom,active_pic);
			else			grOverlayImageVirtual(vb,loc_r.left,loc_r.top,
								   loc_r.right,loc_r.bottom,trans,active_pic);
			}
		else
			{
			if (trans==-1)	grPutImageVirtual(vb,loc_r.left,loc_r.top,
								loc_r.right,loc_r.bottom,inactive_pic);
			else			grOverlayImageVirtual(vb,loc_r.left,loc_r.top,
									loc_r.right,loc_r.bottom,trans,inactive_pic);
			}
		}
	else
		{
		if (trans==-1)	grPutImageVirtual(vb,loc_r.left,loc_r.top,
							loc_r.right,loc_r.bottom,active_pic);
		else			grOverlayImageVirtual(vb,loc_r.left,loc_r.top,
							loc_r.right,loc_r.bottom,trans,active_pic);

		grDrawShadedRectVirtual(vb,loc_r.left,loc_r.top,loc_r.right,loc_r.bottom,
								grDARKGRAY,2);
		}
	}

// a quick check to see whether or not the mouse is inside 'r'
bool 	PictureButton::MouseInside(grPoint& p)
	{
	// find out if we're inside the button quickly....
	bool isInside = Button::MouseInside(p);

	// if we're checking for transparent colors
	if (trans!=-1)
		{
		// if we are inside...
		if (isInside)
			{
			// convert global 'p' to local coordinates
			p.x -= r.left;
			p.y -= r.top;

			// return true if the pixel selected is not the transparent color
			grImage* pic = inactive_pic;
			if (isSelected) pic = active_pic;

			return (pic->data[p.y * pic->hdr.width + p.x] != trans );
			}
		}

	// return if mouse is inside the coordinates
	return isInside;
	}

void	PictureButton::SetActivePic(grImage* active_image)
	{
	active_pic = active_image;
	}

void	PictureButton::SetInactivePic(grImage* inactive_image)
	{
	inactive_pic = inactive_image;
	}

void	PictureButton::SetPics(grImage* active_image,grImage* inactive_image)
	{
	active_pic = active_image;
	inactive_pic = inactive_image;
	}

