//	-------------------------------------------------------------------------
//
//	ACCESSRY.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains defines for the Accessory base class,
//					and some simple derived classes - Button, Bar & Box.
//					Also functions for installing accessory bitmaps, etc.
//					eg. InstallGUI() and ExitGUI()
//
//	-------------------------------------------------------------------------

#ifndef __accessry_hpp
#define __accessry_hpp

#include "graphics.hpp"
#include "event.hpp"

//	-------------------------------------------------------------------------
//
//	CLASS: 		Accessory
//	DESC: 		base class for all objects contained inside an event handler
//      		Examples for a simple gui include Buttons, Bars, Boxes, etc.
//
//	NOTES:		ALL ACCESSORIES _SHOULD_ RESPOND TO BOTH KEYBOARD & MOUSE!!!!
//
//	-------------------------------------------------------------------------

// options for all Accessory's
#define ac_NoOption				0x0000		// no option (0 suffices)
#define ac_SaveUnderAccessory	0x0001		// defines save image.
#define ac_IsNotSelectable		0x0002		// cannot be selected

class Accessory
	{
	protected:
		int 	returnEvent;  		// command event returned from HandleEvent()

		int 	selected_options; 	// the options set for this item

		int 	active_bs;			// active (selected) border style
		int 	inactive_bs;		// inactive (deselected) border style

		uchar	active_color;		// active (selected) background color
		uchar	inactive_color;		// inactive (deselected) background color

	public:

		// global variables.... :-(

		bool 	isSelected;			// the accessory is currently selected
		bool	isActive;			// the accessory is active

		grRect	r;					// global screen coords of the accessory
		grRect	inset_r;			// global screen coords inside the accessory's border
		grRect	loc_r;				// local coords of the accessory
		grRect	loc_inset_r;		// local coords inside the border

		Accessory(int x1,int y1,int x2,int y2,			// the coordinates (local to handler)
				  uchar active_clr,uchar inactive_clr,	// the background colors
				  int active_border,int inactive_border,// the border styles
				  int options,							// the options for this accessory
				  int retevent);						// the command event returned by HandleEvent

		virtual ~Accessory();

		// adjusts global screen coordinates of accessory.
		virtual	void 	AdjustCoords(int dx,int dy);

		// a quick check to see whether or not the mouse is inside 'r'
		virtual bool 	MouseInside(grPoint& p);

		// sets the command event to be returned on being selected
		virtual void 	SetReturnEvent(int retevent) { returnEvent = retevent; }

		// --- DRAW FUNCTIONS

		// function which is called on activating the accessory
		virtual void 	Select();

		// function which is called on deactivating the accessory
		virtual void 	DeSelect();

		// toggle accessory ON (active) & OFF (deactive)
		virtual void 	Activate();
		virtual void 	DeActivate();

		// this function will draw the accessory into grSCREEN
		// when called by an OnScreenHandler's Draw() functions.
		virtual void 	Draw();

		// this function will draw the accessory into a virtual buffer
		// when called by an OnScreenHandler's Draw(vb) functions.
		virtual	void	Draw(grVirtualBuffer* vb);

		// this function will literally put the accessory on the screen
		// after Draw()ing it
		virtual void Refresh();

		// --- OTHER FUNCTIONS

		// handle any events required by the accessory!
		virtual void HandleEvent(Event &e);
	};

#endif