//	-------------------------------------------------------------------------
//
//	DIALOG.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	this file contains the definition for a standard
//					Dialog Box
//
//	-------------------------------------------------------------------------

#ifndef __dialog_hpp
#define __dialog_hpp

#include "onscreen.hpp"

// dialog box reserves 0x00010000 - 0x00080000

#define dlg_OK_DEFAULT		0x00010000		// OK is assumed on default
#define dlg_CANCEL_DEFAULT  0x00020000		// CANCEL is assumed on default

//	-------------------------------------------------------------------------
//
//	CLASS:	DialogBox
//
//	DESC:   This is the basic dialog box object which allows simple access
//			to "pop-up" windows. An example of a dialog box is the
//			FileDialogBox
//
//	-------------------------------------------------------------------------

class DialogBox : public OnScreenHandler
	{
	protected:
		int		prev_zfactor;		// the previous zfactor
		int		prev_handler;		// the previously selected handler

		int		parent_handler;		// the [evhno_] number of the handler
									// which responds to returnEvent

		int		OKreturnEvent;		// the command event generated on 'OK'
		int		CANCELreturnEvent;  // the command event generated on 'CANCEL'

		grImage*	savedImage;		// area to save whats underneath!!!

	public:
		int		isOpen;				// the dialog box is currently open

		DialogBox(int x1,int y1,int x2,int y2,	// the coords
					uchar bg,					// the background color
					uchar active_border,		// and border styles
					uchar inactive_border,
					int* evhno,					// the event-handler number
					int parent_handler_num,		// the parent's evhno
//					int accessory_selected,		// the accessory selected on open
					int options,				// the dialog box options
					int zfact,					// its z-factor
					int okretevent,				// command event returned on OK
					int cancelretevent);        // command event returned on CANCEL

		DialogBox(int x1,int y1,int x2,int y2,	// the coords
					grImage* active_bitmap,		// the bitmaps
					grImage* inactive_bitmap,
					int* evhno,					// the event-handler number
					int parent_handler_num,		// the parent's evhno
//					int accessory_selected,		// the accessory selected on open
					int options,				// the dialog box options
					int zfact,					// its z-factor
					int okretevent,				// command event returned on OK
					int cancelretevent);        // command event returned on CANCEL

		virtual ~DialogBox();

		virtual void Draw();

		virtual void OpenDialog();				// opens the dialog box
		virtual void CloseDialog();				// closes the dialog box
		virtual void HandleEvent(Event &e);		// the handleevent function
	};

#endif

