
FormattedInputBar::FormattedInputBar(int x1,int y1,String* st,String* formatstr,
							int options,int retevent)
	: TextInputBar(x1,y1,formatstr->Length(),st,options,retevent)
	{
	format_str = formatstr->Contents();
	BuildFormattedString(st);
	if (!(selected_options&inputbar_LEFT_JUSTIFIED))
		selected_options |= inputbar_RIGHT_JUSTIFIED;
	}

FormattedInputBar::FormattedInputBar(int x1,int y1,String* st,const char* formatstr,
							int options,int retevent)
	: TextInputBar(x1,y1,strlen(formatstr),st,options,retevent)
	{
	format_str = (char*) formatstr;
	BuildFormattedString(st);
	if (!(selected_options&inputbar_LEFT_JUSTIFIED))
		selected_options |= inputbar_RIGHT_JUSTIFIED;
	}

// builds the edit string required based upon the current format string
// and a specified string

void FormattedInputBar::BuildFormattedString(String* st)
	{
	// set up our edit str to represent the blank format str
	char* tmpstr = st->Contents();
	int n;
	int posn;
	int max = strlen(format_str);
	int max_posn = strlen(tmpstr);
	char ch = 0;
	str->Set("");

	if (selected_options&inputbar_LEFT_JUSTIFIED)
		{
		n = 0;
		posn = 0;
		while (n<max)
			{
//			DBG(110,"FORMAT_STRING == >%s<",format_str);
//			DBG(120,"TEMP_STRING   == >%s<",tmpstr);
//			DBG(130,"posn = %.2d, n == %.2d   ",posn,n);
//			DBG(140,"FORMAT_STR[n] == >%c<",format_str[n]);
//			DBG(150,"TEMP_STRIN[p] == >%c<",tmpstr[posn]);
			ch = 0;
			switch (format_str[n])
				{
				case	format_ALPHA:			if (ch==0) ch = 32;	// ' '
				case	format_NUMERIC:			if (ch==0) ch = 48; // '0'
				case	format_ALPHANUMERIC:	if (ch==0) ch = 32; // ' '
				case	format_SIGN:			if (ch==0) ch = 43; // '+'
												if (posn<max_posn)
													str->Append(tmpstr[posn++]);
												else
													{
													if (ch==0) ch = 32;
													str->Append(ch);
													}
												break;
				default:						str->Append(format_str[n]);
												break;
				}
			n++;
//			DBG(160,"STR == >%s<",str->Contents());
//			delay(1000);
			}
		}
	else
		{
		n = max;
		posn = max_posn-1;
		while (n>=0)
			{
//			DBG(110,"FORMAT_STRING == >%s<",format_str);
//			DBG(120,"TEMP_STRING   == >%s<",tmpstr);
//			DBG(130,"posn = %.2d, n == %.2d   ",posn,n);
//			DBG(140,"FORMAT_STR[n] == >%c<",format_str[n]);
//			DBG(150,"TEMP_STRIN[p] == >%c<",tmpstr[posn]);
			ch = 0;
			switch (format_str[n])
				{
				case	format_ALPHA:			if (ch==0) ch = 32;	// ' '
				case	format_NUMERIC:			if (ch==0) ch = 48; // '0'
				case	format_ALPHANUMERIC:	if (ch==0) ch = 32; // ' '
				case	format_SIGN:			if (ch==0) ch = 43; // '+'
												if (posn>=0)
													str->Insert(0,tmpstr[posn--]);
												else
													{
													if (ch==0) ch = 32;
													str->Insert(0,ch);
													}
												break;
				default:						str->Insert(0,format_str[n]);
												break;
				}
			n--;
//			DBG(160,"STR == >%s<",str->Contents());
//			delay(100);
			}
		}

	laststr->Set(str);
	}

void FormattedInputBar::BuildUnformattedString(String* st)
	{
	// reformat the original string to represent the inputted data
	char* tmpstr = str->Contents();
	int posn = 0;
	int max = str->Length();
	st->Set("");
	while (posn<max)
		{
		switch (format_str[posn])
			{
			case	format_SIGN:			;
			case	format_ALPHA:			;
			case	format_NUMERIC:			;
			case	format_ALPHANUMERIC:	st->Append(tmpstr[posn]);
											break;
			}
		posn++;
		}
	}

void FormattedInputBar::BuildReturnString()
	{
	BuildUnformattedString(origstr);
	}

void FormattedInputBar::SetEditStr(long val)
	{
	String* tmpstr = new String("0",20);
	tmpstr->Set(val);
	SetEditStr(tmpstr);
	delete tmpstr;
	}

int last_pos = 99;

void FormattedInputBar::SetEditPosn(int x,bool mouse_selection)
	{
	// set the position
	edit_pos = x;
	char tmpch = 0;

	if (edit_pos<=0)
		{
		edit_pos = 0;
		tmpch = format_str[edit_pos];
		if ((tmpch!=format_SIGN)&&(tmpch!=format_ALPHA)&&
			(tmpch!=format_NUMERIC)&&(tmpch!=format_ALPHANUMERIC))
			IncEditPosn();
		}

	if (edit_pos>=str->Length())
		{
		edit_pos = str->Length();
		tmpch = format_str[edit_pos];
		if ((tmpch!=format_SIGN)&&(tmpch!=format_ALPHA)&&
			(tmpch!=format_NUMERIC)&&(tmpch!=format_ALPHANUMERIC))
			DecEditPosn();
		last_pos = edit_pos;
		}

	cursor_pos = edit_pos;
	}

void FormattedInputBar::IncEditPosn()
	{
	char tmpch;
	int max = str->Length();

	// scan through till we find a valid editting char
	do
		{
		if (edit_pos<max)
			{
			edit_pos++;
			tmpch = format_str[edit_pos];
			}
		else
			{
			edit_pos = max;
			tmpch=format_SIGN;	// skip out, finished
			}
		}
	while ((tmpch!=format_SIGN)&&(tmpch!=format_ALPHA)&&
			 (tmpch!=format_NUMERIC)&&(tmpch!=format_ALPHANUMERIC));

	// if we're too far over
	if (edit_pos>=max)
		{
		edit_pos=max;
		tmpch = format_str[edit_pos];
		if ((tmpch!=format_SIGN)&&(tmpch!=format_ALPHA)&&
			(tmpch!=format_NUMERIC)&&(tmpch!=format_ALPHANUMERIC))
			DecEditPosn();
		last_pos = edit_pos;
		}

	cursor_pos = edit_pos;
	}

void FormattedInputBar::DecEditPosn()
	{
	char tmpch;
	// scan through till we find a valid editting char
	do
		{
		if (edit_pos>0)
			{
			edit_pos--;
			tmpch = format_str[edit_pos];
			}
		else
			{
			tmpch = format_SIGN;
			}
		}
	while ((tmpch!=format_SIGN)&&(tmpch!=format_ALPHA)&&
		   (tmpch!=format_NUMERIC)&&(tmpch!=format_ALPHANUMERIC));

	// if we've gone too far
	if (edit_pos<=0)
		{
		edit_pos = 0;
		tmpch = format_str[edit_pos];
		if ((tmpch!=format_SIGN)&&(tmpch!=format_ALPHA)&&
			(tmpch!=format_NUMERIC)&&(tmpch!=format_ALPHANUMERIC))
			IncEditPosn();
		}

	cursor_pos = edit_pos;
	}

//	DESC:	Returns the relative edit position for the unformatted string
int FormattedInputBar::UnformattedEditPos()
	{
	int cnt = 0,index =0;

	// scan through to edit_pos counting the number of format chars...
	while (index<edit_pos)
		{
		char tmpch = format_str[index];
		if ((tmpch==format_SIGN)||(tmpch==format_ALPHA)||
			(tmpch==format_NUMERIC)||(tmpch==format_ALPHANUMERIC))
			cnt++;
		index++;
		}
	return cnt;
	}

bool insert_mode = false;

void FormattedInputBar::HandleEvent(Event &e)
	{
	// pre-process keyboard input
	if (e.what==ev_keyboard)
		{
		String* tmpstr = NULL;
		switch (e.keyboard.event)
			{
			case	LT_ARROW_UP:		if (!isSelected)
											{
											isSelected = true;
											SetEditPosn(0,false);
											Refresh();
											}
										else
											{
											DecEditPosn();
											Refresh();
											}
										EVENTMGR->ClearEvent(e);
										break;
			case	RT_ARROW_UP:		if (!isSelected)
											{
											isSelected = true;
											SetEditPosn(str->Length(),false);
											Refresh();
											}
										else
											{
											IncEditPosn();
											Refresh();
											}
										EVENTMGR->ClearEvent(e);
										break;
			case	DEL_UP:				;
			case	BS_UP:				if (isSelected)
											{
											tmpstr = new String("",str->Length());
											BuildUnformattedString(tmpstr);
											tmpstr->Delete(UnformattedEditPos(),1);
											SetEditStr(tmpstr);
											delete tmpstr;
											Refresh();
											}
										EVENTMGR->ClearEvent(e);
										break;
			case	TAB_UP:				insert_mode = false;
			case	ENTER_UP:			insert_mode = false;
										break;
			case	ESC_UP:				if (isSelected)
											{
											isSelected = false;
											SetEditStr(origstr);
											SetEditPosn(0);
											Refresh();
											EVENTMGR->ClearEvent(e);
											insert_mode = false;
											}
										return;

			default:	// check if we should clear the input field,
						// or if we should simply manipulate the input..
						if (keyboard_IsReleased(e.keyboard.event))
							{
							int keyval = keyboard_ASCII(e.keyboard.event,e.keyboard.modifiers);
							if ((keyval>=32)&&(keyval<=127))
								{
								if (!isSelected)
									{
									// left justify assumes normal entry
									if (selected_options&inputbar_LEFT_JUSTIFIED)
										SetEditPosn(0);
									else
										// right justify assumes insertion
										{
										SetEditPosn(str->Length());
										tmpstr = new String("",str->Length());
										BuildFormattedString(tmpstr);
										delete tmpstr;
										insert_mode = true;
										}
									}

								bool ok = false;
								switch (format_str[edit_pos])
									{
									case	format_SIGN:		if ((keyval==43)||(keyval==45)) ok = true;
																break;
									case	format_ALPHA:		if ((keyval>=65)&&(keyval<=90)||
																	(keyval>=97)&&(keyval<=122)||
																	(keyval==32)) ok = true;
																break;
									case	format_NUMERIC:		if ((keyval>=48)&&(keyval<=57)) ok = true;
																break;
									case	format_ALPHANUMERIC:if ((keyval>=48)&&(keyval<=57)||
																	(keyval>=65)&&(keyval<=90)||
																	(keyval>=97)&&(keyval<=122)||
																	(keyval==32)) ok = true;
																break;
									default:					ok = true;
															break;
									}
								if (ok)
									{
									if (insert_mode)
									// get the raw edit string, delete the first char,
									// append the keyval and then rebuild the edit string
										{
										tmpstr = new String("",str->Length());
										BuildUnformattedString(tmpstr);
										tmpstr->Delete(0,1);
										tmpstr->Insert(UnformattedEditPos(),(char)keyval);
										SetEditStr(tmpstr);
										delete tmpstr;
										}
									else
										{
										// set the appropriate char & inc the edit position
										str->Contents()[edit_pos] = (char)keyval;
										IncEditPosn();
										}

									if (isSelected)
										Refresh();
									else
										{
										isSelected = true;
										Refresh();
										}
									}
								EVENTMGR->ClearEvent(e);
								}
							}
						break;
			}
		}

	// and let the textinputbar do the rest
	if (e.what!=ev_nothing) TextInputBar::HandleEvent(e);

	// if we have finished editting the text & have returned, extract the
	// formatting information from the returning string & let it return
	if (e.what==ev_command)
		if (e.command.event==returnEvent)
			{
			BuildReturnString();
			}
	}
