//	-------------------------------------------------------------------------
//
//	CLASS:		FormattedInputBar
//	DESC:		This input bar accepts user input formatted as shown below
//
//				Examples:	"$999,999,000"	- simple dollar input field
//                          "99-ZZZ-9999"   - simple date input field.
//							"99:99:99"		- simple time input field
//
//				Format chars are used as follows:
//						'A'		- Alpha input (A..Z,a..z,space,comma,etc.)
//						'+'		- Sign input (plus or minus only)
//						'9'		- Numeric input (0..9)
//						'Z'		- AlphaNumeric input (Alpha & Numeric & Sign)
//
//	-------------------------------------------------------------------------

#define	format_SIGN			43	// '+'
#define format_ALPHA		65	// 'A'
#define format_NUMERIC		57	// '9'
#define format_ALPHANUMERIC	90	// 'Z'

#define inputbar_LEFT_JUSTIFIED			0x0100
#define inputbar_RIGHT_JUSTIFIED		0x0200

class	FormattedInputBar : public TextInputBar
	{
	protected:
		char* 	format_str;
	public:
		FormattedInputBar(int x1,int y1,String* st,String* formatstr,
							int options,int retevent);
		FormattedInputBar(int x1,int y1,String* st,const char* formatstr,
							int options,int retevent);

		virtual void BuildFormattedString(String* st);
		virtual void BuildUnformattedString(String* st);
		virtual void BuildReturnString();

		virtual void SetEditStr(String* st) { BuildFormattedString(st); }
		virtual void SetEditStr(long val);

		virtual void SetEditPosn(int x,bool mouse_selection = false);
		virtual void IncEditPosn();
		virtual void DecEditPosn();
		virtual int  UnformattedEditPos();

		virtual void HandleEvent(Event &e);
	};


//	-------------------------------------------------------------------------
//
//	CLASS:		TensInputBar
//	DESC:		This input bar accepts a dollar value from 0-99
//
//	-------------------------------------------------------------------------

class	TensInputBar : public FormattedInputBar
	{
	public:
		TensInputBar(int x1,int y1,String* st,int options,int retevent)
			: FormattedInputBar(x1,y1,st,"$ 99",options,retevent) {}
	};

//	-------------------------------------------------------------------------
//
//	CLASS:		HundredsInputBar
//	DESC:		This input bar accepts a dollar value from 0-999
//
//	-------------------------------------------------------------------------

class	HundredsInputBar : public FormattedInputBar
	{
	public:
		HundredsInputBar(int x1,int y1,String* st,int options,int retevent)
			: FormattedInputBar(x1,y1,st,"$ 999",options,retevent) {}
	};

//	-------------------------------------------------------------------------
//
//	CLASS:		ThousandsInputBar
//	DESC:		This input bar accepts a dollar value from 1,000-999,000
//
//	-------------------------------------------------------------------------

class	ThousandsInputBar : public FormattedInputBar
	{
	public:
		ThousandsInputBar(int x1,int y1,String* st,int options,int retevent)
			: FormattedInputBar(x1,y1,st,"$ 999,000",options,retevent) {}
	};
//	-------------------------------------------------------------------------
//
//	CLASS:	    MillionsInputBar
//	DESC:		This input bar accepts a dollar value from 1,000-999,999,000
//
//	-------------------------------------------------------------------------

class	MillionsInputBar : public FormattedInputBar
	{
	public:
		MillionsInputBar(int x1,int y1,String* st,int options,int retevent)
			: FormattedInputBar(x1,y1,st,"$ 999,999,000",options,retevent) {}
	};
/*
//	-------------------------------------------------------------------------
//
//	CLASS:		NumInputBar
//	DESC:		This input bar accepts numerical input only.
//
//	-------------------------------------------------------------------------

class	NumInputBar : public FormattedInputBar
	{
	public:
		NumInputBar(int x1,int y1,String* st,int maxval,int minval,
							int options,int retevent);
		virtual ~NumInputBar();
	};

*/


