//	-------------------------------------------------------------------------
//
//	INPUTBAR.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains code for a number of InputBars
//					including TextInputBar, FormattedInputBar,
//					NumericalInputBar, & DollarInputBar.
//
//	-------------------------------------------------------------------------

#ifndef __inputbar_hpp
#define __inputbar_hpp

#include "accessry.hpp"
#include "buttons.hpp"
#include "strings.hpp"

#define cmdTextInputBar 		1102
#define cmdFormattedInputBar	1103

//	-------------------------------------------------------------------------
//
//	CLASS:		TextInputBar
//	DESC:		This bar accepts users keyboard input and modifies a String
//				to reflect the editted string.
//				Upon exitting normally, generates the OKevent with the
//				cmd_event data field as a String* to the last editted string.
//				Upon exitting with rmb, ESCAPE, generates the CANCELevent
//
//	-------------------------------------------------------------------------

class TextInputBar : public Accessory
	{
	protected:
		String*		orig_str;		// the original string passed in
		String*		edit_str;		// the current edit string

		int			num_columns;	// the number of columns displayed
		int			max_columns;	// the maximum length of the edit string

		int			col_offset;		// the offset of the display
		int			max_col_offset;	// the maximum col_offset

		int			cursor_pos;		// the position of the cursor

		int			edit_pos;		// the editting position
		int			max_edit_pos;	// the maximum editting position

	public:
		TextInputBar(int x1,int y1,int num_cols,
					 String* input_str,int max_len,
					 int active_border,int inactive_border,
					 int options,
					 int OKretevent,CANCELretevent);

		virtual ~TextInputBar();

		virtual void SetEditStr(String* st);	// sets the edit string to 'st'
		virtual String* GetEditStr();			// returns a pointer to the
												// current edit string

		virtual void SetEditPos(int offset,int pos);	// sets the current editting position
		virtual void SetEditPos(grPoint& mouse_pt);		// sets the current editting position
														// based on mouse cursor input

		virtual void IncEditPosn();				// increments the edit_pos by 1
		virtual void DecEditPosn();				// decrements the edit_pos by 1

		virtual void Draw(grVirtualBuffer* vb);	// draws the input bar
		virtual void HandleEvent(Event &e);		// handles event input
	};


#endif