//	-------------------------------------------------------------------------
//
//	SCROLBAR.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains defines for Vertical and Horizontal
//					scroll bars (VScrollBar & HScrollBar)
//
//	-------------------------------------------------------------------------

#ifndef __scrolbar_hpp
#define __scrolbar_hpp

#include "accessry.hpp"
#include "buttons.hpp"
#include "strings.hpp"

#define cmdVScrollBar 		1100
#define cmdHScrollBar 		1101

// options for scrollbars

#define scrollbar_SINGLE_CLICK	0x0010		// buttons work on single click
#define scrollbar_DOUBLE_CLICK	0x0020		// buttons work on double click?
#define scrollbar_AUTO_REPEAT	0x0040		// buttons auto repeat

//	-------------------------------------------------------------------------
//
//	CLASS:		VScrollBar
//	DESC:		This bar contains an UpArrowButton, a DnArrowButton and
//				a BlankButton.
//
//				The BlankButton moves up and down at its position relative
//				to a maximum value & the height of the scroll bar.
//
//	-------------------------------------------------------------------------

class VScrollBar : public Accessory
	{
	protected:
		int		min_pos;		// the minimum position
		int		max_pos;		// the maximum position
		int		cur_pos;		// the current position
		int		increment;		// the amount to increment/decrement

		float 	scale;
		int 	scaleheight;
		int 	scalestart;
		int 	scaleend;

		int 	buttonsize;

		int 	blankpos;
		int 	maxpos;
		int 	pos;
		int 	lastpos;

	public:
		BlankButton* Blank;
		UpArrowButton* UpArrow;
		DnArrowButton* DnArrow;

		VScrollBar(	int xx,int y1,int y2,
					int current_position,int min_position,int max_position,
					int options,int retevent);

		virtual ~VScrollBar() { delete Blank;
								delete DnArrow;
								delete UpArrow; }

		virtual void SetCurrentPos(int newpos,bool refresh = true);
		virtual void SetMaxPos(int newmaxpos,bool refresh = true);
		virtual void SetMinPos(int newminpos,bool refresh = true);

		virtual void SetMaxPos(int maxpositions,bool redraw = true)	// set the maximum number of positions
							{ maxpos = maxpositions;
							  SetPos(pos,redraw); }

		virtual void SetPos(int position,bool redraw = true)			// set the current position
						{
						pos = position;
						if (pos>maxpos) pos = maxpos;
						if (pos<0) pos = 0;
						if (redraw) Refresh();
						}

		virtual void SetPosn(int max_posn,int posn)	// set the current & max positions
						{ maxpos = max_posn; pos = posn; }

		// overridden functions
		virtual void AdjustCoords(int dx,int dy);
		virtual void DeSelect();
		virtual void Draw(grVirtualBuffer* vb);
		virtual void HandleEvent(Event &e);
	};


//	-------------------------------------------------------------------------
//
//	CLASS:		HScrollBar
//	DESC:		This bar contains a LtArrowButton, a RtArrowButton and
//				a BlankButton.
//
//				The BlankButton moves left and right from a position relative
//				to a maximum value & the width of the scroll bar.
//
//	-------------------------------------------------------------------------

class HScrollBar : public Accessory
	{
	protected:
		float 	scale;
		int 	scalewidth;
		int 	scalestart;
		int 	scaleend;
		int 	buttonsize;
		int 	blankpos;
		int 	maxpos;
		int 	pos;
		int 	lastpos;

	public:
		BlankButton* Blank;
		LtArrowButton* LtArrow;
		RtArrowButton* RtArrow;

		HScrollBar(int yy,int x1,int x2,int position,int maxpositions,int options,int retevent);
		virtual ~HScrollBar() { delete Blank;
								delete RtArrow;
								delete LtArrow; }

		virtual void SetMaxPos(int maxpositions,bool redraw = true)	// set the maximum position
						{ maxpos = maxpositions;
							SetPos(pos,redraw); }

		virtual void SetPos(int position,bool redraw = true)			// set the current position
						{
						pos = position;
						if (pos>maxpos) pos = maxpos;
						if (pos<0) pos = 0;
						if (redraw) Refresh();
						}

		virtual void AdjustGlobalCoords(int dx,int dy);	// adjust coordinates
		virtual void DeSelect();					// deselect
		virtual void Refresh();
		virtual void HandleEvent(Event &e);
		virtual void RefreshBlank();					// draw the blank button (quickly)

		virtual void SetPosn(int max_posn,int posn)	// set the current & maximum positions
			{ maxpos = max_posn; pos = posn; }
	};

#endif