//	-------------------------------------------------------------------------
//
//	TEXTBOX.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains defines for a Box class as well as
//					some simple Box objects such as TextBox, DirBox,
//					etc. All Box's are derived from base Accessory.
//
//	-------------------------------------------------------------------------

#ifndef __textbox_hpp
#define __textbox_hpp

#include "event.hpp"
#include "accessry.hpp"
#include "scrolbar.hpp"

// options for textboxes

#define textbox_DONT_CALC	0x0100		// don't calculate string bits (used internally)
#define textbox_SINGLECLICK	0x0200		// responds to a single mouse click
#define textbox_DOUBLECLICK 0x0400		// responds to a double mouse click

// the maximum number of rows in a textbox

#define TEXTBOX_MAX_ELEMS	1024		// 1024 elements max

//	-------------------------------------------------------------------------
//
//	CLASS: 		TextBox
//	DESC: 		This is a class which displays a number of text strings on
//				a row per row basis chosen from a larger array of strings.
//				A VScrollBar is used to allow the user to "slide" the display
//				up or down through the strings.
//
//	PARAMS:     str - The actual data used in the display formatted as:
//					  	"JAN/FEB/MAR/APR/MAY/JUN/JUL/AUG"
//						where "JAN","FEB","MAR",etc. are the rows displayed
//						and "/" is the row terminator.
//
//	-------------------------------------------------------------------------

class TextBox : public Accessory
	{
	private:
		char*	rowstart[TEXTBOX_MAX_ELEMS];	// pointer to the start of each 'row'
		int		rowsize[TEXTBOX_MAX_ELEMS];	// the number of chars in each 'row'

		int		row_height;	// the height of each row

		VScrollBar* vbar;	// the vertical scroll bar
		int vbar_result;	// the result from the vscrollbar
		int	vbar_width;		// the width of the vscrollbar

		int num_rows;		// the number of rows to display
		int num_cols;		// the number of cols to display

		int max_rows;		// the max number of rows in the element list

		int row_offset;		// the row offset
		int last_row_offset;// the last row offset (used in draw's)
		int	max_row_offset;	// the maximum row offset

		int row_posn;		// the row position (from top of box)
		int last_row_posn;	// the last row position

		int timesclicked; 	// # of times lmb has been clicked on a row

	public:
		String* rowstr;		// pointer to the info displayed in box

		TextBox(int x1,int y1,int numrows,int numcols,
				int rownum,String* str,
				int options,int retevent);
		virtual ~TextBox();

		virtual void AdjustCoords(int dx,int dy);  	// adjusts coords
		virtual void CalcRowSizes();				// calcs start location
													// and length of each row
													// (used internally)

		virtual void IncRowPosn(bool refresh = true);	// select the next row
		virtual void DecRowPosn(bool refresh = true);	// select the prev row
		virtual void SetRowPosn(int posn,bool update_vbar = true); // select a certain row posn
		virtual int  GetRowPosn(grPoint& pt);				// get the row posn

		virtual void DrawRow( int rownum );			// draw a specified row
		virtual void DrawAllRows();					// draw all rows

		virtual void Draw();
		virtual void Refresh();
		virtual void HandleEvent(Event &e);
	};

#endif