//	-------------------------------------------------------------------------
//
//	SCROLBOX.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains the definition for the basic
//					ScrollBox accessory
//
//	-------------------------------------------------------------------------

#ifndef __scrolbox_hpp
#define __scrolbox_hpp

#include "accessry.hpp"
#include "graphics.hpp"

// button options

#define	ac_CentreScroll			0x00000002
#define	ac_EndScroll			0x00000004

//	-------------------------------------------------------------------------
//
//	CLASS: 		ScrollBox
//	DESC: 		This is a simple 2-way scrollbox
//
//	NOTES:		Assumes ODD number of rows
//
//	-------------------------------------------------------------------------

class	ScrollBox : public Accessory
	{
	protected:
		int		num_rows;
		int		max_rows;

		int		current_row;	// currently highlighted row

		int 	row_pos;
		int 	max_row_pos;

		int		row_offset;		// offset to 1st row of display
		int		max_row_offset;

		int		centre_pos;

		grImage*	ti_tile;
		grImage*	lo_tile;
		grImage*	hi_tile;

		int			ti_bstyle;
		int			lo_bstyle;
		int			hi_bstyle;

		char    title_bg;
		char	title_text;
		char	title_border;
		char	lo_bg;
		char	lo_text;
		char	lo_border;
		char	hi_bg;
		char	hi_text;
		char	hi_border;

	public:
		ScrollBox(int x1,int y1,int x2,int y2,
					int rows,int maxrows,
					grImage* title_tile, int title_bstyle,
					grImage* normal_tile, int normal_bstyle,
					grImage* highlighted_tile, int highlighted_bstyle,
					int options,
					int hit_row_event,int hit_hi_row_event);
		~ScrollBox();

		//	--- data manip functions

		virtual	int		GetCurrentRow();

		virtual	void	SetRows(int rows);
		virtual	void	SetMaxRows(int maxrows);
		virtual	void	SetEvents(int hit_row_event,int hit_hi_row_event);

		//	---	scroll functions

		virtual	void	ScrollUp(int rows=1);
		virtual	void	ScrollDn(int rows=1);

		virtual	void	ScrollTo(int row);

		virtual	void	ScrollTop();
		virtual	void	ScrollBottom();

		//	--- draw functions

		virtual void	DrawTitle();

		virtual	void	DrawRow(int row,bool auto_fill=true);
		virtual	void	DrawHighlightedRow(int row,bool auto_fill=true);

		virtual	void	DrawAllRows();
		virtual	void	RefreshRows(int row1,int row2);

		//	---	accessory functions

		virtual	int		WhichRowSelected(grPoint& ms_pt);

		virtual	void	Draw();
		virtual	void	Draw(grVirtualBuffer* vb);

		virtual	void	HandleEvent(Event& e);

	};

#endif