//	-------------------------------------------------------------------------
//
//	BORDERS.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer (Taken mostly from from FONTS.CPP)
//
//	DESCRIPTION:	This file defines the border functions used
//					by the game development library.
//
//	-------------------------------------------------------------------------
#include	"borders.hpp"
#include 	"tiles.hpp"

#include 	"resource.hpp"
//#include 	"program.hpp"

// list of border resource files current borders are loaded from
ResourceFile*	border_file[MAX_LOADABLE_BORDERS];

// list of currently loaded borders
grBorder**		border_list[MAX_LOADABLE_BORDERS];

// and the number of borders loaded
int				border_count = 0;

//	-------------------------------------------------------------------------
//
//	FUNC:		grInitBorder
//	DESC:		load a border into memory and return a tag value
//
//	PARAMS:		bordername = path and name of the border in DOS
//
// --------------------------------------------------------------------------
int	grInitBorder(char* bordername)
	{
	return grInitBorder("",bordername);
	}

int	grInitBorder(char* path,char* bordername)
	{
	// check if we have available space in border list
	if (border_count >= MAX_LOADABLE_BORDERS) return border_TooManyBorders;

	// create a border resource
	border_file[border_count] = new ResourceFile(path,bordername);

//	if (!border_file[border_count]) DBG(200,"ERROR: COULD NOT 'NEW' RESOURCEFILE >%s%s<",path,bordername);
	if (!border_file[border_count]) return border_InitBorderFailed;

	// skip out if the border doesn't exist
//	if (!border_file[border_count]->Exists()) DBG(210,"ERROR: RESOURCEFILE >%s%s< DOES NOT EXIST",path,bordername);
	if (!border_file[border_count]->Exists()) return border_NoSuchBorder;

	// load all resources from bordername into borderpointer
	border_list[border_count] = (grBorder**) border_file[border_count]->ReadAllRecords();

	// if we couldn't load the border images
	if (border_list[border_count]==NULL)
		{
//		DBG(220,"ERROR: COULD NOT READ RESOURCEFILE >%s%s<",path,bordername);
		// clean up
		delete border_file[border_count]; border_file[border_count] = NULL;

		// and return an error message
		return border_InitBorderFailed;
		}

	// we've now added a border to the list
	border_count++;

	// return border number used
	return border_count - 1;
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grExitBorder
//	DESC:		delete the last installed border from memory
//	RETURN:		if there are any borders left in memory
//
// --------------------------------------------------------------------------
void	grExitBorder()
	{
	// if there are no borders present then return
	if (border_count<1) return;

	// decrease border count as 1 border loaded == 1 border_count  	[index == 0]
	border_count --;

	// delete the border elements
	border_list[border_count] = (grBorder**) border_file[border_count]->FreeAllRecords((char**)border_list[border_count]);

	// delete our border file pointer
	delete border_file[border_count];
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grExitALLBorders
//	DESC:		delete all installed borders from memory
//
// --------------------------------------------------------------------------
void grExitALLBorders()
	{
	// free all resources of all borders until border_present is false
	while (border_count > 0) grExitBorder();
	}

#define	brd_topleft		0
#define	brd_topright	1
#define	brd_botleft		2
#define	brd_botright	3
#define	brd_top			4
#define	brd_left		5
#define	brd_right		6
#define	brd_bottom		7

// draw a border in a virtual buffer
void grDrawBorderVirtual(grVirtualBuffer* vb,
				int x1,int y1,int x2,int y2,int bstyle)
	{
	if (border_list[bstyle]==NULL) return;

	// use temporaries
	grImage** pic = (grImage**) border_list[bstyle];

	// draw top-left corner
	grOverlayImageVirtual(vb,x1,y1,
							x1+pic[brd_topleft]->hdr.width-1,
							y1+pic[brd_topleft]->hdr.height-1,
							0,pic[brd_topleft]);

	// draw top-right corner
	grOverlayImageVirtual(vb,x2-pic[brd_topright]->hdr.width+1,y1,
							x2,y1+pic[brd_topright]->hdr.height-1,
							0,pic[brd_topright]);

	// draw bottom-left corner
	grOverlayImageVirtual(vb,x1,y2-pic[brd_botleft]->hdr.height+1,
							x1+pic[brd_botleft]->hdr.width-1,y2,
							0,pic[brd_botleft]);

	// draw bottom-right corner
	grOverlayImageVirtual(vb,x2-pic[brd_botright]->hdr.width+1,y2-pic[brd_botright]->hdr.height+1,
							 x2,y2,
							 0,pic[brd_botright]);

	// draw top frame
	grTileRectVirtual(vb,x1+pic[brd_topleft]->hdr.width,
						 y1,
						 x2-pic[brd_topright]->hdr.width,
						 y1+pic[brd_top]->hdr.height-1,
						 0,pic[brd_top],NO_ROTATE);

	// draw left frame
	grTileRectVirtual(vb,x1,
						 y1+pic[brd_topleft]->hdr.height,
						 x1+pic[brd_left]->hdr.width-1,
						 y2-pic[brd_botleft]->hdr.height+1,
						 0,pic[brd_left],NO_ROTATE);

	// draw right frame
	grTileRectVirtual(vb,x2-(pic[brd_right]->hdr.width-1),
						 y1+pic[brd_topright]->hdr.height,
						 x2,
						 y2-pic[brd_botright]->hdr.height+1,
						 0,pic[brd_right],NO_ROTATE);

	// draw bottom frame
	grTileRectVirtual(vb,x1+pic[brd_botleft]->hdr.width,
						 y2-(pic[brd_bottom]->hdr.height-1),
						 x2-pic[brd_botright]->hdr.width,
						 y2,
						 0,pic[brd_bottom],NO_ROTATE);
	}


void grDrawBorderSectionVirtual(grVirtualBuffer* vb,
				int x1,int y1,int x2,int y2,
				int s_x1,int s_y1,int s_x2,int s_y2,
				int bstyle)
	{
	if (border_list[bstyle]==NULL) return;

	// use temporaries
	grImage** pic = (grImage**) border_list[bstyle];

	// draw top-left corner
	if ((s_x1<x1+pic[brd_topleft]->hdr.width-1) ||
		(s_y1<y1+pic[brd_topleft]->hdr.height-1))

		grOverlayImageVirtual(vb,x1,y1,
							x1+pic[brd_topleft]->hdr.width-1,
							y1+pic[brd_topleft]->hdr.height-1,
							0,pic[brd_topleft]);

	// draw top-right corner
	if ((s_x2>x2-pic[brd_topright]->hdr.width+1) ||
		(s_y1>y1+pic[brd_topright]->hdr.height-1))

		grOverlayImageVirtual(vb,x2-pic[brd_topright]->hdr.width+1,y1,
							x2,y1+pic[brd_topright]->hdr.height-1,
							0,pic[brd_topright]);

	// draw bottom-left corner
	if ((s_x1<x1+pic[brd_botleft]->hdr.width-1) ||
		(s_y2>y2-pic[brd_botleft]->hdr.height+1))

		grOverlayImageVirtual(vb,x1,y2-pic[brd_botleft]->hdr.height+1,
							x1+pic[brd_botleft]->hdr.width-1,y2,
							0,pic[brd_botleft]);

	// draw bottom-right corner
	if ((s_x1>x2-pic[brd_botright]->hdr.width+1) ||
		(s_y2>y2-pic[brd_botright]->hdr.height+1))

		grOverlayImageVirtual(vb,x2-pic[brd_botright]->hdr.width+1,
							 y2-pic[brd_botright]->hdr.height+1,
							 x2,y2,
							 0,pic[brd_botright]);

	// draw top frame
	if (s_y1<y1+pic[brd_top]->hdr.height-1)
		grTileRectVirtual(vb,x1+pic[brd_topleft]->hdr.width,
						 y1,
						 x2-pic[brd_topright]->hdr.width,
						 y1+pic[brd_top]->hdr.height-1,
						 0,pic[brd_top],NO_ROTATE);

	// draw left frame
	if (s_x1<x1+pic[brd_left]->hdr.width-1)
		grTileRectVirtual(vb,x1,
						 y1+pic[brd_topleft]->hdr.height,
						 x1+pic[brd_left]->hdr.width-1,
						 y2-pic[brd_botleft]->hdr.height+1,
						 0,pic[brd_left],NO_ROTATE);

	// draw right frame
	if (s_x2>x2-pic[brd_right]->hdr.width+1)
		grTileRectVirtual(vb,x2-(pic[brd_right]->hdr.width-1),
						 y1+pic[brd_topright]->hdr.height,
						 x2,
						 y2-pic[brd_botright]->hdr.height+1,
						 0,pic[brd_right],NO_ROTATE);

	// draw bottom frame
	if (s_y2>y2-pic[brd_bottom]->hdr.height+1)
		grTileRectVirtual(vb,x1+pic[brd_botleft]->hdr.width,
						 y2-(pic[brd_bottom]->hdr.height-1),
						 x2-pic[brd_botright]->hdr.width,
						 y2,
						 0,pic[brd_bottom],NO_ROTATE);
	}