//	-------------------------------------------------------------------------
//
//	BORDER.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file declares the border functions used
//					by the game development library.
//					Mostly taken from BORDERS.HPP
//
//	-------------------------------------------------------------------------

#ifndef __borders_hpp
#define __borders_hpp

#include	"graphics.hpp"

// -----------------
//
//	NOTES:	* this is a simple way of using borders, consisting of a
//			  ResourceFile containing 8 images as follows:
//
//				top-left,top-right,bottom-left,bottom-right,top,left,right,bottom
//
//			* at the moment usage is:
//
//				int bstyle = grInitBorder(PROGRAM_PATH,"RESOURCE\\MAP.BRD");
//				.
//              .
//				.
//				grDrawBorder(x1,y1,x2,y2,bstyle);
//				.
//				.
//				.
//				grExitBorder();
//
//			* borders assume transparent color is 0
//
// -----------------

// our borders are really just a grImage
#define 	grBorder					grImage

// as our border headers really just a grImageHeader
#define 	grBorderHeader				grImageHeader

// border engine error messages returned from grBorder... functions
#define		border_NoError				0
#define		border_NoSuchBorder			-1
#define		border_TooManyBorders		-2
#define		border_InitBorderFailed		-3
#define		border_NoBordersLoaded		-4

#define		border_OutOfTags			-5
#define		border_OutOfBounds			-6
#define		border_OutOfMemory			-7
#define		border_NoSuchCharacter		-8
#define		border_FileNotFound			-9
#define		border_Error				-10

// the maximum number of borders loadable at any one time
#define		MAX_LOADABLE_BORDERS		20

//-------------------------------------------------------------------------

// load a border into memory and return a tag value
int		grInitBorder(char* bordername);
int		grInitBorder(char* path,char* bordername);

// delete the last installed border from memory
void	grExitBorder();

// delete all installed borders from memory
void	grExitALLBorders();


// this function will draw a border in a virtual buffer
// fitting all images flush with the x1,y1,x2,y2 coords passed as the
// outside boundary....

#define	grDrawBorder(X1,Y1,X2,Y2,BSTYLE) \
			grDrawBorderVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),(BSTYLE))

void grDrawBorderVirtual(grVirtualBuffer* vb,
				int x1,int y1,int x2,int y2,int bstyle);


// this function will draw a SECTION of a border in a virtual buffer
// using the x1,y1,x2,y2 coords passed as the normal border boundary
// (as used in grDrawBorderVirtual), but this function uses a 'best fit'
// method to draw only whats inside the s_x1,s_y1,s_x2,s_y2
// section.....
#define	grDrawBorderSection(X1,Y1,X2,Y2, \
					S_X1,S_Y1,S_X2,S_Y2, \
					BSTYLE) \
			grDrawBorderSectionVirtual( grSCREEN,(X1),(Y1),(X2),(Y2), \
					(S_X1),(S_Y1),(S_X2),(S_Y2),(BSTYLE))

void grDrawBorderSectionVirtual(grVirtualBuffer* vb,
				int x1,int y1,int x2,int y2,
				int s_x1,int s_y1,int s_x2,int s_y2,
				int bstyle);

// this function will draw a border in a virtual buffer
// fitting all images flush with the x1,y1,x2,y2 coords passed as the
// outside boundary....

#define	grDrawCachedBorder(X1,Y1,X2,Y2,BSTYLE) \
			grDrawCachedBorderVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),(BSTYLE))

void grDrawCachedBorderVirtual(grVirtualBuffer* vb,
				int x1,int y1,int x2,int y2,int bstyle);


// this function will draw a SECTION of a border in a virtual buffer
// using the x1,y1,x2,y2 coords passed as the normal border boundary
// (as used in grDrawBorderVirtual), but this function uses a 'best fit'
// method to draw only whats inside the s_x1,s_y1,s_x2,s_y2
// section.....
#define	grDrawCachedBorderSection(X1,Y1,X2,Y2, \
					S_X1,S_Y1,S_X2,S_Y2, \
					BSTYLE) \
			grDrawCachedBorderSectionVirtual( grSCREEN,(X1),(Y1),(X2),(Y2), \
					(S_X1),(S_Y1),(S_X2),(S_Y2),(BSTYLE))

void grDrawCachedBorderSectionVirtual(grVirtualBuffer* vb,
				int x1,int y1,int x2,int y2,
				int s_x1,int s_y1,int s_x2,int s_y2,
				int bstyle);

#endif