#include "circles.hpp"
#include "points.hpp"
#include "lines.hpp"

#include "mouse.hpp"

// --------------------------------------------------------------------------
//
//	FUNC:		grDrawCircleVirtual()
// 	DESC:		draws a circle in a virtual buffer with origin and radius
//
// --------------------------------------------------------------------------
void 	grDrawCircleVirtual(grVB* vb,int xOrig, int yOrig, int radius, int color)
	{
	if (vb==NULL) return;

	int x = 0;
	int y = radius;
	int error = 0;

	while (x <= y)
		{
			{
			int x1,y1;

			x1 = xOrig+x;
			y1 = yOrig+y;
			if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
				grPutPixelVirtual(vb, x1,y1, color);

			x1 = xOrig+x;
			y1 = yOrig-y;
			if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
				grPutPixelVirtual(vb, x1,y1, color);

			x1 = xOrig-x;
			y1 = yOrig+y;
			if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
				grPutPixelVirtual(vb, x1,y1, color);

			x1 = xOrig-x;
			y1 = yOrig-y;
			if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
				grPutPixelVirtual(vb, x1,y1, color);
			}

			{
			int x1,y1;

			x1 = xOrig+y;
			y1 = yOrig+x;
			if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
				grPutPixelVirtual(vb, x1,y1, color);

			x1 = xOrig+y;
			y1 = yOrig-x;
			if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
				grPutPixelVirtual(vb, x1,y1, color);

			x1 = xOrig-y;
			y1 = yOrig+x;
			if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
				grPutPixelVirtual(vb, x1,y1, color);

			x1 = xOrig-y;
			y1 = yOrig-x;
			if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
				grPutPixelVirtual(vb, x1,y1, color);
			}

//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END

		error += 2*x + 1;
		x += 1;

		if (error >= y)
			{
			error -= 2*y - 1;
			y -= 1;
			}
		}
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grDrawFilledCircleVirtual()
// 	DESC:		draws a circle in a virtual buffer with origin and radius
//
// --------------------------------------------------------------------------
void grDrawFilledCircleVirtual(grVB* vb,int xOrig, int yOrig, int radius, int color)
	{
	if (vb==NULL) return;

	int x = 0;
	int y = radius;
	int error = 0;

	while (x <= y)
		{
			{
			int x1,x2,y1;
			x1 = xOrig-x;
			x2 = xOrig+x;

			y1 = yOrig+y;
			if ((y1>=0) && (y1<vb->hdr.height))
				memset(&vb->data[Index(vb,x1,y1)],color,x2-x1+1);

			y1 = yOrig-y;
			if ((y1>=0) && (y1<vb->hdr.height))
				memset(&vb->data[Index(vb,x1,y1)],color,x2-x1+1);
			}

			{
			int x1,x2,y1;
			x1 = xOrig-y;
			x2 = xOrig+y;

			y1 = yOrig+x;
			if ((y1>=0) && (y1<vb->hdr.height))
				memset(&vb->data[Index(vb,x1,y1)],color,x2-x1+1);

			y1 = yOrig-x;
			if ((y1>=0) && (y1<vb->hdr.height))
				memset(&vb->data[Index(vb,x1,y1)],color,x2-x1+1);
			}

//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END

		error += 2*x + 1;
		x += 1;

		if (error >= y)
			{
			error -= 2*y - 1;
			y -= 1;
			}
		}
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grDrawEllipseVirtual()
// 	DESC:		draws an ellipse in a virtual buffer with origin, a and b
//				radius
//
// --------------------------------------------------------------------------
void grDrawEllipseVirtual(grVB* vb,int xOrig, int yOrig, int xRadius, int yRadius, int color)
	{
	if (vb==NULL) return;

	// check for horizontal or vertical lines
	if (yRadius<1) { grDrawHLineVirtual(vb,yOrig,xOrig-xRadius,xOrig+xRadius,color); return; }
	if (xRadius<1) { grDrawVLineVirtual(vb,xOrig,yOrig-yRadius,yOrig+yRadius,color); return; }

	int aSqrd = xRadius*xRadius;
	int bSqrd = yRadius*yRadius;
	int twoASqrd = 2*aSqrd;
	int twoBSqrd = 2*bSqrd;

		// plot the octant from top to top-right
		{
		int x = 0;
		int y = yRadius;
		int twoXTimesBSqrd = 0;
		int twoYTimesASqrd = y * twoASqrd;

		// yRadius^2 * x^2 + xRadius^2 * y^2 - xRadius^2 * yRadius^2 - xRadius^2 * y
		int error = -y * aSqrd;

		while (twoXTimesBSqrd <= twoYTimesASqrd)
			{
				{
				int x1,y1;

				x1 = xOrig+x;
				y1 = yOrig+y;
				if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
					grPutPixelVirtual(vb, x1,y1, color);

				x1 = xOrig+x;
				y1 = yOrig-y;
				if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
					grPutPixelVirtual(vb, x1,y1, color);

				x1 = xOrig-x;
				y1 = yOrig+y;
				if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
					grPutPixelVirtual(vb, x1,y1, color);

				x1 = xOrig-x;
				y1 = yOrig-y;
				if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
					grPutPixelVirtual(vb, x1,y1, color);
				}

			x += 1;
			twoXTimesBSqrd += twoBSqrd;
			error += twoXTimesBSqrd - bSqrd;

//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END

			if (error >= 0)
				{
				y -= 1;
				twoYTimesASqrd -= twoASqrd;
				error -= twoYTimesASqrd;
				}
			}
		}

		// plot the octant from right to top-right
		{
		int x = xRadius;
		int y = 0;
		int twoXTimesBSqrd = x * twoBSqrd;
		int twoYTimesASqrd = 0;

		// yRadius^2 * x^2 + xRadius^2 * y^2 - xRadius^2 * yRadius^2 - yRadius^2 * x
		int error = -x * bSqrd;

		while (twoYTimesASqrd <= twoXTimesBSqrd)
			{
				{
				int x1,y1;

				x1 = xOrig+x;
				y1 = yOrig+y;
				if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
					grPutPixelVirtual(vb, x1,y1, color);

				x1 = xOrig+x;
				y1 = yOrig-y;
				if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
					grPutPixelVirtual(vb, x1,y1, color);

				x1 = xOrig-x;
				y1 = yOrig+y;
				if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
					grPutPixelVirtual(vb, x1,y1, color);

				x1 = xOrig-x;
				y1 = yOrig-y;
				if ((x1>=0) && (x1<vb->hdr.width) && (y1>=0) && (y1<vb->hdr.height))
					grPutPixelVirtual(vb, x1,y1, color);
				}

			y += 1;
			twoYTimesASqrd += twoASqrd;
			error += twoYTimesASqrd - aSqrd;

//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END

			if (error >= 0)
				{
				x -= 1;
				twoXTimesBSqrd -= twoBSqrd;
				error -= twoXTimesBSqrd;
				}
			}
		}
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grDrawFilledEllipseVirtual()
// 	DESC:		draws an ellipse in a virtual buffer with origin, a and b
//				radius
//
// --------------------------------------------------------------------------
void grDrawFilledEllipseVirtual(grVB* vb,int xOrig, int yOrig, int xRadius, int yRadius, int color)
	{
	if (vb==NULL) return;

	// check for horizontal or vertical lines
	if (yRadius<1) { grDrawHLineVirtual(vb,yOrig,xOrig-xRadius,xOrig+xRadius,color); return; }
	if (xRadius<1) { grDrawVLineVirtual(vb,xOrig,yOrig-yRadius,yOrig+yRadius,color); return; }

	int aSqrd = xRadius*xRadius;
	int bSqrd = yRadius*yRadius;
	int twoASqrd = 2*aSqrd;
	int twoBSqrd = 2*bSqrd;

		// plot the octant from top to top-right
		{
		int x = 0;
		int y = yRadius;
		int twoXTimesBSqrd = 0;
		int twoYTimesASqrd = y * twoASqrd;

		// yRadius^2 * x^2 + xRadius^2 * y^2 - xRadius^2 * yRadius^2 - xRadius^2 * y
		int error = -y * aSqrd;

		while (twoXTimesBSqrd <= twoYTimesASqrd)
			{
				{
				int x1,x2,y1;
				x1 = xOrig-x;
				x2 = xOrig+x;

				y1 = yOrig+y;
				if ((y1>=0) && (y1<vb->hdr.height))
					memset(&vb->data[Index(vb,x1,y1)],color,x2-x1+1);

				y1 = yOrig-y;
				if ((y1>=0) && (y1<vb->hdr.height))
					memset(&vb->data[Index(vb,x1,y1)],color,x2-x1+1);
				}

			x += 1;
			twoXTimesBSqrd += twoBSqrd;
			error += twoXTimesBSqrd - bSqrd;

//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END

			if (error >= 0)
				{
				y -= 1;
				twoYTimesASqrd -= twoASqrd;
				error -= twoYTimesASqrd;
				}
			}
		}

		// plot the octant from right to top-right
		{
		int x = xRadius;
		int y = 0;
		int twoXTimesBSqrd = x * twoBSqrd;
		int twoYTimesASqrd = 0;

		// yRadius^2 * x^2 + xRadius^2 * y^2 - xRadius^2 * yRadius^2 - yRadius^2 * x
		int error = -x * bSqrd;

		while (twoYTimesASqrd <= twoXTimesBSqrd)
			{
				{
				int x1,x2,y1;
				x1 = xOrig-x;
				x2 = xOrig+x;

				y1 = yOrig+y;
				if ((y1>=0) && (y1<vb->hdr.height))
					memset(&vb->data[Index(vb,x1,y1)],color,x2-x1+1);

				y1 = yOrig-y;
				if ((y1>=0) && (y1<vb->hdr.height))
					memset(&vb->data[Index(vb,x1,y1)],color,x2-x1+1);
				}

			y += 1;
			twoYTimesASqrd += twoASqrd;
			error += twoYTimesASqrd - aSqrd;

//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END

			if (error >= 0)
				{
				x -= 1;
				twoXTimesBSqrd -= twoBSqrd;
				error -= twoXTimesBSqrd;
				}
			}
		}
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grPlotFour()
// 	DESC:		draws a kaleidescope of four points to a virtual buffer
//				around the origin with offset
//				Note: useful for drawing circles and ellipses
//
// --------------------------------------------------------------------------
void grPutFourVirtual(grVB* vb,int xOrig, int yOrig, int xOffset, int yOffset, int color)
	{
	if (vb==NULL) return;

	int x,y;

	x = xOrig+xOffset;
	y = yOrig+yOffset;
	if ((x>=0) && (x<vb->hdr.width) && (y>=0) && (y<vb->hdr.height))
		grPutPixelVirtual(vb, x,y, color);

	x = xOrig+xOffset;
	y = yOrig-yOffset;
	if ((x>=0) && (x<vb->hdr.width) && (y>=0) && (y<vb->hdr.height))
		grPutPixelVirtual(vb, x,y, color);

	x = xOrig-xOffset;
	y = yOrig+yOffset;
	if ((x>=0) && (x<vb->hdr.width) && (y>=0) && (y<vb->hdr.height))
		grPutPixelVirtual(vb, x,y, color);

	x = xOrig-xOffset;
	y = yOrig-yOffset;
	if ((x>=0) && (x<vb->hdr.width) && (y>=0) && (y<vb->hdr.height))
		grPutPixelVirtual(vb, x,y, color);
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grPlotFourFilled()
// 	DESC:		draws a kaleidescope of four points to a virtual buffer
//				around the origin with offset
//				Note: useful for drawing circles and ellipses
//
// --------------------------------------------------------------------------
void grPutFourFilledVirtual(grVB* vb,int xOrig, int yOrig, int xOffset, int yOffset, int color)
	{
	if (vb==NULL) return;

	int x1,x2,y;
	x1 = xOrig-xOffset;
	x2 = xOrig+xOffset;

	y = yOrig+yOffset;
	if ((y>=0) && (y<vb->hdr.height))
		memset(&vb->data[Index(vb,x1,y)],color,x2-x1+1);

	y = yOrig-yOffset;
	if ((y>=0) && (y<vb->hdr.height))
		memset(&vb->data[Index(vb,x1,y)],color,x2-x1+1);
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grPlotEight()
// 	DESC:		draws a kaleidescope of eight points to a virtual buffer
//				around the origin with offset
//				Note: useful for drawing circles
//
// --------------------------------------------------------------------------
void grPutEightVirtual(grVB* vb,int xOrig, int yOrig, int xOffset, int yOffset, int color)
	{
	if (vb==NULL) return;

	grPutFourVirtual(vb, xOrig,yOrig, xOffset,yOffset, color);
	grPutFourVirtual(vb, xOrig,yOrig, yOffset,xOffset, color);
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grPlotEightFilled()
// 	DESC:		draws a kaleidescope of eight points to a virtual buffer
//				around the origin with offset
//				Note: useful for drawing circles
//
// --------------------------------------------------------------------------
void grPutEightFilledVirtual(grVB* vb,int xOrig, int yOrig, int xOffset, int yOffset, int color)
	{
	if (vb==NULL) return;

	grPutFourFilledVirtual(vb, xOrig,yOrig, xOffset,yOffset, color);
	grPutFourFilledVirtual(vb, xOrig,yOrig, yOffset,xOffset, color);
	}

