#ifndef __circles_hpp
#define __circles_hpp

#include "vbuffer.hpp"

// --------------------------------------------------------------------------
//
//	CIRCLE MACROS (BACKWARDS COMPATIBILITY & EASE OF READING)
//
// --------------------------------------------------------------------------

#define grDrawCircle(X,Y,RAD,COLOR) 	\
			grDrawCircleVirtual(grSCREEN,(X),(Y),(RAD),(COLOR))

#define grDrawFilledCircle(X,Y,RAD,COLOR) 	\
			grDrawFilledCircleVirtual(grSCREEN,(X),(Y),(RAD),(COLOR))

#define grDrawEllipse(X,Y,XRAD,YRAD,COLOR) 	\
			grDrawEllipseVirtual(grSCREEN,(X),(Y),(XRAD),(YRAD),(COLOR))

#define grDrawFilledEllipse(X,Y,XRAD,YRAD,COLOR) 	\
			grDrawFilledEllipseVirtual(grSCREEN,(X),(Y),(XRAD),(YRAD),(COLOR))


#define grPutFour(X,Y,XOFS,YOFS,COLOR)	\
			grPutFourVirtual(grSCREEN,(X),(Y),(XOFS),(YOFS),(COLOR))

#define grPutFourFilled(X,Y,XOFS,YOFS,COLOR)	\
			grPutFourFilledVirtual(grSCREEN,(X),(Y),(XOFS),(YOFS),(COLOR))

#define grPutEight(X,Y,XOFS,YOFS,COLOR)	\
			grPutEightVirtual(grSCREEN,(X),(Y),(XOFS),(YOFS),(COLOR))

#define grPutEightFilled(X,Y,XOFS,YOFS,COLOR)	\
			grPutEightFilledVirtual(grSCREEN,(X),(Y),(XOFS),(YOFS),(COLOR))

// --------------------------------------------------------------------------
//
//	CIRCLE IMPLEMENTATION FUNCTIONS
//
// --------------------------------------------------------------------------


// ---	draw a circle in a virtual buffer with specified origin,
//		radius and color

void 	grDrawCircleVirtual(grVB* vb,
						int xOrig, int yOrig,
						int radius,
						int color );


// ---	draw a filled circle in a virtual buffer with specified origin,
//		radius and color

void 	grDrawFilledCircleVirtual(grVB* vb,
						int xOrig, int yOrig,
						int radius,
						int color );


// ---	draw an ellipse in a virtual buffer with origin, a and b radius,
//		and color

void 	grDrawEllipseVirtual(grVB* vb,
						int xOrig, int yOrig,
						int xRadius, int yRadius,
						int	color );


// ---	draw a filled ellipse in a virtual buffer with origin,
//		a and b radius, and color

void 	grDrawFilledEllipseVirtual(grVB* vb,
						int xOrig, int yOrig,
						int xRadius, int yRadius,
						int color );


// ---	draws a kaleidescope of four points in a virtual buffer
//		around the origin and offset, using color

void 	grPutFourVirtual(grVB* vb,
						int xOrig, int yOrig,
						int xOffset, int yOffset,
						int color );


// ---	draws a kaleidescope of four points to a virtual buffer
//		around the origin with offset, using color

void 	grPutFourFilledVirtual(grVB* vb,
						int xOrig, int yOrig,
						int xOffset, int yOffset,
						int color );


// ---	draws a kaleidescope of eight points to a virtual buffer
//		around the origin with offset, using color

void 	grPutEightVirtual(grVB* vb,
						int xOrig, int yOrig,
						int xOffset, int yOffset,
						int color );


// ---	draws a kaleidescope of eight points to a virtual buffer
//		around the origin with offset, using color

void 	grPutEightFilledVirtual(grVB* vb,
						int xOrig, int yOrig,
						int xOffset, int yOffset,
						int color );

#endif