#include "debug.hpp"

#include "graphics.hpp"
#include "program.hpp"
#include "resmgr.hpp"

#include "stdarg.h"
#include "stdio.h"
#include "conio.h"
#include "malloc.h"


bool heap_corrupted = false;

char chkbuf[1024];

#ifdef __BORLANDC__
#define _heapchk		heapcheck
#endif

// =========================================================================
//
//	DEBUGGING FUNCTIONS
//
// =========================================================================
//
//	FUNC:	DBG()
//	DESC:	clears a rectangle & draws a string on the graphics screen
//			int LIGHTRED
//
//	PARAMS:	y	- the y coord to draw the string
//			fmt	- the format string (same as in printf)
//			... - any other parameters used by the format string
//
// --------------------------------------------------------------------------
void 	DBG(int y,char *fmt, ...)
	{
	if (CurrentFont==-1) return;

	va_list	argptr;			// pointer to argument list
	char	buf[255];		// buffer to store transposed string
	int 	start = 0;		// start of buffer output (used for checking heap)

	va_start(argptr,fmt);	// figure out the argument starting address

//	if (_heapchk()!=_HEAPOK)
//		{
//		sprintf(buf,"HEAP CORRUPT!! ");
//		start = 15;
//		}

	// note: disable if debugging mouse
//	mouse_UpdateCursor();

	vsprintf(&buf[start],fmt,argptr);	// shove it all into a buffer

	// we always use the SYSTEM FONT for debugging
	int savefont = grGetFont();
	grSetFont(SYSTEM_FONT);

	// make sure the string will fit!!
	int tmpheight = grFontHeight();
	if (tmpheight<0) return;

	while (y+tmpheight>grMaxy) y--;

	// splat the info into it
	grDrawFilledRect(0,y,639,y+grFontHeight(),grBLACK);
	grDrawRect(0,y,639,y+grFontHeight(),grLIGHTRED);
	grDrawString(0,y,buf);

	// and then splat it onto the screen
	grBlitRegion(0,y,639,y+grFontHeight());

	// restore the current font
	grSetFont(savefont);

	va_end(argptr);				// to perform a normal return from this func
	}
// --------------------------------------------------------------------------
//
//	FUNC:	HEAPCHK()
//	DESC:	Checks the heap to see if it is corrupt & if so, displays the
//			supplied reporting information on the screen.
//
//	PARAMS: fmt	- the format string (same as in printf)
//			... - any other parameters used by the format string
//
// --------------------------------------------------------------------------
void HEAPCHK(char *fmt, ...)
	{
	if (CurrentFont==-1) return;
	if (_heapchk()==_HEAPOK) return;

	if (heap_corrupted) return;

	va_list	argptr;			// pointer to argument list
	int 	start = 0;		// start of buffer output (used for checking heap)

	va_start(argptr,fmt);	// figure out the argument starting address

//	sound(300);

	sprintf(chkbuf,"HEAP CORRUPT!! ");
	start = 15;

	vsprintf(&chkbuf[start],fmt,argptr);	// shove it all into a buffer

	// we always use the SYSTEM FONT for debugging
	int savefont = grGetFont();
	grSetFont(SYSTEM_FONT);

	// and display the text onto the screen
	grDrawFilledRect(0,0,639,grFontHeight()*10,grBLACK);
	grDrawRect(0,0,639,grFontHeight()*10,grLIGHTRED);

	int x = 2;
	int y = 2;

	// display text onto the screen

	int		tmpwidth;		// temporary width
	int 	last = font_START_ASCII + font_NUM_ASCII - 1;
	char 	ch;
	grImage* img;

	// display using cached font
	for (int i=0;chkbuf[i]!='\0';i++)
		{
		ch = chkbuf[i];
		if ((ch<font_START_ASCII) || (ch>last)) continue;

		// *NOTE:: the font is stored in the same order as the ascii set
		// but the first character is the space key 32, the list is contiguous
		// after this for 95 characters. See top of file for full list

		img = grFontImagePtr(CurrentFont,ch-font_START_ASCII);

		// set our temp width
		tmpwidth = img->hdr.width;

		// put image into the virtual buffer
		grOverlay2CImage(x,y,x+tmpwidth-1,y+img->hdr.height-1,grWHITE,img);

		// increment currentx by last char's image width
		x += tmpwidth;

		if ((x>639-2)||(chkbuf[i+1]=='~'))
			{
			y += grFontHeight();
			x = 2;
			}
		}

	grBlitRegion(0,0,639,grFontHeight()*10);

	SYS_Status();

	sound(300);
	delay(1000);
	nosound();

	heap_corrupted = true;
	delay(3000);

	grSetFont(savefont);

	va_end(argptr);				// to perform a normal return from this func
	}

void	DELAY(int milliseconds)
	{
	int inc = milliseconds / 1000000L;
	if (inc<1) inc = 1;

	int tmp = 0;
	while (tmp<milliseconds)
		{
		if (tmp>0) delay(tmp);
		tmp += inc;
//		mouse_UpdateCursor();
		}
	}