//	-------------------------------------------------------------------------
//
//	DEFFILE.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew C. Evans.
//
//	DESCRIPTION:	This file contains definitions for the DEFinitionFILE
//					class
//
//	-------------------------------------------------------------------------

#include	"deffile.hpp"

DEFFile::DEFFile(char* filename)
	: TextFile(filename) {}

DEFFile::DEFFile(String* filename)
	: TextFile(filename) {}

DEFFile::DEFFile(String* path,String* filename)
	: TextFile(path,filename) {}

DEFFile::DEFFile(char* path,char* filename)
	: TextFile(path,filename) {}

String*	DEFFile::ResourceName()
	{
	bool		finished = false;
	String*		resource_name = new String("????????.???",256);

	// open the definition file
	Open();
	SeekTOF();

	do
		{
		// read the next line from the definition file
		String* tmpstr = ReadLine();

		// skip if we've hit EOF
		if (AtEOF()) continue;

		// ignore all blank lines
		if (tmpstr->Length()==0) { delete tmpstr; continue; }

		// check for the resource name & make sure it's uppercase
		finished = tmpstr->Contains("@RESOURCE");
        if (finished)
			{
			resource_name->Set((char*) &tmpstr->Contents()[12]);
			resource_name->ToUpper();
			}

		delete tmpstr;
		}
	// continue till we find all appropriate information or we reach end-of-file
	while (!AtEOF() && !finished);

	return resource_name;
	}

int		DEFFile::ResourceType()
	{
	bool		finished = false;
	int			resource_type = -1;

	// open the definition file
	Open();
	SeekTOF();

	do
		{
		// read the next line from the definition file
		String* tmpstr = ReadLine();

		// skip if we've hit EOF
		if (AtEOF()) continue;

		// ignore all blank lines
		if (tmpstr->Length()==0) { delete tmpstr; continue; }

		// check for the resource name & make sure it's uppercase
		finished = tmpstr->Contains("@TYPE");
        if (finished)
			{
			String* tmpstr1 = new String((char*) &tmpstr->Contents()[12]);
			resource_type = tmpstr1->StringToInt();
			delete tmpstr1;
			}

		delete tmpstr;
		}
	// continue till we find all appropriate information or we reach end-of-file
	while (!AtEOF() && !finished);

	return resource_type;
	}

long	DEFFile::MaxRecords()
	{
	bool		finished = false;
	int			max_records = -1;

	// open the definition file
	Open();
	SeekTOF();

	do
		{
		// read the next line from the definition file
		String* tmpstr = ReadLine();

		// skip if we've hit EOF
		if (AtEOF()) continue;

		// ignore all blank lines
		if (tmpstr->Length()==0) { delete tmpstr; continue; }

		// check for the resource name & make sure it's uppercase
		finished = tmpstr->Contains("@RECORDS");
		if (finished)
			{
			String* tmpstr1 = new String((char*) &tmpstr->Contents()[12]);
			max_records = tmpstr1->StringToInt();
			delete tmpstr1;
			}

		delete tmpstr;
		}
	// continue till we find all appropriate information or we reach end-of-file
	while (!AtEOF() && !finished);

	return max_records;
	}

int		DEFFile::DisplayMode()
	{
	bool		finished = false;
	int			display_mode = -1;

	// open the definition file
	Open();
	SeekTOF();

	do
		{
		// read the next line from the definition file
		String* tmpstr = ReadLine();

		// skip if we've hit EOF
		if (AtEOF()) continue;

		// ignore all blank lines
		if (tmpstr->Length()==0) { delete tmpstr; continue; }

		// check for the resource name & make sure it's uppercase
		finished = tmpstr->Contains("@DISPLAY");
		if (finished)
			{
			String* tmpstr1 = new String((char*) &tmpstr->Contents()[12]);
			if (tmpstr1->Contains("GRAPHICS")) display_mode = GRAPHICS;
			else display_mode = TEXT;
			delete tmpstr1;
			}

		delete tmpstr;
		}
	// continue till we find all appropriate information or we reach end-of-file
	while (!AtEOF() && !finished);

	return display_mode;
	}

String*	DEFFile::Record(int record_number)
	{
	int			record_cnt = 0;
	String*		tmpstr = NULL;

	// --- scan through the file line-by-line, starting at the top??
	Open();
	SeekTOF();

	do
		{
		// get the next line in the definition file....
		tmpstr = ReadLine();

		// skip out if we reached EOF
		if (AtEOF()) continue;

		// skip the line if it's blank
		if (tmpstr->Length()==0) { delete tmpstr; continue; }

		// skip the line if it's a comment
		if (tmpstr->Contents()[0]=='#') { delete tmpstr; continue; }

		// skip the line if it's a declaration
		if (tmpstr->Contents()[0]=='@') { delete tmpstr; continue; }

		// we have found a record
		record_cnt++;

		// delete it if it's not the right one
		if (record_cnt<record_number)
			{
			delete tmpstr;
			tmpstr = NULL;
			}
		}
	// continue till we have read the correct record or
	// we hit the end of the definition file
	while ( (record_cnt<record_number) && !AtEOF() );

	// return either a NULL or a string
	return tmpstr;
	}

String*	DEFFile::RecordList()
	{
	String*		liststr = new String(100000L);

	// --- scan through the file line-by-line, starting at the top??
	Open();
	SeekTOF();

	do
		{
		// get the next line in the definition file....
		String*	tmpstr = ReadLine();

		// skip out if we reached EOF
		if (AtEOF()) continue;

		// skip the line if it's blank
		if (tmpstr->Length()==0) { delete tmpstr; continue; }

		// skip the line if it's a comment
		if (tmpstr->Contents()[0]=='#') { delete tmpstr; continue; }

		// skip the line if it's a declaration
		if (tmpstr->Contents()[0]=='@') { delete tmpstr; continue; }

		// we have found a record
		liststr->Append(tmpstr);		// add the record to our list
		liststr->Append('/');			// put in a separator
		}
	// continue till we hit the end of the definition file
	while ( !AtEOF() );

	// discard last / separator
	liststr->Delete(liststr->Length());

	// return the list string
	return liststr;
	}
