//	-------------------------------------------------------------------------
//
//	EVENT.HPP
//
//	-------------------------------------------------------------------------
//
//	DESCRIPTION:	This file contains defines for the Event object used by
//					the EVENT MANAGER as well as the different types of events
//					possible.
//
//	-------------------------------------------------------------------------

#ifndef __event_hpp
#define __event_hpp

#include "graphics.hpp"
#include "stddefns.hpp"

//	the different types of events

#define		ev_nothing		0x0000
#define		ev_mouse		0x0001
#define		ev_keyboard		0x0002
#define		ev_timer		0x0003
#define		ev_command		0x0004
#define		ev_broadcast	0x0005

// the mouse event definition

struct MouseEvent
	{
	int		event;   	// the actual event
	int		modifiers;	// which modifiers (such as LEFT_ALT,RIGHT_CTRL,etc.)
	grPoint where;		// where the button was pressed
	};

// the keyboard event & relevant defines

struct KeyboardEvent
	{
	int		event;		// which key was pressed
	int		modifiers;	// which modifiers (such as LEFT_ALT,RIGHT_CTRL,etc.)
	grPoint	where;		// where the mouse is!!!
	};

// the timer event & relevant defines

struct TimerEvent
	{
	int		event;		// the event generated
	int		time;		// the actual time when event was generated
	};

struct CommandEvent
	{
	int		event;		// the command event
	void*	data;		// the data being passed with the event
	};

struct BroadcastEvent
	{
	int		event;		// the broadcast message
	void*	data;		// any relevant data to accompany the message
	};

struct Event
	{
	int		where;	// the event handler where the event is going
					// if (where==0) event handlers are called in order
					// of creation (using AddEventHandler)
	int		what;	// the type of event
	union   		// the actual information being passed
		{
		MouseEvent		mouse;
		KeyboardEvent 	keyboard;
		TimerEvent		timer;
		CommandEvent 	command;
		BroadcastEvent	broadcast;
		};
	};

#endif