//	-------------------------------------------------------------------------
//
//	FILE.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file implements the FILE class
//
//	-------------------------------------------------------------------------

#include "file.hpp"
#include "program.hpp"

// global counter
int files_open = 0;


File::File()
	{
	file_path = NULL;
	file_name = NULL;
	file_isopen = false;
	}

File::File(char* filename)
	{
	file_path = new String("");
	file_name = new String(filename);
	file_isopen = false;

	Open();
	}

File::File(String* filename)
	{
	file_path = new String("");
	file_name = new String(filename->Contents());
	file_isopen = false;

	Open();
	}


File::File(char* path,char* filename)
	{
	file_path = new String(path);
	file_name = new String(filename);
	file_isopen = false;

	Open();
	}

File::File(String* path,String* filename)
	{
	file_path = new String(path->Contents());
	file_name = new String(filename->Contents());
	file_isopen = false;

	Open();
	}


File::~File()
	{
	Close();
	if (file_name) { delete file_name; file_name = NULL; }
	if (file_path) { delete file_path; file_path = NULL; }
	}

bool 	File::Exists()
	// returns true if file exists
	{
	// quick check as we can't open unless the file is there!!! ta doods...
	if (file_isopen) return true;

	char buf[256];

	strcpy(buf,file_path->Contents());
	strcat(buf,file_name->Contents());

//	DBG(400,"EXISTS FILE: >%s<",buf);

	FILE* tmp = fopen(buf,"r+b");

//	DBG(410,"EXISTS:: FOPEN() RETURNS %p, TOTAL FILES OPEN == %d",tmp,files_open);

	if (tmp!=NULL) { fclose(tmp); return true; }

	return false;
	}

bool	File::isOpen()
	// returns true if file is open
	{
	return file_isopen;
	}

int		File::Open()
	// opens the file
	{
	// iff already open, return the fact
	if (file_isopen) return file_FileIsOpen;

	// open the file for read+write
	char 	buf[256];

	strcpy(buf,file_path->Contents());
	strcat(buf,file_name->Contents());

//	DBG(0,"OPENING FILE: >%s<",buf);

	file_ptr = fopen(buf,"r+b");

	// set an open flag if true
	file_isopen = (file_ptr!=NULL);
//	if (!file_isopen) DBG(420,"OPEN:: FOPEN() FAILED ON FILE: >%s<, FILES OPEN ==%d",buf,files_open);

	// and return the appropriate error message
	if (file_isopen) { files_open++; return file_NoError; }
	return file_OpenFailed;
	}

int 	File::Close()
	// closes the file
	{
	// iff not open, return the fact
	if (!file_isopen) return file_FileNotOpen;

	// close the file
	if (file_ptr) { fclose(file_ptr); files_open--; }

	// set the open flag
	file_isopen = false;

	// and there's no error (hopefully??) >:-?
	return file_NoError;
	}

int		File::Create()
	// creates the file iff !Exists()
	{
	// iff the file is open, return the fact
	if (file_isopen) return file_FileIsOpen;

	// iff the file already exists, return the fact
	if (Exists()) return file_FileAlreadyExists;

	// create the file
	char 	buf[256];

	strcpy(buf,file_path->Contents());
	strcat(buf,file_name->Contents());

//	DBG(0,"CREATING FILE: >%s<",buf);

	file_ptr = fopen(buf,"wb");

	// and return any appropriate error messages
	if (file_ptr==NULL) return file_CreateFailed;

	// make sure the file is closed now!!!
	fclose(file_ptr);

	// return the fact that the file was created successfully
	return file_NoError;
	}

int		File::Delete()
	// deletes the file iff Exists()
	{
	// iff the file is open, return the fact
	if (file_isopen) return file_FileIsOpen;

	// iff we can delete the file, return the fact
	char 	buf[256];
//	char* buf = &buffer[0];

	strcpy(buf,file_path->Contents());
	strcat(buf,file_name->Contents());

//	DBG(0,"DELETING FILE: >%s<",buf);

	if (remove(buf)==0) return file_NoError;

	// otherwise an error occurred
	return file_DeleteFailed;
	}

int		File::Seek(long pos)
	{
	// skip out if file is not open
	if (!file_isopen) return file_FileNotOpen;

	// attempt to seek to the new pos
	if (fseek(file_ptr,pos,SEEK_SET) != 0) return file_SeekFailed;

	// all is ok
	return file_NoError;
	}

int		File::SeekTOF()
	{
	// skip out if file is not open
	if (!file_isopen) return file_FileNotOpen;

	// attempt to seek to the TOF
	if (fseek(file_ptr,0,SEEK_SET) != 0) return file_SeekFailed;

	// all is ok
	return file_NoError;
	}

int		File::SeekEOF()
	{
	// skip out if the file is not open
	if (!file_isopen) return file_FileNotOpen;

	// attempt to seek to the EOF
	if (fseek(file_ptr,0,SEEK_END) != 0) return file_SeekFailed;

	// all is ok
	return file_NoError;
	}

int		File::Read(char* buf,long size)
	// reads a block of data from a binary file
	{
	// skip out if file is not open
	if (!file_isopen) return file_FileNotOpen;

	// read the appropriate block into buf if poss....
	if (fread(buf,size,1,file_ptr)!=1) return file_ReadFailed;

	// everything is ok
	return file_NoError;
	}

int		File::Write(char* buf,long size)
	{
	// skip out if file is not open
	if (!file_isopen) return file_FileNotOpen;

	// write the line to the file at the current file position
	if (fwrite(buf,size,1,file_ptr)!=1) return file_WriteFailed;

	// all is ok
	return file_NoError;
	}

long	File::FileSize()
	{
	// seek to the end-of-file
	SeekEOF();

	// and return our current file position
	long file_pos = ftell(file_ptr);

	return file_pos;
	}

char*   File::FileName()
    {
	return file_name->Contents();
	}

char*	File::FilePath()
	{
	return file_path->Contents();
	}

bool	File::AtEOF()
	{
	// return true iff at EOF
	return (feof(file_ptr)!=0);
	}
