//	-------------------------------------------------------------------------
//
//	FILE.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains definitions for the FILE class
//
//	-------------------------------------------------------------------------

#ifndef __file_hpp
#define __file_hpp

#include "strings.hpp"

#define file_NoError			0
#define file_FileIsOpen			-1
#define file_FileNotOpen		-2
#define file_FileAlreadyExists	-3
#define file_OpenFailed			-4
#define file_CloseFailed		-5
#define file_CreateFailed		-6
#define file_DeleteFailed		-7
#define	file_SeekFailed			-8
#define file_ReadFailed			-9
#define file_WriteFailed		-10

class File
	{
	public:
						File();
						File(char* filename);
						File(String* filename);
						File(char* path,char* filename);
						File(String* path,String* filename);

		virtual 		~File();

		virtual bool	Exists();		// returns true if file exists
		virtual bool	isOpen();		// returns true if file is open

		virtual int		Open();			// opens the file
		virtual int 	Close();        // closes the file

		virtual	int		Create();		// creates the file iff !Exists()
		virtual	int		Delete();		// deletes the file iff Exists()

		virtual int		Seek(long pos);
		virtual int		SeekTOF();
		virtual	int		SeekEOF();

		virtual int 	Read(char* buf,long size);
		virtual int		Write(char* buf,long size);

		virtual	long	FileSize();
		virtual char*	FileName();
		virtual	char*	FilePath();

		virtual	bool	AtEOF();

	protected:
		FILE*		file_ptr;		// the file pointer
		String* 	file_name;		// file name including full path
		String*		file_path;		// full path of file
		bool		file_isopen;	// true if file is open
	};

#endif
