//	-------------------------------------------------------------------------
//
//	FONTS.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew C. Evans & Darren J. Dwyer
//
//	DESCRIPTION:	This file declares the font functions used
//					by the game development library.
//
//	-------------------------------------------------------------------------

#ifndef __fonts_hpp
#define __fonts_hpp

#include "vbuffer.hpp"
#include "images.hpp"

// --------------------------------------------------------------------------
//
//	FONT DEFINITIONS
//
// --------------------------------------------------------------------------

// our fonts are really just a grImage (temporarily)

#define 	grFont				int
#define 	grFontHeader		grImageHeader

// external use by graphics.cpp
extern		int					CurrentFont;

// font engine error messages returned from grFont... functions
#define		font_NoError			0
#define		font_NoSuchFont			-1
#define		font_TooManyFonts		-2
#define		font_InitFontFailed		-3
#define		font_NoFontsLoaded		-4
#define		font_NoFontSet			-11

#define		font_OutOfTags			-5
#define		font_OutOfBounds		-6
#define		font_OutOfMemory		-7
#define		font_NoSuchCharacter	-8
#define		font_FileNotFound		-9
#define		font_Error				-10

// the maximum number of fonts loadable at any one time
#define		MAX_LOADABLE_FONTS	20

// the ascii set is represented starting from START and the next NUM characters
#define		font_START_ASCII	32		// space " "
#define		font_NUM_ASCII		95		// to the 127th character

// --------------------------------------------------------------------------
//
//	FONT IMPLEMENTATION FUNCTIONS
//
// --------------------------------------------------------------------------


// ---	load a font into memory and return a tag value

int		grInitFont(char* fontname);
int		grInitFont(char* path,char* fontname);


// ---	delete the last installed font from memory

void	grExitFont();


// ---	delete all installed fonts from memory

void	grExitAllFonts();


// ---	get the currently active font number

int		grGetFont();


// ---	set the current font to 'id'

int		grSetFont(int font_id);


// ---	returns the height of the first character in the current font

int		grFontHeight();


// ---	returns the height of 'ch' in the current font

int		grFontCharHeight(char ch);


// ---	returns the max height of any char in the string 'st'
//		(using the current font)

int		grFontHeight(char* st);


// ---	returns the width of 'ch' in the current font

int		grFontCharWidth(char ch);


// ---	returns the width of all characters in 'st' in the current font

int		grFontWidth(char* st);


// ---	draws a string at the specified coordinates using variable args

void 	grDrawString(int x,int y,char* fmt,...);


// ---	draws a 2-color string in a specified 'color'

void 	grDraw2CString(int x,int y,int color,char* fmt,...);


// ---	draws a string and blits it immediately

void	grBlitString(int x,int y,char* fmt,...);


// ---	draws a string and blits it immediately filling background with
//		'bgcolor'

void	grBlitString(int x,int y,int bgcolor,char* fmt,...);


// ---	draws a 2-color string and blits it immediately
//		filling background with 'bgcolor'

void	grBlit2CString(int x,int y,int bgcolor,int txtcolor,char* fmt,...);


// ---	draw a string in a virtual buffer at the specified coordinates
//		using variable args

void 	grDrawStringVirtual(grVB* vb,int x,int y,char* fmt,...);


// ---	draw a string in a virtual buffer at the specified coordinates
//		using specified color

void 	grDraw2CStringVirtual(grVB* vb,int x,int y,int color,char* fmt,...);

#endif