//	-------------------------------------------------------------------------
//
//	GRAPHICS.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHORS:		Darren J. Dwyer
//					Andrew C. Evans  (Font Engine, misc.)
//
//	DESCRIPTION:	This file contains the base level graphics functions used
//					by all of the game development library.
//
//					Note that these functions support double-buffering with
//					automatic mouse cursor updates (software cursor)
//
//	-------------------------------------------------------------------------

#include "graphics.hpp"
#include "mouse.hpp"
#include "realmode.hpp"
#include "fonts.hpp"
#include "palettes.hpp"

#include "conio.h"
#include "mem.h"
#include "dos.h"
#include "stdio.h"
#include "stdlib.h"
#include "malloc.h"

//#define __display_banks	// disable/enable bank switching display
//#define __debugging		// disable/enable debugging information

// --- globally defined variables

int 			grMaxx;				// the maximum x coordinate
int 			grMaxy;				// the maximum y coordinate
int				grScreenWidth;		// the width of the current mode
int				grScreenHeight;		// the height of the current mode
int				grLogicalWidth;		// the width of the logical display
int				grLogicalHeight;	// the height of the logical display

int				grMaxColors;		// the maximum number of colors in mode

long			grMemoryInKb;		// total memory on card in Kb
long			grMemoryInBytes;	// total memory on card in bytes

int				grOrigVideoMode = 3;// the last video mode
int				grCurrentVideoMode;	// the current video mode

int				grNumberOfBanks = 8;// the number of ?Kb banks in current mode
int				grNumberOfPages = 2;// the number of pages in current mode

int				grVideoSpeed = 0;

grVirtualBuffer*	grSCREEN;		// the double-buffer

// =========================================================================
//
//	MOUSE FUNCTIONS
//
// =========================================================================
//
//	NOTE:	No mouse functions exist as they are simple macros reflecting
//			specified mouse functions. Refer to graphics.hpp & mouse.cpp
//			for mouse routines.
//
// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
//
//	MISCELLANEOUS FUNCTIONS
//
// --------------------------------------------------------------------------
void grScaleScreen(int x1,int y1,int x2,int y2,
				   int dx1,int dy1,int scalesize )
	{
	CLIP2(x1,y1,0,0,grMaxx,grMaxy);
	CLIP2(x2,y2,0,0,grMaxx,grMaxy);

	// create and load the image to be scaled
	grImage* img = grCreateImage(x1,y1,x2,y2);
	grGetImage(x1,y1,x2,y2,img);

	grScaleImage(img,dx1,dy1,scalesize);

	grDestroyImage(img);
	}

void grScaleImage(grImage* img, int dx1,int dy1, int scalesize)
	{
	int	width = img->hdr.width;
	int height = img->hdr.height;

	uchar pix;

	// figure out the size of the scaled image
	width = img->hdr.width * scalesize;
	height = img->hdr.height * scalesize;

	// display the scaled image
	for (int y=0;y<img->hdr.height;y++)
		for (int x=0;x<img->hdr.width;x++)
			{
			pix = grGetPixelVirtual(img,x,y);
			grDrawFilledRect(dx1+x*scalesize,dy1+y*scalesize,
							 dx1+(x+1)*scalesize,dy1+(y+1)*scalesize,pix);
			}

	grBlitRegion(dx1,dy1,dx1+width,dy1+height);
	}

//	-------------------------------------------------------------------
//	-------------------------------------------------------------------
//	--- New stuff for GRAPHICS.CPP ------------------------------------
//	-------------------------------------------------------------------
//	-------------------------------------------------------------------


// this function will re-map pixels in the region
// by adjusting their colors to the new color offset
void grReMapColorsVirtual(grVirtualBuffer* vb,
				int x1,int y1,int x2,int y2,
				int color_offset)
	{
	}

//DJD END

