//	-------------------------------------------------------------------------
//
//	GRAPHICS.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file defines the base level graphics functions used
//					by the game development library.
//
//	-------------------------------------------------------------------------

#ifndef __graphics_hpp
#define __graphics_hpp

#include "stddefns.hpp"
#include "vesadrv.hpp"

#include "grmisc.hpp"
#include "vbuffer.hpp"
#include "points.hpp"
#include "lines.hpp"
#include "rects.hpp"
#include "circles.hpp"
#include "images.hpp"
#include "tiles.hpp"
#include "fonts.hpp"
#include "borders.hpp"
#include "palettes.hpp"
#include "sprites.hpp"
#include "debug.hpp"

#include "string.h"	// temporary

// --------------- GLOBAL variable access to display information

extern int 		grMaxx;				// the maximum x coordinate (grScreenWidth-1)
extern int 		grMaxy;				// the maximum y coordinate (grScreenHeight-1)
extern int		grScreenWidth;		// the width of the current screen mode
extern int		grScreenHeight;		// the height of the current screen mode
extern int		grLogicalWidth;		// the width of the logical display
extern int	 	grLogicalHeight;	// the height of the logical display

extern int		grMaxColors;		// the number of colors available for display

extern long		grMemoryInKb;		// total memory on card in Kb
extern long		grMemoryInBytes;	// total memory on card in bytes

extern int		grOrigVideoMode;	// the default video mode on startup
extern int		grCurrentVideoMode;	// the current video mode

extern int		grNumberOfBanks;	// the number of ?Kb banks in current mode
extern int		grNumberOfPages;	// the number of pages in current mode

extern int		grVideoSpeed;		// the speed of the video device (in Hz)

extern grVirtualBuffer* grSCREEN;	// the double-buffer used for
									// screen display

// --------------------------------------------------------------------------
//
//	GRAPHICS DRIVER FUNCTIONS
//
// --------------------------------------------------------------------------

#define	grInitGraphics		grInitVesa
#define	grExitGraphics		grExitVesa

#define	grInitMouse			mouse_InstallHandler
#define	grExitMouse			mouse_RemoveHandler


/*
	DOES THIS LITTLE BIT OF CODE WORK????

#define	draw_(F)	draw_#F

int		line,circle;

		draw_(line)();			// equivalent to: draw_line();
		draw_(circle)();		// equivalent to: draw_circle();

*/


// this function will re-map pixels in the region
// by adjusting their colors to the new color offset
void grReMapColorsVirtual(grVirtualBuffer* vb,
				int x1,int y1,int x2,int y2,
				int color_offset);

//DJD END

void 		grScaleScreen(int x1,int y1,int x2,int y2,
						   int dx1,int dy1,int scalesize );

void 		grScaleImage(grImage* img, int dx1,int dy1, int scalesize);

#endif

