// Andrew Carvel Evans did this. Soon to become an Object/Accessory.

#include "grid.hpp"

Grid::Grid(int posx,int posy,int numx,int numy,int sizx,int sizy,int bt,uchar bc,int et,uchar ec,uchar bkc)
	{
	pos.x = posx;
	pos.y = posy;
	num_cells.x = numx;
	num_cells.y = numy;
	cell_size.x = sizx;
	cell_size.y = sizy;

	borderthickness = bt;
	bordercolour = bc;
	edgethickness = et;
	edgecolour = ec;
	backcolour = bkc;
	}

Grid::Grid(grPoint p,grPoint nc,grPoint cs,int bt,uchar bc,int et,uchar ec,uchar bkc)
	{
	pos = p;
	num_cells = nc;
	cell_size = cs;

	borderthickness = bt;
	bordercolour = bc;
	edgethickness = et;
	edgecolour = ec;
	backcolour = bkc;
	}

Grid::Grid(int posx,int posy,int numx,int numy,int sizx,int sizy)
	{
	pos.x = posx;
	pos.y = posy;
	num_cells.x = numx;
	num_cells.y = numy;
	cell_size.x = sizx;
	cell_size.y = sizy;

	borderthickness = 1;	// default to 1 border thickness
	bordercolour = 0;		// default to invisible borders
	edgethickness = 1;		// default to 1 edge thickness
	edgecolour = 0;			// default to invisible edges
	backcolour = 0;	// default to invisible background
	}

Grid::Grid(grPoint p,grPoint nc,grPoint cs)
	{
	pos = p;
	num_cells = nc;
	cell_size = cs;

	borderthickness = 1;	// default to 1 border thickness
	bordercolour = 0;		// default to invisible borders
	edgethickness = 1;		// default to 1 edge thickness
	edgecolour = 0;			// default to invisible edges
	backcolour = 0;	// default to invisible background
	}

void		Grid::Draw(int options)
	{
	grPoint br = BottomRight();

	int		xx,yy,i,x,y;

	if ((options&grid_Overlay) != grid_Overlay)
		grDrawFilledRect(pos.x,pos.y, br.x,br.y, backcolour);

	// draw horizontal borders
	for (i=0; i<borderthickness; i++)
		{
		grDrawHLine(pos.y+i, pos.x, br.x, bordercolour);
		grDrawHLine(br.y-i, pos.x, br.x, bordercolour);
		}

	for (yy=1; yy<num_cells.y; yy++)
		{
		y = pos.y + (yy-1)*edgethickness + yy*cell_size.y + borderthickness;

		for (i=0; i<edgethickness; i++)
			grDrawHLine(y+i, pos.x, br.x, edgecolour);
		}

	// draw vertical borders
	for (i=0; i<borderthickness; i++)
		{
		grDrawVLine(pos.x+i, pos.y, br.y, bordercolour);
		grDrawVLine(br.x-i, pos.y, br.y, bordercolour);
		}

	// draw vertical lines
	for (xx=1; xx<num_cells.x; xx++)
		{
		x = pos.x + (xx-1)*edgethickness + xx*cell_size.x + borderthickness;

		for (i=0; i<edgethickness; i++)
			grDrawVLine(x+i, pos.y, br.y, edgecolour);
		}
	}

void		Grid::BlitGrid()
	{
	grBlitRegion(TopLeft().x,TopLeft().y, BottomRight().x,BottomRight().y);
	}

void		Grid::BlitCell(int cx,int cy)
	{
	grBlitRegion(CellTopLeft(cx,cy).x-edgethickness,CellTopLeft(cx,cy).y-edgethickness, CellBottomRight(cx,cy).x+edgethickness,CellBottomRight(cx,cy).y+edgethickness);
	}

int			Grid::Width()
	{
	return num_cells.x;
	}

int			Grid::Height()
	{
	return num_cells.y;
	}

void		Grid::SetWidth(int w)
	{
	num_cells.x = w;
	}

void		Grid::SetHeight(int h)
	{
	num_cells.y = h;
	}

void		Grid::SetCellWidth(int w)
	{
	cell_size.x = w;
	}

void		Grid::SetCellHeight(int h)
	{
	cell_size.y = h;
	}

void		Grid::FillCell(int cx,int cy, uchar colour)
	{
	if ((cx<0) || (cx>num_cells.x-1) || (cy<0) || (cy>num_cells.y-1)) return;

	grPoint	tl = CellTopLeft(cx,cy);
	grPoint	br = CellBottomRight(cx,cy);

	grDrawFilledRect(tl.x, tl.y, br.x, br.y, colour);
	}

bool		Grid::IsSelected(int px,int py)
	{
	// see if p is inside the grid
	return grPtInRect(px,py,pos,BottomRight());
	}

grCell		Grid::WhichCell(int px,int py)
	{
	// return value. Note: -1,-1 is the initial error condition
	// cell values range from 0...num_cells-1 inclusive
	grPoint	rp = {-1,-1};

	if (!IsSelected(px,py)) return rp;

	// localize coordinate to the inside of the grid at top left
	px = px - pos.x + borderthickness;
	py = py - pos.y + borderthickness;

	// divide the local coordinate by how large the cells are then add 1
	rp.x = px / (cell_size.x + edgethickness);
	rp.y = py / (cell_size.y + edgethickness);

	return rp;
	}

void		Grid::SelectCell(int cx,int cy, uchar colour)
	{
	if ((cx<0) || (cx>num_cells.x-1) || (cy<0) || (cy>num_cells.y-1)) return;

	grPoint	tl = CellTopLeft(cx,cy);
	grPoint	br = CellBottomRight(cx,cy);

	grDrawRect(tl.x-1, tl.y-1, br.x+1, br.y+1, colour);
	}

void		Grid::DeselectCell(int cx,int cy)
	{
	if ((cx<0) || (cx>num_cells.x-1) || (cy<0) || (cy>num_cells.y-1)) return;

	grPoint	tl = CellTopLeft(cx,cy);
	grPoint	br = CellBottomRight(cx,cy);

	grDrawRect(tl.x-1, tl.y-1, br.x+1, br.y+1, edgecolour);
	}

grPoint		Grid::Size()
	{
	grPoint rp;

	// calculate the size of the whole grid
	rp.x = (num_cells.x-1) * edgethickness
			+ num_cells.x * cell_size.x + 2*borderthickness;
	rp.y = (num_cells.y-1) * edgethickness
			+ num_cells.y * cell_size.y + 2*borderthickness;

	return rp;
	}

grPoint		Grid::TopLeft()
	{
	return pos;
	}

grPoint		Grid::BottomRight()
	{
	grPoint br;
	grPoint measure = Size();

	// calculate the bottom left corner of the grid
	br.x = pos.x + measure.x - 1;
	br.y = pos.y + measure.y - 1;

	return br;
	}

grPoint		Grid::CellTopLeft(int cx,int cy)
	{
	grPoint	rp;
	rp.x = pos.x + borderthickness + cx*edgethickness + cx*cell_size.x;
	rp.y = pos.y + borderthickness + cy*edgethickness + cy*cell_size.y;
	return rp;
	}

grPoint		Grid::CellBottomRight(int cx,int cy)
	{
	grPoint	rp;
	rp.x = pos.x + borderthickness + cx*edgethickness + (cx+1)*cell_size.x - 1;
	rp.y = pos.y + borderthickness + cy*edgethickness + (cy+1)*cell_size.y - 1;
	return rp;
	}

int			Grid::CellToIndex(int cx,int cy)
	{
	return (cy * num_cells.x + cx);
	}

grCell		Grid::IndexToCell(int i)
	{
	grCell rc;

	rc.x = i % num_cells.x;
	rc.y = i / num_cells.x;

	return rc;
	}
