// --------------------------------------------------------------------------
//
//	GRID FUNCTIONS
//
//	PARAMS:     p - real world coordinate used to locate a cell
//				c - a cell coordinate, between 1 and num_cells inclusive
//				pos - position where the grid is located
//				num_cells - number of cells
//				cell_size - cell size excluding edges
//				edgethickness - edge thickness
//				edgecolour - colour of the grid edges and border
//
// --------------------------------------------------------------------------

#ifndef __grid_hpp
#define __grid_hpp

#include "graphics.hpp"

#define		grCell			grPoint

#define		grid_NoOption	0
#define		grid_Overlay	1

class Grid
	{
	private:
		grPoint		pos;
		grPoint		num_cells;
		grPoint		cell_size;

		int			borderthickness;
		uchar		bordercolour;

		int			edgethickness;
		uchar		edgecolour;

		uchar		backcolour;

	public:
					// no defaults
					Grid(int posx,int posy,int numx,int numy,int sizx,int sizy,int bt,uchar bc,int et,uchar ec,uchar bkc);
					Grid(grPoint p,grPoint nc,grPoint cs,int bt,uchar bc,int et,uchar ec,uchar bkc);

					// defaults assumed
					Grid(int posx,int posy,int numx,int numy,int sizx,int sizy);
					Grid(grPoint p,grPoint nc,grPoint cs);

					~Grid() {}

					// draws the grid
		void		Draw			(int options = grid_NoOption);

					// blits the region that the grid takes up
		void		BlitGrid		();
		void		BlitCell		(int cx,int cy);
		void		BlitCell		(grCell c) { BlitCell(c.x,c.y); }

					// return the number of cells across and down
		int			Width			();
		int			Height			();

					// set the number of cells across and down
		void		SetWidth		(int w);
		void		SetHeight		(int h);

					// set the size of the cells across and down
		void		SetCellWidth	(int w);
		void		SetCellHeight	(int h);

					// fills a cell with a colour
		void 		FillCell		(int cx,int cy, uchar colour);
		void 		FillCell		(grCell c, uchar colour) { FillCell(c.x,c.y,colour); }

					// returns true if a point is inside a grid
		bool		IsSelected		(int px,int py);
		bool		IsSelected		(grPoint p) { return IsSelected(p.x,p.y); }
					// finds which cell is being pointed at
		grCell		WhichCell		(int px,int py);	// -1,-1 if no cell is selected
		grCell		WhichCell		(grPoint p) { return WhichCell(p.x,p.y); }

					// draws/undraws a box around a selected cell
		void		SelectCell		(int cx,int cy, uchar colour);
		void		SelectCell		(grCell c, uchar colour) { SelectCell(c.x,c.y,colour); }
		void		DeselectCell	(int cx,int cy);
		void		DeselectCell	(grCell c) { DeselectCell(c.x,c.y); }

					// returns the size of the grid in pixels
		grPoint		Size			();

					// returns the screen coordinate of top left or
					// bottom right corner of the grid
		grPoint		TopLeft			();
		grPoint		BottomRight		();

					// returns the screen coordinate of top left or
					// bottom right corner of a cell
		grPoint		CellTopLeft			(int cx,int cy);
		grPoint		CellTopLeft			(grCell c) { return CellTopLeft(c.x,c.y); }
		grPoint		CellBottomRight		(int cx,int cy);
		grPoint		CellBottomRight		(grCell c) { return CellBottomRight(c.x,c.y); }

					// converts between a cell and a contiguous index.
					//i.e. cell 1,1 = 0 / 2,1 = 1...
		int 		CellToIndex		(int cx,int cy);
		int 		CellToIndex		(grCell c) { return CellToIndex(c.x,c.y); }
		grCell		IndexToCell		(int i);
	};

#endif