//	-------------------------------------------------------------------------
//
//	GUI.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains pre-defined picture buttons, etc.
//					for ready use by any "Program"
//
//	-------------------------------------------------------------------------

#ifndef __gui_hpp
#define __gui_hpp

//	--------------

#include "buttons.hpp"

//	--------------

void InstallGUI(char* path,char* gui_res);
void ExitGUI();

//	--------------

extern grImage**  	GUI_BITMAPS;

//	--------------

// default PictureButton return events

#define cmdOKButton				500
#define cmdCANCELButton			501
#define cmdUpArrowButton 		502
#define cmdDnArrowButton 		503
#define cmdLtArrowButton 		504
#define cmdRtArrowButton 		505
#define cmdDblUpArrowButton 	506
#define cmdDblDnArrowButton 	507
#define cmdDblLtArrowButton 	508
#define cmdDblRtArrowButton 	509
#define cmdUpDnArrowButton		510
#define cmdLtRtArrowButton		511
#define cmdTickButton			512
#define cmdCrossButton			513
#define cmdQuestionMarkButton 	514
#define cmdSmallButton			515
#define	cmdMediumButton			516
#define cmdLargeButton			517
#define	cmdBlankButton			518

//	--------------

class	GUIButton : public PictureButton
	{
	public:
		GUIButton(int x,int y,int pic1,int pic2,int options,int lmb_event,int rmb_event)
			: PictureButton(x,y,
					GUI_BITMAPS[pic1],GUI_BITMAPS[pic2],
					options,lmb_event,rmb_event) {}
	};


//	-------------------------------------------------------------------------
//
//	PRE-DEFINED BUTTON TYPES
//
//  Notes: 	* the following buttons are available iff 'Program' constructor
//			  is given the prg_InstallGUI option
//
//	-------------------------------------------------------------------------

class OKButton : public GUIButton
	{
	public:
		OKButton(int x,int y,int options,int lmb_event,int rmb_event)
			: GUIButton( x,y, 0,1, options,lmb_event,rmb_event) {}
	};

//	--------------

class CANCELButton: public GUIButton
	{
	public:
		CANCELButton(int x,int y,int options,int lmb_event,int rmb_event)
			: GUIButton( x,y, 2,3, options,lmb_event,rmb_event) {}
	};

//	--------------

class UpArrowButton : public GUIButton
	{
	public:
		UpArrowButton(int x,int y,int options,int lmb_event,int rmb_event)
			: GUIButton( x,y, 4,5, options,lmb_event,rmb_event) {}
	};

//	--------------

class DnArrowButton : public GUIButton
	{
	public:
		DnArrowButton(int x,int y,int options,int lmb_event,int rmb_event)
			: GUIButton( x,y, 6,7, options,lmb_event,rmb_event) {}
	};

//	--------------

class LtArrowButton : public GUIButton
	{
	public:
		LtArrowButton(int x,int y,int options,int lmb_event,int rmb_event)
			: GUIButton( x,y, 8,9, options,lmb_event,rmb_event) {}
	};

//	--------------

class RtArrowButton : public GUIButton
	{
	public:
		RtArrowButton(int x,int y,int options,int lmb_event,int rmb_event)
			: GUIButton( x,y, 10,11, options,lmb_event,rmb_event) {}
	};

//	--------------

class DblUpArrowButton : public GUIButton
	{
	public:
		DblUpArrowButton(int x,int y,int options,int lmb_event,int rmb_event)
			: GUIButton( x,y, 4,5, options,lmb_event,rmb_event) {}
	};

//	--------------

class DblDnArrowButton : public GUIButton
	{
	public:
		DblDnArrowButton(int x,int y,int options,int lmb_event,int rmb_event)
			: GUIButton( x,y, 6,7, options,lmb_event,rmb_event) {}
	};

//	--------------

class DblLtArrowButton : public GUIButton
	{
	public:
		DblLtArrowButton(int x,int y,int options,int lmb_event,int rmb_event)
				: GUIButton(x,y, 8,9, options,lmb_event,rmb_event) {}
	};

//	--------------

class DblRtArrowButton : public GUIButton
	{
	public:
		DblRtArrowButton(int x,int y,int options,int lmb_event,int rmb_event)
				: GUIButton(x,y, 10,11, options,lmb_event,rmb_event) {}
	};

//	--------------

class UpDnArrowButton : public GUIButton
	{
	public:
		UpDnArrowButton(int x,int y,int options,int lmb_event,int rmb_event)
				: GUIButton(x,y, 0,0, options,lmb_event,rmb_event) {}
	};

//	--------------

class LtRtArrowButton : public GUIButton
	{
	public:
		LtRtArrowButton(int x,int y,int options,int lmb_event,int rmb_event)
				: GUIButton(x,y, 0,0, options,lmb_event,rmb_event) {}
	};

//	--------------

class TickButton : public GUIButton
	{
	public:
		TickButton(int x,int y,int options,int lmb_event,int rmb_event)
				: GUIButton(x,y, 0,0, options,lmb_event,rmb_event) {}
	};

//	--------------

class CrossButton : public GUIButton
	{
	public:
		CrossButton(int x,int y,int options,int lmb_event,int rmb_event)
				: GUIButton(x,y, 0,0, options,lmb_event,rmb_event) {}
	};

//	--------------

class QuestionMarkButton : public GUIButton
	{
	public:
		QuestionMarkButton(int x,int y,int options,int lmb_event,int rmb_event)
				: GUIButton(x,y, 0,0, options,lmb_event,rmb_event) {}
	};

//	--------------

class SmallButton : public GUIButton
	{
	public:
		SmallButton(int x,int y,int options,int lmb_event,int rmb_event)
				: GUIButton(x,y, 0,0, options,lmb_event,rmb_event) {}
	};

//	--------------

class MediumButton : public GUIButton
	{
	public:
		MediumButton(int x,int y,int options,int lmb_event,int rmb_event)
				: GUIButton(x,y, 0,0, options,lmb_event,rmb_event) {}
	};

//	--------------

class LargeButton : public GUIButton
	{
	public:
		LargeButton(int x,int y,int options,int lmb_event,int rmb_event)
				: GUIButton(x,y, 0,0, options,lmb_event,rmb_event) {}
	};

//	--------------

#define	FussyButton			PictureButton

//	--------------

class BlankButton : public Button
	{
		BlankButton(int x1,int y1,int x2,int y2,int options,int lmb_event,int rmb_event)
			:	Button(x1,y1,x2,y2,options,lmb_event,rmb_event) {}

		virtual	void	Draw() {}
		virtual	void 	Draw(grVirtualBuffer* vb) {}
	};

#endif