//	-------------------------------------------------------------------------
//
//	HANDLER.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains code for the Event Handler object
//					used by the EVENT MANAGER. Note that some routines
//					directly call event manager functions!
//
//	-------------------------------------------------------------------------

#include "handler.hpp"
#include "eventmgr.hpp"
#include "keyboard.hpp"
#include "mouse.hpp"
#include "malloc.h"
#include "dos.h"


//	-------------------------------------------------------------------------
//
//	FUNC:		CONSTRUCTOR for EventHandler
//	DESC:		The default constructor which defines the behaviour for
//				all derived EventHandler()'s
//
//	-------------------------------------------------------------------------
EventHandler::EventHandler(int* evhno,int options,int zfact)
	{
//	DBG(470,"IN EVENTHANDLER CONSTRUCTOR");
	ext_handler_number = evhno;
	handler_number = 0;
	selected_options = options;
	zfactor = zfact;

	isOnScreenHandler 	= false;
	isBackgroundTask 	= false;
	isRespondingMouse	= !(selected_options&evh_NO_KBD_RESPONSE);
	isRespondingKbd		= !(selected_options&evh_NO_MOUSE_RESPONSE);

	HEAPCHK("by EventHandler CONSTRUCTOR");
	}

void EventHandler::SetHandlerNumber(int handler_num)
	{
//	if (isDialogBox) DBG(440,"IN SETHANDLERNUMBER, HANDLER_NUM == %d",handler_num);
	handler_number = handler_num;
	*ext_handler_number = handler_num;
//	if (isDialogBox) DBG(430,"OUT SETHANDLERNUMBER, HANDLER_NUM == %d",handler_num);
//	if (isDialogBox) DBG(420,"ADDRESS OF EXT_HANDLER_NUMBER == %p",ext_handler_number);
//	if (isDialogBox) DBG(410,"ADDRESS OF HANDLER_NUMBER == %p",&handler_number);
//	if (isDialogBox) delay(200);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		HandleEvent()
//	DESC:		This is the generic HandleEvent() function which should be
//              called by any derived objects HandleEvent() function as follows
//
//				eg. MyWindow::HandleEvent(Event &e)
//						{
//
//						(interpret events here to override EventHandler)
//
//						// call parent HandleEvent() to handle accessories!!
//						EventHandler::HandleEvent(e);
//
//						(interpret events here after accessories handle event)
//
//						}
//
//				by default this HandleEvent interprets the following keys:
//					TAB 	- move to next accessory
//					SH_TAB 	- move to previous accessory
//					RET 	- select current accessory
//					SPACE   - select current accessory
//
//	PARAMS:		e - the specified event to be handled
//
//	-------------------------------------------------------------------------
void EventHandler::HandleEvent(Event &e)
	{
	// does nothing by default
	}
