#include "images.hpp"
#include "mouse.hpp"

// --------------------------------------------------------------------------
//
//	FUNC:		grPutImageVirtual()
// 	DESC:		puts an image into a virtual buffer starting at x1,y1
//
// --------------------------------------------------------------------------
void grPutImageVirtual(grVirtualBuffer* vb,int x1,int y1,int x2,int y2,grImage* image)
	{
	if (image==NULL) return;
	if (vb==NULL) return;

	int vb_width = vb->hdr.width;
	int vb_height = vb->hdr.height;
	int image_width = image->hdr.width;
//	int image_height = image->hdr.height;

	// adjust width, height for display bounds
	int display_width = x2-x1+1;
//	int display_height = y2-y1+1;

	// clip the image to the bounds of the virtual buffer

	int vb_maxx = vb_width-1;
	int vb_maxy = vb_height-1;

	int x_clip = 0, y_clip = 0;
	int dx = 0,dy = 0;

	if (x1<0) { x_clip += -x1; x1 = 0; dx = x_clip; }
	if (y1<0) { y_clip += -y1; y1 = 0; dy = y_clip; }

	if (x2>vb_maxx) { x_clip += x2 - vb_maxx; x2 = vb_maxx; }
	if (y2>vb_maxy) { y_clip += y2 - vb_maxy; y2 = vb_maxy; }

	// clip it to the display rect

//	if (image_width>display_width) { x_clip += image_width-display_width; }
//	if (image_height>display_height) { y_clip += image_height-display_height; }

	// calc base addresses of virtual buffer & image data
	long	vb_addr 	= (long) &vb->data[y1 * vb_width + x1];
	long	image_addr	= (long) &image->data[dy * image_width + dx];

	int copy_width  	= image_width - x_clip;
	if (copy_width>display_width) copy_width = display_width;

	int dwords_to_copy  = copy_width >> 2;			// dwords to copy
	int words_to_copy   = ((copy_width&2)!=0);		// if word-aligned, copy 1 word
	int bytes_to_copy 	= ((copy_width&1)!=0); 		// if byte-aligned, copy 1 byte

	int x,y;

	// blit the image

	DWORD* dest;
	DWORD* srce;
	for (y=y1;y<=y2;y++)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END
		dest = (DWORD*) (vb_addr);
		vb_addr += vb_width;

		// and our offset into the picture we're working with
		srce = (DWORD*) (image_addr);
		// adjust our index into the picture to reflect line being drawn
		image_addr += image_width;

		// splat all DWORDS across
		for (x=0;x<dwords_to_copy;x++)
			{
			*dest = *srce;
			srce++;
			dest++;
			}

		// if we need to copy a WORD
		if (words_to_copy)
			{
			WORD* wdest = (WORD*) dest;
			WORD* wsrce = (WORD*) srce;
			*wdest = *wsrce;
			wdest++;
			wsrce++;

			// we may need to copy a BYTE as well
			if (bytes_to_copy)
				{
				BYTE* bdest = (BYTE*) wdest;
				BYTE* bsrce = (BYTE*) wsrce;
				*bdest = *bsrce;
				}
			}
		else
			// we don't need to copy a word
			// but we may still need to copy a BYTE
			{
			if (bytes_to_copy)
				{
				BYTE* bdest1 = (BYTE*) dest;
				BYTE* bsrce1 = (BYTE*) srce;
				*bdest1 = *bsrce1;
				}
			}
		}
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grOverlayImageVirtual()
// 	DESC:		overlays an image into a virtual buffer starting at x1,y1
//				with transparent color tr_color
//
// --------------------------------------------------------------------------
void grOverlayImageVirtual(grVirtualBuffer* vb,int x1,int y1,int x2,int y2,int tr_color,grImage* image)
	{
	if (image==NULL) return;
	if (vb==NULL) return;

	int vb_width = vb->hdr.width;
	int vb_height = vb->hdr.height;
	int image_width = image->hdr.width;
//	int image_height = image->hdr.height;

	// adjust width, height for display bounds
	int display_width = x2-x1+1;
//	int display_height = y2-y1+1;

	// clip the image to the bounds of the virtual buffer

	int vb_maxx = vb_width-1;
	int vb_maxy = vb_height-1;


	int x_clip = 0, y_clip = 0;
	int dx = 0,dy = 0;

	if (x1<0) { x_clip += -x1; x1 = 0; dx = x_clip; }
	if (y1<0) { y_clip += -y1; y1 = 0; dy = y_clip; }

	if (x2>vb_maxx) { x_clip += x2 - vb_maxx; x2 = vb_maxx; }
	if (y2>vb_maxy) { y_clip += y2 - vb_maxy; y2 = vb_maxy; }

	// clip it to the display rect

//	if (image_width>display_width) { x_clip += image_width-display_width; }
//	if (image_height>display_height) { y_clip += image_height-display_height; }

	// calc base addresses of virtual buffer & image data
	long	vb_addr 	= (long) &vb->data[y1 * vb_width + x1];
	long	image_addr	= (long) &image->data[dy * image_width + dx];

	int copy_width  	= image_width - x_clip;
	if (copy_width>display_width) copy_width = display_width;

	int x,y;

	// blit the image

	BYTE* dest;
	BYTE* srce;
	BYTE  tmp_srce;

	for (y=y1;y<=y2;y++)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END
		dest = (BYTE*) (vb_addr);
		vb_addr += vb_width;

		// and our offset into the picture we're working with
		srce = (BYTE*) (image_addr);
		// adjust our index into the picture to reflect line being drawn
		image_addr += image_width;

		// splat all BYTES across
		for (x=0;x<copy_width;x++)
			{
			tmp_srce = *srce;
			if (tmp_srce!=tr_color) *dest = tmp_srce;
			srce++;
			dest++;
			}
		}

//	dest = (BYTE*) (vb_addr + y1 * vb_width + x1);
//	*dest = grLIGHTRED;
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grOverlay2CImageVirtual()
// 	DESC:		overlays a 2-color image into a virtual buffer starting at x1,y1
//
// --------------------------------------------------------------------------
void grOverlay2CImageVirtual(grVirtualBuffer* vb,int x1,int y1,int x2,int y2,int color,grImage* image)
	{
	if (image==NULL) return;
	if (vb==NULL) return;

	int vb_width = vb->hdr.width;
	int vb_height = vb->hdr.height;
	int image_width = image->hdr.width;
//	int image_height = image->hdr.height;

	// adjust width, height for display bounds
	int display_width = x2-x1+1;
//	int display_height = y2-y1+1;

	// clip the image to the bounds of the virtual buffer

	int vb_maxx = vb_width-1;
	int vb_maxy = vb_height-1;

	int x_clip = 0, y_clip = 0;
	int dx = 0,dy = 0;

	if (x1<0) { x_clip += -x1; x1 = 0; dx = x_clip; }
	if (y1<0) { y_clip += -y1; y1 = 0; dy = y_clip; }

	if (x2>vb_maxx) { x_clip += x2 - vb_maxx; x2 = vb_maxx; }
	if (y2>vb_maxy) { y_clip += y2 - vb_maxy; y2 = vb_maxy; }

	// clip it to the display rect

//	if (image_width>display_width) { x_clip += image_width-display_width; }
//	if (image_height>display_height) { y_clip += image_height-display_height; }

	// calc base addresses of virtual buffer & image data
	long	vb_addr 	= (long) &vb->data[y1 * vb_width + x1];
	long	image_addr	= (long) &image->data[dy * image_width + dx];

	int copy_width  	= image_width - x_clip;

	if (copy_width>display_width) copy_width = display_width;

	int x,y;

	// blit the image

	BYTE* dest;
	BYTE* srce;
	BYTE  tmp_srce;
	BYTE	bfill = (BYTE) color;

	for (y=y1;y<=y2;y++)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END
		dest = (BYTE*) (vb_addr);
		vb_addr += vb_width;

		// and our offset into the picture we're working with
		srce = (BYTE*) (image_addr);
		// adjust our index into the picture to reflect line being drawn
		image_addr += image_width;

		// splat all BYTES across
		for (x=0;x<copy_width;x++)
			{
			tmp_srce = *srce;
			if (tmp_srce!=grBLACK) *dest = bfill;
			srce++;
			dest++;
			}
		}
//	dest = (BYTE*) (vb_addr + y1 * vb_width + x1);
//	*dest = grLIGHTGREEN;
	}

// --------------------------------------------------------------------------
// put an image section into a virtual buffer starting at x1,y1
// --------------------------------------------------------------------------

void grPutImageSectionVirtual(grVirtualBuffer* vb,
				int x1,int y1,int x2,int y2,
				int vb_x1,int vb_y1,int vb_x2,int vb_y2,
				grImage* image)
	{
	if (image==NULL) return;
	if (vb==NULL) return;

	int vb_width = vb->hdr.width;
	int vb_height = vb->hdr.height;
	int image_width = image->hdr.width;
//	int image_height = image->hdr.height;

	int vb_maxx = vb_width-1;
	int vb_maxy = vb_height-1;

	// make sure the image is clipped correctly
	int x_clip = 0, y_clip = 0;
	int dx = vb_x1,dy = vb_y1;

	if (x1<0) { x_clip += -x1; x1 = 0; dx += x_clip; }
	if (y1<0) { y_clip += -y1; y1 = 0; dy += y_clip; }

	if (x2>vb_maxx) { x_clip += x2 - vb_maxx; x2 = vb_maxx; }
	if (y2>vb_maxy) { y_clip += y2 - vb_maxy; y2 = vb_maxy; }

	// calc base addresses of virtual buffer & image data
	long	vb_addr 	= (long) &vb->data[y1 * vb_width + x1];
	long	image_addr	= (long) &image->data[dy * image_width + dx];

	int 	copy_width  = (vb_x2 - vb_x1 + 1) - x_clip;

	int dwords_to_copy  = copy_width >> 2;			// dwords to copy
	int words_to_copy   = ((copy_width&2)!=0);		// if word-aligned, copy 1 word
	int bytes_to_copy 	= ((copy_width&1)!=0); 		// if byte-aligned, copy 1 byte

	int x,y;

	// blit the image

	DWORD* dest;
	DWORD* srce;
	for (y=y1;y<=y2;y++)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END
		dest = (DWORD*) (vb_addr);
		vb_addr += vb_width;

		// and our offset into the picture we're working with
		srce = (DWORD*) (image_addr);
		// adjust our index into the picture to reflect line being drawn
		image_addr += image_width;

		// splat all DWORDS across
		for (x=0;x<dwords_to_copy;x++)
			{
			*dest = *srce;
			srce++;
			dest++;
			}

		// if we need to copy a WORD
		if (words_to_copy)
			{
			WORD* wdest = (WORD*) dest;
			WORD* wsrce = (WORD*) srce;
			*wdest = *wsrce;
			wdest++;
			wsrce++;

			// we may need to copy a BYTE as well
			if (bytes_to_copy)
				{
				BYTE* bdest = (BYTE*) wdest;
				BYTE* bsrce = (BYTE*) wsrce;
				*bdest = *bsrce;
				}
			}
		else
			// we don't need to copy a word
			// but we may still need to copy a BYTE
			{
			if (bytes_to_copy)
				{
				BYTE* bdest1 = (BYTE*) dest;
				BYTE* bsrce1 = (BYTE*) srce;
				*bdest1 = *bsrce1;
				}
			}
		}
	}

// --------------------------------------------------------------------------
// overlay an image section into a virtual buffer starting at x1,y1 with transparent color tr_color
// --------------------------------------------------------------------------

void grOverlayImageSectionVirtual(grVirtualBuffer* vb,
				int x1,int y1,int x2,int y2,
				int vb_x1,int vb_y1,int vb_x2,int vb_y2,
				int tr_color,grImage* image)
	{
	if (image==NULL) return;
	if (vb==NULL) return;

	int vb_width = vb->hdr.width;
	int vb_height = vb->hdr.height;
	int image_width = image->hdr.width;
//	int image_height = image->hdr.height;

	int vb_maxx = vb_width-1;
	int vb_maxy = vb_height-1;

	// make sure the image is clipped correctly
	int x_clip = 0, y_clip = 0;
	int dx = vb_x1, dy = vb_y1;

	if (x1<0) { x_clip += -x1; x1 = 0; dx += x_clip; }
	if (y1<0) { y_clip += -y1; y1 = 0; dy += y_clip; }

	if (x2>vb_maxx) { x_clip += x2 - vb_maxx; x2 = vb_maxx; }
	if (y2>vb_maxy) { y_clip += y2 - vb_maxy; y2 = vb_maxy; }

	// calc base addresses of virtual buffer & image data
	long	vb_addr 	= (long) &vb->data[y1 * vb_width + x1];
	long	image_addr	= (long) &image->data[dy * image_width + dx];

	int copy_width  	= (vb_x2-vb_x1+1) - x_clip;

	int x,y;

	// blit the image

	BYTE* dest;
	BYTE* srce;
	BYTE  tmp_srce;

	for (y=y1;y<=y2;y++)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END
		dest = (BYTE*) (vb_addr);
		vb_addr += vb_width;

		// and our offset into the picture we're working with
		srce = (BYTE*) (image_addr);
		// adjust our index into the picture to reflect line being drawn
		image_addr += image_width;

		// splat all BYTES across
		for (x=0;x<copy_width;x++)
			{
			tmp_srce = *srce;
			if (tmp_srce!=tr_color) *dest = tmp_srce;
			srce++;
			dest++;
			}
		}
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grPutScaledImageVirtual()
// 	DESC:		puts an image (scaled according to x1,y1,x2,y2)
//				into a virtual buffer starting at x1,y1
//
// --------------------------------------------------------------------------
void grPutScaledImageVirtual(grVirtualBuffer* vb,int x1,int y1,int x2,int y2,grImage* image)
	{
	if (image==NULL) return;
	if (vb==NULL) return;

	// This function uses a modified form of Bresenham's line drawing
	// algorithm to scale a 2d bitmap to the required size.

	// make sure coords are clipped!!!
	CLIP2(x1,y1,0,0,vb->hdr.width-1,vb->hdr.height-1);
	CLIP2(x2,y2,0,0,vb->hdr.width-1,vb->hdr.height-1);

	int dest_width = x2-x1+1;
	int dest_height = y2-y1+1;

	int srce_width = image->hdr.width;
	int srce_height = image->hdr.height;

	int vb_width = vb->hdr.width;

	BYTE* 	srce;
	BYTE*	dest;
	BYTE* 	lastsrce = NULL;
	BYTE* 	lastdest = NULL;


	int srce_x,srce_y,dest_x,dest_y,numerator_x,numerator_y;


	// starting at y=0

	srce_y = 0;
	dest_y = 0;
	numerator_y = 0;

	// scan through till we finish all rows

	while (dest_y<dest_height)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END

		// --- scan through doing all columns

		dest = (BYTE*) &vb->data[(y1+dest_y) * vb_width + x1];
		srce = (BYTE*) &image->data[srce_width * srce_y];

		srce_x = 0;
		dest_x = 0;

		// if we trying to scale the last row again....
		if (srce==lastsrce)
			{
			// simply copy it from the previous row....
			while (dest_x<dest_width)
				{
				dest[dest_x] = lastdest[dest_x];
				dest_x++;
				}
			}
		else
			{
			// do the bresenham's scale for the row
			numerator_x = 0;

			while (dest_x<dest_width)
				{
				dest[dest_x++] = srce[srce_x];
				numerator_x += srce_width;

				while (numerator_x > dest_width)
					{
					numerator_x -= dest_width;
					srce_x++;
					}
				}
			}

		// save the last row we scaled
		lastsrce = srce;
		lastdest = dest;

		// increment our destination row
		dest_y++;
		// and add the height to the numerator
		numerator_y += srce_height;

		// while the numerator is bigger than our destination height
		while (numerator_y > dest_height)
			{
			// finished row
			numerator_y -= dest_height;
			srce_y++;
			}
		}
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grOverlayScaledImageVirtual()
// 	DESC:		overlays an image (scaled according to x1,y1,x2,y2)
//				into a virtual buffer starting at x1,y1
//
// --------------------------------------------------------------------------
void grOverlayScaledImageVirtual(grVirtualBuffer* vb,int x1,int y1,int x2,int y2,int tr_color,grImage* image)
	{
	if (image==NULL) return;
	if (vb==NULL) return;

	// This function uses a modified form of Bresenham's line drawing
	// algorithm to scale a 2d bitmap to the required size.

	// make sure coords are clipped!!!
	CLIP2(x1,y1,0,0,vb->hdr.width-1,vb->hdr.height-1);
	CLIP2(x2,y2,0,0,vb->hdr.width-1,vb->hdr.height-1);

	int dest_width = x2-x1+1;
	int dest_height = y2-y1+1;

	int srce_width = image->hdr.width;
	int srce_height = image->hdr.height;

	int vb_width = vb->hdr.width;

	BYTE* 	srce;
	BYTE*	dest;
	BYTE* 	lastsrce = NULL;
	BYTE* 	lastdest = NULL;


	int srce_x,srce_y,dest_x,dest_y,numerator_x,numerator_y;

	BYTE 	srce_data;
	BYTE	tr_byte = (BYTE) tr_color;


	// starting at y=0

	srce_y = 0;
	dest_y = 0;
	numerator_y = 0;

	// scan through till we finish all rows

	while (dest_y<dest_height)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END

		// --- scan through doing all columns

		dest = (BYTE*) &vb->data[(y1+dest_y) * vb_width + x1];
		srce = (BYTE*) &image->data[srce_width * srce_y];

		srce_x = 0;
		dest_x = 0;

		// if we trying to scale the last row again....
		if (srce==lastsrce)
			{
			// simply copy it from the previous row....
			while (dest_x<dest_width)
				{
				srce_data = lastdest[dest_x];
				if (srce_data!=tr_byte) dest[dest_x] = srce_data;
				dest_x++;
				}
			}
		else
			{
			// do the bresenham's scale for the row
			numerator_x = 0;

			while (dest_x<dest_width)
				{
				srce_data = srce[srce_x];
				if (srce_data!=tr_byte) dest[dest_x] = srce_data;
				dest_x++;
				numerator_x += srce_width;

				while (numerator_x > dest_width)
					{
					numerator_x -= dest_width;
					srce_x++;
					}
				}
			}

		// save the last row we scaled
		lastsrce = srce;
		lastdest = dest;

		// increment our destination row
		dest_y++;
		// and add the height to the numerator
		numerator_y += srce_height;

		// while the numerator is bigger than our destination height
		while (numerator_y > dest_height)
			{
			// finished row
			numerator_y -= dest_height;
			srce_y++;
			}
		}
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grGetImageVirtual()
// 	DESC:		gets an image from a virtual buffer starting at x1,y1
//
// --------------------------------------------------------------------------
void grGetImageVirtual(grVirtualBuffer* vb,int x1,int y1,int x2,int y2,grImage* image)
	{
	if (image==NULL) return;
	if (vb==NULL) return;

	// make sure coords are clipped!!!
	CLIP2(x1,y1,0,0,vb->hdr.width-1,vb->hdr.height-1);
	CLIP2(x2,y2,0,0,vb->hdr.width-1,vb->hdr.height-1);

	int image_width  = x2-x1+1;
	int image_height = y2-y1+1;
	int	vb_width   = vb->hdr.width;

	// calc base addresses of virtual buffer & image data
	long vb_addr = (long) &vb->data[y1 * vb_width + x1];
	long image_addr= (long) &image->data[0];

	int copy_width   = image_width;

	int dwords_to_copy  = copy_width >> 2;			// dwords to copy
	int words_to_copy   = ((copy_width&2)!=0);		// if word-aligned, copy 1 word
	int bytes_to_copy 	= ((copy_width&1)!=0); 		// if byte-aligned, copy 1 byte
	int x,y;

	// setup the image header
	image->hdr.width  = image_width;
	image->hdr.height = image_height;

	// blit the image

	DWORD* dest;
	DWORD* srce;
	for (y=y1;y<=y2;y++)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END
		srce = (DWORD*) (vb_addr);
		vb_addr += vb_width;

		// and our offset into the picture we're working with
		dest = (DWORD*) (image_addr);
		// adjust our index into the picture to reflect line being drawn
		image_addr += image_width;

		// splat all DWORDS across
		for (x=0;x<dwords_to_copy;x++)
			{
			*dest = *srce;
			srce++;
			dest++;
			}

		// if we need to copy a WORD
		if (words_to_copy)
			{
			WORD* wdest = (WORD*) dest;
			WORD* wsrce = (WORD*) srce;
			*wdest = *wsrce;
			wdest++;
			wsrce++;

			// we may need to copy a BYTE as well
			if (bytes_to_copy)
				{
				BYTE* bdest = (BYTE*) wdest;
				BYTE* bsrce = (BYTE*) wsrce;
				*bdest = *bsrce;
				}
			}
		else
			// we don't need to copy a word
			// but we may still need to copy a BYTE
			{
			if (bytes_to_copy)
				{
				BYTE* bdest1 = (BYTE*) dest;
				BYTE* bsrce1 = (BYTE*) srce;
				*bdest1 = *bsrce1;
				}
			}
		}
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grCopyImage()
//	DESC:		copy from srce to dest with memory checking
//
// --------------------------------------------------------------------------
void grCopyImage(grImage* dest, grImage* srce)
	{
	if (dest==NULL) return;
	if (srce==NULL) return;

	// find out the sizes of the existing images
	int srcesize = grImageSize(0,0,srce->hdr.width-1,srce->hdr.height-1);
	int destsize = grImageSize(0,0,dest->hdr.width-1,dest->hdr.height-1);

	// work out which is larger and use the smaller for copying with
	int size = (destsize>srcesize) ? srcesize : destsize;

	// if the dest is larger then set it blank to fill out the extra space
	if (destsize>srcesize)
		SYS_Memset(dest,0,destsize);

	// copy the data from srce to dest
	SYS_Memcpy(dest,srce,size);
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grCopyImageData()
//	DESC:		just copies the image data row by row,
//				good for resizing images
//
// --------------------------------------------------------------------------
void grCopyImageData(grImage* dest, grImage* srce)
	{
	if (dest==NULL) return;
	if (srce==NULL) return;

	// set up sizes
	int destwidth = dest->hdr.width;
	int destheight = dest->hdr.height;
	int srcewidth = srce->hdr.width;
	int srceheight = srce->hdr.height;

	int destindex;
	int srceindex;

	int srcey = 0;
	// count through all the destination rows
	for (int desty=0; desty<destheight; desty++)
		{
		// set up indexes within the images
		destindex = desty*destwidth;
		srceindex = srcey*srcewidth;

		// then set the destination row to blank
		SYS_Memset(&dest->data[destindex],0,destwidth);

		// if the source image is still within bounds to copy from
		if (srcey < srceheight)
			{
			// then copy from source, depending on which is larger
			if (destwidth > srcewidth)
				// use either the srcewidth
				SYS_Memcpy(&dest->data[destindex],&srce->data[srceindex],srcewidth);
			else
				// or the destwidth
				SYS_Memcpy(&dest->data[destindex],&srce->data[srceindex],destwidth);
			}
		srcey++;
		}
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grFlipImageHorizontal()
//	DESC:		will flip an image horizontally
//
// --------------------------------------------------------------------------
void grFlipImageHorizontal(grImage* img)
	{
	if (img==NULL) return;

	int xx,yy,swap;
	int		width = img->hdr.width;
	int		height = img->hdr.height;

	for (xx=0; xx<width/2; xx++)
		{
//        mouse_UpdateCursor();
		for (yy=0; yy<height; yy++)
			{
			swap = img->data[Index(img,xx,yy)];
			img->data[Index(img,xx,yy)] = img->data[Index(img,width-xx-1,yy)];
			img->data[Index(img,width-xx-1,yy)] = swap;
			}
		}
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grFlipImageVertical()
//	DESC:		will flip an image vertically
//
// --------------------------------------------------------------------------
void grFlipImageVertical(grImage* img)
	{
	if (img==NULL) return;

	int xx,yy,swap;
	int		width = img->hdr.width;
	int		height = img->hdr.height;

	for (yy=0; yy<height/2; yy++)
		{
//		mouse_UpdateCursor();
		for (xx=0; xx<width; xx++)
			{
			swap = img->data[Index(img,xx,yy)];
			img->data[Index(img,xx,yy)] = img->data[Index(img,xx,height-yy-1)];
			img->data[Index(img,xx,height-yy-1)] = swap;
			}
		}
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grRotateImage()
//	DESC:		will rotate an image clockwise by 90,180 or 270 degrees,
//				if not square it will change the header values to
//				reflect orientation
//
// --------------------------------------------------------------------------
void grRotateImage(grImage* img,int degrees)
	{
	if (img==NULL) return;

	int		xx,yy;
	int		width = img->hdr.width;
	int		height = img->hdr.height;
	grImage*swap;

	if (degrees!=ONEEIGHTY)
		swap = grCreateImage(0,0,height-1,width-1);
	else
		swap = grCreateImage(0,0,width-1,height-1);

	for (yy=0; yy<height; yy++)
		{
//		mouse_UpdateCursor();
		for (xx=0; xx<width; xx++)
			switch (degrees)
				{
				case NINETY:
					swap->data[Index(swap,height-yy-1,xx)] = img->data[Index(img,xx,yy)];
					break;
				case ONEEIGHTY:
					swap->data[Index(swap,width-xx-1,height-yy-1)] = img->data[Index(img,xx,yy)];
					break;
				case TWOSEVENTY:
					swap->data[Index(swap,yy,width-xx-1)] = img->data[Index(img,xx,yy)];
					break;
				}
		}
	grCopyImage(img,swap);
	grDestroyImage(swap);
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grTranslateImageHorizontal()
//	DESC:		will "slide" an image to the right by dx with scrolling
//
// --------------------------------------------------------------------------
void grScrollImageHorizontal(grImage* img,int dx)
	{
	if (img==NULL) return;

	int 	xx,yy,srcexx,destxx;
	int		width = img->hdr.width;
	int		height = img->hdr.height;

	BYTE* swap = (BYTE*) SYS_Malloc (width);

	for (yy=0; yy<height; yy++)
		{
//		mouse_UpdateCursor();
		// save all the data for a row
		for (xx=0; xx<width; xx++)
			{
			swap[xx] = img->data[Index(img,xx,yy)];
			}

		for (xx=0; xx<width; xx++)
			{
			srcexx = xx - dx;
			while (srcexx>=width) srcexx -= width;
			while (srcexx<0) srcexx += width;

			destxx = xx + dx;
			while (destxx>=width) destxx -= width;
			while (destxx<0) destxx += width;

			img->data[Index(img,xx,yy)] = swap[srcexx];
			img->data[Index(img,destxx,yy)] = swap[xx];
			}
		}
	SYS_Free(swap);
	}

// --------------------------------------------------------------------------
//
//	FUNC:		grTranslateImageVertical()
//	DESC:		will "slide" an image to the bottom by dy with scrolling
//
// --------------------------------------------------------------------------
void grScrollImageVertical(grImage* img,int dy)
	{
	if (img==NULL) return;

	int xx,yy,srceyy,destyy;
	int		width = img->hdr.width;
	int		height = img->hdr.height;

	BYTE* swap = (BYTE*) SYS_Malloc(height);

	for (xx=0; xx<width; xx++)
		{
//		mouse_UpdateCursor();
		// save all the data for a column
		for (yy=0; yy<height; yy++)
			{
			swap[yy] = img->data[Index(img,xx,yy)];
			}

		for (yy=0; yy<height; yy++)
			{
			srceyy = yy - dy;
			while (srceyy>=height) srceyy -= height;
			while (srceyy<0) srceyy += height;

			destyy = yy + dy;
			while (destyy>=height) destyy -= height;
			while (destyy<0) destyy += height;

			img->data[Index(img,xx,yy)] = swap[srceyy];
			img->data[Index(img,xx,destyy)] = swap[yy];
			}
		}
	SYS_Free(swap);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		grMergeImage
//	DESC:		will merge 2 images overlaying over grImage on under grImage
//
//	PARAMS:		grImage* over, under - 2 images to be merged
//	RETURNS:	grImage* dest - pointer to some malloced memory
//					NULL = error
//
//	-------------------------------------------------------------------------
grImage*	grMergeImage(grImage* over, grImage* under,int tr_color)
	{
	// make sure the images are the same size
	if (over->hdr.width != under->hdr.width)
		return NULL;
	if (over->hdr.height!= under->hdr.height)
		return NULL;

	// malloc the dest grImage to be returned; return NULL on fail
	grImage* dest = (grImage*)SYS_Malloc(grImageSize(0,0,over->hdr.width,over->hdr.height));
	if (!dest) return dest;

	// initialise the destination bitmap grImage header
	dest->hdr.height = over->hdr.height;
	dest->hdr.width= over->hdr.width;

	// set up pointers to step through the data after the header
	char* d = (char*)&dest->data[0];
	char* o = (char*)&over->data[0];
	char* u = (char*)&under->data[0];

	BYTE  	tr_byte = (BYTE) tr_color;

	long	pixelcount = 0;
	long	maxpixelcount = (over->hdr.width) * (over->hdr.height);

	int	dest_width = dest->hdr.width;

	while (pixelcount < maxpixelcount)
		{
//		if ((pixelcount % dest_width)==0) mouse_UpdateCursor();
		// set background data
		*d = *u;
		// if overlay data is not mask 0 then overide data
		if (*o!=tr_byte)
			*d = *o;
		// increase all grImage pointers to next pixel
		*d++;
		*o++;
		*u++;

		pixelcount++;
		}
	return dest;
	}

