#ifndef __images_hpp
#define __images_hpp

#include "vbuffer.hpp"

// --------------------------------------------------------------------------
//
//	IMAGE MACROS (BACKWARDS COMPATIBILITY & EASE OF READING)
//
// --------------------------------------------------------------------------

#define	grImageHeader			grVBHeader
#define	grImage					grVirtualBuffer

#define	grCreateImage			grCreateVirtual
#define	grDestroyImage			grDestroyVirtual
#define	grImageSize				grVirtualSize

#define	grCreateCachedImage		grInitCachedImage
#define	grDestroyCachedImage	grExitCachedImage

#define	grClearImage(IMG,COLOR)			\
			grClearVirtual((IMG),(COLOR))

#define	grPutImage(X1,Y1,X2,Y2,IMG)		\
			grPutImageVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),(IMG))

#define	grOverlayImage(X1,Y1,X2,Y2,TR_COLOR,IMG)		\
			grOverlayImageVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),(TR_COLOR),(IMG))

#define	grOverlay2CImage(X1,Y1,X2,Y2,COLOR,IMG)		\
			grOverlay2CImageVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),(COLOR),(IMG))

#define	grPutImageSection(X1,Y1,X2,Y2,VB_X1,VB_Y1,VB_X2,VB_Y2,IMG)		\
			grPutImageSectionVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),\
					(VB_X1),(VB_Y1),(VB_X2),(VB_Y2),(IMG))

#define	grOverlayImageSection(X1,Y1,X2,Y2,VB_X1,VB_Y1,VB_X2,VB_Y2,TR_COLOR,IMG)		\
			grOverlayImageSectionVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),\
					(VB_X1),(VB_Y1),(VB_X2),(VB_Y2),(TR_COLOR),(IMG))

#define	grPutScaledImage(X1,Y1,X2,Y2,IMG)		\
			grPutScaledImageVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),(IMG))

#define	grOverlayScaledImage(X1,Y1,X2,Y2,TR_COLOR,IMG)		\
			grOverlayScaledImageVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),(TR_COLOR),(IMG))

#define	grGetImage(X1,Y1,X2,Y2,IMG)		\
			grGetImageVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),(IMG))

#define	grCachedToStaticImage(STATIC,CACHED)	\
			grCopyImage((STATIC),grImagePtr((CACHED)))

// ---	this definition produces the linear data position
//		that x,y represents within an image...

#define		Index(IMG,X,Y) ((IMG)->hdr.width * (Y) + (X))
#define		grIndex(IMG,X,Y) ((IMG)->hdr.width * (Y) + (X))

// --------------------------------------------------------------------------
//
//	IMAGE IMPLEMENTATION FUNCTIONS
//
// --------------------------------------------------------------------------

// ---	put an image into a virtual buffer clipped to x1,y1,x2,y2

void 	grPutImageVirtual(grVB* vb,
						int x1,int y1,int x2,int y2,
						grImage* image );

void	grPutImageVirtual(grVB* vb,
						int x1,int y1,int x2,int y2,
						int image );


// ---	overlay an image into a virtual buffer clipped to x1,y1,x2,y2
//		ignoring all pixels of color 'tr_color'

void 	grOverlayImageVirtual(grVB* vb,
						int x1,int y1,int x2,int y2,
						int tr_color,
						grImage* image);


// ---	overlay an image into a virtual buffer clipped to x1,y1,x2,y2
//		replacing all non-zero pixels with 'color'

void 	grOverlay2CImageVirtual(grVB* vb,
						int x1,int y1,int x2,int y2,
						int color,
						grImage* image );


// ---	put a section of an image into a virtual buffer
//		clipped to x1,y1,x2,y2.
//
// 		note: assumes coords have been clipped appropriately

void 	grPutImageSectionVirtual(grVB* vb,
						int x1,int y1,int x2,int y2,
						int vb_x1,int vb_y1,int vb_x2,int vb_y2,
						grImage* image );


// ---	overlay an section of an image into a virtual buffer
//		clipped to x1,y1,x2,y2 and ignoring all pixels of color 'tr_color'

void 	grOverlayImageSectionVirtual(grVB* vb,
						int x1,int y1,int x2,int y2,
						int vb_x1,int vb_y1,int vb_x2,int vb_y2,
						int tr_color,
						grImage* image );


// ---	put an image into the virtual buffer scaled to x1,y1,x2,y2

void 	grPutScaledImageVirtual(grVB* vb,
						int x1,int y1,int x2,int y2,
						grImage* image );


// ---	overlay an image into the virtual buffer scaled to x1,y1,x2,y2

void 	grOverlayScaledImageVirtual(grVB* vb,
						int x1,int y1,int x2,int y2,
						int tr_color,
						grImage* image );


// ---	gets an image from a virtual buffer, assuming that
//		the image has already been created before calling

void 	grGetImageVirtual(grVB* vb,
						int x1,int y1,int x2,int y2,
						grImage* image );



// ---	copies a given image to another image with memory checking

void 	grCopyImage(grImage* dest, grImage* srce);


// ---	just copies an images data, not the header.
//		Note: dest must be same size or larger

void 	grCopyImageData(grImage* dest, grImage* srce);


// ---	flips an image horizontally, modifying the image data

void 	grFlipImageHorizontal(grImage* img);


// ---	flips an image vertically, modifying the image data

void 	grFlipImageVertical(grImage* img);


// ---	rotate an image clockwise by 90,180 or 270 degrees if it is a
//		square image, modifying the image data
//
//		params:		degrees is one of the following:

#define		NINETY		90
#define		ONEEIGHTY	180
#define		TWOSEVENTY	270

void 		grRotateImage(grImage* img,int degrees);


// ---	will "slide" an image right by +ve dx wrapping pixels that
//		scroll off the edge of the image, modifying the image data

void 	grScrollImageHorizontal(grImage* img,int dx);


// ---	will "slide" an image down by +ve dy wrapping pixels that
//		scroll off the edge of the image, modifying the image data

void 	grScrollImageVertical(grImage* img,int dy);

// ---	merges two images by putting 'over' on top of 'under'
//		returning a newly created image

grImage* grMergeImage(grImage* over, grImage* under, int tr_color = 0);


#endif