#include "lines.hpp"
#include "points.hpp"
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
#include "mouse.hpp"
//DJD --- END

// -------------

void grDrawLineVirtual(grVB* vb,int x1, int y1, int x2, int y2, int color)
	{
	if (!vb) return;

	// check for horizontal or vertical lines
	if (y1==y2) { grDrawHLineVirtual(vb,y1,x1,x2,color); return; }
	if (x1==x2) { grDrawVLineVirtual(vb,x1,y1,y2,color); return; }

	// make sure coords are clipped!!!
	CLIP2(x1,y1,0,0,vb->hdr.width-1,vb->hdr.height-1);
	CLIP2(x2,y2,0,0,vb->hdr.width-1,vb->hdr.height-1);

	int dx,dy,			// x & y differences
		x_inc,y_inc,	// pixels moved during draw
		error=0,        // the discriminant i.e. error i.e. decision variable
		x,y;          	// used for looping


	int 	vb_width 	= vb->hdr.width;
	long 	dest_addr 	= (long) &vb->data[y1 * vb_width + x1];
	BYTE* 	dest;

	// compute deltas
	dx = x2-x1;
	dy = y2-y1;

	// test which direction the line is going in i.e. slope angle
	if (dx>=0) x_inc = 1;
	else { x_inc = -1; dx = -dx; } // need absolute value for dx

	// test y component of slope
	if (dy>=0) y_inc = vb_width;
	else { y_inc = -vb_width; dy = -dy; } // need absolute value for dy

	// if |slope| <= 1 we draw in the x-direction
	if (dx>dy)
	   {
	   // draw the line
	   for (x=0;x<=dx;x++)
		   {
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END
		   dest = (BYTE*) dest_addr;

		   // set the pixel
		   *dest = color;

		   // adjust the discriminate
		   error+=dy;

		   // test if error overflowed
		   if (error>dx)
			  {
			  error-=dx;
			  // move to next line
			  dest_addr += y_inc;
			  }

		   // move to the next pixel
		   dest_addr += x_inc;
		   }
	   }
	else
	   {
	   // if |slope| > 1 we draw in the y-direction
	   for (y=0;y<=dy;y++)
		   {
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END
		   dest = (BYTE*) dest_addr;

		   // set the pixel
		   *dest = color;

		   // adjust the discriminate
		   error+=dx;

		   // test if error overflowed
		   if (error>0)
			  {
			  error-=dy;

			  // move to next line
			  dest_addr += x_inc;
			  }

		   // move to the next pixel
		   dest_addr += y_inc;
		   }
	   }
	}

// -------------

void grDrawVLineVirtual(grVB* vb,int x,int y1,int y2,int color)
	{
	if (!vb) return;

	// clip coords
	int vb_width = vb->hdr.width;

	CLIP2(x,y1,0,0,vb->hdr.width-1,vb->hdr.height-1);
	CLIP2(x,y2,0,0,vb->hdr.width-1,vb->hdr.height-1);

	int yy1 = y1;
	int yy2 = y2;

	if (yy1==yy2) { grPutPixelVirtual(vb,x,yy1,color); return; }
	if (yy1>yy2)  { yy1 = y2; yy2 = y1; }

	long vb_addr  = (long) &vb->data[yy1 * vb_width + x];

	for (int y=yy1;y<=yy2;y++)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END
		BYTE* dest = (BYTE*) (vb_addr);
		vb_addr += vb_width;
		*dest = color;
		}

	}

// -------------

void grDrawHLineVirtual(grVB* vb,int y,int x1,int x2,int color)
	{
	if (vb==NULL) return;

	CLIP2(x1,y,0,0,vb->hdr.width-1,vb->hdr.height-1);
	CLIP2(x2,y,0,0,vb->hdr.width-1,vb->hdr.height-1);

	if (x1==x2) { grPutPixelVirtual(vb,x1,y,color); return; }

	WORD  wcolor = color + (color<<8);
	DWORD dcolor = wcolor + (wcolor<<16);

	long vb_addr  = (long) vb + sizeof(grVBHeader);

	int vb_width  = vb->hdr.width;
	int copy_width  = x2 - x1 + 1;

	int dwords_to_copy = copy_width >> 2;			// dwords to copy
	int words_to_copy  = ((copy_width&2)!=0);		// if word-aligned, copy 1 word
	int bytes_to_copy  = ((copy_width&1)!=0); 		// if byte-aligned, copy 1 byte

	int x;

	DWORD* dest = (DWORD*) (vb_addr + (y * vb_width) + x1);

	// splat all DWORDS across
	for (x=0;x<dwords_to_copy;x++)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
//mouse_UpdateCursor();
//DJD --- END
		*dest = dcolor;
		dest++;
		}

	// if we need to copy a WORD
	if (words_to_copy)
		{
		WORD* wdest = (WORD*) dest;
		*wdest = wcolor;
		// we may need to copy a BYTE as well
		if (bytes_to_copy)
			{
			BYTE* bdest = (BYTE*) wdest;
			*bdest = color;
			}
		}
	else
		// we don't need to copy a word
		// but we may still need to copy a BYTE
		{
		if (bytes_to_copy)
			{
			BYTE* bdest1 = (BYTE*) dest;
			*bdest1 = color;
			}
		}
	}

