//	-------------------------------------------------------------------------
//
//	MOUSE.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains code for the mouse ISR which is
//					capable of auto-detection & direct hardware response.
//
//	-------------------------------------------------------------------------

#include "stddefns.hpp"
#include "mouse.hpp"
#include "keyboard.hpp"
#include "event.hpp"
#include "eventmgr.hpp"
#include "graphics.hpp"
#include "sys.hpp"

#include "dos.h"
#include "mem.h"

#ifdef __BORLANDC__
#define		regs_w 		regs.x
#define		int386		int86
#else
#include "i86.h"
#define		regs_w		regs.w
#endif

#include "conio.h"
#include "stdio.h"
#include "stdlib.h"

bool MOUSE_FIRST_TIME_DRAW = true;

// the mouse interrupt number

#define MOUSE_INT       0x33

// a few sample cursors

MouseCursor		ARROW_CURSOR =
				{ 24,24,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 15,15,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,

				0,0,8,7, 7,7,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 15,15,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 7,7,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 7,7,7,7, 15,15,0,0, 0,0,0,0, 0,0,0,0,

				0,0,8,7, 7,7,7,15, 15,15,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,15, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 15,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 7,15,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,

				0,0,8,7, 7,7,7,7, 7,7,15,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 8,7,7,7, 7,7,7,15, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 8,8,7,7, 7,7,7,7, 15,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,8, 0,0,8,7, 7,7,7,7, 7,15,0,0, 0,0,0,0, 0,0,0,0,

				0,0,8,0, 0,0,0,8, 7,7,7,7, 7,7,15,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 8,7,7,7, 7,7,8,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,8,7,7, 7,8,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,8,7, 8,0,0,0, 0,0,0,0, 0,0,0,0,

				0,0,0,0, 0,0,0,0, 0,0,0,8, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0   };

MouseCursor		DummyCursor[] =
				{ 16, 16,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,9, 0,9,0,9, 0,9,0,9, 0,9,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,0, 0,0,0,0, 0,0,0,0, 0,9,0,0,

				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,0, 0,0,0,0, 0,0,0,0, 0,9,0,0,
				0,0,0,0, 0,0,4,4, 4,4,0,0, 0,0,0,0,
				0,9,0,0, 0,0,4,4, 4,4,0,0, 0,9,0,0,

				0,0,0,0, 0,0,4,4, 4,4,0,0, 0,0,0,0,
				0,9,0,0, 0,0,4,4, 4,4,0,0, 0,9,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,0, 0,0,0,0, 0,0,0,0, 0,9,0,0,

				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,9, 0,9,0,9, 0,9,0,9, 0,9,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0 };

// global variables used by the mouse routines

MouseCursor*	DEFAULT_CURSOR = &ARROW_CURSOR;

// the delay for the auto repeat

#define mouse_AUTO_DELAY			3

// mouse library internal variables

static 	int		mouseX;				// current mouse coords
static 	int		mouseY;
static 	int		lastmouseX;			// last drawn mouse coords
static 	int		lastmouseY;

static 	int		leftBut;			// current button status, mouse_PRESSED or
static 	int		rightBut;			// 						  mouse_RELEASED
static 	int		middleBut;

static 	long	LMBTicks  = 0;		// count of ticks between button presses
static 	long	RMBTicks  = 0;
static 	long	MMBTicks  = 0;

static 	bool   	LMBAuto = false;	// if true, mouse is in 'auto generate' mode
static 	bool   	RMBAuto	= false;
static 	bool   	MMBAuto	= false;

static 	int		hotSpotX = 0;		// hotspot in mouse cursor
static 	int		hotSpotY = 0;		// hotspot in mouse cursor

static 	bool 	MoveEventsOn = true;// if true, mouse movement events are generated
static 	bool 	HasMoved = false;	// if true, mouse has moved since last cursor update

static	bool	MOUSE_IS_HIDDEN = true;// if true, mouse is not displayed on the screen

static 	MouseEvent	EventInfo;		// mouse event information, once

MouseCursor*	mouse_Cursor = &ARROW_CURSOR;	// the current mouse cursor pointer

static 	grImage*		mouse_UnderCursor = NULL;// image under current mouse cursor

static	bool	MOUSE_INSTALLED = false;		// if true, mouse library is installed
bool	MOUSE_EXTERNAL_UPDATE = false;	// if true, don't update the cursor automatically
bool	MOUSE_ISR_DISABLED = false;


//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_IsInstalled()
//	DESC:		returns true if mouse driver is currently installed
//
//	-------------------------------------------------------------------------
bool mouse_IsInstalled()
	{
	return (MOUSE_INSTALLED);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_Reset()
//	DESC:		resets the mouse to its original state
//
//	-------------------------------------------------------------------------
void mouse_Reset()
	{
	if (!MOUSE_INSTALLED) return;

	mouseX = grMaxx / 2;
	mouseY = grMaxy / 2;
	leftBut = mouse_RELEASED;
	rightBut = mouse_RELEASED;
	middleBut = mouse_RELEASED;
	lastmouseX = mouseX;
	lastmouseY = mouseY;

//	mouse_SetCursor(0,0,DEFAULT_CURSOR);
	mouse_SetRange(0,0,grMaxx,grMaxy);
	mouse_SetPos(mouseX,mouseY);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_EvableMoveEvents()
//	DESC:		upon calling this function, mouse_GetEvent() will generate
//				mb_move events. (Note: this may cause a large number of
//				events to be placed in the event queue!!!)
//
//	-------------------------------------------------------------------------
void mouse_EnableMoveEvents()
	{
	MoveEventsOn = true;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DisableMoveEvents()
//	DESC:		stops mouse_GetEvent from generating mouse move events into
//				the EVENT QUEUE.
//
//	-------------------------------------------------------------------------
void mouse_DisableMoveEvents()
	{
	MoveEventsOn = false;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetPos()
//	DESC:		returns the current position of the mouse as a point
//
//	-------------------------------------------------------------------------
grPoint mouse_GetPos()
	{
	grPoint p;
	p.x = mouseX;
	p.y = mouseY;
	return p;
/*
	union REGPACK       regs;

	memset( &regs, 0, sizeof( union REGPACK ) );
	regs.w.ax = 3;
	intr( MOUSE_INT, &regs );
	pos->xcoord = regs.w.cx;
	pos->ycoord = regs.w.dx;
	*left = regs.w.bx & 1;
*/
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_SetPos()
//	DESC:		sets the current position of the mouse given a point
//
//	-------------------------------------------------------------------------
void mouse_SetPos(grPoint& pt)
	{
	if (!MOUSE_INSTALLED) return;

	union REGPACK       regs;

	SYS_Memset( &regs, 0, sizeof( union REGPACK ) );
	regs_w.ax = 4;
	regs_w.cx = pt.x;
	regs_w.dx = pt.y;
	intr( MOUSE_INT, &regs );

	mouseX = pt.x;
	mouseY = pt.y;
	}

void    mouse_SetPos(int x1,int y1)
	{
	if (!MOUSE_INSTALLED) return;

	union REGPACK       regs;

	SYS_Memset( &regs, 0, sizeof( union REGPACK ) );
	regs_w.ax = 4;
	regs_w.cx = x1;
	regs_w.dx = y1;
	intr( MOUSE_INT, &regs );

	mouseX = x1;
	mouseY = y1;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetStatus()
//	DESC:		gets the position and the button's status
//
//	-------------------------------------------------------------------------
void	mouse_GetStatus(mouse_Status* ms_status)
	{
	ms_status->x = mouseX;
	ms_status->y = mouseY;
	ms_status->leftBut = leftBut;
	ms_status->rightBut = rightBut;
	ms_status->middleBut = middleBut;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_SetRange()
//	DESC:		sets the range of the mouse cursor
//
//	-------------------------------------------------------------------------
void mouse_SetRange(int x1,int y1,int x2,int y2)
	{
	if (!MOUSE_INSTALLED) return;

	union REGPACK       regs;

	memset( &regs, 0, sizeof( union REGPACK ) );
	regs.w.ax = 7;
	regs.w.cx = x1;
	regs.w.dx = x2;
	intr( MOUSE_INT, &regs );
	regs.w.ax = 8;
	regs.w.cx = y1;
	regs.w.dx = y2;
	intr( MOUSE_INT, &regs );
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_IsHit()
//	DESC:		returns true if mouse is waiting to be processed
//
//	-------------------------------------------------------------------------
bool mouse_IsHit()
	{
	if (!MOUSE_INSTALLED) return false;

	bool ishit = ((leftBut!=mouse_RELEASED)||
				  (rightBut!=mouse_RELEASED)||
				  (middleBut!=mouse_RELEASED));
	return ishit;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_Flush()
//	DESC:		flushes the mouse input buffer
//
//	-------------------------------------------------------------------------
void mouse_Flush()
	{
	if (!MOUSE_INSTALLED) return;

	leftBut = mouse_RELEASED;
	rightBut = mouse_RELEASED;
	middleBut = mouse_RELEASED;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_InRect()
//	DESC:		Checks to see whether the mouse cursor is in the specified
//				coordinates. Returns true if it is.
//
//	-------------------------------------------------------------------------
bool mouse_InRect(int x1,int y1,int x2,int y2)
	{
	if (!MOUSE_INSTALLED) return false;
	if (mouse_IsHidden()) return false;

	int xx 			= mouseX;
	int yy 			= mouseY;
	int xx1 		= xx + mouse_Cursor->hdr.width-1;
	int yy1 		= yy + mouse_Cursor->hdr.height-1;

	// check coords
	bool result 	= ((xx>=x1)&&(xx<=x2)&&(yy>=y1)&&(yy<=y2)||
					   (xx1>=x1)&&(xx1<=x2)&&(yy1>=y1)&&(yy1<=y2));

	return result;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DrawCursor()
//	DESC:		Draws the mouse on currently active page
//
//	-------------------------------------------------------------------------
void mouse_DrawCursor( int options )
	{
	if (!MOUSE_INSTALLED) return;
	if (mouse_Cursor==NULL) return;
	if (mouse_IsHidden()) return;

//	bool save_disabled = MOUSE_ISR_DISABLED;
//	MOUSE_ISR_DISABLED = true;

	int x,y,tmp;

	int cursor_width = mouse_Cursor->hdr.width;
	int x1 = mouseX;
	int y1 = mouseY;
	int x2 = mouseX+cursor_width-1;
	int y2 = mouseY+mouse_Cursor->hdr.height-1;

	CLIP2(x1,y1,0,0,grMaxx,grMaxy);
	CLIP2(x2,y2,0,0,grMaxx,grMaxy);

	BYTE*	screen_pos;
	BYTE* 	cursor_pos;
	BYTE*	underneath_pos;

	// figure out the screen address of the top-left corner of the cursor
	long*	row_addr 		= &VESA_RowPtr[y1];
	long	cursor_addr		= (long) &mouse_Cursor->data[0];
	long	underneath_addr	= (long) &mouse_UnderCursor->data[0];

//	int*  	bank_ofs		= &VESA_RowBank[y1];
	int		bank_num;


	// scan through the cursor vertically
	for (y=y1;y<=y2;y++)
		{
		// figure out the required bank & change iff required
//		bank_num = *bank_ofs++ + VESA_Write_Bank_Offset;
		bank_num = VESA_RowBank[y] + VESA_Write_Bank_Offset;
		if (bank_num!=VESA_Current_Write_Bank) VESA_SwitchWriteBank( bank_num );
		bank_num = VESA_RowBank[y] + VESA_Read_Bank_Offset;
		if (bank_num!=VESA_Current_Read_Bank) VESA_SwitchReadBank( bank_num );

		// create a pointer to the screen at the position to start blitting
		screen_pos = (BYTE*) (*row_addr + x1);
		cursor_pos = (BYTE*) cursor_addr;
		underneath_pos = (BYTE*) underneath_addr;

		// blit the current row of the cursor
		for (x=x1;x<=x2;x++)
			{
			tmp = *cursor_pos;		// get the next pixel in the row


			// if the appropriate pixel in the mouse cursor needs to be drawn
			if (tmp)
				{
				// save whats underneath the cursor iff required
				if ((options&SAVE_UNDERNEATH_CURSOR)!=0) *underneath_pos = *screen_pos;

				// and draw the pixel
				*screen_pos = tmp;
				}

			cursor_pos++;			// skip to the next cursor pixel
			screen_pos++;
			underneath_pos++;

//if (MOUSE_FIRST_TIME_DRAW) delay(50);

			}

		// for the next row, simply add the logical width to the address
		row_addr++;
		cursor_addr += cursor_width;
		underneath_addr += cursor_width;
		}

	// save the coords at which we last drew the mouse
	lastmouseX = mouseX;
	lastmouseY = mouseY;

//	MOUSE_ISR_DISABLED = save_disabled;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_UnDrawCursor()
//	DESC:		Erases the mouse on the currently active page
//
//	-------------------------------------------------------------------------
void mouse_UndrawCursor()
	{
	if (!MOUSE_INSTALLED) return;
	if (mouse_Cursor==NULL) return;
	if (mouse_IsHidden()) return;

//	bool save_disabled = MOUSE_ISR_DISABLED;
//	MOUSE_ISR_DISABLED = true;

	int x,y,tmp;

	int cursor_width = mouse_Cursor->hdr.width;
	int x1 = lastmouseX;
	int y1 = lastmouseY;
	int x2 = lastmouseX+cursor_width-1;
	int y2 = lastmouseY+mouse_Cursor->hdr.height-1;

	CLIP2(x1,y1,0,0,grMaxx,grMaxy);
	CLIP2(x2,y2,0,0,grMaxx,grMaxy);

	// figure out the screen address of the top-left corner of the cursor
	BYTE*	screen_pos;
	BYTE* 	cursor_pos;
	BYTE*	underneath_pos;

	long*	row_addr		= &VESA_RowPtr[y1];
	long	cursor_addr		= (long) &mouse_Cursor->data[0];
	long	underneath_addr	= (long) &mouse_UnderCursor->data[0];

//	int*  	bank_ofs		= &VESA_RowBank[y1];
	int		bank_num;

	// scan through the cursor vertically
	for (y=y1;y<=y2;y++)
		{
		// figure out the required bank & change iff required
//		bank_num = *bank_ofs++ + VESA_Write_Bank_Offset;
		bank_num = VESA_RowBank[y] + VESA_Write_Bank_Offset;
		if (bank_num!=VESA_Current_Write_Bank) VESA_SwitchWriteBank( bank_num );
		bank_num = VESA_RowBank[y] + VESA_Read_Bank_Offset;
		if (bank_num!=VESA_Current_Read_Bank) VESA_SwitchReadBank( bank_num );

		// create a pointer to the screen at the position to start blitting
		screen_pos = (BYTE*) (*row_addr + x1);
		cursor_pos = (BYTE*) cursor_addr;
		underneath_pos = (BYTE*) underneath_addr;


		// blit the current row of the cursor
		for (x=x1;x<=x2;x++)
			{
			tmp = *cursor_pos;		// get the next pixel in the row

			if (tmp)				// if its set
				*screen_pos = *underneath_pos;
									// put whats underneath the cursor
									// onto the screen (_SLOW_ -> AT Bus)

			cursor_pos++;			// skip to the next cursor pixel
			screen_pos++;
			underneath_pos++;
			}

		// for the next row, simply add the logical width to the address
		row_addr++;
		cursor_addr += cursor_width;
		underneath_addr += cursor_width;
		}

//	MOUSE_ISR_DISABLED = save_disabled;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DrawCursorRow()
//	DESC:		Draws a row of the mouse cursor
//
//	-------------------------------------------------------------------------
void mouse_DrawCursorRow( int row_y, int row_x1, int row_x2, int options )
	{
	if (!MOUSE_INSTALLED) return;
	if (mouse_Cursor==NULL) return;
	if (mouse_IsHidden()) return;

//	bool save_disabled = MOUSE_ISR_DISABLED;
//	MOUSE_ISR_DISABLED = true;

	// no point drawing if mouse cursor not using given row

	if (row_y<lastmouseY) return;
	if (row_y>=lastmouseY+mouse_Cursor->hdr.height) return;

	// no point drawing if mouse cursor outside row start/end coords

	int m_x1 = lastmouseX;
	int m_x2 = lastmouseX+mouse_Cursor->hdr.width-1;

	if (row_x1<m_x1) row_x1 = m_x1;
	if (row_x2>m_x2) row_x2 = m_x2;

	// make sure we're only drawing to the actual screen dimensions

	CLIP2(row_x1,row_y,0,0,grMaxx,grMaxy);
	CLIP2(row_x2,row_y,0,0,grMaxx,grMaxy);

	int x,tmp;

	// figure out the screen address of the row to be redrawn

	BYTE*	screen_pos;
	BYTE* 	cursor_pos;
	BYTE*	underneath_pos;

	long*	row_addr 		= &VESA_RowPtr[row_y];

	// figure the addresses of the appropriate row of the mouse cursor

	int		relative_y		= row_y - lastmouseY;
	int		relative_x		= row_x1 - lastmouseX;
	int		first_pixel		= relative_y * mouse_Cursor->hdr.width + relative_x;

	long	cursor_addr		= (long) &mouse_Cursor->data[first_pixel];
	long	underneath_addr	= (long) &mouse_UnderCursor->data[first_pixel];

	int		bank_num;

	// figure out the required bank & change iff required
	bank_num = VESA_RowBank[row_y] + VESA_Write_Bank_Offset;
	if (bank_num!=VESA_Current_Write_Bank) VESA_SwitchWriteBank( bank_num );
	bank_num = VESA_RowBank[row_y] + VESA_Read_Bank_Offset;
	if (bank_num!=VESA_Current_Read_Bank) VESA_SwitchReadBank( bank_num );

	// create a pointer to the screen at the position to start blitting
	screen_pos = (BYTE*) (*row_addr + row_x1);

	cursor_pos = (BYTE*) cursor_addr;
	underneath_pos = (BYTE*) underneath_addr;

	// blit the current row of the cursor
	for (x=row_x1;x<=row_x2;x++)
		{
		tmp = *cursor_pos;		// get the next pixel in the row

//		tmp =  0x03;
//		if (x=m_x1) tmp = 0x0f;
//		if (x=m_x2) tmp = 0x0f;


		// if the appropriate pixel in the mouse cursor needs to be drawn
		if (tmp!=0)
			{
			// save whats underneath the cursor iff required
			if ((options&SAVE_UNDERNEATH_CURSOR)!=0) *underneath_pos = *screen_pos;

			// and draw the pixel
			*screen_pos = tmp;
			}

		cursor_pos++;			// skip to the next cursor pixel
		screen_pos++;
		underneath_pos++;
		}

//	MOUSE_ISR_DISABLED = save_disabled;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_Hide()
//	DESC:		Hides the mouse if required.
//
//	-------------------------------------------------------------------------
void mouse_Hide()
	{
	if (!MOUSE_INSTALLED) return;
	if (MOUSE_IS_HIDDEN) return;

	mouse_UndrawCursor();
	MOUSE_IS_HIDDEN = true;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_Show()
//	DESC:		Shows the mouse if required.
//
//	-------------------------------------------------------------------------
void mouse_Show()
	{
	if (!MOUSE_INSTALLED) return;
	if (!MOUSE_IS_HIDDEN) return;

//if (MOUSE_FIRST_TIME_DRAW) {
//DBG(0,"ABOUT TO DRAW MOUSE CURSOR, SAVING UNDERNEATH");
//delay(1000);
//}

	mouse_DrawCursor( SAVE_UNDERNEATH_CURSOR );
	MOUSE_IS_HIDDEN = false;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_IsHidden()
//	DESC:		returns true if mouse is currently hidden
//
//	-------------------------------------------------------------------------
bool mouse_IsHidden()
	{
	return (MOUSE_IS_HIDDEN);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_UpdateCursor()
//	DESC:		Updates cursor on screen iff the mouse moves.
//
//	-------------------------------------------------------------------------
void mouse_UpdateCursor()
	{
	if (!MOUSE_INSTALLED) return;
	if (mouse_IsHidden()) return;

	// update only if the cursor has moved!
	if (HasMoved)
		{
//		bool save_disabled = MOUSE_ISR_DISABLED;
//		MOUSE_ISR_DISABLED = true;

		// undraw the cursor at the last drawn coords

		if (!MOUSE_FIRST_TIME_DRAW) mouse_UndrawCursor();

		// draw the cursor at the new x,y coords

		mouse_DrawCursor( SAVE_UNDERNEATH_CURSOR );

		// reset flag so that we don't keep drawing cursor

		HasMoved = false;
		MOUSE_FIRST_TIME_DRAW = false;

//		MOUSE_ISR_DISABLED = save_disabled;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_SetCursor()
//	DESC:		Sets the mouse cursor to 'curs' as well as the mouse hotspot
//
//	-------------------------------------------------------------------------
void mouse_SetCursor(int hotx,int hoty,MouseCursor* curs)
	{
	if (!MOUSE_INSTALLED) return;
	if (mouse_IsHidden()) return;

	// iff the cursor is different... change the cursor
	if (curs!=mouse_Cursor)
		{
//		bool save_disabled = MOUSE_ISR_DISABLED;
//		MOUSE_ISR_DISABLED = true;

		// undraw the current cursor

		mouse_UndrawCursor();

		// set the new cursor

		mouse_Cursor = curs;

		// draw the new cursor

		mouse_DrawCursor( SAVE_UNDERNEATH_CURSOR );

//		MOUSE_ISR_DISABLED = save_disabled;
		}
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetEvent()
//	DESC:		checks and generates mouse_auto?? events as required
//
//	-------------------------------------------------------------------------

void mouse_GetEvent()
	{
	if (!MOUSE_INSTALLED) return;
	if (mouse_IsHidden()) return;
	if (!eventmgr_INSTALLED) return;

	int i,n;
	long newticks;

	// note: modifiers, x,y set already

	newticks = GetTicks();

	// --- check the left button

	if (leftBut == mouse_PRESSED)
		{
		long ticks = newticks - LMBTicks;
		if ((ticks>mouse_AUTO_DELAY) || (LMBAuto))
			{
			LMBAuto  = true;
			LMBTicks = newticks;
			EventInfo.event = mb_leftauto;
			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}
		}

	// --- check the right button

	if (rightBut == mouse_PRESSED)
		{
		long ticks = newticks - RMBTicks;
		if ((ticks>mouse_AUTO_DELAY) || (RMBAuto))
			{
			RMBAuto  = true;
			RMBTicks = newticks;
			EventInfo.event = mb_rightauto;
			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}
		}

	// --- check the middle button

	if (middleBut == mouse_PRESSED)
		{
		long ticks = newticks - MMBTicks;
		if ((ticks>mouse_AUTO_DELAY) || (MMBAuto))
			{
			MMBAuto  = true;
			MMBTicks = newticks;
			EventInfo.event = mb_middleauto;
			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_handler()
//	DESC:		this is the mouse bios interrupt handler
//
//	-------------------------------------------------------------------------

int	last_ax;
int last_bx;
int last_cx;
int last_dx;

extern char far*        GetStack( void );
extern void             SetStack( char far* );


  #pragma aux 	GetStack = 				\
						"mov dx,ss" 	\
						"mov eax,esp"	\
						value [ dx eax ];

  #pragma aux   SetStack = 				\
						"mov ss,dx" 	\
						"mov esp,eax" 	\
						parm [ dx eax ];

#define STACKSIZE       1024

//===================================================
static void _loadds far mouse_handler(int m_ax,int m_bx,int m_cx,int m_dx)

  #pragma aux 	mouse_handler 			\
						parm [EAX EBX ECX EDX];

	{
	static char         thestack[ STACKSIZE ];

	static long 		thestack_addr = (long) &thestack;
	static long			newstack_addr = thestack_addr + STACKSIZE - 1;

	static char far*    oldstack;
	static char far*	newstack = (char far*) newstack_addr;

//	MOUSE_ISR_DISABLED = true;

	last_ax = m_ax;
	last_bx = m_bx;
	last_cx = m_cx;
	last_dx = m_dx;

	int tmp_leftBut = leftBut;
	int tmp_rightBut = rightBut;
	int tmp_middleBut = middleBut;

	leftBut = ((m_cx & bit0)!=0);
	rightBut = ((m_cx & bit1)!=0);
	middleBut = ((m_cx & bit2)!=0);

	HasMoved = ((m_ax & bit0)!=0);

	mouseX = m_dx;
	mouseY = m_bx;

	if (MOUSE_ISR_DISABLED) return;

	if (eventmgr_INSTALLED)
		{
		// save state ready for event generation
		EventInfo.modifiers = keyboard_GetState();
		EventInfo.where.x = mouseX;
		EventInfo.where.y = mouseY;

		// if the left button has changed state
		if (leftBut != tmp_leftBut)
			{
			// figure appropriate event to generate
			if (leftBut == mouse_PRESSED)
				{
				EventInfo.event = mb_leftdn;
				LMBTicks = GetTicks();
				}
			else
				{
				EventInfo.event = mb_leftup;
				LMBAuto  = false;
				}

			// generate the event
			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}

		// check the right button
		if (rightBut != tmp_rightBut)
			{
			if (rightBut == mouse_PRESSED)
				{
				EventInfo.event = mb_rightdn;
				RMBTicks = GetTicks();
				}
			else
				{
				EventInfo.event = mb_rightup;
				RMBAuto  = false;
				}

			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}

		// and the middle button
		if (middleBut != tmp_middleBut)
			{
			if (middleBut == mouse_PRESSED)
				{
				EventInfo.event = mb_middledn;
				MMBTicks = GetTicks();
				}
			else
				{
				EventInfo.event = mb_middleup;
				MMBAuto  = false;
				}

			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}

		// if we're generating move events
		if ((MoveEventsOn) && (HasMoved))
			{
			EventInfo.event = mb_move;
			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}
		}

	if (mouse_IsHidden()) return;
	if (MOUSE_EXTERNAL_UPDATE) return;

	if (HasMoved)
		{
		oldstack = GetStack();
		SetStack( newstack );

		mouse_UpdateCursor();

		SetStack( oldstack );
		}

//	MOUSE_ISR_DISABLED = false;
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DriverLoaded()
//	DESC:		Returns true if mouse driver is loaded.
//
//	-------------------------------------------------------------------------
bool mouse_DriverLoaded()
	{
	union REGPACK regs;
	SYS_Memset(&regs,0,sizeof(regs));

	// try resetting mouse driver
	regs_w.ax = 0;
	intr(0x33, &regs);

	// and return true if driver is loaded
	return (regs_w.ax==0xFFFF);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_InstallHandler()
//	DESC:		Either	(a) installs the mouse interrupt handler.
//						(b) installs mouse autodetection functions.
//
//	-------------------------------------------------------------------------
void mouse_InstallHandler()
	{
	if (MOUSE_INSTALLED) return;

	// make sure we have a mouse driver in memory
	if (!mouse_DriverLoaded()) return;

	int                 result;
	union REGPACK       	regs;

	SYS_Memset( &regs, 0, sizeof( union REGPACK ) );

	regs_w.ax = 0;
	intr( MOUSE_INT, &regs );
	result = regs_w.ax;

	if( result != 0 )
		{
		// create room for whats under cursor.... (max. size == 32 * 32)
		mouse_UnderCursor = grCreateImage(0,0,31,31,0);

		// VESA/SuperVGA mode, we have to draw the mouse cursor ourselves
		void (_loadds far* function_ptr)(int,int,int,int);
		function_ptr = &mouse_handler;

		regs_w.ax  = 0x000C;
		regs_w.cx  = 0x007F;
		regs.x.edx = FP_OFF( function_ptr );
		regs_w.es  = FP_SEG( function_ptr );

		intr( MOUSE_INT, &regs );

		// flag to say we are autodetecting
		MOUSE_INSTALLED = true;

		// set up initial mouse environment
		mouse_Reset();

		// and setup our ticks
		LMBTicks = GetTicks();
		RMBTicks = GetTicks();
		MMBTicks = GetTicks();
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_RemoveHandler()
//	DESC:		Removes the mouse interrupt handler.
//
//	-------------------------------------------------------------------------
void mouse_RemoveHandler()
	{
	if (!MOUSE_INSTALLED) return;

	mouse_SetCursor(0,0,DEFAULT_CURSOR);
	mouse_Hide();

	// tell the mouse driver to init, ie. remove our event handler.... :-)
	union REGPACK       regs;

	SYS_Memset( &regs, 0, sizeof( union REGPACK ) );
	regs_w.ax = 0;
	intr( MOUSE_INT, &regs );

	// get rid of what was under cursor (after removing handler!!!)
	grDestroyImage(mouse_UnderCursor);

	// and set the flag
	MOUSE_INSTALLED = false;
	}

/*

	// this is a concept for a coded mouse cursor to speed up mouse cursor
	// drawing

	p_index = 0;

		// 1000000001000000
		// 1100000001100000
		// 1110000001110000
		// 1111111111111100
		// 0010000000010000

		mouseCursor ARROW_CURSOR = {
		0x04,	// num_rows
		0x03, 	// num_elems in row

		0x01, 0xFF, 0x08, 0x00, 0x01, 0xFF,	// row data (ie. 1xFF, 8x00, 1xFF)
		0x03, 0x02, 0xFF, 0x07, 0x00, 0x02, 0xFF,
		0x03, 0x03, 0xFF, 0x06, 0x00, 0x03, 0xFF,
		0x02, 0x0E, 0xFF, 0x02, 0x00,

		}
		*p = 1xFF,8x00,1xFF,6x00

		rle_length = p[p_index++] * 2;

		do
			{
			num_colors  = p[p_index++];
			color 		= p[p_index++];

			if (color==0) { screen_pos += num_colors; continue; }


			if (num_colors>3)

			w_color_mask= (color<<8) + color;
			d_color_mask= (w_color_mask<<16) + w_color_mask;

			dpos		= (DWORD*) screen_pos;

			i 			=  num_colors;
			while (i>3)
				{
				*dpos = d_color_mask;
				dpos++;
				i-=4;
				}

			wpos = (WORD*) dpos;
			while (i>1)
				{
				*wpos = w_color_mask;
				i -= 2;
				}

			bpos = (WORD*) bpos;
			while (i>0)
				{
				*bpos = color;
				}

			screen_pos += num_colors;
			}
		while (p_index<rle_length);

		BYTE* bpos;
		WORD* wpos;
		DWORD* dpos;

		switch (yrow)
			{
			case 0:		bpos = (BYTE*) screen_pos; *bpos = 0xFF;
						break;
			case 1:		wpos = (WORD*) screen_pos; *wpos = 0xFFFF;
						break;
			case 2:		bpos = (BYTE*) screen_pos++; *bpos = 0xFF;
						wpos = (WORD*) screen_pos;   *wpos = 0xFFFF;
						break;
			case 4:		dpos = (DWORD*) screen_pos;  *dpos = 0xFFFFFFFF;
						break;
			case 3:		screen_pos++;
						screen_pos++;
						bpos = (BYTE*) screen_pos; *bpos = 0xFF;
						break;
			}
*/

