//	-------------------------------------------------------------------------
//
//	PALETTES.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file declares the palette functions used
//					by the game development library.
//
//	-------------------------------------------------------------------------
#ifndef __palettes_hpp
#define __palettes_hpp

#include	"stddefns.hpp"

// our palette header

struct grPaletteHeader
	{
	uchar numentries;
	};

// and the generic "palette" structure

struct grPalette
	{
	grPaletteHeader hdr;
	uchar data[];
	};

extern grPalette*	grPALETTE;

//-------------------------------------------------------------------------

// create a palette of the specified size
grPalette*	grCreatePalette(int numentries);

// destroy a specified palette...
void		grDestroyPalette(grPalette* pal);

// return the size in bytes of a palette with numentries colours
int			grPaletteSize(int numentries);

// set palette entry #num to r,g,b
void		grSetPaletteEntry(uchar entry,uchar r,uchar g,uchar b);

// gets palette entry #num
void		grGetPaletteEntry(uchar entry,uchar* r,uchar* g,uchar* b);

// set an individual entry on the real palette
void		grSetPaletteEntry(grPalette* pal,int index,int r,int g,int b);

// ????
void		grSetStandardPalette(int mult,int red,int green,int blue);

// sets palette to palette_ptr, from 'start' to 'end'
void		grSetPalette(grPalette* pal,uchar start,uchar end);

// gets palette similar to SetPalette()
void		grGetPalette(grPalette* pal,uchar start,uchar end);

// fades out the palette
void		grFadeOutPalette(grPalette* pal,uchar start,uchar end,uchar timesin);
// fades in the palette
void		grFadeInPalette(uchar start,uchar end,uchar timesin);

// rotates the palette
void		grRotatePalette(uchar start,uchar end);

#endif