//	-------------------------------------------------------------------------
//
//	PALFILE.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew Carvel Evans
//
//	DESCRIPTION:	This file implements the PALFILE class
//
//	-------------------------------------------------------------------------

#include "palfile.hpp"

PALFile::PALFile(char* filename)
	: File(filename) {}
PALFile::PALFile(String* filename)
	: File(filename) {}
PALFile::PALFile(char* path,char* filename)
	: File(path,filename) {}
PALFile::PALFile(String* path,String* filename)
	: File(path,filename) {}

// loads a palette, converts it to a grPalette & returns the grPalette...
grPalette*	PALFile::LoadPalette()
	{
	// skip out if file is not open
	if (!file_isopen) return NULL;

	grPalette*		palette;

	// create the palette space
	palette = grCreatePalette(PALETTE_SIZE/3);

	// if we created room....
	if (palette)
		{
		// load the palette

		// if the file isn't open, open it now
		bool opened = false;
		if (!file_isopen) { Open(); opened = true; }

		// make sure there is a full palette there
		if (FileSize() != PALETTE_SIZE) { grDestroyPalette(palette); return NULL; }

		// load the image into memory
		SeekTOF();
		Read((char*) &palette->data[0], PALETTE_SIZE);

		// close the file if we opened it....
		if (opened) Close();

		// and return our grPalette....
		return palette;
		}

	// an error occurred
	return NULL;
	}

// saves a palette, just as a raw 768 byte file without header
int		PALFile::SavePalette(grPalette* palette)
	{
	// save the palette to disk
	SeekTOF();
	return Write((char*) &palette->data[0], PALETTE_SIZE);
	}

void 		PALFile::Convert256PaletteTo64Palette(grPalette* palette)
	{
	// adjust the palette
	for (int i=0; i<PALETTE_SIZE; i++)
		palette->data[i] = palette->data[i] >> 2;
	}
