//	-------------------------------------------------------------------------
//
//	PCXFILE.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew C. Evans & Darren J. Dwyer
//
//	DESCRIPTION:	This file implements the PCXFILE class
//
//	-------------------------------------------------------------------------

#include "pcxfile.hpp"
#include "pcx.hpp"
#include "dos.h"
#include "keyboard.hpp"

static	PCX_imageHeader	standard_header;
static	bool			standard_header_loaded = false;

PCXFile::PCXFile(char* filename) : File(filename)
	{
	if (!standard_header_loaded)
		{
		standard_header.manufacturer = 10;
		standard_header.version = 5;
		standard_header.encoding = 1;
		standard_header.bits_per_pixel = 8;
		standard_header.x = 0;
		standard_header.y = 0;
		standard_header.horz_res = 640;
		standard_header.vert_res = 480;
		standard_header.num_color_planes = 1;
		}
	}

PCXFile::PCXFile(String* filename) : File(filename)
	{
	if (!standard_header_loaded)
		{
		standard_header.manufacturer = 10;
		standard_header.version = 5;
		standard_header.encoding = 1;
		standard_header.bits_per_pixel = 8;
		standard_header.x = 0;
		standard_header.y = 0;
		standard_header.horz_res = 640;
		standard_header.vert_res = 480;
		standard_header.num_color_planes = 1;
		}
	}

PCXFile::PCXFile(char* path,char* filename) : File(path,filename)
	{
	if (!standard_header_loaded)
		{
		standard_header.manufacturer = 10;
		standard_header.version = 5;
		standard_header.encoding = 1;
		standard_header.bits_per_pixel = 8;
		standard_header.x = 0;
		standard_header.y = 0;
		standard_header.horz_res = 640;
		standard_header.vert_res = 480;
		standard_header.num_color_planes = 1;
		}
	}

PCXFile::PCXFile(String* path,String* filename) : File(path,filename)
	{
	if (!standard_header_loaded)
		{
		standard_header.manufacturer = 10;
		standard_header.version = 5;
		standard_header.encoding = 1;
		standard_header.bits_per_pixel = 8;
		standard_header.x = 0;
		standard_header.y = 0;
		standard_header.horz_res = 640;
		standard_header.vert_res = 480;
		standard_header.num_color_planes = 1;
		}
	}

void		PCXFile::LoadImage(grImage* image)
	{
//HEAPCHK("in LoadImage (EXTERNAL)");

	bool	opened = false;

	// Open if file is not open
	if (!file_isopen) { Open(); opened = true; }

//DBG(470,"IN PCXFILE:: LOADIMAGE(),   BEFORE DOING ANYTHING");
//delay(500);
	long				image_size;
	PCX_image*			pcx_img;

	// get the size of the completed grImage
	image_size = SizeOfImage();

//DBG(470,"IN PCXFILE:: LOADIMAGE(),   AFTER SIZEOFIMAGE()");
//delay(500);
	// if image exists
	if (image)
		{
		// load the pcx image
//DBG(470,"IN PCXFILE:: LOADIMAGE(),   BEFORE LOADPCXIMAGE()");
//delay(500);
		pcx_img = LoadPCXImage();

		if (!pcx_img)
			{
DBG(470,"IN PCXFILE:: LOADIMAGE(),   LOADPCXIMAGE() ERROR");
			DBG(460,"Could not load %s's PCX_image data.",file_name->Contents());
			keyboard_Pause();
			return;
			}

		// decompress the PCX_image into a grImage
//DBG(470,"IN PCXFILE:: LOADIMAGE(),   CONVERTING PCX IMAGE TO IMAGE");
//delay(500);
		ConvertPCXImageToImage(image, pcx_img);

		// free the pcx image
//DBG(460,"PCX IMAGE:   WIDTH = %d,   HEIGHT = %d",image->hdr.width,image->hdr.height);
//DBG(470,"IN PCXFILE:: LOADIMAGE(),   BEFORE FREEING ");
//keyboard_Pause();
		SYS_Free(pcx_img); pcx_img = NULL;

//DBG(470,"IN PCXFILE:: LOADIMAGE(),   HERE7");
		if (opened) Close();

		// and return our grImage....
		return;
		}
	else
		{
		DBG(470,"PCXFile::LoadImage(buffer) has been passed a NULL pointer");
		delay(1000);
		}

//DBG(470,"IN PCXFILE:: LOADIMAGE(),   BEFORE CLOSING FILE");
	if (opened) Close();

	// an error occurred
	return;
	}


// loads a pcx image, converts it to a grImage & returns the grImage...
grImage*	PCXFile::LoadImage()
	{
//HEAPCHK("in LoadImage(), the one that returns a pointer to a new grImage");

//DBG(470,"PCXFILE:: IN LOADIMAGE()");
	bool	opened = false;

	// Open if file is not open
	if (!file_isopen) { Open(); opened = true; }

	grImage*			image;
	int					image_size;
	PCX_image*			pcx_img;

	// get the size of the completed grImage
	image_size = SizeOfImage();

//HEAPCHK("in PCXFile::LoadImage(), after SizeOfImage()");

	// create the image space
//DBG(400,"Malloc'ing %d bytes for DECOMPRESSED image",image_size);

	image = (grImage*) SYS_Malloc(image_size);
//if (image)
//DBG(410,"Malloc'd %d bytes for DECOMPRESSED image",image_size);
//else
//DBG(410,"Malloc %d bytes FAILED for DECOMPRESSED image",image_size);

//HEAPCHK("in PCXFile::LoadImage(), after SYS_Malloc(%d)",image_size);

	// if we created room....
	if (image)
		{
		// load the pcx image
//DBG(470,"IN PCXFILE:: LOADIMAGE(),   HERE1");
		pcx_img = LoadPCXImage();

//	HEAPCHK("in PCXFile::LoadImage(), after LoadPCXImage() 		ZERO");

		if (!pcx_img)
			{
//			DBG(460,"Could not load %s's PCX_image data.",file_name->Contents());
			SYS_Free(image);

//	HEAPCHK("in PCXFile::LoadImage(), after LoadPCXImage(),    ONE");

			return NULL;
			}

		// decompress the PCX_image into a grImage
		ConvertPCXImageToImage(image, pcx_img);


//	HEAPCHK("in PCXFile::LoadImage(), after ConvertPCXImageToImage");

		// free the pcx image
		SYS_Free(pcx_img); pcx_img = NULL;

//	HEAPCHK("in PCXFile::LoadImage(), after LoadPCXImage(),    TWO");

		if (opened) Close();

//	HEAPCHK("in PCXFile::LoadImage(), after LoadPCXImage(),    THREE");

		// and return our grImage....
//	DBG(460,"In PCXFile::LoadImage(), about to return image as follows:");
//	DBG(470,"Image Addr: %X    Size: %d bytes   Width: %d    Height: %d",
//			image,grImageSize(image),image->hdr.width,image->hdr.height);
//	keyboard_Pause();
		return image;
		}
	else
		{
		DBG(470,"Could not create room for %s's grImage data of %ld bytes.",file_name->Contents(),image_size);
		keyboard_Pause();
//		delay(1000);
		}

	if (opened) Close();

	// an error occurred
	return NULL;
	}

// converts a grImage to a PCX image and saves it using the current palette
void		PCXFile::SaveImage(grImage* image)
	{
	bool	opened = false;
	// Open if file is not open
	if (!file_isopen) { Open(); opened = true; }

	long				pcx_siz;
	PCX_image* 			pcx_img;

	// figure out the maximum size of the compressed image (based on 1:1)
	pcx_siz = (image->hdr.width * image->hdr.height) + PCX_HEADER_SIZE + PCX_PALETTE_SIZE;

	// create a temporary space for the compressed PCX image
	pcx_img = (PCX_image*) SYS_Malloc(pcx_siz);

	// if we could create a PCX image
	if (pcx_img)
		{
		// compress the grImage returning the size of the compressed PCX image
		pcx_siz = ConvertImageToPCXImage(pcx_img, image);

		grPalette* grpal = grCreatePalette(256);

		// get the current system palette
		grGetPalette(grpal,0,255);

		// and copy it into the pcx image
		ConvertPaletteToPCXPalette(pcx_img,grpal);

		grDestroyPalette(grpal);

		// save PCX_image
		SavePCXImage(pcx_img,pcx_siz);

		// free the temporary pcx image
		SYS_Free(pcx_img);
		}
	if (opened) Close();

	return;
	}

// loads a pcx palette, converts it to a grPalette & returns the grPalette...
grPalette*	PCXFile::LoadPalette()
	{
	bool	opened = false;
	// Open if file is not open
	if (!file_isopen) { Open(); opened = true; }

	grPalette*			palette;
	PCX_image*			pcx_img;

	// create the palette space
	palette = grCreatePalette(PCX_PALETTE_SIZE/3);

	// if we created room....
	if (palette)
		{
		// load the pcx image
		pcx_img = LoadPCXImage();
		if (!pcx_img) { free(palette); return NULL; }

		// convert the PCX_image's palette into a grPalette
		ConvertPCXPaletteToPalette(palette, pcx_img);

		// free the pcx image
		SYS_Free(pcx_img); pcx_img = NULL;

		if (opened) Close();

		// and return our grPalette....
		return palette;
		}

	if (opened) Close();

	// an error occurred
	return NULL;
	}

long		PCXFile::SizeOfImage()
	{
//DBG(470,"PCXFILE:: IN SIZEOFIMAGE()");

	// load PCX header information
	PCX_imageHeader* hdr = LoadPCXHeader();

	// calc size of decompressed image. NOTE: NO -1 ON MAX WIDTH,HEIGHT
	long	decompressed_size = grImageSize(0,0,hdr->width,hdr->height);

//DBG(470,"PCXFILE:: IN LOADIMAGE(), decompressed size = %d",decompressed_size);

	// clean up header
	SYS_Free(hdr);

	// and return our decompressed size
	return decompressed_size;
	}

long		PCXFile::SizeOfPCXImage()
	{
	// it's simply the filesize :-)
	return FileSize();
	}

PCX_imageHeader*	PCXFile::LoadPCXHeader()
	{
	// if the file isn't open, open it now
	bool opened = false;
	if (!file_isopen) { Open(); opened = true; }

	// create space for the PCX header
	PCX_imageHeader*	hdr = (PCX_imageHeader*) SYS_Malloc(PCX_HEADER_SIZE);

	// if we allocated space
	if (hdr)
		{
		// read the PCX header from the top of the file
		SeekTOF();
		Read((char*) hdr,PCX_HEADER_SIZE);
		}
	else
		return NULL;

	// close the file if we opened it....
	if (opened) Close();

	if (!standard_header_loaded)
		{
		SYS_Memcpy(&standard_header,hdr,sizeof(PCX_imageHeader));
		standard_header_loaded = true;
		}

	// and return the newly created header
	return hdr;
	}

PCX_image*			PCXFile::LoadPCXImage()
	{
//HEAPCHK("In LoadPCXImage(), just coming in");

	// if the file isn't open, open it now
	bool opened = false;
	if (!file_isopen) { Open(); opened = true; }

	// get size of the complete PCX image
	long	pcx_size = SizeOfPCXImage();

//HEAPCHK("In LoadPCXImage(), after SizeOfPCXImage()");

//DBG(450,"Size of pcx image == %d bytes",pcx_size);

	// allocate memory for the PCX image
	PCX_image* pcx_img = (PCX_image*) SYS_Malloc(pcx_size);

//HEAPCHK("In LoadPCXImage(), after SYS_Malloc(%d) bytes",pcx_size);

	// if we created the image
	if (pcx_img)
		{
		// load the image into memory
		SeekTOF();
		Read((char*) &pcx_img->header, PCX_HEADER_SIZE);
		Read((char*) &pcx_img->buffer[0], pcx_size - PCX_HEADER_SIZE - PCX_PALETTE_SIZE);
		Read((char*) &pcx_img->palette[0], PCX_PALETTE_SIZE);

//HEAPCHK("In LoadPCXImage(), after reading",pcx_size);

		}
	else
		{
//DBG(450,"Could not create room for %s's PCX_image data of %ld bytes.",file_name->Contents(),pcx_size);
//		delay(1000);
//HEAPCHK("In LoadPCXImage(), after NOT reading",pcx_size);

		}

	// close the file if we opened it....
	if (opened) Close();

//DBG(460,"In LoadPCXImage(), before returning");

	// and return the newly created PCX_image
	return pcx_img;
	}

void				PCXFile::SavePCXImage(PCX_image* pcx_img,long pcx_size)
	{
	// if this file exists on disk, then delete it!!!
	if (Exists()) Delete();

	// then create it
	Create();

	// if the file isn't open, open it now
	bool opened = false;
	if (!file_isopen) { Open(); opened = true; }

	// if there is an image to write
	if (pcx_img)
		{
		// then write the image to disk
		SeekTOF();
		Write((char*) &pcx_img->header, PCX_HEADER_SIZE);
		Write((char*) &pcx_img->buffer[0], pcx_size - PCX_HEADER_SIZE - PCX_PALETTE_SIZE);
		Write((char*) &pcx_img->palette[0], PCX_PALETTE_SIZE);
		}

	// close the file if we opened it....
	if (opened) Close();
	}

void		PCXFile::ConvertPCXImageToImage(grImage* image,PCX_image* pcx_img)
	{
	// setup grImage header
	image->hdr.width = pcx_img->header.width + 1;
	image->hdr.height = pcx_img->header.height + 1;

//DBG(410,"CONVERTING PCX IMAGE TO grImage of width,height == %d,%d",image->hdr.width,image->hdr.height);

	long 	pcx_index = 0;
	long 	gr_index = 0;
	long    run_length = 0;

	// note: * 	Standard pcx images seem to be compressed on a row-by-row
	//		   	basis.
	//
	//		 *	Odd width images are compressed using an even bytes_per_line
	//			count (rounded up)
	//			ie. a 23x23 image has a bytes per line of 24!!! (23+1)
	//

	// for each row in the finished image
	for (int y=0; y<image->hdr.height; y++)
		{
		// decompress each scan line
		run_length = PCX_Decompress( (char*) &image->data[gr_index],
									 (char*) &pcx_img->buffer[pcx_index],
									 pcx_img->header.bytes_per_line,
									 image->hdr.width );

		// increment our pcx_index by the amount of compressed data
		pcx_index += run_length;

		// and increment our gr_index by the amount of decompressed data
		gr_index += image->hdr.width;

//HEAPCHK("in PCXFile::ConvertPCXImageToImage(), y == %d",y);

		}
	}

long		PCXFile::ConvertImageToPCXImage(PCX_image* pcx_img,grImage* image)
	{
	// setup PCX_image header
	pcx_img->header.manufacturer = standard_header.manufacturer;
	pcx_img->header.version = standard_header.version;
	pcx_img->header.encoding = standard_header.encoding;
	pcx_img->header.bits_per_pixel = standard_header.bits_per_pixel;
	pcx_img->header.x = standard_header.x;
	pcx_img->header.y = standard_header.y;
	pcx_img->header.horz_res = standard_header.horz_res;
	pcx_img->header.vert_res = standard_header.vert_res;
	pcx_img->header.num_color_planes = standard_header.num_color_planes;

	// pcx files seem to use EVEN bytes per line counts even with odd
	// sizes.... simply round up iff width is odd
	pcx_img->header.width = image->hdr.width - 1;// 23x23 / size for a 24x24
	pcx_img->header.height = image->hdr.height - 1;// 23x23 / size for a 24x24
	pcx_img->header.bytes_per_line = image->hdr.width + (image->hdr.width%2);

	long 	gr_index = 0;
	long 	pcx_index = 0;
	long	run_length = 0;

	// for each row in the image
	for (int y=0; y<image->hdr.height; y++)
		{
		// compress the appropriate scanline
		run_length = PCX_Compress( (char*) &pcx_img->buffer[pcx_index],
								   (char*) &image->data[gr_index],
								   pcx_img->header.bytes_per_line,
								   image->hdr.width );

		// increment the image pointer by the size of the uncompressed data
		gr_index += image->hdr.width;

		// and inc the pcx image pointer by the size of the compressed data
		pcx_index += run_length;
		}

	// for purposes of saving this file we return the size of the PCX_image
	return PCX_HEADER_SIZE + pcx_index + PCX_PALETTE_SIZE;
	}

void		PCXFile::ConvertPCXPaletteToPalette(grPalette* palette, PCX_image* pcx_img)
	{
	// adjust the palette
	for (int i=0; i<PCX_PALETTE_SIZE; i++)
		palette->data[i] = pcx_img->palette[i] >> 2;
	}

void		PCXFile::ConvertPaletteToPCXPalette(PCX_image* pcx_img, grPalette* palette)
	{
	// adjust the palette
	for (int i=0; i<PCX_PALETTE_SIZE; i++)
		pcx_img->palette[i] = palette->data[i] << 2;
	}
