//	-------------------------------------------------------------------------
//
//	PCXFILE.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew C. Evans & Darren J. Dwyer
//
//	DESCRIPTION:	This file contains definitions for the PCXFILE class
//
//	-------------------------------------------------------------------------

#ifndef __pcxfile_hpp
#define __pcxfile_hpp

#include "file.hpp"
#include "pcx.hpp"
#include "graphics.hpp"
#include "palettes.hpp"

#define			PCX_HEADER_SIZE				128L
#define			PCX_PALETTE_SIZE			768L

//	PCX_Header:	a header for a bitmap using standard PCX graphics
//					compression format
struct PCX_imageHeader
	{
	BYTE		manufacturer;		// 10
	BYTE		version;			// 5
	BYTE		encoding;			// 1 for Run Length Encoding (RLE)
	BYTE		bits_per_pixel;		// 8
	WORD		x, y;				// 0,0
	WORD		width, height;		// 23x23 / size for a 24x24
	WORD		horz_res, vert_res;	// 640x480 / screen x and y direction
	BYTE		ega_palette[48];	// ignore
	BYTE		reserved;			// ignore
	BYTE		num_color_planes;	// 8 / number of planes in image
	WORD		bytes_per_line;		// 24 (for a 24x24) bytes per one horizontal line
	WORD		palette_type;		// ignore
	BYTE		padding[58];		// extra space
	};

struct PCX_image
	{
	PCX_imageHeader	header;
	BYTE			palette[PCX_PALETTE_SIZE];
	BYTE			buffer[];
	};


class PCXFile : public File
	{
	public:
						PCXFile(char* filename);
						PCXFile(String* filename);
						PCXFile(char* path,char* filename);
						PCXFile(String* path,String* filename);

		// malloced before load, i.e. loaded before or for memory management
		virtual void		LoadImage(grImage* image);

		virtual grImage*	LoadImage();
		virtual void		SaveImage(grImage* image);

		virtual grPalette*	LoadPalette();
//		virtual void		SavePalette(grPalette);

//	protected:

		virtual	long				SizeOfImage();
		virtual long				SizeOfPCXImage();

		virtual	PCX_imageHeader*	LoadPCXHeader();
		virtual	PCX_image*			LoadPCXImage();
		virtual	void				SavePCXImage(PCX_image* pcx_img,long pcx_size);

		virtual	void				ConvertPCXImageToImage(grImage* image,PCX_image* pcx_img);
		virtual	long				ConvertImageToPCXImage(PCX_image* pcx_img,grImage* image);

		virtual void				ConvertPCXPaletteToPalette(grPalette* palette, PCX_image* pcx_img);
		virtual void				ConvertPaletteToPCXPalette(PCX_image* pcx_img, grPalette* palette);
	};

#endif