//	-------------------------------------------------------------------------
//
//	PROGRAM.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file defines the Program class which enables rapid
//					program development by standardising the basic program
//					to include all libraries, etc. in one base object.
//
//					=== THIS FILE IS MEANT TO BE USED AS A TEMPLATE FOR ===
//                      ALL EVENT-DRIVEN GUI PROJECTS!!!
//
//	-------------------------------------------------------------------------

// the following line includes all libraries in appropriate order!!

#include "program.hpp"
#include "accessry.hpp"
#include "buttons.hpp"
#include "gui.hpp"
#include "fonts.hpp"
#include "grmisc.hpp"
#include "resmgr.hpp"

// set our stack to be 30K (approx)

extern unsigned _stklen = 30000;

// -----------------

char		PROGRAM_PATH_BUFFER[256];
char*		PROGRAM_PATH = (char*) &PROGRAM_PATH_BUFFER;

int			SYSTEM_FONT = 0;

//	-------------------------------------------------------------------------
//
//	FUNC:		CONSTRUCTOR for Program
//	DESC:       This is the general-purpose constructor for any Program
//				& derived objects. It sets up the event-driven GUI and
//				other associated libraries.
//
//	-------------------------------------------------------------------------

Program::Program(char* title,char* init_string,int options)
	{
#ifdef __WATCOMC__
	_grow_handles(50);
#endif

	// do a simple memory check & set up the memory management functions

	SYS_CheckMemory();

	HEAPCHK("After SYS_CheckMemory()");

	// initialize our cache management system

	InitResourceManager();

	HEAPCHK("After InitResourceManager()");

	// create the program's title

	int len = strlen(title);

	TITLE = (char*) SYS_Malloc(len+1);
	if (TITLE) SYS_Memcpy(TITLE, title,len);
	TITLE[len] = 0x00;

	// create default path
//	printf("\n\n\nGDE: Init String == >%s<\n",init_string);

	len = strlen(init_string);
//	printf("len = %d\n",len);

	while (init_string[len]!='\\') len--;

//	printf("len = %d\n",len);

	for (int i=0;i<len;i++)
		PROGRAM_PATH[i] = init_string[i];
	PROGRAM_PATH[i++] = '\\';
	PROGRAM_PATH[i++] = 0x00;

	printf("\n\n\nGDE: Executing %s --> %s\n\n\n",PROGRAM_PATH,TITLE);
//	delay(2000);

//	getch();

	// save selected options
	selected_options = options;

	// install graphics library

	printf("Installing VESA driver\n\n");
	delay(2);
//    getch();

	if (!grInitVesa(gr640x480x256))
		{
		VESA_ResetMode(grOrigVideoMode);
		printf("--- ERROR: Could not initialize 640x480x256 VESA mode!!!\n");
		getch();
		exit(1);
		}


//	getch();

	// install system font
//	grDrawFilledRect(0,0,100,100,grLIGHTBLUE);
//	grBlitRegion(0,0,100,100);

//	getch();

//	grDrawString(0,0,"Installing SYSTEM FONT");
//	grBlitRegion(0,0,grMaxx,grFontHeight());

	if (selected_options&prg_InstallSystemFont)
		{
		SYSTEM_FONT = grInitFont(PROGRAM_PATH,"RESOURCE\\SYSTEM.FNT");

		if (SYSTEM_FONT<0)
			{

			grExitVesa();
			printf("ERROR: Could not initialise %sSYSTEM.FNT\n",PROGRAM_PATH);
			exit(1);
			}
		else
			grSetFont(SYSTEM_FONT);

		grDrawString(0,0,"Installed SYSTEM FONT");
		grBlitRegion(0,0,grMaxx,grFontHeight());
		delay(2);
		}

	HEAPCHK("After grInitFont(SYSTEM_FONT)");

	// install mouse functions

	grDrawString(0,10,"Installing MOUSE functions");
	grBlitRegion(0,10,grMaxx,10+grFontHeight());
	mouse_InstallHandler();
	delay(2);

	HEAPCHK("After mouse_InstallHandler");

	// install keyboard functions

	grDrawString(0,20,"Installing KEYBOARD functions");
	grBlitRegion(0,20,grMaxx,20+grFontHeight());
	keyboard_InstallHandler();
	delay(2);

	HEAPCHK("After keyboard_InstallHandler");

	// install timer functions

	grDrawString(0,30,"Installing TIMER functions");
	grBlitRegion(0,30,grMaxx,30+grFontHeight());
	timer_InstallHandler();
	delay(2);

	HEAPCHK("After timer_InstallHandler");

	// install standard GUI functions
	if (selected_options&prg_InstallGUI)
		{
		grDrawString(0,40,"Installing GUI functions");
		grBlitRegion(0,40,grMaxx,40+grFontHeight());
		InstallGUI(PROGRAM_PATH,"RESOURCE\\GUI.RES");
		delay(2);
		}

	HEAPCHK("After InstallGUI()");

	// install the Event Manager

	grDrawString(0,50,"Installing EVENT MANAGER functions");
	grBlitRegion(0,50,grMaxx,50+grFontHeight());
	if (!eventmgr_InstallEVENTMGR())
		{
		grExitVesa();
		printf("--- ERROR OCCURRED INSTALLING EVENT MANAGER!\n");
		exit(1);
		}
	delay(2);

	HEAPCHK("After InstallEVENTMGR()");

	// install the debugging functions

//	grDrawString(0,60,"Installing DEBUGGING functions");
//	delay(2);
//	DBG = new DebugWindow();

	// initialize the random-number-generator with an unknown seed

	randomize();

	// finished installing everything

	grDrawString(0,100,"Program initialise complete");
	grBlitRegion(0,100,grMaxx,100+grFontHeight());
	delay(2);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		DESTRUCTOR for Program
//	DESC:		Deletes/Removes all libraries initialised in the constructor.
//
//	-------------------------------------------------------------------------
Program::~Program()
	{
	// clear the video
	grClearScreen(grBACKGROUND);

	// remove debugging functions

//	grDrawString(0,0,"Removing DEBUGGING functions");
//	delay(2);
//	delete DBG;

	// remove the Event Manager

	grDrawString(0,0,"Removing EVENT MANAGER functions");
	grBlitRegion(0,0,grMaxx,0+grFontHeight());
	delay(2);
	eventmgr_RemoveEVENTMGR();

	// remove the standard GUI functions
	if (selected_options&prg_InstallGUI)
		{
		grDrawString(0,10,"Removing GUI functions");
		grBlitRegion(0,10,grMaxx,10+grFontHeight());
		delay(2);
		ExitGUI();
		}

	// remove the timer functions

	grDrawString(0,20,"Removing TIMER functions");
	grBlitRegion(0,20,grMaxx,20+grFontHeight());
	delay(2);
	timer_RemoveHandler();

	// remove the keyboard functions

	grDrawString(0,30,"Removing KEYBOARD functions");
	grBlitRegion(0,30,grMaxx,30+grFontHeight());
	delay(2);
	keyboard_RemoveHandler();

	// remove the mouse functions

	grDrawString(0,40,"Removing MOUSE functions");
	grBlitRegion(0,40,grMaxx,40+grFontHeight());
	delay(2);
	mouse_RemoveHandler();

	// remove the system font
	if (selected_options&prg_InstallSystemFont)
		{
		grDrawString(0,50,"Removing SYSTEM FONT");
		grBlitRegion(0,50,grMaxx,50+grFontHeight());
		delay(2);
		grExitFont();
		}

	// remove the graphics library

	grDrawString(0,60,"Removing VESA functions");
	grBlitRegion(0,60,grMaxx,60+grFontHeight());
	delay(2);
	grExitVesa();

	// reset default drive & path

//	setdisk(DefaultDrive);
//	chdir(DefaultPath->Contents());
//	delete DefaultPath;

	printf("\n\n\nGDE: Finished executing %s",TITLE);

	// clean up after our title

	SYS_Free(TITLE);

	// finished program termination
	ExitResourceManager();
	}

void	Program::Init()
	{
	}


void	Program::Exit()
	{
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		MainLoop()
//	DESC:       This is the main loop of the program.
//				It does the following:
//					(1)	Create all EventHandler objects used in the MainLoop
//					(2)	Tell the EVENTMGR to Execute()
//					(3)	Delete all EventHandler objects used in the MainLoop
//
//	-------------------------------------------------------------------------
void Program::MainLoop()
	{
	// create all objects used in the main game phase

	// tell the event manager to execute using the handlers!!!
	EVENTMGR->Execute();

	// and delete the objects now that we have finished with them
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Execute()
//	DESC:		This is the function called to execute the engine....
//
//	-------------------------------------------------------------------------
void Program::Execute()
	{
	Init();
	MainLoop();
	Exit();
	}