//	-------------------------------------------------------------------------
//
//	PROGRAM.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file defines the Program class which enables rapid
//					program development by standardising the basic program
//					to include all libraries, etc. in one base object.
//
//					=== THIS FILE IS MEANT TO BE USED AS A TEMPLATE FOR ===
//                      ALL EVENT-DRIVEN GUI PROJECTS!!!
//
//	-------------------------------------------------------------------------

#ifndef __program_hpp
#define __program_hpp

// include relevent c/c++ libraries
#include "stdlib.h"
#include "dos.h"
#include "conio.h"
#include "stdio.h"

// the following line includes all libraries in appropriate order!!

#include "eventmgr.hpp"
#include "keyboard.hpp"
#include "timer.hpp"
#include "mouse.hpp"
#include "graphics.hpp"

//	-------------------------------------------------------------------------
//
//	CLASS:		Program
//	DESC:		The base object designed to be used by ALL programs. This
//				object automatically "sets up" the event-driven GUI & any of
//				the miscellaneous libraries.
//
//  NOTE:		This object is created & executed from a simple _TEMPLATE_
//				file called MAIN.CPP.
//
//	-------------------------------------------------------------------------

#define PROGRAM_TITLE_LENGTH 	30

#define	prg_InstallGUI			0x00000001
#define	prg_InstallSystemFont	0x00000002


class Program
	{
	public:
		// the program TITLE
		char* TITLE;

		// the constructor which sets up the event-driven GUI system
		Program(char* title,char* init_string,int options);

		// the destructor which terminates the event-driven GUI system
		virtual ~Program();

		// this function initializes anything required by the program...
		// such as declaring globals, etc.
		virtual void Init();

		// This function is the main program loop
		virtual void MainLoop();

		// this cleans up bits created by Initialize()
		virtual void Exit();

		// this executes the program components in the order as follows:
		//  		Init();
		//			MainLoop();
		//			Exit();
		virtual void Execute();

	private:
		int		selected_options;
	};

// -----------------

extern	char*   	PROGRAM_PATH;
extern	int			SYSTEM_FONT;

#endif