#ifndef __rects_hpp
#define __rects_hpp

#include "vbuffer.hpp"
#include "points.hpp"

// --------------------------------------------------------------------------
//
//	RECT DEFINITIONS
//
// --------------------------------------------------------------------------


// the format of the generic "rectangle"

struct grRect
	{
	union 	{ int left;		int	l; };
	union	{ int top;		int	t; };
	union	{ int right;	int r; };
	union	{ int bottom;	int b; };
	};

// --------------------------------------------------------------------------
//
//	RECT MACROS (BACKWARDS COMPATIBILITY & EASE OF READING)
//
// --------------------------------------------------------------------------

#define	grDrawRect(X1,Y1,X2,Y2,COLOR)	\
			grDrawRectVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),(COLOR))

#define	grDrawFilledRect(X1,Y1,X2,Y2,COLOR)	\
			grDrawFilledRectVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),(COLOR))

#define	grDrawShadedRect(X1,Y1,X2,Y2,COLOR,SPACING)	\
			grDrawShadedRectVirtual(grSCREEN,(X1),(Y1),(X2),(Y2),(COLOR),(SPACING))

#define grPutRect(X1,Y1,X2,Y2,COLOR) 	\
			grDrawRect((X1),(Y1),(X2),(Y2),(COLOR)); \
			grBlitRegion((X1),(Y1),(X2),(Y2))

#define grPutFilledRect(X1,Y1,X2,Y2,COLOR) 	\
			grDrawFilledRect((X1),(Y1),(X2),(Y2),(COLOR)); \
			grBlitRegion((X1),(Y1),(X2),(Y2))

#define grPutShadedRect(X1,Y1,X2,Y2,COLOR,SPACING) 	\
			grDrawShadedRect((X1),(Y1),(X2),(Y2),(COLOR),(SPACING)); \
			grBlitRegion((X1),(Y1),(X2),(Y2))

// --------------------------------------------------------------------------
//
// RECT	IMPLEMENTATION FUNCTIONS
//
// --------------------------------------------------------------------------


// ---	draw a rectangle in a virtual buffer from x1,y1 to x2,y2
//		in specified color

void 	grDrawRectVirtual(grVB* vb,
							int x1, int y1, int x2, int y2, int	color );


// ---	draw a filled rectangle in a virtual buffer from x1,y1 to x2,y2
//		in specified color

void 	grDrawFilledRectVirtual(grVB* vb,
									int x1,int y1,int x2,int y2, int color);


// ---	draw a gray-dot shaded rectangle in a virtual buffer
//		from x1,y1 to x2,y2 with variable spacing

void 	grDrawShadedRectVirtual(grVB* vb,
								int x1,int y1,int x2,int y2,
								int color, int spacing );

// ---	simple 'point inside rect' functions
//
//		return: true  - point is located inside specified rect
//				false - point is located outside specified rect

bool	grPtInRect(grPoint p,	grRect r);
bool	grPtInRect(int x,int y,	grRect r);
bool	grPtInRect(grPoint p,	grPoint r1,grPoint r2);
bool	grPtInRect(int x,int y, grPoint r1,grPoint r2);
bool	grPtInRect(grPoint p,	int x1,int y1,int x2,int y2);
bool	grPtInRect(int x,int y, int x1,int y1,int x2,int y2);

#endif