
#include "resmgr.hpp"
#include "sys.hpp"

// -------------------------------------------------------------------------

ResourceMgr*	RESOURCE_MGR;
int				CachedFont[MAX_CACHED_FONTS][font_NUM_ASCII];
int				NumCachedFonts = 0;

void	InitResourceManager()
	{
	RESOURCE_MGR = new ResourceMgr();
	}

void	ExitResourceManager()
	{
	delete RESOURCE_MGR;
	}

// -------------------------------------------------------------------------

ResourceMgr::ResourceMgr()
	{
	CacheUsed  		= 0;
	CacheSize  		= max((SYS_MemSize/5),1024);
	CacheIdsUsed  	= 0;

	CacheMem = (char*) SYS_Malloc(CacheSize);
	if (!CacheMem) printf("------- ERROR: COULD NOT MALLOC CACHEMEM -------\n");

	CacheItem = (CacheItemStruct*) SYS_Malloc(sizeof(CacheItemStruct) * MAX_CACHE_ENTRIES);
	if (!CacheItem) printf("------- ERROR: COULD NOT MALLOC CACHEITEM -------\n");

	LastBlockAddr = 0;
	BiggestBlockAddr = (long) CacheMem;
	BiggestBlockSize = CacheSize - sizeof(MemBlockHeader);

	for (int id=0;id<MAX_CACHE_ENTRIES;id++) CacheItem[id].type = C_AVAILABLE;
	}

// -------------------------------------------------------------------------

ResourceMgr::~ResourceMgr()
	{
	SYS_Free(CacheItem);
	SYS_Free(CacheMem);
	}

// -------------------------------------------------------------------------

void	ResourceMgr::FindBiggestFreeBlock()
	{
	if (CacheUsed==0) return;

//	DBG(470,"IN FIND BIGGEST FREE BLOCK, CacheUsed == %d",CacheUsed);
	MemBlockHeader* block;

	long		block_addr  = (long) &CacheMem[0];
	long 		cache_pos 	= 0;
	long		jump;

	BiggestBlockSize = 0;

	// scan through all blocks, checking each size for the biggest size
	do
		{
		block = (MemBlockHeader*) block_addr;

		if (block->size>BiggestBlockSize)
			{
			BiggestBlockSize = block->size;
			BiggestBlockAddr = block_addr;
			}

		jump = sizeof(MemBlockHeader) + block->size;

		block_addr += jump;
		cache_pos += jump;

//		DBG(450,"");
//		DBG(450,"sizeof(MemBlockHeader) == %d, block->size == %d",sizeof(MemBlockHeader),block->size);
//		DBG(460,"");
//		DBG(460,"CACHE_POS == %d, BIGGEST SIZE, ADDR == %d    %p,   JUMP == %d",
//				cache_pos,BiggestBlockSize,BiggestBlockAddr,jump);
		}
	// until we've run out of blocks
	while ((cache_pos<CacheUsed) && (jump>0));

//	DBG(470,"OUT FIND BIGGEST FREE BLOCK");
	}

// -------------------------------------------------------------------------

void	ResourceMgr::SplitBlock(MemBlockHeader* block,int new_size)
	{
//	DBG(470,"IN SPLITBLOCK");
	// figure block sizes & addresses
	long block_addr     = (long) block;
	long block_size     = block->size;

	long new_block_addr = block_addr + sizeof(MemBlockHeader) + block_size;
	long new_block_size = block_size - new_size - sizeof(MemBlockHeader);

	// adjust first block
	block->size = new_size;

	// and setup second block
	MemBlockHeader* new_block = (MemBlockHeader*) (new_block_addr);
	new_block->prev = block_addr;
	new_block->size = new_block_size;

	// if we've changed the biggest block, modify the biggest block info
	if (BiggestBlockAddr == block_addr) FindBiggestFreeBlock();
//	DBG(470,"OUT SPLITBLOCK");
	}

// -------------------------------------------------------------------------


void	ResourceMgr::MergeBlocks(MemBlockHeader* new_block,MemBlockHeader* dead_block)
	{
	if (CacheUsed==0) return;

	// figure block sizes & addresses
	long	new_block_size = new_block->size + dead_block->size + sizeof(MemBlockHeader);
	long    new_block_addr = (long) new_block;
	long	dead_block_addr= (long) dead_block;

	long 	next_block_addr = (long) dead_block + sizeof(MemBlockHeader) + dead_block->size;
	MemBlockHeader* next_block = (MemBlockHeader*) next_block_addr;

	// and merge the two blocks, eradicating 'dead_block'
	new_block->size = new_block_size;
	next_block->prev = new_block_addr;

	// make sure we update the biggest block
	if (BiggestBlockAddr == new_block_addr) BiggestBlockSize = new_block_size;
	if (BiggestBlockAddr == dead_block_addr)
		{
		BiggestBlockAddr   = new_block_addr;
		BiggestBlockSize   = new_block_size;
		}
	}


// -------------------------------------------------------------------------

void*	ResourceMgr::AllocateBlock(int nbytes)
	{
	MemBlockHeader* new_blk;

	// if there's room at the end of the heap, allocate a block there
	if (CacheUsed + nbytes + sizeof(MemBlockHeader) < CacheSize)
		{
		new_blk = (MemBlockHeader*) &CacheMem[CacheUsed];
//		DBG(450,"ALLOCATED NEW BLOCK AT %p, CacheUsed == %d",new_blk,CacheUsed);

		// set up the memory blocks header
		new_blk->size = nbytes;
		new_blk->prev = LastBlockAddr;
		new_blk->used = true;

		// save last block ptr
		LastBlockAddr = (long) new_blk;

		// and add the size of the blockheader + the size of the block
		CacheUsed += sizeof(MemBlockHeader) + nbytes;

		// make sure we update the biggest block
		long 	new_block_addr = (long) new_blk;
		if (nbytes > BiggestBlockSize)
			{
			BiggestBlockAddr   = new_block_addr;
			BiggestBlockSize   = nbytes;
			}
		}
	else
		{
		// there's not enough room at the end of the heap,
		// so blow away the least used Id till we have a big enough block
		int tmp_id;

		do
			{
			// figure least used id
			tmp_id = LeastUsedId();

			// blow away it's memory block
//			DBG(450,"DEALLOCATING LEAST USED ID %d BLOCK AT %p",tmp_id,CacheItem[tmp_id].data);

			DeAllocateBlock(CacheItem[tmp_id].data);
			CacheItem[tmp_id].data = NULL;
			}
		// keep doing it till the biggest block accomodates allocation request
		while (BiggestBlockSize<nbytes);

		// save pointer to the biggest block
		new_blk = (MemBlockHeader*) BiggestBlockAddr;

//		DBG(460,"BLEW AWAY EXISTING BLOCK, ALLOCATED NEW BLOCK AT %p, CacheUsed == %d",new_blk,CacheUsed);

		// if the block found is larger than required, split it into 2 blocks
		// and use the first block for object storage
		if (BiggestBlockSize>nbytes) SplitBlock(new_blk,nbytes);
		}

	long new_blk_addr = (long) new_blk;

	return (void*) (new_blk_addr + sizeof(MemBlockHeader));
	}

// -------------------------------------------------------------------------

void	ResourceMgr::DeAllocateBlock(void* block_data_ptr)
	{
	if (CacheUsed==0) return;

	// figure block headers & addresses
	long block_addr 		= (long) block_data_ptr - sizeof(MemBlockHeader);
	MemBlockHeader* block 	= (MemBlockHeader*) block_addr;

//	DBG(460,"DEALLOCATING BLOCK %p",block);

	long prev_block_addr 		= block->prev;
	long next_block_addr		= block_addr + sizeof(MemBlockHeader) + block->size;

	MemBlockHeader* prev_block 	= (MemBlockHeader*) prev_block_addr;
	MemBlockHeader* next_block 	= (MemBlockHeader*) next_block_addr;

	// free the block
	block->used = false;

	// if the previous block is also free, merge the blocks
	if (!prev_block->used) MergeBlocks(prev_block,block);

	// if the next block is also free, merge the blocks
	if (!next_block->used) MergeBlocks(block,next_block);
	}


// -------------------------------------------------------------------------

int		ResourceMgr::FirstAvailId()
	{
	if (CacheIdsUsed==0) { CacheIdsUsed++; return 0; }

	// start at block 0
	int tmp_id = 0;

	// and continue till we find an available block or we run out of blocks
	while ((CacheItem[tmp_id].type!=C_AVAILABLE) && (tmp_id<MAX_CACHE_ENTRIES))
		tmp_id++;

	// store number of ids used...
	if (tmp_id>=CacheIdsUsed) CacheIdsUsed++;

	return tmp_id;
	}

// -------------------------------------------------------------------------

int		ResourceMgr::LeastUsedId()
	{
	int tmp_id = 0;
	int least_used = 0;
	int	least_hits = MAX_HITS;

	while (tmp_id<CacheIdsUsed)
		{
		// note: this should only return an id that has data IN MEMORY
		if (CacheItem[tmp_id].data)
			if (CacheItem[tmp_id].hits < least_hits)
				{
				least_used = tmp_id;
				least_hits = CacheItem[tmp_id].hits;
				}

			tmp_id++;
		}

	return least_used;
	}

// -------------------------------------------------------------------------

int		ResourceMgr::RegisterImage(ResourceFile* file,int record)
	{
	// allocate a new item
	int id = FirstAvailId();
	if (id>=MAX_CACHE_ENTRIES) return err_NoAvailableId;

	CacheItem[id].type = C_IMAGE;
	CacheItem[id].size = file->DecompressedRecordSize(record);
//	DBG(200,"FOUND IMAGE SIZE %d",CacheItem[id].size);
	CacheItem[id].hits = 0;
	CacheItem[id].file = file;
	CacheItem[id].ofs  = record;
	CacheItem[id].data = NULL;    // don't load it yet... let Image() do it

	if ((int) CacheItem[id].size < 0) return (int) CacheItem[id].size;

	return id;
	}

// -------------------------------------------------------------------------

void	ResourceMgr::DeRegisterImage(int id)
	{
	// if the cache item exists, remove it from memory
	if (CacheItem[id].data) DeAllocateBlock(CacheItem[id].data);

	// and set flag so that item record is again available for use
	CacheItem[id].type = C_AVAILABLE;
	}

// -------------------------------------------------------------------------

//int dbg_offset = 0;

grImage*		ResourceMgr::ImagePtr(int id)
	{
//	dbg_offset+=10;
//	dbg_offset%=30;

	// if the cache item exists, return appropriate ptr
	if (CacheItem[id].data)
		{
//		DBG(390+dbg_offset,"CACHE ITEM[%d] ALREADY EXISTS",id);
		HIT(id);
		return (grImage*) CacheItem[id].data;
		}

	// otherwise reallocate memory for the item
	CacheItem[id].data = AllocateBlock(CacheItem[id].size);
//	DBG(390+dbg_offset,"CACHE ITEM[%d] DOESN'T EXIST, ALLOCATED BLOCK of size %d at %p",id,CacheItem[id].size,CacheItem[id].data);

	// re-read the relevant record
//	DBG(350,"CACHE ITEM[%d], READING ITEM OFF DISK",id);
	CacheItem[id].file->ReadRecord(CacheItem[id].ofs,(char*) CacheItem[id].data);

	CacheItem[id].hits = 1;

	// and return a pointer to it
//	DBG(360,"CACHE ITEM[%d], READ ITEM OFF DISK AS %p",id,CacheItem[id].data);
	return (grImage*) CacheItem[id].data;
	}

// -------------------------------------------------------------------------

int		ResourceMgr::RegisterFont(ResourceFile* file)
	{
//	grDrawFilledRect(50,0,59,99,grWHITE);
//	grBlitRegion(50,0,59,99);

	if (NumCachedFonts>=MAX_CACHED_FONTS) return err_TooManyCachedFonts;

	int x = 0;
	int y = 0;

	for (int i=0;i<font_NUM_ASCII;i++)
		{
		x += 10;
		if (x>grMaxx) { x = 0; y += 10; }
		if (y>grMaxy) y = 0;
//		grDrawFilledRect(x,y,x+9,y+9,i);
//		grBlitRegion(x,y,x+9,y+9);

		CachedFont[NumCachedFonts][i] = RegisterImage(file,i);
		}

	NumCachedFonts++;


	// pre-cache all fonts
	x = 0; y = 0;
	grImage* img;

	for (int j=0;j<font_NUM_ASCII;j++)
		{
		x += 10;
		if (x>grMaxx) { x = 0; y += 10; }
		if (y>grMaxy) y = 0;
		img = FontImagePtr(NumCachedFonts-1,j);
//		grPutImage(x,y,x+img->hdr.width-1,y+img->hdr.height-1,img);
//		grBlitRegion(x,y,x+img->hdr.width-1,y+img->hdr.height-1);
		}

	return NumCachedFonts-1;
	}


// -------------------------------------------------------------------------

void	ResourceMgr::DeRegisterFont(int id)
	{
	if (id>NumCachedFonts) return;
	//err_UnknownCachedFont;

	for (int i=0;i<font_NUM_ASCII;i++)
		grExitCachedImage(CachedFont[id][i]);

	if (NumCachedFonts>0) NumCachedFonts--;
	}


grImage*	ResourceMgr::FontImagePtr(int font_id,int ch)
	{
	if (font_id>=NumCachedFonts) return NULL;

	// figure actual cache item id
	int id = CachedFont[font_id][ch];

	// if the cache item exists, return appropriate ptr
	if (CacheItem[id].data)
		{
		HIT(id);
//		DBG(390+dbg_offset,"CACHE ITEM[%d] ALREADY EXISTS",id);
		return (grImage*) CacheItem[id].data;
		}

	// otherwise reallocate memory for the item
	CacheItem[id].data = AllocateBlock(CacheItem[id].size);
//	DBG(390+dbg_offset,"CACHE ITEM[%d] DOESN'T EXIST, ALLOCATED BLOCK of size %d at %p",id,CacheItem[id].size,CacheItem[id].data);

	// re-read the relevant record
//	DBG(350,"CACHE ITEM[%d], READING ITEM OFF DISK",id);
	CacheItem[id].file->ReadRecord(CacheItem[id].ofs,(char*) CacheItem[id].data);

	CacheItem[id].hits = 1;

	// and return a pointer to it
//	DBG(360,"CACHE ITEM[%d], READ ITEM OFF DISK AS %p",id,CacheItem[id].data);
	return (grImage*) CacheItem[id].data;
	}
/*
int		ResourceMgr::RegisterImage(PCXFile* file,int record);
int		ResourceMgr::RegisterFont(ResourceFile* file,int record);
int		ResourceMgr::RegisterPalette(PALFile* file,int record);
int		ResourceMgr::RegisterBStyle(ResourceFile* file,int record);
int		ResourceMgr::RegisterTile(ResourceFile* file,int record);

void	ResourceMgr::DeRegisterImage(int id);
void	ResourceMgr::DeRegisterFont(int id);
void	ResourceMgr::DeRegisterPalette(int id);
void	ResourceMgr::DeRegisterBStyle(int id);
void	ResourceMgr::DeRegisterTile(int id);

grFont*			ResourceMgr::Font(int id);
grPalette*		ResourceMgr::Palette(int id);
grBorderStyle*	ResourceMgr::BStyle(int id);
grImage*		ResourceMgr::Tile(int id);
*/

// -------------------------------------------------------------------------

void	ResourceMgr::Hit(int id)
	{
	// simply increment
	CacheItem[id].hits++;

	// and skip out if we've not hit max
	if (CacheItem[id].hits<MAX_HITS) return;

	// divide each cache item's hits by 2
//	DBG(0,"--- HIT MAX ---, CacheIdsUsed == %d",CacheIdsUsed);
	for (int item=0;item<CacheIdsUsed;item++) CacheItem[item].hits >>= 1;
	}