//	-------------------------------------------------------------------------
//
//	SPRITES
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew C. Evans
//
//	DESCRIPTION:	This file declares the sprite functions used
//					by the game development library.
//
//	-------------------------------------------------------------------------
#ifndef __sprites_hpp
#define __sprites_hpp

#include	"graphics.hpp"

//	the sprite header
struct grSpriteHeader
	{
	int		number_frames;		// total number of frames
	int		width,				// width of image in frames
			height;				// height of image in frames

	// this has to be initialised when sprite is loaded
	int		x,
			y,				// current x y position
			oldx,
			oldy;			// last x y position
	int		current_frame;	// current frame we are showing
	int		state;			// turning, dead, stopped, going etc
	};

// the sprite format
struct grSprite
	{
	grSpriteHeader	hdr;			// contains info for memory, number of frames and sizes

	grImage*	background;		// background under sprite

	grImage*	frame[];// list of images to make animation
	};

//-------------------------------------------------------------------------

// malloc all memory associated with a sprite and initialise data
grSprite*	grCreateSprite( int number_frames, int width, int height );

// create a sprite then store a number of grImages in that sprite
grSprite*	grMakeSprite(grImage** imagelist, int number_frames);

// free all memory associated with a sprite
void 	grDestroySprite( grSprite* spr );

// will put the background image over the sprite's position
void	grEraseSprite( grSprite* spr );

// saves sprite's old position then increases x & y by dx & dy
void	grMoveSprite( grSprite* spr, int dx, int dy );

// will save background behind the sprite's position
void	grSaveUnderSprite( grSprite* spr );

// overlay current image of sprite onto screen
void	grDrawSprite( grSprite* spr );

//#endif

#endif