//
//	stddefns.hpp
//
//	contains any definitions possibly required by all applications.
//
//	AUTHOR:			Darren J. Dwyer
//

#ifndef __stddefns_hpp
#define __stddefns_hpp

#include "stdlib.h"
#include "math.h"

// standard defns

#define	bit0	1
#define bit1	2
#define bit2	4
#define bit3	8
#define bit4	16
#define bit5	32
#define bit6	64
#define bit7	128
#define bit8	256
#define bit9	512
#define bit10	1024
#define bit11	2048
#define bit12	4096
#define bit13	8192
#define bit14	16384
#define bit15	32768

#ifndef true
#define	true	1
#endif
#ifndef false
#define false	0
#endif


#ifndef schar
#define	schar	signed char
#endif
#ifndef uchar
#define uchar 	unsigned char
#endif
#ifndef byte
#define byte	unsigned char
#endif
#ifndef BYTE
#define BYTE	unsigned char
#endif
#ifndef sshort
#define	sshort	signed short
#endif
#ifndef	ushort
#define ushort	unsigned short
#endif
#ifndef	WORD
#define WORD	unsigned short
#endif
#ifndef word
#define word 	unsigned short
#endif
#ifndef sint
#define sint	signed int
#endif
#ifndef uint
#define uint 	unsigned int
#endif
#ifndef slong
#define	slong	signed long
#endif
#ifndef ulong
#define ulong 	unsigned long
#endif
#ifndef dword
#define	dword	unsigned long
#endif
#ifndef DWORD
#define	DWORD	unsigned long
#endif
#ifndef bool
#define bool 	char
#endif


#ifdef __BORLANDC__
extern	void	delay(int sec);
#endif

// global clipping macros for ease-of-use by all functions

#ifndef CLIP2
#define CLIP2(X,Y,X1,Y1,X2,Y2)		if ( (X) < (X1) ) (X)=(X1); \
									if ( (X) > (X2) ) (X)=(X2); \
									if ( (Y) < (Y1) ) (Y)=(Y1); \
									if ( (Y) > (Y2) ) (Y)=(Y2); \
									;   // note: seems to not work otherwise
#endif

#ifndef CLIP1
#define CLIP1(NUM,MIN,MAX)			if ( (NUM) < (MIN) ) (NUM)=(MIN); \
									if ( (NUM) > (MAX) ) (NUM)=(MAX); \
									;	// note: seems to not work otherwise
#endif


//  seed the random number generator
void randomize();

//  return a random number in range of 0-N
#ifndef random
#define random(N)		(rand() % (N))
#endif

//	return the difference between A and B
#ifndef diff
#define diff(A,B)		abs((A)-(B))
#endif

#ifndef max
#define max(A,B)	    ( ((A)>(B)) ? (A) : (B))
#endif
#ifndef min
#define min(A,B)	    ( ((A)<(B)) ? (A) : (B))
#endif

// returns the current system ticks value
#ifndef GetTicks
#define GetTicks()		*((long*)0x46C)
#endif

// packs a Dayofweek,daTe,Month,Year into a 32-bit value
// ie. Sunday, 16th April 1968 == 0xYYYYMTTD == 19684160
#ifndef PACKDATE
#define PACKDATE( PACKED_DATE, DAY, DATE, MONTH, YEAR ) \
			(PACKED_DATE) =  ((long)(DAY)   & 0x0000000F) + \
							(((long)(DATE)  & 0x000000FF) << 4) + \
							(((long)(MONTH) & 0x0000000F) << 12) + \
							(((long)(YEAR)  & 0x0000FFFF) << 16)
#endif

// unpacks a Dayofweek,daTe,Month,Year from a 32-bit value
// ie. 0xYYYYMTTD == 19684160 == Sunday, 16th April 1968
#ifndef UNPACKDATE
#define UNPACKDATE( PACKED_DATE, DAY, DATE, MONTH, YEAR ) \
			(DAY)   =  ((PACKED_DATE)        & 0x0000000F); \
			(DATE)  = (((PACKED_DATE) >> 4)  & 0x000000FF); \
			(MONTH) = (((PACKED_DATE) >> 12) & 0x0000000F); \
			(YEAR)  = (((PACKED_DATE) >> 16) & 0x0000FFFF);
#endif

// define SYSTEM commands used in cmdevent mechanism here
// SYSTEM commands are in range 0x0000 to 0x0FFF
// user-defined commands can be in range 0x1000 to 0xF000

#define cmdNothing		100	// a null event
#define cmdExitProgram 	101	// to tell the current event loop to exit
#define cmdTerminate	102	// to terminate the program completely
#define cmdAnimate		103	// animate an accessory
#define cmdRefresh		104	//
#define cmdRedraw		105	// to tell specific handler to redraw

#define cmdOpen			106 // to open an onscreen handler
#define cmdClose		107	// to close an onscreen handler

//#define	cmdOK			108	// standard OK response
//#define cmdCANCEL		109	// standard CANCEL response

// the following are used to 'poll' our input devices, allowing transparent
// event generation only when necessary & not when processing...???

#define cmdPollMouse	200	// used when polling the mouse for input
#define	cmdPollKbd		201	// used when polling the keyboard for input
#define cmdPollJoy		202 // used when polling the joystick for input
#define cmdPollSer		203 // used when polling the serial port for input
#define cmdPollPar		204 // used when polling the parallel port for input
#define cmdPollNet		205	// used when polling a network

#define cmdOK			300	// standard positive response
#define cmdCANCEL		301 // standard negative response

#endif

