//	-------------------------------------------------------------------------
//
//	STRINGS.HPP
//
//	-------------------------------------------------------------------------
//
//	DESCRIPTION:	This file contains defines for a simple string class
//
//	-------------------------------------------------------------------------

#ifndef __strings_hpp
#define __strings_hpp

#include "stddefns.hpp"
#include "sys.hpp"

#include "stdio.h"
#include "conio.h"
#include "string.h"
#include "malloc.h"

// the default length of a String is 256 chars

#define STRING_DEFAULT_LENGTH	128

//	the String class

// for old function call names, please change your use to correct one
#define StringToInt		StringToInteger
#define Upper			ToUpper
#define Lower			ToLower

class String
	{
	protected:
		char* str;			// internal string representation
		long length;		// internal string's current length
		long maximumlength;	// internal string's maximum length

		long position;		// position within in str, 0 by default
		char token;			// divider for elements, '/' by default

	public:

		// === CONSTRUCTORS & DESTRUCTORS ===

		// create a blank string of STRING_DEFAULT_LENGTH chars
		String()
			{
			position = 0;
			token = '/';

			maximumlength = STRING_DEFAULT_LENGTH;

			str = (char*) SYS_Malloc(maximumlength+1);
			if (!str) { length = maximumlength = 0; return; }

			length = 0;
			str[length] = 0x00;
			}

		// create a blank string of a defined size
		String(long size)
			{
			position = 0;
			token = '/';

//			if (size<0) size = 0;
			maximumlength = size;

			str = (char*) SYS_Malloc(maximumlength+1);
			if (!str) { length = maximumlength = 0; return; }

			length = 0;
			str[length] = 0x00;
			}

		// create a string based upon a char*
		String(const char* st)
	{
	if (!st) { length = maximumlength = 0; return; }

	position = 0;
	token = '/';

	maximumlength = strlen(st);

	str = (char*) SYS_Malloc(maximumlength+1);
	if (!str) { length = maximumlength = 0; return; }

	length = maximumlength;
	strcpy(str, st);
	str[length] = 0x00;
	}


		// create a string based upon a char* of a specified size
		String(const char* st, long size )
	{
	if (!st) { length = maximumlength = 0; return; }

	position = 0;
	token = '/';

	maximumlength = size;

	str = (char*) SYS_Malloc(maximumlength+1);
	if (!str) { length = maximumlength = 0; return; }

	strncpy(str, st, maximumlength);
	//(strlen(st)<maximumlength) ? strlen(st):maximumlength);

	length = strlen(str);
	if (length>maximumlength) length = maximumlength;
	str[length] = 0x00;
	}


		// COPY CONSTRUCTOR - create this string based upon another string
		String(String* s)// : String(s->Contents()) {}
	{
	if (!s) { length = maximumlength = 0; return; }

	position = s->position;
	token = s->token;

	maximumlength = s->maximumlength;

	str = (char*) SYS_Malloc(maximumlength+1);
	if (!str) { length = maximumlength = 0; return; }

	*this = s;				// using overloaded assignment operator
	str[length] = 0x00;
	}


		// delete a string
		~String() { if (str) SYS_Free(str); }



		// === MEMBER FUNCTIONS ===

		// return what str contains
		char* Contents()					{ return str; }

		// returns maximum length and current length for this String
		long MaxLength()					{ return maximumlength; }
		long Length()						{ return length; }
		void SetLength(long len,bool fill_extra = false);// set the current length to len

		// set/reset this String to something
		void Set(char ch)					{ length=0; Append(ch); }
		void Set(const char* st)			{ length=0; Append(st); }
		void Set(long number)				{ length=0; Append(number); }
		void Set(String* s)					{ length=0; Append(s->Contents()); }
		void Set()							{ length=0; str[0]=0x00; }// make str empty

		// insert something into this String at a position
		void Insert(long pos,char ch);
		void Insert(long pos,const char* st);
		void Insert(long pos,long number);
		void Insert(long pos,String* s)		{ Insert(pos,s->Contents()); }
		void Insert(long pos)				{ Insert(pos,' '); }// insert space

		// delete char or character from pos in String
		void Delete(long pos);
		void Delete(long pos,long numchars);

		// append something to the end of this String
		void Append(char ch)				{ Insert(length,ch); }
		void Append(const char* st)			{ Insert(length,st); }
		void Append(long number)			{ Insert(length,number); }
		void Append(String* s)				{ Insert(length,s->Contents()); }
		void Append()						{ Insert(length,' '); }// append space

		// sets str to uppercase/lowercase
		void ToUpper()						{ strupr(str); }
		void ToLower()						{ strlwr(str); }

		// returns the integer value contained in the String ignoring all but digits and sign
		long StringToInteger();
		long StringToInteger(long min,long max);// with min/max adjustment

		// scanning
		void SetPosition(long pos)			{ position = pos; }
		long GetPosition()					{ return position; }
		long ScanRight(long pos,char tok);	// return position of first tok after pos
		long ScanRight(char tok)			{ return position = ScanRight(position,tok); }
		long ScanLeft(long pos,char tok);	// return position of last tok before pos
		long ScanLeft(char tok)				{ return position = ScanLeft(position,tok); }

		bool Contains(char ch);
		bool Contains(char* st);

		// returns a created array in uppercase/lowercase from String
		String* GetUpper();
		String* GetLower();

		// returns a created substring from String
		String* SubStr(long start,long end);

		// elements
		void SetToken(char tok)   			{ token = tok; }
		char GetToken()   					{ return token; }
		long NumElems(char tok);			// return (number of tok in str)+1
		long NumElems()						{ return NumElems(token); }
		long NumWords()						{ return NumElems(' '); }
		String* Elem(long n,char tok);		// return n'th element
		String* Elem(long n)				{ return Elem(n,token); }
		String* Word(long n)				{ return Elem(n,' '); }


		// === COMPARISON OPERATORS ===

		// equality to
		bool operator ==	(char ch) { return (str[position] == ch); }
		bool operator ==	(const char* st) { return (memcmp(str,st,length)==0); }
		bool operator ==	(long number) { return (StringToInteger() == number); }
		bool operator ==	(String* s) { return (memcmp(str,s->Contents(),length)==0); }

		// inequality to
		bool operator !=	(char ch) { return (str[position]) != ch; }
		bool operator !=	(const char* st) { return (memcmp(str,st,length)!=0); }
		bool operator !=	(long number) { return (StringToInteger() != number); }
		bool operator !=	(String* s) { return (memcmp(str,s->Contents(),length)!=0); }

		// less than
		bool operator <		(char ch) { return (str[position]) < ch; }
		bool operator <		(const char* st) { return (memcmp(str,st,length)<0); }
		bool operator <		(long number) { return (StringToInteger() < number); }
		bool operator <		(String* s) { return (memcmp(str,s->Contents(),length)<0); }

		// greater than
		bool operator >		(char ch) { return (str[position]) > ch; }
		bool operator >		(const char* st) { return (memcmp(str,st,length)>0); }
		bool operator >		(long number) { return (StringToInteger() > number); }
		bool operator >		(String* s) { return (memcmp(str,s->Contents(),length)>0); }
	};

#endif