//
// SYS.CPP
//
//
//	DJD.

#include "mem.h"
#include "malloc.h"
#include "conio.h"
#include "stdio.h"
#include "stdlib.h"

#include "stddefns.hpp"
#include "mouse.hpp"
#include "keyboard.hpp"

bool	SYS_DEBUG = false;

long	SYS_MemSize	 = 0;
long	SYS_MemAvail = 0;
long	SYS_MemUsed  = 0;

void**	SYS_MemPtr;
int		SYS_MemBlocks = 0;

void	SYS_CheckMemory()
	{
	SYS_MemAvail 	= 0;
	SYS_MemUsed		= 0;

	// allocate an array of void* 's
	SYS_MemPtr 		= (void**) malloc(sizeof(void*) * 10000);
	if (!SYS_MemPtr) return;

	// and include this in the total available memory
	SYS_MemUsed		+= sizeof(void*) * 10000;

	int		chunk	= 1024 << 10;	// 1MB CHUNKS INITIALLY

	// allocate all available memory in best-size chunks
	// starting at 1MB & going down by halves
	do
		{
		bool done = false;

		do
			{
			SYS_MemPtr[SYS_MemBlocks] = malloc(chunk);
			if (SYS_MemPtr[SYS_MemBlocks])
				{
				SYS_MemUsed += chunk;
				SYS_MemBlocks++;
				}
			else
				done = true;
			}
		while (!done);

		// choose half-size chunk
		chunk >>= 1;
		}
	while (chunk>=1024);

	// save memory status
	SYS_MemSize		= SYS_MemUsed;
	SYS_MemAvail 	= SYS_MemUsed;
	SYS_MemUsed		= 0;

	// and deallocate the memory used
	while (SYS_MemBlocks>0)
		{
		SYS_MemBlocks--;
		free(SYS_MemPtr[SYS_MemBlocks]);
		}
	free(SYS_MemPtr);
	}

void*	SYS_Malloc(long nbytes)
	{
	void*	tmp = malloc(nbytes);
	if (tmp)
		{
		SYS_MemUsed += nbytes;
		SYS_MemAvail -= nbytes;

//		if (SYS_DEBUG) DBG(0,"SYS_MemSize: %d bytes   SYS_MemUsed: %d bytes   SYS_MemAvail: %d bytes",
//							SYS_MemSize,SYS_MemUsed,SYS_MemAvail);
		}
	else
		{
//		if (SYS_DEBUG) DBG(200,"SYS_Malloc()   Failed on Allocating %d bytes",nbytes);
//		if (SYS_DEBUG) SYS_Status();
		}
	return tmp;
	}

void*	SYS_Free(void* block)
	{
	if (block)
		{
#ifdef __WATCOMC__
		long siz = _msize(block); SYS_MemUsed -= siz; SYS_MemAvail += siz;
//		if (SYS_DEBUG)
//			{
//			DBG(0,"SYS_MemSize: %d bytes   SYS_MemUsed: %d bytes   SYS_MemAvail: %d bytes",
//							SYS_MemSize,SYS_MemUsed,SYS_MemAvail);
//			grDrawFilledRect(0,0,grMaxx,9,grBLACK);
//			grBlitRegion(0,0,grMaxx,9);
//			}
#endif
		free(block);
		}
	return NULL;
	}

void*	SYS_Memset(void* s,int c,long n)
	{
	if (!s) return NULL;

	DWORD*	dest = (DWORD*) s;
	int 	dwords_to_copy  = n >> 2;		// dwords to copy
	int 	word_to_copy	= ((n&2)!=0);	// if word-aligned, copy 1 word
	int 	byte_to_copy 	= ((n&1)!=0); 	// if byte-aligned, copy 1 byte

	BYTE	bfill = (BYTE) c;
	WORD 	wfill = bfill + (bfill<<8);
	DWORD   dfill = wfill + (wfill<<16);

	// splat all DWORDS across
	for (int x=0;x<dwords_to_copy;x++)
		{
//		if (x&1000) mouse_UpdateCursor();
		*dest = dfill;
		dest++;
		}

	// if we need to copy a WORD
	if (word_to_copy)
		{
		WORD* wdest = (WORD*) dest;
		*wdest = wfill;
		wdest++;

		// we may need to copy a BYTE as well
		if (byte_to_copy)
			{
			BYTE* bdest = (BYTE*) wdest;
			*bdest = bfill;
			}
		}
	else
		// we don't need to copy a word
		// but we may still need to copy a BYTE
		{
		if (byte_to_copy)
			{
			BYTE* bdest1 = (BYTE*) dest;
			*bdest1 = bfill;
			}
		}

	return (void*) s;
	}

void*	SYS_Memcpy(void* d,void* s,long n)
	{
	if (!s) return NULL;

	DWORD*	dest = (DWORD*) d;
	DWORD*	srce = (DWORD*) s;
	int 	dwords_to_copy  = n >> 2;		// dwords to copy
	int 	word_to_copy	= ((n&2)!=0);	// if word-aligned, copy 1 word
	int 	byte_to_copy 	= ((n&1)!=0); 	// if byte-aligned, copy 1 byte


	// splat all DWORDS across
	for (int x=0;x<dwords_to_copy;x++)
		{
//		if (x&100) mouse_UpdateCursor();
		*dest = *srce;
		srce++;
		dest++;
		}

	// if we need to copy a WORD
	if (word_to_copy)
		{
		WORD* wdest = (WORD*) dest;
		WORD* wsrce = (WORD*) srce;
		*wdest = *wsrce;
		wdest++;
		wsrce++;

		// we may need to copy a BYTE as well
		if (byte_to_copy)
			{
			BYTE* bdest = (BYTE*) wdest;
			BYTE* bsrce = (BYTE*) wsrce;
			*bdest = *bsrce;
			}
		}
	else
		// we don't need to copy a word
		// but we may still need to copy a BYTE
		{
		if (byte_to_copy)
			{
			BYTE* bdest1 = (BYTE*) dest;
			BYTE* bsrce1 = (BYTE*) srce;
			*bdest1 = *bsrce1;
			}
		}

	return (void*) d;
	}


void	SYS_Status()
	{
	DBG(220,"SYS_MemSize  == %d bytes",SYS_MemSize);
	DBG(230,"SYS_MemUsed  == %d bytes",SYS_MemUsed);
	DBG(240,"SYS_MemAvail == %d bytes",SYS_MemAvail);

	long tmp_SYS_MemSize = SYS_MemSize;
	long tmp_SYS_MemUsed = SYS_MemUsed;
	long tmp_SYS_MemAvail= SYS_MemAvail;

	SYS_CheckMemory();

	DBG(260,"Actual MemAvail == %d bytes",SYS_MemSize);

	SYS_MemSize  = tmp_SYS_MemSize;
	SYS_MemAvail = tmp_SYS_MemAvail;
	SYS_MemUsed  = tmp_SYS_MemUsed;

	while (!keyboard_IsHit()) ;
	keyboard_Flush();
	}