//	-------------------------------------------------------------------------
//
//	MOUSE.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains code for the mouse ISR which is
//					capable of auto-detection & direct hardware response.
//
//	-------------------------------------------------------------------------

#include "stddefns.hpp"
#include "mouse.hpp"
#include "keyboard.hpp"
#include "event.hpp"
#include "eventmgr.hpp"
#include "graphics.hpp"
#include "sys.hpp"

#include "dos.h"
#include "mem.h"

#ifdef __BORLANDC__
#define		regs_w 		regs.x
#define		int386		int86
#else
#include "i86.h"
#define		regs_w		regs.w
#endif

#include "conio.h"
#include "stdio.h"
#include "stdlib.h"


// a few sample cursors

MouseCursor		ARROW_CURSOR =
				{ 24,24,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 15,15,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,

				0,0,8,7, 7,7,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 15,15,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 7,7,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 7,7,7,7, 15,15,0,0, 0,0,0,0, 0,0,0,0,

				0,0,8,7, 7,7,7,15, 15,15,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,15, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 15,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 7,15,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,

				0,0,8,7, 7,7,7,7, 7,7,15,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 8,7,7,7, 7,7,7,15, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 8,8,7,7, 7,7,7,7, 15,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,8, 0,0,8,7, 7,7,7,7, 7,15,0,0, 0,0,0,0, 0,0,0,0,

				0,0,8,0, 0,0,0,8, 7,7,7,7, 7,7,15,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 8,7,7,7, 7,7,8,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,8,7,7, 7,8,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,8,7, 8,0,0,0, 0,0,0,0, 0,0,0,0,

				0,0,0,0, 0,0,0,0, 0,0,0,8, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0   };

MouseCursor		DummyCursor[] =
				{ 16, 16,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,9, 0,9,0,9, 0,9,0,9, 0,9,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,0, 0,0,0,0, 0,0,0,0, 0,9,0,0,

				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,0, 0,0,0,0, 0,0,0,0, 0,9,0,0,
				0,0,0,0, 0,0,4,4, 4,4,0,0, 0,0,0,0,
				0,9,0,0, 0,0,4,4, 4,4,0,0, 0,9,0,0,

				0,0,0,0, 0,0,4,4, 4,4,0,0, 0,0,0,0,
				0,9,0,0, 0,0,4,4, 4,4,0,0, 0,9,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,0, 0,0,0,0, 0,0,0,0, 0,9,0,0,

				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,9, 0,9,0,9, 0,9,0,9, 0,9,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0 };

// global variables used by the mouse routines

MouseCursor*	DEFAULT_CURSOR = &ARROW_CURSOR;

// the delay for the auto repeat

#define mouse_AUTO_DELAY			3

// mouse library internal variables

static 	int		mouseX;				// current mouse coords
static 	int		mouseY;
static 	int		lastmouseX;			// last drawn mouse coords
static 	int		lastmouseY;

static 	int		leftBut;			// current button status, mouse_PRESSED or
static 	int		rightBut;			// 						  mouse_RELEASED
static 	int		middleBut;

static 	long	LMBTicks  = 0;		// count of ticks between button presses
static 	long	RMBTicks  = 0;
static 	long	MMBTicks  = 0;

static 	bool   	LMBAuto = false;	// if true, mouse is in 'auto generate' mode
static 	bool   	RMBAuto	= false;
static 	bool   	MMBAuto	= false;

static 	int		hotSpotX = 0;		// hotspot in mouse cursor
static 	int		hotSpotY = 0;		// hotspot in mouse cursor

static 	bool 	MoveEventsOn = false;// if true, mouse movement events are generated
static 	bool 	HasMoved = false;	// if true, mouse has moved since last cursor update

static 	int		TimesHidden = 1;	// count of how many times we've hidden the cursor
							// note: =1 due to first time cursor display

static 	MouseEvent	EventInfo;		// mouse event information, once

MouseCursor*	mouse_Cursor = &ARROW_CURSOR;	// the current mouse cursor pointer

static 	grImage*		mouse_UnderCursor = NULL;		// image under current mouse cursor
static 	grImage*		mouse_UnderCursor_1 = NULL;		// image under current mouse cursor

static	bool	MOUSE_INSTALLED = false;	// if true, mouse library is installed

// Mouse Library

#define MOUSE_INT       0x33

extern char far*        GetStack( void );
extern void             SetStack( char far* );


  #pragma aux 	GetStack = 				\
						"mov dx,ss" 	\
						"mov eax,esp"	\
						value [ dx eax ];

  #pragma aux   SetStack = 				\
						"mov ss,dx" 	\
						"mov esp,eax" 	\
						parm [ dx eax ];

#define STACKSIZE       1024

//===================================================
static void _loadds far mouse_handler(int m_ax,int m_bx,int m_cx,int m_dx)

  #pragma aux 	mouse_handler 			\
						parm [EAX EBX ECX EDX];

	{
	static char         thestack[ STACKSIZE ];

	static long 		thestack_addr = (long) &thestack;
	static long			newstack_addr = thestack_addr + STACKSIZE - 1;

	static char far*    oldstack;
	static char far*	newstack = (char far*) newstack_addr;

	int tmp_leftBut = leftBut;
	int tmp_rightBut = rightBut;
	int tmp_middleBut = middleBut;
	int tmp_x = mouseX;
	int tmp_y = mouseY;

	if (TimesHidden==0)
		{
		oldstack = GetStack();
		SetStack( newstack );

		leftBut = (m_ax != 0);
		rightBut = m_ax;
		middleBut = m_ax;

		mouseX = m_dx;
		mouseY = m_bx;

		mouse_UpdateCursor();

		SetStack( oldstack );
		}
	else
		{
		mouseX = m_cx;
		mouseY = m_dx;
		}

	if (eventmgr_INSTALLED)
		{
		// save state ready for event generation
		EventInfo.modifiers = keyboard_GetState();
		EventInfo.where.x = mouseX;
		EventInfo.where.y = mouseY;

		// if the left button has changed state
		if (leftBut != tmp_leftBut)
			{
			// figure appropriate event to generate
			if (leftBut == mouse_PRESSED)
				{
				EventInfo.event = mb_leftdn;
				LMBTicks = GetTicks();
				}
			else
				{
				EventInfo.event = mb_leftup;
				LMBAuto  = false;
				}

			// generate the event
			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}

		// check the right button
		if (rightBut != tmp_rightBut)
			{
			if (rightBut == mouse_PRESSED)
				{
				EventInfo.event = mb_rightdn;
				RMBTicks = GetTicks();
				}
			else
				{
				EventInfo.event = mb_rightup;
				RMBAuto  = false;
				}

			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}

		// and the middle button
		if (middleBut != tmp_middleBut)
			{
			if (middleBut == mouse_PRESSED)
				{
				EventInfo.event = mb_middledn;
				MMBTicks = GetTicks();
				}
			else
				{
				EventInfo.event = mb_middleup;
				MMBAuto  = false;
				}

			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}

		// check for mouse movement
		if ((mouseX != tmp_x) || (mouseY != tmp_y))
			{
			// signal that mouse has moved
			HasMoved = true;

			// if we're generating move events
			if (MoveEventsOn)
				{
				EventInfo.event = mb_move;

				EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
				}
			}
		}
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_IsInstalled()
//	DESC:		returns true if mouse driver is currently installed
//
//	-------------------------------------------------------------------------
bool mouse_IsInstalled()
	{
	return (MOUSE_INSTALLED);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_Reset()
//	DESC:		resets the mouse to its original state
//
//	-------------------------------------------------------------------------
void mouse_Reset()
	{
	if (!MOUSE_INSTALLED) return;

	mouseX = grMaxx / 2;
	mouseY = grMaxy / 2;
	leftBut = mouse_RELEASED;
	rightBut = mouse_RELEASED;
	middleBut = mouse_RELEASED;

	mouse_SetCursor(0,0,DEFAULT_CURSOR);
	mouse_SetRange(0,0,grMaxx,grMaxy);
	mouse_SetPos(mouseX,mouseY);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_EvableMoveEvents()
//	DESC:		upon calling this function, mouse_GetEvent() will generate
//				mb_move events. (Note: this may cause a large number of
//				events to be placed in the event queue!!!)
//
//	-------------------------------------------------------------------------
void mouse_EnableMoveEvents()
	{
	MoveEventsOn = true;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DisableMoveEvents()
//	DESC:		stops mouse_GetEvent from generating mouse move events into
//				the EVENT QUEUE.
//
//	-------------------------------------------------------------------------
void mouse_DisableMoveEvents()
	{
	MoveEventsOn = false;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetPos()
//	DESC:		returns the current position of the mouse as a point
//
//	-------------------------------------------------------------------------
grPoint mouse_GetPos()
	{
	grPoint p;
	p.x = mouseX;
	p.y = mouseY;
	return p;
/*
	union REGPACK       regs;

	memset( &regs, 0, sizeof( union REGPACK ) );
	regs.w.ax = 3;
	intr( MOUSE_INT, &regs );
	pos->xcoord = regs.w.cx;
	pos->ycoord = regs.w.dx;
	*left = regs.w.bx & 1;
*/
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_SetPos()
//	DESC:		sets the current position of the mouse given a point
//
//	-------------------------------------------------------------------------
void mouse_SetPos(grPoint& pt)
	{
	}

void    mouse_SetPos(int x1,int y1)
	{
	union REGPACK       regs;

	SYS_Memset( &regs, 0, sizeof( union REGPACK ) );
	regs_w.ax = 4;
	regs_w.cx = x1;
	regs_w.dx = y1;
	intr( MOUSE_INT, &regs );
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetStatus()
//	DESC:		gets the position and the button's status
//
//	-------------------------------------------------------------------------
void	mouse_GetStatus(mouse_Status* ms_status)
	{
	ms_status->x = mouseX;
	ms_status->y = mouseY;
	ms_status->leftBut = leftBut;
	ms_status->rightBut = rightBut;
	ms_status->middleBut = middleBut;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_SetRange()
//	DESC:		sets the range of the mouse cursor
//
//	-------------------------------------------------------------------------
void mouse_SetRange(int x1,int y1,int x2,int y2)
	{
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_IsHit()
//	DESC:		returns true if mouse is waiting to be processed
//
//	-------------------------------------------------------------------------
bool mouse_IsHit()
	{
	if (!MOUSE_INSTALLED) return false;

	bool ishit = ((leftBut!=mouse_RELEASED)||
				  (rightBut!=mouse_RELEASED)||
				  (middleBut!=mouse_RELEASED));
	return ishit;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_Flush()
//	DESC:		flushes the mouse input buffer
//
//	-------------------------------------------------------------------------
void mouse_Flush()
	{
	if (!MOUSE_INSTALLED) return;

	leftBut = mouse_RELEASED;
	rightBut = mouse_RELEASED;
	middleBut = mouse_RELEASED;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_InRect()
//	DESC:		Checks to see whether the mouse cursor is in the specified
//				coordinates. Returns true if it is.
//
//	-------------------------------------------------------------------------
bool mouse_InRect(int x1,int y1,int x2,int y2)
	{
	if (!MOUSE_INSTALLED) return false;

	int xx 			= mouseX;
	int yy 			= mouseY;
	int xx1 		= xx + mouse_Cursor->hdr.width-1;
	int yy1 		= yy + mouse_Cursor->hdr.height-1;

	// check coords
	bool result 	= ((xx>=x1)&&(xx<=x2)&&(yy>=y1)&&(yy<=y2)||
					   (xx1>=x1)&&(xx1<=x2)&&(yy1>=y1)&&(yy1<=y2));

	return result;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DrawCursor()
//	DESC:		Draws the mouse on currently active page
//
//	-------------------------------------------------------------------------
void mouse_DrawCursor( int options )
	{
	if (!MOUSE_INSTALLED) return;

	int x,y,tmp;

	int x1 = mouseX;
	int y1 = mouseY;
	int x2 = mouseX+mouse_Cursor->hdr.width-1;
	int y2 = mouseY+mouse_Cursor->hdr.height-1;

	CLIP2(x1,y1,0,0,grMaxx,grMaxy);
	CLIP2(x2,y2,0,0,grMaxx,grMaxy);

	BYTE*	screen_pos;
	BYTE* 	cursor_pos		= &mouse_Cursor->data[0];
	BYTE*	underneath_pos	= &mouse_UnderCursor->data[0];

	// figure out the screen address of the top-left corner of the cursor
	long*	row_addr 		= &VESA_RowPtr[y1];
	int*  	bank_ofs		= &VESA_RowBank[y1];
	int		bank_num;


	// scan through the cursor vertically
	for (y=y1;y<=y2;y++)
		{
		// figure out the required bank & change iff required
		bank_num = *bank_ofs++ + VESA_Write_Bank_Offset;
		if (bank_num!=VESA_Current_Write_Bank) VESA_SwitchWriteBank( bank_num );

		// create a pointer to the screen at the position to start blitting
		screen_pos = (BYTE*) (*row_addr + x1);

		// blit the current row of the cursor
		for (x=x1;x<=x2;x++)
			{
			tmp = *cursor_pos;		// get the next pixel in the row


			// if the appropriate pixel in the mouse cursor needs to be drawn
			if (tmp)
				{
				// save whats underneath the cursor iff required
				if (options&SAVE_UNDERNEATH_CURSOR)
					*underneath_pos = *screen_pos;

				// and draw the pixel
				*screen_pos = tmp;
				}

			cursor_pos++;			// skip to the next cursor pixel
			screen_pos++;
			underneath_pos++;
			}

		// for the next row, simply add the logical width to the address
		row_addr++;
		}

	// save the coords at which we last drew the mouse
	lastmouseX = mouseX;
	lastmouseY = mouseY;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_UnDrawCursor()
//	DESC:		Erases the mouse on the currently active page
//
//	-------------------------------------------------------------------------
void mouse_UndrawCursor()
	{
	if (!MOUSE_INSTALLED) return;

	int x,y,tmp;

	int x1 = lastmouseX;
	int y1 = lastmouseY;
	int x2 = lastmouseX+mouse_Cursor->hdr.width-1;
	int y2 = lastmouseY+mouse_Cursor->hdr.height-1;

	CLIP2(x1,y1,0,0,grMaxx,grMaxy);
	CLIP2(x2,y2,0,0,grMaxx,grMaxy);

	// figure out the screen address of the top-left corner of the cursor
	BYTE*	screen_pos;
	BYTE* 	cursor_pos		= &mouse_Cursor->data[0];
	BYTE*	underneath_pos	= &mouse_UnderCursor->data[0];

	int		underneath_width= mouse_UnderCursor->hdr.width;
	long*	row_addr		= &VESA_RowPtr[y1];
	int*  	bank_ofs		= &VESA_RowBank[y1];
	int		bank_num;

	// scan through the cursor vertically
	for (y=y1;y<=y2;y++)
		{
		// figure out the required bank & change iff required
		bank_num = *bank_ofs++ + VESA_Write_Bank_Offset;
		if (bank_num!=VESA_Current_Write_Bank) VESA_SwitchWriteBank( bank_num );

		// create a pointer to the screen at the position to start blitting
		screen_pos = (BYTE*) (*row_addr + x1);

		// blit the current row of the cursor
		for (x=x1;x<=x2;x++)
			{
			tmp = *cursor_pos;		// get the next pixel in the row

			if (tmp)				// if its set
				*screen_pos = *underneath_pos;
									// put whats underneath the cursor
									// onto the screen (_SLOW_ -> AT Bus)

			cursor_pos++;			// skip to the next cursor pixel
			screen_pos++;
			underneath_pos++;
			}

		// for the next row, simply add the logical width to the address
		row_addr++;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DrawCursorRow()
//	DESC:		Draws a row of the mouse cursor
//
//	-------------------------------------------------------------------------
void mouse_DrawCursorRow( int row, int options )
	{
	if (!MOUSE_INSTALLED) return;

	int x,y,tmp;

	int x1 = mouseX;
	int x2 = mouseX+mouse_Cursor->hdr.width-1;

	CLIP2(x1,x2,0,0,grMaxx,grMaxx);
	CLIP1(row,0,grMaxy);

	int 	cursor_width	= mouse_Cursor->hdr.width;
	int		rel_row			= row - mouseY;
	int 	cursor_ofs		= cursor_width * rel_row;

	BYTE*	screen_pos;
	BYTE* 	cursor_pos		= &mouse_Cursor->data[cursor_ofs];
	BYTE*	underneath_pos	= &mouse_UnderCursor->data[cursor_ofs];

	// figure out the screen address of the top-left corner of the cursor
	long*	row_addr 		= &VESA_RowPtr[row];
	int*  	bank_ofs		= &VESA_RowBank[row];
	int		bank_num;


	// figure out the required bank & change iff required
	bank_num = *bank_ofs++ + VESA_Write_Bank_Offset;
	if (bank_num!=VESA_Current_Write_Bank) VESA_SwitchWriteBank( bank_num );

	// create a pointer to the screen at the position to start blitting
	screen_pos = (BYTE*) (*row_addr + x1);

	// blit the current row of the cursor
	for (x=x1;x<=x2;x++)
		{
		tmp = *cursor_pos;		// get the next pixel in the row


		// if the appropriate pixel in the mouse cursor needs to be drawn
		if (tmp)
			{
			// save whats underneath the cursor iff required
			if (options&SAVE_UNDERNEATH_CURSOR)
				*underneath_pos = *screen_pos;

			// and draw the pixel
			*screen_pos = tmp;
			}

		cursor_pos++;			// skip to the next cursor pixel
		screen_pos++;
		underneath_pos++;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_Hide()
//	DESC:		Hides the mouse if required.
//
//	-------------------------------------------------------------------------
void mouse_Hide()
	{
	if (!MOUSE_INSTALLED) return;

	// only clear the cursor for the final time it's hidden
	if (TimesHidden==0) mouse_UndrawCursor();

	TimesHidden++;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_Show()
//	DESC:		Shows the mouse if required.
//
//	-------------------------------------------------------------------------
void mouse_Show()
	{
	if (!MOUSE_INSTALLED) return;

	// if we're not back to the appropriate Show(),
	// simply decrement
	if (TimesHidden>0) TimesHidden--;

	if (TimesHidden==0) mouse_DrawCursor( SAVE_UNDERNEATH_CURSOR );
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_UpdateCursor()
//	DESC:		Updates cursor on screen iff the mouse moves.
//
//	-------------------------------------------------------------------------
void mouse_UpdateCursor()
	{
	if (!MOUSE_INSTALLED) return;

	// update only if the cursor has moved!
	if (HasMoved)
		{
		// undraw the cursor at the last drawn coords

		mouse_UndrawCursor();

		// draw the cursor at the new x,y coords

		mouse_DrawCursor( SAVE_UNDERNEATH_CURSOR );

		// reset flag so that we don't keep drawing cursor

		HasMoved = false;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_SetCursor()
//	DESC:		Sets the mouse cursor to 'curs' as well as the mouse hotspot
//
//	-------------------------------------------------------------------------
void mouse_SetCursor(int hotx,int hoty,MouseCursor* curs)
	{
	if (!MOUSE_INSTALLED) return;

	// iff the cursor is different... change the cursor
	if (curs!=mouse_Cursor)
		{
		// skip out if we have a null cursor
		if (curs==NULL) return;

		// undraw the current cursor

		mouse_UndrawCursor();

		// set the new cursor

		mouse_Cursor = curs;

		// draw the new cursor

		mouse_DrawCursor( SAVE_UNDERNEATH_CURSOR );
		}
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetEvent()
//	DESC:		checks and generates mouse_auto?? events as required
//
//	-------------------------------------------------------------------------

void mouse_GetEvent()
	{
	if (!MOUSE_INSTALLED) return;

	int i,n;
	long newticks;

	if (!eventmgr_INSTALLED) return;

	// note: modifiers, x,y set already

	newticks = GetTicks();

	// --- check the left button

	if (leftBut == mouse_PRESSED)
		{
		long ticks = newticks - LMBTicks;
		if ((ticks>mouse_AUTO_DELAY) || (LMBAuto))
			{
			LMBAuto  = true;
			LMBTicks = newticks;
			EventInfo.event = mb_leftauto;
			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}
		}

	// --- check the right button

	if (rightBut == mouse_PRESSED)
		{
		long ticks = newticks - RMBTicks;
		if ((ticks>mouse_AUTO_DELAY) || (RMBAuto))
			{
			RMBAuto  = true;
			RMBTicks = newticks;
			EventInfo.event = mb_rightauto;
			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}
		}

	// --- check the middle button

	if (middleBut == mouse_PRESSED)
		{
		long ticks = newticks - MMBTicks;
		if ((ticks>mouse_AUTO_DELAY) || (MMBAuto))
			{
			MMBAuto  = true;
			MMBTicks = newticks;
			EventInfo.event = mb_middleauto;
			EVENTMGR->GenEvent(0,ev_mouse,&EventInfo);
			}
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DriverLoaded()
//	DESC:		Returns true if mouse driver is loaded.
//
//	-------------------------------------------------------------------------
bool mouse_DriverLoaded()
	{
	union REGPACK regs;
	SYS_Memset(&regs,0,sizeof(regs));

	// try resetting mouse driver
	regs_w.ax = 0;
	intr(0x33, &regs);

	// and return true if driver is loaded
	return (regs_w.ax==0xFFFF);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_InstallHandler()
//	DESC:		Either	(a) installs the mouse interrupt handler.
//						(b) installs mouse autodetection functions.
//
//	-------------------------------------------------------------------------
void mouse_InstallHandler()
	{
	if (MOUSE_INSTALLED) return;

	// make sure we have a mouse driver in memory
	if (!mouse_DriverLoaded()) return;

	int                 result;
	union REGPACK       	regs;

	SYS_Memset( &regs, 0, sizeof( union REGPACK ) );

	regs_w.ax = 0;
	intr( MOUSE_INT, &regs );
	result = regs_w.ax;

	if( result != 0 )
		{
		// SuperVGA mode, we have to draw the mouse cursor ourselves
		void (_loadds far* function_ptr)(int,int,int,int);
		function_ptr = &mouse_handler;

		regs_w.ax  = 0x000C;
		regs_w.cx  = 0x007F;
		regs.x.edx = FP_OFF( function_ptr );
		regs_w.es  = FP_SEG( function_ptr );

		intr( MOUSE_INT, &regs );

		// create room for whats under cursor.... (max. size == 32 * 32)
		mouse_UnderCursor = grCreateImage(0,0,31,31,0);
		mouse_UnderCursor_1 = grCreateImage(0,0,31,31,0);

		// setup our initial mouse cursor bits & pieces
		mouse_Reset();
		mouse_SetCursor(0,0,DEFAULT_CURSOR);

		// put cursor on the screen
		mouse_Show();

		// and setup our ticks
		LMBTicks = GetTicks();
		RMBTicks = GetTicks();
		MMBTicks = GetTicks();

		// flag to say we are autodetecting
		MOUSE_INSTALLED = true;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_RemoveHandler()
//	DESC:		Removes the mouse interrupt handler.
//
//	-------------------------------------------------------------------------
void mouse_RemoveHandler()
	{
	if (!MOUSE_INSTALLED) return;

	mouse_SetCursor(0,0,DEFAULT_CURSOR);
	mouse_Hide();

	// get rid of what was under cursor
	grDestroyImage(mouse_UnderCursor_1);
	grDestroyImage(mouse_UnderCursor);

	// init again, to clear handler
	union REGPACK       regs;

	SYS_Memset( &regs, 0, sizeof( union REGPACK ) );
	regs_w.ax = 0;
	intr( MOUSE_INT, &regs );

	MOUSE_INSTALLED = false;
	}

/*

	// this is a concept for a coded mouse cursor to speed up mouse cursor
	// drawing

	p_index = 0;

		// 1000000001000000
		// 1100000001100000
		// 1110000001110000
		// 1111111111111100
		// 0010000000010000

		mouseCursor ARROW_CURSOR = {
		0x04,	// num_rows
		0x03, 	// num_elems in row

		0x01, 0xFF, 0x08, 0x00, 0x01, 0xFF,	// row data (ie. 1xFF, 8x00, 1xFF)
		0x03, 0x02, 0xFF, 0x07, 0x00, 0x02, 0xFF,
		0x03, 0x03, 0xFF, 0x06, 0x00, 0x03, 0xFF,
		0x02, 0x0E, 0xFF, 0x02, 0x00,

		}
		*p = 1xFF,8x00,1xFF,6x00

		rle_length = p[p_index++] * 2;

		do
			{
			num_colors  = p[p_index++];
			color 		= p[p_index++];

			if (color==0) { screen_pos += num_colors; continue; }


			if (num_colors>3)

			w_color_mask= (color<<8) + color;
			d_color_mask= (w_color_mask<<16) + w_color_mask;

			dpos		= (DWORD*) screen_pos;

			i 			=  num_colors;
			while (i>3)
				{
				*dpos = d_color_mask;
				dpos++;
				i-=4;
				}

			wpos = (WORD*) dpos;
			while (i>1)
				{
				*wpos = w_color_mask;
				i -= 2;
				}

			bpos = (WORD*) bpos;
			while (i>0)
				{
				*bpos = color;
				}

			screen_pos += num_colors;
			}
		while (p_index<rle_length);

		BYTE* bpos;
		WORD* wpos;
		DWORD* dpos;

		switch (yrow)
			{
			case 0:		bpos = (BYTE*) screen_pos; *bpos = 0xFF;
						break;
			case 1:		wpos = (WORD*) screen_pos; *wpos = 0xFFFF;
						break;
			case 2:		bpos = (BYTE*) screen_pos++; *bpos = 0xFF;
						wpos = (WORD*) screen_pos;   *wpos = 0xFFFF;
						break;
			case 4:		dpos = (DWORD*) screen_pos;  *dpos = 0xFFFFFFFF;
						break;
			case 3:		screen_pos++;
						screen_pos++;
						bpos = (BYTE*) screen_pos; *bpos = 0xFF;
						break;
			}
*/

