//	-------------------------------------------------------------------------
//
//	TESTMOUS.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is an example showing how to use a standard template
//					to enable rapid application development.
//
//	-------------------------------------------------------------------------

#include "testmous.hpp"
#include "palettes.hpp"
#include "palfile.hpp"
#include "fonts.hpp"
#include "borders.hpp"

// -----------------

#include "system.hpp"		// system (gde) information

//	-------------------------------------------------------------------------
//
//	FUNC:		Init()
//	DESC:		Here we create the global map data, etc.
//
//	-------------------------------------------------------------------------
extern bool SYS_DEBUG;

void	TestMouse::Init()
	{
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Exit()
//	DESC:		Here we destroy the global map data, etc.
//
//	-------------------------------------------------------------------------
void	TestMouse::Exit()
	{
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		MainLoop()
//	DESC:       This is the main loop of the program.
//
//	-------------------------------------------------------------------------
void TestMouse::MainLoop()
	{
	// set up our timer to work at 100 times a second
	timer_ChangeInterruptRate(timer_100Hz);

	// create all objects used in the program (in order of priority)

	DBG(470,"Creating EVENT HANDLERS"); delay(2);

	// --- create all debugging objects

	EVENTMGR->AddEventHandler(new System());	// used for debugging engine

	// --- select appropriate handler on startup

	EVENTMGR->SelectEventHandler(1);		// Generic Screen

	// --- display our handler

	EVENTMGR->RefreshAllHandlers();

	// add any tickers required by the main game
	GAME_TICKER = timer_AddTicker(GAME_DELAY,cmdGameTick);

	// turn on mouse move events
	mouse_EnableMoveEvents();

	// tell the event manager to execute using the handlers!!!
	EVENTMGR->Execute();

	// disable mouse move events
	mouse_DisableMoveEvents();

	// delete the relevent tickers...
	timer_DelTicker( GAME_TICKER );

	// delete the objects now that we have finished with them
	EVENTMGR->DelAllEventHandlers();
	}


void	TestMouse::Execute()
	{
	Init();
	if (!EVENTMGR->isTerminating) MainLoop();
	Exit();
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		main()
//	DESC:		This is the function called at program execution.
//				At the moment it simply creates our game, executes it,
//				then deletes it, exiting normally
//
//	-------------------------------------------------------------------------
void main(int argc,char** argv)
	{
	// create our program
	TestMouse* theProgram= new TestMouse(argv[0],prg_InstallGUI|prg_InstallSystemFont);

	// run it's intro, then setup, then mainloop, then finale
	theProgram->Execute();

	// and clear up after ourselves
	delete theProgram;
	}
