//	-------------------------------------------------------------------------
//
//	MOUSE.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains code for the mouse ISR which is
//					capable of auto-detection & direct hardware response.
//
//	-------------------------------------------------------------------------

#include "stddefns.hpp"
#include "mouse.hpp"
#include "keyboard.hpp"
#include "event.hpp"
#include "eventmgr.hpp"
#include "graphics.hpp"

#include "dos.h"
#include "mem.h"

#ifdef __BORLANDC__
#define		regs_w 		regs.x
#define		int386		int86
#else
#include "i86.h"
#define		regs_w		regs.w
#endif

#include "conio.h"
#include "stdio.h"
#include "stdlib.h"
#include "malloc.h"

// some interrupt & port numbers

#define COM1_INT				0x0C
#define COM2_INT				0x0B
#define COM3_INT				0x0A
#define COM1_PORT				0x03F8
#define COM2_PORT				0x02F8

// mouse handler internal status definitions

#define mouse_READYTOINSTALL		0	// ready to install
#define mouse_DETECTING             1	// auto detecting
#define mouse_SETPORT               2	// found relevent port
#define mouse_INSTALLED             3	// mouse driver fully installed

// a few sample cursors

MouseCursor		ARROW_CURSOR =
				{ 24,24,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 15,15,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,

				0,0,8,7, 7,7,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 15,15,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 7,7,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 7,7,7,7, 15,15,0,0, 0,0,0,0, 0,0,0,0,

				0,0,8,7, 7,7,7,15, 15,15,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,15, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 15,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 7,7,7,7, 7,15,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,

				0,0,8,7, 7,7,7,7, 7,7,15,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 8,7,7,7, 7,7,7,15, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,7, 8,8,7,7, 7,7,7,7, 15,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,8,8, 0,0,8,7, 7,7,7,7, 7,15,0,0, 0,0,0,0, 0,0,0,0,

				0,0,8,0, 0,0,0,8, 7,7,7,7, 7,7,15,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 8,7,7,7, 7,7,8,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,8,7,7, 7,8,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,8,7, 8,0,0,0, 0,0,0,0, 0,0,0,0,

				0,0,0,0, 0,0,0,0, 0,0,0,8, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0   };

MouseCursor		DummyCursor[] =
				{ 16, 16,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,9, 0,9,0,9, 0,9,0,9, 0,9,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,0, 0,0,0,0, 0,0,0,0, 0,9,0,0,

				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,0, 0,0,0,0, 0,0,0,0, 0,9,0,0,
				0,0,0,0, 0,0,4,4, 4,4,0,0, 0,0,0,0,
				0,9,0,0, 0,0,4,4, 4,4,0,0, 0,9,0,0,

				0,0,0,0, 0,0,4,4, 4,4,0,0, 0,0,0,0,
				0,9,0,0, 0,0,4,4, 4,4,0,0, 0,9,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,0, 0,0,0,0, 0,0,0,0, 0,9,0,0,

				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,9,0,9, 0,9,0,9, 0,9,0,9, 0,9,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
				0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0 };

// global variables used by the mouse routines

MouseCursor*	DEFAULT_CURSOR = &ARROW_CURSOR;


// the delay for the auto repeat
#define mouse_AUTO_DELAY			3


uchar	mouse_xthresh = 80;			// x distance moved before acceleration
uchar	mouse_ythresh = 80;			// y distance moved before acceleration

int		mouse_HotSpot_x = 0;		// hotspot in mouse cursor
int		mouse_HotSpot_y = 0;		// hotspot in mouse cursor

bool 	mouse_MoveEventsOn = false;	// whether or not to generate move events
bool 	mouse_HasMoved = false;		// whether or not the mouse has moved (internal)

int    	mouse_cur_x = 320;			// last cursor x position
int		mouse_cur_y = 240;			// last cursor y position

int		mouse_x1    = 320;
int		mouse_y1	= 240;
int		mouse_x2	= 320 + 32;
int		mouse_y2	= 240 + 32;

int		mouse_HiddenCnt  = 1;		// count to tell how many times we've hidden the mouse
									// note: =1 due to first time cursor display

long	mouse_LMBTicks  = 0;		// ticks between left button presses
long	mouse_RMBTicks  = 0;		// ticks between right button presses
long	mouse_MMBTicks  = 0;		// ticks between middle button presses

bool   	mouse_LMBAuto	= false;
bool   	mouse_RMBAuto	= false;
bool   	mouse_MMBAuto	= false;

mouse_Status	mouse;				// current mouse status
mouse_Status 	lastmouse;			// last mouse status
MouseEvent 		mouse_EventInfo;    // mouse event information

MouseCursor*	mouse_Cursor = &ARROW_CURSOR;	// the current mouse cursor pointer
grImage*		mouse_UnderCursor = NULL;		// image under current mouse cursor
grImage*		mouse_UnderCursor_1 = NULL;		// image under current mouse cursor

uchar 			mouse_numbuttons = 0;			// number of mouse buttons

// global definitions of mouse variables

int		MOUSE_STATE = mouse_READYTOINSTALL;	// ready to install driver by default
int		MOUSE_INT = 0;						// unknown mouse interrupt
int		MOUSE_PORT = 0;						// unknown mouse port

// storage of previous ISR's

#ifdef __cplusplus
#ifdef __WATCOMC__
#define	CPPARGS
#else
#define CPPARGS		...
#endif
#else
#define	CPPARGS
#endif

void _interrupt (_far *Old_Mouse_Isr)(CPPARGS);
void _interrupt (_far *Old_Com1_Isr)(CPPARGS);
void _interrupt (_far *Old_Com2_Isr)(CPPARGS);

// variables used by mouse ISR

BYTE byte1,byte2,byte3,byte4,byte5;
int  mouse_IntCnt = 0;			// interrupt counter for ISR


//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_IsInstalled()
//	DESC:		returns true if mouse driver is currently installed
//
//	-------------------------------------------------------------------------
bool mouse_IsInstalled()
	{
	return (MOUSE_STATE == mouse_INSTALLED);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_Reset()
//	DESC:		resets the mouse to its original state
//
//	-------------------------------------------------------------------------
void mouse_Reset()
	{
	// and other relevant data
	memset(&mouse,0,sizeof(mouse));
	memset(&lastmouse,0,sizeof(lastmouse));

	mouse.x = grMaxx / 2;
	mouse.y = grMaxy / 2;
	mouse_cur_x = mouse.x;
	mouse_cur_y = mouse.y;
	lastmouse.x = mouse.x;
	lastmouse.y = mouse.y;

	mouse_x1 = mouse.x;
	mouse_y1 = mouse.y;
	mouse_x2 = mouse_x1 + mouse_Cursor->hdr.width-1;
	mouse_y2 = mouse_y1 + mouse_Cursor->hdr.height-1;

	CLIP2(mouse_x1,mouse_y1,0,0,grMaxx,grMaxy);

	mouse_MoveEventsOn = false;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_EvableMoveEvents()
//	DESC:		upon calling this function, mouse_GetEvent() will generate
//				mb_move events. (Note: this may cause a large number of
//				events to be placed in the event queue!!!)
//
//	-------------------------------------------------------------------------
void mouse_EnableMoveEvents()
	{
	mouse_MoveEventsOn = true;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DisableMoveEvents()
//	DESC:		stops mouse_GetEvent from generating mouse move events into
//				the EVENT QUEUE.
//
//	-------------------------------------------------------------------------
void mouse_DisableMoveEvents()
	{
	mouse_MoveEventsOn = false;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_SetSpeedThreshold()
//	DESC:		Use this to set the desired movement (in pixels) before
//				the mouse cursor is accelerated
//
//	PARAMS:		xthresh - the distance required in x direction
//				ythresh - the distance required in y direction
//
//	-------------------------------------------------------------------------
void mouse_SetAcceleration(uchar xthresh,uchar ythresh)
	{
	mouse_xthresh = xthresh;
	mouse_ythresh = ythresh;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetPos()
//	DESC:		returns the current position of the mouse as a point
//
//	-------------------------------------------------------------------------
grPoint mouse_GetPos()
	{
	grPoint p = {0,0};
	switch (MOUSE_STATE)
		{
		case	mouse_SETPORT:		mouse_InstallHandler();
									break;
		}
	p.x = mouse.x;
	p.y = mouse.y;
	return p;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_IsHit()
//	DESC:		returns true if mouse is waiting to be processed
//
//	-------------------------------------------------------------------------
bool mouse_IsHit()
	{
	bool ishit = ((mouse.leftBut!=mouse_RELEASED)||
				  (mouse.rightBut!=mouse_RELEASED)||
				  (mouse.middleBut!=mouse_RELEASED));
	switch (MOUSE_STATE)
		{
		case mouse_SETPORT:		mouse_InstallHandler();
								break;
		}
	return ishit;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_Flush()
//	DESC:		flushes the mouse input buffer
//
//	-------------------------------------------------------------------------
void mouse_Flush()
	{
	switch (MOUSE_STATE)
		{
		case mouse_SETPORT:		mouse_InstallHandler();
								break;
		}
	mouse.leftBut = mouse_RELEASED;
	mouse.rightBut = mouse_RELEASED;
	mouse.middleBut = mouse_RELEASED;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_InRect()
//	DESC:		Checks to see whether the mouse cursor is in the specified
//				coordinates. Returns true if it is.
//
//	-------------------------------------------------------------------------
bool mouse_InRect(int x1,int y1,int x2,int y2)
	{
	int xx,yy,xx1,yy1;

	bool result = false;

	switch (MOUSE_STATE)
		{
		case	mouse_INSTALLED:

			xx = mouse.x;
			yy = mouse.y;
			xx1 = xx + mouse_Cursor->hdr.width-1;
			yy1 = yy + mouse_Cursor->hdr.height-1;

			// check coords
			result = ((xx>=x1)&&(xx<=x2)&&(yy>=y1)&&(yy<=y2)||
					  (xx1>=x1)&&(xx1<=x2)&&(yy1>=y1)&&(yy1<=y2));
									break;

		case	mouse_SETPORT:

			mouse_InstallHandler();
			break;
		}
	return result;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DrawCursor()
//	DESC:		Draws the mouse on currently active page
//
//	-------------------------------------------------------------------------
void mouse_DrawCursor()
	{
	int x,y,tmp;

	// figure out the screen address of the top-left corner of the cursor
	long*	row_addr 		= &VESA_RowPtr[mouse_y1];

	BYTE*	screen_pos;
	BYTE* 	cursor_pos		= &mouse_Cursor->data[0];

	int		bank_num;

	// scan through the cursor vertically
	for (y=mouse_y1;y<=mouse_y2;y++)
		{
		// figure out the required bank & change iff required
		bank_num = VESA_RowBank[y] + VESA_Write_Bank_Offset;
		if (bank_num!=VESA_Current_Write_Bank) VESA_SwitchWriteBank( bank_num );

		// create a pointer to the screen at the position to start blitting
		screen_pos = (BYTE*) (*row_addr + mouse_x1);

		// blit the current row of the cursor
		for (x=mouse_x1;x<=mouse_x2;x++)
			{
			tmp = *cursor_pos;		// get the next pixel in the row

			if (tmp)				// if its set
				*screen_pos = tmp;	// put the pixel
									// onto the screen (_SLOW_ -> AT Bus)

			cursor_pos++;			// skip to the next cursor pixel
			screen_pos++;
			}

		// for the next row, simply add the logical width to the address
		row_addr++;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_UnDrawCursor()
//	DESC:		Erases the mouse on the currently active page
//
//	-------------------------------------------------------------------------
void mouse_UndrawCursor()
	{
	int x,y,tmp;

	// figure out the screen address of the top-left corner of the cursor
	long*	row_addr		= &VESA_RowPtr[mouse_y1];
	long	underneath_addr	= (long)&mouse_UnderCursor->data[0];

	BYTE*	screen_pos;
	BYTE* 	cursor_pos		= &mouse_Cursor->data[0];
	BYTE*	underneath_pos;

	int		underneath_width= mouse_UnderCursor->hdr.width;
	int		bank_num;

	// scan through the cursor vertically
	for (y=mouse_y1;y<=mouse_y2;y++)
		{
		// figure out the required bank & change iff required
		bank_num = VESA_RowBank[y] + VESA_Write_Bank_Offset;
		if (bank_num!=VESA_Current_Write_Bank) VESA_SwitchWriteBank( bank_num );

		// create a pointer to the screen at the position to start blitting
		screen_pos = (BYTE*) (*row_addr + mouse_x1);
		underneath_pos = (BYTE*) underneath_addr;

		// blit the current row of the cursor
		for (x=mouse_x1;x<=mouse_x2;x++)
			{
			tmp = *cursor_pos;		// get the next pixel in the row

			if (tmp)				// if its set
				*screen_pos = *underneath_pos;
									// put whats underneath the cursor
									// onto the screen (_SLOW_ -> AT Bus)

			cursor_pos++;			// skip to the next cursor pixel
			screen_pos++;
			underneath_pos++;
			}

		// for the next row, simply add the logical width to the address
		row_addr++;
		underneath_addr += underneath_width;
		}
	}

void mouse_UndrawCursor_1()
	{
	int x,y,tmp;

	// figure out the screen address of the top-left corner of the cursor
	long*	row_addr		= &VESA_RowPtr[mouse_y1];
	long	underneath_addr	= (long)&mouse_UnderCursor_1->data[0];

	BYTE*	screen_pos;
	BYTE* 	cursor_pos		= &mouse_Cursor->data[0];
	BYTE*	underneath_pos;

	int		underneath_width= mouse_UnderCursor_1->hdr.width;
	int		bank_num;

	// scan through the cursor vertically
	for (y=mouse_y1;y<=mouse_y2;y++)
		{
		// figure out the required bank & change iff required
		bank_num = VESA_RowBank[y] + VESA_Write_Bank_Offset;
		if (bank_num!=VESA_Current_Write_Bank) VESA_SwitchWriteBank( bank_num );

		// create a pointer to the screen at the position to start blitting
		screen_pos = (BYTE*) (*row_addr + mouse_x1);
		underneath_pos = (BYTE*) underneath_addr;

		// blit the current row of the cursor
		for (x=mouse_x1;x<=mouse_x2;x++)
			{
			tmp = *cursor_pos;		// get the next pixel in the row

			if (tmp)				// if its set
				*screen_pos = *underneath_pos;
									// put whats underneath the cursor
									// onto the screen (_SLOW_ -> AT Bus)

			cursor_pos++;			// skip to the next cursor pixel
			screen_pos++;
			underneath_pos++;
			}

		// for the next row, simply add the logical width to the address
		row_addr++;
		underneath_addr += underneath_width;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetUnderCursor()
//	DESC:		Saves what is underneath mouse on the currently active page
//
//	-------------------------------------------------------------------------
void mouse_GetUnderCursor()
	{
	int x,y,tmp;

	// figure out the screen address of the top-left corner of the cursor
	long*	row_addr		= &VESA_RowPtr[mouse_y1];
	long	underneath_addr	= (long)&mouse_UnderCursor->data[0];

	BYTE*	screen_pos;
	BYTE* 	cursor_pos		= &mouse_Cursor->data[0];
	BYTE*	underneath_pos;

	int		underneath_width= mouse_UnderCursor->hdr.width;
	int		bank_num;

	// scan through the cursor vertically
	for (y=mouse_y1;y<=mouse_y2;y++)
		{
		// figure out the required bank & change iff required
		bank_num = VESA_RowBank[y] + VESA_Read_Bank_Offset;
		if (bank_num!=VESA_Current_Read_Bank) VESA_SwitchReadBank( bank_num );

		// create a pointer to the screen at the position to start blitting
		screen_pos = (BYTE*) (*row_addr + mouse_x1);
		row_addr++;

		underneath_pos = (BYTE*) underneath_addr;

		// blit the current row of the cursor
		for (x=mouse_x1;x<=mouse_x2;x++)
			{
			tmp = *cursor_pos;		// get the next pixel in the row

			if (tmp)				// if its set
				*underneath_pos = *screen_pos;
									// get whats on the screen
									// and save it

			cursor_pos++;			// skip to the next cursor pixel
			screen_pos++;
			underneath_pos++;
			}

		// for the next row, simply add the logical width to the address
		underneath_addr += underneath_width;
		}
	}

void mouse_GetUnderCursor_1()
	{
	int x,y,tmp;

	// figure out the screen address of the top-left corner of the cursor
	long*	row_addr		= &VESA_RowPtr[mouse_y1];
	long	underneath_addr	= (long)&mouse_UnderCursor_1->data[0];

	BYTE*	screen_pos;
	BYTE* 	cursor_pos		= &mouse_Cursor->data[0];
	BYTE*	underneath_pos;

	int		underneath_width= mouse_UnderCursor_1->hdr.width;
	int		bank_num;

	// scan through the cursor vertically
	for (y=mouse_y1;y<=mouse_y2;y++)
		{
		// figure out the required bank & change iff required
		bank_num = VESA_RowBank[y] + VESA_Read_Bank_Offset;
		if (bank_num!=VESA_Current_Read_Bank) VESA_SwitchReadBank( bank_num );

		// create a pointer to the screen at the position to start blitting
		screen_pos = (BYTE*) (*row_addr + mouse_x1);
		row_addr++;

		underneath_pos = (BYTE*) underneath_addr;

		// blit the current row of the cursor
		for (x=mouse_x1;x<=mouse_x2;x++)
			{
			tmp = *cursor_pos;		// get the next pixel in the row

			if (tmp)				// if its set
				*underneath_pos = *screen_pos;
									// get whats on the screen
									// and save it

			cursor_pos++;			// skip to the next cursor pixel
			screen_pos++;
			underneath_pos++;
			}

		// for the next row, simply add the logical width to the address
		underneath_addr += underneath_width;
		}
	}

void mouse_CopyUnderCursor_1()
	{
	int bytes_to_copy = mouse_UnderCursor->hdr.width * mouse_UnderCursor->hdr.height;
	int dwords_to_copy= bytes_to_copy >> 2;
	int word_to_copy  = (bytes_to_copy & 2);
	int byte_to_copy  = (bytes_to_copy & 1);

	DWORD* 	ddest;
	DWORD*	dsrce;

	ddest = (DWORD*) &mouse_UnderCursor->data[0];
	dsrce = (DWORD*) &mouse_UnderCursor_1->data[0];

	for (int dwords=0;dwords<dwords_to_copy;dwords++) *ddest++ = *dsrce++;

	if (word_to_copy!=0)
		{
		WORD* 	wdest = (WORD*) ddest;
		WORD*	wsrce = (WORD*) dsrce;

		*wdest++ = *wsrce++;

		if (byte_to_copy!=0)
			{
			BYTE* 	bdest = (BYTE*) wdest;
			BYTE*	bsrce = (BYTE*) wsrce;

			*bdest = *bsrce;
			}
		}
	else
		if (byte_to_copy!=0)
			{
			BYTE*	bdest = (BYTE*) ddest;
			BYTE*	bsrce = (BYTE*)	dsrce;

			*bdest = *bsrce;
			}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_Hide()
//	DESC:		Hides the mouse if required.
//
//	-------------------------------------------------------------------------
void mouse_Hide()
	{
	switch (MOUSE_STATE)
		{
		case	mouse_INSTALLED:

			// if its the first time called
			if (mouse_HiddenCnt==0)
				{
				int save_page = VESA_Active_Page;

				// hide the mouse by replacing what was underneath the last cursor drawn
				VESA_SetActivePage(VESA_Visual_Page);
				mouse_UndrawCursor();
				VESA_SetActivePage(save_page);
				}

			mouse_HiddenCnt++;
			break;

		case	mouse_SETPORT:

			mouse_InstallHandler();
			break;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_Show()
//	DESC:		Shows the mouse if required.
//
//	-------------------------------------------------------------------------
void mouse_Show()
	{
	switch (MOUSE_STATE)
		{
		case	mouse_INSTALLED:

			// if we're not back to the appropriate Show(),
			// simply decrement
			if (mouse_HiddenCnt>0)
				mouse_HiddenCnt--;

			if (mouse_HiddenCnt==0)
				{
				int save_page = VESA_Active_Page;

				VESA_SetActivePage(VESA_Visual_Page);

				// at zero we show the cursor again
				mouse_GetUnderCursor();
				mouse_DrawCursor();

				VESA_SetActivePage(save_page);
				}
			break;

		case	mouse_SETPORT:

			mouse_InstallHandler();
			break;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_UpdateCursor()
//	DESC:		Updates cursor on screen iff the mouse moves.
//
//	-------------------------------------------------------------------------
void mouse_UpdateCursor()
	{
	int save_page = VESA_Active_Page;

	switch (MOUSE_STATE)
		{
		case	mouse_INSTALLED:

			// update only if the cursor has moved!
			if (mouse_HasMoved && (mouse_HiddenCnt==0))
				{
				VESA_SetActivePage(VESA_Visual_Page);

				int oldx = mouse_cur_x;	// save coords of last-drawn
				int oldy = mouse_cur_y;	// mouse cursor
				int newx = mouse.x;		// save coords of where the
				int newy = mouse.y;		// mouse cursor is to be redrawn

				// --- UNDRAW CURSOR AT OLD X,Y COORDS

				mouse_cur_x = oldx;
				mouse_cur_y = oldy;
				mouse_x1 = mouse_cur_x;
				mouse_y1 = mouse_cur_y;
				mouse_x2 = mouse_cur_x + mouse_Cursor->hdr.width-1;
				mouse_y2 = mouse_cur_y + mouse_Cursor->hdr.height-1;
				CLIP2(mouse_x1,mouse_y1,0,0,grMaxx,grMaxy);
				mouse_UndrawCursor();

				// --- GET UNDER CURSOR AT NEW X,Y COORDS

				mouse_cur_x = newx;
				mouse_cur_y = newy;
				mouse_x1 = mouse_cur_x;
				mouse_y1 = mouse_cur_y;
				mouse_x2 = mouse_cur_x + mouse_Cursor->hdr.width-1;
				mouse_y2 = mouse_cur_y + mouse_Cursor->hdr.height-1;
				CLIP2(mouse_x1,mouse_y1,0,0,grMaxx,grMaxy);
				mouse_GetUnderCursor();

				// --- DRAW CURSOR AT NEW X,Y COORDS

				mouse_cur_x = newx;
				mouse_cur_y = newy;
				mouse_x1 = mouse_cur_x;
				mouse_y1 = mouse_cur_y;
				mouse_x2 = mouse_cur_x + mouse_Cursor->hdr.width-1;
				mouse_y2 = mouse_cur_y + mouse_Cursor->hdr.height-1;
				CLIP2(mouse_x1,mouse_y1,0,0,grMaxx,grMaxy);

				mouse_DrawCursor();
				mouse_HasMoved = false;

				VESA_SetActivePage(save_page);
				}
			break;

		case	mouse_SETPORT:

			mouse_InstallHandler();
			break;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_SetCursor()
//	DESC:		Sets the mouse cursor to 'curs' as well as the mouse hotspot
//
//	-------------------------------------------------------------------------
void mouse_SetCursor(int hotx,int hoty,MouseCursor* curs)
	{
	// set the hot spot....
	mouse_HotSpot_x = hotx;
	mouse_HotSpot_y = hoty;

	// iff the cursor is different... change the cursor
	if (curs!=mouse_Cursor)
		{
		// skip out if we have a null cursor
		if (curs==NULL) return;

		int save_page = VESA_Active_Page;
		VESA_SetActivePage(VESA_Visual_Page);

		// undraw the current cursor

		mouse_x1     = mouse_cur_x;
		mouse_y1	 = mouse_cur_y;
		mouse_x2	 = mouse_x1 + mouse_Cursor->hdr.width-1;
		mouse_y2	 = mouse_y1 + mouse_Cursor->hdr.height-1;

		mouse_UndrawCursor();

		// get underneath the new cursor

		mouse_Cursor = curs;
		mouse_x1 = mouse_cur_x;
		mouse_y1 = mouse_cur_y;
		mouse_x2 = mouse_x1 + mouse_Cursor->hdr.width-1;
		mouse_y2 = mouse_y1 + mouse_Cursor->hdr.height-1;

		CLIP2(mouse_x1,mouse_y1,0,0,grMaxx,grMaxy);

		mouse_GetUnderCursor();

		// draw the new cursor

		mouse_DrawCursor();

		// reset the page
		VESA_SetActivePage(save_page);
		}
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetEvent()
//	DESC:		checks and generates mouse_auto?? events as required
//
//	-------------------------------------------------------------------------

void mouse_GetEvent()
	{
	int i,n;
	long newticks;
	switch (MOUSE_STATE)
		{
		case	mouse_INSTALLED:

			if (!eventmgr_INSTALLED) return;

			// note: modifiers, x,y set already

			newticks = GetTicks();

			// --- check the left button

			if (mouse.leftBut == mouse_PRESSED)
				{
				long ticks = newticks - mouse_LMBTicks;

				if ((ticks>mouse_AUTO_DELAY) || (mouse_LMBAuto))
					{
					mouse_LMBAuto  = true;
					mouse_LMBTicks = newticks;
					mouse_EventInfo.event = mb_leftauto;
					EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
					}
				}

			// --- check the right button

			if (mouse.rightBut == mouse_PRESSED)
				{
				long ticks = newticks - mouse_RMBTicks;
				if ((ticks>mouse_AUTO_DELAY) || (mouse_RMBAuto))
					{
					mouse_RMBAuto  = true;
					mouse_RMBTicks = newticks;
					mouse_EventInfo.event = mb_rightauto;
					EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
					}
				}

			// --- check the middle button

			if (mouse.middleBut == mouse_PRESSED)
				{
				long ticks = newticks - mouse_MMBTicks;
				if ((ticks>mouse_AUTO_DELAY) || (mouse_MMBAuto))
					{
					mouse_MMBAuto  = true;
					mouse_MMBTicks = newticks;
					mouse_EventInfo.event = mb_middleauto;
					EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
					}
				}
			break;

		case	mouse_SETPORT:

			mouse_InstallHandler();
			break;
		}
	}

//	-------------------------------------------------------------------------
//	FUNCTION:		mouse_testcom1
//	DESCRIPTION:	this is used (internally) to test the COM1 interrupt
//					to see if a mouse is there
//	-------------------------------------------------------------------------

//  note: unknown to me why, but the mouse seems to drop out of sync over
// 		  a period of time... Seems well documented, so who knows... :(
//
//	define syncro masks
//	2-button: bit 6 is set only when syncro'd
//	3-button: bit 7 set & bits 6-3 clear only when syncro'd

const uchar sync2 = bit6;	// 64
const uchar sync3 = bit7 | bit6 | bit5 | bit4 | bit3;

void _interrupt _far mouse_COM1_ISR(CPPARGS)
	{
	_disable();

	// read a byte from the com1 port
	byte1 = inp(COM1_PORT);

	// check if we've found the mouse!!
	if ( ((byte1&sync2)==sync2)||
		 ((byte1&sync3)==bit7) )
		{
		// save that the mouse data came in from COM1
		MOUSE_INT = COM1_INT;
		MOUSE_PORT = COM1_PORT;

		// assume a 2 button mouse & check for a 3 button
		mouse_numbuttons=2;
		if ((byte1&sync3)==bit7) mouse_numbuttons++;

		// signal that we've found the relevent port
		MOUSE_STATE = mouse_SETPORT;
		}

	_enable();

	// acknowledge interrupt to hardware
	outp(0x20,0x20);
	}

//	-------------------------------------------------------------------------
//	FUNCTION:		mouse_testcom2
//	DESCRIPTION:	this is used (internally) to test the COM2 interrupt
//					to see if a mouse is there
//	-------------------------------------------------------------------------

void _interrupt _far mouse_COM2_ISR(CPPARGS)
	{
	_disable();

	// read a byte from the com1 port
	byte1 = inp(COM2_PORT);

	// check if we've found the mouse!!
	if ( ((byte1&sync2)==sync2)||
		 ((byte1&sync3)==bit7) )
		{
		// save that the mouse data came in from COM1
		MOUSE_INT = COM2_INT;
		MOUSE_PORT = COM2_PORT;

		// assume a 2 button mouse & check for a 3 button
		mouse_numbuttons=2;
		if ((byte1&sync3)==bit7) mouse_numbuttons++;

		// signal that we've found the relevent port
		MOUSE_STATE = mouse_SETPORT;
		}

	_enable();

	// acknowledge interrupt to hardware
	outp(0x20,0x20);
	}

//	-------------------------------------------------------------------------
//	FUNCTION:		mouse_handler
//	DESCRIPTION:	this is the actual interrupt handler for the mouse
//
//	NOTES:			Depending on the type of mouse, the information differs
//					as follows:
//
//					2-BUTTON MOUSE:
//
//									BIT(S): 	DESC:
//
//					[FIRST BYTE]	 6			if set, 2 button mouse
//									 5			if set, left button pressed
//									 4			if set, right button pressed
//					  				 3 & 2    	if set, moving up??
//									 1 & 0		if set, moving left??
//
//									VALUES(S):	DESC:
//					[SECOND BYTE]	-128..127	x pixels moved
//					[THIRD BYTE]	-128..127	y pixels moved
//
//
//					3-BUTTON MOUSE:
//
//									BIT(S):		DESC:
//
//					[FIRST BYTE]    0			if set, right button released
//									1			if set, middle button released
//									2			if set, left button released
//									7			if set, 3 button mouse
//
//									VALUE(S):	DESC:
//					[SECOND BYTE]	-128..127	x pixels moved
//					[THIRD BYTE]	127..-128	y pixels moved (note: -ve normal)
//
//					[FOURTH BYTE]	unknown		unknown
//					[FIFTH BYTE]	unknown		unknown
//
//	-------------------------------------------------------------------------

void _interrupt _far mouse_ISR(CPPARGS)
	{
	_disable();

	int lastx,lasty;
	int	leftBut,rightBut,middleBut;
	bool event_ready = false;

	// as the mouse generates a number of bytes, switch the byte cnt....
	switch (mouse_IntCnt)
		{
		case 0:	// read the first byte
				byte1 = inp(MOUSE_PORT);
				// check for syncronization
				if (((byte1&sync2)==sync2) || ((byte1&sync3)==bit7))
					 mouse_IntCnt++;
				break;
		case 1:	// read byte 2
				byte2 = inp(MOUSE_PORT);
				mouse_IntCnt++;
				break;
		case 2:	// read byte 3
				byte3 = inp(MOUSE_PORT);
				// as 2-button & 3-button mice respond differently, we
				// need to handle them seperately....

				// if 2-button mouse
				if (mouse_numbuttons==2)
					{
					// reset sync.
					mouse_IntCnt = 0;

					byte4 = byte2 | (byte1<<6);
					byte5 = byte3 | ((byte1&12) << 4);

					// calc new mouse coords
					lastx = mouse.x;
					lasty = mouse.y;

					mouse.x += (signed char) byte4;
					mouse.y += (signed char) byte5;

					// fix (wierdo bug) in acceleration (-1 for -ve x,y dirn)
//					char xthresh = mouse_xthresh;
//					char ythresh = mouse_ythresh;

					// if the movement is negative....
					// decrease the size of the movement
//					if ((byte4&0x80)==0x80) byte4++;
//					if ((byte5&0x80)==0x80) byte5++;

					// accelerate mouse iff required
//					if ((byte4 & 0x7F) > xthresh) mouse.x += (signed char) byte4;
//					if ((byte5 & 0x7F) > ythresh) mouse.y += (signed char) byte5;

					// check mouse boundaries
					if (mouse.x<0) mouse.x = 0;
					if (mouse.x>grMaxx) mouse.x = grMaxx;
					if (mouse.y<0) mouse.y = 0;
					if (mouse.y>grMaxy) mouse.y = grMaxy;

					// calc button status
					leftBut 	= mouse.leftBut;
					rightBut  	= mouse.rightBut;
					middleBut 	= mouse.middleBut;

					mouse.leftBut  = (byte1 & bit5) >> 5;
					mouse.rightBut = (byte1 & bit4) >> 4;
					mouse.middleBut= (mouse.leftBut && mouse.rightBut);

					event_ready = true;
					}
				else
					// 3-button mouse
					{
					// we need to get the last 2 bytes to sync. correctly
					mouse_IntCnt++;

					lastx = mouse.x;
					lasty = mouse.y;

					// calc new mouse coords
					// Note: Y coords are reversed, so use -= instead of +=
					mouse.x += (signed char) byte2;
					mouse.y -= (signed char) byte3;

					// accelerate mouse iff required
					if ( (byte2 & 0x7F) > mouse_xthresh) mouse.x += (signed char) byte2;
					if ( (byte3 & 0x7F) > mouse_ythresh) mouse.y -= (signed char) byte3;

					// check mouse boundaries
					if (mouse.x<0) mouse.x = 0;
					if (mouse.x>grMaxx) mouse.x = grMaxx;
					if (mouse.y<0) mouse.y = 0;
					if (mouse.y>grMaxy) mouse.y = grMaxy;

					// calc button status
					leftBut 	= mouse.leftBut;
					rightBut  	= mouse.rightBut;
					middleBut 	= mouse.middleBut;

					mouse.leftBut  = !( (byte1 & bit2) >> 2);
					mouse.middleBut= !( (byte1 & bit1) >> 1);
					mouse.rightBut = !( (byte1 & bit0));

					event_ready = true;
					}
				break;
		case 3:	byte4 = inp(MOUSE_PORT);	// byte info is unknown
				mouse_IntCnt++;
				break;
		case 4:	byte5 = inp(MOUSE_PORT);	// byte info is unknown
				// note: no need to check for syncro as it will happen anyway
				mouse_IntCnt = 0;
				break;
		}

	if ((event_ready) && (eventmgr_INSTALLED))
		{
		// save state ready for event generation
		mouse_EventInfo.modifiers = keyboard_GetState();
		mouse_EventInfo.where.x = mouse.x;
		mouse_EventInfo.where.y = mouse.y;

		// if the left button has changed state
		if (mouse.leftBut != leftBut)
			{
			// figure appropriate event to generate
			if (mouse.leftBut == mouse_PRESSED)
				{
				mouse_EventInfo.event = mb_leftdn;
				mouse_LMBTicks = GetTicks();
				}
			else
				{
				mouse_EventInfo.event = mb_leftup;
				mouse_LMBAuto  = false;
				}

			// generate the event
			EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
			}

		// check the right button
		if (mouse.rightBut != rightBut)
			{
			if (mouse.rightBut == mouse_PRESSED)
				{
				mouse_EventInfo.event = mb_rightdn;
				mouse_RMBTicks = GetTicks();
				}
			else
				{
				mouse_EventInfo.event = mb_rightup;
				mouse_RMBAuto  = false;
				}

			EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
			}

		// and the middle button
		if (mouse.middleBut != middleBut)
			{
			if (mouse.middleBut == mouse_PRESSED)
				{
				mouse_EventInfo.event = mb_middledn;
				mouse_MMBTicks = GetTicks();
				}
			else
				{
				mouse_EventInfo.event = mb_middleup;
				mouse_MMBAuto  = false;
				}

			EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
			}

		// check for mouse movement
		if ((mouse.x != lastx) || (mouse.y != lasty))
			{
			// signal that mouse has moved
			mouse_HasMoved = true;

			// if we're generating move events
			if (mouse_MoveEventsOn)
				{
				mouse_EventInfo.event = mb_move;

				EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
				}
			}
		}

	outp(0x20,0x20);	// acknowledge interrupt to hardware
	_enable();			// re-enable interrupts
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DriverLoaded()
//	DESC:		Returns true if mouse driver is loaded.
//
//	-------------------------------------------------------------------------
bool mouse_DriverLoaded()
	{
	union REGS regs;
	memset(&regs,0,sizeof(regs));

	// try resetting mouse driver
	regs_w.ax = 0;
	int386(0x33, &regs, &regs);

	// and return true if driver is loaded
	return (regs_w.ax==0xFFFF);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_InstallHandler()
//	DESC:		Either	(a) installs the mouse interrupt handler.
//						(b) installs mouse autodetection functions.
//
//	-------------------------------------------------------------------------
void mouse_InstallHandler()
	{
	switch (MOUSE_STATE)
		{
		case	mouse_READYTOINSTALL:

			// make sure we have a mouse driver in memory
			if (!mouse_DriverLoaded()) return;

			// create room for whats under cursor.... (max. size == 32 * 32)
			mouse_UnderCursor = grCreateImage(0,0,31,31,true);
			mouse_UnderCursor_1 = grCreateImage(0,0,31,31,true);

			// setup our initial mouse cursor bits & pieces
			mouse_Reset();			    // reset the mouse
			mouse_SetCursor(0,0,DEFAULT_CURSOR);

			// save old COM1 & COM2 ISRs
			Old_Com1_Isr = _dos_getvect(COM1_INT);
			Old_Com2_Isr = _dos_getvect(COM2_INT);

			// and set them to our autodetect ISRs
			_dos_setvect(COM1_INT,mouse_COM1_ISR);
			_dos_setvect(COM2_INT,mouse_COM2_ISR);

			// flag to say we are autodetecting
			MOUSE_STATE = mouse_DETECTING;
			break;

		case	mouse_SETPORT:

			// restore previous ISRs
			_dos_setvect(COM2_INT,Old_Com2_Isr);
			_dos_setvect(COM1_INT,Old_Com1_Isr);

			// and install the mouse ISR
			Old_Mouse_Isr = _dos_getvect(MOUSE_INT);
			_dos_setvect(MOUSE_INT,mouse_ISR);

			// our mouse is now installed
			MOUSE_STATE = mouse_INSTALLED;

			// put cursor on the screen
			mouse_Show();

			// and setup our ticks
			mouse_LMBTicks = GetTicks();
			mouse_RMBTicks = GetTicks();
			mouse_MMBTicks = GetTicks();

			if (mouse_numbuttons==2) mouse_SetAcceleration(80,80);
			else mouse_SetAcceleration(25,25);
			break;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_RemoveHandler()
//	DESC:		Removes the mouse interrupt handler.
//
//	-------------------------------------------------------------------------
void mouse_RemoveHandler()
	{
	// act upon relevent status to restore order
	switch (MOUSE_STATE)
		{
		case	mouse_INSTALLED:

			mouse_SetCursor(0,0,DEFAULT_CURSOR);
			mouse_Hide();

			// get rid of what was under cursor
			grDestroyImage(mouse_UnderCursor_1);
			grDestroyImage(mouse_UnderCursor);

			// called normally, simply remove the mouse handler
			_dos_setvect(MOUSE_INT,Old_Mouse_Isr);
			break;

		case	mouse_DETECTING:

			// get rid of what was under cursor
			grDestroyImage(mouse_UnderCursor_1);
			grDestroyImage(mouse_UnderCursor);

			// called in middle of autodetection
			_dos_setvect(COM2_INT,Old_Com2_Isr);
			_dos_setvect(COM1_INT,Old_Com1_Isr);
			break;

		case	mouse_SETPORT:

			// get rid of what was under cursor
			grDestroyImage(mouse_UnderCursor_1);
			grDestroyImage(mouse_UnderCursor);

			// called between autodetection & setting
			// mouse to appropriate com port
			_dos_setvect(COM2_INT,Old_Com2_Isr);
			_dos_setvect(COM1_INT,Old_Com1_Isr);
			break;

		case	mouse_READYTOINSTALL:

			// called before mouse_InstallHandler()
			break;

		}
	// we're now ready to install again
	MOUSE_STATE = mouse_READYTOINSTALL;

	HEAPCHK("by mouse_RemoveHandler()");
	}

/*

	// this is a concept for a coded mouse cursor to speed up mouse cursor
	// drawing

	p_index = 0;

		// 1000000001000000
		// 1100000001100000
		// 1110000001110000
		// 1111111111111100
		// 0010000000010000

		mouseCursor ARROW_CURSOR = {
		0x04,	// num_rows
		0x03, 	// num_elems in row

		0x01, 0xFF, 0x08, 0x00, 0x01, 0xFF,	// row data (ie. 1xFF, 8x00, 1xFF)
		0x03, 0x02, 0xFF, 0x07, 0x00, 0x02, 0xFF,
		0x03, 0x03, 0xFF, 0x06, 0x00, 0x03, 0xFF,
		0x02, 0x0E, 0xFF, 0x02, 0x00,

		}
		*p = 1xFF,8x00,1xFF,6x00

		rle_length = p[p_index++] * 2;

		do
			{
			num_colors  = p[p_index++];
			color 		= p[p_index++];

			if (color==0) { screen_pos += num_colors; continue; }


			if (num_colors>3)

			w_color_mask= (color<<8) + color;
			d_color_mask= (w_color_mask<<16) + w_color_mask;

			dpos		= (DWORD*) screen_pos;

			i 			=  num_colors;
			while (i>3)
				{
				*dpos = d_color_mask;
				dpos++;
				i-=4;
				}

			wpos = (WORD*) dpos;
			while (i>1)
				{
				*wpos = w_color_mask;
				i -= 2;
				}

			bpos = (WORD*) bpos;
			while (i>0)
				{
				*bpos = color;
				}

			screen_pos += num_colors;
			}
		while (p_index<rle_length);

		BYTE* bpos;
		WORD* wpos;
		DWORD* dpos;

		switch (yrow)
			{
			case 0:		bpos = (BYTE*) screen_pos; *bpos = 0xFF;
						break;
			case 1:		wpos = (WORD*) screen_pos; *wpos = 0xFFFF;
						break;
			case 2:		bpos = (BYTE*) screen_pos++; *bpos = 0xFF;
						wpos = (WORD*) screen_pos;   *wpos = 0xFFFF;
						break;
			case 4:		dpos = (DWORD*) screen_pos;  *dpos = 0xFFFFFFFF;
						break;
			case 3:		screen_pos++;
						screen_pos++;
						bpos = (BYTE*) screen_pos; *bpos = 0xFF;
						break;
			}
*/

