//	-------------------------------------------------------------------------
//
//	TEXTFILE.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file implements the TEXTFILE class
//
//	-------------------------------------------------------------------------

#include "textfile.hpp"

TextFile::TextFile(char* filename) : File(filename) {}
TextFile::TextFile(String* filename) : File(filename) {}
TextFile::TextFile(char* path,char* filename) : File(path,filename) {}
TextFile::TextFile(String* path,String* filename) : File(path,filename) {}

int		TextFile::ReadLine(char* buf,int maxlen)
	// reads a single line of text from the file
	{
	// skip out if file is not open
	if (!file_isopen) return file_FileNotOpen;

	int 	cnt = 0;
	bool 	Finished = false;

	// read till we find an EOF, till we've read the max. chars,
	// or until we find a CR/LF combination

	while (!AtEOF() && (cnt<maxlen) && !Finished)
		{
		// read a single char into the buffer
		Read(&buf[cnt],1);

		// if we have found a Carriage-Return
		if (buf[cnt]==13)
			{
			// read the Line-Feed char
			Read(&buf[cnt],1);

			// and we've now finished reading the line
			Finished = true;
			}
		else
			// get the next char
			cnt++;
		}

	// null-terminate the string
	buf[cnt] = 0x00;

//	printf("Finished reading line :\n%s\n",buf);
//	if (AtEOF()) printf("EOF encountered!\n");
//	else printf("Not at EOF!\n");

	// return an error if we encountered EOF
	if (AtEOF()) return textfile_ReadLineFailed;

	// everything is ok
	return file_NoError;
	}

String*		TextFile::ReadLine(int maxlen)
	// reads a single line of text from the file
	{
	// skip out if file is not open
	if (!file_isopen) return NULL;

	// create the string to be returned
	String*	tmpstr = new String(maxlen);

	char	buf;
	int 	cnt = 0;
	bool 	Finished = false;

	// read till we find an EOF, till we've read the max. chars,
	// or until we find a CR/LF combination

	while (!AtEOF() && (cnt<maxlen) && !Finished)
		{
		// read a single char into the buffer
		Read(&buf,1);

		// if we have found a Carriage-Return
		if (buf==13)
			{
			// read the Line-Feed char
			Read(&buf,1);

			// and we've now finished reading the line
			Finished = true;
			}
		else
			{
			// append the char to our string
			tmpstr->Append(buf);

			// get the next char
			cnt++;
			}
		}

	// return an error if we encountered EOF
	if (AtEOF()) { delete tmpstr; return NULL; }

	// everything is ok
	return tmpstr;
	}

int		TextFile::WriteLine(char* buf,int maxlen)
	{
	// skip out if file is not open
	if (!file_isopen) return file_FileNotOpen;

	int 	cnt = 0;
	bool 	Finished = false;

	// write till we find the end of the data, or till we've written the
	// max. line, or till we find an error.

	while ((buf[cnt]!=0x00) && (cnt<maxlen) && !Finished)
		{
		// write a single char to the buffer
		if (Write(&buf[cnt++],1) != file_NoError) Finished = true;
		}

	// if there wasn't an error
	if (!Finished)
		{
		char eoln[2] = { 13, 10 };

		Write(&eoln[0],1);
		Write(&eoln[1],1);
		}

	// everything is ok
	return file_NoError;
	}

