#ifndef __vbuffer_hpp
#define __vbuffer_hpp

#include "stddefns.hpp"
#include "sys.hpp"

// --------------------------------------------------------------------------
//
//	VIRTUAL BUFFER DEFINITIONS
//
// --------------------------------------------------------------------------

// the header for our virtual buffer

struct	grVBHeader
	{
	int			width;				// width of vb in pixels
	int			height;				// height of vb in pixels
	};

// the format of our generic "virtual buffer"

struct	grVirtualBuffer
	{
	grVBHeader	hdr;
	uchar 		data[];
	};

#define	grVB	grVirtualBuffer

// a global buffer for all screen drawing

extern	grVB*	grSCREEN;

// --------------------------------------------------------------------------
//
//	VIRTUAL BUFFER MACROS (BACKWARDS COMPATIBILITY & EASE OF READING)
//
// --------------------------------------------------------------------------

#define	grClearScreen(COLOR)	\
			grClearVirtual(grSCREEN,(COLOR))

// --------------------------------------------------------------------------
//
//	VB IMPLEMENTATION FUNCTIONS
//
// --------------------------------------------------------------------------


// ---  create a virtual buffer of the specified dimensions,
// 		optionally clearing to color (-1 = don't clear)
//
//		returns: NULL = not enough memory to create VB

grVB* 	grCreateVirtual(int x1,int y1,int x2,int y2,int clear = -1);


// ---	destroy a virtual buffer, resetting the pointer to NULL

#define grDestroyVirtual(BUF) 	\
			SYS_Free((BUF)); \
			(BUF) = NULL


// ---	returns the number of bytes required to store a virtual buffer
//		of the specified dimensions.

long	grVirtualSize(int x1,int y1,int x2,int y2);

// ---	returns the size of a given virtual buffer (0 if none exists)

long	grVirtualSize(grVB* vb);

// ---	clear a virtual buffer (to color 0 by default)

void 	grClearVirtual(grVB* vb,int color = 0);


#endif