//	-------------------------------------------------------------------------
//
//	WINDOW.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the definition of the basic "WINDOW" eventhandler
//
//	-------------------------------------------------------------------------

#ifndef __window_hpp
#define __window_hpp

//#include "event.hpp"
#include "handler.hpp"
#include "accessry.hpp"

// options for Select() (used internally)

#define evh_FIRST_ACCESSORY			0		// select
#define evh_LAST_ACCESSORY          1
#define	evh_CURRENT_ACCESSORY		2

// options for event handlers (onscreen reserves bits 8-15: 0x00000100 - 0x00008000)

#define evh_NO_KBD_RESPONSE 		0x00000100
#define evh_NO_MOUSE_RESPONSE		0x00000200

#define evh_MOVES					0x00001000
#define evh_RESIZES					0x00002000
#define	evh_MINIMIZES				0x00004000
#define evh_MAXIMIZES				0x00008000

//	-------------------------------------------------------------------------
//
//	CLASS:	WindowHandler
//
//	DESC:   This is the generic on-screen window used throughout any program
//
//	-------------------------------------------------------------------------

class OnScreenHandler : public EventHandler
	{
	protected:
		// --- handler region

		grRect	r;					// rectangular region taken up by handler
		grRect	inset_r;			// rectangular region inside borders, etc.

		// --- cursor functionality

		bool	AutoResetCursor;	// true-> handler auto. resets cursor on
									//		  mb_move commands
									// false-> assumes cursor reset manually

		// --- window display data

		bool	isBordered;			// true-> the window uses a bordered window

		int		bgcolor;			// the background color of the window
		int		active_bs;			// the active border style
		int		inactive_bs;		// the inactive border style


		bool	isBitmapped;		// true-> the window uses bitmaps for background
									// false-> the window uses background color & borders

		grImage* active_pic;		// the active bitmap
		grImage* inactive_pic;		// the inactive bitmap

		// --- accessory's storage

		Accessory* accessory[MAX_ACCESSORIES_IN_HANDLER];

		int		numAccessories;		// number of accessories contained in hanlder
		int		accessorySelected;  // accessory which is currently selected for
									// keyboard response
		bool	accessoriesAtMax;	// true if no more accessories can be added

		// --- flags

		bool	isOnScreen;			// true-> handler is "onscreen" & can interact
		bool	isAnimatable;		// true-> handler "animates" automatically

		bool 	isSelectable;		// if true, handler is selectable for input
		bool	isSelected;			// handler is currently selected for kbd/mouse input



	public:

		// --- constructor & destructor

		OnScreenHandler(int x1,int y1,int x2,int y2,	// the coords
						uchar bg,						// the bg color
						uchar active_border,			// and border styles
						uchar inactive_border,
						int* evhno,						// the event handler no.
						int options,                    // the window options
						int zfact);						// the z-factor

		OnScreenHandler(int x1,int y1,int x2,int y2,	// the coords
						grImage* active_bitmap,     	// the bitmaps
						grImage* inactive_bitmap,
						int* evhno,						// the event handler no.
						int options,					// the window options
						int zfact);						// the z-factor

		virtual ~OnScreenHandler();

		// --- implementation functions

		virtual	void 	Open();
		virtual	void 	Close();

		// --- function to check if mouse is inside handler

		virtual bool 	MouseInside(grPoint& p);

		// --- accessory functions

		Accessory* 		AddAccessory(Accessory* accessry);
		void 			DelAccessory();
		void			DelAllAccessories();

		// which accessory contains the cursor... -1 == no accessory selected
		int		WhichAccessorySelected(grPoint &mouse_pos);

		// enable/disable accessories
		void	ActivateAccessory(int acc);
		void	DeActivateAccessory(int acc);

		void	ActivateAllAccessories();
		void	DeActivateAllAccessories();

		// draw all accessories
		void 	DrawAllAccessories();

		void 	SelectNextAccessory();	// select the next (selectable) accessory
		void	SelectPrevAccessory();	// select the prev (selectable) accessory

		// --- drawing functions

		virtual void Select(int which);			// selects the window
		virtual void DeSelect();				// deselects the window

		virtual void Draw();					// draws the contents of the handler
		virtual	void Refresh();					// refreshs to the screen
		virtual void Animate();                 // animates the window (1 iteration only)

		// --- end drawing functions

		virtual void HandleEvent(Event &e);		// the handleevent function
	};

#endif

