#include "city.hpp"
#include "graphics.hpp"
#include "gamedefn.hpp"
#include "map.hpp"
#include "uidefn.hpp"

char	NUM_CITIES;
CityStruct	CITY[MAX_CITIES];


void	CreateRandomCities()
	{
	int i;

	// for all the cities in the game
	for (i=0;i<NUM_CITIES;i++)
		CreateNewCity(i);

	DBG(470,"Figuring closest cities");

	// figure out the 10 closest cities to each city
	for (i=0;i<NUM_CITIES;i++)
		{
		PlotCity(i);
		FindClosestCities(i);
		}
	}

void	CreateHistoricCities()
	{
	CreateRandomCities();
	}

// -------------------------------------------------------------------------
//
// CITY FUNCTIONS
//
// -------------------------------------------------------------------------

void	CreateNewCity(int citynum)
	{
	DBG(470,"Creating City #%d",citynum);
	int x,y,terrain;

	// create random city location on land
	do
		{
		x = random(map_width-12)+6;
		y = random(map_height-12)+6;
		terrain = GetTerrain(x,y);
		}
	while (terrain == sea);

	CITY[citynum].x = x;
	CITY[citynum].y = y;

	// and check that the city is at least 10 pixels away from the other
	// cities....

	bool all_cities_ok;

	do
		{
		all_cities_ok = true;
		// for all the cities previously created
		for (int j=0;j<citynum;j++)
			if (all_cities_ok)
				{
				// if the distance between the cities is too small
				if (DistBetweenCities(citynum,j)<10)
					{
					all_cities_ok = false;

					// choose another random location on land
					do
						{
						x = random(map_width-12)+6;
						y = random(map_height-12)+6;
						terrain = GetTerrain(x,y);
						}
					while (terrain == sea);

					CITY[citynum].x = x;
					CITY[citynum].y = y;
					}
				}
		}
	while (!all_cities_ok);

	// make sure our city connections are valid
	CITY[citynum].NumConnections = 0;
	for (int j=0;j<4;j++)
		{
		CITY[citynum].CityConnection[j] = NOT_CONNECTED;
		CITY[citynum].RouteConnection[j] = NOT_CONNECTED;
		}

	CITY[citynum].Size = random(10) + 1;
	CITY[citynum].Name = citynames[citynum % NUM_CITY_NAMES];

	CITY[citynum].ExtremeLeft 	= CITY[citynum].x - 1;
	CITY[citynum].ExtremeRight 	= CITY[citynum].x + 1;
	CITY[citynum].ExtremeTop 	= CITY[citynum].y - 1;
	CITY[citynum].ExtremeBottom = CITY[citynum].y + 1;
	}

int		ClosestCity(int x,int y)
	{
	int max_dist = 100000;
	int closest_city = 0;

	for (int citynum=0;citynum<NUM_CITIES;citynum++)
		{
		int dist = DistBetweenPts(x,y,CITY[citynum].x,CITY[citynum].y);
		if (dist<max_dist)
			{
			max_dist = dist;
			closest_city = citynum;
			}
		}

	return closest_city;
	}


int		ClosestConnectedCity(int x,int y)
	{
	int max_dist = 100000;
	int closest_city = 0;

	for (int citynum=0;citynum<NUM_CITIES;citynum++)
		if (CITY[citynum].NumConnections>0)
			{
			int dist = DistBetweenPts(x,y,CITY[citynum].x,CITY[citynum].y);
			if (dist<max_dist)
				{
				max_dist = dist;
				closest_city = citynum;
				}
			}

	return closest_city;
	}



int 	DistBetweenCities(int city1,int city2)
	{
	return DistBetweenPts(CITY[city1].x,CITY[city1].y,CITY[city2].x,CITY[city2].y);
	}


void	FindClosestCities(int startcity)
	{
	int city_dist[N_CLOSEST_CITIES];
	int i;
	int citynum;
	int last = N_CLOSEST_CITIES-1;

	for (i=0;i<N_CLOSEST_CITIES;i++)
		{
		city_dist[i] = 100000;
		CITY[startcity].ClosestCity[i] = -1;
		}

	// scan through all cities
	for (citynum=0;citynum<NUM_CITIES;citynum++)
		// ignoring the current city
		if (citynum!=startcity)
			{
			// figure out the distance between the chosen cities
			int dist = DistBetweenCities(startcity,citynum);

			// scan starting at the smallest distance
			bool finished = false;
			for (i=0;i<N_CLOSEST_CITIES;i++)
				if (!finished)
				// if we've found a smaller distance
				if (dist<city_dist[i])
					{
					// and we're not at the last element
					if (i!=last)
						// shuffle elements up
						for (int j=last;j>i;j--)
							{
							city_dist[j] = city_dist[j-1];
							CITY[startcity].ClosestCity[j] = CITY[startcity].ClosestCity[j-1];
							}

					// save this closer city
					city_dist[i] = dist;
					// save this closer city
					CITY[startcity].ClosestCity[i] = citynum;

					// and terminate our scan
					finished = true;
					}
			}
	}

void 	PlotCity(int citynum,bool set_overlaid)
	{
	if (set_overlaid)
		{
		// temporarily, just place a 3x3 grid of city icons
		SetOverlaid(CITY[citynum].x-1,CITY[citynum].y-1,FIRST_CITY + 0);
		SetOverlaid(CITY[citynum].x  ,CITY[citynum].y-1,FIRST_CITY + 1);
		SetOverlaid(CITY[citynum].x+1,CITY[citynum].y-1,FIRST_CITY + 2);
		SetOverlaid(CITY[citynum].x-1,CITY[citynum].y  ,FIRST_CITY + 3);
		SetOverlaid(CITY[citynum].x+1,CITY[citynum].y  ,FIRST_CITY + 2);
		SetOverlaid(CITY[citynum].x-1,CITY[citynum].y+1,FIRST_CITY + 1);
		SetOverlaid(CITY[citynum].x  ,CITY[citynum].y+1,FIRST_CITY + 3);
		SetOverlaid(CITY[citynum].x+1,CITY[citynum].y+1,FIRST_CITY + 0);
		}

	int cnum = citynum + CITY_START_OFFSET;
	int x = CITY[citynum].x;
	int y = CITY[citynum].y;

//	if (GetOverlaid(x-1,y-1)==NOT_OVERLAID) SetData(x-1,y-1,cnum);
//	if (GetOverlaid(x  ,y-1)==NOT_OVERLAID) SetData(x  ,y-1,cnum);
//	if (GetOverlaid(x+1,y-1)==NOT_OVERLAID) SetData(x+1,y-1,cnum);
//	if (GetOverlaid(x-1,y  )==NOT_OVERLAID) SetData(x-1,y  ,cnum);
//	if (GetOverlaid(x  ,y  )==NOT_OVERLAID) SetData(x  ,y  ,cnum);
//	if (GetOverlaid(x+1,y  )==NOT_OVERLAID) SetData(x+1,y  ,cnum);
//	if (GetOverlaid(x-1,y+1)==NOT_OVERLAID) SetData(x-1,y+1,cnum);
//	if (GetOverlaid(x  ,y+1)==NOT_OVERLAID) SetData(x  ,y+1,cnum);
//	if (GetOverlaid(x+1,y+1)==NOT_OVERLAID) SetData(x+1,y+1,cnum);

	SetData(x-1,y-1,cnum);
	SetData(x  ,y-1,cnum);
	SetData(x+1,y-1,cnum);
	SetData(x-1,y  ,cnum);
	SetData(x  ,y  ,cnum);
	SetData(x+1,y  ,cnum);
	SetData(x-1,y+1,cnum);
	SetData(x  ,y+1,cnum);
	SetData(x+1,y+1,cnum);

	PlotCityStation(citynum);
	}


void 	PlotCityStation(int citynum)
	{
	// place the station icon in the centre of the grid

// NOTE: possible code follows

	bool up = (CITY[citynum].CityConnection[UP] != NOT_CONNECTED);
	bool dn = (CITY[citynum].CityConnection[DN] != NOT_CONNECTED);
	bool lt = (CITY[citynum].CityConnection[LT] != NOT_CONNECTED);
	bool rt = (CITY[citynum].CityConnection[RT] != NOT_CONNECTED);

	int icon = 15;

	if (up) icon = 0;
	if (dn) icon = 1;
	if (lt) icon = 2;
	if (rt) icon = 3;
	if (up && dn) icon = 4;
	if (lt && rt) icon = 5;
	if (up && rt) icon = 6;
	if (rt && dn) icon = 7;
	if (dn && lt) icon = 8;
	if (lt && up) icon = 9;
	if (up && dn && rt) icon = 10;
	if (up && dn && lt) icon = 11;
	if (lt && rt && up) icon = 12;
	if (lt && rt && dn) icon = 13;
	if (up && dn && lt && rt) icon = 14;

	SetOverlaid(CITY[citynum].x,CITY[citynum].y,FIRST_STATION + icon);

	SetData(CITY[citynum].x,CITY[citynum].y,citynum + CITY_START_OFFSET);
	}




