//	-------------------------------------------------------------------------
//
//	CITY.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	These are all of the bits required for the cities
//
//	-------------------------------------------------------------------------

#ifndef __city_hpp
#define __city_hpp

#include "stddefns.hpp"
#include "gamedefn.hpp"
//#include "subsid.hpp"

// DEFINITIONS

#define	MAX_STATIONS	1	// the number of stations a city can have
#define	MAX_HOUSES		30	// the number of houses a city can have
#define	MAX_SUBSID		10	// the number of subsidiaries a city can have
#define	MAX_CONNECT		4	// the number of cities that can be connected to

#define N_CLOSEST_CITIES	20

#define	STATION			0   // the type of station in the city
#define	TERMINAL		1


#define NOT_CONNECTED			-1

struct	CityStruct
	{
	char*	Name;			// the name of the city
	char*	Abbrev;			// the abbreviation of the city name

	long	Size;			// the size of the city

	char	StationType;	// the type of station for the city

	int		x;				// the x position of the city
	int		y;				// the y position of the city

	int		ExtremeLeft;	// the boundaries of the city including all
	int		ExtremeRight;	// houses and subsidiaries, etc.
	int		ExtremeTop;
	int		ExtremeBottom;

	int		PeopleWaiting;	// the number of people waiting on platform
	int		MailWaiting;	// the amount of mail waiting on platform
	int		RawWaiting;		// the amount of raw materials waiting on platform
	int		GoodsWaiting;	// the amount of manufactured goods waiting on platform

	int		PeopleWanted;
	int		MailWanted;
	int 	RawWanted;
	int		GoodsWanted;

	char	NumConnections;	// the number of cities connected to
	char	NumHouses;		// the number of houses in this city
	char	NumSubsidiaries;// the number of subsidiaries in this city

	int		CityConnection[MAX_CONNECT];	// the cities that are connected to
	int		RouteConnection[MAX_CONNECT];	// the route numbers of the connections

	int		ClosestCity[N_CLOSEST_CITIES];	// the N closest cities

//	SubsidStruct	House[MAX_HOUSES];		// storage of the individual houses
//	SubsidStruct	Subsid[MAX_SUBSID]; 	// storage of the individual subsidiaries
	};



// GLOBALS

extern	char		NUM_CITIES;			// the actual number of cities in game
extern	CityStruct	CITY[MAX_CITIES];	// storage of all cities data


// FUNCTIONS

void	CreateRandomCities();
void	CreateHistoricCities();

void	CreateNewCity(int citynum);

int		ClosestCity(int x,int y);				// returns closest city to map x,y position
int		ClosestConnectedCity(int x,int y);		// returns closest connected city to map x,y position

int		DistBetweenCities(int city1, int city2);// returns distance between 2 cities
void	FindClosestCities(int startcity);		// finds the closest cities to 'startcity'

void	PlotCity(int citynum,bool set_overlaid = true);
										// plots a specified city...
										// if set_overlaid==false, doesn't set overlaid icon #

void	PlotCityStation(int citynum);   // plots a specified city station

#endif
