//
//	GAMEDEFNS.HPP
//
//	These are the definitions required by the game. The definitions
//	include Event-Handler-Numbers, global command events, screen layout
//	bits & pieces, etc.

#ifndef __gamedefn_hpp
#define __gamedefn_hpp

#include "stddefns.hpp"
#include "graphics.hpp"

#define GAMETEST

//  NOTES:

// 	cities		- 	char	... MAX 254 CITIES
//	subsid		- 	char	... MAX 254 SUBSID
//	houses		- 	char	... MAX 254 HOUSE
//	routes		- 	int		... MAX ??? ROUTES
//	railroads	-	char	... MAX 255 RRS
//	players		-	char	...	MAX 255 PLAYERS

#define	MIN_PLAYERS	1
#define	MIN_CITIES	30
#define	MIN_RRS		30
#define	MIN_ROUTES	10
#define	MIN_TRAINS	0

#define	MAX_PLAYERS		20		// the maximum number of players in the game
#define	MAX_CITIES		100		// the maximum number of cities in the game
#define	MAX_RRS			100		// the maximum number of railroads in the game
#define	MAX_ROUTES		200		// the maximum number of routes in the game
#define	MAX_TRAINS		1000	// the maximum number of trains in the game

#define	MAX_HISTORY	20

// SCREEN LAYOUT

// FUNNY TASKS

extern	int		evhno_SetupScreen;	// game setup screen

// BACKGROUND TASKS

extern 	int		evhno_System;		// used for engine testing
extern	int		evhno_AI;			// the artificial-intelligence
extern	int 	evhno_GameProcessor;// raw game processing area
extern	int 	evhno_Interface;	// controls the user-interface (UI)
extern	int		evhno_Network;		// the network interface

// ONSCREEN TASKS

extern	int		evhno_StatusBar;	// the status bar
extern	int		evhno_PlayerActionsBar;	// the player actions bar
extern	int		evhno_RRActionsBar;		// the railroad actions bar
extern	int		evhno_PlayerInfoBar;	// the player info selection bar
extern	int		evhno_RRInfoBar;		// the rr info selection bar
extern	int		evhno_MapWin;		// the map window
extern	int		evhno_OpponentInfo;	// the opponent information window
extern	int		evhno_MessageWin;	// the message window

extern	int		evhno_WhichRR;		// the "which rr?" window
extern	int		evhno_WhichRoute;	// the "which route?" window
extern	int		evhno_WhichCity;	// the "which city?" window
extern	int		evhno_HowMany;		// the "howmany?" window
extern	int		evhno_TrainConfig;	// the "configure train" window

extern	int		evhno_MarketBox;	// the stock/bond market info box

// PLAYER ACTIONS

#define cmdBuyStocks			1101
#define cmdSellStocks			1102
#define cmdBuyBonds				1103
#define cmdSellBonds			1104
#define cmdGetLoan				1105
#define cmdRepayLoan			1106
#define	cmdSpyOnPlayer			1107
#define	cmdDealWithPlayer		1108
#define	cmdSpyOnRR				1109
#define cmdControlRR			1110
#define cmdResignControlRR		1111
#define	cmdGiveRRLoan			1112
#define cmdRecallRRLoan			1113
#define cmdIncorporateRR		1114
#define cmdMergeRRs				1115
#define cmdActForRR				1116

// RAILROAD ACTIONS

#define cmdRR_BuyStocks			1201
#define cmdRR_SellStocks		1202
#define cmdRR_BuyBonds			1203
#define cmdRR_SellBonds			1204
#define cmdRR_RepayBank			1205
#define cmdRR_RepayDelinquent	1206
#define cmdRR_SetDividend		1207
#define cmdRR_RaiseEfficiency	1208
#define	cmdRR_BuySubsidiary		1209
#define	cmdRR_SellSubsidiary	1210
#define	cmdRR_BuyTrain			1211
#define	cmdRR_SellTrain			1212
#define	cmdRR_SurveyRoute		1213
#define	cmdRR_SurveyRoute_1		1214
#define cmdRR_CharterRoute		1215
#define cmdRR_BuildTrack		1216
#define cmdRR_ActForPlayer		1217

// PLAYER INFO SELECTIONS

#define	cmdStockMarket			1150
#define	cmdBondMarket			1151
#define	cmdBank					1152
#define	cmdSpyReport			1153
#define	cmdPlayerDealings		1154
#define	cmdRRReport				1155


//
// GENERAL COMMANDS

#define	cmdGameTick					1000
#define	cmdDayTick					1001
#define	cmdMonthTick				1002
#define	cmdQuarterTick				1003
#define	cmdYearTick					1004

#define cmdDateChanged      		1010
#define cmdPlayerStatusChanged		1011
#define cmdPlayerModeChanged		1012
#define cmdEconomyChanged			1013

#define	cmdCreateNewRoute			2003

#define	cmdRedrawMap				4000
#define	cmdRefreshMap				4001
#define	cmdRedrawMapRegion			4002
#define	cmdRefreshMapRegion			4003
#define	cmdRedrawScreenRegion		4004
#define	cmdRefreshScreenRegion		4005

#define	cmdExternalRefresh			4003
#define	cmdRefreshMapSection		4004

#define	cmdMessageTimeout			5000

// interface
#define	cmdDoPlayerAction			6000
#define	cmdDoPlayerResponse			6001
#define	cmdDoRRAction				6002
#define	cmdDoRRResponse				6003

#define	cmdHowManyOK				6010
#define	cmdHowManyCANCEL			6011
#define	cmdWhichRROK				6012
#define	cmdWhichRRCANCEL			6013
#define	cmdWhichRouteOK				6014
#define	cmdWhichRouteCANCEL			6015
#define	cmdWhichCityOK				6016
#define	cmdWhichCityCANCEL			6017
#define	cmdTrainConfigOK			6018
#define	cmdTrainConfigCANCEL		6019

#define	cmdSetPlayerMode			6200
#define	cmdSetRRMode				6201

#define	cmdOpenQuery				6100
#define	cmdCloseQuery				6101
#define	cmdOpenInfoBox              6102
#define	cmdCloseInfoBox				6103
#define	cmdOpenMessageWindow		6104
#define	cmdCloseMessageWindow		6105

// networking
#define	cmdSendToServer				9000
#define	cmdSendToGameProcessor		cmdSendToServer
#define	cmdSendToClient				9001
#define	cmdSendToPlayer				cmdSendToClient
#define	cmdSentPacket				9002
#define	cmdReceivedPacket			9003

// PLAYER MESSAGES

// buy stocks
#define	msgSentBuyStocksMessenger	10000
#define	msgNoStocksAvailable		10001
#define	msgCantAffordAnyStocks		10002
#define	msgCantBuyAllStocks			10003
#define	msgCantAffordAllStocks		10004
#define	msgBoughtStocks				10005

// sell stocks
#define	msgSentSellStocksMessenger	10010
#define	msgSoldStocks				10015

// buy bonds
#define	msgSentBuyBondsMessenger	10020
#define	msgBoughtBonds				10025

// sell bonds
#define	msgSentSellBondsMessenger	10030
#define	msgSoldBonds				10035

// get loan
#define	msgSentGetLoanMessenger		10040

// repay loan
#define	msgSentRepayLoanMessenger	10050

// control rr
#define	msgSentControlRRMessenger	10100
#define	msgControlledRR				10101
#define	msgCantControlRR			10102

// resign control rr
#define	msgResignedControl			10103
#define	msgCantResignControl		10104

#define	msgNotPresident				10111

// RAILROAD MESSAGES

// buy trains
#define	msgSentBuyTrainMessenger		10500
#define	msgCantBuyTrain					10501
#define	msgBoughtTrain					10502

// survey route
#define	msgSentSurveyRouteMessenger		10400
#define	msgCantSurveyRoute				10401
#define	msgSurveyedRoute				10402

// charter route
#define	msgSentCharterRouteMessenger	10200
#define	msgCharteredRoute				10201
#define	msgAlreadyBuildingTrack			10202
#define	msgCharteredByOther				10203
#define	msgFullyBuiltRoute				10204
#define	msgNotAdjoiningRoute			10205

// build track
#define	msgSentBuildTrackMessenger		10300
#define	msgTrackFullyBuilt				10301
#define	msgFinishedBuildingSection		10302
#define	msgBuildingInProgress			10303

// miscellaneous
#define	msgTrainMoved					11000
#define	msgTrainArriving				11001
#define	msgTrainDeparting				11002

// PLAYER MODES

#define	MPLAYER          	0
#define	MRAILROAD			1

// OTHER BITS

extern	bool	RANDOM_GAME;

extern	long	CURRENT_DATE;
extern	char	CURRENT_ECONOMY;

extern	char	LOAN_RATE[4];

extern	int		GAME_TICKER;
extern	int		GAME_DELAY;

// FONTS

extern	int	SYSTEMFONT;	// used for debugging purposes
extern	int	BASICFONT;	// the 'all-the-time' font
extern	int	TITLEFONT;	// used when displaying pretty titles, etc.
extern	int	NEWSPAPERFONT;	// meant for newspaper headings, etc.
extern	int	TINYFONT;	// used for stockmarket titles, etc.
extern	int	HUGEFONT;	// used for BIG titles, etc.
extern	int	SETUPFONT;	// used by the setup program

// COLORS

#define	TEXT_COLOR	grWHITE
#define	BG_COLOR	grDARKGRAY

// MISCELLANEOUS

#define	NUM_HISTORIC_RRS		50
#define	NUM_HISTORIC_PLAYERS	25
#define	NUM_CITY_NAMES			51

extern	char*	railroadnames[];
extern	char*	citynames[];
extern	char*	playernames[];
extern	char*   econstr[];
extern	char*	monthstr[];
extern	char*	daystr[];
extern	char*	dayextstr[];
extern	char	daysinmonth[];

#define	SINGLE_PLAYER		0
#define	SERIAL_NETWORK		1
#define	IPX_NETWORK			2
#define	TCP_NETWORK			3


struct	ActionHeaderStruct
	{
	int		Action;
	union
		{
		int		Player;
		int 	RR;
		};
	};

struct	ResponseHeaderStruct
	{
	int		Action;
	int		Response;
	union
		{
		int		Player;
		int		RR;
		};
	};

#define	MAX_RESPONSES			32

extern	ActionHeaderStruct		ACTION_HEADER;

extern	ResponseHeaderStruct	RESPONSE_HEADER[MAX_RESPONSES];
extern	int						NEXT_RESPONSE;


#endif
