//	-------------------------------------------------------------------------
//
//	MAP.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	These are all of the bits required for the map
//
//	-------------------------------------------------------------------------

#ifndef __map_hpp
#define __map_hpp

#include "stddefns.hpp"
#include "gamedefn.hpp"

#include "city.hpp"
#include "route.hpp"

// DEFINITIONS

// these are the template definitions
#define		template_sea	1
#define		template_land	2

// these are the terrain icon definitions
//#include "terrain.def"
#define 	sea			0
#define 	grass			1
#define 	forest			2
#define		mountains		3

// these are the different responses from WhatsAt(x,y);
#define		MTERRAIN		0
#define		MCITY			1
#define		MROUTE			2
#define		MTRAIN			3

// these are the overlaid icon definitions

#define		FIRST_STATION		0
#define		LAST_STATION		15

#define     FIRST_SURVEYED		16
#define		LAST_SURVEYED		37

#define		FIRST_BUILT			38
#define		LAST_BUILT			59

#define		FIRST_HIGHLIGHTED	60
#define		LAST_HIGHLIGHTED	81

#define		FIRST_CITY			82	//60
#define		LAST_CITY			89	//67

#define		FIRST_SUBSIDIARY	47
#define		LAST_SUBSIDIARY		56

#define		FIRST_TRAIN			57
#define		LAST_TRAIN			58

#define		LAST_ICON			46	// last icon number + 1

#define		NOT_OVERLAID		255

// these are used to distinguish bits of the map

#define		TERRAIN_ELEMENT		0

#define		CITY_START_OFFSET	(TERRAIN_ELEMENT + 1)
#define		CITY_END_OFFSET     (CITY_START_OFFSET + MAX_CITIES)

#define		ROUTE_START_OFFSET	(CITY_END_OFFSET+1)
#define		ROUTE_END_OFFSET	(ROUTE_START_OFFSET + MAX_ROUTES)

// COLOR LOOKUP TABLES

extern	char				terrain_color_table[256];
extern	char				overlaid_color_table[256];

// BASIC MAP FUNCTIONS


void	InitMaps();		// create space for all the required maps
void	ExitMaps();		// destroy all maps

void	SetTerrain(int x,int y,char terrain);
void	SetOverlaid(int x,int y,char overlaid);
void	SetTerrainHeight(int x,int y,char height);
void	SetData(int x,int y,int data);

char	GetTerrain(int x,int y);
char	GetOverlaid(int x,int y);
char	GetTerrainHeight(int x,int y);
int		GetData(int x,int y);

#define	GetTerrainNumber		GetTerrain
#define	GetCityNumber(X,Y) 		(GetData((X),(Y)) - CITY_START_OFFSET)
#define	GetRouteNumber(X,Y)		(GetData((X),(Y)) - ROUTE_START_OFFSET)
//#define	GetTrainNumber(X,Y)		(GetData((X),(Y)) - TRAIN_START_OFFSET)

int		WhatsAt(int x,int y);
int		DistBetweenPts(int x1,int y1,int x2,int y2);

// MAP GENERATION

void	CreateRandomMap();		// create a random map
void	CreateHistoricMap();

void 	CreateBaseTerrain();
void	CreateRandomTerrain();	// create random terrain feature

void	CreateRandomMountains();// create random mountain ranges
void	CreateRandomForests();	// create random forests
void	CreateRandomRivers();	// create rivers

#endif
