//	-------------------------------------------------------------------------
//
//	PLAYER.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	These are the bits and pieces for players
//
//	-------------------------------------------------------------------------

#ifndef __player_hpp
#define __player_hpp

#include "gamedefn.hpp"
#include "railroad.hpp"

// DEFINITIONS

#define	MAX_LOANS		10	// the player can have only 10 loans at once

#define	NOT_IN_GAME		0	// the player is not active in game
#define	HUMAN			1	// is a human player
#define	COMPUTER		2	// is a computer player


// structures used for the different player actions

struct	PL_BuyStocksStruct
	{
	char	Cnt;
	int		RR;
	int		Amount;
	};

#define	PL_SellStocksStruct		PL_BuyStocksStruct
#define	PL_BuyBondsStruct		PL_BuyStocksStruct
#define	PL_SellBondsStruct		PL_BuyStocksStruct

struct	PL_GetLoanStruct
	{
	char	Cnt;
	int		Amount;
	int		Rate;
	};

#define	PL_RepayLoanStruct		PL_GetLoanStruct

struct	PL_SpyOnPlayerStruct
	{
	char	Cnt;
	int		PlayerNum;
	};

struct	PL_DealWithPlayerStruct
	{
	char	Cnt;
	char	OtherPlayer;
	char	Deal;
	};


struct	PL_ControlRRStruct
	{
	char	Cnt;
	int		RR;
	};

#define	PL_ResignControlRRStruct	PL_ControlRRStruct

struct	PL_GiveRRLoanStruct
	{
	char	Cnt;
	int		RR;
	int		Amount;
	};

#define	PL_RecallRRLoanStruct		PL_GiveRRLoanStruct

struct	PL_IncorporateRRStruct
	{
	char	Cnt;
	char*	Name;
	char*	Abbrev;
	int		SeedAmount;
	};

struct	PL_MergeRRsStruct
	{
	char	Cnt;
	int		RR1;
	int		RR2;
	};

// structure used to store player data

struct	PlayerStruct
	{
	int		Status;					// the players' status
	char* 	Name;					// the players' name

	long 	Cash;					// current cash assets, 1000s

	int		Stocks[MAX_RRS];		// storage of stocks held
	int		Bonds[MAX_RRS];			// storage of bonds held

	char	TotalStocks;			// total rr stocks owned
	char	TotalBonds;				// total rr bonds owned

	long	LoanDue[MAX_LOANS];		// the date the loan is due
	long	LoanAmount[MAX_LOANS];	// the amount borrowed
	char	LoanRate[MAX_LOANS];	// the rate borrowed at

	char	TotalLoans;				// total bank loans

	char	TotalSubsid;			// total subsidiaries owned
	char	TotalPres;				// total presidencies

	long	TotalAssets;			// total of all assets owned
	long	TotalDebt;				// total of all debt

	bool	ActingForRR;			// the player is acting for a railroad

	// start of storage for actions that are currently pending

	char	QuickActionsPending;
	char	DailyActionsPending;
	char	MonthlyActionsPending;

	PL_BuyStocksStruct			BUYSTOCKS;
	PL_SellStocksStruct			SELLSTOCKS;
	PL_BuyBondsStruct			BUYBONDS;
	PL_SellBondsStruct			SELLBONDS;
	PL_GetLoanStruct			GETLOAN;
	PL_RepayLoanStruct			REPAYLOAN;
	PL_SpyOnPlayerStruct		SPYONPLAYER;
	PL_DealWithPlayerStruct		DEALWITHPLAYER;
	PL_ControlRRStruct			CONTROLRR;
	PL_ResignControlRRStruct	RESIGNCONTROLRR;
	PL_GiveRRLoanStruct			GIVERRLOAN;
	PL_RecallRRLoanStruct		RECALLRRLOAN;
	PL_IncorporateRRStruct		INCORPORATERR;
	PL_MergeRRsStruct			MERGERRS;

	// end of storage for actions that are currently pending

	int		NETWORK_TYPE;
	};

extern	char			NUM_PLAYERS;
extern	PlayerStruct    PLAYER[MAX_PLAYERS];


void	CreateRandomPlayers();
void	CreateHistoricPlayers();

int		player_BuyStocks(int player);
int		player_SellStocks(int player);
int		player_BuyBonds(int player);
int		player_SellBonds(int player);

int		player_GetLoan(int player,int howmuch,int rate,int period);
int		player_RepayLoan(int player,int loannum);

int		player_SpyOnPlayer(int player,int other_player);
int		player_DealWithPlayer(int player,int other_player,int deal);

int		player_SpyOnRR(int player,int rr);

int		player_ControlRR(int player);
int		player_ResignControlRR(int player,int rr);

int		player_GiveRRLoan(int player,int rr,int howmuch);
int		player_RecallRRLoan(int player,int rr,int howmuch);

int 	player_IncorporateRR(int player,char* rrname,int starting_cash);
int		player_MergeRRs(int player,int rr0,int rr1);

int		player_ActForRR(int player,int rr);

#endif
