//	-------------------------------------------------------------------------
//
//	GAMEPROC.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the Game Processor.
//
//	-------------------------------------------------------------------------

#include "gameproc.hpp"
#include "eventmgr.hpp"
#include "gamedefn.hpp"

#include "player.hpp"
#include "railroad.hpp"
#include "route.hpp"
#include "map.hpp"

#include "dos.h"

GameProcessor::GameProcessor()
	: BackgroundTask(&evhno_GameProcessor,0)
	{
	ticker_cnt = 0;
	player = 0;
	rr = 0;
	}

void	GameProcessor::UpdateTime()
	{
	// count how many times we've been called
	ticker_cnt++;
	if (ticker_cnt<24) return;

	ticker_cnt = 0;

	int brdcast;

	// we've been called enough times, its a new day
	int	dayofweek,date,month,year;
	UNPACKDATE(CURRENT_DATE,dayofweek,date,month,year);

	// increment our day counter
	dayofweek++;
	if (dayofweek>6) dayofweek = 0;

	// and our date
	date++;
	brdcast = cmdDayTick;

	// checking for end of month
	if (date>=daysinmonth[month])
		{
		// check for leap-year (simple method!)
		if ((month==1) && ((year%4)==0) && (date==27))
			{
			PACKDATE(CURRENT_DATE,dayofweek,date,month,year);
			return;
			}

		// increment our month
		date = 0;
		month++;
		brdcast = cmdMonthTick;

		// check for quarter
		if ((month % 3) == 0)
			brdcast = cmdQuarterTick;

		// and check for end-of-year
		if (month>11)
			{
			month = 0;
			year++;
			brdcast = cmdYearTick;

			// program ends if year > 1930
			if (year>1930)
				{
				EVENTMGR->GenCommandEvent(0, cmdExitProgram, NULL);
				}
			}
		}

	PACKDATE(CURRENT_DATE,dayofweek,date,month,year);

	EVENTMGR->GenBroadcastEvent(brdcast,NULL);
	}


void   	GameProcessor::UpdateRRHistory()
	{
	for (int rr=0;rr<NUM_RRS;rr++)
		{
		int hist = RAILROAD[rr].CurrentHistory;
		int hist1 = hist;

		hist1++;
		if (hist1>=MAX_HISTORY) hist1 = 0;

		RAILROAD[rr].CurrentHistory = hist1;
		RAILROAD[rr].StockPrice[hist1] = RAILROAD[rr].StockPrice[hist];
		RAILROAD[rr].BondPrice[hist1] = RAILROAD[rr].BondPrice[hist];
		}
	}

long	RunTrains_Cnt = 0;

void	GameProcessor::RunTrains()
	{
	// run next batch of trains
//	DBG(0,"RUNNING TRAINS %d",RunTrains_Cnt++);

	for (int train=0;train<NUM_TRAINS;train++)

		switch (TRAIN[train].Status)
			{
			case MOVING_FORWARDS:
			case MOVING_BACKWARDS:

				train_DoMove(train);
				break;

			case ARRIVING:

				train_DoArrival(train);
				break;

			case DEPARTING:

				train_DoDeparture(train);
				break;
			}
	}

//long	ThinkForAI_Cnt = 0;

void	GameProcessor::ThinkForAI()
	{
	// process some AI
//	DBG(10,"THINKING FOR AI %d",ThinkForAI_Cnt++);
	}

void	GameProcessor::UpdateCities()
	{
	// calc city supply/demand
	}

void	GameProcessor::CheckBankLoans()
	{
	// check if any bank loans are due
	}

void	GameProcessor::CheckNewsReports()
	{
	// check if there's any news
	}

void	GameProcessor::SendPlayerResult(int player,int action,int result)
	{
	RESPONSE_HEADER[NEXT_RESPONSE].Player = player;
	RESPONSE_HEADER[NEXT_RESPONSE].Action = action;
	RESPONSE_HEADER[NEXT_RESPONSE].Response = result;

	EVENTMGR->GenCommandEvent(evhno_Interface,cmdDoPlayerResponse,(void*) NEXT_RESPONSE);

	NEXT_RESPONSE++;
	if (NEXT_RESPONSE==MAX_RESPONSES) NEXT_RESPONSE = 0;
	}



void	GameProcessor::SendRRResult(int rr,int action,int result)
	{
	RESPONSE_HEADER[NEXT_RESPONSE].RR 	    = rr;
	RESPONSE_HEADER[NEXT_RESPONSE].Action   = action;
	RESPONSE_HEADER[NEXT_RESPONSE].Response = result;

	EVENTMGR->GenCommandEvent(evhno_Interface,cmdDoRRResponse,(void*) NEXT_RESPONSE);
	NEXT_RESPONSE++;
	if (NEXT_RESPONSE==MAX_RESPONSES) NEXT_RESPONSE = 0;
	}



void	GameProcessor::PerformQuickActions()
	{
	int result;

	// perform an action for the next player (quick in, quick out)
	player++;
	if (player==NUM_PLAYERS) player = 0;

	// if there are any actions waiting to be processed
	if (PLAYER[player].QuickActionsPending != 0)
		{
		// if buystocks is set
		if (PLAYER[player].BUYSTOCKS.Cnt>0)
			{
			// decrease the counter
			PLAYER[player].BUYSTOCKS.Cnt--;
			// and if it's zero, do the action
			if (PLAYER[player].BUYSTOCKS.Cnt==0)
				{
				PLAYER[player].QuickActionsPending--;

				result = player_BuyStocks(player);

				// and return the result to the appropriate interface
				SendPlayerResult(player,cmdBuyStocks,result);
				}
			}

		if (PLAYER[player].SELLSTOCKS.Cnt>0)
			{
			PLAYER[player].SELLSTOCKS.Cnt--;
			if (PLAYER[player].SELLSTOCKS.Cnt==0)
				{
				PLAYER[player].QuickActionsPending--;

				result = player_SellStocks(player);
				SendPlayerResult(player,cmdSellStocks,result);
				}
			}

		if (PLAYER[player].BUYBONDS.Cnt>0)
			{
			PLAYER[player].BUYBONDS.Cnt--;
			if (PLAYER[player].BUYBONDS.Cnt==0)
				{
				PLAYER[player].QuickActionsPending--;

				result = player_BuyBonds(player);
				SendPlayerResult(player,cmdBuyBonds,result);
				}
			}

		if (PLAYER[player].SELLBONDS.Cnt>0)
			{
			PLAYER[player].SELLBONDS.Cnt--;
			if (PLAYER[player].SELLBONDS.Cnt==0)
				{
				PLAYER[player].QuickActionsPending--;

				result = player_SellBonds(player);
				SendPlayerResult(player,cmdSellBonds,result);
				}
			}
		}

	// perform an action for the next rr
	rr = 0;

			if (RAILROAD[rr].BUILDTRACK.Cnt>0)
				{
				RAILROAD[rr].BUILDTRACK.Cnt--;
				if (RAILROAD[rr].BUILDTRACK.Cnt==0)
					RAILROAD[rr].DailyActionsPending--;

				result = railroad_BuildTrack(rr);
				if (result!=msgBuildingInProgress)
					SendRRResult(rr,cmdRR_BuildTrack,result);
				}

	}

void	GameProcessor::PerformDailyActions()
	{
	int result;

//	DBG(30,"Performing Daily Player Actions");

	// perform any actions for all players
	int first_player = player;
	do
		{
		player++;
		if (player==NUM_PLAYERS) player = 0;

		if (PLAYER[player].DailyActionsPending != 0)
			{
			if (PLAYER[player].CONTROLRR.Cnt>0)
				{
				PLAYER[player].CONTROLRR.Cnt--;
				if (PLAYER[player].CONTROLRR.Cnt==0)
					{
					PLAYER[player].DailyActionsPending--;

					result = player_ControlRR(player);
					SendPlayerResult(player,cmdControlRR,result);
					}
				}
			}
		}
	while (player!=first_player);

//	DBG(30,"Performing Daily Railroad Actions");

	// perform an action for the next rr
	int first_rr = rr;

	do
		{
		rr++;
		if (rr==NUM_RRS) rr = 0;

		if (RAILROAD[rr].DailyActionsPending != 0)
			{
			if (RAILROAD[rr].BUYTRAIN.Cnt>0)
				{
				RAILROAD[rr].BUYTRAIN.Cnt--;
				if (RAILROAD[rr].BUYTRAIN.Cnt==0)
					{
					RAILROAD[rr].DailyActionsPending--;

					result = railroad_BuyTrain(rr);
					SendRRResult(rr,cmdRR_BuyTrain,result);
					}
				}

			if (RAILROAD[rr].CHARTERROUTE.Cnt>0)
				{
				RAILROAD[rr].CHARTERROUTE.Cnt--;
				if (RAILROAD[rr].CHARTERROUTE.Cnt==0)
					{
					RAILROAD[rr].DailyActionsPending--;

					result = railroad_CharterRoute(rr);
					SendRRResult(rr,cmdRR_CharterRoute,result);
					}
				}

			if (RAILROAD[rr].BUILDTRACK.Cnt>0)
				{
				RAILROAD[rr].BUILDTRACK.Cnt--;
				if (RAILROAD[rr].BUILDTRACK.Cnt==0)
					RAILROAD[rr].DailyActionsPending--;

//				DBG(30 + rr * 10,"Building Track for railroad %d",rr);
				result = railroad_BuildTrack(rr);
				if (result!=msgBuildingInProgress)
					SendRRResult(rr,cmdRR_BuildTrack,result);
				}

			if (RAILROAD[rr].SURVEYROUTE.Cnt>0)
				{
				RAILROAD[rr].SURVEYROUTE.Cnt--;
				if (RAILROAD[rr].SURVEYROUTE.Cnt==0)
					{
					RAILROAD[rr].DailyActionsPending--;

					result = railroad_SurveyRoute(rr);
					SendRRResult(rr,cmdRR_SurveyRoute,result);
					}
				}
			}
		}
	while (rr != first_rr);
	}

void GameProcessor::HandleEvent(Event &e)
	{
	switch (e.what)
		{
		case ev_timer:

			switch (e.timer.event)
				{
				case cmdGameTick:

					RunTrains();
					ThinkForAI();

					UpdateCities();
					UpdateTime();

					// 9 to 5 only
					if ((ticker_cnt>=9) && (ticker_cnt<=17))
						PerformQuickActions();
					break;
				}

		case ev_broadcast:

			switch (e.broadcast.event)
				{
				case cmdDayTick:

					CheckBankLoans();
					UpdateRRHistory();

					PerformDailyActions();
					break;

				case cmdMonthTick:

					CheckNewsReports();
					break;

				case cmdQuarterTick:

					break;

				case cmdYearTick:

					break;
				}
			break;
		}
	}

