#include "railroad.hpp"
#include "graphics.hpp"
#include "gamedefn.hpp"
#include "player.hpp"
#include "economy.hpp"
#include "route.hpp"
#include "map.hpp"
#include "eventmgr.hpp"

char	NUM_RRS;
RailroadStruct	RAILROAD[MAX_RRS];

void	CreateRandomRailroads()
	{
	// create random railroads
	for (int rr=0;rr<NUM_RRS;rr++)
		{
		DBG(470,"Creating random railroad %d",rr);

		RAILROAD[rr].Status 	= NOT_FLOATED;
		RAILROAD[rr].Name 		= railroadnames[(rr * 2) % NUM_HISTORIC_RRS];
		RAILROAD[rr].Abbrev 	= railroadnames[(rr * 2 + 1) % NUM_HISTORIC_RRS];
		RAILROAD[rr].President 	= NO_PRESIDENT;

		RAILROAD[rr].Cash 		= random(20)*1000 + random(999) + 500;
		RAILROAD[rr].BankLoan 	= 0;
		RAILROAD[rr].PresLoan 	= 0;

		RAILROAD[rr].StocksInHand 	= random(40)*5 + random(5) + 1;
		RAILROAD[rr].BondsInHand 	= random(10) + 1;

		RAILROAD[rr].StocksOnMarket = random(RAILROAD[rr].StocksInHand)/2;
		RAILROAD[rr].BondsOnMarket 	= random(RAILROAD[rr].BondsInHand)/2;

		if (RAILROAD[rr].StocksOnMarket) STOCKS_ON_MARKET++;
		if (RAILROAD[rr].BondsOnMarket) BONDS_ON_MARKET++;

		RAILROAD[rr].StocksInPlayer = 0;
		RAILROAD[rr].BondsInPlayer 	= 0;

		char StockPrice 	= 10 + random(10);
		char BondPrice 		= random(10)*100 + 1;

		for (int history=0;history<MAX_HISTORY;history++)
			{
			RAILROAD[rr].StockPrice[history] = StockPrice;
			RAILROAD[rr].BondPrice[history] = BondPrice;
			}

		RAILROAD[rr].CurrentHistory = 0;

		RAILROAD[rr].Efficiency 	= 50 + random(10)*5;
		RAILROAD[rr].DividendRate 	= random(25) + 1;
		RAILROAD[rr].RevenueYTD 	= 0;
		RAILROAD[rr].CostsYTD 		= 0;

		RAILROAD[rr].TurnsNotPaid 	= 0;

		for (int reorg=0;reorg<MAX_REORG;reorg++)
			RAILROAD[rr].BondInterest[reorg] = 0;

		RAILROAD[rr].CharteredRoute = -1;

		RAILROAD[rr].TrainsRunning  = 0;
		RAILROAD[rr].MaxTrainsRunning = 5;
		}
	}

void	CreateHistoricRailroads()
	{
	CreateRandomRailroads();
	}


int		railroad_BuyTrain(int rr)
	{
	if (RAILROAD[rr].TrainsRunning>=RAILROAD[rr].MaxTrainsRunning)
		return msgCantBuyTrain;

	if (NUM_TRAINS>=MAX_TRAINS) return msgCantBuyTrain;

	// set the train bits
	CreateNewTrain(rr,PASSENGER_TRAIN,&RAILROAD[rr].BUYTRAIN.CheckPoint[0],
					RAILROAD[rr].BUYTRAIN.NumCheckPoints );

	RAILROAD[rr].TrainsRunning++;

	return msgBoughtTrain;
	}

int		railroad_SurveyRoute(int rr)
	{
	int city1 = RAILROAD[rr].SURVEYROUTE.City1;
	int city2 = RAILROAD[rr].SURVEYROUTE.City2;

	if (CreateNewRoute(city1,city2)) return msgSurveyedRoute;
	return msgCantSurveyRoute;
	}

int		railroad_CharterRoute(int rr)
	{
	int route = RAILROAD[rr].CHARTERROUTE.Route;

	// skip if the route is not owned by the rr in question & it _is_ owned
	if ((ROUTE[route].Owner != rr) &&
		(ROUTE[route].Owner != -1))
		return msgCharteredByOther;

	// skip if the route has not yet been chartered & the railroad doesn't
	// already have a chartered route & the route is connected to the rr
	if ((ROUTE[route].Status == SURVEYED) &&
		(RAILROAD[rr].CharteredRoute == -1))
		{
		if (route_IsConnected(route,rr))
			{
			ROUTE[route].Status = CHARTERED;
			ROUTE[route].Owner = rr;
			RAILROAD[rr].CharteredRoute = route;
			return msgCharteredRoute;
			}
		else
			return msgNotAdjoiningRoute;
		}

	// skip out if we're already building track
	int chartered = RAILROAD[rr].CharteredRoute;
	if (ROUTE[chartered].BuiltLength < ROUTE[chartered].Length)
		return msgAlreadyBuildingTrack;

	// otherwise the route is fully built & cannot be re-chartered
	return msgFullyBuiltRoute;
	}

int		railroad_BuildTrack(int rr)
	{
	int dots = RAILROAD[rr].BUILDTRACK.DotsLeft;

	int	chartered	 = RAILROAD[rr].CharteredRoute;
	if (chartered == -1) return -1;

	route_BuildNextSection(chartered);

	int built_length = ROUTE[chartered].BuiltLength;
	int length   	 = ROUTE[chartered].Length;

	RAILROAD[rr].BUILDTRACK.DotsLeft--;

	if (built_length==length)
		{
		RAILROAD[rr].CharteredRoute = -1;
		ROUTE[chartered].Status = FULLY_BUILT;
		return msgTrackFullyBuilt;
		}
	else
		ROUTE[chartered].Status = PARTIALLY_BUILT;

	if (RAILROAD[rr].BUILDTRACK.DotsLeft==0)
		return msgFinishedBuildingSection;

	return msgBuildingInProgress;
	}


