//	-------------------------------------------------------------------------
//
//	RAILROAD.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:		Darren J. Dwyer
//
//	DESCRIPTION:	These are the definitions required for the RAILROADS
//
//	-------------------------------------------------------------------------

#ifndef __railroad_hpp
#define __railroad_hpp

#include "stddefns.hpp"
#include "gamedefn.hpp"
#include "train.hpp"

// DEFINITIONS

#define	MAX_REORG	5	// 5 years till re-organization of railroad

#define	NOT_IN_GAME		0	// the railroad doesn't exist
#define	NOT_FLOATED		1	// the railroad hasn't floated
#define	FLOATED			2	// the railroad has been floated
							// (at least 1 dot has been built)
#define	CONTROLLED		3	// the railroad is controlled
#define	RECEIVERSHIP	4   // the railroad is in receivership
#define	REORGANIZING	5	// the railroad is reorganizing

#define	NO_PRESIDENT	255



// structures used for the different railroad actions

struct	RR_BuyStocksStruct
	{
	char	Cnt;
	bool	PresOrMkt;
	int		Amount;
	};

#define	RR_SellStocksStruct		RR_BuyStocksStruct
#define	RR_BuyBondsStruct		RR_BuyStocksStruct
#define	RR_SellBondsStruct      RR_BuyStocksStruct

struct	RR_RepayBankStruct
	{
	char	Cnt;
	int		Amount;
	};

#define	RR_RepayDelinquentStruct	RR_RepayBankStruct
#define	RR_SetDividendStruct		RR_RepayBankStruct
#define	RR_RaiseEfficiencyStruct	RR_RepayBankStruct

struct	RR_BuySubsidiaryStruct
	{
	char	Cnt;
	int		Type;
	int		City;
	};

#define	RR_SellSubsidiaryStruct		RR_BuySubsidiaryStruct

struct	RR_BuyTrainStruct
	{
	char	Cnt;
	int		Owner;
	int		LocoType;
	int		CarType;
	int		NumCheckPoints;
	int		CheckPoint[MAX_CHECKPTS];
	};

#define	RR_SellTrainStruct			RR_BuyTrainStruct

struct	RR_SurveyRouteStruct
	{
	char	Cnt;
	int		City1;
	int		City2;
	};

struct	RR_CharterRouteStruct
	{
	char	Cnt;
	int		Route;
	};

struct	RR_BuildTrackStruct
	{
	char	Cnt;
	int		DotsLeft;
	};


// structure used to store railroad data

struct RailroadStruct
	{
	char	Status;				// railroad status

	char*	Name;				// name of the railroad
	char* 	Abbrev;				// abbreviated name of the railroad

	char	President;			// the player # who is president

	long	Cash;				// current cash assets

	long	BankLoan;			// current bank loan
	long	PresLoan;			// current president loan

	int		StocksInHand;		// unsold stocks still in railroad
	int		BondsInHand;		// unsold bonds still in railroad

	int		StocksOnMarket;		// sold stocks that are on the market
	int		BondsOnMarket;		// sold bonds that are on the market

	int		StocksInPlayer;		// stocks in player hands
	int		BondsInPlayer;		// bonds in player hands

	char	StockPrice[MAX_HISTORY];// storage of stock prices
	char	BondPrice[MAX_HISTORY];	// storage of bond prices

	char	CurrentHistory;

	char	Efficiency;			// the current efficiency level
	char	DividendRate;		// the current dividend rate
	long	RevenueYTD;			// the total revenue for the year-to-date
	long	CostsYTD;			// the total operating costs for year-to-date

	char	TurnsNotPaid;		// the count of how many times the railrad
								// hasn't paid out on bonds
	long	BondInterest[MAX_REORG];// the amount that needs to be paid out

	int		CharteredRoute; 	// the route # that is chartered

	int		TrainsRunning;		// the # of trains running for this rr
	int		MaxTrainsRunning;	// the max # of trains this rr can have

	// start of storage for actions that are currently pending

	char	QuickActionsPending;
	char	DailyActionsPending;
	char	MonthlyActionsPending;

	RR_BuyStocksStruct			BUYSTOCKS;
	RR_SellStocksStruct			SELLSTOCKS;
	RR_BuyBondsStruct			BUYBONDS;
	RR_SellBondsStruct			SELLBONDS;
	RR_RepayBankStruct			REPAYBANK;
	RR_RepayDelinquentStruct	REPAYDELINQUENT;
	RR_SetDividendStruct		SETDIVIDEND;
	RR_RaiseEfficiencyStruct	RAISEEFFICIENCY;
	RR_BuySubsidiaryStruct		BUYSUBSIDIARY;
	RR_SellSubsidiaryStruct		SELLSUBSIDIARY;
	RR_BuyTrainStruct			BUYTRAIN;
	RR_SellTrainStruct			SELLTRAIN;
	RR_SurveyRouteStruct		SURVEYROUTE;
	RR_CharterRouteStruct		CHARTERROUTE;
	RR_BuildTrackStruct			BUILDTRACK;

	// end of storage for actions that are currently pending
	};


extern	char			NUM_RRS;
extern	RailroadStruct	RAILROAD[MAX_RRS];

void	CreateRandomRailroads();
void	CreateHistoricRailroads();

int		railroad_BuyTrain(int rr);
int		railroad_SurveyRoute(int rr);
int 	railroad_CharterRoute(int rr);
int		railroad_BuildTrack(int rr);

#endif
