//	-------------------------------------------------------------------------
//
//	RAILS.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file defines the Rails class which enables rapid
//
//	-------------------------------------------------------------------------

#include "rails.hpp"
#include "palettes.hpp"
#include "palfile.hpp"
#include "fonts.hpp"
#include "borders.hpp"

// -----------------

#include "gamedefn.hpp"		// game-specific definitions
#include "setup.hpp"		// game setup screen

// -----------------

#include "system.hpp"		// system (gde) information
#include "ai.hpp"			// AI routines
#include "gameproc.hpp"		// game processor

// -----------------

#include "guidefn.hpp"
#include "map.hpp"
#include "economy.hpp"
#include "player.hpp"
#include "railroad.hpp"
#include "route.hpp"

// -----------------

#include "uidefn.hpp"
#include "mapwin.hpp"
#include "terrinfo.hpp"
#include "cityinfo.hpp"
#include "routinfo.hpp"

// -----------------

#include "folder.hpp"
#include "foldrsel.hpp"

#include "stksbox.hpp"
#include "bndsbox.hpp"
#include "bankbox.hpp"

#include "plyrfold.hpp"
#include "stksfold.hpp"
#include "bndsfold.hpp"
#include "bankfold.hpp"
#include "rrsfold.hpp"
#include "spyfold.hpp"
#include "oppfold.hpp"

// -----------------

#include "rrinfold.hpp"
#include "presfold.hpp"
#include "fncefold.hpp"
#include "operfold.hpp"
#include "routfold.hpp"
#include "trnfold.hpp"
#include "subfold.hpp"
#include "rrplfold.hpp"

// -----------------

#include "gamefold.hpp"

// -----------------

#include "qbuybnds.hpp"

// -----------------

#include "econbar.hpp"

// -----------------

void	Rails::FadeOutScreen(grPalette* pal,int speed)
	{
	if (pal)
		for (int i=0;i<64;i++)
			{
			grFadeOutPalette(pal,0,255,i);
			delay(speed);
//			if (keyboard_IsHit()) { keyboard_Flush(); return; }
			}
	}

void	Rails::FadeInScreen(int speed)
	{
	for (int i=0;i<64;i++)
		{
		grFadeInPalette(0,255,i);
		delay(speed);
//		if (keyboard_IsHit()) { keyboard_Flush(); return; }
		}
	}

void	Rails::ClearScreen(int color)
	{
	grPalette*	pal = grCreatePalette(256);
	if (!pal) return;

	grGetPalette(pal,0,255);

	FadeOutScreen(pal,1);
	grClearScreen(color);
	grBlitScreen();
	FadeInScreen(1);

	grDestroyPalette(pal);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Intro()
//	DESC:		This is the introduction sequence for the program.
//
//	-------------------------------------------------------------------------
void Rails::Intro()
	{
	HEAPCHK("At start of RAILS::Intro()");

	ClearScreen(grBLACK);

	PCXFile* introfile = new PCXFile(PROGRAM_PATH,"INTRO\\INTRO.PCX");
	if (!introfile)
		{
		DBG(470,"ERROR: Could not find intro data file %sINTRO\\INTRO.PCX",PROGRAM_PATH);
		delay(2000);
		return;
		}

	HEAPCHK("Rails::Intro()    after create introfile");

	grImage* intropic  = introfile->LoadImage();
	if (!intropic)
		{
		DBG(470,"ERROR: Could not load intro image");
		delay(2000);
		delete introfile;
		return;
		}


	HEAPCHK("Rails::Intro()    after introfile->LoadImage()");

	int x1 = 319;
	int x2 = 321;
	int y1 = 239;
	int y2 = 241;

	grPalette* pal = grCreatePalette(256);

	HEAPCHK("Rails::Intro()    after grCreatePalette()");

	if (pal) grGetPalette(pal,5,6);

	int i;
	if (pal) for (i=0;i<64;i++) grFadeOutPalette(pal,5,6,i);
	int cnt = 0;

//	keyboard_Flush();
//	mouse_Flush();

	do
		{
		grPutScaledImage(x1,y1,x2,y2,intropic);
		grBlitRegion(x1,y1,x2,y2);

		x1 --; x2 ++; y1 --; y2 ++;

		cnt++;

		i = cnt / 3;
		grFadeInPalette(5,6,i);
		}
	while ((cnt<200) && !keyboard_IsHit() && !mouse_IsHit());

	grDrawFilledRect(x1,y1,x2,y2,grBLACK);
	grBlitRegion(x1,y1,x2,y2);

	while (i<64) grFadeInPalette(5,6,i++);

	grDestroyPalette(pal);


	HEAPCHK("Rails::Intro()    after grDestroyPalette()");

	if (keyboard_IsHit()) keyboard_Flush();
	if (mouse_IsHit()) mouse_Flush();

	grDestroyImage(intropic);

	HEAPCHK("Rails::Intro()    after grDestroyImage()");

	delete introfile;


	HEAPCHK("Rails::Intro()    after - delete introfile");
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Finale()
//	DESC:		This is the finale for the program.
//
//	-------------------------------------------------------------------------
void Rails::Finale()
	{
	PCXFile* finalefile = new PCXFile(PROGRAM_PATH,"FINALE\\FINALE.PCX");
	if (!finalefile)
		{
		DBG(470,"ERROR: Could not find data file %sFINALE\\FINALE.PCX",PROGRAM_PATH);
		delay(2000);
		return;
		}
	grImage* finalepic  = finalefile->LoadImage();
	if (!finalepic)
		{
		DBG(470,"ERROR: Could not load finale image");
		delay(2000);
		delete finalefile;
		return;
		}

	int x1 = 319;
	int x2 = 321;
	int y1 = 239;
	int y2 = 241;
	int cnt = 0;

	keyboard_Flush();
	mouse_Flush();

	do
		{
		grPutScaledImage(x1,y1,x2,y2,finalepic);
		grBlitRegion(x1,y1,x2,y2);

		x1 --; x2 ++; y1 --; y2 ++;

		cnt++;
		}
	while ((cnt<200) && !keyboard_IsHit() && !mouse_IsHit());

	if (keyboard_IsHit()) keyboard_Flush();
	if (mouse_IsHit()) mouse_Flush();

	grDrawFilledRect(x1,y1,x2,y2,grBLACK);
	grBlitRegion(x1,y1,x2,y2);

	grDestroyImage(finalepic);
	delete finalefile;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Init()
//	DESC:		Here we create the global map data, etc.
//
//	-------------------------------------------------------------------------
extern bool SYS_DEBUG;

void	Rails::Init()
	{
	HEAPCHK("On RAILS::Init()");

	InitGameUI();

	HEAPCHK("After InitGameUI()");

	// install any relevant fonts
	SETUPFONT = grInitFont(PROGRAM_PATH,"RESOURCE\\SETUP.FNT");
	if (SETUPFONT<0)
		{
		DBG(470,"ERROR %d: Could not initialize %s\\RESOURCE\\SETUP.FNT",
					SETUPFONT,PROGRAM_PATH);
		while (!keyboard_IsHit()) ;
		keyboard_Flush();
		}

	HEAPCHK("After grInitFont(SETUP.FNT)");

	BASICFONT = grInitFont(PROGRAM_PATH,"RESOURCE\\BASIC.FNT");
	if (BASICFONT<0)
		{
		DBG(470,"ERROR %d: Could not initialise %s\\RESOURCE\\BASIC.FNT",BASICFONT,PROGRAM_PATH);
		while (!keyboard_IsHit()) ;
		keyboard_Flush();
		}

	HEAPCHK("After grInitFont(BASIC.FNT)");

	TITLEFONT = grInitFont(PROGRAM_PATH,"RESOURCE\\TITLE.FNT");
	if (TITLEFONT<0)
		{
		DBG(470,"ERROR %d: Could not initialize %s\\RESOURCE\\TITLE.FNT",TITLEFONT,PROGRAM_PATH);
		while (!keyboard_IsHit()) ;
		keyboard_Flush();
		}

	HEAPCHK("After grInitFont(TITLE.FNT)");

	NEWSPAPERFONT = grInitFont(PROGRAM_PATH,"RESOURCE\\NEWSPAPR.FNT");
	if (NEWSPAPERFONT<0)
		{
		DBG(470,"ERROR %d: Could not initialize %s\\RESOURCE\\NEWSPAPR.FNT",NEWSPAPERFONT,PROGRAM_PATH);
		while (!keyboard_IsHit()) ;
		keyboard_Flush();
		}

	HEAPCHK("After grInitFont(NEWSPAPR.FNT)");

	TINYFONT = grInitFont(PROGRAM_PATH,"RESOURCE\\TINY.FNT");
	if (TINYFONT<0)
		{
		DBG(470,"ERROR %d: Could not initialize %s\\RESOURCE\\TINY.FNT",TINYFONT,PROGRAM_PATH);
		while (!keyboard_IsHit()) ;
		keyboard_Flush();
		}

	HEAPCHK("After grInitFont(TINY.FNT)");

	HUGEFONT = grInitFont(PROGRAM_PATH,"RESOURCE\\HUGE.FNT");
	if (HUGEFONT<0)
		{
		DBG(470,"ERROR %d: Could not initialize %s\\RESOURCE\\HUGE.FNT",HUGEFONT,PROGRAM_PATH);
		while (!keyboard_IsHit()) ;
		keyboard_Flush();
		}

	HEAPCHK("After grInitFont(HUGE.FNT)");

	SYS_DEBUG = true;

	InitMaps();

	HEAPCHK("After InitMaps()");
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Exit()
//	DESC:		Here we destroy the global map data, etc.
//
//	-------------------------------------------------------------------------
void	Rails::Exit()
	{
	ExitMaps();

	SYS_DEBUG = false;

	grExitFont();
	grExitFont();
	grExitFont();
	grExitFont();
	grExitFont();
	grExitFont();

	ExitGameUI();
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		Setup()
//	DESC:		This is the game setup sequence for RAILS
//
//	-------------------------------------------------------------------------
void Rails::Setup()
	{
	// ie. palettes, fonts, resourcefiles, etc.
	PALFile* palfile = new PALFile(PROGRAM_PATH,"RESOURCE\\MAINLOOP.PAL");
	grPalette* pal = palfile->LoadPalette();

	HEAPCHK("After RAILS::Setup::  palfile->LoadPalette()");

	if (pal) grSetPalette(pal,0,255);
	else DBG(470,"Could not load %sRESOURCE\\MAINLOOP.PAL",PROGRAM_PATH);

	HEAPCHK("After RAILS::Setup:: grSetPalette()");

	NUM_PLAYERS = MIN_PLAYERS;
	NUM_RRS 	= MIN_RRS;
	NUM_CITIES 	= MIN_CITIES;
	NUM_ROUTES 	= MIN_ROUTES;

	CURRENT_PLAYER 	= 0;
	CURRENT_ECONOMY = 0;

	PACKDATE(CURRENT_DATE,0,0,0,1830);

	GAME_DELAY 		= 5;

	// do the setup screen
	RANDOM_GAME = true;

	EVENTMGR->AddEventHandler(new SetupScreen());

	// setup anything required _only_ for the mainloop
//	grPalette* pal = grCreatePalette(256);
//	grGetPalette(pal,0,255);

	FadeOutScreen(pal,1);

	grTileRect(0,0,grMaxx,grMaxy,-1,GUI_TILES[SCREEN_TILE],NO_ROTATE);
//	grClearScreen(grDARKGRAY);

	EVENTMGR->DrawAllHandlers();

	grBlitScreen();

	FadeInScreen(1);

	EVENTMGR->Execute();
	EVENTMGR->DelEventHandler();

	ClearScreen(grBLACK);

	if (EVENTMGR->isTerminating) return;

	if (RANDOM_GAME)
		{
		DBG(0,"Creating Random Game");
		CreateRandomPlayers();
		CreateRandomRailroads();
		CreateRandomMap();
		CreateRandomCities();
		CreateRandomRoutes();
		}
	else
		{
		DBG(0,"Beginning Historic Game (currently random)");
		CreateHistoricPlayers();
		CreateHistoricRailroads();
		CreateHistoricMap();
		CreateHistoricCities();
		CreateHistoricRoutes();
		}

	// create test railroad
	DBG(470,"Creating Test RAILROAD");
	int rr = 0;

	RAILROAD[rr].Status 	= FLOATED;
	RAILROAD[rr].Name 		= "The Testerfield Railroad";
	RAILROAD[rr].Abbrev 	= "TTR";
	RAILROAD[rr].President 	= 0;

	RAILROAD[rr].Cash 		= 1000;
	RAILROAD[rr].BankLoan 	= 100;
	RAILROAD[rr].PresLoan 	= 100;

	RAILROAD[rr].StocksInHand 	= 50;
	RAILROAD[rr].BondsInHand 	= 5;
	RAILROAD[rr].StocksOnMarket = 100;
	RAILROAD[rr].BondsOnMarket 	= 2;
	RAILROAD[rr].StocksInPlayer = 0;
	RAILROAD[rr].BondsInPlayer 	= 0;

	RAILROAD[rr].StockPrice[0] 	= 27;
	RAILROAD[rr].BondPrice[0]	= 40;

	RAILROAD[rr].Efficiency 	= 75;
	RAILROAD[rr].DividendRate 	= 10;
	RAILROAD[rr].RevenueYTD 	= 0;
	RAILROAD[rr].CostsYTD 		= 0;

	RAILROAD[rr].TurnsNotPaid 	= 0;

	for (int reorg=0;reorg<MAX_REORG;reorg++)
		RAILROAD[rr].BondInterest[reorg] = 0;

	RAILROAD[1].CharteredRoute = 1;
	RAILROAD[2].CharteredRoute = 2;

	// create test routes
	ROUTE[0].Owner = 0;
	ROUTE[1].Owner = 1;
	ROUTE[2].Owner = 2;

	ROUTE[0].BuiltLength = ROUTE[0].Length;
	ROUTE[1].BuiltLength = ROUTE[1].Length / 2;
	ROUTE[2].BuiltLength = 0;

	ROUTE[0].Status = FULLY_BUILT;
	ROUTE[1].Status = PARTIALLY_BUILT;
	ROUTE[2].Status = CHARTERED;

	route_Plot(0);
	route_Plot(1);
	route_Plot(2);

	// create test player
	DBG(470,"Creating Test PLAYER");
	int pl = 0;

	PLAYER[pl].Status		= HUMAN;
	PLAYER[pl].Name 		= "Darren J. Dwyer";
	PLAYER[pl].Cash			= 5000;

	for (rr=0;rr<NUM_RRS;rr++)
		{
		PLAYER[pl].Stocks[rr] = random(RAILROAD[rr].StocksInHand);
		PLAYER[pl].Bonds[rr] = random(RAILROAD[rr].BondsInHand);
		}
	PLAYER[pl].TotalStocks	= MAX_RRS;
	PLAYER[pl].TotalBonds	= MAX_RRS;

	for (int loan=0;loan<MAX_LOANS;loan++)
		{
		PLAYER[pl].LoanDue[loan] = 0;
		PLAYER[pl].LoanAmount[loan] = 0;
		PLAYER[pl].LoanRate[loan] = 0;
		}
	PLAYER[pl].TotalLoans = 0;

	PLAYER[pl].TotalLoans = 1;
	PLAYER[pl].LoanAmount[0] = 1000;
	PLAYER[pl].LoanRate[0] = 54;
	PACKDATE(PLAYER[pl].LoanDue[0],0,16,4,1968);

	PLAYER[pl].TotalSubsid = 0;
	PLAYER[pl].TotalPres = 0;
	PLAYER[pl].TotalAssets = 0;
	PLAYER[pl].TotalDebt = 0;

	PLAYER[pl].ActingForRR = false;

	CalcEconomyVars();

	grDestroyPalette(pal);
	delete palfile;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		MainLoop()
//	DESC:       This is the main loop of the program.
//
//	-------------------------------------------------------------------------
void Rails::MainLoop()
	{
	// setup anything required _only_ for the mainloop
	// ie. palettes, fonts, resourcefiles, etc.
	PALFile* palfile = new PALFile(PROGRAM_PATH,"RESOURCE\\MAINLOOP.PAL");
	grPalette* pal = palfile->LoadPalette();
	if (pal) grSetPalette(pal,0,255);
	else DBG(470,"Could not load %sRESOURCE\\MAINLOOP.PAL",PROGRAM_PATH);

	// set up our timer to work at 100 times a second
	timer_ChangeInterruptRate(timer_100Hz);

	// create all objects used in the program (in order of priority)

	DBG(470,"Creating EVENT HANDLERS"); delay(2);

	// --- create all debugging objects

	EVENTMGR->AddEventHandler(new System());	// used for debugging engine

	// --- add process (server) bits and pieces

	EVENTMGR->AddEventHandler(new GameProcessor());

	// --- add AI bits and pieces

	EVENTMGR->AddEventHandler(new AI());

	// --- add map bits

	MAPWIN = (MapWin*) EVENTMGR->AddEventHandler(new MapWin());
	EVENTMGR->AddEventHandler(new TerrainInfo());
	EVENTMGR->AddEventHandler(new CityInfo());
	EVENTMGR->AddEventHandler(new RouteInfo());

	// --- add folder selection bar

	EVENTMGR->AddEventHandler(new FolderSelection());

	// --- add folder info boxes

	EVENTMGR->AddEventHandler(new StockMarketBox());
	EVENTMGR->AddEventHandler(new BondMarketBox());
	EVENTMGR->AddEventHandler(new BankBox());

	// 	   --- player folders

	EVENTMGR->AddEventHandler(new PlayerFolder());
	EVENTMGR->AddEventHandler(new StocksFolder());
	EVENTMGR->AddEventHandler(new BondsFolder());
	EVENTMGR->AddEventHandler(new BankFolder());
	EVENTMGR->AddEventHandler(new RRsFolder());
	EVENTMGR->AddEventHandler(new SpyFolder());
	EVENTMGR->AddEventHandler(new OpponentFolder());

	//		--- railroad folders

	EVENTMGR->AddEventHandler(new RRInfoFolder());
	EVENTMGR->AddEventHandler(new PresFolder());
	EVENTMGR->AddEventHandler(new FinanceFolder());
	EVENTMGR->AddEventHandler(new OperationsFolder());
	EVENTMGR->AddEventHandler(new RoutesFolder());
	EVENTMGR->AddEventHandler(new TrainsFolder());
	EVENTMGR->AddEventHandler(new SubsidFolder());
	EVENTMGR->AddEventHandler(new ProfitLossFolder());

	// 		--- other folders

	EVENTMGR->AddEventHandler(new GameFolder());

	//		--- other bits and pieces

	EVENTMGR->AddEventHandler(new EconomyBar());

	//		--- add query folders

	EVENTMGR->AddEventHandler(new BuyBondsQuery());

	//		--- select appropriate handler on startup

	EVENTMGR->SelectEventHandler(7);		// FolderSelection

	// draw all the relevant handlers
	FadeOutScreen(pal,1);

//	grClearScreen(grDARKGRAY);
	grTileRect(0,0,grMaxx,grMaxy,-1,GUI_TILES[SCREEN_TILE],NO_ROTATE);

	EVENTMGR->DrawAllHandlers();

	grBlitScreen();

	FadeInScreen(1);

	// add any tickers required by the main game
	GAME_TICKER = timer_AddTicker(GAME_DELAY,cmdGameTick);

	// turn on mouse move events
	mouse_EnableMoveEvents();

	// tell the event manager to execute using the handlers!!!
	EVENTMGR->Execute();

	// clear the screen again
	ClearScreen(grBLACK);

	// disable mouse move events
	mouse_DisableMoveEvents();

	// delete the relevent tickers...
	timer_DelTicker( GAME_TICKER );

	// delete the objects now that we have finished with them
	EVENTMGR->DelAllEventHandlers();

	grDestroyPalette(pal);
	delete palfile;
	}


void	Rails::Execute()
	{
	Intro();
	Init();
	Setup();
	if (!EVENTMGR->isTerminating) MainLoop();
	Exit();
	Finale();
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		main()
//	DESC:		This is the function called at program execution.
//				At the moment it simply creates our game, executes it,
//				then deletes it, exiting normally
//
//	-------------------------------------------------------------------------
void main(int argc,char** argv)
	{
	// create our program
	Rails* theGame= new Rails(argv[0],prg_InstallGUI|prg_InstallSystemFont);

	// run it's intro, then setup, then mainloop, then finale
	theGame->Execute();

	// and clear up after ourselves
	delete theGame;
	}
