
#include "route.hpp"
#include "map.hpp"
#include "city.hpp"
#include "railroad.hpp"
#include "mapwin.hpp"

#include "graphics.hpp"
#include "eventmgr.hpp"

// GLOBALS

char	NUM_ROUTES;
RouteStruct	ROUTE[MAX_ROUTES];

bool	surveying_finished = false;	// true-> all possible routes have been surveyed



// ROUTE FUNCTIONS
void	CreateRandomRoutes()
	{
	NUM_ROUTES = 0;
	surveying_finished = false;

	int 	rnd_routes = NUM_RRS;
	bool 	found_route = true;

	do
		{
		DBG(470,"Surveying Route #%d",NUM_ROUTES);

		found_route = CreateNewRoute();
		}
	while ((NUM_ROUTES<rnd_routes) && found_route);

	DBG(470,"Finished Surveying %d Routes",NUM_ROUTES);
	}

void	CreateHistoricRoutes()
	{
	CreateRandomRoutes();
	}


bool	CreateNewRoute()
	{
	if (surveying_finished) return false;
	if (NUM_ROUTES>=MAX_ROUTES) { surveying_finished = true; return false; }

	int	 	startcity,endcity;
	bool	surveyed;

	// if we're surveying for the very first time choose a totally random
	// startcity and attempt to build it to it's closest city

	if (NUM_ROUTES==0)
		{
		do
			{
			surveyed = false;
			startcity = random(NUM_CITIES);
			endcity = CITY[startcity].ClosestCity[0];
			if (endcity==-1) continue;

			if (route_Valid(startcity,endcity))
				{
				surveyed = route_Survey(NUM_ROUTES,endcity,startcity);
				if (!surveyed) surveyed = route_Survey(NUM_ROUTES,startcity,endcity);
				}
			}
		while (!surveyed);

		// make sure the route is removed from the list so we don't check later
		int nth;
		for (nth=0;nth<N_CLOSEST_CITIES-1;nth++)
			CITY[startcity].ClosestCity[nth] = CITY[startcity].ClosestCity[nth+1];
		CITY[startcity].ClosestCity[N_CLOSEST_CITIES-1] = -1;

		// make sure we don't bother checking the other way....
		int start = -1;
		for (nth=0;nth<N_CLOSEST_CITIES-1;nth++)
			if (CITY[endcity].ClosestCity[nth]==startcity)
				start=nth;
		if (start!=-1)
			{
			for (nth=start;nth<N_CLOSEST_CITIES-1;nth++)
				CITY[endcity].ClosestCity[nth] = CITY[endcity].ClosestCity[nth+1];
			CITY[endcity].ClosestCity[N_CLOSEST_CITIES-1] = -1;
			}

		// we must have plotted a route
		route_Plot(NUM_ROUTES);
		NUM_ROUTES++;

		return true;
		}


	// repeat until we find a valid route or we exhaust all possible routes
	int  closest_dist;
	do
		{
		// 5% chance that the city we are building from can be unconnected
		// 95% means that the city must already be connected to another city
		bool random_chance = (random(100)<5);

		closest_dist = 100000;
		startcity = -1;
		endcity   = -1;

		// scan through all connected cities figuring out the closest
		// city to any specific node.

		// scan through all cities
		for (int citynum=0;citynum<NUM_CITIES;citynum++)
			{
			// if a city is connected (or random chance thats it's not)
			// and there's a spare connection
			if ( ((CITY[citynum].NumConnections>0) || random_chance) &&
				  (CITY[citynum].NumConnections<4) )
				{
				// find the closest city to this one (skip if can't)
				int nearby_city = CITY[citynum].ClosestCity[0];
				if (nearby_city==-1) continue;

				// figure the distance between cities
				int min_dist = DistBetweenCities(citynum,nearby_city);

				// and if its the smallest distance so far....
				if (min_dist<closest_dist)
					{
					// save the cities (temporarily till loop ends, then they're real)
					startcity = citynum;
					endcity   = nearby_city;

					// save the current closest distance
					closest_dist = min_dist;
					}
				}
			}

		// if we found a valid possibility

		if (closest_dist!=100000)
			{
			if (startcity==endcity)
				{
				surveying_finished = true;
				return false;
				}

			// attempt to survey the route
			surveyed = false;

			if (route_Valid(startcity,endcity))
				{
				surveyed = route_Survey(NUM_ROUTES,startcity,endcity);
				if (!surveyed) surveyed = route_Survey(NUM_ROUTES,endcity,startcity);
				}

			// make sure we don't check this route again

			int nth;
			for (nth=0;nth<N_CLOSEST_CITIES-1;nth++)
				CITY[startcity].ClosestCity[nth] = CITY[startcity].ClosestCity[nth+1];
			CITY[startcity].ClosestCity[N_CLOSEST_CITIES-1] = -1;

			// make sure we don't bother checking the other way....
			int start = -1;
			for (nth=0;nth<N_CLOSEST_CITIES-1;nth++)
				if (CITY[endcity].ClosestCity[nth]==startcity)
					start=nth;
			if (start!=-1)
				{
				for (nth=start;nth<N_CLOSEST_CITIES-1;nth++)
					CITY[endcity].ClosestCity[nth] = CITY[endcity].ClosestCity[nth+1];
				CITY[endcity].ClosestCity[N_CLOSEST_CITIES-1] = -1;
				}

			// and if the route was surveyed correctly skip out
			if (surveyed)
				{
				route_Plot(NUM_ROUTES);
				NUM_ROUTES++;
				return true;
				}
			}
		}
	while (closest_dist!=100000);

	// we've exhausted all possible routes, make sure we don't try surveying
	// any more
	surveying_finished = true;
	return false;
	}




bool	CreateNewRoute(int startcity,int endcity)
	{
	// creates a new route between two chosen cities iff at all possible
	if (surveying_finished) return false;
	if (NUM_ROUTES>=MAX_ROUTES) { surveying_finished = true; return false; }

	bool	surveyed;

	int		nth,first = -1,second = -1;
	bool	can_attempt = false;

	for (nth=0;nth<N_CLOSEST_CITIES-1;nth++)
		{
		if (CITY[startcity].ClosestCity[nth] == endcity)
			{
			can_attempt = true;
			first = nth;
			}
		if (CITY[endcity].ClosestCity[nth] == startcity)
			{
			can_attempt = true;
			second = nth;
			}
		}

	if (!can_attempt) return false;


	if (route_Valid(startcity,endcity))
		{
		surveyed = route_Survey(NUM_ROUTES,endcity,startcity);
		if (!surveyed) surveyed = route_Survey(NUM_ROUTES,startcity,endcity);
		}

	if (!surveyed) return false;

	// make sure the route is removed from the list so we don't check later
	if (first!=-1)
		{
		for (nth=first;nth<N_CLOSEST_CITIES-1;nth++)
			CITY[startcity].ClosestCity[nth] = CITY[startcity].ClosestCity[nth+1];

		CITY[startcity].ClosestCity[N_CLOSEST_CITIES-1] = -1;
		}

	// make sure we don't bother checking the other way....
	if (second!=-1)
		{
		for (nth=second;nth<N_CLOSEST_CITIES-1;nth++)
			CITY[endcity].ClosestCity[nth] = CITY[endcity].ClosestCity[nth+1];

		CITY[endcity].ClosestCity[N_CLOSEST_CITIES-1] = -1;
		}

	// we must have plotted a route
	route_Plot(NUM_ROUTES);

	NUM_ROUTES++;
	return true;
	}

bool	route_Survey(int routenum,int startcity,int endcity)
	{
//	DBG(0,"Surveying route%.3d between city%.3d and city%.3d",routenum,startcity,endcity);
	bool route_is_valid = true;
	bool route_is_finished = false;

	int route_length = 0;



	// --- figure out the general direction of city2 from city1

	int startcityx = CITY[startcity].x;
	int startcityy = CITY[startcity].y;

	int endcityx = CITY[endcity].x;
	int endcityy = CITY[endcity].y;

	int dirn = -1;

	if (endcityx<startcityx) dirn = LT;
	if (endcityx>startcityx) dirn = RT;
	if (endcityy<startcityy)
		{
		if (dirn==LT) dirn = UPLT;
		else if (dirn==RT) dirn = UPRT;
			 else dirn = UP;
		}
	if (endcityy>startcityy)
		{
		if (dirn==LT) dirn = DNLT;
		else if (dirn==RT) dirn = DNRT;
			 else dirn = DN;
		}


	// --- choose the direction of the start & end stations

	int xdist = endcityx - startcityx; if (xdist<0) xdist = startcityx - endcityx;
	int ydist = endcityy - startcityy; if (ydist<0) ydist = startcityy - endcityy;

	int start_station_dirn;
	int end_station_dirn;

	// if dx is smaller than dy

	if (xdist<ydist)
		{
		switch (dirn)
			{
			case UP:	start_station_dirn = UP;
						end_station_dirn   = DN;
						break;
			case DN:	start_station_dirn = DN;
						end_station_dirn   = UP;
						break;
			case UPLT:	end_station_dirn = DN;
						if (xdist<3) start_station_dirn = UP;
						else		 start_station_dirn = LT;
						break;
			case DNLT:	end_station_dirn = UP;
						if (xdist<3) start_station_dirn = DN;
						else		 start_station_dirn = LT;
						break;
			case UPRT:	end_station_dirn = DN;
						if (xdist<3) start_station_dirn = UP;
						else		 start_station_dirn = RT;
						break;
			case DNRT:	end_station_dirn = UP;
						if (xdist<3) start_station_dirn = DN;
						else		 start_station_dirn = RT;
						break;
			}
		}
	else
		// if dy is smaller than dx
		if (ydist<xdist)
			{
			// we're heading up or down
			switch (dirn)
				{
				case LT:	start_station_dirn = LT;
							end_station_dirn   = RT;
							break;
				case RT:	start_station_dirn = RT;
							end_station_dirn   = LT;
							break;
				case UPLT:	end_station_dirn = RT;
							if (ydist<3) start_station_dirn = LT;
							else		 start_station_dirn = UP;
							break;
				case DNLT:	end_station_dirn = RT;
							if (ydist<3) start_station_dirn = LT;
							else		 start_station_dirn = DN;
							break;
				case UPRT:	end_station_dirn = LT;
							if (ydist<3) start_station_dirn = RT;
							else		 start_station_dirn = UP;
							break;
				case DNRT:	end_station_dirn = LT;
							if (ydist<3) start_station_dirn = RT;
							else		 start_station_dirn = DN;
							break;
				}
			}
		else
			{
			// dx == dy  (a square)... choose random start direction

			int rnd = random(2);
			switch (dirn)
				{
				case UPLT:	if (rnd)
								{
								start_station_dirn = LT;
								end_station_dirn   = DN;
								}
							else
								{
								start_station_dirn = UP;
								end_station_dirn   = RT;
								}
							break;
				case UPRT:	if (rnd)
								{
								start_station_dirn = RT;
								end_station_dirn   = DN;
								}
							else
								{
								start_station_dirn = UP;
								end_station_dirn   = LT;
								}
							break;
				case DNLT:	if (rnd)
								{
								start_station_dirn = LT;
								end_station_dirn   = UP;
								}
							else
								{
								start_station_dirn = DN;
								end_station_dirn   = RT;
								}
							break;
				case DNRT:  if (rnd)
								{
								start_station_dirn = RT;
								end_station_dirn   = UP;
								}
							else
								{
								start_station_dirn = DN;
								end_station_dirn   = LT;
								}
							break;
				}
			}


	// --- skip out if either station is already connected in chosen direction

	if ((CITY[startcity].CityConnection[start_station_dirn]!=NOT_CONNECTED) ||
		(CITY[endcity].CityConnection[end_station_dirn]!=NOT_CONNECTED)) return false;

	// --- move the cursor to the first position after the station

	int xpos = CITY[startcity].x;
	int ypos = CITY[startcity].y;

	// if it's the first track to be plotted
	// set last_xpos,last_ypos to the startcity location

	switch (start_station_dirn)
		{
		case UP:	ypos--; break;
		case DN:	ypos++; break;
		case LT:	xpos--; break;
		case RT:	xpos++; break;
		}

	// --- plot the required track

	int		ptr = 0;

	// --- if we need to turn the track, make a corner now

	bool turning_corners = ((dirn == UPLT) || (dirn == UPRT) ||
							(dirn == DNLT) || (dirn == DNRT));

	if (turning_corners)
		{
		int 	diag_icon1,diag_icon2;
		int		xinc1,xinc2,yinc1,yinc2;

		// based on our station start direction

		switch (start_station_dirn)
			{
			case UP:    // and based upon our city's direction
						if (dirn==UPLT)
							{
							// plot relevant corner turning left
							if (!route_ValidSection(routenum,ptr,xpos,ypos--,3)) return false;
							if (!route_ValidSection(routenum,ptr,xpos--,ypos,2)) return false;
							diag_icon1 = 21; xinc1 = 0; yinc1 = -1;
							diag_icon2 = 20; xinc2 = -1; yinc2 = 0;
							}
						else
							{
							// and turning right
							if (!route_ValidSection(routenum,ptr,xpos,ypos--,5)) return false;
							if (!route_ValidSection(routenum,ptr,xpos++,ypos,4)) return false;
							diag_icon1 = 19; xinc1 = 0; yinc1 = -1;
							diag_icon2 = 18; xinc2 = 1; yinc2 = 0;
							}
						break;
			case DN:	if (dirn==DNLT)
							{
							// turning lt
							if (!route_ValidSection(routenum,ptr,xpos,ypos++,6)) return false;
							if (!route_ValidSection(routenum,ptr,xpos--,ypos,7)) return false;
							diag_icon1 = 18; xinc1 = 0; yinc1 = 1;
							diag_icon2 = 19; xinc2 = -1; yinc2 = 0;
							}
						else
							{
							// turning rt
							if (!route_ValidSection(routenum,ptr,xpos,ypos++,8)) return false;
							if (!route_ValidSection(routenum,ptr,xpos++,ypos,9)) return false;
							diag_icon1 = 20; xinc1 = 0; yinc1 = 1;
							diag_icon2 = 21; xinc2 = 1; yinc2 = 0;
							}
						break;
			case LT:	if (dirn==UPLT)
							{
							// turning up
							if (!route_ValidSection(routenum,ptr,xpos--,ypos,15)) return false;
							if (!route_ValidSection(routenum,ptr,xpos,ypos--,14)) return false;
							diag_icon1 = 20; xinc1 = -1; yinc1 = 0;
							diag_icon2 = 21; xinc2 = 0; yinc2 = -1;
							}
						else
							{
							// turning dn
							if (!route_ValidSection(routenum,ptr,xpos--,ypos,17)) return false;
							if (!route_ValidSection(routenum,ptr,xpos,ypos++,16)) return false;
							diag_icon1 = 19; xinc1 = -1; yinc1 = 0;
							diag_icon2 = 18; xinc2 = 0; yinc2 = 1;
							}
						break;
			case RT:	if (dirn==UPRT)
							{
							// turning up
							if (!route_ValidSection(routenum,ptr,xpos++,ypos,10)) return false;
							if (!route_ValidSection(routenum,ptr,xpos,ypos--,11)) return false;
							diag_icon1 = 18; xinc1 = 1; yinc1 = 0;
							diag_icon2 = 19; xinc2 = 0; yinc2 = -1;
							}
						else
							{
							// turning dn
							if (!route_ValidSection(routenum,ptr,xpos++,ypos,12)) return false;
							if (!route_ValidSection(routenum,ptr,xpos,ypos++,13)) return false;
							diag_icon1 = 21; xinc1 = 1; yinc1 = 0;
							diag_icon2 = 20; xinc2 = 0; yinc2 = 1;
							}
						break;
			default:	return false;
			}

		// --- we're now going diagonal... plot till we hit the appropriate
		//	   x or y axis

		int diag_cnt = 0;

		while ((xpos != CITY[endcity].x) && (ypos != CITY[endcity].y))
			{
			switch (diag_cnt)
				{
				case 0:		if (!route_ValidSection(routenum,ptr,xpos,ypos,diag_icon1)) return false;
							xpos += xinc1; ypos += yinc1;
							break;
				case 1:     if (!route_ValidSection(routenum,ptr,xpos,ypos,diag_icon2)) return false;
							xpos += xinc2; ypos += yinc2;
							break;
				}

			// make diag_cnt toggle between 0 and 1
			++diag_cnt %= 2;
			}


		// --- turn the second corner...

		int 	min_dist = min(xdist,ydist);

		// check for special cases where the distance is too small to
		// plot a normal curve

		if (min_dist<3)
			{
			// implement the special cases
			switch (start_station_dirn)
				{
				case UP:	if (dirn==UPLT)
								{
								if (!route_ValidSection(routenum,ptr,xpos,ypos--,9)) return false;
								if (!route_ValidSection(routenum,ptr,xpos,ypos--,8)) return false;
								}
							else
								{
								if (!route_ValidSection(routenum,ptr,xpos,ypos--,7)) return false;
								if (!route_ValidSection(routenum,ptr,xpos,ypos--,6)) return false;
								}
							break;
				case DN:	if (dirn==DNLT)
								{
								if (!route_ValidSection(routenum,ptr,xpos,ypos++,4)) return false;
								if (!route_ValidSection(routenum,ptr,xpos,ypos++,5)) return false;
								}
							else
								{
								if (!route_ValidSection(routenum,ptr,xpos,ypos++,2)) return false;
								if (!route_ValidSection(routenum,ptr,xpos,ypos++,3)) return false;
								}
							break;
				case LT:	if (dirn==UPLT)
								{
								if (!route_ValidSection(routenum,ptr,xpos--,ypos,13)) return false;
								if (!route_ValidSection(routenum,ptr,xpos--,ypos,12)) return false;
								}
							else
								{
								if (!route_ValidSection(routenum,ptr,xpos--,ypos,11)) return false;
								if (!route_ValidSection(routenum,ptr,xpos--,ypos,10)) return false;
								}
							break;
				case RT:	if (dirn==UPRT)
								{
								if (!route_ValidSection(routenum,ptr,xpos++,ypos,16)) return false;
								if (!route_ValidSection(routenum,ptr,xpos++,ypos,17)) return false;
								}
							else
								{
								if (!route_ValidSection(routenum,ptr,xpos++,ypos,14)) return false;
								if (!route_ValidSection(routenum,ptr,xpos++,ypos,15)) return false;
								}
							break;

				default:	return false;
				}
			}
		else

		// otherwise we handle the normal track curve
			{
			switch (start_station_dirn)
				{
				case UP:	if (dirn==UPLT)
								{
								// turning left
								if (!route_ValidSection(routenum,ptr,xpos--,ypos,13)) return false;
								if (!route_ValidSection(routenum,ptr,xpos--,ypos,12)) return false;
								}
							else
								{
								// turning right
								if (!route_ValidSection(routenum,ptr,xpos++,ypos,16)) return false;
								if (!route_ValidSection(routenum,ptr,xpos++,ypos,17)) return false;
								}
							break;
				case DN:	if (dirn==DNLT)
								{
								// turning left
								if (!route_ValidSection(routenum,ptr,xpos--,ypos,11)) return false;
								if (!route_ValidSection(routenum,ptr,xpos--,ypos,10)) return false;
								}
							else
								{
								// turning right
								if (!route_ValidSection(routenum,ptr,xpos++,ypos,14)) return false;
								if (!route_ValidSection(routenum,ptr,xpos++,ypos,15)) return false;
								}
							break;
				case LT:	if (dirn==UPLT)
								{
								// turning up
								if (!route_ValidSection(routenum,ptr,xpos,ypos--,9)) return false;
								if (!route_ValidSection(routenum,ptr,xpos,ypos--,8)) return false;
								}
							else
								{
								// turning down
								if (!route_ValidSection(routenum,ptr,xpos,ypos++,4)) return false;
								if (!route_ValidSection(routenum,ptr,xpos,ypos++,5)) return false;
								}
							break;
				case RT:	if (dirn==UPRT)
								{
								// turning up
								if (!route_ValidSection(routenum,ptr,xpos,ypos--,7)) return false;
								if (!route_ValidSection(routenum,ptr,xpos,ypos--,6)) return false;
								}
							else
								{
								// turning down
								if (!route_ValidSection(routenum,ptr,xpos,ypos++,2)) return false;
								if (!route_ValidSection(routenum,ptr,xpos,ypos++,3)) return false;
								}
							break;

				default:	return false;
				}
			}	// finished normal track curve2


		} // finished turning corners


	// --- plot the straight piece of track as required
	// 	   (until we reach the end city station)

	// vertical track
	if (xpos==endcityx)
		{
		while (ypos<endcityy) if (!route_ValidSection(routenum,ptr,xpos,ypos++,0)) return false;
		while (ypos>endcityy) if (!route_ValidSection(routenum,ptr,xpos,ypos--,0)) return false;
		}

	// horizontal track
	if (ypos==endcityy)
		{
		while (xpos<endcityx) if (!route_ValidSection(routenum,ptr,xpos++,ypos,1)) return false;
		while (xpos>endcityx) if (!route_ValidSection(routenum,ptr,xpos--,ypos,1)) return false;
		}

	// all done....

	ROUTE[routenum].x[ptr] = CITY[endcity].x;
	ROUTE[routenum].y[ptr] = CITY[endcity].y;

	route_length = ptr;

	// save the route if its finished surveying correctly
	if (route_is_valid)
		{
		// save relevent details for the route
		ROUTE[routenum].Status = SURVEYED;
		ROUTE[routenum].StartCity = startcity;
		ROUTE[routenum].EndCity = endcity;
		ROUTE[routenum].Owner = -1;
		ROUTE[routenum].BuiltLength = 0;
		ROUTE[routenum].Length = route_length;
		ROUTE[routenum].TotalRevenueYTD = 0;

		// figure out the land grants available for the route
		ROUTE[routenum].LandGrants = random(9999);

		// adjust the cities also
		CITY[startcity].NumConnections++;
		CITY[startcity].CityConnection[start_station_dirn] = endcity;
		CITY[startcity].RouteConnection[start_station_dirn] = routenum;

		PlotCityStation(startcity);

		CITY[endcity].NumConnections++;
		CITY[endcity].CityConnection[end_station_dirn] = startcity;
		CITY[endcity].RouteConnection[end_station_dirn] = routenum;

		PlotCityStation(endcity);

		return true;
		}

	return false;
	}


bool route_Valid(int startcity,int endcity)
// checks to see if a route is a valid route or not. used by CreateRandom()
// the only rule for checking (at the moment) is 2 tracks max between same
// 2 cities
	{
	int samecities = 0;

	for (int i=0;i<NUM_ROUTES;i++)
			{
			// check that we are not covering the same route
			if ((ROUTE[i].StartCity==startcity)&&
				(ROUTE[i].EndCity==endcity))
				// increment the similar tracks
				samecities++;
			// also check the reverse of the route
			if ((ROUTE[i].StartCity==endcity)&&
				(ROUTE[i].EndCity==startcity))
				// increment the similar tracks
				samecities++;
			}

	// one track only is valid
	if (samecities<1) return true;
	else
		// if attempting 2 tracks, valid iff either of the cities>10
		if (samecities<2)
			if ((CITY[startcity].Size>10)&&
				(CITY[endcity].Size>10))
				return true;
	return false;
	}



bool	route_ValidSection(int routenum,int& ptr,int x,int y,int icon)
	{
	if (ptr>=MAX_ROUTE_LENGTH) return false;

	int ter = GetTerrain(x,y);
	int ovl = GetOverlaid(x,y);

	if (ter==sea) return false;	// pretty obvious
	if (ovl<FIRST_CITY) return false;	// any form of track incl. stations

	ROUTE[routenum].x[ptr] = x;
	ROUTE[routenum].y[ptr] = y;
	ROUTE[routenum].icon[ptr] = icon;

	ptr++;

	return true;
	}


void	route_Plot(int routenum)
	{
	// skip out if the route hasn't been surveyed
	if (ROUTE[routenum].Status == NOT_SURVEYED) return;

	// plot all unbuilt track
	int section;
	int len = ROUTE[routenum].BuiltLength;
	for (section=0; section<len;section++) route_PlotSection(routenum,section,FIRST_BUILT);

	// plot all built track (continuing on from the last bit of track
	len = ROUTE[routenum].Length;
	for (;section<len;section++) route_PlotSection(routenum,section,FIRST_SURVEYED);
	}

void	route_Highlight(int routenum)
	{
	if (ROUTE[routenum].Status == NOT_SURVEYED) return;

	int section;
	int len = ROUTE[routenum].Length;

	for (section=0; section<len;section++) route_PlotSection(routenum,section,FIRST_HIGHLIGHTED);
	}


void	route_PlotSection(int routenum,int section,int offset)
	{
	// assume start position
	int xpos,ypos,ptr,icon;
	int rnum = routenum + ROUTE_START_OFFSET;

	ptr = section;

	xpos = ROUTE[routenum].x[ptr];
	ypos = ROUTE[routenum].y[ptr];
	icon = ROUTE[routenum].icon[ptr];

	SetOverlaid(xpos,ypos,icon+offset);
	SetData(xpos,ypos,rnum);

grPutPixel(xpos,ypos,icon+offset);
grBlitRegion(xpos,ypos,xpos,ypos);

	if (GetData(xpos-1,ypos-1)!=rnum) SetData(xpos-1,ypos-1,rnum);
	if (GetData(xpos-1,ypos  )!=rnum) SetData(xpos-1,ypos  ,rnum);
	if (GetData(xpos-1,ypos+1)!=rnum) SetData(xpos-1,ypos+1,rnum);
	if (GetData(xpos+1,ypos-1)!=rnum) SetData(xpos+1,ypos-1,rnum);
	if (GetData(xpos+1,ypos  )!=rnum) SetData(xpos+1,ypos  ,rnum);
	if (GetData(xpos+1,ypos+1)!=rnum) SetData(xpos+1,ypos+1,rnum);
	if (GetData(xpos  ,ypos-1)!=rnum) SetData(xpos  ,ypos-1,rnum);
	if (GetData(xpos  ,ypos+1)!=rnum) SetData(xpos  ,ypos+1,rnum);

	PlotCity(ROUTE[routenum].StartCity,false);
	PlotCity(ROUTE[routenum].EndCity,false);
	}

void	route_BuildSections(int route)
	{
	int built_length = ROUTE[route].BuiltLength;
	int section = 0;
	while (section<built_length) route_PlotSection(route,section++,FIRST_BUILT);
	}

void	route_BuildNextSection(int route)
	{
	if (ROUTE[route].BuiltLength<ROUTE[route].Length)
		{
		int built_length = ROUTE[route].BuiltLength;

		route_PlotSection(route,built_length,FIRST_BUILT);
		MAPWIN->DrawRouteSection(route,built_length);

		ROUTE[route].BuiltLength++;
		}
	}

bool	route_IsConnected(int route,int rr)
	{
	int startcity = ROUTE[route].StartCity;
	int endcity = ROUTE[route].EndCity;

	// scan through start city connections
	int	connections = CITY[startcity].NumConnections;
	int conn;
	for (conn=0;conn<connections;conn++)
		// and skip out if we find a connection is this route
		if (CITY[startcity].RouteConnection[conn]==route)
			return true;

	// do same for endcity
	connections = CITY[endcity].NumConnections;
	for (conn=0;conn<connections;conn++)
		if (CITY[endcity].RouteConnection[conn]==route)
			return true;

	// didn't find a connection
	return false;
	}