//	-------------------------------------------------------------------------
//
//	ROUTE.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	These are all of the bits required for the routes
//
//	-------------------------------------------------------------------------

#ifndef __route_hpp
#define __route_hpp

#include "stddefns.hpp"
#include "gamedefn.hpp"

// DEFINITIONS

#define MAX_ROUTE_LENGTH	512


#define	NOT_SURVEYED		1	// the route status
#define	SURVEYED			2
#define	CHARTERED			3
#define	PARTIALLY_BUILT     4
#define	FULLY_BUILT         5



#define UP		0
#define	DN		1
#define LT		2
#define RT		3
#define UPLT	4
#define UPRT	5
#define DNLT	6
#define	DNRT	7

struct	RouteStruct
	{
	char	Status;			// the status of the route
	int		Owner;			// the owner of this route

	long	LandGrants;		// the landgrants for this route

	char	StartCity;		// the start city for the route
	char	EndCity;		// the end city for the route

	int		Length;			// the length of the route
	int		BuiltLength;	// the built length of the route

	int		TurnsLeft;		// how many turns to go before we liquidate RR

	short	x[MAX_ROUTE_LENGTH];
	short	y[MAX_ROUTE_LENGTH];
	char	icon[MAX_ROUTE_LENGTH];

	long	TotalRevenueYTD;
	};

// GLOBALS

extern	char			NUM_ROUTES;			// the number of routes in the game
extern	bool			ROUTES_EXHAUSTED;	// the routes are all exhausted
extern	RouteStruct		ROUTE[MAX_ROUTES];	// storage of route information
extern	bool			surveying_finished;	// true-> all possible routes
											//        have been surveyed


// FUNCTIONS

void	CreateRandomRoutes();
void	CreateHistoricRoutes();

bool	CreateNewRoute();
bool	CreateNewRoute(int startcity,int endcity);

//	warning: don't use this to manually survey routes, use CreateNewRoute(...)
// survey a route between two cities, returning true &storing info in
// ROUTE[route] on successful survey
bool	route_Survey(int routenum,
						  int startcity,
						  int endcity);

// returns true if route between two cities is deemed "okay" due to city sizes
bool	route_Valid(int startcity,int endcity);

// returns true if the section can be built
bool	route_ValidSection(int routenum,int& ptr,int x,int y,int icon);

// plots a route onto the map
void	route_Plot(int routenum);

// highlights a full route
void	route_Highlight(int routenum);

// plots a section of a route onto the map using icon offset. ie. FIRST_BUILT
void	route_PlotSection(int routenum,int section,int offset);

// builds all required route sections
void	route_BuildSections(int routenum);

// builds the next section of a route
void	route_BuildNextSection(int routenum);


//bool	BendLeft(int routenum,int current_dirn,int howmuch);
//bool	BendRight(int routenum,int current_dirn,int howmuch);
//bool	TurnLeft(int routenum,int current_dirn,int x,int y);
//bool	TurnRight(int routenum,int current_dirn,int x,int y);
//bool	GoStraight(int routenum,int current_dirn,int x,int y);
//bool	Tunnel(int routenum,int current_dirn,int x,int y);
//bool	CrossRiver(int routenum,int current_dirn,int x,int y);

//bool	AddSection(int routenum,int& ptr,int x,int y,int what);
//bool	Backtrack(int routenum,int num_dots);

// returns true if the specified route is connected to 'rr'
bool	route_IsConnected(int route,int rr);

#endif
