//	-------------------------------------------------------------------------
//
//	MAP.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer & Andrew C. Evans
//
//	DESCRIPTION:	These are all of the bits required for the map
//
//	-------------------------------------------------------------------------

#include "map.hpp"
#include "stddefns.hpp"
#include "gamedefn.hpp"
#include "keyboard.hpp"
#include "pcxfile.hpp"

#include "city.hpp"
#include "route.hpp"
#include "uidefn.hpp"
//#include "terrain.hpp"

#include "dos.h"	// for delay()
#include "assert.h"

//
// 	DEBUGGING DEFINES
//

#define DRAW_MAP_ON_CREATION
//#define DRAW_TRACKS_ON_PLOTSECTION

//#define DRAW_ENLARGED
//#define DRAW_ON_SCALING


// GLOBALS

grVirtualBuffer*	terrain_map;	// base terrain data
grVirtualBuffer*	overlaid_map;	// overlaid terrain data
grVirtualBuffer*	height_map;		// terrain heights
grVirtualBuffer*	data_map_a;		// other data
grVirtualBuffer*	data_map_b;
grVirtualBuffer*	composite_map;	// composite map (1:1)

char				terrain_color_table[256]; 	// color lookup tables
char				overlaid_color_table[256];

// -------------------------------------------------------------------------
//
// BASIC MAP FUNCTIONS
//
// -------------------------------------------------------------------------


void	InitMaps()		// create space for all the required maps
	{
	// create color tables
	int color;
	for (color=0;color<256;color++)
		{
		terrain_color_table[color] = grBLACK;
		overlaid_color_table[color] = grBLACK;
		}
	terrain_color_table[sea] = grBLUE;
	terrain_color_table[swamp] = grMAGENTA;
	terrain_color_table[grass] = grLIGHTGREEN;
	terrain_color_table[desert] = grYELLOW;
	terrain_color_table[plains] = grGREEN;
	terrain_color_table[hills] = grDARKGRAY;
	terrain_color_table[mountains] = grLIGHTGRAY;
//	terrain_color_table[forest] = grLIGHTGREEN;

	for (color=FIRST_STATION;color<=LAST_STATION;color++)
		overlaid_color_table[color] = grLIGHTRED;
	for (color=FIRST_SURVEYED;color<=LAST_SURVEYED;color++)
		overlaid_color_table[color] = grDARKGRAY;
	for (color=FIRST_BUILT;color<=LAST_BUILT;color++)
		overlaid_color_table[color] = grLIGHTGRAY;
	for (color=FIRST_CITY;color<=LAST_CITY;color++)
		overlaid_color_table[color] = grRED;

	for (color=FIRST_HIGHLIGHTED;color<=LAST_HIGHLIGHTED;color++)
		overlaid_color_table[color] = grWHITE;

	// create terrain map
	terrain_map = grCreateVirtual(map_r.left,map_r.top,map_r.right,map_r.bottom,false);
	if (!terrain_map)
		{
		DBG(470,"ERROR (MAP): Could not create Terrain Map");
		delay(2000);
		}

	// create overlaid map
	overlaid_map = grCreateVirtual(map_r.left,map_r.top,map_r.right,map_r.bottom,false);
	if (!overlaid_map)
		{
		DBG(470,"ERROR (MAP): Could not create Overlaid Map");
		delay(2000);
		}
	else
		grClearVirtual(overlaid_map,NOT_OVERLAID);

	// create our terrain heights map
	height_map = grCreateVirtual(map_r.left,map_r.top,map_r.right,map_r.bottom,false);
	if (!height_map)
		{
		DBG(470,"ERROR (MAP): Could not create Terrain Heights Map");
		delay(2000);
		}

	// create the composite map storage space
	composite_map = grCreateVirtual(map_r.left,map_r.top,map_r.right,map_r.bottom,false);
	if (!composite_map)
		{
		DBG(470,"ERROR (MAP): Could not create Composite Map");
		delay(2000);
		}

	// create the data map storage space
	data_map_a = grCreateVirtual(map_r.left,map_r.top,map_r.right,map_r.bottom,true);
	if (!data_map_a)
		{
		DBG(470,"ERROR (MAP): Could not create Data Map A");
		delay(2000);
		}
	// create the data map storage space
	data_map_b = grCreateVirtual(map_r.left,map_r.top,map_r.right,map_r.bottom,true);
	if (!data_map_b)
		{
		DBG(470,"ERROR (MAP): Could not create Data Map B");
		delay(2000);
		}
	}


void	ExitMaps()
	{
	// remove our maps
	if (data_map_b)				grDestroyVirtual(data_map_b);
	if (data_map_a)				grDestroyVirtual(data_map_a);
	if (composite_map)			grDestroyVirtual(composite_map);
	if (height_map) 			grDestroyVirtual(height_map);
	if (overlaid_map) 			grDestroyVirtual(overlaid_map);
	if (terrain_map) 			grDestroyVirtual(terrain_map);

	HEAPCHK("ERROR: Heap is corrupted at end of MAPSCR destructor");
	}



void	SetTerrain(int x,int y,char terrain)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	int pos = x + y * map_width;
	terrain_map->data[pos] = terrain;
	composite_map->data[pos] = terrain_color_table[terrain];
	}

void	SetOverlaid(int x,int y,char overlaid)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	int pos = x + y * map_width;
	overlaid_map->data[pos] = overlaid;
	composite_map->data[pos] = overlaid_color_table[overlaid];
	}

void	SetTerrainHeight(int x,int y,char height)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	int pos = x + y * map_width;
	height_map->data[pos] = height;
	}

void	SetData(int x,int y,int	data)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	int pos = x + y * map_width;

	char lo = data & 0xFF;
	char hi = (data >> 8) & 0xFF;

	data_map_a->data[pos] = lo;
	data_map_b->data[pos] = hi;
	}


char	GetTerrain(int x,int y)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	return terrain_map->data[x + y * map_width];
	}

char	GetOverlaid(int x,int y)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	return overlaid_map->data[x + y * map_width];
	}

char	GetTerrainHeight(int x,int y)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	return height_map->data[x + y * map_width];
	}

int		GetData(int x,int y)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	int pos = x + y * map_width;

	char lo = data_map_a->data[pos];
	char hi = data_map_b->data[pos];

	int val = lo + (hi << 8);

	return val;
	}

int 	WhatsAt(int x,int y)
	{
	int data = GetData(x,y);

//	DBG(10,"WHATSAT:: GetData(%d,%d) returns %d",x,y,data);

	if ((data>=CITY_START_OFFSET) && (data<=CITY_END_OFFSET))
		{
		return MCITY;
		}

	if ((data>=ROUTE_START_OFFSET) && (data<=ROUTE_END_OFFSET))
		{
		return MROUTE;
		}

	return MTERRAIN;
	}

int		DistBetweenPts(int x1,int y1,int x2,int y2)
	{
	int vert = 0;
	int horiz = 0;
	int dist = 0;

	// figure out the absolute x distance in pixels
	if (x1>x2) horiz = x1-x2;
	else horiz = x2-x1;

	// and the absolute y distance in pixels
	if (y1>y2) vert = y1-y2;
	else vert = y2-y1;

	// figure out the length of the connecting line using pythagorus...
	if ((horiz!=0)||(vert!=0))
		{
		dist = sqrt( pow(horiz,2) + pow(vert,2) );
		}
	else
		dist = 0;

	// and return it...
	return dist;
	}


// -------------------------------------------------------------------------
//
// MAP GENERATION VARIABLES
//
// -------------------------------------------------------------------------

// storage for global variables (with some pre-defined values)

int ctrl_pts_x = 98;
int	ctrl_pts_y = 98;

int minimum_height		= 20;	// from min height to sea max is for lakes
int sea_max				= 10 + minimum_height;
int swamp_max			= 10 + sea_max;
int grass_max			= 10 + swamp_max;
int plains_max			= 10 + grass_max;
int desert_max			= 10 + plains_max;
int hills_max			= 10 + desert_max;
int mountains_max		= 10 + hills_max;
int maximum_height		= mountains_max+10;

// number of seperate starting locations for:
int num_lakes			= 20;
int num_swamp			= 5;
int num_grass			= 5;
int num_plains			= 5;
int num_desert			= 5;
int num_hills			= 10;
int num_mountains		= 5;

// how long a mountain range will grow
int range_length		= 10;

// how far they spread:
int spread_lakes		= 50;
int spread_swamp		= 50;
int spread_grass		= 1000;
int spread_plains		= 5000;
int spread_desert		= 100;
int spread_hills		= 100;
int spread_mountains	= 50;

// how much the height will change from control point to control point
// keep as odd numbers
int variation_lakes		= 3;
int variation_swamp		= 3;
int variation_grass		= 3;
int variation_plains	= 3;
int variation_desert	= 3;
int variation_hills		= 3;
int variation_mountains	= 3;

// -------------------------------------------------------------------------
//
// MAP GENERATION FUNCTIONS
//
// -------------------------------------------------------------------------

void 	CreateBaseTerrain()
	{
	// --- load our template map
	DBG(470,"Loading template map MAPS\\AMERICA.PCX");
	// create our template file
	PCXFile* 	template_file = new PCXFile("maps\\america.pcx");
	if (!template_file)
		{
		DBG(470,"ERROR (MAPSCR): Could not create template file!!!");
		delay(2000);
		return;
		}

	// load the template image
	grImage*	template_map = template_file->LoadImage();

	grCopyImageData(height_map,template_map);

	// clean up
	grDestroyImage(template_map);
	delete template_file;
	}

void	CreateRandomMap()
	{
	// create random terrain types
	CreateRandomTerrain();
	}

void	CreateHistoricMap()
	{
	CreateRandomMap();
	}

void	CreateRandomTerrain()
	{
	// create grass,plains,etc.
	CreateBaseTerrain();
	Generate3DMap();							//	CreateRandomMountains();

	CreateRandomForests();
	CreateRandomRivers();

	CreateIcons();
	}



void 	CreateRandomForests()
	{
/*	// create forests

	DBG(470,"Creating forests");
	int x,y,terrain;

	for (int num_forests=0;num_forests<5;num_forests++)
		{
		x = random(map_width);
		y = random(map_height);

		for (int forest_size=0;forest_size<2000;forest_size++)
			{
			do
				{
				x += random(3)-1;
				y += random(3)-1;

				CLIP2(x,y,0,0,map_width-1,map_height-1);

				terrain = GetTerrain(x,y);
				}
			while ((terrain==sea) || (terrain==mountains) || (terrain==forest));

			SetTerrain(x,y,forest);
			}
		}
*/	}

void 	CreateRandomRivers()
	{
	DBG(470,"Creating rivers");
	}

void 	CreateIcons()
	{
	DBG(470,"Creating Icon numbers");

	// --- convert the template data as required

	// scan through the height_map map
	for (int y=0;y<map_height;y++)
		for (int x=0;x<map_width;x++)
			{
			int height = GetTerrainHeight(x,y);

			if (height <= sea_max)
				{
				SetTerrain(x,y,sea);

				continue;
				}

			if ((height > sea_max) && (height <= swamp_max))
				{
				SetTerrain(x,y,swamp);

				continue;
				}

			if ((height > swamp_max) && (height <= grass_max))
				{
				SetTerrain(x,y,grass);

				continue;
				}

			if ((height > grass_max) && (height <= plains_max))
				{
				SetTerrain(x,y,plains);

				continue;
				}

			if ((height > plains_max) && (height <= desert_max))
				{
				SetTerrain(x,y,desert);

				continue;
				}

			if ((height > plains_max) && (height <= hills_max))
				{
				SetTerrain(x,y,hills);

				continue;
				}

			if (height > hills_max)
				{
				SetTerrain(x,y,mountains);

				continue;
				}
			}
	}

// --------------------------------------------------
// implementing a fixed point number system
// --------------------------------------------------

#ifndef fp
#define fp long
#define FixedPointShiftAmount 8

#define FPToInteger(FP)		((FP) >> FixedPointShiftAmount)
#define IntegerToFP(Int)	((Int) << FixedPointShiftAmount)
inline fp FPMultFP(fp a,fp b) { return (fp)((long)((long)(a) * (long)(b)) >> FixedPointShiftAmount); }
inline fp FPDivFP(fp a,fp b) { return (fp)(((long)(a) << FixedPointShiftAmount) / (long)(b)); }
#endif


#define grVertical		1
#define grHorizontal	2

FILE* fd;
//---------------------------------------------------------------------------
// draws a bspline into a virtual buffer in either _vertical_ or _horizontal_
// mode, assuming that the calculated values are stored in the z-plane.

// note: this is done using a 2-D spline curve projected onto a 3D map.
//
// params:	max_width 	- the maximum spline length
//			num_ctrl_pts- the number of control points along the length
// 			pos 		- the x or y position to store the data
//			mode		- iff "vertical", will use 'pos' to store the
//						  data going down the map.
//						  iff "horizontal", will use 'pos' to store the
//						  data going left-to-right on the map.
//
void grDrawFixedPointBSpline3D(grVirtualBuffer* virt,grPoint* ControlPoints,
					  int max_width,int num_ctrl_pts,
					  int pos,int mode)
	{
//	--- draw the Bezier-Spline-Curve

	int i,k,m,x,y;

	fp fpt;
	fp fpTSquared,fpTCubed,fpOneMinusT,fpOneMinusTSquared,fpOneMinusTCubed;
	fp fpVal1,fpVal2;

	int n = max_width/num_ctrl_pts;
//	fp fpn = FPDivFP(IntegerToFP(max_width),IntegerToFP(num_ctrl_pts));
	fp fpdelta = IntegerToFP(1)/n;
//	fp fpdelta = FPDivFP(IntegerToFP(1),fpn);

	k = 0;
	for (m=3;m<num_ctrl_pts;m++)
		{
		fprintf(fd,"Position %ld\n",(pos));
		grPoint Pminus3 = ControlPoints[k];
		grPoint Pminus2 = ControlPoints[k+1];
		grPoint Pminus1 = ControlPoints[k+2];
		grPoint P = ControlPoints[k+3];
		k++;

		for (i=1;i<=n;i++)
//		for (i=1;i<=FPToInteger(fpn);i++)
			{
			fpt = i * fpdelta;

			fpTSquared  = FPMultFP(fpt,fpt);
			fpTCubed	= FPMultFP(fpTSquared,fpt);

			fpOneMinusT        = IntegerToFP(1) - fpt;
			fpOneMinusTSquared = FPMultFP(fpOneMinusT,fpOneMinusT);
			fpOneMinusTCubed   = FPMultFP(fpOneMinusTSquared,fpOneMinusT);

			fpVal1			 = 3*(fpTCubed -2*fpTSquared) + IntegerToFP(4);
			fpVal2			 = 3*(fpTSquared + fpt - fpTCubed) + IntegerToFP(1);
			fprintf(fd,"TCubed %ld\n",(fpTCubed));
			fprintf(fd,"OneMinusTCubed %d\n",(fpOneMinusTCubed));
			fprintf(fd,"val1 %ld\n",(fpVal1));
			fprintf(fd,"val2 %ld\n",(fpVal2));

			fprintf(fd,"P3.x %ld\n",(Pminus3.x));
			fprintf(fd,"P2.x %ld\n",(Pminus2.x));
			fprintf(fd,"P1.x %ld\n",(Pminus1.x));
			fprintf(fd,"P .x %ld\n",(P.x));
			fp fp1 = (fp)Pminus3.x*fpOneMinusTCubed;
			fp fp2 = (fp)Pminus2.x*fpVal1;
			fp fp3 = (fp)Pminus1.x*fpVal2;
			fp fp4 = (fp)P.x*fpTCubed;

			fp fp5 = (fp1 + fp2 + fp3 + fp4)/6;

			x = FPToInteger(fp5);

			fprintf(fd,"P3.y %ld\n",(Pminus3.y));
			fprintf(fd,"P2.y %ld\n",(Pminus2.y ));
			fprintf(fd,"P1.y %ld\n",(Pminus1.y));
			fprintf(fd,"P .y %ld\n",(P.y));
			fp1 = (fp)Pminus3.y*fpOneMinusTCubed;
			fp2 = (fp)Pminus2.y*fpVal1;
			fp3 = (fp)Pminus1.y*fpVal2;
			fp4 = (fp)P.y*fpTCubed;

			fp5 = (fp1 + fp2 + fp3 + fp4)/6;

			y = FPToInteger(fp5);

			fprintf(fd,"All previous data for %ld,%ld == %ld\n",(pos),x,y);
			fprintf(fd,"==================================================\n");

			if (mode==grVertical)	grPutPixelVirtual(virt,pos,x,y);
			else					grPutPixelVirtual(virt,x,pos,y);
			}
		}
	}

//---------------------------------------------------------------------------
grPoint ControlPoints[50005];	// heaps of room + small error margin??

void Generate3DMap()
	{
	int ctrl_pt_width = map_width / ctrl_pts_x;
	int ctrl_pt_height = map_height / ctrl_pts_y;

	int x,y,xx[MAX_RANGE_LENGTH],yy[MAX_RANGE_LENGTH],z,i,j;

	if (height_map)
		{
		// generate control points
		GenerateBase(grass_max-5);

		for (i=0;i<num_plains;i++)
			{
			do	{
				x = random(ctrl_pts_x);
				y = random(ctrl_pts_y);
				} while (grGetPixelVirtual(height_map,x*ctrl_pt_width,y*ctrl_pt_height) == template_sea);

			GenerateBumps(x,y,spread_plains,variation_plains,grass_max,plains_max);
			}

		for (i=0;i<num_grass;i++)
			{
			do	{
				x = random(ctrl_pts_x);
				y = random(ctrl_pts_y);
				} while (grGetPixelVirtual(height_map,x*ctrl_pt_width,y*ctrl_pt_height) == template_sea);

			GenerateBumps(x,y,spread_grass,variation_grass,swamp_max,grass_max);
			}

		for (i=0;i<num_swamp;i++)
			{
			do	{
				x = random(ctrl_pts_x);
				y = random(ctrl_pts_y);
				} while (grGetPixelVirtual(height_map,x*ctrl_pt_width,y*ctrl_pt_height) == template_sea);

			GenerateBumps(x,y,spread_swamp,variation_swamp,sea_max,swamp_max);
			}

		for (i=0;i<num_desert;i++)
			{
			do	{
				x = random(ctrl_pts_x);
				y = random(ctrl_pts_y);
				} while (grGetPixelVirtual(height_map,x*ctrl_pt_width,y*ctrl_pt_height) == template_sea);

			GenerateBumps(x,y,spread_desert,variation_desert,plains_max,desert_max);
			}

		for (i=0;i<num_hills;i++)
			{
			do	{
				x = random(ctrl_pts_x);
				y = random(ctrl_pts_y);
				} while (grGetPixelVirtual(height_map,x*ctrl_pt_width,y*ctrl_pt_height) == template_sea);

			GenerateBumps(x,y,spread_hills,variation_hills,plains_max,hills_max);
			}

		for (i=0;i<num_mountains;i++)
			{
			do	{
				x = random(ctrl_pts_x);
				y = random(ctrl_pts_y);
				} while (grGetPixelVirtual(height_map,x*ctrl_pt_width,y*ctrl_pt_height) == template_sea);

			int direction = random(8);

			for (j=0;j<range_length;j++)
				{
				if (j==0)
					{
					xx[j] = x;
					yy[j] = y;
					}
				else
					{
					switch (direction)
						{
						case 0:
							xx[j] = xx[j-1] + (random(3)-1)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] - random(3)*range_length/ctrl_pt_height;
							break;
						case 1:
							xx[j] = xx[j-1] + random(3)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] - random(3)*range_length/ctrl_pt_height;
							break;
						case 2:
							xx[j] = xx[j-1] + random(3)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] + (random(3)-1)*range_length/ctrl_pt_height;
							break;
						case 3:
							xx[j] = xx[j-1] + random(3)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] + random(3)*range_length/ctrl_pt_height;
							break;
						case 4:
							xx[j] = xx[j-1] + (random(3)-1)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] + random(3)*range_length/ctrl_pt_height;
							break;
						case 5:
							xx[j] = xx[j-1] - random(3)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] + random(3)*range_length/ctrl_pt_height;
							break;
						case 6:
							xx[j] = xx[j-1] - random(3)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] + (random(3)-1)*range_length/ctrl_pt_height;
							break;
						case 7:
							xx[j] = xx[j-1] - random(3)*range_length/ctrl_pt_width;
							yy[j] = yy[j-1] - random(3)*range_length/ctrl_pt_height;
							break;
						}
					}
				GenerateBumps(xx[j],yy[j],spread_mountains,variation_mountains,hills_max,mountains_max);
				}
			}

		for (i=0;i<num_lakes;i++)
			{
			do	{
				x = random(ctrl_pts_x);
				y = random(ctrl_pts_y);
				} while (grGetPixelVirtual(height_map,x*ctrl_pt_width,y*ctrl_pt_height) == template_sea);

			GenerateBumps(x,y,spread_lakes,variation_lakes,minimum_height,sea_max);
			}

		fd = fopen("a.dat","w");

		// draw a number of splines from the top to the bottom of the map
		// at fixed intervals.
		for (x=0;x<ctrl_pts_x;x++)
			{
			grDrawFixedPointBSpline3D(height_map,&ControlPoints[x*ctrl_pts_y],map_height,ctrl_pts_y,x*ctrl_pt_width,grVertical);
			}

		// for each row of the map, use the control points (created above)
		// to draw a bspline....
		for (y=0;y<map_height;y++)
			{
			for (x=0;x<ctrl_pts_x;x++)
				{
				ControlPoints[x].x = x * ctrl_pt_width;
				ControlPoints[x].y = grGetPixelVirtual(height_map,x*ctrl_pt_width,y);
				}

			grDrawFixedPointBSpline3D(height_map,ControlPoints,map_width,ctrl_pts_x,y,grHorizontal);
			}
		}
	fclose(fd);
	}

//---------------------------------------------------------------------------
void GenerateBase(int base_height)
	{
	int x,y;
	int ctrl_pt_width = map_width / ctrl_pts_x;
	int ctrl_pt_height = map_height / ctrl_pts_y;

	// create a grid of b-spline control points to represent a
	// large area of plains/grasslands/etc. (with random height)
	int index;
	for (x=0;x<ctrl_pts_x;x++)
		{
		index = x * ctrl_pts_y;
		for (y=0;y<ctrl_pts_y;y++)
			{
			ControlPoints[index+y].x = y * ctrl_pt_height;

			if (grGetPixelVirtual(height_map, x*ctrl_pt_width, y*ctrl_pt_height) > template_sea)
				ControlPoints[index+y].y = base_height;
			else
				ControlPoints[index+y].y = minimum_height;
			}
		}
	}

//---------------------------------------------------------------------------
void GenerateBumps(int x, int y, int spread,int variation,int min,int max)
	{
	int z,i,j;
	int ctrl_pt_width = height_map->hdr.width / ctrl_pts_x;
	int ctrl_pt_height = height_map->hdr.height / ctrl_pts_y;

	// figure out an approximate height for the mountain
	int last_height = random(max-min) + min;

	// for each element of the mountain range
	for (j=0;j<spread;j++)
		{
		// choose a random point nearby.... (+-1 for both x,y)
		int tmpx=x,tmpy=y;
		int count=0;
		bool finished = false;
		do
			{
			count++;
			x += random(3) - 1;
			y += random(3) - 1;

			finished = ((x!=tmpx) || (y!=tmpy));
			finished &= (grGetPixelVirtual(height_map,x*ctrl_pt_width,y*ctrl_pt_height)>template_sea);
			}
		while ((count<20) && !(finished));

		if (count>=20) continue;

		// bounds-check (not perfect but close...)
		if (x<0) x=0;
		if (y<0) y=0;
		if (x>=ctrl_pts_x) x = ctrl_pts_x-1;
		if (y>=ctrl_pts_y) y = ctrl_pts_y-1;

		// and modify ctrl pts around it...
		ControlPoints[x*ctrl_pts_y+y].x = y * ctrl_pt_height;

		int change;
		do
			{
			change = random(variation) - variation/2;
			}
		while ((last_height+change<=min) || (last_height+change>max));

		// and copy it back
		ControlPoints[x*ctrl_pts_y+y].y = last_height+change;
		}
	}
//---------------------------------------------------------------------------
