//	-------------------------------------------------------------------------
//
//	MAP.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	These are all of the bits required for the map
//
//	-------------------------------------------------------------------------

#include "map.hpp"
#include "program.hpp"
#include "stddefns.hpp"
#include "gamedefn.hpp"
#include "keyboard.hpp"
#include "pcxfile.hpp"

#include "city.hpp"
#include "route.hpp"
#include "uidefn.hpp"
//#include "terrain.hpp"

#include "dos.h"	// for delay()

//
// 	DEBUGGING DEFINES
//

#define DRAW_MAP_ON_CREATION
//#define DRAW_TRACKS_ON_PLOTSECTION

//#define DRAW_ENLARGED
//#define DRAW_ON_SCALING


// GLOBALS

grVirtualBuffer*	terrain_map;	// base terrain data
grVirtualBuffer*	overlaid_map;	// overlaid terrain data
grVirtualBuffer*	height_map;		// terrain heights
grVirtualBuffer*	data_map_a;		// other data
grVirtualBuffer*	data_map_b;
grVirtualBuffer*	composite_map;	// composite map (1:1)

char				terrain_color_table[256]; 	// color lookup tables
char				overlaid_color_table[256];

// -------------------------------------------------------------------------
//
// BASIC MAP FUNCTIONS
//
// -------------------------------------------------------------------------


void	InitMaps()		// create space for all the required maps
	{
	// create color tables
	int color;
	for (color=0;color<256;color++)
		{
		terrain_color_table[color] = grBLACK;
		overlaid_color_table[color] = grBLACK;
		}
	terrain_color_table[sea] = grBLUE;
	terrain_color_table[grass] = grGREEN;
	terrain_color_table[forest] = grLIGHTGREEN;
	terrain_color_table[mountains] = grDARKGRAY;

	for (color=FIRST_STATION;color<=LAST_STATION;color++)
		overlaid_color_table[color] = grLIGHTRED;
	for (color=FIRST_SURVEYED;color<=LAST_SURVEYED;color++)
		overlaid_color_table[color] = grDARKGRAY;
	for (color=FIRST_BUILT;color<=LAST_BUILT;color++)
		overlaid_color_table[color] = grLIGHTGRAY;
	for (color=FIRST_CITY;color<=LAST_CITY;color++)
		overlaid_color_table[color] = grRED;

	for (color=FIRST_HIGHLIGHTED;color<=LAST_HIGHLIGHTED;color++)
		overlaid_color_table[color] = grWHITE;

	// create terrain map
	terrain_map = grCreateVirtual(map_r.left,map_r.top,map_r.right,map_r.bottom,false);
	if (!terrain_map)
		{
		DBG(470,"ERROR (MAP): Could not create Terrain Map");
		delay(2000);
		}

	// create overlaid map
	overlaid_map = grCreateVirtual(map_r.left,map_r.top,map_r.right,map_r.bottom,false);
	if (!overlaid_map)
		{
		DBG(470,"ERROR (MAP): Could not create Overlaid Map");
		delay(2000);
		}
	else
		grClearVirtual(overlaid_map,NOT_OVERLAID);

	// create our terrain heights map
	height_map = grCreateVirtual(map_r.left,map_r.top,map_r.right,map_r.bottom,false);
	if (!height_map)
		{
		DBG(470,"ERROR (MAP): Could not create Terrain Heights Map");
		delay(2000);
		}

	// create the composite map storage space
	composite_map = grCreateVirtual(map_r.left,map_r.top,map_r.right,map_r.bottom,false);
	if (!composite_map)
		{
		DBG(470,"ERROR (MAP): Could not create Composite Map");
		delay(2000);
		}

	// create the data map storage space
	data_map_a = grCreateVirtual(map_r.left,map_r.top,map_r.right,map_r.bottom,true);
	if (!data_map_a)
		{
		DBG(470,"ERROR (MAP): Could not create Data Map A");
		delay(2000);
		}
	// create the data map storage space
	data_map_b = grCreateVirtual(map_r.left,map_r.top,map_r.right,map_r.bottom,true);
	if (!data_map_b)
		{
		DBG(470,"ERROR (MAP): Could not create Data Map B");
		delay(2000);
		}
	}


void	ExitMaps()
	{
	// remove our maps
	if (data_map_b)				grDestroyVirtual(data_map_b);
	if (data_map_a)				grDestroyVirtual(data_map_a);
	if (composite_map)			grDestroyVirtual(composite_map);
	if (height_map) 			grDestroyVirtual(height_map);
	if (overlaid_map) 			grDestroyVirtual(overlaid_map);
	if (terrain_map) 			grDestroyVirtual(terrain_map);

	HEAPCHK("ERROR: Heap is corrupted at end of MAPSCR destructor");
	}



void	SetTerrain(int x,int y,char terrain)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	int pos = x + y * map_width;
	terrain_map->data[pos] = terrain;
	composite_map->data[pos] = terrain_color_table[terrain];
	}

void	SetOverlaid(int x,int y,char overlaid)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	int pos = x + y * map_width;
	overlaid_map->data[pos] = overlaid;
	composite_map->data[pos] = overlaid_color_table[overlaid];
	}

void	SetTerrainHeight(int x,int y,char height)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	int pos = x + y * map_width;
	height_map->data[pos] = height;
	}

void	SetData(int x,int y,int	data)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	int pos = x + y * map_width;

	char lo = data & 0xFF;
	char hi = (data >> 8) & 0xFF;

	data_map_a->data[pos] = lo;
	data_map_b->data[pos] = hi;
	}


char	GetTerrain(int x,int y)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	return terrain_map->data[x + y * map_width];
	}

char	GetOverlaid(int x,int y)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	return overlaid_map->data[x + y * map_width];
	}

char	GetTerrainHeight(int x,int y)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	return height_map->data[x + y * map_width];
	}

int		GetData(int x,int y)
	{
	CLIP2(x,y,0,0,map_width-1,map_height-1);
	int pos = x + y * map_width;

	char lo = data_map_a->data[pos];
	char hi = data_map_b->data[pos];

	int val = lo + (hi << 8);

	return val;
	}

int 	WhatsAt(int x,int y)
	{
	int data = GetData(x,y);

//	DBG(10,"WHATSAT:: GetData(%d,%d) returns %d",x,y,data);

	if ((data>=CITY_START_OFFSET) && (data<=CITY_END_OFFSET))
		{
		return MCITY;
		}

	if ((data>=ROUTE_START_OFFSET) && (data<=ROUTE_END_OFFSET))
		{
		return MROUTE;
		}

	return MTERRAIN;
	}

int		DistBetweenPts(int x1,int y1,int x2,int y2)
	{
	int vert = 0;
	int horiz = 0;
	int dist = 0;

	// figure out the absolute x distance in pixels
	if (x1>x2) horiz = x1-x2;
	else horiz = x2-x1;

	// and the absolute y distance in pixels
	if (y1>y2) vert = y1-y2;
	else vert = y2-y1;

	// figure out the length of the connecting line using pythagorus...
	if ((horiz!=0)||(vert!=0))
		{
		dist = sqrt( pow(horiz,2) + pow(vert,2) );
		}
	else
		dist = 0;

	// and return it...
	return dist;
	}


// -------------------------------------------------------------------------
//
// MAP GENERATION FUNCTIONS
//
// -------------------------------------------------------------------------


void 	CreateBaseTerrain()
	{
	// --- load our template map
	DBG(470,"Loading template map MAPS\\AMERICA.PCX");
	// create our template file
	PCXFile* 	template_file = new PCXFile(PROGRAM_PATH,"maps\\america.pcx");
	if (!template_file)
		{
		DBG(470,"ERROR (MAPSCR): Could not create template file!!!");
		delay(2000);
		return;
		}

	// load the template image
//	DBG(470,"HERE");
	grImage*	template_map = template_file->LoadImage();

	// copy it to the terrain heights
//	DBG(470,"HERE1");
	int sizx = min(height_map->hdr.width,template_map->hdr.width);
	int sizy = min(height_map->hdr.height,template_map->hdr.height);
	memcpy(&height_map->data[0],&template_map->data[0],sizx * sizy);

	// clean up
//	DBG(470,"HERE2");
	grDestroyImage(template_map);
	delete template_file;

	// --- convert the template data as required

	DBG(470, "Converting template map");

	// scan through the height_map map
	for (int y=0;y<map_height;y++)
		for (int x=0;x<map_width;x++)
			// if we find sea
			if (GetTerrainHeight(x,y) == template_sea)
				{
				// set the terrain to sea
				SetTerrain(x,y,sea);
				}
			else
				{
				// set the terrain to grass
				SetTerrain(x,y,grass);
				}
	}

void	CreateRandomMap()
	{
	// create random terrain types
	CreateRandomTerrain();
	}

void	CreateHistoricMap()
	{
	CreateRandomMap();
	}

void	CreateRandomTerrain()
	{
	// create grass,plains,etc.
	CreateBaseTerrain();
	CreateRandomMountains();
	CreateRandomForests();
	CreateRandomRivers();
	}


void	CreateRandomMountains()
	{
	// create mountains

	DBG(470,"Creating mountain ranges");

	int x,y,terrain;

	for (int num_ranges=0;num_ranges<5;num_ranges++)
		{
		x = random(map_width);
		y = random(map_height);

		for (int range_size=0;range_size<2000;range_size++)
			{
			do
				{
				x += random(3)-1;
				y += random(3)-1;

				CLIP2(x,y,0,0,map_width-1,map_height-1);

				terrain = GetTerrain(x,y);
				}
			while ((terrain==sea) || (terrain==mountains));

			SetTerrain(x,y,mountains);
			}
		}
	}


void 	CreateRandomForests()
	{
	// create forests

	DBG(470,"Creating forests");
	int x,y,terrain;

	for (int num_forests=0;num_forests<5;num_forests++)
		{
		x = random(map_width);
		y = random(map_height);

		for (int forest_size=0;forest_size<2000;forest_size++)
			{
			do
				{
				x += random(3)-1;
				y += random(3)-1;

				CLIP2(x,y,0,0,map_width-1,map_height-1);

				terrain = GetTerrain(x,y);
				}
			while ((terrain==sea) || (terrain==mountains) || (terrain==forest));

			SetTerrain(x,y,forest);
			}
		}
	}

void 	CreateRandomRivers()
	{
	DBG(470,"Creating rivers");
	}




