//	-------------------------------------------------------------------------
//
//	SETUP.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the code for the SETUP SCREEN
//
//	-------------------------------------------------------------------------

#include "setup.hpp"
#include "gamedefn.hpp"
#include "eventmgr.hpp"
#include "buttons.hpp"

#include "player.hpp"
#include "railroad.hpp"
#include "economy.hpp"
#include "city.hpp"
#include "program.hpp"
#include "borders.hpp"
#include "guidefn.hpp"

#include "dos.h"	// for delay()

#define	cmdRandom		30000
#define cmdIncPlayers	30001
#define	cmdDecPlayers	30002
#define	cmdIncCities	30003
#define	cmdDecCities	30004
#define	cmdIncRailroads	30005
#define	cmdDecRailroads	30006
#define	cmdGenerate		30007

SetupScreen::SetupScreen()
	: OnScreenHandler(0,0,grMaxx,grMaxy,
					  &evhno_SetupScreen,0,1)
	{
	AddAccessory(new TextButton(100,400,TITLEFONT,"  GENERATE NEW GAME  ",0,cmdGenerate));
	AddAccessory(new TextButton(400,400,TITLEFONT,"  QUIT GAME  ",0,cmdTerminate));

	AddAccessory(new TextButton(300,100,TITLEFONT," CHANGE ",0,cmdRandom));
	AddAccessory(new TextButton(300,130,TITLEFONT," + ",ac_AutoMode,cmdIncPlayers));
	AddAccessory(new TextButton(340,130,TITLEFONT," - ",ac_AutoMode,cmdDecPlayers));
	AddAccessory(new TextButton(300,160,TITLEFONT," + ",ac_AutoMode,cmdIncCities));
	AddAccessory(new TextButton(340,160,TITLEFONT," - ",ac_AutoMode,cmdDecCities));
	AddAccessory(new TextButton(300,190,TITLEFONT," + ",ac_AutoMode,cmdIncRailroads));
	AddAccessory(new TextButton(340,190,TITLEFONT," - ",ac_AutoMode,cmdDecRailroads));

	}

void	SetupScreen::DrawRandom(bool refresh)
	{
	grSetFont(SETUPFONT);
	if (refresh)
		{
		if (RANDOM_GAME) grBlit2CString(50,100,grDARKGRAY,TEXT_COLOR,"Random Game          : ON  ");
		else grBlit2CString(50,100,grDARKGRAY,TEXT_COLOR,"Random Game          : OFF");
		}
	else
		if (RANDOM_GAME) grDraw2CString(50,100,TEXT_COLOR,"Random Game          : ON  ");
		else grDraw2CString(50,100,TEXT_COLOR,"Random Game          : OFF");
	}

void	SetupScreen::DrawPlayers(bool refresh)
	{
	grSetFont(SETUPFONT);
	if (refresh) grBlit2CString(50,130,grDARKGRAY,TEXT_COLOR,"Number of Players    : %d  ",NUM_PLAYERS);
	else grDraw2CString(50,130,TEXT_COLOR,"Number of Players    : %d",NUM_PLAYERS);
	}

void	SetupScreen::DrawCities(bool refresh)
	{
	grSetFont(SETUPFONT);

	if (refresh) grBlit2CString(50,160,grDARKGRAY,TEXT_COLOR,"Number of Cities     : %d  ",NUM_CITIES);
	else grDraw2CString(50,160,TEXT_COLOR,"Number of Cities     : %d",NUM_CITIES);
	}

void	SetupScreen::DrawRailroads(bool refresh)
	{
	grSetFont(SETUPFONT);
	if (refresh) grBlit2CString(50,190,grDARKGRAY,TEXT_COLOR,"Number of Railroads  : %d  ",NUM_RRS);
	else grDraw2CString(50,190,TEXT_COLOR,"Number of Railroads  : %d",NUM_RRS);
	}

void SetupScreen::Draw()
	{
	int bord = grInitBorder(PROGRAM_PATH,"RESOURCE\\SETUP.BRD");

	// draw basic window
	grTileRect(r.left,r.top,r.right,r.bottom,-1,GUI_TILES[SCREEN_TILE],NO_ROTATE);
	grDrawBorder(r.left,r.top,r.right,r.bottom,bord);

	grExitBorder();

	grSetFont(HUGEFONT);
	grDraw2CString(100,10,TEXT_COLOR,"GAME SETUP");

	DrawRandom(false);
	DrawPlayers(false);
	DrawCities(false);
	DrawRailroads(false);

	// display all buttons
	DrawAllAccessories();
	}

extern	int	last_ax;
extern	int	last_bx;
extern	int	last_cx;
extern	int	last_dx;

void SetupScreen::HandleEvent(Event &e)
	{
	if (e.what==ev_mouse)
		{
//		DBG(0,"MOUSE ON");
		mouse_Show();
/*		mouse_Status mstat;
		mouse_GetStatus(&mstat);

		DBG(0,"MOUSE @ %d,%d",e.mouse.where.x,e.mouse.where.y);
		switch (e.mouse.event)
			{
			case mb_leftdn:		DBG(10,"LEFT DN");		break;
			case mb_leftup:     DBG(10,"LEFT UP");		break;
			case mb_leftauto:   DBG(10,"LEFT AUTO");	break;

			case mb_rightdn:	DBG(20,"RIGHT DN");		break;
			case mb_rightup:    DBG(20,"RIGHT UP");		break;
			case mb_rightauto:  DBG(20,"RIGHT AUTO");	break;

			case mb_middledn:	DBG(30,"MIDDLE DN");	break;
			case mb_middleup:   DBG(30,"MIDDLE UP");	break;
			case mb_middleauto: DBG(30,"MIDDLE AUTO");	break;

			case mb_move:		DBG(40,"MOVING");		break;
			}

		DBG(50,"LAST REGISTERS PASSED --- AX: %x  BX: %x  CX: %x  DX: %x ---",
			last_ax,last_bx,last_cx,last_dx);

*/		}


	// do all the _boring_ event processing
	OnScreenHandler::HandleEvent(e);

	switch (e.what)
		{
		case ev_mouse:

			mouse_Show();
			EVENTMGR->ClearEvent(e);
			break;

		case ev_command:

			switch (e.command.event)
				{
				case cmdRandom:

					RANDOM_GAME = !RANDOM_GAME;
					DrawRandom();
					break;

				case cmdIncPlayers:

					if (NUM_PLAYERS<MAX_PLAYERS)
						{
						NUM_PLAYERS++;
						DrawPlayers();
						}
					break;

				case cmdDecPlayers:

					if (NUM_PLAYERS>MIN_PLAYERS)
						{
						NUM_PLAYERS--;
						DrawPlayers();
						}
					break;

				case cmdIncCities:

					if (NUM_CITIES<MAX_CITIES)
						{
						NUM_CITIES++;
						DrawCities();
						}
					break;

				case cmdDecCities:

					if (NUM_CITIES>MIN_CITIES)
						{
						NUM_CITIES--;
						DrawCities();
						}
					break;

				case cmdIncRailroads:

					if (NUM_RRS<MAX_RRS)
						{
						NUM_RRS++;
						DrawRailroads();
						}
					break;

				case cmdDecRailroads:

					if (NUM_RRS>MIN_RRS)
						{
						NUM_RRS--;
						DrawRailroads();
						}
					break;

				case cmdGenerate: e.command.event = cmdExitProgram; return;
				case cmdTerminate: return;
				}

			EVENTMGR->ClearEvent(e);
			break;

		case ev_keyboard:

			switch (e.keyboard.event)
				{
				case S_UP:	DBG(60,"MOUSE CURSOR: ON  ");
							mouse_Show();
							break;
				case H_UP:	DBG(60,"MOUSE CURSOR: OFF ");
							mouse_Hide();
							break;
				}
			break;
		}
	}
