//	-------------------------------------------------------------------------
//
//	SYSTEM.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This object is used to display system information during
//					testing of a program.
//
//	-------------------------------------------------------------------------

#include "system.hpp"
#include "stddefns.hpp"
#include "timer.hpp"
#include "eventmgr.hpp"
#include "gamedefn.hpp"
#include "keyboard.hpp"
#include "keydefns.hpp"

#include "dos.h"
#include "stdlib.h"
#include "malloc.h"


#define __USING_TIMER_0
#ifdef 	__USING_TIMER_0
extern 	dostime_t	system_time;
#endif

uint 	system_animcolor = 0;

ulong 	last_coreleft = 0;
uint 	last_speed_rating = 0;
uint	last_pending = 0;

#ifdef __USING_TIMER_0
uint 	timing_rate = 0;
uint 	last_timing_rate = 0;
uint 	last_second = 60;
#endif

System::System()
	: OnScreenHandler(0,0,639,12,&evhno_System,0,0)
	{
	isRespondingKbd = false;
	isRespondingMouse = false;
	}

void System::Animate()
	{
	// select the next animation color & position
	++system_animcolor %= 256;
	int tmp = (system_animcolor % 16) *4;

	// do the rendering
	grDrawFilledRect(r.left,r.top,r.right,r.bottom,system_animcolor);
	// insert the number of event handlers in operation
	grDrawString(r.left+80,r.top,"HAND  %u",EVENTMGR->numEventHandlers);

	// insert the number of timer_tickers in operation
	grDrawString(r.left+180,r.top,"TICK  %u",timer_numTickers);

	// insert the number of events pending in EVENT QUEUE
	grDrawString(r.left+280,r.top,"PEND  %u",EVENTMGR->numEventsPending);

#ifdef __USING_TIMER_0
	timing_rate++;

	// rate over periods of 5 seconds (margin 1 second caJUSTse)
	if (system_time.second != last_second) // why doesn't this work??
		{
		last_second = system_time.second;

		last_timing_rate = timing_rate;
		timing_rate = 0;
		}

	grDrawString(r.left+380,r.top,"ANIM  %u",last_timing_rate);

#endif

	// display our heap status
	grDrawString(r.left+480,r.top,"HEAP");
	if (_heapchk()==_HEAPOK) grDrawString(r.left+520,r.top,"OKAY");
	else
		{
		grDrawString(r.left+520,r.top,"CORRUPT!");
		}

	// we've finished creating our virtual buffer, now it's time to blit...
	grBlitRegion(r.left,r.top,r.right,r.bottom);
	}


void System::HandleEvent(Event& e)
	{
	if (e.what==ev_keyboard)
		{
		if ((e.keyboard.event==S_UP) &&
			(e.keyboard.modifiers& (keyboard_CTRL|keyboard_ALT) ))
			{
			isOnScreen = !isOnScreen;
			EVENTMGR->ClearEvent(e);
			}
		}
	}

