//	-------------------------------------------------------------------------
//
//	TRAIN.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	These are all of the bits required for the trains
//
//	-------------------------------------------------------------------------

#include "train.hpp"
#include "stddefns.hpp"
#include "gamedefn.hpp"
#include "route.hpp"
#include "city.hpp"
#include "railroad.hpp"

#include "mapwin.hpp"

#include "dos.h"

// GLOBALS

int				NUM_TRAINS;
TrainStruct		TRAIN[MAX_TRAINS];


void	CreateRandomTrain()
	{
	}

void	CreateNewTrain(int rr,char train_type,int* check_points,int num_check_points)
	{
	int train = NUM_TRAINS;
	int	route = RAILROAD[rr].CharteredRoute;
	int city  = RAILROAD[rr].BUYTRAIN.CheckPoint[0];

	int startcity = ROUTE[route].StartCity;
	int endcity = ROUTE[route].EndCity;
	if (startcity==city) { startcity = endcity; endcity = city; }

	TRAIN[train].Owner	 = rr;
	TRAIN[train].Status  = ARRIVING;	// route is chosen on departure
	TRAIN[train].City	 = city;
	TRAIN[train].LastCity = -1;
	TRAIN[train].x[0]	 = CITY[startcity].x;
	TRAIN[train].y[0]	 = CITY[startcity].y;
	TRAIN[train].x[ROUTE[route].Length] = CITY[endcity].x;
	TRAIN[train].y[ROUTE[route].Length] = CITY[endcity].y;

	int i;
	for (i=1;i<MAX_CARS;i++)
		{
		TRAIN[train].lastx[i] = TRAIN[train].x[i];
		TRAIN[train].lasty[i] = TRAIN[train].y[i];
		TRAIN[train].x[i] = TRAIN[train].x[i-1];
		TRAIN[train].y[i] = TRAIN[train].y[i-1];
		}
	TRAIN[train].Speed  = 0;
	TRAIN[train].TrainType = train_type;

	for (i=0;i<MAX_CARS;i++)
		TRAIN[train].CarType[i] = i;//RAILROAD[rr].BUYTRAIN.CarType;

	for (i=0;i<num_check_points;i++)
		TRAIN[train].CheckPoint[i] = check_points[i];

	TRAIN[train].NumCheckPoints = num_check_points;
	TRAIN[train].NextCheckPoint = 0;

	NUM_TRAINS++;
	}

void	DestroyTrain(int train)
	{
	}



void	train_DoMove(int train)
	{
	// figure out how far train has moved
	int distance_moved = 1;

	// and if we've moved our minimum amount
	if (distance_moved>0)
		{
		// figure next position to be
		int route = TRAIN[train].Route;

		if (TRAIN[train].Status==MOVING_FORWARDS)
			{
			TRAIN[train].RoutePos += distance_moved;

			int routepos = TRAIN[train].RoutePos;
			for (int i=0;i<MAX_CARS;i++)
				{
				TRAIN[train].lastx[i] = TRAIN[train].x[i];
				TRAIN[train].lasty[i] = TRAIN[train].y[i];

				if (i>0)
					{
					TRAIN[train].x[i] = TRAIN[train].lastx[i-1];
					TRAIN[train].y[i] = TRAIN[train].lasty[i-1];
					}
				else
					{
					TRAIN[train].x[0] = ROUTE[route].x[routepos];
					TRAIN[train].y[0] = ROUTE[route].y[routepos];
					}
				}

			MAPWIN->DrawTrain(train);

			// figure if the train has arrived
			if (TRAIN[train].RoutePos >= ROUTE[route].BuiltLength)
				{
				TRAIN[train].Status = ARRIVING;
				TRAIN[train].City   = ROUTE[route].EndCity;
				}
			}
		else
			{
			TRAIN[train].RoutePos -= distance_moved;

			int routepos = TRAIN[train].RoutePos;
			for (int i=0;i<MAX_CARS;i++)
				{
				TRAIN[train].lastx[i] = TRAIN[train].x[i];
				TRAIN[train].lasty[i] = TRAIN[train].y[i];

				if (i>0)
					{
					TRAIN[train].x[i] = TRAIN[train].lastx[i-1];
					TRAIN[train].y[i] = TRAIN[train].lasty[i-1];
					}
				else
					{
					TRAIN[train].x[0] = ROUTE[route].x[routepos];
					TRAIN[train].y[0] = ROUTE[route].y[routepos];
					}
				}

			MAPWIN->DrawTrain(train);

			// figure if the train has arrived
			if (TRAIN[train].RoutePos <= 0)
				{
				TRAIN[train].Status = ARRIVING;
				TRAIN[train].City   = ROUTE[route].StartCity;
				}
			}
		}
	}

void	train_DoArrival(int train)
	{
	// unload the train
	train_Unload(train);

	// if we're arriving at a checkpoint, choose the next checkpoint
	if (TRAIN[train].CheckPoint[TRAIN[train].NextCheckPoint]==
		TRAIN[train].City)
		train_ChooseNextCheckPoint(train);

	// and we're now ready to depart
	TRAIN[train].Status = DEPARTING;
	}

void	train_DoDeparture(int train)
	{
	// load the train
	train_Load(train);

	// we're departing again
	TRAIN[train].Status = DEPARTING;

	// choose the next direction for the train to head
	train_ChooseNextRoute(train);

	// save the city that we're departing
	TRAIN[train].LastCity = TRAIN[train].City;


	// and figure out if the train is traversing the route forwards
	// or backwards
	int city = ROUTE[TRAIN[train].Route].StartCity;
	if (TRAIN[train].City == city)
		{
/*
		TRAIN[train].x[0] = CITY[city].x;
		TRAIN[train].y[0] = CITY[city].y;

		for (int i=1;i<MAX_CARS;i++)
			{
			TRAIN[train].lastx[i] = TRAIN[train].x[i];
			TRAIN[train].lasty[i] = TRAIN[train].y[i];
			TRAIN[train].x[i] = TRAIN[train].x[i-1];
			TRAIN[train].y[i] = TRAIN[train].y[i-1];
			}
*/
		TRAIN[train].Status = MOVING_FORWARDS;
		TRAIN[train].RoutePos = 0;
		}
	else
		{
/*		city = ROUTE[TRAIN[train].Route].EndCity;

		TRAIN[train].x[0] = CITY[city].x;
		TRAIN[train].y[0] = CITY[city].y;
		for (int i=1;i<MAX_CARS;i++)
			{
			TRAIN[train].lastx[i] = TRAIN[train].x[i];
			TRAIN[train].lasty[i] = TRAIN[train].y[i];
			TRAIN[train].x[i] = TRAIN[train].x[i-1];
			TRAIN[train].y[i] = TRAIN[train].y[i-1];
			}
*/
		TRAIN[train].Status = MOVING_BACKWARDS;
		TRAIN[train].RoutePos = ROUTE[TRAIN[train].Route].BuiltLength;
		}
	}



void	train_Load(int train)
	{
	TRAIN[train].Status = LOADING;
	}

void	train_Unload(int train)
	{
	TRAIN[train].Status = UNLOADING;
	}


void	train_ChooseNextCheckPoint(int train)
	{
	// simply choose the next checkpoint
	TRAIN[train].NextCheckPoint++;
	if (TRAIN[train].NextCheckPoint>=TRAIN[train].NumCheckPoints)
		TRAIN[train].NextCheckPoint = 0;
	}

void	train_ChooseNextRoute(int train)
	{
	// --- assuming we're in a city, figure out which route is the one
	//	   we want to use to get to the next checkpoint

	int current_city  		= TRAIN[train].City;
	int	last_city			= TRAIN[train].LastCity;
	int train_owner 		= TRAIN[train].Owner;
	int	next_checkpoint 	= TRAIN[train].NextCheckPoint;
	int next_checkpoint_city= TRAIN[train].CheckPoint[next_checkpoint];

	// scan through all connected cities, figuring out the closest
	// connected city to the next checkpoint

/*	DBG(0,"TRAIN #%d  OWNER %d   AT CITY %d     WAS AT CITY %d",
			train,train_owner,current_city,last_city);

	DBG(10,"TRAIN CONNECTIONS:   %d, %d, %d, %d   NEXT CHECKPOINT %d, CITY #%d",
		TRAIN[train].CheckPoint[0],
		TRAIN[train].CheckPoint[1],
		TRAIN[train].CheckPoint[2],
		TRAIN[train].CheckPoint[3],
		next_checkpoint,
		next_checkpoint_city);
	DBG(20,"CITY #%d CONNECTIONS:  %d, %d, %d, %d",
		current_city,
		CITY[current_city].CityConnection[0],
		CITY[current_city].CityConnection[1],
		CITY[current_city].CityConnection[2],
		CITY[current_city].CityConnection[3] );
*/
	int new_connection = -1, route_num, end_city, route_owner;

	// scan through all connections from the current city
	int dist,min_dist = 100000;
	for (int c=0;c<MAX_CONNECT;c++)
		{
		// figure out the city at the end of the route
		end_city = CITY[current_city].CityConnection[c];

		// if there is a connection
		if (end_city != -1)
			{
			route_num = CITY[current_city].RouteConnection[c];
			route_owner = ROUTE[route_num].Owner;

			// if the route's owner (RR) is the same as the train's owner (RR)
			if (route_owner == train_owner)
				{
				// if the route is terminating at the next checkpoint city,
				// save the connection number & exit the loop
				if (end_city == next_checkpoint_city)
					{
					new_connection = c;
					c = MAX_CONNECT;
//					DBG(30,"NEXT CHECKPOINT WAS FOUND... SKIPPING DISTANCE CHECK");
					continue;
					}
//				else
//					DBG(30,"");

				// if the endcity isn't the last city we travelled through
				if (end_city != last_city)
					{
//					DBG(40,"ATTEMPTING TO FIND CLOSEST CITY TO NEXT CHECKPOINT");
					// figure the distance between the endcity & the next
					// checkpoint city
					dist = DistBetweenCities(end_city,next_checkpoint_city);

					// if its the smallest so far, assume thats the route we want
					if (dist<min_dist)
						{
						min_dist = dist;
						new_connection = c;
						}
					}
//				else
//					DBG(40,"");
				}
			}
		}

	if (new_connection==-1)
		{
//		DBG(50,"No new route was found.... Following last route");
		}
	else
		{
//		DBG(50,"Chose city #%d",CITY[current_city].CityConnection[new_connection]);
		TRAIN[train].Route = CITY[current_city].RouteConnection[new_connection];
		}
	}

