//	-------------------------------------------------------------------------
//
//	TRAIN.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	These are all of the bits required for the trains
//
//	-------------------------------------------------------------------------

#ifndef __train_hpp
#define __train_hpp

#include "stddefns.hpp"
#include "gamedefn.hpp"

// DEFINITIONS

#define	MAX_CHECKPTS		5

#define	NOTINGAME			-1		// train status
#define	MOVING_FORWARDS		1
#define	MOVING_BACKWARDS	2
#define	ARRIVING			4
#define	UNLOADING			5
#define	LOADING				6
#define	DEPARTING			7

#define	MAX_CARS			5

#define	GRASSHOPPER			0	// locomotive types
#define	PACIFIC				1


#define	PASSENGER_TRAIN		0	// carriage types
#define	MAIL_TRAIN			1
#define	RAW_TRAIN			2
#define	GOODS_TRAIN			3
#define	MIXED_PASS_MAIL		4
#define	MIXED_RAW_GOODS		5
#define	MIXED_TRAIN			6

struct	TrainStruct
	{
	int		Status;

	int		Owner;		// the railroad # who owns this train

	char*	Name;		// the name of the train

	int		Speed;		// the current speed of the train

	char	TrainType;	// the train type (MIXED, etc.)

	char	CarType[MAX_CARS];	// the type of the carriages (incl. engine)

	short	x[MAX_CARS]; // the x position of the train
	short	y[MAX_CARS]; // the y position of the train

	short	lastx[MAX_CARS];	// the last x position of the train
	short	lasty[MAX_CARS];	// the last y position of the train

	int		RoutePos;	// position along the route

	int		Route;  	// route# that train is/was moving along
	int		City;		// city# that train is currently in
	int		LastCity;	// city# that train was last in

	int		NextCheckPoint;	// next checkpoint index into CheckPoint[]

	int		NumCheckPoints;	// number of checkpoints
	int		CheckPoint[MAX_CHECKPTS];	// storage of city checkpts
	};

// GLOBALS

extern  int				NUM_TRAINS;
extern	TrainStruct		TRAIN[MAX_TRAINS];

// FUNCTIONS

void	CreateRandomTrain();
void	CreateNewTrain(int rr,char train_type,int* check_points,int num_check_points);

void	DestroyTrain(int train);

void	train_DoMove(int train);
void	train_DoArrival(int train);
void	train_DoDeparture(int train);

void	train_Load(int train);
void	train_Unload(int train);

void	train_ChooseNextCheckPoint(int train);
void	train_ChooseNextRoute(int train);

#endif