//	-------------------------------------------------------------------------
//
//	BNDSFOLD.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the Bond Holdings Folder code
//
//	-------------------------------------------------------------------------

#include "bndsfold.hpp"
#include "eventmgr.hpp"
#include "uidefn.hpp"
#include "gamedefn.hpp"
#include "player.hpp"
#include "fonts.hpp"
#include "bndsbox.hpp"
#include "foldrsel.hpp"
#include "guidefn.hpp"
#include "buttons.hpp"
#include "qbuybnds.hpp"

#include "railroad.hpp"

// -----------------

BondsFolder*	BONDS_FOLDER;
int				evhno_BondsFolder;

// -----------------

#define		cmdScrollUp				50000
#define		cmdScrollDn				50001
#define		cmdClickedHighlight		50002
#define		cmdClickedRow			50003

// -----------------

BondsFolder::BondsFolder()
	: Folder(&evhno_BondsFolder,"Bond Holdings",
				ac_PlayerFolder|ac_HasBigMode|ac_HasInfoBox,BONDMARKET_BOX)
	{
	BONDS_FOLDER = this;

	int width = r.right-r.left+1;
	int height = r.bottom-r.top+1;

	int x1 = width - 8 - 60;
	int x2 = x1 - 8 - 20;

	AddAccessory(new PictureButton(x1,28,0,
				GUI_BUTTONS[BUY_STOCKS_BUTTON],
				GUI_BUTTONS[BUY_STOCKS_BUTTON+1],0,cmdBuyBonds));
	AddAccessory(new PictureButton(x1,72,0,
				GUI_BUTTONS[SELL_STOCKS_BUTTON],
				GUI_BUTTONS[SELL_STOCKS_BUTTON+1],0,cmdSellBonds));
	AddAccessory(new PictureButton(x2,28,0,
				GUI_BUTTONS[MEDIUM_SCROLL_UP_BUTTON],
				GUI_BUTTONS[MEDIUM_SCROLL_UP_BUTTON+1],ac_AutoMode,cmdScrollUp));
	AddAccessory(new PictureButton(x2,72,0,
				GUI_BUTTONS[MEDIUM_SCROLL_DN_BUTTON],
				GUI_BUTTONS[MEDIUM_SCROLL_DN_BUTTON+1],ac_AutoMode,cmdScrollDn));

	// adjust pics in infobox button
	PictureButton* infobox_button = (PictureButton*) accessory[box_button];
	infobox_button->SetPics(GUI_BUTTONS[OPEN_BONDMARKET_BUTTON],
							GUI_BUTTONS[CLOSE_BONDMARKET_BUTTON]);

	AddAccessory(new BondsScrollBox(8,28,x2-8,height-8,
						3,NUM_RRS,
//						ac_EndScroll,
						ac_CentreScroll,
						cmdClickedRow,cmdClickedHighlight));

	box_open = true;
	big_mode = false;
	}

// -----------------

void	BondsFolder::SetBigMode()
	{
	BondsScrollBox* scroll_box = (BondsScrollBox*) accessory[6];
	scroll_box->r.top = map_r.top + 28 + 8; // +8 fixup
	scroll_box->SetRows(20);

	Folder::SetBigMode();
	}

// -----------------

void	BondsFolder::SetSmallMode()
	{
	BondsScrollBox* scroll_box = (BondsScrollBox*) accessory[6];
	scroll_box->r.top = folder_r.top + 28;
	scroll_box->SetRows(3);

	Folder::SetSmallMode();
	}

// -----------------

void 	BondsFolder::HandleEvent(Event &e)
	{
	// handle buttons, etc.
	Folder::HandleEvent(e);

	BondsScrollBox* scroll_box = (BondsScrollBox*) accessory[6];

	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdScrollUp:

					scroll_box->ScrollUp();
//					SELECTED_RR = scroll_box->GetCurrentRow();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdScrollDn:

					scroll_box->ScrollDn();
//					SELECTED_RR = scroll_box->GetCurrentRow();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdClickedHighlight:

//					SELECTED_RR = scroll_box->GetCurrentRow();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdClickedRow:

//					SELECTED_RR = scroll_box->GetCurrentRow();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdBuyBonds:

					OpenQuery(evhno_BuyBondsQuery);
					EVENTMGR->ClearEvent(e);
					break;
				}
			break;
		}
	}

// -----------------

extern 	int		folder_scrollbox_title_bstyle;
extern	int		folder_scrollbox_low_bstyle;
extern	int		folder_scrollbox_high_bstyle;

BondsScrollBox::BondsScrollBox(int x1,int y1,int x2,int y2,
						int rows,int maxrows,
						int options,
						int hit_row_event,int hit_hi_row_event)
	: ScrollBox(x1,y1,x2,y2,rows,maxrows,
					GUI_TILES[SCROLLBOX_TITLE_TILE], folder_scrollbox_title_bstyle,
					GUI_TILES[SCROLLBOX_LOW_TILE],   folder_scrollbox_low_bstyle,
					GUI_TILES[SCROLLBOX_HIGH_TILE],  folder_scrollbox_high_bstyle,
					options,hit_row_event,hit_hi_row_event)
	{
	title_bg	= grRED;
	title_text	= grWHITE;
	title_border= grRED;
	lo_bg		= grDARKGRAY;
	lo_text		= grLIGHTGRAY;
	lo_border	= grLIGHTGRAY;
	hi_bg		= grLIGHTGRAY;
	hi_text		= grWHITE;
	hi_border	= grDARKGRAY;
	}

// -----------------

void	BondsScrollBox::DrawRow(int row,bool auto_fill)
	{
	int rr = row+row_offset;

	// figure out what colors to use

	if (RAILROAD[rr].President==CURRENT_PLAYER)
		{
		lo_bg = grBLUE;
		switch (RAILROAD[rr].Status)
			{
			case FLOATED: 			lo_text = grLIGHTGRAY; 	break;
			case CONTROLLED:        lo_text = grWHITE; 		break;
			case RECEIVERSHIP:      lo_text = grLIGHTRED; 	break;
			case REORGANIZING:      lo_text = grLIGHTGREEN; break;
			}
		}
	else
		{
		lo_bg = grDARKGRAY;
		switch (RAILROAD[rr].Status)
			{
			case FLOATED: 			lo_text = grLIGHTGRAY; 	break;
			case CONTROLLED:        lo_text = grLIGHTGRAY; 	break;
			case RECEIVERSHIP:      lo_text = grRED; 		break;
			case REORGANIZING:      lo_text	= grGREEN; 		break;
			}
		}
	ScrollBox::DrawRow(row,auto_fill);
	DrawRowContents(row,false);
	}

// -----------------

void	BondsScrollBox::DrawHighlightedRow(int row,bool auto_fill)
	{
	int rr = row+row_offset;

	// figure out what colors to use

	if (RAILROAD[rr].President==CURRENT_PLAYER)
		{
		hi_bg = grLIGHTBLUE;
		switch (RAILROAD[rr].Status)
			{
			case FLOATED: 			hi_text = grLIGHTGRAY; 	break;
			case CONTROLLED:        hi_text = grWHITE; 		break;
			case RECEIVERSHIP:      hi_text = grLIGHTRED; 	break;
			case REORGANIZING:      hi_text = grLIGHTGREEN; break;
			}
		}
	else
		{
		hi_bg = grLIGHTGRAY;
		switch (RAILROAD[rr].Status)
			{
			case FLOATED: 			hi_text = grDARKGRAY; 	break;
			case CONTROLLED:        hi_text = grDARKGRAY; 	break;
			case RECEIVERSHIP:      hi_text = grRED; 		break;
			case REORGANIZING:      hi_text = grGREEN; 		break;
			default:				hi_text = grDARKGRAY;	break;
			}
		}
	ScrollBox::DrawHighlightedRow(row,auto_fill);
	DrawRowContents(row,true);
	}

// -----------------

void	BondsScrollBox::DrawTitle()
	{
	ScrollBox::DrawTitle();

	grSetFont(BASICFONT);
	grDraw2CString(r.left+10,r.top,title_text,"Railroad");
	grDraw2CString(r.left+250,r.top,title_text,"Price");
	grDraw2CString(r.left+310,r.top,title_text,"Bonds");
	grDraw2CString(r.left+410,r.top,title_text,"Value");
	}

// -----------------

void	BondsScrollBox::DrawRowContents(int row,bool is_highlighted)
	{
	int y = r.top + 20 + 20*row;	// title + row
	int rr = row+row_offset;

	int color = lo_text;
	if (is_highlighted) color = hi_text;

	int bondprice = RAILROAD[rr].BondPrice[RAILROAD[rr].CurrentHistory];

	grSetFont(BASICFONT);
	grDraw2CString(r.left+10,y,color,"%s",RAILROAD[rr].Name);
	grDraw2CString(r.left+250,y,color,"$%d",bondprice);

	int bnds = PLAYER[CURRENT_PLAYER].Bonds[rr];
	int mill = bnds / 1000;
	int thou = bnds % 1000;

	if (mill) grDraw2CString(r.left+310,y,color,"%d,%.3d,000",mill,thou);
	else      grDraw2CString(r.left+310,y,color,"%d,000",thou);

	bnds = bnds * bondprice;

	mill = bnds / 1000;
	thou = bnds % 1000;

	if (mill) grDraw2CString(r.left+410,y,color,"$%d,%.3d,000",mill,thou);
	else      grDraw2CString(r.left+410,y,color,"$%d,000",thou);
	}

