//	-------------------------------------------------------------------------
//
//	CITYINFO.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the City Information Box code
//
//	-------------------------------------------------------------------------

#include "cityinfo.hpp"
#include "gamedefn.hpp"
#include "uidefn.hpp"
#include "eventmgr.hpp"
#include "fonts.hpp"
#include "mapwin.hpp"
#include "timer.hpp"

// -----------------

#include "conio.h"
#include "stdio.h"

// -----------------

CityInfo*			CITY_INFOBOX;
int					evhno_CityInfo;


// -----------------

CityInfo::CityInfo()
	: MapInfoBox(cityinfo_r.left,cityinfo_r.top,
				 cityinfo_r.right,cityinfo_r.bottom,
				 &evhno_CityInfo,OTHER_INFO_BOX,"City Information")
	{
	// setup bits for this object
	CITY_INFOBOX = this;
	}

// -----------------

void	CityInfo::DrawContents()
	{
	if (!vb) return;

	// figure out our information display region
	int x1 = 0;
	int y1 = 0;
	int x2 = vb_maxx;
	int y2 = vb_maxy;

	int x = city_pt->x;
	int y = city_pt->y;

	int ter = GetTerrain(x,y);
	int ovl = GetOverlaid(x,y);

	int mill,thou,less;

	grImage* ovl_pic = overlaid_bitmaps[MAX_ZOOM_LEVEL][ovl];

	// figure out which city it is....
	int citynum = GetCityNumber(x,y);

	// display station bitmap
	grOverlayImageVirtual(vb,x1+20,y1+40,x1+20+ovl_pic->hdr.width-1,y1+40+ovl_pic->hdr.height-1,grBLACK,ovl_pic);

	// display station name
	grSetFont(BASICFONT);
	grDrawStringVirtual(vb,x1+70,y1+50,"%s",CITY[citynum].Name);

	// display city size
	mill = CITY[citynum].Size / 1000;
	thou = CITY[citynum].Size % 1000;

	grDrawStringVirtual(vb,x1+10,y1+80,"Population");

	if (mill) grDrawStringVirtual(vb,x1+120,y1+80,": %d,%.3d,000",mill,thou);
	else	  grDrawStringVirtual(vb,x1+120,y1+80,": %d,000",thou);

	// display station type
	grDrawStringVirtual(vb,x1+10,y1+110,"Station Type");
	switch (CITY[citynum].StationType)
		{
		case STATION:
			grDrawStringVirtual(vb,x1+120,y1+110,": Station");
			break;

		case TERMINAL:
			grDrawStringVirtual(vb,x1+120,y1+110,": Terminal");
			break;
		}

	// display currently connected cities
	grDrawStringVirtual(vb,x1+10,y1+130,"Connections");

	int yy = y1 + 130;
	for (int dirn=0;dirn<4;dirn++)
		if (CITY[citynum].CityConnection[dirn] != NOT_CONNECTED)
			{
			grDrawStringVirtual(vb,x1+120,yy,": %s",CITY[CITY[citynum].CityConnection[dirn]].Name);
			yy += 15;
			}
	if (yy==y1+130)	// none printed
		grDrawStringVirtual(vb,x1+120,yy,": None");

	// display supply
	grDrawStringVirtual(vb,x1+10,y1+180,"Economic Indicators:");
	int color;

	if (CITY[citynum].MailWaiting>0) color = grLIGHTGREEN;
	else if (CITY[citynum].MailWanted>0) color = grLIGHTRED;
	else color = grWHITE;
	grDraw2CStringVirtual(vb,x1+10,y1+200,color,"MAIL");

	if (CITY[citynum].PeopleWaiting>0) color = grLIGHTGREEN;
	else if (CITY[citynum].PeopleWanted>0) color = grLIGHTRED;
	else color = grWHITE;
	grDrawStringVirtual(vb,x1+80,y1+200,"PASS");

	if (CITY[citynum].RawWaiting>0) color = grLIGHTGREEN;
	else if (CITY[citynum].RawWanted>0) color = grLIGHTRED;
	else color = grWHITE;
	grDrawStringVirtual(vb,x1+10,y1+220,"TIMBER");

	if (CITY[citynum].GoodsWaiting>0) color = grLIGHTGREEN;
	else if (CITY[citynum].GoodsWanted>0) color = grLIGHTRED;
	else color = grWHITE;
	grDrawStringVirtual(vb,x1+80,y1+220,"GOODS");
	}

// -----------------

void 	CityInfo::HandleEvent(Event &e)
	{
	// handle buttons, etc.
	MapInfoBox::HandleEvent(e);

	// react to button responses only!!
	switch (e.what)
		{
		case ev_broadcast:

			switch (e.broadcast.event)
				{
				case cmdCitySelected:

					city_pt = (grPoint*) e.command.data;

					if (isOnScreen) Refresh();
					else 			Open();
					break;
				}
			break;
		}
	}


