//	-------------------------------------------------------------------------
//
//	FOLDER.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the generic Folder code
//
//	-------------------------------------------------------------------------

#include "folder.hpp"
#include "uidefn.hpp"
#include "gamedefn.hpp"
#include "fonts.hpp"
#include "eventmgr.hpp"
#include "foldrsel.hpp"
#include "guidefn.hpp"
#include "buttons.hpp"
#include "mapwin.hpp"
#include "borders.hpp"
#include "program.hpp"
#include "dos.h"

// -----------------

#include "player.hpp"
#include "railroad.hpp"

// -----------------

Folder*  folder_loader = NULL;
int 	 folder_bstyle = 0;
int		 folder_title_bstyle = 0;
int 	 folder_scrollbox_title_bstyle = 0;
int		 folder_scrollbox_low_bstyle = 0;
int		 folder_scrollbox_high_bstyle = 0;

Folder::Folder(int* evhno,char* name_of_folder,int options,
						FolderBox* folder_box )
	: OnScreenHandler(folder_r.left,folder_r.top,
					  folder_r.right,folder_r.bottom,
					  evhno,options,4)
	{
	// set onscreen handler defaults
	isOnScreen = false;
	isRespondingKbd = false;

	big_mode = false;
	box_open = false;
	query_open = false;

	// initialise our border
	if (folder_loader==NULL)
		{
		folder_title_bstyle = grInitBorder(PROGRAM_PATH,"RESOURCE\\FLDRTITL.BRD");
		if (folder_title_bstyle<0)
			{
			DBG(470,"ERROR %d: could not init border style FLDRTITL.BRD",folder_title_bstyle);
			delay(5000);
			}

		folder_bstyle = grInitBorder(PROGRAM_PATH,"RESOURCE\\FOLDER.BRD");
		if (folder_bstyle<0)
			{
			DBG(470,"ERROR %d: could not init border style FOLDER.BRD",folder_bstyle);
			delay(5000);
			}

		folder_scrollbox_title_bstyle = grInitBorder(PROGRAM_PATH,"RESOURCE\\SCRL_TTL.BRD");
		folder_scrollbox_low_bstyle = grInitBorder(PROGRAM_PATH,"RESOURCE\\SCRL_LO.BRD");
		folder_scrollbox_high_bstyle = grInitBorder(PROGRAM_PATH,"RESOURCE\\SCRL_HI.BRD");

		folder_loader = this;
		}

	// save folder name & type
	folder_name = name_of_folder;
	box_ptr		= folder_box;

	// save the coords we use to draw the folder's title
	grSetFont(BASICFONT);

	int name_width  = grFontWidth(folder_name);
//	int	name_height = grFontHeight(folder_name);

	name_xpos = r.right - 8 - name_width;
	name_ypos = r.top;

	FOLDER_SELECTION->RegisterFolder(this,box_ptr);

	// --- add the default bigmode & infobox buttons
	int pos = r.right-r.left+1;
	int button_num = 0;

	// infobox button bits
	if (selected_options&ac_HasInfoBox)
		{
		box_button = button_num++;
		int width = 8 + GUI_BUTTONS[OPEN_INFOBOX_BUTTON]->hdr.width-1;
		pos -= width;
		name_xpos -= width;
		AddAccessory(new PictureButton(pos,0,0,
					GUI_BUTTONS[CLOSE_INFOBOX_BUTTON],
					GUI_BUTTONS[OPEN_INFOBOX_BUTTON],
					ac_ToggleMode,cmdToggleInfoBox));
		}

	// bigmode button bits
	if (selected_options&ac_HasBigMode)
		{
		big_button = button_num++;
		int width = 8 + GUI_BUTTONS[SMALL_FOLDER_BUTTON]->hdr.width-1;
		pos -= width;
		name_xpos -= width;
		AddAccessory(new PictureButton(pos,0,0,
					GUI_BUTTONS[SMALL_FOLDER_BUTTON],
					GUI_BUTTONS[BIG_FOLDER_BUTTON],
					ac_ToggleMode,cmdToggleBigMode));
		}
	}

// -----------------

Folder::~Folder()
	{
	DelAllAccessories();
	FOLDER_SELECTION->DeRegisterFolder(this);

	if (folder_loader==this)
		{
		grExitBorder();
		grExitBorder();
		grExitBorder();

		grExitBorder();
		grExitBorder();
		folder_loader = NULL;
		folder_bstyle = 0;
		folder_title_bstyle = 0;
		}
	}

// -----------------

bool	Folder::InBigMode()
	{
	return big_mode;
	}

// -----------------

bool	Folder::InfoBoxOpen()
	{
	return box_open;
	}

// -----------------

bool	Folder::InQueryMode()
	{
	return query_open;
	}

// -----------------

void	Folder::SetBigMode()
	{
	if (big_mode) return;

	// we're now big
	big_mode = true;

	// set coords as required
	r.top = map_r.top;

	// move folder title string
	name_ypos = r.top;

	// close the map window
	MAPWIN->isOnScreen = false;

	// move accessories up as required
//	int dy = folder_r.top - map_r.top;
//	if (selected_options&ac_HasBigMode) accessory[big_button]->AdjustCoords(0,-dy);
//	if (selected_options&ac_HasInfoBox) accessory[box_button]->AdjustCoords(0,-dy);

	// disable infobox button as required
	if (selected_options&ac_HasInfoBox) accessory[box_button]->DeActivate();

	// redisplay whole lot
	Refresh();
	}

// -----------------

void	Folder::SetSmallMode()
	{
	if (!big_mode) return;

	// we're now in small mode
	big_mode = false;

	// set new draw coords
	r.top = folder_r.top;

	// move folder title string
	name_ypos = r.top;

	// move accessories dn as required
	int dy = folder_r.top - map_r.top;
//	if (selected_options&ac_HasBigMode) accessory[big_button]->AdjustCoords(0,dy);
//	if (selected_options&ac_HasInfoBox) accessory[box_button]->AdjustCoords(0,dy);

	// enable infobox button as required
	if (selected_options&ac_HasInfoBox) accessory[box_button]->Activate();

	// re-open the map & draw it
	MAPWIN->isOnScreen = true;
	MAPWIN->Draw();

	// redraw the folder
	Draw();

	// blit the lot
	grBlitRegion(r.left,map_r.top,r.right,r.bottom);
	}

// -----------------

void	Folder::OpenInfoBox()
	{
	if (box_ptr==NULL) return;

	// if the box is not already open
	if (!box_open)
		{
		// open it now
		box_ptr->Open();

		// and manually refresh it immediately
		box_ptr->Refresh();

		// save the fact
		box_open = true;
		}
	}

// -----------------

void	Folder::CloseInfoBox()
	{
	if (box_ptr==NULL) return;

	// if the box is already open
	if (box_open)
		{
		// close the box now
		box_ptr->Close();

		// and manually update box region
		grBlitRegion(box_ptr->r.left,box_ptr->r.top,
					 box_ptr->r.right,box_ptr->r.bottom);

		// save the fact
		box_open = false;
		}
	}

// -----------------

void	Folder::OpenQuery(int query_handler)
	{
	if (query_open) return;

	// make sure we can't utilise the folder selection bar
	FOLDER_SELECTION->DeActivate();

	// close our folder & open appropriate query....
	Close();
	EVENTMGR->GenCommandEvent(query_handler,cmdOpen,NULL);

	// save the fact that a query is open
	query_open = true;
	}

// -----------------

void	Folder::CloseQuery()
	{
	if (!query_open) return;

	// we're no longer in query mode...
	query_open = false;

	// if we're in big mode
	if (big_mode)
		{
		// redisplay folder
		isOnScreen = true;
		Refresh();

		// tell the folder selection bar to reactivate, but don't refresh it
		FOLDER_SELECTION->Activate(false);
		}
	else
		{
		// we're back on the screen
		isOnScreen = true;

		// tell the folder selection bar to reactivate, refreshing folder also
		FOLDER_SELECTION->Activate(true);
		}
	}

// -----------------

void	Folder::SetColors()
	{
	// figure the type of the folder
	int folder_type = (selected_options & 0x000F0000) - ac_PlayerFolder;

	switch (folder_type)
		{
		case 0:	// PLAYER FOLDERS

			folder_title_bg 	= PLAYER_TITLE_BG;
			folder_title_text 	= PLAYER_TITLE_TEXT;
			folder_border 		= PLAYER_BORDER;
			folder_bg			= PLAYER_BG;
			folder_text			= PLAYER_TEXT;
			break;

		case 1:	// RAILROAD FOLDERS

			switch (RAILROAD[CURRENT_RR].Status)
				{
				case CONTROLLED:

					folder_title_bg 	= CONTROLLED_TITLE_BG;
					folder_title_text 	= CONTROLLED_TITLE_TEXT;
					folder_border 		= CONTROLLED_BORDER;
					folder_bg			= CONTROLLED_BG;
					folder_text			= CONTROLLED_TEXT;
					break;

				case RECEIVERSHIP:

					folder_title_bg 	= RECEIVERSHIP_TITLE_BG;
					folder_title_text 	= RECEIVERSHIP_TITLE_TEXT;
					folder_border 		= RECEIVERSHIP_BORDER;
					folder_bg			= RECEIVERSHIP_BG;
					folder_text			= RECEIVERSHIP_TEXT;
					break;

				case REORGANIZING:

					folder_title_bg 	= REORGANIZING_TITLE_BG;
					folder_title_text 	= REORGANIZING_TITLE_TEXT;
					folder_border 		= REORGANIZING_BORDER;
					folder_bg			= REORGANIZING_BG;
					folder_text			= REORGANIZING_TEXT;
					break;

				default:

					folder_title_bg 	= RAILROAD_TITLE_BG;
					folder_title_text 	= RAILROAD_TITLE_TEXT;
					folder_border 		= RAILROAD_BORDER;
					folder_bg			= RAILROAD_BG;
					folder_text			= RAILROAD_TEXT;
					break;

				}
			break;

		case 2:	// OTHER FOLDERS

			folder_title_bg 	= OTHER_TITLE_BG;
			folder_title_text 	= OTHER_TITLE_TEXT;
			folder_border 		= OTHER_BORDER;
			folder_bg			= OTHER_BG;
			folder_text			= OTHER_TEXT;
			break;

		}
	}

// -----------------

void	Folder::DrawTitle()
	{
	// draw inside the title area

	grTileRect(r.left,r.top,r.right,r.top+19,-1,GUI_TILES[FOLDER_TITLE_TILE],NO_ROTATE);

//	grDrawFilledRect(r.left+1,r.top+1,r.right-1,r.top+19,folder_title_bg);
//	grDrawFilledRect(r.left,r.top,r.right,r.top+19,folder_title_bg);

	// draw the title's borders

//	grDrawLine(r.left,r.top,r.right,r.top,folder_border);		// top
//	grDrawLine(r.left,r.top,r.left,r.top+19,folder_border);		// lhs
//	grDrawLine(r.right,r.top,r.right,r.top+19,folder_border);	// rhs
//	grDrawLine(r.left,r.top+19,r.right,r.top+19,folder_border);	// bot

	grDrawBorder(r.left,r.top,r.right,r.top+19,folder_title_bstyle);

	// draw the folder's name

	grSetFont(BASICFONT);
	grDraw2CString(name_xpos,name_ypos,folder_title_text,"%s",folder_name);

	// draw the folder's owner
	if (selected_options&ac_PlayerFolder)
		grDraw2CString(r.left+8,r.top,folder_title_text,"%s",PLAYER[CURRENT_PLAYER].Name);
	if (selected_options&ac_RailroadFolder)
		grDraw2CString(r.left+8,r.top,folder_title_text,"%s",RAILROAD[CURRENT_RR].Name);
	if (selected_options&ac_OtherFolder)
		grDraw2CString(r.left+8,r.top,folder_title_text,"--> ? <--");

	}

// -----------------

void	Folder::DrawContents()
	{
	// draw inside the folder

	grTileRect(r.left,r.top+20,r.right,r.bottom,-1,GUI_TILES[FOLDER_TILE],NO_ROTATE);

//	grDrawFilledRect(r.left+1,r.top+20,r.right-1,r.bottom-1,folder_bg);

	// draw the borders

	grDrawBorder(r.left,r.top+20,r.right,r.bottom,folder_bstyle);

//	grDrawLine(r.left,r.top+20,r.right,r.top+20,folder_border);		// top
//	grDrawLine(r.left,r.top+20,r.left,r.bottom,folder_border); 		// lhs
//	grDrawLine(r.right,r.top+20,r.right,r.bottom,folder_border);	// rhs
//	grDrawLine(r.left,r.bottom,r.right,r.bottom,folder_border);		// bot
	}

// -----------------

void	Folder::Draw()
	{
	// pretty standard
	SetColors();

	DrawTitle();
	DrawContents();

	DrawAllAccessories();
	}

// -----------------

void	Folder::Open()
	{
	isOnScreen = true;
	Draw();
	}

// -----------------

void	Folder::Close()
	{
	isOnScreen = false;
	}

// -----------------

void 	Folder::HandleEvent(Event &e)
	{
	// handle buttons, etc.
	OnScreenHandler::HandleEvent(e);

	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdToggleBigMode:

					if (big_mode) SetSmallMode();
					else SetBigMode();

					EVENTMGR->ClearEvent(e);
					break;

				case cmdToggleInfoBox:

					if (box_open) CloseInfoBox();
					else OpenInfoBox();

					EVENTMGR->ClearEvent(e);
					break;

				case cmdCloseQuery:

					CloseQuery();
					EVENTMGR->ClearEvent(e);
					return;

				case cmdCloseFolderBox:

					if (query_open)
						{
						EVENTMGR->ClearEvent(e);
						return;
						}

					if (box_open)
						{
						Button* button = (Button*) accessory[box_button];
						button->Toggle();
						CloseInfoBox();
						}
					EVENTMGR->ClearEvent(e);
					break;
				}
			break;

		}
	}

