//	-------------------------------------------------------------------------
//
//	FOLDER.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the generic Folder object
//
//	-------------------------------------------------------------------------

#ifndef __folder_hpp
#define __folder_hpp

#include "onscreen.hpp"
#include "foldrbox.hpp"

// -----------------

// folder options             - 0x00XX0000
#define		ac_PlayerFolder		0x00010000
#define		ac_RailroadFolder	0x00020000
#define		ac_OtherFolder		0x00040000

#define		ac_HasBigMode		0x00100000
#define		ac_HasInfoBox		0x00200000

// -----------------

class Folder : public OnScreenHandler
	{
	private:
		int				name_xpos;
		int				name_ypos;

	protected:
		char*			folder_name;

		FolderBox*		box_ptr;

		int				box_button;
		int				big_button;

		bool			big_mode;
		bool			box_open;
		bool			query_open;

	public:
						Folder( int* evhno,
								char* name_of_folder,
								int options,
								FolderBox* folder_box );
		virtual 		~Folder();

		//	--- status functions

		virtual	bool	InBigMode();
		virtual	bool	InfoBoxOpen();
		virtual	bool	InQueryMode();

		// 	--- action functions

		virtual	void	SetBigMode();
		virtual	void	SetSmallMode();

		virtual	void	OpenInfoBox();
		virtual	void	CloseInfoBox();

		virtual	void	OpenQuery(int query_handler);
		virtual	void	CloseQuery();

		//	--- draw functions

		virtual	void	SetColors();

		virtual	void	DrawTitle();
		virtual	void	DrawContents();

		virtual void 	Draw();

		// 	--- handler functions

		virtual	void	Open();
		virtual	void	Close();

		virtual void 	HandleEvent(Event &e);
	};


// -----------------

#endif

