//	-------------------------------------------------------------------------
//
//	FOLDRSEL.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the Folder Selection code
//
//	-------------------------------------------------------------------------

#include "foldrsel.hpp"
#include "program.hpp"
#include "folder.hpp"
#include "uidefn.hpp"
#include "eventmgr.hpp"
#include "gbuttons.hpp"
#include "guidefn.hpp"
#include "keydefns.hpp"
#include "mapwin.hpp"

#include "dos.h"

// -----------------

FolderSelection*	FOLDER_SELECTION = NULL;
int					evhno_FolderSelection;

// -----------------

bool box_selected = false;

FolderSelection::FolderSelection()
	: InfoBox(folder_select_r.left,folder_select_r.top,
			  folder_select_r.right,folder_select_r.bottom,
			  &evhno_FolderSelection,OTHER_INFO_BOX,ALWAYS_ON_MAP,"Folder Selection")
	{
	isOnScreen = true;

	// save our pointer
	FOLDER_SELECTION = this;

	// clear folder storage
	num_folders = 0;
	for (int fold=0;fold<MAX_FOLDERS;fold++)
		{
		folder_ptr[fold] = NULL;
		folder_boxptr[fold] = NULL;
		}

	// open the folder 'tabs' file
	tab_file = new ResourceFile(PROGRAM_PATH,"resource\\foldtabs.res");
	if (!tab_file)
		{
		DBG(470,"ERROR (OPTIONS): Could not find %sRESOURCE\\foldtabs.RES",PROGRAM_PATH);
		return;
		}
	// read in all required icons
	tab_icons = (grImage**) tab_file->ReadAllRecords();
	if (!tab_icons)
		{
		DBG(470,"ERROR (OPTIONS): Could not read %sRESOURCE\\foldtabs.RES",PROGRAM_PATH);
		delete tab_file;
		return;
		}

	// create all player folder 'tabs'
	int x = (r.right - r.left + 1) / (NUM_FOLDERS+1);

	AddAccessory(new TabButton( x*0,0,tab_icons[ 0],tab_icons[ 1],grBLACK,0,cmdSelectPlayerInfoFolder));
	AddAccessory(new TabButton( x*1,0,tab_icons[ 2],tab_icons[ 3],grBLACK,0,cmdSelectStockHoldingsFolder));
	AddAccessory(new TabButton( x*2,0,tab_icons[ 4],tab_icons[ 5],grBLACK,0,cmdSelectBondHoldingsFolder));
	AddAccessory(new TabButton( x*3,0,tab_icons[ 6],tab_icons[ 7],grBLACK,0,cmdSelectBankFolder));
	AddAccessory(new TabButton( x*4,0,tab_icons[ 8],tab_icons[ 9],grBLACK,0,cmdSelectRailroadFolder));
	AddAccessory(new TabButton( x*5,0,tab_icons[10],tab_icons[11],grBLACK,0,cmdSelectSpyFolder));
	AddAccessory(new TabButton( x*6,0,tab_icons[12],tab_icons[13],grBLACK,0,cmdSelectOpponentFolder));

	// create all railroad folder 'tabs'

	AddAccessory(new TabButton( x*7,0,tab_icons[14],tab_icons[15],grBLACK,0,cmdSelectRRInfoFolder));
	AddAccessory(new TabButton( x*8,0,tab_icons[16],tab_icons[17],grBLACK,0,cmdSelectRRPresFolder));
	AddAccessory(new TabButton( x*9,0,tab_icons[18],tab_icons[19],grBLACK,0,cmdSelectRRFinanceFolder));
	AddAccessory(new TabButton(x*10,0,tab_icons[20],tab_icons[21],grBLACK,0,cmdSelectRROperationsFolder));
	AddAccessory(new TabButton(x*11,0,tab_icons[22],tab_icons[23],grBLACK,0,cmdSelectRRRoutesFolder));
	AddAccessory(new TabButton(x*12,0,tab_icons[24],tab_icons[25],grBLACK,0,cmdSelectRRTrainsFolder));
	AddAccessory(new TabButton(x*13,0,tab_icons[26],tab_icons[27],grBLACK,0,cmdSelectRRSubsidFolder));
	AddAccessory(new TabButton(x*14,0,tab_icons[28],tab_icons[29],grBLACK,0,cmdSelectRRProfitLossFolder));

	// create extra folder 'tabs'

	AddAccessory(new TabButton(x*15,0,tab_icons[30],tab_icons[31],grBLACK,0,cmdSelectGameFolder));

	// setup folder selections
	accessorySelected 	= 0;
	current_folder		= 0;
	}

// -----------------

FolderSelection::~FolderSelection()
	{
	DelAllAccessories();

	if (tab_icons) tab_file->FreeAllRecords((char**) tab_icons);
	delete tab_file;
	}

// -----------------

void	FolderSelection::RegisterFolder(Folder* folder,FolderBox* boxptr)
	{
	if (num_folders<MAX_FOLDERS)
		{
		folder_ptr[num_folders] = folder;
		folder_boxptr[num_folders] = boxptr;
		num_folders++;
		}
	}

// -----------------

void	FolderSelection::DeRegisterFolder(Folder* folder)
	{
	if (num_folders>0)
		{
		int i,j;
		int cnt = 0;
		do
			{
			// when we find the appropriate box
			if (folder_ptr[cnt]==folder)
				{
				// delete it
				while (cnt<MAX_FOLDERS)
					{
					folder_ptr[cnt] = folder_ptr[cnt+1];
					folder_boxptr[cnt] = folder_boxptr[cnt+1];
					cnt++;
					}

				folder_ptr[MAX_FOLDERS-1] = NULL;
				folder_boxptr[MAX_FOLDERS-1] = NULL;

				// and make sure w'ere one box less
				num_folders--;
				return;
				}

			cnt++;
			}
		while (cnt<MAX_FOLDERS);
		}
	}

// -----------------

void	FolderSelection::Activate(bool refresh)
	{
	isRespondingKbd = true;
	isRespondingMouse = true;
	OpenFolder(current_folder,refresh);
	}

// -----------------

void	FolderSelection::DeActivate()
	{
	isRespondingKbd = false;
	isRespondingMouse = false;
	}

// -----------------

void	FolderSelection::OpenFolder(int folder,bool forced)
	{
//	DBG(200,"IN OPENFOLDER(%d)",folder);

	// if we clicked on the same folder, skip out
	if (folder == current_folder)
		// assuming we're not forced
		if (!forced) return;

	// if we're trying to open an out-of-bounds folder skip out
	if ((folder<0) || (folder>=num_folders)) return;

	// skip out if for some reason the folder doesn't exist
	if (folder_ptr[current_folder]==NULL) return;

	// figure initial blit region
	grRect tmpr = { folder_r.left, folder_select_r.top, folder_r.right, folder_r.bottom };

	// close the current folder
	folder_ptr[current_folder]->Close();

	// if the folder is in big mode
	if (folder_ptr[current_folder]->InBigMode())
		{
		// redraw the map
		MAPWIN->Draw();

		// adjust the blit coords
		tmpr.top = map_r.top;
		}


//	DBG(220,"CLOSED FOLDER %d      [BOX: %x]",current_folder,folder_boxptr[current_folder]);
//	grDrawRect(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom,grLIGHTRED);
//	grBlitRegion(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom);
//	delay(2000);

	// if the current folder has a box
	if (folder_boxptr[current_folder])
		// and the box is open
		if (folder_ptr[current_folder]->InfoBoxOpen())
			{
			// close it
			folder_boxptr[current_folder]->Close();

			// adjusting the bounds of the blit region
			tmpr.top = min(tmpr.top, folder_boxptr[current_folder]->r.top);

//			DBG(230,"CLOSED FOLDER %d INFOBOX",current_folder);
//			grDrawRect(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom,grLIGHTRED);
//			grBlitRegion(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom);
//			delay(2000);
			}

	// open the new folder
	folder_ptr[folder]->Open();

	// if the folder is in big mode
	if (folder_ptr[folder]->InBigMode())
		{
		// adjust the blit coords
		tmpr.top = map_r.top;
		}



//	DBG(220,"OPENED FOLDER %d      [BOX: %x]",folder,folder_boxptr[folder]);
//	grDrawRect(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom,grLIGHTRED);
//	grBlitRegion(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom);
//	delay(2000);

	// if the new folder has a box
	if (folder_boxptr[folder])
		// and the box is open
		if (folder_ptr[folder]->InfoBoxOpen())
			{
			// open the box
			folder_boxptr[folder]->Open();

			// adjusting the bounds of the blit region
			tmpr.top = min(tmpr.top, folder_boxptr[folder]->r.top);

//			DBG(230,"OPENED FOLDER %d INFOBOX",current_folder);
//			grDrawRect(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom,grLIGHTRED);
//			grBlitRegion(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom);
//			delay(2000);
			}

	// and save the new folder status
	current_folder = folder;
	accessorySelected = folder;

	// if the folder is small, draw the folder selection tabs
	if (!folder_ptr[folder]->InBigMode()) DrawAllTabs();

	// blit the region
//	DBG(400,"BLITTING ENTIRE FOLDER");
//	grDrawRect(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom,grLIGHTBLUE);
	grBlitRegion(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom);
//	delay(2000);
	}

// -----------------

void	FolderSelection::OpenPrevFolder()
	{
	int folder = current_folder;

	// choose the prev folder
	folder--;
	if (folder<0) folder= num_folders-1;

	// and open it
	OpenFolder(folder);
	}

// -----------------

void	FolderSelection::OpenNextFolder()
	{
	int folder = current_folder;

	// choose the next folder
	folder++;
	if (folder>=num_folders) folder = 0;

	// and open it
	OpenFolder(folder);
	}

// -----------------

void	FolderSelection::DrawAllTabs()
	{
	if (!vb) return;

	TabButton* tab;
	int folder;

	// draw all the unselected folder 'tabs'
	for (folder=0;folder<current_folder;folder++)
		{
		tab = (TabButton*) accessory[folder];
		tab->Shade();
		tab->Draw(vb);
		}

	// draw all the unselected folder 'tabs' (in reverse order)
	for (folder=num_folders-1;folder>=current_folder;folder--)
		{
		tab = (TabButton*) accessory[folder];
		tab->Shade();
		tab->Draw(vb);
		}

	// draw the selected tab
	tab = (TabButton*) accessory[current_folder];
	tab->UnShade();
	tab->Draw(vb);

	// draw the line which removes all other tabs, lhs then rhs
	int color = folder_title_bg;
	int x1 = accessory[current_folder]->loc_r.left;
	int x2 = accessory[current_folder]->loc_r.right;
	int y  = vb_maxy;

	grDrawLineVirtual(vb,x1,y,x2,y,color);

	// put the vb into grSCREEN
	grOverlayImage(r.left,r.top,r.right,r.bottom,grBLACK,(grImage*)vb);
	}

// -----------------

void	FolderSelection::SelectNextAccessory()
	{
	OpenNextFolder();

/*	DBG(200,"IN FOLDERSELECTION:: SELECTNEXTACCESSORY()");
	DBG(210,"");
	DBG(220,"");
	DBG(230,"");

	// if there are no accessories open the next folder
	if (numAccessories==0)
		{
		DBG(210,"NO ACCESSORIES --> OPENING NEXT FOLDER");
		OpenNextFolder();
		return;
		}

	int folder = current_folder;

	// if we're currently manipulating the folder BOX
	if (box_selected)
		{
		DBG(210,"CURRENTLY MANIPULATING BOX");
		// if the box is at the last accessory, open the next folder
		if (folder_boxptr[folder]->AtLastAccessory())
			{
			DBG(220,"AT LAST ACCESSORY OF BOX --> OPENING NEXT FOLDER");
			OpenNextFolder();
			box_selected = false;
			}

		// otherwise simply scroll through the accessories
		else
			{
			DBG(220,"SELECTING NEXT ACCESSORY OF --- BOX ---");
			folder_boxptr[folder]->SelectNextAccessory();
			}
		}
	else
		{
		DBG(210,"MANIPULATING CURRENT FOLDER");
		// we're playing around with FOLDERS
		// if we're at the last accessory of the folder
		if (folder_ptr[folder]->AtLastAccessory())
			{
			DBG(220,"AT LAST ACCESSORY OF FOLDER");
			// then if there is an info box
			if (folder_boxptr[folder])
				{
				DBG(230,"AN INFO BOX EXISTS --> SWITCHING TO BOX");
				// save the fact that we're sending bits to the folder BOX
				box_selected = true;

				// deselect the folder
				folder_ptr[folder]->DeSelect();

				// select the box
				folder_boxptr[folder]->Select(evh_FIRST_ACCESSORY);
				}
			else
				{
				DBG(230,"NO INFO BOX EXISTS --> OPENING NEXT FOLDER");
				OpenNextFolder();
				box_selected = false;
				}
			}
		else
			{
			DBG(220,"SELECTING NEXT ACCESSORY");
			// tell folder to select the next accessory
			folder_ptr[folder]->SelectNextAccessory();
			}
		}
*/	}

// -----------------

void	FolderSelection::SelectPrevAccessory()
	{
	OpenPrevFolder();
/*
	// if there are no accessories open the prev folder
	if (numAccessories==0)
		{
		OpenPrevFolder();
		return;
		}

	int folder = current_folder;

	// if we're currently manipulating the folder BOX
	if (box_selected)
		{
		// if the box is at the first accessory, open the prev folder
		if (folder_boxptr[folder]->AtFirstAccessory())
			{
			OpenPrevFolder();
			box_selected = false;
			}

		// otherwise simply scroll through the accessories
		else folder_boxptr[folder]->SelectPrevAccessory();
		}
	else
		{
		// we're playing around with FOLDERS
		// if we're at the first accessory of the folder
		if (folder_ptr[folder]->AtFirstAccessory())
			{
			// then if there is an info box
			if (folder_boxptr[folder])
				{
				// save the fact that we're sending bits to the folder BOX
				box_selected = true;

				// deselect the folder
				folder_ptr[folder]->DeSelect();

				// select the box
				folder_boxptr[folder]->Select(evh_LAST_ACCESSORY);
				}
			else
				{
				OpenPrevFolder();
				box_selected = false;
				}
			}
		else
			// tell folder to select the prev accessory
			folder_ptr[folder]->SelectPrevAccessory();
		}
*/	}

// -----------------

void	FolderSelection::Select(int what)
	{
	if (numAccessories>0)
		switch (what)
			{
			case	evh_FIRST_ACCESSORY:

				OpenFolder(0);
				break;

			case	evh_CURRENT_ACCESSORY:

				OpenFolder(current_folder);
				break;

			case	evh_LAST_ACCESSORY:
			default:

				OpenFolder(num_folders-1);
				break;
			}

	isSelected = true;
	}

// -----------------

void	FolderSelection::Draw()
	{
//	DBG(200,"IN FOLDERSELECTION::DRAW");
	if (!vb) return;

//	DBG(210,"IN FOLDERSELECTION::DRAW DRAWINGALLTABS");
	DrawAllTabs();
	}

// -----------------

void	FolderSelection::Refresh()
	{
//	DBG(200,"IN FOLDERSELECTION::REFRESH");
	if (!isOnScreen) return;

//	DBG(210,"IN FOLDERSELECTION::REFRESH DRAWING");
	Draw();
	grBlitRegion(r.left,r.top,r.right,r.bottom);
	}

// -----------------

void 	FolderSelection::HandleEvent(Event &e)
	{
	// handle buttons, etc.
	OnScreenHandler::HandleEvent(e);

	// handle all folder button responses!!
	switch (e.what)
		{
		case ev_command:

			// otherwise select the folder
			OpenFolder(accessorySelected);

			EVENTMGR->ClearEvent(e);
			break;

		case ev_keyboard:

			switch (e.keyboard.event)
				{
				case LT_ARROW_DN:

					OpenPrevFolder();
					EVENTMGR->ClearEvent(e);
					break;

				case RT_ARROW_DN:

					OpenNextFolder();
					EVENTMGR->ClearEvent(e);
					break;

				case LT_ARROW_UP:
				case RT_ARROW_UP:

					EVENTMGR->ClearEvent(e);
					break;
				}
		}
	}


