//	-------------------------------------------------------------------------
//
//	GBUTTONS.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is code for buttons used in the game
//
//	-------------------------------------------------------------------------

#include "gbuttons.hpp"

#include "mouse.hpp"
#include "keydefns.hpp"
#include "eventmgr.hpp"

// -----------------

TabButton::TabButton(int x1,int y1,
					 grImage* active_pic,grImage* inactive_pic,
					 int transparent_color,
					 int options,int ret_event)
	: Accessory( x1,y1,x1+active_pic->hdr.width-1,y1+active_pic->hdr.height-1,
					options,ret_event,ret_event)
	{
	// save bits
	the_active_pic  	= active_pic;
	the_inactive_pic	= inactive_pic;
	tr_color = transparent_color;

	isShaded = false;
	}

// -----------------

bool 	TabButton::MouseInside(grPoint& p)
	{
	// find out if we're inside the button quickly....
	bool isInside = Accessory::MouseInside(p);

	// if we're checking for transparent colors
	if (isInside)
		{
		// convert global 'p' to local coordinates
		p.x -= r.left;
		p.y -= r.top;

		// return true if the pixel selected is not the transparent color
		return (the_active_pic->data[p.y * the_active_pic->hdr.width + p.x] != tr_color);
		}

	// return if mouse is inside the coordinates
	return isInside;
	}

// -----------------

void	TabButton::Shade()
	{
	isShaded = true;
	}

// -----------------

void	TabButton::UnShade()
	{
	isShaded = false;
	}

// -----------------

void 	TabButton::SetPic(grImage* active_pic,grImage* inactive_pic,int transparent_color)
	{
	the_active_pic = active_pic;
	the_inactive_pic = inactive_pic;
    tr_color = transparent_color;
	}

// -----------------

void 	TabButton::Draw()
	{
	if (isShaded)
		grOverlayImage(r.left,r.top,r.right,r.bottom,tr_color,the_inactive_pic);
	else
		grOverlayImage(r.left,r.top,r.right,r.bottom,tr_color,the_active_pic);
	}

// -----------------

void 	TabButton::Draw(grVirtualBuffer* vb)
	{
	if (isShaded)
		grOverlayImageVirtual(vb,loc_r.left,loc_r.top,loc_r.right,loc_r.bottom,tr_color,the_inactive_pic);
	else
		grOverlayImageVirtual(vb,loc_r.left,loc_r.top,loc_r.right,loc_r.bottom,tr_color,the_active_pic);
	}

// -----------------

void	TabButton::HandleEvent(Event& e)
	{
	switch (e.what)
		{
		case ev_mouse:

			switch (e.mouse.event)
				{
				case mb_leftup:
				case mb_rightup:

					EVENTMGR->ModCommandEvent(e,e.where,LMBreturnEvent,NULL);
					return;
				}
			return;
/*
		case ev_keyboard:

			switch (e.keyboard.event)
				{
				case ENTER_UP:
				case SPACE_UP:

					EVENTMGR->ModCommandEvent(e,e.where,LMBreturnEvent,NULL);
					return;
				}
			return;
*/
		}
	}
