//	-------------------------------------------------------------------------
//
//	INFOBOX.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the generic Information Box code
//
//	-------------------------------------------------------------------------

#include "infobox.hpp"
#include "uidefn.hpp"
#include "gamedefn.hpp"
#include "fonts.hpp"
#include "eventmgr.hpp"
#include "mapwin.hpp"
#include "program.hpp"

#include "railroad.hpp"

// -----------------
#include "borders.hpp"

InfoBox* infobox_loader = NULL;
int 	 infobox_border = 0;

InfoBox::InfoBox(int x1,int y1,int x2,int y2,
						int* evhno,int type,int mode,char* name)
	: OnScreenHandler(x1,y1,x2,y2,evhno,0,2)
	{
	// set onscreen handler defaults
	isOnScreen = false;

	// save folder mode & type
	box_type = type;
	box_mode = mode;
	box_name = name;

	// initialise our border
	if (infobox_loader==NULL)
		{
		infobox_border = grInitBorder(PROGRAM_PATH,"RESOURCE\\INFOBOX.BRD");
		infobox_loader = this;
		}

	// create a virtual buffer now if its always on the screen
	if (box_mode==ALWAYS_ON_MAP)
		{
		isOnScreen = true;
		MAPWIN->RegisterInfoBox(this);
		vb = grCreateVirtual(r.left,r.top,r.right,r.bottom,0);

		vb_maxx = vb->hdr.width-1;
		vb_maxy = vb->hdr.height-1;
		}
	}

// -----------------

InfoBox::~InfoBox()
	{
	if (box_mode==ALWAYS_ON_MAP)
		{
		grDestroyVirtual(vb);
		MAPWIN->DeRegisterInfoBox(this);
		}

	if (infobox_loader==this)
		{
		grExitBorder();
		infobox_loader = NULL;
		infobox_border = 0;
		}
	}

// -----------------

void	InfoBox::SetColors()
	{
	switch (box_type)
		{
		case PLAYER_INFO_BOX:

			box_title_bg 	= PLAYER_TITLE_BG;
			box_title_text 	= PLAYER_TITLE_TEXT;
			box_border 		= PLAYER_BORDER;
			box_bg			= PLAYER_BG;
			box_text		= PLAYER_TEXT;
			break;

		case RAILROAD_INFO_BOX:

			switch (RAILROAD[CURRENT_RR].Status)
				{
				case CONTROLLED:

					box_title_bg 	= CONTROLLED_TITLE_BG;
					box_title_text 	= CONTROLLED_TITLE_TEXT;
					box_border 		= CONTROLLED_BORDER;
					box_bg			= CONTROLLED_BG;
					box_text		= CONTROLLED_TEXT;
					break;

				case RECEIVERSHIP:

					box_title_bg 	= RECEIVERSHIP_TITLE_BG;
					box_title_text 	= RECEIVERSHIP_TITLE_TEXT;
					box_border 		= RECEIVERSHIP_BORDER;
					box_bg			= RECEIVERSHIP_BG;
					box_text		= RECEIVERSHIP_TEXT;
					break;

				case REORGANIZING:

					box_title_bg 	= REORGANIZING_TITLE_BG;
					box_title_text 	= REORGANIZING_TITLE_TEXT;
					box_border 		= REORGANIZING_BORDER;
					box_bg			= REORGANIZING_BG;
					box_text		= REORGANIZING_TEXT;
					break;

				default:

					box_title_bg 	= RAILROAD_TITLE_BG;
					box_title_text 	= RAILROAD_TITLE_TEXT;
					box_border 		= RAILROAD_BORDER;
					box_bg			= RAILROAD_BG;
					box_text		= RAILROAD_TEXT;
					break;

				}
			break;

		case OTHER_INFO_BOX:

			box_title_bg 	= OTHER_TITLE_BG;
			box_title_text 	= OTHER_TITLE_TEXT;
			box_border 		= OTHER_BORDER;
			box_bg			= OTHER_BG;
			box_text		= OTHER_TEXT;
			break;

		}
	}

// -----------------

void	InfoBox::DrawBox()
	{
	if (!vb) return;
	grDrawFilledRectVirtual(vb,0,0,vb_maxx,vb_maxy,grBLACK);
	grDrawShadedRectVirtual(vb,0,0,vb_maxx,vb_maxy,box_bg,2);
	grDrawBorderVirtual(vb,0,0,vb_maxx,vb_maxy,infobox_border);
//	grDrawRectVirtual(vb,0,0,vb_maxx,vb_maxy,box_border);
	}

// -----------------

void	InfoBox::DrawTitle()
	{
	if (!vb) return;

	grSetFont(TITLEFONT);
	grDraw2CStringVirtual(vb,20,0,box_title_text,"%s",box_name);

	int width = grFontWidth(box_name);
	int height = grFontHeight(box_name);

	grDrawLineVirtual(vb,20,height+5,20+width,height+5,box_title_text);
	}

// -----------------

void	InfoBox::DrawContents()
	{
	}

// -----------------

void	InfoBox::Draw()
	{
	if (!isOnScreen) return;
	if (!MAPWIN->isOnScreen) return;
	if (!vb) return;

	SetColors();

	DrawBox();
	DrawTitle();
	DrawContents();

	DrawAllAccessories(vb);
	}

// -----------------

void	InfoBox::DrawScreenRegion(int x1,int y1,int x2,int y2)
	{
	if (!isOnScreen) return;
	if (!MAPWIN->isOnScreen) return;
	if (!vb) return;

	// check if the region is at all displayed on the screen
	bool outofbounds = ((x1>r.right) || (y1>r.bottom) ||
						(x2<r.left)  || (y2<r.top));

	// and skip out if its not
	if (outofbounds) return;

	// figure coords of the region inside the vb
	int vx1 = x1 - r.left;
	int vy1 = y1 - r.top;
	int vx2 = x2 - r.left;
	int vy2 = y2 - r.top;

	CLIP2(vx1,vy1,0,0,vb_maxx,vb_maxy);
	CLIP2(vx2,vy2,0,0,vb_maxx,vb_maxy);

	// figure coords of the screen region to update
	CLIP2(x1,y1,r.left,r.top,r.right,r.bottom);
	CLIP2(x2,y2,r.left,r.top,r.right,r.bottom);

//	DBG(110,"OVERLAYING IMAGE SECTION FOR >> %p",this);
//	DBG(120,"OVERLAYING IMAGE COORDS  %d,%d to %d,%d",vx1,vy1,vx2,vy2);
//	DBG(130,"OVERLAYING SCREEN COORDS %d,%d to %d,%d",x1,y1,x2,y2);

	grOverlayImageSection(x1,y1,x2,y2,vx1,vy1,vx2,vy2,grBLACK,vb);
//	grDrawRect(x1,y1,x2,y2,grLIGHTRED);
//	grDrawRect(x1+vx1,y1+vy1,x1+vx2,y1+vy2,grLIGHTGREEN);
	}

// -----------------

void	InfoBox::Refresh()
	{
	if (!isOnScreen) return;
	if (!MAPWIN->isOnScreen) return;
	if (!vb) return;

	Draw();
	MAPWIN->DrawScreenRegion(r.left,r.top,r.right,r.bottom);
	grBlitRegion(r.left,r.top,r.right,r.bottom);
	}

// -----------------

void	InfoBox::Open()
	{
	isOnScreen = true;

	if (box_mode==SOMETIMES_ON_MAP)
		{
		MAPWIN->RegisterInfoBox(this);
		vb = grCreateVirtual(r.left,r.top,r.right,r.bottom,0);

		vb_maxx = vb->hdr.width-1;
		vb_maxy = vb->hdr.height-1;
		}
	}

// -----------------

void	InfoBox::Close()
	{
	isOnScreen = false;
	if (box_mode==SOMETIMES_ON_MAP)
		{
		grDestroyVirtual(vb);
		MAPWIN->DeRegisterInfoBox(this);
		}
	}

// -----------------

void 	InfoBox::HandleEvent(Event &e)
	{
	// handle buttons, etc.
	OnScreenHandler::HandleEvent(e);

	switch (e.what)
		{
		case ev_mouse:

			MAPWIN->HandleEvent(e);
			break;
		}
	}

