//	-------------------------------------------------------------------------
//
//	MAPINFO.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the generic MapInfoBox code
//
//	-------------------------------------------------------------------------

#include "mapinfo.hpp"
#include "uidefn.hpp"
#include "gamedefn.hpp"
#include "fonts.hpp"
#include "eventmgr.hpp"
#include "mapwin.hpp"
#include "buttons.hpp"

#include "railroad.hpp"

// -----------------

MapInfoBox*		MAP_INFO_BOX = NULL;

// -----------------

MapInfoBox::MapInfoBox(int x1,int y1,int x2,int y2,
						int* evhno,
						int type,char* name)
	: InfoBox(x1,y1,x2,y2,evhno,type,SOMETIMES_ON_MAP,name)
	{
	AddAccessory(new TextButton(0,0,SYSTEMFONT,"CL",0,cmdClose,cmdClose));
	}

// -----------------

MapInfoBox::~MapInfoBox()
	{
	DelAllAccessories();
	}

// -----------------

void	MapInfoBox::Refresh()
	{
	if (!isOnScreen) return;
	if (!vb) return;

	// render the box
	Draw();

	// if there are remnants of a different info box
	if ((MAP_INFO_BOX != this) && (MAP_INFO_BOX != NULL))
		{
		// figure the overlap region
		int x1 = min(r.left,MAP_INFO_BOX->r.left);
		int y1 = min(r.top,MAP_INFO_BOX->r.top);
		int x2 = max(r.right,MAP_INFO_BOX->r.right);
		int y2 = max(r.bottom,MAP_INFO_BOX->r.bottom);

		// and tell the mapwin to refresh the entire overlapped region
		MAPWIN->RefreshScreenRegion(x1,y1,x2,y2);
		}
	else
		{
		// redraw just the current box
		MAPWIN->RefreshScreenRegion(r.left,r.top,r.right,r.bottom);
		}
	}

// -----------------

void	MapInfoBox::Open()
	{
	if (MAP_INFO_BOX!=NULL)	MAP_INFO_BOX->Close();
	InfoBox::Open();
	Refresh();
	MAP_INFO_BOX = this;
	}

// -----------------

void	MapInfoBox::Close()
	{
	InfoBox::Close();
	MAPWIN->DrawScreenRegion(r.left,r.top,r.right,r.bottom);
	}

// -----------------

void	MapInfoBox::HandleEvent(Event& e)
	{
	InfoBox::HandleEvent(e);

	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdClose:

					Close();
					grBlitRegion(r.left,r.top,r.right,r.bottom);
					EVENTMGR->ClearEvent(e);
					break;
				}
			break;
		}
	}

