//	-------------------------------------------------------------------------
//
//	HOWMANY.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is window asking "How Many?"
//
//	-------------------------------------------------------------------------

#include "howmany.hpp"
#include "gamedefn.hpp"
#include "eventmgr.hpp"
#include "stddefns.hpp"
#include "buttons.hpp"
#include "mouse.hpp"

#include "player.hpp"
#include "gameproc.hpp"
#include "route.hpp"
#include "msgwin.hpp"

#include "intrface.hpp"

#include "dos.h"

#define	cmdIncrement	30000
#define	cmdDecrement	30001
#define	cmdMinimum		30002
#define	cmdMaximum		30003
#define	cmd25Percent	30004
#define	cmd50Percent	30005
#define	cmd75Percent	30006

HowMany::HowMany()
	: OnScreenHandler(oppinfo_r.left,oppinfo_r.top,oppinfo_r.right,actions_r.bottom,
					  BG_COLOR,bs_NONE,bs_NONE,
					  &evhno_HowMany,0,2)
	{
	// we don't wish to be onscreen unless selected
	isOnScreen = false;
	value = 1;
	selected = cmd50Percent;

	// open the howmany buttons resourcefile
	buttons_file = new ResourceFile("resource\\howmany.res");
	if (!buttons_file)
		{
		DBG(470,"ERROR (OPTIONS): Could not find RESOURCE\\HOWMANY.RES");
		delay(2000);
		return;
		}
	// read in all required buttons 
	buttons = (grImage**) buttons_file->ReadAllRecords();
	if (!buttons)
		{
		DBG(470,"ERROR (OPTIONS): Could not read RESOURCE\\HOWMANY.RES");
		delay(2000);
		delete buttons_file;
		return;
		}

	// we've successfully read in all appropriate buttons....
	// so create all required accessories

	AddAccessory(new PictureButton(24,30,buttons[1],buttons[0],0,cmdMinimum));
	AddAccessory(new PictureButton(48,30,buttons[3],buttons[2],0,cmd25Percent));
	AddAccessory(new PictureButton(72,30,buttons[5],buttons[4],0,cmd50Percent));
	AddAccessory(new PictureButton(96,30,buttons[7],buttons[6],0,cmd75Percent));
	AddAccessory(new PictureButton(120,30,buttons[9],buttons[8],0,cmdMaximum));

	AddAccessory(new TextButton(350,30,"+",ac_AutoRepeats,cmdIncrement));
	AddAccessory(new TextButton(250,30,"-",ac_AutoRepeats,cmdDecrement));

	AddAccessory(new TextButton(400,20,"   OK   ",0,cmdHowManyOK));
	AddAccessory(new TextButton(400,40," CANCEL ",0,cmdHowManyCANCEL));
	}

HowMany::~HowMany()
	{
	DelAllAccessories();
	if (buttons) buttons_file->FreeAllRecords((char**) buttons);
	delete buttons_file;
	}

void	HowMany::CalcValue()
	{
	switch (QUERY_MODE)
		{
		case cmdBuyStocks:

			max_val = RAILROAD[SELECTED_RR].StocksOnMarket;
			break;

		case cmdSellStocks:

			max_val = PLAYER[CURRENT_PLAYER].Stocks[SELECTED_RR];
			break;

		case cmdBuyBonds:

			max_val = RAILROAD[SELECTED_RR].BondsOnMarket;
			break;

		case cmdSellBonds:

			max_val = PLAYER[CURRENT_PLAYER].Bonds[SELECTED_RR];
			break;

		case cmdRR_BuildTrack:

			if (RAILROAD[SELECTED_RR].CharteredRoute != -1)
				{
				int maxlen = ROUTE[RAILROAD[SELECTED_RR].CharteredRoute].Length -
							 ROUTE[RAILROAD[SELECTED_RR].CharteredRoute].BuiltLength;
				max_val = 25;
				if (max_val>maxlen) max_val = maxlen;
				max_val = maxlen;
				}
			else
				max_val = 0;
			break;
		}

	switch (selected)
		{
		case cmdMinimum:

			value = 1;
			if (max_val==0) value = 0;
			break;

		case cmdMaximum:

			value = max_val;
			break;

		case cmd25Percent:

			value = max_val / 4;
			break;

		case cmd50Percent:

			value =  max_val / 2;
			break;

		case cmd75Percent:

			value = (max_val * 3) / 4;
			break;
		}
	}

void	HowMany::DrawValue(bool auto_refresh)
	{
	grSetFont(BASICFONT);
	if (auto_refresh) grBlit2CString(r.left+300,r.top+30,BG_COLOR,TEXT_COLOR,"%d   ",value);
	else grDraw2CString(r.left+300,r.top+30,TEXT_COLOR,"%d   ",value);
	}

void HowMany::Draw()
	{
	// draw the basic window
	grDrawWindow(r.left,r.top,r.right,r.bottom,bgcolor,active_bs);

	String* rrname = new String(100);
	if (SELECTED_RR==-1) rrname->Set("Unknown");
	else rrname->Set(RAILROAD[SELECTED_RR].Name);

	switch (QUERY_MODE)
		{
		case cmdBuyStocks:

			grDraw2CString(r.left,r.top,TEXT_COLOR,"Buy how many stocks in %s?",rrname->Contents());
			break;

		case cmdSellStocks:

			grDraw2CString(r.left,r.top,TEXT_COLOR,"Sell how many stocks in %s?",rrname->Contents());
			break;

		case cmdBuyBonds:

			grDraw2CString(r.left,r.top,TEXT_COLOR,"Buy how many bonds in %s?",rrname->Contents());
			break;

		case cmdSellBonds:

			grDraw2CString(r.left,r.top,TEXT_COLOR,"Sell how many bonds in %s?",rrname->Contents());
			break;

		case cmdRR_BuildTrack:
			grDraw2CString(r.left,r.top,TEXT_COLOR,"Build how many miles of track?");
			break;

		}

	delete rrname;

	DrawValue(false);

	DrawAllAccessories();
	}

void	HowMany::Open()
	{
	if (!isOnScreen)
		{
		// open the howmany window
		CalcValue();
		OnScreenHandler::Open();
		}
	}

void	HowMany::Close()
	{
	if (isOnScreen)
		{
		// we're no longer onscreen
		isOnScreen = false;
		}
	}

void HowMany::HandleEvent(Event& e)
	{
	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdOpen:

					Open();
					EVENTMGR->ClearEvent(e);
					return;

				case cmdRRSelected:

					CalcValue();
					if (isOnScreen) Refresh();
					EVENTMGR->ModCommandEvent(e,evhno_HowMany,cmdHowManyOK,NULL);
					break;
				}
			break;

		case ev_broadcast:

			switch (e.broadcast.event)
				{
				case cmdRRSelected:

					CalcValue();
					if (isOnScreen) Refresh();
					return;
				}
			break;
		}

	// handle the events as normal
	OnScreenHandler::HandleEvent(e);

	// for any mouse click
	switch (e.what)
		{
		case ev_mouse:

			switch (e.mouse.event)
				{
				case mb_leftup:

					EVENTMGR->ModCommandEvent(e,evhno_HowMany,cmdHowManyOK,NULL);
					return;

				case mb_rightup:

					EVENTMGR->ModCommandEvent(e,evhno_HowMany,cmdHowManyCANCEL,NULL);
					return;
				}
			break;

		case ev_command:

			switch (e.command.event)
				{
				case cmdHowManyOK:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,e.command.event,(void*) value);
					break;

				case cmdHowManyCANCEL:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,e.command.event,NULL);
					break;

				case cmdIncrement:

					if (value<max_val)
						{
						value++;
						DrawValue();
						}
					EVENTMGR->ClearEvent(e);
					break;

				case cmdDecrement:

					if (value>0)
						{
						value--;
						DrawValue();
						}
					EVENTMGR->ClearEvent(e);
					break;

				case cmdMinimum:

					selected = e.command.event;
					CalcValue();
					DrawValue();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdMaximum:

					selected = e.command.event;
					CalcValue();
					DrawValue();
					EVENTMGR->ClearEvent(e);
					break;

				case cmd25Percent:

					selected = e.command.event;
					CalcValue();
					DrawValue();
					EVENTMGR->ClearEvent(e);
					break;

				case cmd50Percent:

					selected = e.command.event;
					CalcValue();
					DrawValue();
					EVENTMGR->ClearEvent(e);
					break;

				case cmd75Percent:

					selected = e.command.event;
					CalcValue();
					DrawValue();
					EVENTMGR->ClearEvent(e);
					break;
				}
			break;
		}
	}


