//	-------------------------------------------------------------------------
//
//	INTRFACE.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the UI (user interface) for human players
//
//	-------------------------------------------------------------------------

#include "intrface.hpp"
#include "eventmgr.hpp"
#include "gamedefn.hpp"

#include "train.hpp"
#include "mapwin.hpp"

int		INFOBOX_MODE	= -1;
int		INFOBOX_HANDLER	= -1;
int		QUERY_MODE      = -1;
int		QUERY_HANDLER   = -1;

int		PLAYER_MODE		= MPLAYER;
int		PLAYER_NETWORK_TYPE	= SINGLE_PLAYER;

int		CURRENT_PLAYER	= 0;
int		SELECTED_RR		= 0;
int		SELECTED_ROUTE	= -1;
int		SELECTED_CITY	= -1;

Interface::Interface()
	: BackgroundTask(&evhno_Interface,0)
	{
	}


void 	Interface::SetPlayerMode(int player)
	{
	PLAYER_MODE = MPLAYER;
	CURRENT_PLAYER = player;

	EVENTMGR->GenCommandEvent(evhno_RRActionsBar,cmdClose,NULL);
	EVENTMGR->GenCommandEvent(evhno_RRInfoBar,cmdClose,NULL);
	EVENTMGR->GenCommandEvent(evhno_PlayerActionsBar,cmdOpen,NULL);
	EVENTMGR->GenCommandEvent(evhno_PlayerInfoBar,cmdOpen,NULL);
	EVENTMGR->GenCommandEvent(evhno_StatusBar,cmdRefresh,NULL);
	}

void 	Interface::SetRRMode(int railroad)
	{
	PLAYER_MODE = MRAILROAD;
	SELECTED_RR = railroad;

	EVENTMGR->GenCommandEvent(evhno_PlayerActionsBar,cmdClose,NULL);
	EVENTMGR->GenCommandEvent(evhno_PlayerInfoBar,cmdClose,NULL);
	EVENTMGR->GenCommandEvent(evhno_RRActionsBar,cmdOpen,NULL);
	EVENTMGR->GenCommandEvent(evhno_RRInfoBar,cmdOpen,NULL);
	EVENTMGR->GenCommandEvent(evhno_StatusBar,cmdRefresh,NULL);
	}

void	Interface::OpenQuery( int handler, int mode )
	{
	// if we want to open a new query handler
	if (handler != -1)
		{
		// close the windows we want to obscure
		EVENTMGR->GenCommandEvent(evhno_OpponentInfo,cmdClose,NULL);
		if (PLAYER_MODE==MPLAYER) EVENTMGR->GenCommandEvent(evhno_PlayerActionsBar,cmdClose,NULL);
		else EVENTMGR->GenCommandEvent(evhno_RRActionsBar,cmdClose,NULL);

		// and open the appropriate query
		QUERY_HANDLER = handler;
		QUERY_MODE = mode;
		EVENTMGR->GenCommandEvent(QUERY_HANDLER,cmdOpen,NULL);
		}
	}

void	Interface::CloseQuery()
	{
	if (QUERY_HANDLER != -1)
		{
		EVENTMGR->GenCommandEvent(QUERY_HANDLER,cmdClose,NULL);
		QUERY_HANDLER = -1;
		QUERY_MODE = -1;

		EVENTMGR->GenCommandEvent(evhno_OpponentInfo,cmdOpen,NULL);
		if (PLAYER_MODE==MPLAYER) EVENTMGR->GenCommandEvent(evhno_PlayerActionsBar,cmdOpen,NULL);
		else EVENTMGR->GenCommandEvent(evhno_RRActionsBar,cmdOpen,NULL);
		}
	}

void	Interface::OpenInfoBox( int handler, int mode )
	{
	if (handler != -1)
		{
		// if there is an infobox already open,
		// close it without redrawing it
		if (INFOBOX_HANDLER!=-1)
			EVENTMGR->GenCommandEvent(INFOBOX_HANDLER,cmdClose,(void*) true); // not NULL

		// set the mode & open the infobox
		INFOBOX_HANDLER = handler;
		INFOBOX_MODE = mode;

		EVENTMGR->GenCommandEvent(INFOBOX_HANDLER,cmdOpen,NULL);
		}
	}

void	Interface::CloseInfoBox()
	{
	if (INFOBOX_HANDLER != -1)
		{
		EVENTMGR->GenCommandEvent(INFOBOX_HANDLER,cmdClose,NULL);

		INFOBOX_HANDLER = -1;
		INFOBOX_MODE = -1;
		}
	}

void	Interface::OpenMessage( int action, int message )
	{
	RESPONSE_HEADER[NEXT_RESPONSE].Player = CURRENT_PLAYER;
	RESPONSE_HEADER[NEXT_RESPONSE].Action = action;
	RESPONSE_HEADER[NEXT_RESPONSE].Response = message;

	EVENTMGR->GenCommandEvent(evhno_MessageWin,cmdOpen,(void*) NEXT_RESPONSE);

	NEXT_RESPONSE++;
	if (NEXT_RESPONSE==MAX_RESPONSES) NEXT_RESPONSE = 0;
	}

void	Interface::CloseMessage()
	{
	EVENTMGR->GenCommandEvent(evhno_MessageWin,cmdClose,NULL);
	}





void	Interface::SendPlayerAction()
	{
	switch (PLAYER_NETWORK_TYPE)
		{
		case SINGLE_PLAYER:

			switch (ACTION_HEADER.Action)
				{
				case cmdBuyStocks:

					PLAYER[ACTION_HEADER.Player].QuickActionsPending++;
					OpenMessage(cmdBuyStocks,msgSentBuyStocksMessenger);
					break;

				case cmdSellStocks:

					PLAYER[ACTION_HEADER.Player].QuickActionsPending++;
					OpenMessage(cmdSellStocks,msgSentSellStocksMessenger);
					break;
				}
			break;

		case TCP_NETWORK:

			switch (ACTION_HEADER.Action)
				{
				case cmdBuyStocks:

//					TCP_SendHeader(&ACTION_HEADER,sizeof(ActionHeaderStruct));
//					TCP_SendPacket(&PL_BUYSTOCKS,sizeof(PL_BuyStocksStruct));
					OpenMessage(cmdBuyStocks,msgSentBuyStocksMessenger);
					break;
				}
			break;
		}

	}

void	Interface::SendRRAction()
	{
	switch (PLAYER_NETWORK_TYPE)
		{
		case SINGLE_PLAYER:

			switch (ACTION_HEADER.Action)
				{
				case cmdRR_BuyTrain:

					RAILROAD[ACTION_HEADER.RR].DailyActionsPending++;
					OpenMessage(cmdRR_BuyTrain,msgSentBuyTrainMessenger);
					break;

				case cmdRR_SurveyRoute:

					RAILROAD[ACTION_HEADER.RR].DailyActionsPending++;
					OpenMessage(cmdRR_SurveyRoute,msgSentSurveyRouteMessenger);
					break;

				case cmdRR_CharterRoute:

					RAILROAD[ACTION_HEADER.RR].DailyActionsPending++;
					OpenMessage(cmdRR_CharterRoute,msgSentCharterRouteMessenger);
					break;

				case cmdRR_BuildTrack:

					RAILROAD[ACTION_HEADER.RR].DailyActionsPending++;
					OpenMessage(cmdRR_BuildTrack,msgSentBuildTrackMessenger);
					break;
				}
			break;
		}
	}

void	Interface::DoPlayerResponse(int response_num)
	{
	bool display_message = true;

	if (display_message)
		EVENTMGR->GenCommandEvent(evhno_MessageWin,cmdOpen,(void*) response_num);
	}

void	Interface::DoRRResponse(int response_num)
	{
	bool display_message = true;

	switch (RESPONSE_HEADER[response_num].Response)
		{
		case msgSurveyedRoute:

			MAPWIN->DrawRoute(NUM_ROUTES-1);
			break;
		}

	if (display_message)
		EVENTMGR->GenCommandEvent(evhno_MessageWin,cmdOpen,(void*) response_num);
	}



void 	Interface::HandleEvent(Event &e)
	{
	int tmp;

	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdDoPlayerAction:

					ACTION_HEADER.Action = (int) e.command.data;
					ACTION_HEADER.Player = CURRENT_PLAYER;

					orig_infobox_handler = INFOBOX_HANDLER;
					orig_infobox_mode = INFOBOX_MODE;

					switch (ACTION_HEADER.Action)
						{
						case cmdBuyStocks:

							if ((INFOBOX_HANDLER != evhno_MarketBox) ||
								(INFOBOX_MODE != cmdStockMarket))
								OpenInfoBox(evhno_MarketBox,cmdStockMarket);

							OpenQuery(evhno_WhichRR,cmdBuyStocks);
							break;

						case cmdSellStocks:

							if ((INFOBOX_HANDLER != evhno_MarketBox) ||
								(INFOBOX_MODE != cmdStockMarket))
								OpenInfoBox(evhno_MarketBox,cmdStockMarket);

							OpenQuery(evhno_WhichRR,cmdSellStocks);
							break;

						case cmdBuyBonds:
						case cmdSellBonds:

							CloseInfoBox();
							OpenQuery(evhno_WhichRR,ACTION_HEADER.Action);
							break;

						case cmdControlRR:
						case cmdResignControlRR:

							CloseInfoBox();
							OpenQuery(evhno_WhichRR,ACTION_HEADER.Action);
							break;

						case cmdActForRR:

							CloseInfoBox();
							OpenQuery(evhno_WhichRR,ACTION_HEADER.Action);
							break;
						}

					EVENTMGR->ClearEvent(e);
					break;


				case cmdDoRRAction:

					ACTION_HEADER.Action = (int) e.command.data;
					ACTION_HEADER.RR 	 = SELECTED_RR;

					switch (ACTION_HEADER.Action)
						{
						case cmdRR_BuyTrain:

							OpenQuery(evhno_TrainConfig,ACTION_HEADER.Action);
							break;

						case cmdRR_SurveyRoute:

							OpenQuery(evhno_WhichCity,ACTION_HEADER.Action);
							break;

						case cmdRR_CharterRoute:

							OpenQuery(evhno_WhichRoute,ACTION_HEADER.Action);
							break;

						case cmdRR_BuildTrack:

							OpenQuery(evhno_HowMany,ACTION_HEADER.Action);
							break;

						case cmdRR_ActForPlayer:

							ACTION_HEADER.Action = -1;
							SetPlayerMode(CURRENT_PLAYER);
							break;
						}

					EVENTMGR->ClearEvent(e);
					break;



				case cmdWhichRROK:

					CloseQuery();

					switch (ACTION_HEADER.Action)
						{
						case cmdBuyStocks:

							PLAYER[CURRENT_PLAYER].BUYSTOCKS.RR  = (int) e.command.data;

							OpenQuery(evhno_HowMany,ACTION_HEADER.Action);
							break;

						case cmdSellStocks:

							PLAYER[CURRENT_PLAYER].SELLSTOCKS.RR = (int) e.command.data;

							OpenQuery(evhno_HowMany,ACTION_HEADER.Action);
							break;

						case cmdBuyBonds:

							PLAYER[CURRENT_PLAYER].BUYBONDS.RR = (int) e.command.data;
							OpenQuery(evhno_HowMany,ACTION_HEADER.Action);
							OpenInfoBox(evhno_MarketBox,cmdBondMarket);
							break;

						case cmdSellBonds:

							PLAYER[CURRENT_PLAYER].SELLBONDS.RR = (int) e.command.data;
							OpenQuery(evhno_HowMany,ACTION_HEADER.Action);
							OpenInfoBox(evhno_MarketBox,cmdBondMarket);
							break;

						case cmdControlRR:

							PLAYER[CURRENT_PLAYER].CONTROLRR.RR = (int) e.command.data;
							PLAYER[CURRENT_PLAYER].CONTROLRR.Cnt = 5;
							SendPlayerAction();
							break;

						case cmdActForRR:

							ACTION_HEADER.Action = -1;
							SetRRMode( (int) e.command.data );
							break;
						}

					EVENTMGR->ClearEvent(e);
					break;


				case cmdHowManyOK:

					CloseQuery();

					if (PLAYER_MODE == MPLAYER)
						{
						switch (ACTION_HEADER.Action)
							{
							case cmdBuyStocks:

								PLAYER[CURRENT_PLAYER].BUYSTOCKS.Amount = (int) e.command.data;
								PLAYER[CURRENT_PLAYER].BUYSTOCKS.Cnt 	 = 1;

								if (orig_infobox_handler==-1)
									{
									CloseInfoBox();
									}
								else
									{
									OpenInfoBox(orig_infobox_handler,orig_infobox_mode);
									orig_infobox_handler = -1;
									orig_infobox_mode = -1;
									}
								SendPlayerAction();
								break;

							case cmdSellStocks:

								PLAYER[CURRENT_PLAYER].SELLSTOCKS.Amount = (int) e.command.data;
								PLAYER[CURRENT_PLAYER].SELLSTOCKS.Cnt 	 = 1;
								OpenInfoBox(orig_infobox_handler,orig_infobox_mode);
								orig_infobox_handler = -1;
								orig_infobox_mode 	 = -1;
								SendPlayerAction();
								break;

							case cmdBuyBonds:

								PLAYER[CURRENT_PLAYER].BUYBONDS.Amount = (int) e.command.data;
								PLAYER[CURRENT_PLAYER].BUYBONDS.Cnt    = 2;
								OpenInfoBox(orig_infobox_handler,orig_infobox_mode);
								orig_infobox_handler = -1;
								orig_infobox_mode = -1;
								SendPlayerAction();
								break;

							case cmdSellBonds:

								PLAYER[CURRENT_PLAYER].SELLBONDS.Amount = (int) e.command.data;
								PLAYER[CURRENT_PLAYER].SELLBONDS.Cnt 	= 2;
								OpenInfoBox(orig_infobox_handler,orig_infobox_mode);
								orig_infobox_handler = -1;
								orig_infobox_mode = -1;
								SendPlayerAction();
								break;
							}
						}
					else
						switch (ACTION_HEADER.Action)
							{
							case cmdRR_BuildTrack:

								RAILROAD[ACTION_HEADER.RR].BUILDTRACK.DotsLeft = (int) e.command.data;
								RAILROAD[ACTION_HEADER.RR].BUILDTRACK.Cnt = (int) e.command.data;
								SendRRAction();
								break;
							}

					EVENTMGR->ClearEvent(e);
					break;


				case cmdWhichRouteOK:

					CloseQuery();

					switch (ACTION_HEADER.Action)
						{
						case cmdRR_CharterRoute:

							RAILROAD[ACTION_HEADER.RR].CHARTERROUTE.Route = (int) e.command.data;
							RAILROAD[ACTION_HEADER.RR].CHARTERROUTE.Cnt   = 2;
							SendRRAction();
							break;
						}
					EVENTMGR->ClearEvent(e);
					break;

				case cmdWhichCityOK:

					CloseQuery();

					switch (ACTION_HEADER.Action)
						{
						case cmdRR_SurveyRoute:

							RAILROAD[ACTION_HEADER.RR].SURVEYROUTE.City1 = (int) e.command.data;

							ACTION_HEADER.Action = cmdRR_SurveyRoute_1;

							OpenQuery(evhno_WhichCity,ACTION_HEADER.Action);
							break;

						case cmdRR_SurveyRoute_1:

							RAILROAD[ACTION_HEADER.RR].SURVEYROUTE.City2 = (int) e.command.data;
							RAILROAD[ACTION_HEADER.RR].SURVEYROUTE.Cnt = 7;

							ACTION_HEADER.Action = cmdRR_SurveyRoute;
							SendRRAction();
							break;
						}
					EVENTMGR->ClearEvent(e);
					break;

				case cmdTrainConfigOK:

					CloseQuery();

					switch (ACTION_HEADER.Action)
						{
						case cmdRR_BuyTrain:

							RAILROAD[ACTION_HEADER.RR].BUYTRAIN.Cnt = 1;

							SendRRAction();
							break;
						}

				case cmdTrainConfigCANCEL:
				case cmdWhichCityCANCEL:
				case cmdWhichRouteCANCEL:
				case cmdWhichRRCANCEL:
				case cmdHowManyCANCEL:

					ACTION_HEADER.Action = -1;

					CloseQuery();

					if (orig_infobox_handler==-1)
						CloseInfoBox();
					else
						{
						OpenInfoBox(orig_infobox_handler,orig_infobox_mode);
						orig_infobox_handler = -1;
						orig_infobox_mode = -1;
						}

					EVENTMGR->ClearEvent(e);
					break;



				case cmdDoPlayerResponse:

					DoPlayerResponse( (int) e.command.data );
					EVENTMGR->ClearEvent(e);
					break;

				case cmdDoRRResponse:

					DoRRResponse( (int) e.command.data );
					EVENTMGR->ClearEvent(e);
					break;
				}
			break;
		}
	}
