//	-------------------------------------------------------------------------
//
//	MAPWIN.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the code for the MAP_WINDOW
//
//	-------------------------------------------------------------------------

#include "mapwin.hpp"
#include "eventmgr.hpp"
#include "stddefns.hpp"
#include "graphics.hpp"
#include "keyboard.hpp"
#include "keydefns.hpp"
#include "mouse.hpp"
#include "buttons.hpp"

#include "gamedefn.hpp"
#include "railroad.hpp"
#include "intrface.hpp"

#include "math.h"
#include "mem.h"
#include "dos.h"
#include "malloc.h"
#include "stdlib.h"

//#define DEBUG0
//#define DEBUG1
//#define DEBUG2
#define DEBUG_DRAWTRAIN

// terrain type strings

char*	terrain_string[4] = { "SEA","GRASS","FOREST","MOUNTAINS" };

int					num_terrain_bitmaps;
int					num_overlaid_bitmaps;
int					num_train_bitmaps;

grImage**			terrain_bitmaps[NUM_ZOOM_LEVELS];
grImage**			overlaid_bitmaps[NUM_ZOOM_LEVELS];
grImage**			train_bitmaps[NUM_ZOOM_LEVELS];

#ifdef GAMETEST
extern	grImage*	terrain_map;
extern	grImage*	overlaid_map;
extern	grImage*	height_map;
extern	grImage*	data_map_a;
extern	grImage*	data_map_b;
#endif

extern	grVirtualBuffer*	composite_map;	// composite map (1:1)

MapWin*				MAPWIN;

struct 	LevelDataStruct
	{
	int		icon_width;
	int		icon_height;
	int		icons_wide;
	int		icons_high;

	};


LevelDataStruct		LevelData[NUM_ZOOM_LEVELS] =
	{   { 1, 1, 0, 0 },
		{ 2, 2, 0, 0 },
		{ 4, 4, 0, 0 },
		{ 8, 8, 0, 0 },
		{ 12, 12, 0, 0 },
		{ 16, 16, 0, 0 },
		{ 20, 20, 0, 0 },
		{ 24, 24, 0, 0 },
		{ 28, 28, 0, 0 },
		{ 32, 32, 0, 0 }	};

// -------------------------------------------------------------------------
//
// CONSTRUCTOR & DESTRUCTOR
//
// -------------------------------------------------------------------------

MapWin::MapWin()
	: OnScreenHandler(map_r.left,map_r.top,map_r.right,map_r.bottom,
					  0,bs_NONE,bs_NONE,
					  &evhno_MapWin,0,2)
	{
	// set up manual cursor changes
	AutoResetCursor = false;

	// make sure we don't redraw the map till we're ready
	map_ready = false;

	// create variables for Draw() routines
	map_centre_x = r.left + map_width/2 - 1;
	map_centre_y = r.top + map_height/2 - 1;

	map_xpos = map_centre_x;
	map_ypos = map_centre_y;

	for (int z=0;z<NUM_ZOOM_LEVELS;z++)
		{
		LevelData[z].icons_wide = map_width / LevelData[z].icon_width   + 1;
		LevelData[z].icons_high = map_height / LevelData[z].icon_height + 1;
		}

	// there's no info displayed
	info_box_displayed = NO_INFO;

	finished_zoom = false;

	// load in the mouse cursors used by the map
//	cursor_file = new ResourceFile("RESOURCE\\CURSORS.RES");
//	if (!cursor_file)
//		{
//		DBG(470,"ERROR (MAPSCR): Could not find RESOURCE\\CURSORS.RES");
//		delay(2000);
//		}

//	int num_mouse_cursors = cursor_file->NumRecords();
//	mouse_cursors = (MouseCursor**) cursor_file->ReadAllRecords();
//	if (!mouse_cursors)
//		{
//		DBG(470,"ERROR (MAPSCR): Could not read Mouse Cursors");
//		delay(2000);
//		}

	// load in terrain bitmaps
	terrain_file = new ResourceFile("RESOURCE\\TERRAIN.RES");
	if (!terrain_file)
		{
		DBG(470,"ERROR (MAP): Could not find RESOURCE\\TERRAIN.RES");
		delay(2000);
		}

	num_terrain_bitmaps = terrain_file->NumRecords();
	terrain_bitmaps[MAX_ZOOM_LEVEL] = (grImage**) terrain_file->ReadAllRecords();
	if (!terrain_bitmaps)
		{
		DBG(470,"ERROR (MAPSCR): Could not read Terrain Bitmaps");
		delay(2000);
		}

	// pre-scale all terrain bitmaps
	int zoom;
	int y1 = 0;

	zoom_level = 1;
	for (zoom=0;zoom<MAX_ZOOM_LEVEL;zoom++)
		{
		ZoomTo(zoom);

		terrain_bitmaps[zoom] = (grImage**) malloc(sizeof(grImage*) * num_terrain_bitmaps);

		int x1 = 0;
		int y2 = y1 + icon_height-1;

		for (int bitmap=0;bitmap<num_terrain_bitmaps;bitmap++)
			{
			int x2 = x1 + icon_width-1;

			// create space for scaled bitmap
			terrain_bitmaps[zoom][bitmap] = grCreateImage(x1,y1,x2,y2);

			// scale max size image
			grPutScaledImage(x1,y1,x2,y2,terrain_bitmaps[MAX_ZOOM_LEVEL][bitmap]);
			// get the image
			grGetImage(x1,y1,x2,y2,terrain_bitmaps[zoom][bitmap]);
			}
		}

	// load in overlaid icons
	overlaid_file = new ResourceFile("RESOURCE\\OVERLAID.RES");
	if (!overlaid_file)
		{
		DBG(470,"ERROR (MAPSCR): Could not find RESOURCE\\OVERLAID.RES");
		delay(2000);
		}

	num_overlaid_bitmaps = overlaid_file->NumRecords();
	overlaid_bitmaps[MAX_ZOOM_LEVEL] = (grImage**) overlaid_file->ReadAllRecords();
	if (!overlaid_bitmaps)
		{
		DBG(470,"ERROR (MAPSCR): Could not read Overlaid Bitmaps");
		delay(2000);
		}
	// pre-scale all overlaid bitmaps
	zoom_level = 1;
	y1 = 0;
	for (zoom=0;zoom<MAX_ZOOM_LEVEL;zoom++)
		{
		ZoomTo(zoom);

		overlaid_bitmaps[zoom] = (grImage**) malloc(sizeof(grImage*) * num_overlaid_bitmaps);

		int x1 = 0;
		int y2 = y1 + icon_height-1;

		for (int bitmap=0;bitmap<num_overlaid_bitmaps;bitmap++)
			{
			int x2  = x1 + icon_width-1;

			overlaid_bitmaps[zoom][bitmap] = grCreateImage(0,0,icon_width-1,icon_height-1);
			grPutScaledImage(x1,y1,x2,y2,overlaid_bitmaps[MAX_ZOOM_LEVEL][bitmap]);
			grGetImage(x1,y1,x2,y2,overlaid_bitmaps[zoom][bitmap]);
			}
		}

	// load in train icons
	train_file = new ResourceFile("RESOURCE\\TRAINS.RES");
	if (!train_file)
		{
		DBG(470,"ERROR (MAPSCR): Could not find RESOURCE\\TRAINS.RES");
		delay(2000);
		}

	num_train_bitmaps = train_file->NumRecords();
	train_bitmaps[MAX_ZOOM_LEVEL] = (grImage**) train_file->ReadAllRecords();
	if (!train_bitmaps)
		{
		DBG(470,"ERROR (MAPSCR): Could not read Train Bitmaps");
		delay(2000);
		}
	// pre-scale all train bitmaps
	zoom_level = 1;
	y1 = 0;
	for (zoom=0;zoom<MAX_ZOOM_LEVEL;zoom++)
		{
		ZoomTo(zoom);

		train_bitmaps[zoom] = (grImage**) malloc(sizeof(grImage*) * num_train_bitmaps);

		int x1 = 0;
		int y2 = y1 + icon_height-1;

		for (int bitmap=0;bitmap<num_train_bitmaps;bitmap++)
			{
			int x2  = x1 + icon_width-1;

			train_bitmaps[zoom][bitmap] = grCreateImage(0,0,icon_width-1,icon_height-1);
			grPutScaledImage(x1,y1,x2,y2,train_bitmaps[MAX_ZOOM_LEVEL][bitmap]);
			grGetImage(x1,y1,x2,y2,train_bitmaps[zoom][bitmap]);
			}
		}

	// set the zoom level to level 0
	zoom_level = 1;
	ZoomTo(0);

	// we're now ready to redraw the map when required
	map_ready = true;

	// do the standard HEAPCHK
	HEAPCHK("ERROR: Heap is corrupted at end of MAPSCR constructor");
	}



MapWin::~MapWin()
	{
	HEAPCHK("ERROR: Heap is corrupted at start of MAPSCR destructor");

	// remove all train icons
	if (train_file)
		{
		// clean up all pre-scaled train bitmaps
		for (int zoom=MAX_ZOOM_LEVEL-1;zoom>=0;zoom--)
			{
			for (int bitmap=num_train_bitmaps-1;bitmap>=0;bitmap--)
				{
				grDestroyImage(train_bitmaps[zoom][bitmap]);

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					during removal of train icons~\
					just after grDestroyImage of train_bitmaps[%d][%d]",zoom,bitmap);
				}

		free(train_bitmaps[zoom]);

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					during removal of train icons~\
					just after free train_bitmaps[%d]",zoom);

			}

		if (train_bitmaps[MAX_ZOOM_LEVEL]) train_file->FreeAllRecords((char**) train_bitmaps[MAX_ZOOM_LEVEL]);

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					during removal of train icons~\
					just after FreeAllRecords()");

		delete train_file;

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					during removal of train icons~\
					just after delete train_file");

		}

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					Just after removing train icons");

	// remove all overlaid icons
	if (overlaid_file)
		{
		// clean up all pre-scaled overlaid bitmaps
		for (int zoom=MAX_ZOOM_LEVEL-1;zoom>=0;zoom--)
			{
			for (int bitmap=num_overlaid_bitmaps-1;bitmap>=0;bitmap--)
				{
				grDestroyImage(overlaid_bitmaps[zoom][bitmap]);

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					during removal of overlaid icons~\
					just after grDestroyImage of overlaid_bitmaps[%d][%d]",zoom,bitmap);
				}

		free(overlaid_bitmaps[zoom]);

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					during removal of overlaid icons~\
					just after free overlaid_bitmaps[%d]",zoom);

			}

		if (overlaid_bitmaps[MAX_ZOOM_LEVEL]) overlaid_file->FreeAllRecords((char**) overlaid_bitmaps[MAX_ZOOM_LEVEL]);

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					during removal of overlaid icons~\
					just after FreeAllRecords()");

		delete overlaid_file;

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					during removal of overlaid icons~\
					just after delete overlaid_file");

		}

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					Just after removing overlaid icons");

	// remove all terrain icons
	if (terrain_file)
		{
		// clean up all pre-scaled terrain bitmaps
		for (int zoom=MAX_ZOOM_LEVEL-1;zoom>=0;zoom--)
			{
			for (int bitmap=num_terrain_bitmaps-1;bitmap>=0;bitmap--)
				{
				grDestroyImage(terrain_bitmaps[zoom][bitmap]);

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					during removal of terrain icons~\
					just after grDestroyImage of terrain_bitmaps[%d][%d]",zoom,bitmap);

				}

			free(terrain_bitmaps[zoom]);

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					during removal of terrain icons~\
					just after free terrain_bitmaps[%d]",zoom);
			}
		if (terrain_bitmaps[MAX_ZOOM_LEVEL]) terrain_file->FreeAllRecords((char**) terrain_bitmaps[MAX_ZOOM_LEVEL]);

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					during removal of terrain icons~\
					just after FreeAllRecords()");

		delete terrain_file;

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					during removal of terrain icons~\
					just after delete terrain_file");

		}

	HEAPCHK("ERROR: Heap is corrupted during MAPWIN DESTRUCTOR~\
					Just after removing terrain icons");

	// remove all mouse cursors
//	mouse_SetCursor(0,0,DEFAULT_CURSOR);
//	if (cursor_file)
//		{
//		if (mouse_cursors) cursor_file->FreeAllRecords((char**) mouse_cursors);
//		delete cursor_file;
//		}

//	HEAPCHK("ERROR: Heap is corrupted during MAPSCR destructor~\
//					Just after removing mouse cursors");
	}

// -------------------------------------------------------------------------
//
// MAP DRAW FUNCTIONS
//
// -------------------------------------------------------------------------

#ifdef DEBUG1
bool	draw_blank = false;
bool	draw_grid = false;
#endif
// draws a map made up of a 2D grid of scaled bitmaps
void	MapWin::DrawScaledMap()
	{
#ifdef DEBUG0
	DBG(400,"ZOOM LEVEL   :  %d      MAP XPOS,YPOS : %d,%d",zoom_level,map_xpos,map_ypos);
	DBG(410,"MAP WIDTH,HGT:  %d,%d   CENTRE_X,Y    : %d,%d",map_width,map_height,map_centre_x,map_centre_y);
	DBG(420,"MAP COORDS   :  %d,%d to %d,%d",map_x1,map_y1,map_x2,map_y2);
	DBG(430,"SCREEN COORDS:  %d,%d to %d,%d",screen_x1,screen_y1,screen_x2,screen_y2);
	DBG(440,"DISPLAY COORD:  %d,%d to %d,%d",displayed_x1,displayed_y1,displayed_x2,displayed_y2);
	DBG(450,"ICON WDTH,HGT:  %d,%d   HALF: %d,%d",icon_width,icon_height,half_icon_width,half_icon_height);
	DBG(460,"ICONS WD,HIGH:  %d,%d   HALF: %d,%d",icons_wide,icons_high,half_icons_wide,half_icons_high);
#endif
	if (zoom_level==0) { DrawLevel0Map(); return; }
	if (zoom_level==1) { DrawLevel1Map(); return; }
	if (zoom_level==2) { DrawLevel2Map(); return; }

	int x,y;
	int x1, x2, y1, y2;
	int xx2 = screen_x1 + icon_width-1;
	int yy2 = screen_y1 + icon_height-1;
	int tmp;

#ifdef DEBUG1
	if (draw_blank || draw_grid)
		{
		y1 = screen_y1; y2 = yy2;
		for (y=map_y1;y<=map_y2;y++)
			{
			x1 = screen_x1;
			x2 = xx2;
			for (x=map_x1;x<=map_x2;x++)
				{
				if (draw_blank) grDrawFilledRect(x1,y1,x2,y2,grBLACK);
				if (draw_grid) grDrawRect(x1,y1,x2,y2,grMAGENTA);
				x1 += icon_width;
				x2 += icon_width;
				}

			y1 += icon_height;
			y2 += icon_height;
			}
		draw_blank = false;
		draw_grid = false;

		grDrawRect(screen_x1,screen_y1,x2-icon_width,y2-icon_height,grWHITE);
		grBlitRegion(screen_x1,screen_y1,x2-icon_width,y2-icon_height);
		}
#endif

	y1 = screen_y1;
	y2 = yy2;
	for (y=map_y1;y<=map_y2;y++)
		{
		x1 = screen_x1;
		x2 = xx2;
		for (x=map_x1;x<=map_x2;x++)
			{
			tmp = GetTerrain(x,y);
			grPutImage(x1,y1,x2,y2,terrain_bitmaps[zoom_level][tmp]);

			tmp = GetOverlaid(x,y);
			if (tmp!=NOT_OVERLAID) grOverlayImage(x1,y1,x2,y2,grBLACK,overlaid_bitmaps[zoom_level][tmp]);

			x1 += icon_width;
			x2 += icon_width;
			}

		y1 += icon_height;
		y2 += icon_height;
		}
	}

// these are (mostly) optimized functions for drawing
// the level 0,1,2 maps because they're _heaps_ quicker
// than trying to draw a scaled map for these regions

//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
extern bool mouse_HasMoved;
//DJD --- END

void	MapWin::DrawLevel0Map()
	{
	int x,y;

	DWORD* 		dsrce;
	DWORD*		ddest;
	WORD*		wsrce;
	WORD*		wdest;
	BYTE*		bsrce;
	BYTE*		bdest;

	int			row_ofs = screen_y1 * grScreenWidth + screen_x1;
	long		row_pos = (long) &grSCREEN->data[row_ofs];
	long		map_pos = (long) &composite_map->data[map_y1 * map_width + map_x1];

	int			width	= map_x2 - map_x1 + 1;
	int 		dwords	= width >> 2;
	int 		aword	= ((width&2)!=0);
	int 		abyte	= ((width&1)!=0);


	for (y=map_y1;y<=map_y2;y++)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
if (mouse_HasMoved) mouse_UpdateCursor();
//DJD --- END


		dsrce = (DWORD*) map_pos;
		ddest = (DWORD*) row_pos;

		for (x=0;x<dwords;x++) *ddest++ = *dsrce++;

		if (aword)
			{
			wsrce = (WORD*) dsrce;
			wdest = (WORD*) ddest;

			*wdest++ = *wsrce++;

			if (abyte)
				{
				bsrce = (BYTE*) wsrce;
				bdest = (BYTE*) wdest;

				*bdest = *bsrce;
				}
			}
		else
			if (abyte)
				{
				bsrce = (BYTE*) dsrce;
				bdest = (BYTE*) ddest;

				*bdest = *bsrce;
				}

		row_pos += grScreenWidth;
		map_pos += map_width;
		}
	}

void	MapWin::DrawLevel1Map()
	{
	int 		x,y;

	BYTE		abyte;
	WORD 		aword;

	BYTE*		map_ptr;

	WORD* 		row1_ptr;
	WORD* 		row2_ptr;

	int			row1_ofs = screen_y1 * grScreenWidth + screen_x1;
	long		row1_pos = (long) &grSCREEN->data[row1_ofs];
	long		row2_pos = (long) &grSCREEN->data[row1_ofs + grScreenWidth];
	long		map_pos  = (long) &composite_map->data[map_y1 * map_width + map_x1];

	int			incr 	 = grScreenWidth << 1;	// 2 rows

	for (y=map_y1;y<=map_y2;y++)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
if (mouse_HasMoved) mouse_UpdateCursor();
//DJD --- END


		row1_ptr = (WORD*) row1_pos;
		row2_ptr = (WORD*) row2_pos;
		map_ptr = (BYTE*) map_pos;

		for (x=map_x1;x<=map_x2;x++)
			{
			// get a byte from the composite map
			abyte = *map_ptr++;

			// convert it to 2 bytes
			aword = abyte + (abyte << 8);

			// put it into the double-buffer on both rows!
			*row1_ptr++ = aword;
			*row2_ptr++ = aword;
			}

		row1_pos += incr;
		row2_pos += incr;
		map_pos  += map_width;
		}
	}

void	MapWin::DrawLevel2Map()
	{
	int 		x,y;

	BYTE		abyte;
	WORD		aword;
	DWORD 		adword;

	BYTE*		map_ptr;

	DWORD* 		row1_ptr;
	DWORD* 		row2_ptr;
	DWORD*		row3_ptr;
	DWORD*		row4_ptr;

	// figure out the starting location in the double-buffer for four rows
	int			row1_ofs = screen_y1 * grScreenWidth + screen_x1;
	long		row1_pos = (long) &grSCREEN->data[row1_ofs];
	long		row2_pos = (long) &grSCREEN->data[row1_ofs + grScreenWidth];
	long		row3_pos = (long) &grSCREEN->data[row1_ofs + grScreenWidth + grScreenWidth];
	long		row4_pos = (long) &grSCREEN->data[row1_ofs + grScreenWidth + grScreenWidth + grScreenWidth];
	long		map_pos  = (long) &composite_map->data[map_y1 * map_width + map_x1];

	int			incr	 = grScreenWidth << 2;	// 4 rows

	// scan through the map row-by-row
	for (y=map_y1;y<=map_y2;y++)
		{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
if (mouse_HasMoved) mouse_UpdateCursor();
//DJD --- END


		row1_ptr = (DWORD*) row1_pos;
		row2_ptr = (DWORD*) row2_pos;
		row3_ptr = (DWORD*) row3_pos;
		row4_ptr = (DWORD*)	row4_pos;

		// figure out the appropriate position in the composite map
		map_ptr = (BYTE*)	map_pos;

		// scan through entire row
		for (x=map_x1;x<=map_x2;x++)
			{
			// get a byte from the composite map
			abyte = *map_ptr++;

			// convert it to 4 bytes
			aword = abyte + (abyte << 8);
			adword = aword + (aword << 16);

			// put it into the double-buffer on all 4 rows!
			*row1_ptr++ = adword;
			*row2_ptr++ = adword;
			*row3_ptr++ = adword;
			*row4_ptr++ = adword;
			}

		// increment our row starting positions by four double-buffer rows
		row1_pos += incr;
		row2_pos += incr;
		row3_pos += incr;
		row4_pos += incr;
		map_pos  += map_width;
		}
	}


// draws a point in a map of a 2D grid of scaled bitmaps
void	MapWin::DrawScaledPoint()
	{
	if (zoom_level==0) { DrawLevel0Point(-1); return; }
	if (zoom_level==1) { DrawLevel1Point(-1); return; }
	if (zoom_level==2) { DrawLevel2Point(-1); return; }

	int ter = GetTerrain(map_x1,map_y1);
	grPutImage(screen_x1,screen_y1,screen_x2,screen_y2,terrain_bitmaps[zoom_level][ter]);

	int ovl = GetOverlaid(map_x1,map_y1);
	if (ovl!=NOT_OVERLAID) grOverlayImage(screen_x1,screen_y1,screen_x2,screen_y2,grBLACK,overlaid_bitmaps[zoom_level][ovl]);
	}

// draws a point as above but overlays pic in scaled mode
// or draws the color in levels 0,1,2
void	MapWin::DrawScaledPoint(grImage* img,int color)
	{
#ifdef DEBUG1
	DBG(470,"DRAWING SCALED POINT (IMAGE)");
#endif

	if (zoom_level==0) { DrawLevel0Point(color); return; }
	if (zoom_level==1) { DrawLevel1Point(color); return; }
	if (zoom_level==2) { DrawLevel2Point(color); return; }

	int ter = GetTerrain(map_x1,map_y1);
	grPutImage(screen_x1,screen_y1,screen_x2,screen_y2,terrain_bitmaps[zoom_level][ter]);
	grOverlayImage(screen_x1,screen_y1,screen_x2,screen_y2,grBLACK,img);
	}



// (mostly) optimized functions for level 0,1,2 points
void	MapWin::DrawLevel0Point(int color)
	{
//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
if (mouse_HasMoved) mouse_UpdateCursor();
//DJD --- END


#ifdef DEBUG1
	DBG(470,"DRAWING LEVEL0 POINT");
#endif

	BYTE*		srce;
	BYTE*		dest;

	int			row_ofs = screen_y1 * grScreenWidth + screen_x1;
	long		row_pos = (long) &grSCREEN->data[row_ofs];
	long		map_pos = (long) &composite_map->data[map_y1 * map_width + map_x1];

	srce = (BYTE*) map_pos;
	dest = (BYTE*) row_pos;

	if (color==-1) *dest = *srce;
	else *dest = color;
	}

void	MapWin::DrawLevel1Point(int color)
	{
#ifdef DEBUG1
	DBG(470,"DRAWING LEVEL1 POINT");
	DBG(460,"SCREEN: %d,%d To %d,%d",screen_x1,screen_y1,screen_x2,screen_y2);
#endif

//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
if (mouse_HasMoved) mouse_UpdateCursor();
//DJD --- END


	BYTE		abyte;
	WORD 		aword;

	BYTE*		map_ptr;

	WORD* 		row1_ptr;
	WORD* 		row2_ptr;

	int			row1_ofs = screen_y1 * grScreenWidth + screen_x1;
	long		row1_pos = (long) &grSCREEN->data[row1_ofs];
	long		row2_pos = (long) &grSCREEN->data[row1_ofs + grScreenWidth];
	long		map_pos  = (long) &composite_map->data[map_y1 * map_width + map_x1];

	row1_ptr = (WORD*) row1_pos;
	row2_ptr = (WORD*) row2_pos;
	map_ptr  = (BYTE*) map_pos;

	// get a byte from the composite map
	if (color==-1) abyte = *map_ptr;
	else abyte = (BYTE) color;

	// convert it to 2 bytes
	aword = abyte + (abyte << 8);

	// put it into the double-buffer on both rows!
	*row1_ptr = aword;
	*row2_ptr = aword;
	}

void	MapWin::DrawLevel2Point(int color)
	{
#ifdef DEBUG1
	DBG(470,"DRAWING LEVEL2 POINT");
	DBG(460,"SCREEN: %d,%d To %d,%d",screen_x1,screen_y1,screen_x2,screen_y2);
#endif

//DJD --- TEMPORARILY TILL HARDWARE CURSOR INTERFACE
if (mouse_HasMoved) mouse_UpdateCursor();
//DJD --- END


	BYTE		abyte;
	WORD		aword;
	DWORD 		adword;

	BYTE*		map_ptr;

	DWORD* 		row1_ptr;
	DWORD* 		row2_ptr;
	DWORD*		row3_ptr;
	DWORD*		row4_ptr;

	// figure out the starting location in the double-buffer for four rows
	int			row1_ofs = screen_y1 * grScreenWidth + screen_x1;
	long		row1_pos = (long) &grSCREEN->data[row1_ofs];
	long		row2_pos = (long) &grSCREEN->data[row1_ofs + grScreenWidth];
	long		row3_pos = (long) &grSCREEN->data[row1_ofs + grScreenWidth + grScreenWidth];
	long		row4_pos = (long) &grSCREEN->data[row1_ofs + grScreenWidth + grScreenWidth + grScreenWidth];
	long		map_pos  = (long) &composite_map->data[map_y1 * map_width + map_x1];

	row1_ptr = (DWORD*) row1_pos;
	row2_ptr = (DWORD*) row2_pos;
	row3_ptr = (DWORD*) row3_pos;
	row4_ptr = (DWORD*)	row4_pos;

	// figure out the appropriate position in the composite map
	map_ptr  = (BYTE*)	map_pos;

	// get a byte from the composite map
	if (color==-1) abyte = *map_ptr;
	else abyte = (BYTE) color;

	// convert it to 4 bytes
	aword = abyte + (abyte << 8);
	adword = aword + (aword << 16);

	// put it into the double-buffer on all 4 rows!
	*row1_ptr = adword;
	*row2_ptr = adword;
	*row3_ptr = adword;
	*row4_ptr = adword;
	}



// draws a single icon if its in the current map display
// region, returning true if drawn, false if not drawn
bool	MapWin::DrawMapPoint(int x,int y)
	{
	// check if the region is at all displayed on the screen
	bool outofbounds = ((x>displayed_x2) || (y>displayed_y2) ||
						(x<displayed_x1) || (y<displayed_y1));

	// and skip out if its not
	if (outofbounds) return false;

	// copy the topleft coords to screen coords

	screen_x1 = topleft_x;
	screen_y1 = topleft_y;

	// save the number of icons from the start (top-leftmost) icon
	int icons_x = x - displayed_x1;
	int icons_y = y - displayed_y1;

	// update screen coords
	screen_x1 += icons_x * icon_width;
	screen_y1 += icons_y * icon_height;
	screen_x2 = screen_x1 + icon_width-1;
	screen_y2 = screen_y1 + icon_height-1;

	// save the point to draw
	map_x1 = x;
	map_y1 = y;

	// draw the appropriate icons
	DrawScaledPoint();
	return true;
	}

// calls DrawMapPoint() and then blits it as required
void	MapWin::RefreshMapPoint(int x,int y)
	{
	if (DrawMapPoint(x,y))
		grBlitRegion(max(screen_x1,r.left),max(screen_y1,r.top),min(screen_x2,r.right),min(screen_y2,r.bottom));
	}

// draws a user-specified icon if its in the current map display
// region, returning true if drawn, false if not drawn
bool	MapWin::DrawMapPoint(int x,int y,grImage* pic,int color)
	{
	// check if the region is at all displayed on the screen
	bool outofbounds = ((x>displayed_x2) || (y>displayed_y2) ||
						(x<displayed_x1) || (y<displayed_y1));

	// and skip out if its not
	if (outofbounds) return false;

	// copy the topleft coords to screen coords

	screen_x1 = topleft_x;
	screen_y1 = topleft_y;

	// save the number of icons from the start (top-leftmost) icon
	int icons_x = x - displayed_x1;
	int icons_y = y - displayed_y1;

	// update screen coords
	screen_x1 += icons_x * icon_width;
	screen_y1 += icons_y * icon_height;
	screen_x2 = screen_x1 + icon_width-1;
	screen_y2 = screen_y1 + icon_height-1;

	// save the point to draw
	map_x1 = x;
	map_y1 = y;

	// draw the appropriate icons
	DrawScaledPoint(pic,color);
	return true;
	}


// calls DrawMapPoint(pic) and then blits it as required
void	MapWin::RefreshMapPoint(int x,int y,grImage* pic,int color)
	{
	if (DrawMapPoint(x,y,pic,color))
		grBlitRegion(max(screen_x1,r.left),max(screen_y1,r.top),min(screen_x2,r.right),min(screen_y2,r.bottom));
	}



// draws the complete map
void	MapWin::DrawMap()
	{
	// figure map coords
	map_x1 = displayed_x1;
	map_y1 = displayed_y1;
	map_x2 = displayed_x2;
	map_y2 = displayed_y2;

	// copy the screen coords of the top-left icon as this is where we start
	screen_x1 = topleft_x;
	screen_y1 = topleft_y;

	screen_x2 = screen_x1 + icons_wide * icon_width - 1;
	screen_y2 = screen_y1 + icons_high * icon_height - 1;

#ifdef DEBUG1
	DBG(390,"DRAWING COMPLETE MAP");
#endif
	DrawScaledMap();
	}

// calls DrawCompleteMap() and then blits it
void	MapWin::RefreshMap()
	{
	DrawMap();
	grBlitRegion(r.left,r.top,r.right,r.bottom);
	}



// call this function to redraw a specific (rectangular)
// region of the map using the given map coordinates
// returning true if any part of the region is refreshed,
// false if none of the region was redrawn
bool	MapWin::DrawMapRegion(int m_x1,int m_y1,int m_x2,int m_y2)
	{
	// check if the region is in the boundaries currently displayed
	bool outofbounds = ((m_x1>displayed_x2) || (m_y1>displayed_y2) ||
						(m_x2<displayed_x1) || (m_y2<displayed_y1));

	// and skip out if its not displayed on the screen
	if (outofbounds) return false;

	// figure coords of the map icons we want to plot
	map_x1 = m_x1; map_x2 = m_x2;
	map_y1 = m_y1; map_y2 = m_y2;

	// copy the topleft coords to screen coords

	screen_x1 = topleft_x;
	screen_y1 = topleft_y;

	// save the number of icons from the start (top-leftmost) icon
	int icons_x1 = m_x1 - displayed_x1;
	int icons_y1 = m_y1 - displayed_y1;
	int icons_x2 = m_x2 - displayed_x1;
	int icons_y2 = m_y2 - displayed_y1;

	// update screen coords
	screen_x1 += icons_x1 * icon_width;
	screen_y1 += icons_y1 * icon_height;
	screen_x2 += icons_x2 * icon_width - 1;
	screen_y2 += icons_y2 * icon_height - 1;

#ifdef DEBUG1
	DBG(390,"DRAWING MAP REGION");
	draw_blank = true;
	draw_grid  = true;
//	grDrawFilledRect(screen_x1,screen_y1,screen_x2,screen_y2,grBLACK);
//	grBlitRegion(screen_x1,screen_y1,screen_x2,screen_y2);
#endif
	DrawScaledMap();
	return true;
	}

// calls DrawMapRegion() and then blits to the screen
void	MapWin::RefreshMapRegion(int m_x1,int m_y1,int m_x2,int m_y2)
	{
	if (DrawMapRegion(m_x1,m_y1,m_x2,m_y2))
		grBlitRegion(max(screen_x1,r.left),max(screen_y1,r.top),min(screen_x2,r.right),min(screen_y2,r.bottom));
	}


// call this function to redraw a specific (rectangular)
// region of the map using the given screen coordinates
// returning true if any part of the region is refreshed,
// false if none of the region was redrawn
bool	MapWin::DrawScreenRegion(int s_x1,int s_y1,int s_x2,int s_y2)
	{
	// copy the topleft coords to screen coords
	screen_x1 = topleft_x;
	screen_y1 = topleft_y;

	// save the number of icons from the start (top-leftmost) icon
	int icons_x1 = (s_x1 - screen_x1) / icon_width;
	int icons_y1 = (s_y1 - screen_y1) / icon_height;
	int icons_x2 = (s_x2 - screen_x1) / icon_width;
	int icons_y2 = (s_y2 - screen_y1) / icon_height;

	// figure the icons to display

	map_x1 = displayed_x1 + icons_x1;
	map_y1 = displayed_y1 + icons_y1;
	map_x2 = displayed_x1 +	icons_x2;
	map_y2 = displayed_y1 + icons_y2;

	// and update the screen position

	screen_x1 += icons_x1 * icon_width;
	screen_y1 += icons_y1 * icon_height;
	screen_x2 += icons_x2 * icon_width - 1;
	screen_y2 += icons_y2 * icon_height - 1;

#ifdef DEBUG1
	DBG(380,"DRAWING SCREEN MAP");
	draw_blank = false;
	draw_grid  = true;
#endif
	DrawScaledMap();
	return true;
	}

// calls DrawScreenRegion() and then blits to the screen
void	MapWin::RefreshScreenRegion(int s_x1,int s_y1,int s_x2,int s_y2)
	{
	if (DrawScreenRegion(s_x1,s_y1,s_x2,s_y2))
		grBlitRegion(max(s_x1,r.left),max(s_y1,r.top),min(s_x2,r.right),min(s_y2,r.bottom));
	}




// -------------------------------------------------------------------------
//
// MAP MOVEMENT FUNCTIONS
//
// -------------------------------------------------------------------------

// zooms the map to the specified level
void MapWin::ZoomTo(int new_level)
	{
	// skip out if newlevel out of bounds
	if ((new_level<0)||(new_level>MAX_ZOOM_LEVEL)) return;

	// if we need to adjust the zoom level
	if (new_level!=zoom_level)
		{
		// set the new zoom level
		zoom_level = new_level;

		// create some global information about current level

		icon_width = LevelData[zoom_level].icon_width;
		icon_height= LevelData[zoom_level].icon_height;
		icons_wide = LevelData[zoom_level].icons_wide;
		icons_high = LevelData[zoom_level].icons_high;

		half_icon_width = icon_width / 2;
		half_icon_height= icon_height / 2;
		half_icons_wide = icons_wide / 2;
		half_icons_high = icons_high / 2;

		// check for edge of map

		ScrollTo(map_xpos,map_ypos);
		}
	}


void MapWin::ZoomIn()
	{
	ZoomTo(zoom_level+1);
	}

void MapWin::ZoomOut()
	{
	ZoomTo(zoom_level-1);
	}


// scrolls the map to be centred on x,y as required

void MapWin::ScrollTo(int x,int y)
	{
	// skip out if we're trying to go to the current position
	// set our new coordinates
	map_xpos = x;
	map_ypos = y;

	// make sure the pos is in the valid region
	int xbound = 1;
	int ybound = 1;
	if (map_xpos<xbound) map_xpos = xbound;
	if (map_ypos<ybound) map_ypos = ybound;
	xbound = map_width-2;
	ybound = map_height-2;
	if (map_xpos>xbound) map_xpos = xbound;
	if (map_ypos>ybound) map_ypos = ybound;

	// figure coords of the map icons we want to plot
	// make sure that there is at least 1 icon around the map region
	switch (zoom_level)
		{
		case 0:		displayed_x1 = 0;
					displayed_x2 = map_width-1;
					displayed_y1 = 0;
					displayed_y2 = map_height-1;
					break;

		case 1:
		case 2:		displayed_x1 = map_xpos - half_icons_wide;
					displayed_x2 = map_xpos + half_icons_wide - 1;
					displayed_y1 = map_ypos - half_icons_high;
					displayed_y2 = map_ypos + half_icons_high - 1;
					break;

		default:	displayed_x1 = map_xpos - half_icons_wide;
					displayed_x2 = map_xpos + half_icons_wide;
					displayed_y1 = map_ypos - half_icons_high;
					displayed_y2 = map_ypos + half_icons_high;
					break;
		}

	// figure the screen coords for the topleft icon of a full-screen plot

	if (zoom_level<3)
		{
		topleft_x = 0;
		topleft_y = 0;
		}
	else
		{
		topleft_x = map_centre_x - half_icon_width - icon_width * half_icons_wide;
		topleft_y = map_centre_y - half_icon_height - icon_height * half_icons_high;
		}

	if (zoom_level==0) return;

	xbound = 1;
	ybound = 1;
	if (displayed_x1<xbound) { displayed_x2 += xbound - displayed_x1; displayed_x1 = xbound; }
	if (displayed_y1<ybound) { displayed_y2 += ybound - displayed_y1; displayed_y1 = ybound; }

	xbound = map_width-2;
	ybound = map_height-2;

	if (displayed_x2>xbound) { displayed_x1 += xbound - displayed_x2; displayed_x2 = xbound; }
	if (displayed_y2>ybound) { displayed_y1 += ybound - displayed_y2; displayed_y2 = ybound; }
	}

void MapWin::Scroll(int dx,int dy)
	{
	// skip out if we don't really need to scroll
	if ((dx==0)&&(dy==0)) return;

	// set our new coordinates
	ScrollTo(map_xpos+dx,map_ypos+dy);
	}







void 	MapWin::DrawMapInfo(int x,int y,bool auto_refresh)
	{
	last_infox = x;
	last_infoy = y;

	int what = WhatsAt(x,y);

	switch (what)
		{
		case MTERRAIN:		DrawTerrainInfo(x,y,auto_refresh); 	break;
		case MCITY:			DrawCityInfo(x,y,auto_refresh);		break;
		case MROUTE:		DrawRouteInfo(x,y,auto_refresh);	break;
		}
	}

void	MapWin::UndrawMapInfo()
	{
	switch (info_box_displayed)
		{
		case TERRAIN_INFO:
			DrawScreenRegion(terrinfo_r.left,terrinfo_r.top,terrinfo_r.right,terrinfo_r.bottom);
			grBlitRegion(terrinfo_r.left,terrinfo_r.top,terrinfo_r.right,terrinfo_r.bottom);
			break;

		case CITY_INFO:
			DrawScreenRegion(cityinfo_r.left,cityinfo_r.top,cityinfo_r.right,cityinfo_r.bottom);
			grBlitRegion(cityinfo_r.left,cityinfo_r.top,cityinfo_r.right,cityinfo_r.bottom);
			break;

		case ROUTE_INFO:
			DrawScreenRegion(routeinfo_r.left,routeinfo_r.top,routeinfo_r.right,routeinfo_r.bottom);
			grBlitRegion(routeinfo_r.left,routeinfo_r.top,routeinfo_r.right,routeinfo_r.bottom);
			break;
		}
	info_box_displayed = NO_INFO;
	}


void	MapWin::DrawTerrainInfo(int x,int y,bool auto_refresh)
	{
	// figure out our information display region
	int x1 = terrinfo_r.left;
	int y1 = terrinfo_r.top;
	int x2 = terrinfo_r.right;
	int y2 = terrinfo_r.bottom;

	int ter = GetTerrain(x,y);

	// create a shaded box
	grDrawShadedRect(x1,y1,x2,y2,grDARKGRAY,2);
	grDrawRect(x1,y1,x2,y2,grLIGHTRED);

	grImage* ter_pic = terrain_bitmaps[MAX_ZOOM_LEVEL][ter];

	// display window title
	grSetFont(TITLEFONT);
	grDraw2CString(x1+10,y1+10,grLIGHTRED,"Terrain Information");

	// display terrain bitmap
	grOverlayImage(x1+20,y1+40,x1+ter_pic->hdr.width-1,y1+40+ter_pic->hdr.height-1,grBLACK,ter_pic);

	// display terrain string
	grSetFont(BASICFONT);
	grDrawString(x1+70,y1+50,"%s",terrain_string[ter]);

	// display cost to build
	grDrawString(x1+10,y1+80,"Cost to build");
	grDrawString(x1+120,y1+80,": %d,000 per Mile",25);

	// display terrain height
	grDrawString(x1+10,y1+100,"Approx. height");
	grDrawString(x1+120,y1+100,": %d,000 Feet",GetTerrainHeight(x,y) * 5);

	info_box_displayed = TERRAIN_INFO;

	if (auto_refresh) grBlitRegion(x1,y1,x2,y2);
	}


void	MapWin::DrawCityInfo(int x,int y,bool auto_refresh)
	{
	// figure out our information display region
	int x1 = cityinfo_r.left;
	int y1 = cityinfo_r.top;
	int x2 = cityinfo_r.right;
	int y2 = cityinfo_r.bottom;

	int mill,thou,less;

	int ovl = GetOverlaid(x,y);
	grImage* ovl_pic = overlaid_bitmaps[MAX_ZOOM_LEVEL][ovl];

	// create a shaded box
	grDrawShadedRect(x1,y1,x2,y2,grDARKGRAY,2);
	grDrawRect(x1,y1,x2,y2,grLIGHTRED);

	// figure out which city it is....
	int citynum = GetCityNumber(x,y);

	// display window title
	grSetFont(TITLEFONT);
	grDraw2CString(x1+10,y1+10,grLIGHTRED,"City Information");

	// display station bitmap
	grOverlayImage(x1+20,y1+40,x1+ovl_pic->hdr.width-1,y1+40+ovl_pic->hdr.height-1,grBLACK,ovl_pic);

	// display station name
	grSetFont(BASICFONT);
	grDrawString(x1+70,y1+50,"%s",CITY[citynum].Name);

	// display city size
	mill = CITY[citynum].Size / 1000;
	thou = CITY[citynum].Size % 1000;

	grDrawString(x1+10,y1+80,"Population");

	if (mill) grDrawString(x1+120,y1+80,": %d,%.3d,000",mill,thou);
	else	  grDrawString(x1+120,y1+80,": %d,000",thou);

	// display station type
	grDrawString(x1+10,y1+110,"Station Type");
	switch (CITY[citynum].StationType)
		{
		case STATION:
			grDrawString(x1+120,y1+110,": Station");
			break;

		case TERMINAL:
			grDrawString(x1+120,y1+110,": Terminal");
			break;
		}

	// display currently connected cities
	grDrawString(x1+10,y1+130,"Connections");

	int yy = y1 + 130;
	for (int dirn=0;dirn<4;dirn++)
		if (CITY[citynum].CityConnection[dirn] != NOT_CONNECTED)
			{
			grDrawString(x1+120,yy,": %s",CITY[CITY[citynum].CityConnection[dirn]].Name);
			yy += 15;
			}
	if (yy==y1+130)	// none printed
		grDrawString(x1+120,yy,": None");

	// display supply
	grDrawString(x1+10,y1+180,"Economic Indicators:");
	int color;

	if (CITY[citynum].MailWaiting>0) color = grLIGHTGREEN;
	else if (CITY[citynum].MailWanted>0) color = grLIGHTRED;
	else color = grWHITE;
	grDraw2CString(x1+10,y1+200,color,"MAIL");

	if (CITY[citynum].PeopleWaiting>0) color = grLIGHTGREEN;
	else if (CITY[citynum].PeopleWanted>0) color = grLIGHTRED;
	else color = grWHITE;
	grDrawString(x1+80,y1+200,"PASS");

	if (CITY[citynum].RawWaiting>0) color = grLIGHTGREEN;
	else if (CITY[citynum].RawWanted>0) color = grLIGHTRED;
	else color = grWHITE;
	grDrawString(x1+10,y1+220,"TIMBER");

	if (CITY[citynum].GoodsWaiting>0) color = grLIGHTGREEN;
	else if (CITY[citynum].GoodsWanted>0) color = grLIGHTRED;
	else color = grWHITE;
	grDrawString(x1+80,y1+220,"GOODS");

	info_box_displayed = CITY_INFO;

	if (auto_refresh) grBlitRegion(x1,y1,x2,y2);
	}


void	MapWin::DrawRouteInfo(int x,int y,bool auto_refresh)
	{
	// figure out our information display region
	int x1 = routeinfo_r.left;
	int y1 = routeinfo_r.top;
	int x2 = routeinfo_r.right;
	int y2 = routeinfo_r.bottom;

	int mill,thou,less;

	// create a shaded box
	grDrawShadedRect(x1,y1,x2,y2,grDARKGRAY,2);
	grDrawRect(x1,y1,x2,y2,grLIGHTRED);

	// figure out the closest city
	int citynum  = GetCityNumber(x,y);
	int routenum = GetRouteNumber(x,y);

	// display window title
	grSetFont(TITLEFONT);
	grDraw2CString(x1+10,y1+10,grLIGHTRED,"Route Information");

	// display connected cities
	grSetFont(BASICFONT);
	int startcity = ROUTE[routenum].StartCity;
	int endcity   = ROUTE[routenum].EndCity;
	grDrawString(x1+10,y1+40,"%s to %s",CITY[startcity].Name,CITY[endcity].Name);

	// display length
	grDrawString(x1+10,y1+60,"Track length");
	thou = ROUTE[routenum].Length / 1000;
	less = ROUTE[routenum].Length % 1000;
	if (thou) grDrawString(x1+120,y1+60,": %d,%.3d Miles",thou,less);
	else      grDrawString(x1+120,y1+60,": %d Miles",less);

	// display ownership
	grDrawString(x1+10,y1+90,"Owner");
	if (ROUTE[routenum].Owner==-1)
		grDrawString(x1+120,y1+90,": None");
	else
		grDrawString(x1+120,y1+90,": %s",RAILROAD[ROUTE[routenum].Owner].Abbrev);

	// display route status
	grDrawString(x1+10,y1+110,"Status");
	switch (ROUTE[routenum].Status)
		{
		case SURVEYED:
			grDrawString(x1+120,y1+110,": Not Chartered");
			break;

		case CHARTERED:
			grDrawString(x1+120,y1+110,": Chartered");
			break;

		case PARTIALLY_BUILT:
			grDrawString(x1+120,y1+110,": Under Construction");
			grDrawString(x1+120,y1+125,"  (%d Years to Completion)",ROUTE[routenum].TurnsLeft);
			break;

		case FULLY_BUILT:
			grDrawString(x1+120,y1+110,": Completed");
			break;
		}

	// display revenue
	mill = ROUTE[routenum].TotalRevenueYTD / 1000;
	thou = ROUTE[routenum].TotalRevenueYTD % 1000;

	grDrawString(x1+10,y1+140,"Revenue YTD");

	if (mill) grDrawString(x1+120,y1+140,": $%d,%.3d,000",mill,thou);
	else	  grDrawString(x1+120,y1+140,": $%d,000",thou);

	// display land grants
	mill = ROUTE[routenum].LandGrants / 1000;
	thou = ROUTE[routenum].LandGrants % 1000;

	grDrawString(x1+10,y1+170,"Land Grants");
	if (mill) grDrawString(x1+120,y1+170,": $%d,%.3d,000",mill,thou);
	else	  grDrawString(x1+120,y1+170,": $%.3d,000",thou);

	info_box_displayed = ROUTE_INFO;

	if (auto_refresh) grBlitRegion(x1,y1,x2,y2);
	}




void	MapWin::DrawCity(int city)
	{
//	PlotCity(city);
//
	grRect tmpr = { CITY[city].ExtremeLeft,
					CITY[city].ExtremeTop,
					CITY[city].ExtremeRight,
					CITY[city].ExtremeBottom };

	RefreshMapRegion(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom);
	}


void	MapWin::DrawRoute(int route)
	{
	route_Plot(route);

	int city1 = ROUTE[route].StartCity;
	int	city2 = ROUTE[route].EndCity;

	grRect tmpr = { min(CITY[city1].ExtremeLeft,CITY[city2].ExtremeLeft),
					min(CITY[city1].ExtremeTop,CITY[city2].ExtremeTop),
					max(CITY[city1].ExtremeRight,CITY[city2].ExtremeRight),
					max(CITY[city1].ExtremeBottom,CITY[city2].ExtremeBottom) };

	RefreshMapRegion(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom);
	}


void	MapWin::DrawRouteSection(int route,	int section)
	{
	RefreshMapPoint(ROUTE[route].x[section],ROUTE[route].y[section]);
	}


void	MapWin::DrawTrain(int train)
	{
	grImage* train_pic;

	bool drawn = false;
	bool draw_1;
	bool draw_2;

	int tmp_x1 = 100000;
	int tmp_y1 = 100000;
	int tmp_x2 = 0;
	int tmp_y2 = 0;

	for (int i=0;i<MAX_CARS;i++)
		{
		draw_1 = DrawMapPoint(TRAIN[train].lastx[i],TRAIN[train].lasty[i]);
		if (draw_1)
			{
			tmp_x1 = min(tmp_x1,screen_x1);
			tmp_y1 = min(tmp_y1,screen_y1);
			tmp_x2 = max(tmp_x2,screen_x2);
			tmp_y2 = max(tmp_y2,screen_y2);
			}

#ifdef DEBUG_DRAWTRAIN
//	DBG(300+10*i,"REFRESHING TRAIN CARRIAGE %d AT %d,%d",
//		TRAIN[train].CarType[i],TRAIN[train].x[i],TRAIN[train].y[i]);
#endif

		train_pic = train_bitmaps[zoom_level][TRAIN[train].CarType[i]];
		draw_2 = DrawMapPoint(TRAIN[train].x[i],TRAIN[train].y[i],train_pic,grWHITE);
		if (draw_2)
			{
			tmp_x1 = min(tmp_x1,screen_x1);
			tmp_y1 = min(tmp_y1,screen_y1);
			tmp_x2 = max(tmp_x2,screen_x2);
			tmp_y2 = max(tmp_y2,screen_y2);
			}

		drawn |= draw_1 | draw_2;
		}

	if (drawn)
		{
		tmp_x1 = max(tmp_x1,r.left);
		tmp_y1 = max(tmp_y1,r.top);
		tmp_x2 = min(tmp_x2,r.right);
		tmp_y2 = min(tmp_y2,r.bottom);

#ifdef DEBUG_DRAWTRAIN
		grDrawRect(tmp_x1,tmp_y1,tmp_x2,tmp_y2,grMAGENTA);
#endif
		grBlitRegion(tmp_x1,tmp_y1,tmp_x2,tmp_y2);
		}
	}


void	MapWin::DrawTrainRoute(int train)
	{
	DrawRoute(TRAIN[train].Route);
	}


void	MapWin::DrawAllRoutes(int rr)
	{
	for (int route=0;route<NUM_ROUTES;route++)
		{
		if (ROUTE[route].Owner==rr)
			DrawRoute(route);
		}
	}

void	MapWin::DrawAllTrains(int rr)
	{
	for (int train=0;train<NUM_TRAINS;train++)
		{
		if (TRAIN[train].Owner==rr)
			DrawTrain(train);
		}
	}

void	MapWin::DrawAllTrainRoutes(int rr)
	{
	for (int train=0;train<NUM_TRAINS;train++)
		{
		if (TRAIN[train].Owner==rr)
			DrawTrainRoute(train);
		}
	}

// -------------------------------------------------------------------------
//
// CITY FUNCTIONS
//
// -------------------------------------------------------------------------

void	MapWin::HighlightCity(int citynum,int color)
	{
	int x = CITY[citynum].x + r.left;
	int y = CITY[citynum].y + r.top;
	grDrawRect(x-5,y-5,x+5,y+5,color);
	grBlitRegion(x-5,y-5,x+5,y+5);
	}

void	MapWin::HighlightRoute(int route,int color)
	{
	int city1 = ROUTE[route].StartCity;
	int	city2 = ROUTE[route].EndCity;

	grRect tmpr = { min(CITY[city1].ExtremeLeft,CITY[city2].ExtremeLeft),
					min(CITY[city1].ExtremeTop,CITY[city2].ExtremeTop),
					max(CITY[city1].ExtremeRight,CITY[city2].ExtremeRight),
					max(CITY[city1].ExtremeBottom,CITY[city2].ExtremeBottom) };

	grDrawRect(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom,color);
	grBlitRegion(tmpr.left,tmpr.top,tmpr.right,tmpr.bottom);
	}


int		MapWin::WhichCitySelected(grPoint& mouse_pt)
	{
	return 0;
	}

// -------------------------------------------------------------------------
//
// MISCELLANEOUS FUNCTIONS
//
// -------------------------------------------------------------------------

grPoint MapWin::WhichMapPos(grPoint &mouse_pt)
	{
	grPoint p;
	int dx,dy;
	int centre_x,centre_y;

	// convert our input coords to the window boundaries
//	mouse_pt.x -= r.left;
//	mouse_pt.y -= r.top;

	// and figure out the distance from the centre icon's edges
	if (mouse_pt.x>map_centre_x)	dx = mouse_pt.x - (map_centre_x - half_icon_width);
	else				   			dx = mouse_pt.x - (map_centre_x + half_icon_width);
	if (mouse_pt.y>map_centre_y) 	dy = mouse_pt.y - (map_centre_y - half_icon_height);
	else					 		dy = mouse_pt.y - (map_centre_y + half_icon_height);

	// simple formula to calculate the icon number
	p.x = displayed_x1 + half_icons_wide + dx / icon_width;

	// do the same for the y coord
	p.y = displayed_y1 + half_icons_high + dy / icon_height;

	// return our result
	return p;
	}



int		MapWin::WhichRRNumber(int x,int y)
	{
	if (WhatsAt(x,y)==MROUTE)
		{
		int route_num = GetRouteNumber(x,y);
		return ROUTE[route_num].Owner;
		}
	return -1;
	}

int		MapWin::WhichRouteNumber(int x,int y)
	{
	if (WhatsAt(x,y)==MROUTE) return GetRouteNumber(x,y);
	return -1;
	}

int		MapWin::WhichCityNumber(int x,int y)
	{
	if (WhatsAt(x,y)==MCITY) return GetCityNumber(x,y);
	return -1;
	}

int last_mx1 = 0;
int last_mx2 = 10;
int last_my1 = 0;
int last_my2 = 10;
bool	magnified_onscreen = false;

void	MapWin::MagnifyUnderCursor(grPoint& mouse_pt)
	{
	magnified_onscreen = true;

	// figure out which map point is at the mouse cursor
	grPoint pt = WhichMapPos(mouse_pt);

	// figure out the region we are going to magnify

	int x1 = pt.x - 5; 	// 11 x 11 region
	int y1 = pt.y - 5;
	int x2 = pt.x + 5;
	int y2 = pt.y + 5;

	if (x1<0) { x2 += x1; x1 = 0; }
	if (y1<0) { y2 += y1; y1 = 0; }

	if (x2>=map_width) 	{ int dx = x2 - map_width;  x1 -= dx; x2 -= dx; }
	if (y2>=map_height) { int dy = y2 - map_height; y1 -= dy; y2 -= dy; }

	// and figure out the location to put the magnification

	int mx1 = mouse_pt.x - 5 * 4;
	int my1 = mouse_pt.y - 5 * 4;
	int mx2 = mouse_pt.x + 5 * 4;
	int my2 = mouse_pt.y + 5 * 4;

	if (mx1<r.left) { int dx = r.left - mx1; mx1 += dx; mx2 += dx; }
	if (my1<r.top)  { int dy = r.top  - my1; my1 += dy; my2 += dy; }

	if (mx2>r.right)  { int dx = mx2 - r.right;  mx1 -= dx; mx2 -= dx; }
	if (my2>r.bottom) { int dy = my2 - r.bottom; my1 -= dy; my2 -= dy; }

	// actually draw the magnified image

	grImage* under = grCreateImage(mx1,my1,mx2,my2);
	if (under) grGetImage(mx1,my1,mx2,my2,under);

	int mx;
	int my = my1;

	for (int yy=y1;yy<y2;yy++)
		{
		mx = mx1;

		for (int xx=x1;xx<x2;xx++)
			{
			int ter = GetTerrain(xx,yy);
			int ovl = GetOverlaid(xx,yy);

			if (ovl==NOT_OVERLAID) grDrawFilledRect(mx,my,mx+4,my+4,terrain_color_table[ter]);
			else grDrawFilledRect(mx,my,mx+4,my+4,overlaid_color_table[ovl]);

			mx += 4;
			}
		my += 4;
		}

	grDrawRect(mx1,my1,mx2,my2,grLIGHTRED);
	grBlitRegion(min(mx1,last_mx1),min(my1,last_my1),
				 max(mx2,last_mx2),max(my2,last_my2));

	// display underneath the last region drawn

	if (under)
		{
		grPutImage(mx1,my1,mx2,my2,under);
		grDestroyImage(under);
		}

	last_mx1 = mx1;
	last_my1 = my1;
	last_mx2 = mx2;
	last_my2 = my2;
	}


// -------------------------------------------------------------------------
//
// EVENTHANDLER FUNCTIONS
//
// -------------------------------------------------------------------------

void MapWin::Draw()
	{
	// skip out if not ready to draw
	if (!map_ready) return;

	// display the map using current zoom level & position
	DrawMap();

	// display the info box iff required
	if (info_box_displayed!=NO_INFO) DrawMapInfo(last_infox,last_infoy,false);
	}

void MapWin::Animate()
	{
	}



void MapWin::HandleEvent(Event &e)
	{
	if (!isOnScreen) return;

	int jump;
	bool redraw = false;
	grPoint p;
	grRect* region;
	grRect	screen_region;

	switch (e.what)
		{
// COMMAND EVENTS
		case ev_command:

			switch (e.command.event)
				{
				case cmdRedraw:

					redraw = true;
					break;

				case cmdRefresh:

					redraw = true;
					break;

				case cmdRedrawMap:

					DrawMap();
					EVENTMGR->ClearEvent(e);
					return;

				case cmdRefreshMap:

					RefreshMap();
					EVENTMGR->ClearEvent(e);
					return;

				case cmdRedrawMapRegion:

					region = (grRect*) e.command.data;
					DrawMapRegion(region->left,region->top,region->right,region->bottom);
					EVENTMGR->ClearEvent(e);
					return;

				case cmdRefreshMapRegion:

					region = (grRect*) e.command.data;
					RefreshMapRegion(region->left,region->top,region->right,region->bottom);
					EVENTMGR->ClearEvent(e);
					return;

				case cmdRedrawScreenRegion:

					region = (grRect*) e.command.data;
					DrawScreenRegion(region->left,region->top,region->right,region->bottom);
					EVENTMGR->ClearEvent(e);
					return;

				case cmdRefreshScreenRegion:

					region = (grRect*) e.command.data;
					RefreshScreenRegion(region->left,region->top,region->right,region->bottom);
					EVENTMGR->ClearEvent(e);
					return;
				}
			break;

// MOUSE EVENTS
		case ev_mouse:

			switch (e.mouse.event)
				{
				case mb_leftdn:

					// we're not yet auto repeating
					lmb_auto = false;

					// figure out where the user clicked
					p = WhichMapPos(e.mouse.where);

					// if WhichRR is open, respond with RR number
					if ((QUERY_HANDLER == evhno_WhichRR) ||
						(QUERY_HANDLER == evhno_HowMany))
						{
						int rr = WhichRRNumber(p.x,p.y);

						if (SELECTED_RR == -1)
							{
							SELECTED_RR = rr;
							EVENTMGR->ModBroadcastEvent(e,cmdRRSelected,NULL);
							return;
							}

						if (rr != -1)
							{
							if (rr == SELECTED_RR)
								{
								EVENTMGR->ClearEvent(e);
								EVENTMGR->GenCommandEvent(QUERY_HANDLER,cmdRRSelected,NULL);
								return;
								}
							else
								{
								SELECTED_RR = rr;
								EVENTMGR->ModBroadcastEvent(e,cmdRRSelected,NULL);
								return;
								}
							}
						}

					// if WhichRoute is open, respond with route number
					if (QUERY_HANDLER == evhno_WhichRoute)
						{
						int route = WhichRouteNumber(p.x,p.y);

						if (SELECTED_ROUTE == -1)
							{
							SELECTED_ROUTE = route;
							EVENTMGR->ModBroadcastEvent(e,cmdRouteSelected,NULL);
							return;
							}

						if (route != -1)
							{
							if (route == SELECTED_ROUTE)
								{
								EVENTMGR->ClearEvent(e);
								EVENTMGR->GenCommandEvent(QUERY_HANDLER,cmdRouteSelected,NULL);
								return;
								}
							else
								{
								SELECTED_ROUTE = route;
								EVENTMGR->ModBroadcastEvent(e,cmdRouteSelected,NULL);
								return;
								}
							}
						}

					// if WhichCity is open, respond with city number
					if (QUERY_HANDLER == evhno_WhichCity)
						{
						int city = WhichCityNumber(p.x,p.y);

						if (SELECTED_CITY == -1)
							{
							SELECTED_CITY = city;
							EVENTMGR->ModBroadcastEvent(e,cmdCitySelected,NULL);
							return;
							}

						if (city != -1)
							{
							if (city == SELECTED_CITY)
								{
								EVENTMGR->ClearEvent(e);
								EVENTMGR->GenCommandEvent(QUERY_HANDLER,cmdCitySelected,NULL);
								return;
								}
							else
								{
								SELECTED_CITY = city;
								EVENTMGR->ModBroadcastEvent(e,cmdCitySelected,NULL);
								return;
								}
							}
						}

					// if TrainConfig is open, respond with city number
					if (QUERY_HANDLER == evhno_TrainConfig)
						{
						int city = WhichCityNumber(p.x,p.y);

						if (city!=-1)
							{
							EVENTMGR->ModBroadcastEvent(e,cmdCitySelected,(void*) city);
							return;
							}
						}


					// if we've clicked at a different location
					if ((p.x!=map_xpos)||(p.y!=map_ypos))
						{
						// scroll to the new location
						ScrollTo(p.x,p.y);
						redraw = true;
						}

				   // finished
				   EVENTMGR->ClearEvent(e);
				   break;


				case mb_leftauto:

					// we're obviously repeating some command
					lmb_auto = true;

					// and the zoom level isn't at maximum
					if (zoom_level!=MAX_ZOOM_LEVEL)
						{
						// zoom-in to the next level
						ZoomIn();
						redraw = true;

						// assume we've not finished zooming
						// so that we have to wait for button to be
						// released before auto-scrolling
						finished_zoom = false;
						}
					else
						// if we've finished zooming in
						if (finished_zoom)
							{
							// simply auto-scroll to where the mouse is
							p = WhichMapPos(e.mouse.where);
							ScrollTo(p.x,p.y);

							redraw = true;
							}
					break;

					// finished
					EVENTMGR->ClearEvent(e);
					break;


				case mb_leftup:

					// assume that we've finished zooming
					finished_zoom = true;

					EVENTMGR->ClearEvent(e);
					break;


				case mb_rightdn:

					// no auto repeating as yet
					rmb_auto = false;

					EVENTMGR->ClearEvent(e);
					break;


				case mb_rightauto:

					// we're now auto repeating
					rmb_auto = true;

					ZoomOut();
					redraw = true;

					EVENTMGR->ClearEvent(e);
					break;


				case mb_rightup:

					// if we didn't auto repeat, we want
					// to toggle the information box
					if (!rmb_auto)
						{
						// if the info box isn't displayed
						if (info_box_displayed==NO_INFO)
							{
							// figure out where we clicked
							p = WhichMapPos(e.mouse.where);

							// display relevant info
							DrawMapInfo(p.x,p.y);
							}
						else
							// otherwise get rid of the information
							UndrawMapInfo();
						}

					EVENTMGR->ClearEvent(e);
					break;



				case mb_move:

					// display mouse coords (temporarily)
					if (e.mouse.modifiers & keyboard_SHIFT)
						MagnifyUnderCursor(e.mouse.where);
					else
						if (magnified_onscreen)
							{
							grBlitRegion(last_mx1,last_my1,last_mx2,last_my2);
							magnified_onscreen = false;
							}

					EVENTMGR->ClearEvent(e);
					break;
				}

// KEYBOARD EVENTS
		case ev_keyboard:

			if (!isOnScreen) return;

			switch (e.keyboard.event)
				{

#ifdef GAMETEST
				case ONE_UP:

					if (e.keyboard.modifiers&keyboard_CTRL)
						{
						grPutImage(r.left,r.top,r.right,r.bottom,terrain_map);
						grDraw2CString(r.left,r.top,grWHITE,"TERRAIN MAP");
						grBlitRegion(r.left,r.top,r.right,r.bottom);
						}
					EVENTMGR->ClearEvent(e);
					break;

				case TWO_UP:

					if (e.keyboard.modifiers&keyboard_CTRL)
						{
						grPutImage(r.left,r.top,r.right,r.bottom,overlaid_map);
						grDraw2CString(r.left,r.top,grWHITE,"OVERLAID MAP");
						grBlitRegion(r.left,r.top,r.right,r.bottom);
						}
					EVENTMGR->ClearEvent(e);
					break;

				case THREE_UP:

					if (e.keyboard.modifiers&keyboard_CTRL)
						{
						grPutImage(r.left,r.top,r.right,r.bottom,height_map);
						grDraw2CString(r.left,r.top,grWHITE,"HEIGHT MAP");
						grBlitRegion(r.left,r.top,r.right,r.bottom);
						}
					EVENTMGR->ClearEvent(e);
					break;

				case FOUR_UP:

					if (e.keyboard.modifiers&keyboard_CTRL)
						{
						grPutImage(r.left,r.top,r.right,r.bottom,data_map_a);
						grDraw2CString(r.left,r.top,grWHITE,"DATA MAP A");
						grBlitRegion(r.left,r.top,r.right,r.bottom);
						}
					EVENTMGR->ClearEvent(e);
					break;

				case FIVE_UP:

					if (e.keyboard.modifiers&keyboard_CTRL)
						{
						grPutImage(r.left,r.top,r.right,r.bottom,data_map_b);
						grDraw2CString(r.left,r.top,grWHITE,"DATA MAP B");
						grBlitRegion(r.left,r.top,r.right,r.bottom);
						}
					EVENTMGR->ClearEvent(e);
					break;
#endif

				}
			break;

		}

	if (redraw)
		{
		if (INFOBOX_HANDLER!=-1)
			{
			Draw();

			EVENTMGR->GenCommandEvent(INFOBOX_HANDLER,cmdExternalRefresh,NULL);
			EVENTMGR->GenCommandEvent(evhno_MapWin,cmdRefreshMap,NULL);
			}
		else
			Refresh();

		EVENTMGR->ClearEvent(e);
		}
	}
