//	-------------------------------------------------------------------------
//
//	MKTBOX.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the code for the MARKET_BOX
//
//	-------------------------------------------------------------------------

#include "mktbox.hpp"
#include "gamedefn.hpp"
#include "eventmgr.hpp"
#include "buttons.hpp"

#include "player.hpp"
#include "railroad.hpp"
#include "economy.hpp"
#include "msgwin.hpp"

#include "intrface.hpp"

#include "dos.h"	// for delay()

MarketBox::MarketBox()
	: OnScreenHandler(mktbox_r.left,mktbox_r.top,mktbox_r.right,mktbox_r.bottom,
					  BG_COLOR,bs_NONE,bs_NONE,
					  &evhno_MarketBox,0,1)
	{
	isOnScreen = false;
	}

void	MarketBox::DrawGraph(bool auto_refresh)
	{
	int x1 = r.left + 20;
	int x2 = x1 + MAX_HISTORY * 10;
	int y1 = r.top + 10;
	int y2 = y1 + 200;

	// outline
	grDrawLine(x1+10,y1,x1+10,y2,grWHITE);
	grDrawLine(x1,y2,x2,y2,grWHITE);	// incl. dash

	// draw horiz dashes
	grDrawLine(x1,y1,x1+10,y1,grWHITE);
	grDrawLine(x1,y1+20,x1+10,y1+20,grWHITE);
	grDrawLine(x1,y1+40,x1+10,y1+40,grWHITE);
	grDrawLine(x1,y1+60,x1+10,y1+60,grWHITE);
	grDrawLine(x1,y1+80,x1+10,y1+80,grWHITE);
	grDrawLine(x1,y1+100,x1+10,y1+100,grWHITE);
	grDrawLine(x1,y1+120,x1+10,y1+120,grWHITE);
	grDrawLine(x1,y1+140,x1+10,y1+140,grWHITE);
	grDrawLine(x1,y1+160,x1+10,y1+160,grWHITE);
	grDrawLine(x1,y1+180,x1+10,y1+180,grWHITE);

	if (SELECTED_RR!=-1)
		{
		int hist = RAILROAD[SELECTED_RR].CurrentHistory + 1;
		if (hist>=MAX_HISTORY) hist = 0;

		int lastx = x1 + 10;
		int lasty = y2 - RAILROAD[SELECTED_RR].StockPrice[hist] * 2;

		for (int i=1;i<MAX_HISTORY;i++)
			{
			hist++;
			if (hist>=MAX_HISTORY) hist = 0;

			int x = lastx + 10;
			int y = y2 - RAILROAD[SELECTED_RR].StockPrice[hist] * 2;

			switch (INFOBOX_MODE)
				{
				case cmdStockMarket:

					grDrawLine(lastx,lasty,x,y,grLIGHTRED);
					break;

				case cmdBondMarket:

					grDrawLine(lastx,lasty,x,y,grLIGHTRED);
					break;
				}

			lastx = x;
			lasty = y;
			}

		grDraw2CString(x1+30,y1,TEXT_COLOR,"%s",RAILROAD[SELECTED_RR].Name);
		}

	if (auto_refresh) grBlitRegion(x1,y1,x2,y2);
	}

void	MarketBox::DrawInfo(bool auto_refresh)
	{
	int x1 = r.left + 20;
	int x2 = r.right - 20;
	int y1 = r.top + 220;
	int y2 = r.bottom - 20;

	switch (INFOBOX_MODE)
		{
		case cmdStockMarket:

			grDraw2CString(x1,y1,TEXT_COLOR,"Stock Price:  $%d",RAILROAD[SELECTED_RR].StockPrice[RAILROAD[SELECTED_RR].CurrentHistory]);
			grDraw2CString(x1,y1+20,TEXT_COLOR,"Stock Supply:  %d,000",RAILROAD[SELECTED_RR].StocksOnMarket);
			grDraw2CString(x1,y1+40,TEXT_COLOR,"Stock Demand:  %d,000",RAILROAD[SELECTED_RR].StocksInPlayer);
			grDraw2CString(x1,y1+60,TEXT_COLOR,"Stocks Owned:  %d,000",PLAYER[0].Stocks[SELECTED_RR]);
			break;

		case cmdBondMarket:

			grDraw2CString(x1,y1,TEXT_COLOR,"Bond Price:  $%d",RAILROAD[SELECTED_RR].BondPrice[RAILROAD[SELECTED_RR].CurrentHistory]);
			grDraw2CString(x1,y1+20,TEXT_COLOR,"Bond Supply:  %d,000",PLAYER[0].Bonds[SELECTED_RR]);
			grDraw2CString(x1,y1+40,TEXT_COLOR,"Bond Demand:  %d,000",PLAYER[0].Bonds[SELECTED_RR]);
			grDraw2CString(x1,y1+60,TEXT_COLOR,"Bonds Owned:  %d,000",PLAYER[0].Bonds[SELECTED_RR]);
			break;
		}

	if (auto_refresh) grBlitRegion(x1,y1,x2,y2);
	}

void MarketBox::Draw()
	{
	// draw basic window
	grDrawShadedRect(r.left,r.top,r.right,r.bottom,bgcolor,2);
	grDrawRect(r.left,r.top,r.right,r.bottom,grLIGHTGREEN);

	if (SELECTED_RR==-1)
		{
		grDraw2CString(r.left+10,r.top+50,TEXT_COLOR,"No Railroad Selected");
		}
	else
		{
		DrawGraph(false);
		DrawInfo(false);
		}
	}

void	MarketBox::Open()
	{
	if (!isOnScreen)
		{
		isOnScreen = true;

		EVENTMGR->GenCommandEvent(evhno_MarketBox,cmdRedraw,NULL);
		}
	}

void	MarketBox::Close()
	{
	if (isOnScreen)
		{
		isOnScreen = false;
		}
	}


void MarketBox::HandleEvent(Event &e)
	{
	// pre-process cmdOpen & cmdClose
	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdRedraw:

					EVENTMGR->GenCommandEvent(evhno_MapWin,cmdRedrawScreenRegion,&r);
					EVENTMGR->GenCommandEvent(evhno_MarketBox,cmdRefresh,NULL);
					EVENTMGR->ClearEvent(e);
					break;

				case cmdExternalRefresh:

					Draw();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdOpen:

					Open();
					EVENTMGR->ClearEvent(e);
					return;

				case cmdClose:

					Close();
					if (e.command.data==NULL) EVENTMGR->GenCommandEvent(evhno_MapWin,cmdRefreshScreenRegion,&r);
					else EVENTMGR->GenCommandEvent(evhno_MapWin,cmdRedrawScreenRegion,&r);
					EVENTMGR->ClearEvent(e);
					break;
				}
			break;

		case ev_broadcast:

			switch (e.broadcast.event)
				{
				case cmdRRSelected:

					EVENTMGR->GenCommandEvent(evhno_MarketBox,cmdRedraw,NULL);
					return;

				case cmdDayTick:

					if (isOnScreen)
//						if (SELECTED_RR!=-1)
							EVENTMGR->GenCommandEvent(evhno_MarketBox,cmdRedraw,NULL);
					return;
				}
			break;
		}

	// do all the _boring_ event processing
	OnScreenHandler::HandleEvent(e);
	}
