//	-------------------------------------------------------------------------
//
//	MSGWIN.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the code for the MESSAGE_WINDOW
//
//	-------------------------------------------------------------------------

#include "msgwin.hpp"
#include "gamedefn.hpp"
#include "eventmgr.hpp"
#include "buttons.hpp"
#include "mouse.hpp"
#include "timer.hpp"

#include "player.hpp"
#include "railroad.hpp"
#include "route.hpp"
#include "city.hpp"
#include "intrface.hpp"

#include "dos.h"	// for delay()

MessageWindow::MessageWindow()
	: OnScreenHandler(oppinfo_r.left,oppinfo_r.top,oppinfo_r.right,actions_r.bottom,
                      BG_COLOR,bs_NONE,bs_NONE,
					  &evhno_MessageWin,0,10)
	{
	isOnScreen = false;
	}

MessageWindow::~MessageWindow()
	{
	}

void MessageWindow::Draw()
	{
	// draw basic window
	grDrawWindow(r.left,r.top,r.right,r.bottom,bgcolor,active_bs);

	switch (RESPONSE_HEADER[response].Response)
		{
		// ---- PLAYER MESSAGES ----

		// BUY STOCKS
		case msgSentBuyStocksMessenger:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Sent messenger to buy %d,000 stocks in %s",
				PLAYER[CURRENT_PLAYER].BUYSTOCKS.Amount,
				RAILROAD[PLAYER[CURRENT_PLAYER].BUYSTOCKS.RR].Name);
			break;

		case msgNoStocksAvailable:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"There aren't any stocks on the market!");
			break;

		case msgCantAffordAnyStocks:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"You can't even afford 1,000 of the cheapest stocks!!");
			break;

		case msgCantBuyAllStocks:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"Messenger: I couldn't get you all the stocks,");
			grDraw2CString(r.left+10,r.top+30,TEXT_COLOR,
				"           But I was able to get you %d,000 stocks in %s",
				PLAYER[CURRENT_PLAYER].BUYSTOCKS.Amount,
				RAILROAD[PLAYER[CURRENT_PLAYER].BUYSTOCKS.RR].Name);
			break;

		case msgBoughtStocks:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Messenger: Bought %d,000 stocks in the %s",
				PLAYER[CURRENT_PLAYER].BUYSTOCKS.Amount,
				RAILROAD[PLAYER[CURRENT_PLAYER].BUYSTOCKS.RR].Name);
			break;

		// SELL STOCKS

		case msgSentSellStocksMessenger:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Sent messenger to sell %d,000 stocks in the %s",
				PLAYER[CURRENT_PLAYER].SELLSTOCKS.Amount,
				RAILROAD[PLAYER[CURRENT_PLAYER].SELLSTOCKS.RR].Name);
			break;

		case msgSoldStocks:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Messenger: Sold %d,000 %s stocks",
				PLAYER[CURRENT_PLAYER].SELLSTOCKS.Amount,
				RAILROAD[PLAYER[CURRENT_PLAYER].SELLSTOCKS.RR].Name);
			break;

		// BUY BONDS

		case msgSentBuyBondsMessenger:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Sent messenger to Buy Bonds");
			break;

		case msgBoughtBonds:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Messenger: Bought the bonds");
			break;

		// SELL BONDS

		case msgSentSellBondsMessenger:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Sent messenger to Sell Bonds");
			break;

		case msgSoldBonds:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Messenger: Sold the bonds");
			break;

		// CONTROL RR
		case msgSentControlRRMessenger:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Attempting to take control of the %s",
				RAILROAD[PLAYER[CURRENT_PLAYER].CONTROLRR.RR].Name);
			break;

		case msgControlledRR:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"You have successfully taken control or %s",
				RAILROAD[PLAYER[CURRENT_PLAYER].CONTROLRR.RR].Name);
			break;

		case msgCantControlRR:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"You don't have enough stocks to control the %s",
				RAILROAD[PLAYER[CURRENT_PLAYER].CONTROLRR.RR].Name);
			break;

		// RESIGN CONTROL RR
		case msgResignedControl:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"You have resigned control of the %s",
				RAILROAD[PLAYER[CURRENT_PLAYER].CONTROLRR.RR].Name);
			break;

		case msgCantResignControl:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"You don't have control of the %s",
				RAILROAD[PLAYER[CURRENT_PLAYER].CONTROLRR.RR].Name);
			break;

		// ACT FOR RR
		case msgNotPresident:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"You are not the president of the %s",
				RAILROAD[PLAYER[CURRENT_PLAYER].CONTROLRR.RR].Name);
			break;


		// ---- RR MESSAGES ----

		// BUY TRAIN
		case msgSentBuyTrainMessenger:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Sent messenger to buy train");
			break;

		case msgCantBuyTrain:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Cant buy train");
			break;

		case msgBoughtTrain:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Bought the train");
			break;

		// SURVEY ROUTE
		case msgSentSurveyRouteMessenger:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Surveying route between %s and %s",
				CITY[RAILROAD[SELECTED_RR].SURVEYROUTE.City1].Name,
				CITY[RAILROAD[SELECTED_RR].SURVEYROUTE.City2].Name);
			break;

		case msgCantSurveyRoute:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Could not survey the route between %s and %s",
				CITY[RAILROAD[SELECTED_RR].SURVEYROUTE.City1].Name,
				CITY[RAILROAD[SELECTED_RR].SURVEYROUTE.City2].Name);
			break;

		case msgSurveyedRoute:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Survey between %s and %s was successful",
				CITY[RAILROAD[SELECTED_RR].SURVEYROUTE.City1].Name,
				CITY[RAILROAD[SELECTED_RR].SURVEYROUTE.City2].Name);
			break;

		// CHARTER ROUTE
		case msgSentCharterRouteMessenger:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Sent messenger to charter route between %s and %s",
				CITY[ROUTE[RAILROAD[SELECTED_RR].CHARTERROUTE.Route].StartCity].Name,
				CITY[ROUTE[RAILROAD[SELECTED_RR].CHARTERROUTE.Route].EndCity].Name);
			break;

		case msgCharteredRoute:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"You have successfully chartered the route from %s to %s",
				CITY[ROUTE[RAILROAD[SELECTED_RR].CHARTERROUTE.Route].StartCity].Name,
				CITY[ROUTE[RAILROAD[SELECTED_RR].CHARTERROUTE.Route].EndCity].Name);
			break;

		case msgAlreadyBuildingTrack:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"You can't charter a new route till you finished building %s to %s",
				CITY[ROUTE[RAILROAD[SELECTED_RR].CharteredRoute].StartCity].Name,
				CITY[ROUTE[RAILROAD[SELECTED_RR].CharteredRoute].EndCity].Name);
			break;

		case msgCharteredByOther:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"That route is already chartered by the %s",
				RAILROAD[ROUTE[RAILROAD[SELECTED_RR].CHARTERROUTE.Route].Owner].Name);
			break;

		case msgFullyBuiltRoute:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"You cannot charter a route that has been completely built!");
			break;

		case msgNotAdjoiningRoute:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"You can only charter a route thats connected to the %s",
				RAILROAD[SELECTED_RR].Name);
			break;

		// BUILD TRACK

		case msgSentBuildTrackMessenger:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Track building commenced between %s and %s",
				CITY[ROUTE[RAILROAD[SELECTED_RR].CharteredRoute].StartCity].Name,
				CITY[ROUTE[RAILROAD[SELECTED_RR].CharteredRoute].EndCity].Name);
			break;

		case msgTrackFullyBuilt:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Track has been fully built");
			break;

		case msgFinishedBuildingSection:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Finished building current track section");
			break;

		case msgBuildingInProgress:

			grDraw2CString(r.left+10,r.top+20,TEXT_COLOR,
				"Track building currently in progress");
			break;
		}
	}

void	MessageWindow::Open()
	{
	if (!isOnScreen)
		{
		isOnScreen = true;

		Ticker = timer_AddTicker(GAME_DELAY * 40,cmdMessageTimeout);

		EVENTMGR->GenCommandEvent(evhno_OpponentInfo,cmdClose,NULL);
		if (PLAYER_MODE==MPLAYER) EVENTMGR->GenCommandEvent(evhno_PlayerActionsBar,cmdClose,NULL);
		else EVENTMGR->GenCommandEvent(evhno_RRActionsBar,cmdClose,NULL);
		EVENTMGR->GenCommandEvent(evhno_MessageWin,cmdRefresh,NULL);
		}
	}

void	MessageWindow::Close()
	{
	if (isOnScreen)
		{
		isOnScreen = false;

		timer_DelTicker(Ticker);

		EVENTMGR->GenCommandEvent(evhno_OpponentInfo,cmdOpen,NULL);
		if (PLAYER_MODE==MPLAYER) EVENTMGR->GenCommandEvent(evhno_PlayerActionsBar,cmdOpen,NULL);
		else EVENTMGR->GenCommandEvent(evhno_RRActionsBar,cmdOpen,NULL);
		}
	}

void MessageWindow::HandleEvent(Event &e)
	{
	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdOpen:

					response = (int) e.command.data;
					if (!isOnScreen) Open();
					else
						{
						timer_ResetTicker(Ticker);
						Refresh();
						}

					EVENTMGR->ClearEvent(e);
					break;

				case cmdClose:

					Close();
					EVENTMGR->ClearEvent(e);
					break;
				}

		case ev_timer:

			switch (e.timer.event)
				{
				case cmdMessageTimeout:

					Close();
					EVENTMGR->ClearEvent(e);
					break;
				}
		}

	// do all the _boring_ event processing
	OnScreenHandler::HandleEvent(e);

	switch (e.what)
		{
		case ev_mouse:

			switch (e.mouse.event)
				{
				case mb_leftup:	;
				case mb_rightup:

					Close();
					EVENTMGR->ClearEvent(e);
					break;
				}
			break;
		}

	}
