//	-------------------------------------------------------------------------
//
//	PLACTION.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the code for the PLAYER_ACTIONS_BAR
//
//	-------------------------------------------------------------------------

#include "plaction.hpp"
#include "gamedefn.hpp"
#include "eventmgr.hpp"
#include "buttons.hpp"
#include "mouse.hpp"

#include "player.hpp"
#include "railroad.hpp"
#include "economy.hpp"

#include "intrface.hpp"
#include "msgwin.hpp"

#include "dos.h"	// for delay()

PlayerActionsBar::PlayerActionsBar()
	: OnScreenHandler(actions_r.left,actions_r.top,actions_r.right,actions_r.bottom,
					  BG_COLOR,bs_NONE,bs_NONE,
					  &evhno_PlayerActionsBar,0,1)
	{
	// open the action buttons resourcefile
	buttons_file = new ResourceFile("resource\\plaction.res");
	if (!buttons_file)
		{
		DBG(470,"ERROR (OPTIONS): Could not find RESOURCE\\PLACTION.RES");
		delay(2000);
		return;
		}
	// read in all required buttons
	buttons = (grImage**) buttons_file->ReadAllRecords();
	if (!buttons)
		{
		DBG(470,"ERROR (OPTIONS): Could not read RESOURCE\\PLACTION.RES");
		delay(2000);
		delete buttons_file;
		return;
		}

	// we've successfully read in all appropriate buttons....
	// so create all required accessories

	int x = 0;
	AddAccessory(new PictureButton(x,0,buttons[1],buttons[0],0,cmdBuyStocks)); 			x += 40;
	AddAccessory(new PictureButton(x,0,buttons[3],buttons[2],0,cmdSellStocks));    		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[5],buttons[4],0,cmdBuyBonds));      		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[7],buttons[6],0,cmdSellBonds));    		x += 40;

	AddAccessory(new PictureButton(x,0,buttons[9],buttons[8],0,cmdGetLoan));      		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[11],buttons[10],0,cmdRepayLoan));  		x += 40;

	AddAccessory(new PictureButton(x,0,buttons[13],buttons[12],0,cmdSpyOnPlayer));		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[15],buttons[14],0,cmdDealWithPlayer));	x += 40;

	AddAccessory(new PictureButton(x,0,buttons[17],buttons[16],0,cmdSpyOnRR));			x += 40;
	AddAccessory(new PictureButton(x,0,buttons[19],buttons[18],0,cmdControlRR));		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[21],buttons[20],0,cmdResignControlRR));	x += 40;
	AddAccessory(new PictureButton(x,0,buttons[23],buttons[22],0,cmdGiveRRLoan));		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[25],buttons[24],0,cmdRecallRRLoan));		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[27],buttons[26],0,cmdIncorporateRR));	x += 40;
	AddAccessory(new PictureButton(x,0,buttons[29],buttons[28],0,cmdMergeRRs));			x += 40;
	AddAccessory(new PictureButton(x,0,buttons[31],buttons[30],0,cmdActForRR));			x += 40;
	}

PlayerActionsBar::~PlayerActionsBar()
	{
	DelAllAccessories();

	if (buttons) buttons_file->FreeAllRecords((char**) buttons);
	delete buttons_file;
	}

void 	PlayerActionsBar::Draw()
	{
	// draw basic window
	grDrawWindow(r.left,r.top,r.right,r.bottom,bgcolor,active_bs);

	// display all buttons
	DrawAllAccessories();
	}


void 	PlayerActionsBar::HandleEvent(Event &e)
	{
	// do all the _boring_ event processing
	OnScreenHandler::HandleEvent(e);

	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case 	cmdBuyStocks:
// BUY STOCKS
					// if we're in the process of buying stocks, don't
					// do anything
					if (PLAYER[CURRENT_PLAYER].BUYSTOCKS.Cnt>0)
						{
						EVENTMGR->ClearEvent(e);
						return;
						}

					// if there are no stocks on market
					if (STOCKS_ON_MARKET==0)
						{
						// display appropriate message & skip out
						EVENTMGR->ModCommandEvent(e,evhno_MessageWin,cmdOpen,(void*) msgNoStocksAvailable);
						return;
						}

					// if the player can't afford even 1000 stocks
					if (PLAYER[CURRENT_PLAYER].Cash < RAILROAD[MIN_STOCK].StockPrice[RAILROAD[MIN_STOCK].CurrentHistory])
						{
						// display appropriate message & skip out
						EVENTMGR->ModCommandEvent(e,evhno_MessageWin,cmdOpen,(void*) msgCantAffordAnyStocks);
						return;
						}

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// SELL STOCKS
				case cmdSellStocks:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// BUY BONDS
				case cmdBuyBonds:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// SELL BONDS
				case cmdSellBonds:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// GET LOAN
				case cmdGetLoan:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// REPAY LOAN
				case cmdRepayLoan:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// SPY ON PLAYER
				case cmdSpyOnPlayer:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// DEAL WITH PLAYER
				case cmdDealWithPlayer:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// SPY ON RR
				case cmdSpyOnRR:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// CONTROL RR
				case cmdControlRR:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// RESIGN CONTROL RR
				case cmdResignControlRR:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// GIVE RR LOAN
				case cmdGiveRRLoan:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// RECALL RR LOAN
				case cmdRecallRRLoan:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// INCORPORATE RR
				case cmdIncorporateRR:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// MERGE RRS
				case cmdMergeRRs:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;

// ACT FOR RR
				case cmdActForRR:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoPlayerAction,(void*) e.command.event);
					break;
				}
			break;
		}
	}
