//	-------------------------------------------------------------------------
//
//	RRACTION.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the code for the RR_ACTIONS_BAR
//
//	-------------------------------------------------------------------------

#include "rraction.hpp"
#include "gamedefn.hpp"
#include "eventmgr.hpp"
#include "buttons.hpp"
#include "mouse.hpp"

#include "player.hpp"
#include "railroad.hpp"
#include "economy.hpp"

#include "msgwin.hpp"

#include "dos.h"	// for delay()

RRActionsBar::RRActionsBar()
	: OnScreenHandler(actions_r.left,actions_r.top,actions_r.right,actions_r.bottom,
                      BG_COLOR,bs_NONE,bs_NONE,
					  &evhno_RRActionsBar,0,1)
	{
	// open the action buttons resourcefile
	buttons_file = new ResourceFile("resource\\rraction.res");
	if (!buttons_file)
		{
		DBG(470,"ERROR (OPTIONS): Could not find RESOURCE\\RRACTION.RES");
		delay(2000);
		return;
		}
	// read in all required buttons
	buttons = (grImage**) buttons_file->ReadAllRecords();
	if (!buttons)
		{
		DBG(470,"ERROR (OPTIONS): Could not read RESOURCE\\RRACTION.RES");
		delay(2000);
		delete buttons_file;
		return;
		}

	// we've successfully read in all appropriate buttons....
	// so create all required accessories

	int x = 0;
	AddAccessory(new PictureButton(x,0,buttons[1],buttons[0],0,cmdRR_BuyStocks)); 			x += 40;
	AddAccessory(new PictureButton(x,0,buttons[3],buttons[2],0,cmdRR_SellStocks));    		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[5],buttons[4],0,cmdRR_BuyBonds));      		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[7],buttons[6],0,cmdRR_SellBonds));    		x += 40;

	AddAccessory(new PictureButton(x,0,buttons[9],buttons[8],0,cmdRR_RepayBank));      		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[11],buttons[10],0,cmdRR_RepayDelinquent));  	x += 40;
	AddAccessory(new PictureButton(x,0,buttons[13],buttons[12],0,cmdRR_SetDividend));		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[15],buttons[14],0,cmdRR_RaiseEfficiency));	x += 40;

	AddAccessory(new PictureButton(x,0,buttons[17],buttons[16],0,cmdRR_BuySubsidiary));		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[19],buttons[18],0,cmdRR_SellSubsidiary));	x += 40;
	AddAccessory(new PictureButton(x,0,buttons[21],buttons[20],0,cmdRR_BuyTrain));			x += 40;
	AddAccessory(new PictureButton(x,0,buttons[23],buttons[22],0,cmdRR_SellTrain));			x += 40;
	AddAccessory(new PictureButton(x,0,buttons[25],buttons[24],0,cmdRR_SurveyRoute));		x += 40;

	AddAccessory(new PictureButton(x,0,buttons[27],buttons[26],0,cmdRR_CharterRoute));		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[29],buttons[28],0,cmdRR_BuildTrack));		x += 40;
	AddAccessory(new PictureButton(x,0,buttons[31],buttons[30],0,cmdRR_ActForPlayer));		x += 40;

	isOnScreen = false;
	}

RRActionsBar::~RRActionsBar()
	{
	DelAllAccessories();

	if (buttons) buttons_file->FreeAllRecords((char**) buttons);
	delete buttons_file;
	}

void 	RRActionsBar::Draw()
	{
	// draw basic window
	grDrawWindow(r.left,r.top,r.right,r.bottom,bgcolor,active_bs);

	// display all buttons
	DrawAllAccessories();
	}


void 	RRActionsBar::HandleEvent(Event &e)
	{
	// do all the _boring_ event processing
	OnScreenHandler::HandleEvent(e);

	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case 	cmdRR_BuyStocks:
				case 	cmdRR_SellStocks:
				case	cmdRR_BuyBonds:
				case	cmdRR_SellBonds:
				case	cmdRR_RepayBank:
				case	cmdRR_RepayDelinquent:
				case	cmdRR_SetDividend:
				case	cmdRR_RaiseEfficiency:
				case	cmdRR_BuySubsidiary:
				case	cmdRR_SellSubsidiary:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoRRAction,(void*) e.command.event);
					break;

				case	cmdRR_BuyTrain:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoRRAction,(void*) e.command.event);
					break;

				case	cmdRR_SellTrain:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoRRAction,(void*) e.command.event);
					break;

				case	cmdRR_SurveyRoute:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoRRAction,(void*) e.command.event);
					break;

				case	cmdRR_CharterRoute:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoRRAction,(void*) e.command.event);
					break;

				case	cmdRR_BuildTrack:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoRRAction,(void*) e.command.event);
					break;

				case	cmdRR_ActForPlayer:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,cmdDoRRAction,(void*) e.command.event);
					break;
				}
			break;
		}
	}
