//	-------------------------------------------------------------------------
//
//	RRINFBAR.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the code for the RR_INFO_BAR
//
//	-------------------------------------------------------------------------

#include "rrinfbar.hpp"
#include "gamedefn.hpp"
#include "eventmgr.hpp"
#include "buttons.hpp"
#include "mouse.hpp"

#include "player.hpp"
#include "railroad.hpp"
#include "economy.hpp"

#include "intrface.hpp"
#include "msgwin.hpp"

#include "dos.h"	// for delay()

RRInfoBar::RRInfoBar()
	: OnScreenHandler(infobar_r.left,infobar_r.top,infobar_r.right,infobar_r.bottom,
                      BG_COLOR,bs_NONE,bs_NONE,
					  &evhno_RRInfoBar,0,1)
	{
	// open the action buttons resourcefile
	buttons_file = new ResourceFile("resource\\rrinfbar.res");
	if (!buttons_file)
		{
		DBG(470,"ERROR (OPTIONS): Could not find RESOURCE\\RRINFBAR.RES");
		delay(2000);
		return;
		}
	// read in all required buttons
	buttons = (grImage**) buttons_file->ReadAllRecords();
	if (!buttons)
		{
		DBG(470,"ERROR (OPTIONS): Could not read RESOURCE\\RRINFBAR.RES");
		delay(2000);
		delete buttons_file;
		return;
		}

	// we've successfully read in all appropriate buttons....
	// so create all required accessories

	int x = 0;
	AddAccessory(new PictureButton(x,0,buttons[1],buttons[0],0,cmdStockMarket));	x += 80;
	AddAccessory(new PictureButton(x,0,buttons[3],buttons[2],0,cmdBondMarket)); 	x += 80;
	AddAccessory(new PictureButton(x,0,buttons[5],buttons[4],0,cmdBank));       	x += 80;
	AddAccessory(new PictureButton(x,0,buttons[7],buttons[6],0,cmdBank));       	x += 80;
	AddAccessory(new PictureButton(x,0,buttons[9],buttons[8],0,cmdBank));       	x += 80;
	AddAccessory(new PictureButton(x,0,buttons[11],buttons[10],0,cmdBank));       	x += 80;
	AddAccessory(new PictureButton(x,0,buttons[13],buttons[12],0,cmdBank));       	x += 80;
	AddAccessory(new PictureButton(x,0,buttons[15],buttons[14],0,cmdBank));       	x += 80;

	isOnScreen = false;
	}

RRInfoBar::~RRInfoBar()
	{
	DelAllAccessories();

	if (buttons) buttons_file->FreeAllRecords((char**) buttons);
	delete buttons_file;
	}

void RRInfoBar::Draw()
	{
	// draw basic window
	grDrawWindow(r.left,r.top,r.right,r.bottom,bgcolor,active_bs);

	// display all buttons
	DrawAllAccessories();
	}


void	RRInfoBar::OpenInfoBox( int handler, int mode )
	{
	// if no infobox is open
	if (INFOBOX_HANDLER==-1)
		{
		// open the required handler in the required mode
		INFOBOX_HANDLER = handler;
		INFOBOX_MODE = mode;
		EVENTMGR->GenCommandEvent(INFOBOX_HANDLER,cmdOpen,NULL);
		}
	else
		{
		// if the box is different, close the current box & open the new box
		if ((INFOBOX_HANDLER != handler) || (INFOBOX_MODE != mode))
			{
			// since we're going to redraw a new box,
			// redraw underneath the old handler but don't blit it
			EVENTMGR->GenCommandEvent(INFOBOX_HANDLER,cmdClose,(void*) true); // not NULL

			INFOBOX_HANDLER = handler;
			INFOBOX_MODE = mode;

			EVENTMGR->GenCommandEvent(handler,cmdOpen,NULL);
			}
		else
			{
			// simply close the box
			EVENTMGR->GenCommandEvent(INFOBOX_HANDLER,cmdClose,NULL);

			INFOBOX_HANDLER = -1;
			INFOBOX_MODE = -1;
			}
		}
	}
void RRInfoBar::HandleEvent(Event &e)
	{
	// do all the _boring_ event processing
	OnScreenHandler::HandleEvent(e);

	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case 	cmdStockMarket:
				case 	cmdBondMarket:

					OpenInfoBox(evhno_MarketBox,e.command.event);
					EVENTMGR->ClearEvent(e);
					break;

				case 	cmdBank:

					OpenInfoBox(evhno_BankBox,e.command.event);
					EVENTMGR->ClearEvent(e);
					break;
				}
			break;
		}
	}
