//	-------------------------------------------------------------------------
//
//	TRAINCFG.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is window asking "Configure the Train"
//
//	-------------------------------------------------------------------------

#include "traincfg.hpp"
#include "gamedefn.hpp"
#include "eventmgr.hpp"
#include "stddefns.hpp"
#include "gbuttons.hpp"
#include "buttons.hpp"
#include "mouse.hpp"

#include "player.hpp"
#include "gameproc.hpp"
#include "route.hpp"
#include "city.hpp"
#include "train.hpp"
#include "intrface.hpp"
#include "mapwin.hpp"

#include "dos.h"

#define		cmdSelectCheckPt1	30000
#define		cmdSelectCheckPt2	30001
#define		cmdSelectCheckPt3	30002
#define		cmdSelectCheckPt4	30003

TrainConfig::TrainConfig()
	: OnScreenHandler(oppinfo_r.left,oppinfo_r.top,oppinfo_r.right,actions_r.bottom,
					  BG_COLOR,bs_NONE,bs_NONE,
					  &evhno_TrainConfig,0,2)
	{
	// we don't wish to be onscreen unless selected
	isOnScreen = false;

	check[0] = AddAccessory(new CitySelectButton(10,10,110,55,grDARKGRAY,grLIGHTGRAY,bs_RAISED,bs_LOWERED,0,cmdSelectCheckPt1));
	check[1] = AddAccessory(new CitySelectButton(115,10,215,55,grDARKGRAY,grLIGHTGRAY,bs_RAISED,bs_LOWERED,0,cmdSelectCheckPt2));
	check[2] = AddAccessory(new BlankButton(220,10,320,55,grDARKGRAY,grLIGHTGRAY,bs_RAISED,bs_LOWERED,0,cmdSelectCheckPt3));
	check[3] = AddAccessory(new BlankButton(325,10,425,55,grDARKGRAY,grLIGHTGRAY,bs_RAISED,bs_LOWERED,0,cmdSelectCheckPt4));

	done   = AddAccessory(new TextButton(430,20,"   OK   ",0,cmdTrainConfigOK));
	cancel = AddAccessory(new TextButton(430,40," CANCEL ",0,cmdTrainConfigCANCEL));

	new_train = true;
	}


void TrainConfig::Draw()
	{
	// draw the basic window
	grDrawWindow(r.left,r.top,r.right,r.bottom,bgcolor,active_bs);

	DrawAllAccessories();

	// draw static labels
	grSetFont(BASICFONT);

	int i,color;

	int x = r.left + 10;
	int y = r.top + 15;

	for (i=0;i<4;i++)
		{
		if (i==selected_check_point) color = grWHITE;
		else color = grGREEN;
		grDraw2CString(x+30,y,color," City");
		x += 105;
		}

	// draw the city names
	switch (QUERY_MODE)
		{
		// RAILROAD ACTIONS
		case cmdRR_BuyTrain:

			for (i=0;i<4;i++)
				if (check_points[i]!=-1)
					grDraw2CString(r.left+10+i*105+20,r.top+30,grBLUE,"%s",CITY[check_points[i]].Name);
			break;
		}
	}

void	TrainConfig::Open()
	{
	if (!isOnScreen)
		{
		if (new_train)
			{
			selected_check_point = 0;
			check_points_entered = 0;

			for (int i=0;i<4;i++) check_points[i] = -1;

			check[0]->Activate();
			check[1]->DeActivate();
			check[2]->DeActivate();
			check[3]->DeActivate();

			done->DeActivate();
			cancel->Activate();
			}

		// open the howmany window
		OnScreenHandler::Open();
		}
	}

void	TrainConfig::Close()
	{
	if (isOnScreen)
		{
		// we're no longer onscreen
		isOnScreen = false;
		}
	}

void	TrainConfig::HighlightSelectedRoute()
	{
	if (check_points[0]==-1) return;

	int i,num_pts = 0;
	for (i=0;i<4;i++)
		if (check_points[i]!=-1) num_pts++;

	// display checkpoint0 to checkpoint1
	int save_trains = NUM_TRAINS;

	CreateNewTrain(SELECTED_RR,PASSENGER_TRAIN,&check_points[0],num_pts);

	do
		{
		train_DoArrival(train);
		train_DoDeparture(train);
		MAPWIN->HighlightRoute(TRAIN[train].Route,grWHITE);
		train_DoMove(train);
		}
	while (TRAIN[train].LastCity != check_points[0]);
	}

void 	TrainConfig::HandleEvent(Event& e)
	{
	int i,route;

	// pre-process broadcasts so we can skip out quickly.....
	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdOpen:

					Open();
					EVENTMGR->ClearEvent(e);
					return;

				}
			break;

		case ev_broadcast:

			switch (e.broadcast.event)
				{
				case cmdCitySelected:

					if (isOnScreen)
						{
						check_points[selected_check_point] = (int) e.command.data;

						if (selected_check_point==check_points_entered)
							{
							if (selected_check_point<3)
								{
								selected_check_point++;
								done->Activate();
								}
							if (check_points_entered<4)
								{
								check_points_entered++;
								check[check_points_entered]->Activate();
								}
							}
						else
							if (check_points_entered>0)
								{
								selected_check_point = check_points_entered-1;
								}
							else
								selected_check_point = 0;
/*
						if ((check_points_entered<4) &&
							(selected_check_point==check_points_entered))
							check_points_entered++;

						if (selected_check_point<4)
							{
							if (selected_check_point==check_points_entered)
								selected_check_point++;
							else
								selected_check_point = check_points_entered-1;
							}
*/
						Refresh();

//						HighlightSelectedRoute();
						}
					return;
				}
		}

	// let OnScreenHandler check all other events.....
	OnScreenHandler::HandleEvent(e);

	// and lets work with whats left
	switch (e.what)
		{
		case ev_mouse:

			switch (e.mouse.event)
				{
				// simply redirect the event as a command event
				case mb_leftup:

					EVENTMGR->ModCommandEvent(e,evhno_TrainConfig,cmdTrainConfigOK,NULL);
					break;

				case mb_rightup:

					EVENTMGR->ModCommandEvent(e,evhno_TrainConfig,cmdTrainConfigCANCEL,NULL);
					break;
				}
			break;

		case ev_command:

			switch (e.command.event)
				{
				case cmdTrainConfigOK:

					RAILROAD[SELECTED_RR].BUYTRAIN.Owner 	= SELECTED_RR;
					RAILROAD[SELECTED_RR].BUYTRAIN.LocoType = GRASSHOPPER;
					RAILROAD[SELECTED_RR].BUYTRAIN.CarType  = PASSENGER_TRAIN;

					RAILROAD[SELECTED_RR].BUYTRAIN.NumCheckPoints = 0;

					for (i=0;i<4;i++)
						if (check_points[i] != -1)
							{
							RAILROAD[SELECTED_RR].BUYTRAIN.CheckPoint[
								RAILROAD[SELECTED_RR].BUYTRAIN.NumCheckPoints++] = check_points[i];
							}

					EVENTMGR->ModCommandEvent(e,evhno_Interface,e.command.event,NULL);
					break;

				case cmdTrainConfigCANCEL:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,e.command.event,NULL);
					break;

				case cmdSelectCheckPt1:

					selected_check_point = 0;
					Refresh();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdSelectCheckPt2:

					selected_check_point = 1;
					Refresh();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdSelectCheckPt3:

					selected_check_point = 2;
					Refresh();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdSelectCheckPt4:

					selected_check_point = 3;
					Refresh();
					EVENTMGR->ClearEvent(e);
					break;
				}
			break;
		}
	}




