//	-------------------------------------------------------------------------
//
//	WHICHCIT.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is window asking "Which City?"
//
//	-------------------------------------------------------------------------

#include "whichcit.hpp"
#include "gamedefn.hpp"
#include "eventmgr.hpp"
#include "stddefns.hpp"
#include "buttons.hpp"
#include "mouse.hpp"

#include "player.hpp"
#include "gameproc.hpp"
#include "route.hpp"
#include "city.hpp"
#include "intrface.hpp"

#include "dos.h"

WhichCity::WhichCity()
	: OnScreenHandler(oppinfo_r.left,oppinfo_r.top,oppinfo_r.right,actions_r.bottom,
					  BG_COLOR,bs_NONE,bs_NONE,
					  &evhno_WhichCity,0,2)
	{
	// we don't wish to be onscreen unless selected
	isOnScreen = false;

	AddAccessory(new TextButton(400,20,"   OK   ",0,cmdWhichCityOK));
	AddAccessory(new TextButton(400,40," CANCEL ",0,cmdWhichCityCANCEL));
	}


void WhichCity::Draw()
	{
	// draw the basic window
	grDrawWindow(r.left,r.top,r.right,r.bottom,bgcolor,active_bs);

	// draw static labels
//	grSetFont(TITLEFONT);
	switch (QUERY_MODE)
		{
		// RAILROAD ACTIONS
		case cmdRR_BuyTrain:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"Starting from which city?");
			break;

		case cmdRR_SurveyRoute:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"Survey from which city?");
			break;

		case cmdRR_SurveyRoute_1:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"To which city?");
			break;
		}

	// as well as all accessories
	DrawAllAccessories();
	}

void	WhichCity::Open()
	{
	if (!isOnScreen)
		{
		// open the howmany window
		OnScreenHandler::Open();
		}
	}

void	WhichCity::Close()
	{
	if (isOnScreen)
		{
		// we're no longer onscreen
		isOnScreen = false;
		}
	}

void WhichCity::HandleEvent(Event& e)
	{
	// pre-process broadcasts so we can skip out quickly.....
	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdOpen:

					Open();
					EVENTMGR->ClearEvent(e);
					return;

				case cmdCitySelected:

					if (isOnScreen) Refresh();
					EVENTMGR->ModCommandEvent(e,evhno_WhichCity,cmdWhichCityOK,NULL);
					break;
				}
			break;

		case ev_broadcast:

			switch (e.broadcast.event)
				{
				case cmdCitySelected:

					if (isOnScreen) Refresh();
					return;
				}
		}

	// let OnScreenHandler check all other events.....
	OnScreenHandler::HandleEvent(e);

	// and lets work with whats left
	switch (e.what)
		{
		case ev_mouse:

			switch (e.mouse.event)
				{
				// simply redirect the event as a command event
				case mb_leftup:

					EVENTMGR->ModCommandEvent(e,evhno_WhichCity,cmdWhichCityOK,NULL);
					break;

				case mb_rightup:

					EVENTMGR->ModCommandEvent(e,evhno_WhichCity,cmdWhichCityCANCEL,NULL);
					break;
				}
			break;

		case ev_command:

			switch (e.command.event)
				{
				case cmdWhichCityOK:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,e.command.event,(void*) SELECTED_CITY);
					break;

				case cmdWhichCityCANCEL:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,e.command.event,NULL);
					break;
				}
			break;
		}
	}




