//	-------------------------------------------------------------------------
//
//	WHICHROU.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is window asking "Which Route?"
//
//	-------------------------------------------------------------------------

#include "whichrou.hpp"
#include "gamedefn.hpp"
#include "eventmgr.hpp"
#include "stddefns.hpp"
#include "buttons.hpp"
#include "mouse.hpp"

#include "player.hpp"
#include "gameproc.hpp"
#include "route.hpp"
#include "city.hpp"
#include "intrface.hpp"

#include "dos.h"

WhichRoute::WhichRoute()
	: OnScreenHandler(oppinfo_r.left,oppinfo_r.top,oppinfo_r.right,actions_r.bottom,
					  BG_COLOR,bs_NONE,bs_NONE,
					  &evhno_WhichRoute,0,2)
	{
	// we don't wish to be onscreen unless selected
	isOnScreen = false;

	AddAccessory(new TextButton(400,20,"   OK   ",0,cmdWhichRouteOK));
	AddAccessory(new TextButton(400,40," CANCEL ",0,cmdWhichRouteCANCEL));
	}


void WhichRoute::Draw()
	{
	// draw the basic window
	grDrawWindow(r.left,r.top,r.right,r.bottom,bgcolor,active_bs);

	// draw static labels
//	grSetFont(TITLEFONT);
	switch (QUERY_MODE)
		{
		// RAILROAD ACTIONS
		case cmdRR_BuyTrain:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"Buy train for which route?");
			break;

		case cmdRR_CharterRoute:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"Charter which Route?");
			break;

		}

	if (SELECTED_ROUTE != -1) grDraw2CString(r.left+10,r.top+30,TEXT_COLOR,
						"%s to %s",
						CITY[ROUTE[SELECTED_ROUTE].StartCity].Name,
						CITY[ROUTE[SELECTED_ROUTE].EndCity].Name);

	// as well as all accessories
	DrawAllAccessories();
	}

void	WhichRoute::Open()
	{
	if (!isOnScreen)
		{
		// open the howmany window
		OnScreenHandler::Open();
		}
	}

void	WhichRoute::Close()
	{
	if (isOnScreen)
		{
		// we're no longer onscreen
		isOnScreen = false;
		}
	}


void WhichRoute::HandleEvent(Event& e)
	{
	// pre-process broadcasts so we can skip out quickly.....
	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdOpen:

					Open();
					EVENTMGR->ClearEvent(e);
					return;

				case cmdRouteSelected:

					if (isOnScreen) Refresh();
					EVENTMGR->ModCommandEvent(e,evhno_WhichRoute,cmdWhichRouteOK,NULL);
					break;
				}
			break;

		case ev_broadcast:

			switch (e.broadcast.event)
				{
				case cmdRouteSelected:

					if (isOnScreen) Refresh();
					return;
				}
		}

	// let OnScreenHandler check all other events.....
	OnScreenHandler::HandleEvent(e);

	// and lets work with whats left
	switch (e.what)
		{
		case ev_mouse:

			switch (e.mouse.event)
				{
				// simply redirect the event as a command event
				case mb_leftup:

					EVENTMGR->ModCommandEvent(e,evhno_WhichRoute,cmdWhichRouteOK,NULL);
					break;

				case mb_rightup:

					EVENTMGR->ModCommandEvent(e,evhno_WhichRoute,cmdWhichRouteCANCEL,NULL);
					break;
				}
			break;

		case ev_command:

			switch (e.command.event)
				{
				case cmdWhichRouteOK:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,e.command.event,(void*) SELECTED_ROUTE);
					break;

				case cmdWhichRouteCANCEL:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,e.command.event,NULL);
					break;
				}
			break;
		}
	}


