//	-------------------------------------------------------------------------
//
//	WHICHRR.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is window asking "Which RR?"
//
//	-------------------------------------------------------------------------

#include "whichrr.hpp"
#include "gamedefn.hpp"
#include "eventmgr.hpp"
#include "stddefns.hpp"
#include "buttons.hpp"
#include "mouse.hpp"

#include "player.hpp"
#include "gameproc.hpp"
#include "msgwin.hpp"
#include "intrface.hpp"

#include "dos.h"

#define	cmdScrollUp		30002
#define	cmdScrollDn		30003

WhichRR::WhichRR()
	: OnScreenHandler(oppinfo_r.left,oppinfo_r.top,oppinfo_r.right,actions_r.bottom,
					  BG_COLOR,bs_NONE,bs_NONE,
					  &evhno_WhichRR,0,2)
	{
	// we don't wish to be onscreen unless selected
	isOnScreen = false;

	AddAccessory(new TextButton(600,10,"   OK   ",0,cmdWhichRROK));
	AddAccessory(new TextButton(600,30," CANCEL ",0,cmdWhichRRCANCEL));

	AddAccessory(new UpArrowButton(550,10,0,cmdScrollUp));
	AddAccessory(new DnArrowButton(550,30,0,cmdScrollDn));
	}


void WhichRR::Draw()
	{
	// draw the basic window
	grDrawWindow(r.left,r.top,r.right,r.bottom,bgcolor,active_bs);

	// draw static labels
//	grSetFont(TITLEFONT);
	switch (QUERY_MODE)
		{
		// PLAYER ACTIONS
		case cmdBuyStocks:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"Buy stocks in WhichRR?");
			break;

		case cmdSellStocks:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"Sell stocks in WhichRR?");
			break;

		case cmdBuyBonds:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"Buy bonds in which RR?");
			break;

		case cmdSellBonds:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"Buy bonds in which RR?");
			break;

		case cmdControlRR:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"Control Which RR?");
			break;

		case cmdResignControlRR:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"Resign control of which RR?");
			break;

		case cmdActForRR:

			grDraw2CString(r.left+10,r.top+10,TEXT_COLOR,
				"Act for which RR?");
			break;

		}

	if (SELECTED_RR != -1) grDraw2CString(r.left+10,r.top+30,TEXT_COLOR,"%s",RAILROAD[SELECTED_RR].Name);
	else	grDraw2CString(r.left+10,r.top+30,TEXT_COLOR,"No railroad selected");

	// as well as all accessories
	accessory[0]->Draw();
	accessory[1]->Draw();
	switch (QUERY_MODE)
		{
		case cmdActForRR:

			accessory[2]->Draw();
			accessory[3]->Draw();
			break;
		}
	}

void	WhichRR::Open()
	{
	if (!isOnScreen)
		{
		// open the howmany window
		OnScreenHandler::Open();

		switch (QUERY_MODE)
			{
			case cmdActForRR:

				accessory[2]->Activate();
				accessory[3]->Activate();
				break;

			default:
				accessory[2]->DeActivate();
				accessory[3]->DeActivate();
				break;
			}
		}
	}

void	WhichRR::Close()
	{
	if (isOnScreen)
		{
		// we're no longer onscreen
		isOnScreen = false;
		}
	}

void WhichRR::HandleEvent(Event& e)
	{
	// pre-process broadcasts so we can skip out quickly.....
	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdOpen:

					Open();
					EVENTMGR->ClearEvent(e);
					return;

				case cmdRRSelected:

					if (isOnScreen) Refresh();
					EVENTMGR->ModCommandEvent(e,evhno_WhichRR,cmdWhichRROK,NULL);
					break;
				}
			break;

		case ev_broadcast:

			switch (e.broadcast.event)
				{
				case cmdRRSelected:

					if (isOnScreen) Refresh();
					return;
				}
		}

	// let OnScreenHandler check all other events.....
	OnScreenHandler::HandleEvent(e);

	// and lets work with whats left
	switch (e.what)
		{
		case ev_mouse:

			switch (e.mouse.event)
				{
				// simply redirect the event as a command event
				case mb_leftup:

					EVENTMGR->ModCommandEvent(e,evhno_WhichRR,cmdWhichRROK,NULL);
					return;

				case mb_rightup:

					EVENTMGR->ModCommandEvent(e,evhno_WhichRR,cmdWhichRRCANCEL,NULL);
					return;
				}
			break;

		case ev_command:

			switch (e.command.event)
				{
				case cmdWhichRROK:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,e.command.event,(void*) SELECTED_RR);
					break;

				case cmdWhichRRCANCEL:

					EVENTMGR->ModCommandEvent(e,evhno_Interface,e.command.event,NULL);
					break;

				case cmdScrollUp:

					EVENTMGR->ClearEvent(e);
					break;

				case cmdScrollDn:

					EVENTMGR->ClearEvent(e);
					break;
				}
			break;
		}
	}


