//	-------------------------------------------------------------------------
//
//	QBUYBNDS.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the Buy Bonds Query code
//
//	-------------------------------------------------------------------------

#include "qbuybnds.hpp"
#include "eventmgr.hpp"
#include "fonts.hpp"
#include "guidefn.hpp"
#include "buttons.hpp"

// -----------------

#include "uidefn.hpp"
#include "gamedefn.hpp"
#include "player.hpp"
#include "railroad.hpp"
#include "bndsfold.hpp"

// -----------------

BuyBondsQuery*	BUYBONDS_QUERY;
int				evhno_BuyBondsQuery;

// -----------------

#define		cmdDone					50000
#define		cmdCancel				50001
#define		cmdIncrement			50002
#define		cmdDecrement			50003

// -----------------

BuyBondsQuery::BuyBondsQuery()
	: Query(&evhno_BuyBondsQuery,0,
			0,ac_PlayerQuery,"Buying Bonds",&evhno_BondsFolder)
	{
	BUYBONDS_QUERY = this;

	int width = r.right-r.left+1;
	int height = r.bottom-r.top+1;

	int x1 = width - 8 - 60;
	int x2 = x1 - 8 - 20;

	AddAccessory(new PictureButton(x2,28,0,
				GUI_BUTTONS[SMALL_SCROLL_UP_BUTTON],
				GUI_BUTTONS[SMALL_SCROLL_UP_BUTTON+1],ac_AutoMode,cmdIncrement));
	AddAccessory(new PictureButton(x2,72,0,
				GUI_BUTTONS[SMALL_SCROLL_DN_BUTTON],
				GUI_BUTTONS[SMALL_SCROLL_DN_BUTTON+1],ac_AutoMode,cmdDecrement));

	bonds = 0;
	isOnScreen = false;
	}

// -----------------

void	BuyBondsQuery::DrawContents()
	{
	Query::DrawContents();

	int x1 = r.left + 8;
	int x2 = r.right - 8 - 60 - 8;
	int y1 = r.top + 28;
	int y2 = r.bottom - 8;

	grSetFont(BASICFONT);
	grDraw2CString(x1,y1,grWHITE,"%s",RAILROAD[CURRENT_RR].Name);

	int bnds = PLAYER[CURRENT_PLAYER].Bonds[CURRENT_RR];
	int mill = bnds / 1000;
	int thou = bnds % 1000;

	if (mill) grDraw2CString(x1+300,y1,grWHITE,"%d,%.3d,000 owned",mill,thou);
	else      grDraw2CString(x1+300,y1,grWHITE,"%d,000 owned",thou);

	bnds = RAILROAD[CURRENT_RR].BondsOnMarket;
	mill = bnds / 1000;
	thou = bnds % 1000;

	if (mill) grDraw2CString(x1+300,y1+20,grWHITE,"%d,%.3d,000 avail",mill,thou);
	else      grDraw2CString(x1+300,y1+20,grWHITE,"%d,000 avail",thou);

	DrawInput();
	}

// -----------------

void	BuyBondsQuery::DrawInput()
	{
	int x1 = r.left + 8;
	int x2 = r.right - 8 - 60 - 8;
	int y1 = r.top + 28;
	int y2 = r.bottom - 8;

	grSetFont(BASICFONT);
	grDrawFilledRect(x1,y1+60,x2,y2,grRED);
	grDrawRect(x1,y1+60,x2,y2,grRED);

	grDraw2CString(x1+8,y1+60,grWHITE,"%d,000",bonds);
	grDraw2CString(x1+120,y1+60,grWHITE,"@ $%d",27);
	}

// -----------------

void	BuyBondsQuery::RefreshInput()
	{
	int x1 = r.left + 8;
	int x2 = r.right - 8 - 60 - 8;
	int y1 = r.top + 28;
	int y2 = r.bottom - 8;

	DrawInput();
	grBlitRegion(x1,y1+60,x2,y2);
	}

// -----------------

void 	BuyBondsQuery::HandleEvent(Event &e)
	{
	// handle buttons, etc.
	Query::HandleEvent(e);

	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdIncrement:

					bonds++;
					RefreshInput();

					EVENTMGR->ClearEvent(e);
					break;

				case cmdDecrement:

					bonds--;
					RefreshInput();

					EVENTMGR->ClearEvent(e);
					break;

				case cmdDone:

					Close();
					EVENTMGR->GenCommandEvent(*parent_handler,cmdCloseQuery,NULL);
					EVENTMGR->ClearEvent(e);
					break;


				case cmdCancel:

					Close();
					EVENTMGR->GenCommandEvent(*parent_handler,cmdCloseQuery,NULL);
					EVENTMGR->ClearEvent(e);
					break;
				}
			break;
		}
	}

