//	-------------------------------------------------------------------------
//
//	QUERY.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the basic Query code
//
//	-------------------------------------------------------------------------

#include "query.hpp"
#include "eventmgr.hpp"
#include "fonts.hpp"
#include "guidefn.hpp"
#include "buttons.hpp"
#include "borders.hpp"

// -----------------

#include "uidefn.hpp"
#include "gamedefn.hpp"
#include "player.hpp"
#include "railroad.hpp"

// -----------------

#define		cmdDone					50000
#define		cmdCancel				50001
#define		cmdIncrement			50002
#define		cmdDecrement			50003

// -----------------

extern	folder_title_bstyle;

Query::Query( int* evhno,int options,
				int level,int type,char* name,int* parent)
	: OnScreenHandler(query_r.left,query_r.top,query_r.right,query_r.bottom,
						evhno,options,6)
	{
	query_name 	= name;
	query_level	= level;
	query_type	= type;
	parent_handler = parent;

	grSetFont(BASICFONT);

	int name_width  = grFontWidth(query_name);
//	int	name_height = grFontHeight(query_name);

	name_xpos = r.right - 8 - 40 - 8 - 40 - 8 - name_width;
	name_ypos = r.top;

	int width = r.right-r.left+1;
	int height = r.bottom-r.top+1;

	int x1 = width - 8 - 60;

	AddAccessory(new PictureButton(x1,28,0,
				GUI_BUTTONS[DONE_BUTTON],
				GUI_BUTTONS[DONE_BUTTON+1],0,cmdDone));

	AddAccessory(new PictureButton(x1,72,0,
				GUI_BUTTONS[SELL_STOCKS_BUTTON],
				GUI_BUTTONS[SELL_STOCKS_BUTTON+1],0,cmdCancel));

	isOnScreen = false;
	}

// -----------------

void	Query::SetQueryColors()
	{
	int query_type = (selected_options & 0x000F0000) - ac_PlayerQuery;

	switch (query_type)
		{
		case 0:	// PLAYER FOLDERS

			query_title_bg 		= PLAYER_TITLE_BG;
			query_title_text 	= PLAYER_TITLE_TEXT;
			query_border 		= PLAYER_BORDER;
			query_bg			= PLAYER_BG;
			query_text			= PLAYER_TEXT;
			break;

		case 1:	// RAILROAD FOLDERS

			switch (RAILROAD[CURRENT_RR].Status)
				{
				case CONTROLLED:

					query_title_bg 		= CONTROLLED_TITLE_BG;
					query_title_text 	= CONTROLLED_TITLE_TEXT;
					query_border 		= CONTROLLED_BORDER;
					query_bg			= CONTROLLED_BG;
					query_text			= CONTROLLED_TEXT;
					break;

				case RECEIVERSHIP:

					query_title_bg 		= RECEIVERSHIP_TITLE_BG;
					query_title_text 	= RECEIVERSHIP_TITLE_TEXT;
					query_border 		= RECEIVERSHIP_BORDER;
					query_bg			= RECEIVERSHIP_BG;
					query_text			= RECEIVERSHIP_TEXT;
					break;

				case REORGANIZING:

					query_title_bg 		= REORGANIZING_TITLE_BG;
					query_title_text 	= REORGANIZING_TITLE_TEXT;
					query_border 		= REORGANIZING_BORDER;
					query_bg			= REORGANIZING_BG;
					query_text			= REORGANIZING_TEXT;
					break;

				default:

					query_title_bg 		= RAILROAD_TITLE_BG;
					query_title_text 	= RAILROAD_TITLE_TEXT;
					query_border 		= RAILROAD_BORDER;
					query_bg			= RAILROAD_BG;
					query_text			= RAILROAD_TEXT;
					break;

				}
			break;

		case 2:	// OTHER FOLDERS

			query_title_bg 		= OTHER_TITLE_BG;
			query_title_text 	= OTHER_TITLE_TEXT;
			query_border 		= OTHER_BORDER;
			query_bg			= OTHER_BG;
			query_text			= OTHER_TEXT;
			break;

		}
	}

// -----------------

void	Query::DrawTitleBar()
	{
	// draw the query title's insides

	grTileRect(r.left,r.top,r.right,r.top+19,-1,GUI_TILES[FOLDER_TITLE_TILE],NO_ROTATE);

//	grDrawFilledRect(r.left,r.top,r.right,r.top+19,query_title_bg);

	// and draw the borders

//	grDrawLine(r.left,r.top,r.left,r.top+19,query_border);
// 	grDrawLine(r.left,r.top+19,r.right,r.top+19,query_border);
//	grDrawLine(r.right,r.top,r.right,r.top+19,query_border);

	grDrawBorder(r.left,r.top,r.right,r.top+19,folder_title_bstyle);

	// draw the query's name

	grSetFont(BASICFONT);
	grDraw2CString(name_xpos,name_ypos,query_title_text,"%s",query_name);

	}

// -----------------

void	Query::DrawContents()
	{
	// draw whatever
	grDrawFilledRect(r.left,r.top+20,r.right,r.bottom,query_bg);

	// draw borders
	grDrawLine(r.left,r.top+19,r.left,r.bottom,query_border);
	grDrawLine(r.left,r.bottom,r.right,r.bottom,query_border);
	grDrawLine(r.right,r.top+19,r.right,r.bottom,query_border);
	}

// -----------------

void	Query::Draw()
	{
	if (!isOnScreen) return;

	SetQueryColors();

	DrawTitleBar();
	DrawContents();

	DrawAllAccessories();
	}


