//	-------------------------------------------------------------------------
//
//	ROUTINFO.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the Route Information Box code
//
//	-------------------------------------------------------------------------

#include "routinfo.hpp"
#include "gamedefn.hpp"
#include "uidefn.hpp"
#include "eventmgr.hpp"
#include "fonts.hpp"
#include "mapwin.hpp"
#include "timer.hpp"

// -----------------

#include "city.hpp"
#include "route.hpp"
#include "railroad.hpp"

// -----------------

#include "conio.h"
#include "stdio.h"

// -----------------

RouteInfo*		ROUTE_INFOBOX;
int				evhno_RouteInfo;
char*			terrain_string[4] = { "SEA","GRASS","FOREST","MOUNTAINS" };


// -----------------

RouteInfo::RouteInfo()
	: MapInfoBox(routeinfo_r.left,routeinfo_r.top,
				 routeinfo_r.right,routeinfo_r.bottom,
				 &evhno_RouteInfo,OTHER_INFO_BOX,"Route Information")
	{
	// setup bits for this object
	ROUTE_INFOBOX = this;
	}

// -----------------

void	RouteInfo::DrawContents()
	{
	if (!vb) return;

	// figure out our information display region
	int x1 = 0;
	int y1 = 0;
	int x2 = vb_maxx;
	int y2 = vb_maxy;

	int mill,thou,less;

	// figure out the closest city
	int x = route_pt->x;
	int y = route_pt->y;

	int citynum  = GetCityNumber(x,y);
	int routenum = GetRouteNumber(x,y);

	// display connected cities
	grSetFont(BASICFONT);
	int startcity = ROUTE[routenum].StartCity;
	int endcity   = ROUTE[routenum].EndCity;
	grDrawStringVirtual(vb,x1+10,y1+40,"%s to %s",CITY[startcity].Name,CITY[endcity].Name);

	// display length
	grDrawStringVirtual(vb,x1+10,y1+60,"Track length");
	thou = ROUTE[routenum].Length / 1000;
	less = ROUTE[routenum].Length % 1000;
	if (thou) grDrawStringVirtual(vb,x1+120,y1+60,": %d,%.3d Miles",thou,less);
	else      grDrawStringVirtual(vb,x1+120,y1+60,": %d Miles",less);

	// display ownership
	grDrawStringVirtual(vb,x1+10,y1+90,"Owner");
	if (ROUTE[routenum].Owner==-1)
		grDrawStringVirtual(vb,x1+120,y1+90,": None");
	else
		grDrawStringVirtual(vb,x1+120,y1+90,": %s",RAILROAD[ROUTE[routenum].Owner].Abbrev);

	// display route status
	grDrawStringVirtual(vb,x1+10,y1+110,"Status");
	switch (ROUTE[routenum].Status)
		{
		case SURVEYED:
			grDrawStringVirtual(vb,x1+120,y1+110,": Not Chartered");
			break;

		case CHARTERED:
			grDrawStringVirtual(vb,x1+120,y1+110,": Chartered");
			break;

		case PARTIALLY_BUILT:
			grDrawStringVirtual(vb,x1+120,y1+110,": Under Construction");
			grDrawString(x1+120,y1+125,"  (%d Years to Completion)",ROUTE[routenum].TurnsLeft);
			break;

		case FULLY_BUILT:
			grDrawStringVirtual(vb,x1+120,y1+110,": Completed");
			break;
		}

	// display revenue
	mill = ROUTE[routenum].TotalRevenueYTD / 1000;
	thou = ROUTE[routenum].TotalRevenueYTD % 1000;

	grDrawStringVirtual(vb,x1+10,y1+140,"Revenue YTD");

	if (mill) grDrawStringVirtual(vb,x1+120,y1+140,": $%d,%.3d,000",mill,thou);
	else	  grDrawStringVirtual(vb,x1+120,y1+140,": $%d,000",thou);

	// display land grants
	mill = ROUTE[routenum].LandGrants / 1000;
	thou = ROUTE[routenum].LandGrants % 1000;

	grDrawStringVirtual(vb,x1+10,y1+170,"Land Grants");
	if (mill) grDrawStringVirtual(vb,x1+120,y1+170,": $%d,%.3d,000",mill,thou);
	else	  grDrawStringVirtual(vb,x1+120,y1+170,": $%.3d,000",thou);
	}

// -----------------

void 	RouteInfo::HandleEvent(Event &e)
	{
	// handle buttons, etc.
	MapInfoBox::HandleEvent(e);

	// react to button responses only!!
	switch (e.what)
		{
		case ev_broadcast:

			switch (e.broadcast.event)
				{
				case cmdRouteSelected:

					route_pt = (grPoint*) e.command.data;

					if (isOnScreen) Refresh();
					else			Open();
					break;
				}
			break;
		}
	}


