//	-------------------------------------------------------------------------
//
//	RRSFOLD.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the RRs Folder code
//
//	-------------------------------------------------------------------------

#include "rrsfold.hpp"
#include "uidefn.hpp"
#include "gamedefn.hpp"
#include "guidefn.hpp"

// -----------------

#include "player.hpp"
#include "railroad.hpp"

// -----------------

#include "eventmgr.hpp"
#include "fonts.hpp"
#include "buttons.hpp"

// -----------------

RRsFolder*		RRS_FOLDER;
int				evhno_RRsFolder;

// -----------------

#define		cmdScrollUp				50000
#define		cmdScrollDn				50001
#define		cmdClickedRow			50002
#define		cmdClickedHighlight		50003

// -----------------

RRsFolder::RRsFolder()
	: Folder(&evhno_RRsFolder,"Presidents Report",
				ac_PlayerFolder|ac_HasBigMode,NULL)
	{
	int width = r.right-r.left+1;
	int height = r.bottom-r.top+1;

	int x1 = width - 8 - 60;
	int x2 = x1 - 8 - 60;

	AddAccessory(new PictureButton(x1,28,0,
				GUI_BUTTONS[RESIGN_BUTTON],
				GUI_BUTTONS[RESIGN_BUTTON+1],0,cmdResignControlRR));
	AddAccessory(new PictureButton(x1,72,0,
				GUI_BUTTONS[INCORP_BUTTON],
				GUI_BUTTONS[INCORP_BUTTON+1],0,cmdIncorporateRR));
	AddAccessory(new PictureButton(x2,28,0,
				GUI_BUTTONS[CONTROL_BUTTON],
				GUI_BUTTONS[CONTROL_BUTTON+1],0,cmdControlRR));
	AddAccessory(new PictureButton(x2,72,0,
				GUI_BUTTONS[MERGE_BUTTON],
				GUI_BUTTONS[MERGE_BUTTON+1],0,cmdMergeRRs));

	x2 = x2 - 8 - 20;

	AddAccessory(new PictureButton(x2,28,0,
				GUI_BUTTONS[MEDIUM_SCROLL_UP_BUTTON],
				GUI_BUTTONS[MEDIUM_SCROLL_UP_BUTTON+1],ac_AutoMode,cmdScrollUp));
	AddAccessory(new PictureButton(x2,72,0,
				GUI_BUTTONS[MEDIUM_SCROLL_DN_BUTTON],
				GUI_BUTTONS[MEDIUM_SCROLL_DN_BUTTON+1],ac_AutoMode,cmdScrollDn));

	// adjust pics in infobox button
//	PictureButton* infobox_button = (PictureButton*) accessory[box_button];
//	infobox_button->SetPics(GUI_BUTTONS[OPEN_BONDMARKET_BUTTON],
//							GUI_BUTTONS[CLOSE_BONDMARKET_BUTTON]);

	AddAccessory(new RRsScrollBox(8,28,x2-8,height-8,
						3,NUM_RRS,
						ac_EndScroll,
						cmdClickedRow,cmdClickedHighlight));

	big_mode = false;
	}

// -----------------

void	RRsFolder::SetBigMode()
	{
	RRsScrollBox* scroll_box = (RRsScrollBox*) accessory[7];
	scroll_box->r.top = map_r.top + 28 + 8; // +8 fixup
	scroll_box->SetRows(20);

	Folder::SetBigMode();
	}

// -----------------

void	RRsFolder::SetSmallMode()
	{
	RRsScrollBox* scroll_box = (RRsScrollBox*) accessory[7];
	scroll_box->r.top = folder_r.top + 28;
	scroll_box->SetRows(3);

	Folder::SetSmallMode();
	}

// -----------------

void 	RRsFolder::HandleEvent(Event &e)
	{
	// handle buttons, etc.
	Folder::HandleEvent(e);

	RRsScrollBox* scroll_box = (RRsScrollBox*) accessory[7];

	switch (e.what)
		{
		case ev_command:

			switch (e.command.event)
				{
				case cmdScrollUp:

					scroll_box->ScrollUp();
//					SELECTED_RR = scroll_box->GetCurrentRow();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdScrollDn:

					scroll_box->ScrollDn();
//					SELECTED_RR = scroll_box->GetCurrentRow();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdClickedHighlight:

//					SELECTED_RR = scroll_box->GetCurrentRow();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdClickedRow:

//					SELECTED_RR = scroll_box->GetCurrentRow();
					EVENTMGR->ClearEvent(e);
					break;

				case cmdControlRR:
				case cmdResignControlRR:
				case cmdMergeRRs:
				case cmdIncorporateRR:

//					OpenQuery(evhno_BuyBondsQuery);
					EVENTMGR->ClearEvent(e);
					break;
				}
			break;
		}
	}

// -----------------

extern 	int		folder_scrollbox_title_bstyle;
extern	int		folder_scrollbox_low_bstyle;
extern	int		folder_scrollbox_high_bstyle;

RRsScrollBox::RRsScrollBox(int x1,int y1,int x2,int y2,
						int rows,int maxrows,
						int options,
						int hit_row_event,int hit_hi_row_event)
	: ScrollBox(x1,y1,x2,y2,rows,maxrows,
					GUI_TILES[SCROLLBOX_TITLE_TILE], folder_scrollbox_title_bstyle,
					GUI_TILES[SCROLLBOX_LOW_TILE],   folder_scrollbox_low_bstyle,
					GUI_TILES[SCROLLBOX_HIGH_TILE],  folder_scrollbox_high_bstyle,
					options,hit_row_event,hit_hi_row_event)
	{
	title_bg	= grRED;
	title_text	= grWHITE;
	title_border= grRED;
	lo_bg		= grDARKGRAY;
	lo_text		= grLIGHTGRAY;
	lo_border	= grLIGHTGRAY;
	hi_bg		= grLIGHTGRAY;
	hi_text		= grWHITE;
	hi_border	= grDARKGRAY;
	}

// -----------------

void	RRsScrollBox::DrawRow(int row,bool auto_fill)
	{
	int rr = row+row_offset;

	// figure out what colors to use

	if (RAILROAD[rr].President==CURRENT_PLAYER)
		{
//	DBG(10,"IN RRsSCROLLBOX::DRAWROW(%d):   RR -> %d      IS PRESIDENT",row,rr);
		lo_bg = grBLUE;
		switch (RAILROAD[rr].Status)
			{
			case FLOATED: 			lo_text = grLIGHTGRAY; 	break;
			case CONTROLLED:        lo_text = grWHITE; 		break;
			case RECEIVERSHIP:      lo_text = grLIGHTRED; 	break;
			case REORGANIZING:      lo_text = grLIGHTGREEN; break;
			}
		}
	else
		{
		lo_bg = grDARKGRAY;
//	DBG(10,"IN RRsSCROLLBOX::DRAWROW(%d):   RR -> %d      ",row,rr);
		switch (RAILROAD[rr].Status)
			{
			case FLOATED: 			lo_text = grLIGHTGRAY; 	break;
			case CONTROLLED:        lo_text = grLIGHTGRAY; 	break;
			case RECEIVERSHIP:      lo_text = grRED; 		break;
			case REORGANIZING:      lo_text	= grGREEN; 		break;
			}
		}
	ScrollBox::DrawRow(row,auto_fill);
	DrawRowContents(row,false);
	}

// -----------------

void	RRsScrollBox::DrawHighlightedRow(int row,bool auto_fill)
	{
	int rr = row+row_offset;

	// figure out what colors to use

	if (RAILROAD[rr].President==CURRENT_PLAYER)
		{
//	DBG(20,"IN RRsSCROLLBOX::DRAWHIGHLIGHTEDROW(%d):   RR -> %d      IS PRESIDENT",row,rr);
		hi_bg = grLIGHTBLUE;
		switch (RAILROAD[rr].Status)
			{
			case FLOATED: 			hi_text = grLIGHTGRAY; 	break;
			case CONTROLLED:        hi_text = grWHITE; 		break;
			case RECEIVERSHIP:      hi_text = grLIGHTRED; 	break;
			case REORGANIZING:      hi_text = grLIGHTGREEN; break;
			default:				hi_text = grLIGHTGRAY;	break;
			}
		}
	else
		{
//	DBG(20,"IN RRsSCROLLBOX::DRAWHIGHLIGHTEDROW(%d):   RR -> %d",row,rr);
		hi_bg = grLIGHTGRAY;
		switch (RAILROAD[rr].Status)
			{
			case FLOATED: 			hi_text = grDARKGRAY; 	break;
			case CONTROLLED:        hi_text = grDARKGRAY; 	break;
			case RECEIVERSHIP:      hi_text = grRED; 		break;
			case REORGANIZING:      hi_text = grGREEN; 		break;
			default:				hi_text = grDARKGRAY;	break;
			}
		}
	ScrollBox::DrawHighlightedRow(row,auto_fill);
	DrawRowContents(row,true);
	}

// -----------------

void	RRsScrollBox::DrawTitle()
	{
	ScrollBox::DrawTitle();

	grSetFont(BASICFONT);
	grDraw2CString(r.left+10,r.top,title_text,"Railroad");
	grDraw2CString(r.left+250+10,r.top,title_text,"Stocks");
	grDraw2CString(r.left+350+10,r.top,title_text,"Holding");
	}

// -----------------

void	RRsScrollBox::DrawRowContents(int row,bool is_highlighted)
	{
	int y = r.top + 20 + 20*row;	// title + row
	int rr = row+row_offset;

	int color = lo_text;
	if (is_highlighted) color = hi_text;

	grSetFont(BASICFONT);
	grDraw2CString(r.left+10,y,color,"%s",RAILROAD[rr].Name);

	int stks = PLAYER[CURRENT_PLAYER].Stocks[rr];
	int mill = stks / 1000;
	int thou = stks % 1000;

	if (mill) grDraw2CString(r.left+250+10,y,color,"%d,%.3d,000",mill,thou);
	else      grDraw2CString(r.left+250+10,y,color,"%d,000",thou);

	int total_stocks = RAILROAD[rr].StocksOnMarket + RAILROAD[rr].StocksInPlayer;
	if (total_stocks!=0)
		{
		float percent = ((float)stks / (float)total_stocks) * 100.0;
		grDraw2CString(r.left+350+10,y,color,"%.2f%%",percent);
		}
	}

