//	-------------------------------------------------------------------------
//
//	STKSBOX.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the StockMarketBox code
//
//	-------------------------------------------------------------------------

#include "stksbox.hpp"
#include "gamedefn.hpp"
#include "uidefn.hpp"
#include "eventmgr.hpp"
#include "fonts.hpp"
#include "mapwin.hpp"
#include "timer.hpp"
#include "stksfold.hpp"

// -----------------

#include "railroad.hpp"
#include "route.hpp"

// -----------------

#include "conio.h"
#include "stdio.h"
#include "dos.h"

// -----------------

StockMarketBox*		STOCKMARKET_BOX;
int					evhno_StockMarketBox;


// -----------------

StockMarketBox::StockMarketBox()
	: FolderBox(stockmarketbox_r.left,stockmarketbox_r.top,
				 stockmarketbox_r.right,stockmarketbox_r.bottom,
				 &evhno_StockMarketBox,&evhno_StocksFolder,OTHER_INFO_BOX,"Stock Market")
	{
	// setup bits for this object
	STOCKMARKET_BOX = this;
	}


// -----------------

void	StockMarketBox::DrawContents()
	{
	if (!vb) return;

	int x1 = 20;
	int x2 = 20 + MAX_HISTORY * 10;
	int y1 = 30;
	int y2 = y1 + 180;

	grSetFont(BASICFONT);

	// outline
	grDrawLineVirtual(vb,x1+10,y1,x1+10,y2,grWHITE);
	grDrawLineVirtual(vb,x1,y2,x2,y2,grWHITE);	// incl. dash

	// draw horiz dashes
	grDrawLineVirtual(vb,x1,y1,x1+10,y1,grWHITE);
	grDrawLineVirtual(vb,x1,y1+20,x1+10,y1+20,grWHITE);
	grDrawLineVirtual(vb,x1,y1+40,x1+10,y1+40,grWHITE);
	grDrawLineVirtual(vb,x1,y1+60,x1+10,y1+60,grWHITE);
	grDrawLineVirtual(vb,x1,y1+80,x1+10,y1+80,grWHITE);
	grDrawLineVirtual(vb,x1,y1+100,x1+10,y1+100,grWHITE);
	grDrawLineVirtual(vb,x1,y1+120,x1+10,y1+120,grWHITE);
	grDrawLineVirtual(vb,x1,y1+140,x1+10,y1+140,grWHITE);
	grDrawLineVirtual(vb,x1,y1+160,x1+10,y1+160,grWHITE);
	grDrawLineVirtual(vb,x1,y1+180,x1+10,y1+180,grWHITE);

	if (SELECTED_RR!=-1)
		{
		int hist = RAILROAD[SELECTED_RR].CurrentHistory + 1;
		if (hist>=MAX_HISTORY) hist = 0;

		int lastx = x1 + 10;
		int lasty = y2 - RAILROAD[SELECTED_RR].StockPrice[hist] * 2;

		for (int i=1;i<MAX_HISTORY;i++)
			{
			hist++;
			if (hist>=MAX_HISTORY) hist = 0;

			int x = lastx + 10;
			int y = y2 - RAILROAD[SELECTED_RR].StockPrice[hist] * 2;

			grDrawLineVirtual(vb,lastx,lasty,x,y,grLIGHTRED);

			lastx = x;
			lasty = y;
			}

		grDraw2CStringVirtual(vb,x1+30,y1,grWHITE,"%s",RAILROAD[SELECTED_RR].Name);
		grDraw2CStringVirtual(vb,x1,y2+20,box_text,"Stock Supply:  %d,000",RAILROAD[SELECTED_RR].StocksOnMarket);
		grDraw2CStringVirtual(vb,x1,y2+40,box_text,"Stock Demand:  %d,000",RAILROAD[SELECTED_RR].StocksInPlayer);
		}
	}

// -----------------

void 	StockMarketBox::HandleEvent(Event &e)
	{
	// handle buttons, etc.
	FolderBox::HandleEvent(e);

	// react to button responses only!!
	switch (e.what)
		{
		case ev_broadcast:

			switch (e.broadcast.event)
				{
				case cmdDayTick:

					Refresh();
					break;

				case cmdRouteSelected:

					if (isOnScreen)
						{
//						DBG(400,"STOCKMARKET RESPONDING TO cmdRouteSelected");
						grPoint* p = (grPoint*) e.command.data;
						int route = GetRouteNumber(p->x,p->y);
//						DBG(410,"ROUTE CLICKED == %d    ",route);
						SELECTED_RR = ROUTE[route].Owner;
//						DBG(420,"ROUTE OWNER == %d",SELECTED_RR);
						Refresh();
						}
					break;
				}
			break;
		}
	}


