//	-------------------------------------------------------------------------
//
//	TERRINFO.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This is the Terrain Information Box code
//
//	-------------------------------------------------------------------------

#include "terrinfo.hpp"
#include "gamedefn.hpp"
#include "uidefn.hpp"
#include "eventmgr.hpp"
#include "fonts.hpp"
#include "mapwin.hpp"
#include "timer.hpp"

// -----------------

#include "conio.h"
#include "stdio.h"

// -----------------

TerrainInfo*		TERRAIN_INFOBOX;
int					evhno_TerrainInfo;
char*				terrain_string[4] = { "SEA","GRASS","FOREST","MOUNTAINS" };


// -----------------

TerrainInfo::TerrainInfo()
	: MapInfoBox(terrinfo_r.left,terrinfo_r.top,
				 terrinfo_r.right,terrinfo_r.bottom,
				 &evhno_TerrainInfo,OTHER_INFO_BOX,"Terrain Information")
	{
	// setup bits for this object
	TERRAIN_INFOBOX = this;

	scroll_ticker = -1;
	}

// -----------------

void	TerrainInfo::DrawContents()
	{
	if (!vb) return;

	// figure out our information display region
	int x1 = 0;
	int y1 = 0;
	int x2 = vb_maxx;
	int y2 = vb_maxy;

	int x = terrain_pt->x;
	int y = terrain_pt->y;

	int ter = GetTerrain(x,y);

	grImage* ter_pic = terrain_bitmaps[MAX_ZOOM_LEVEL][ter];

	// display terrain bitmap
	grOverlayImageVirtual(vb,x1+20,y1+40,x1+20+ter_pic->hdr.width-1,y1+40+ter_pic->hdr.height-1,grBLACK,ter_pic);

	// display terrain string
	grSetFont(BASICFONT);
	grDrawStringVirtual(vb,x1+70,y1+50,"%s",terrain_string[ter]);

	// display cost to build
	grDrawStringVirtual(vb,x1+10,y1+80,"Cost to build");
	grDrawStringVirtual(vb,x1+120,y1+80,": %d,000 per Mile",25);

	// display terrain height
	grDrawStringVirtual(vb,x1+10,y1+100,"Approx. height");
	grDrawStringVirtual(vb,x1+120,y1+100,": %d,000 Feet",GetTerrainHeight(x,y) * 5);
	}

// -----------------

void	TerrainInfo::StartRefresh()
	{
	if (scroll_ticker==-1)
		{
		scroll_ticker = timer_AddTicker(5,cmdRefreshTerrainInfo);
		width = (r.right-r.left+1);
		dx = 0;
		ScrollingOut = true;
		}
	}

// -----------------

void	TerrainInfo::ScrollOut()
	{
	if (dx<width)
		{
		dx++;
		vb = grCreateVirtual(r.left,r.top,r.right,r.bottom,grBLACK);
		DrawContents();
		grOverlayImage(dx-width,r.top,dx,r.bottom,grBLACK,vb);
		grBlitRegion(dx-width,r.top,dx,r.bottom);
		grDestroyVirtual(vb);
		}
	else
		ScrollingOut = false;
	}

// -----------------

void	TerrainInfo::ScrollIn()
	{
	if (dx>0)
		{
		dx--;
		vb = grCreateVirtual(r.left,r.top,r.right,r.bottom,grBLACK);
		DrawContents();
		grOverlayImage(dx-width,r.top,dx,r.bottom,grBLACK,vb);
		grBlitRegion(dx-width,r.top,dx,r.bottom);
		grDestroyVirtual(vb);
		}
	else
		{
		timer_DelTicker(scroll_ticker);
		scroll_ticker = -1;
		}
	}

// -----------------

void 	TerrainInfo::HandleEvent(Event &e)
	{
	// handle buttons, etc.
	MapInfoBox::HandleEvent(e);

	// react to button responses only!!
	switch (e.what)
		{
		case ev_timer:
			switch (e.timer.event)
				{
				case cmdRefreshTerrainInfo:

//					if (ScrollingOut) ScrollOut();
//					else ScrollIn();
					if (isOnScreen) Refresh();
					else Open();
					break;
				}
			break;

		case ev_broadcast:

			switch (e.broadcast.event)
				{
				case cmdTerrainSelected:

					terrain_pt = (grPoint*) e.command.data;

					if (isOnScreen) Refresh();
					else			StartRefresh();
					break;
				}
			break;
		}
	}


