//	-------------------------------------------------------------------------
//
//	VIEW.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	this file contains the definition for the standard
//					'on screen' event handler
//
//	-------------------------------------------------------------------------

#ifndef __view_hpp
#define __view_hpp

//#include "event.hpp"
#include "handler.hpp"
#include "accessry.hpp"

// options for Select() & responses from SelectNextAccessory()/SelectPrev()

#define	evh_ANY_ACCESSORY			0
#define evh_FIRST_ACCESSORY			1
#define evh_LAST_ACCESSORY          2
#define	evh_CURRENT_ACCESSORY		3

// view handler options		  - 0x0000XX00
#define	evh_CLOSES					0x00000100
#define evh_MOVES					0x00000200
#define evh_RESIZES					0x00000400
#define	evh_MINIMIZES				0x00000800
#define evh_MAXIMIZES				0x00001000

//	-------------------------------------------------------------------------
//
//	CLASS:	View
//
//	DESC:   This is the basic 'on-screen window' used throughout any program
//			By default, it is assumed to interact with the user through
//			mouse, keyboard & (later) joystick....
//
//	-------------------------------------------------------------------------

class View : public EventHandler
	{
	protected:
		bool	AutoResetCursor;	// true-> handler auto. resets cursor on mb_move commands
									// false-> assumes cursor reset manually

		// windows can contain accessories, so store them here

		Accessory* 	accessory[MAX_ACCESSORIES_IN_HANDLER];

	public:

		// --- viewhandler functions

		View(int x1,int y1,int x2,int y2,	// the coords
						int* evhno,						// the event handler no.
						int options,                    // the window options
						int zfact);						// the z-factor

		virtual ~View();

		// --- 'global' variables

		grRect	r;					// rectangular region taken up by handler
		grRect	inset_r;			// rectangular region inside borders, etc.

		int		accessorySelected;	// currently selected accessory
		int		numAccessories;		// accessories created so far

		bool	accessoriesAtMax;	// true-> no more accessories can be added
									// false-> can add more accessories

		bool	isOnScreen;			// true-> handler is open, displayed & can interact
									// false-> handler is hidden

		bool	isSelected;			// true-> handler is currently selected
									// false-> not selected

		// --- accessory manipulation functions

		virtual	int		AddAccessory(Accessory* accessry);
		virtual	void 	DelAccessory();
		virtual	void	DelAllAccessories();

		virtual	int		WhichAccessorySelected(grPoint &mouse_pos);

		virtual	void	ActivateAccessory(int acc);
		virtual	void	DeActivateAccessory(int acc);

		virtual	void	DrawAccessory(int acc);
		virtual	void	DrawAccessory(grVirtualBuffer* vb,int acc);

		virtual	void	ActivateAllAccessories();
		virtual	void	DeActivateAllAccessories();

		virtual	void 	DrawAllAccessories();
		virtual	void	DrawAllAccessories(grVirtualBuffer* vb);

		virtual	void	SelectNextAccessory();
		virtual	void	SelectPrevAccessory();

		virtual	bool	AtFirstAccessory();
		virtual	bool	AtLastAccessory();

		// --- viewhandler functions

		virtual bool 	MouseInside(grPoint& p);

		virtual	void	OnAnimate(Event &e);
		virtual	void	OnRefresh(Event &e);
		virtual	void	OnRedraw(Event &e);

		virtual	void	OnOpen(Event &e);
		virtual	void	OnClose(Event &e);

		virtual void 	Select(int which);
		virtual void 	DeSelect();

		virtual void 	Draw();
		virtual	void 	Refresh();

		virtual void 	Animate();
		virtual	void 	Open();
		virtual	void	Close();

		virtual void 	HandleEvent(Event &e);
	};

#endif

