//
//	NEO2PAL.CPP
//
//	Converts from NEOPAINT palette file format to that required by the GDE
//
//	Written by DJD.

#include "graphics.hpp"
#include "fonts.hpp"
#include "strings.hpp"
#include "textfile.hpp"
#include "palettes.hpp"
#include "stdio.h"
#include "conio.h"

String*	  	srce_filename;
String*		dest_filename;

TextFile* 	srce_file;
File* 	  	dest_file;

void ShowHeader()
	{
	printf("\n\nNEO2PAL v1.0    (c) 1995 Darren J. Dwyer\n\n");
	}

void ShowUsage()
	{
	printf("This program converts a 256 color palette from the NEOPAINT format\n");
	printf("to the packed RGB format used by the GDE.\n");
	printf("\n");
	printf("Usage:      NEO2PAL srce.pal dest.pal\n");
	printf("\n");
	printf("            srce.pal  is the std NEOPAINT palette file\n");
	printf("            dest.pal  is the raw RGB file to be created\n");
	printf("\n");

	exit(1);
	}

void DrawPaletteEntry(int pal)
	{
	int x = 16 + (pal%16)*16;
	int y = 16 + (pal/16)*16;
	grDrawFilledRect(x,y,x+16,y+16,pal);
	grDrawRect(x-1,y-1,x+16+1,y+16+1,grLIGHTRED);
	grBlitRegion(x-1,y-1,x+16+1,y+16+1);
	}

void DrawPalette()
	{
	int x,y;
	for (int pal=0;pal<256;pal++)
		{
		x = 16 + (pal%16)*16;
		y = 16 + (pal/16)*16;
		grDrawRect(x-1,y-1,x+16+1,y+16+1,grLIGHTRED);
		grDrawFilledRect(x,y,x+16,y+16,pal);
		}

	grBlitRegion(16,16,16+256,16+256);
	}

void main(int argc,char** argv)
	{
	ShowHeader();
	if (argc!=3) ShowUsage();

	// save our command-line params
	srce_filename = new String(*++argv);
	dest_filename = new String(*++argv);

	srce_file = new TextFile(srce_filename);
	dest_file = new File(dest_filename);

	if (!srce_file->Exists())
		{
		printf("ERROR: Source file >%s< doesn't exist!\n",srce_filename->Contents());
		delete dest_file;
		delete srce_file;
		delete dest_filename;
		delete srce_filename;
		exit(1);
		}
	if (dest_file->Exists())
		{
		dest_file->Close();
		dest_file->Delete();
		}

	// create our new palette file
	dest_file->Close();
	dest_file->Create();
	dest_file->Open();

	// initialize graphics, clear to black & save original palette
	grInitVesa(gr640x480x256);
	// install system font
	int SYSTEMFONT = grInitFont("\\GAME\\RESOURCE\\SYSTEM.FNT");

	if (SYSTEMFONT<0)
		{
		grExitVesa();
		printf("ERROR: Could not initialise \\GAME\\RESOURCE\\SYSTEM.FNT\n");
		exit(1);
		}

	grSetSystemFont(SYSTEMFONT);

	grPalette* orig_pal = grCreatePalette(256);
	grGetPalette(orig_pal,0,256);

	// display working message
	grDrawString(0,0,"Converting %s to %s",srce_filename->Contents(),dest_filename->Contents());
	grBlitRegion(0,0,grMaxx,10);

	DrawPalette();

	// start translating file formats...
	int		 palsize;

	// skip 2-row header
	String*  linebuf = new String(256);
	srce_file->ReadLine(linebuf->Contents());
	srce_file->ReadLine(linebuf->Contents());
	delete linebuf;

	// read size of palette
	linebuf = srce_file->ReadLine();

	palsize = linebuf->StringToInt();
	if (palsize>256) palsize = 256;
	delete linebuf;

	// create our destination palette structure
	grPalette* workpal = grCreatePalette(palsize);

	// get each row & set the palette accordingly
	int dest_index = 0;
	for (int pal=0;pal<palsize;pal++)
		{
		linebuf = srce_file->ReadLine();

		String* rs = linebuf->Elem(0,' ');
		String* gs = linebuf->Elem(1,' ');
		String* bs = linebuf->Elem(2,' ');

		int r = workpal->data[dest_index++] = rs->StringToInt();
		int g = workpal->data[dest_index++] = gs->StringToInt();
		int b = workpal->data[dest_index++] = bs->StringToInt();

//		grDrawFilledRect(300,20,350,65,grBLACK);
//		grDrawString(300,20,"R: %d",r);
//		grDrawString(300,35,"G: %d",g);
//		grDrawString(300,50,"B: %d",b);
//		grBlitRegion(300,20,350,65);

		grSetPaletteEntry(pal,r,g,b);

		DrawPaletteEntry(pal);

		delete bs;
		delete gs;
		delete rs;

		delete linebuf;
		}

	// write the data to the new file
	int err = dest_file->Write((char*) &workpal->data[0],palsize*3);
	if (err != file_NoError) DBG(470,"WRITE ERROR %d",err);

	// and clean up everything
	grDestroyPalette(workpal);

	grDrawString(0,340,"Finished! Press any key to exit.");
	grBlitRegion(0,340,grMaxx,350);

	getch();

	// restore original palette & terminate graphics system...
	grSetPalette(orig_pal,0,256);

	grExitFont();
	grExitVesa();

	delete dest_file;
	delete srce_file;

	delete dest_filename;
	delete srce_filename;

	printf("\nFinished!\n");
	}