//
//	PAL2NEO.CPP
//
//	Converts from the GDE palette file format to that required by NEOPAINT
//
//	Written by DJD.

#include "graphics.hpp"
#include "fonts.hpp"
#include "strings.hpp"
#include "textfile.hpp"
#include "palettes.hpp"
#include "stdio.h"
#include "conio.h"

String*	  	srce_filename;
String*		dest_filename;

File* 	  	srce_file;
TextFile* 	dest_file;

void ShowHeader()
	{
	printf("\n\nPAL2NEO v1.0    (c) 1995 Darren J. Dwyer\n\n");
	}

void ShowUsage()
	{
	printf("This program converts a 256 color palette from the packed RGB format\n");
	printf("used by the GDE to the format used by NEOPAINT.\n");
	printf("\n");
	printf("Usage:      PAL2NEO srce.pal dest.pal\n");
	printf("\n");
	printf("            srce.pal  is the raw RGB palette file\n");
	printf("            dest.pal  is the std NEOPAINT file to be created\n");
	printf("\n");

	exit(1);
	}

void DrawPaletteEntry(int pal)
	{
	int x = 16 + (pal%16)*16;
	int y = 16 + (pal/16)*16;
	grDrawFilledRect(x,y,x+16,y+16,pal);
	grDrawRect(x-1,y-1,x+16+1,y+16+1,grLIGHTRED);
	grBlitRegion(x-1,y-1,x+16+1,y+16+1);
	}

void DrawPalette()
	{
	int x,y;
	for (int pal=0;pal<256;pal++)
		{
		x = 16 + (pal%16)*16;
		y = 16 + (pal/16)*16;
		grDrawRect(x-1,y-1,x+16+1,y+16+1,grLIGHTRED);
		grDrawFilledRect(x,y,x+16,y+16,pal);
		}

	grBlitRegion(16,16,16+256,16+256);
	}

void main(int argc,char** argv)
	{
	ShowHeader();
	if (argc!=3) ShowUsage();

	// save our command-line params
	srce_filename = new String(*++argv);
	dest_filename = new String(*++argv);

	srce_file = new File(srce_filename);
	dest_file = new TextFile(dest_filename);

	if (!srce_file->Exists())
		{
		printf("ERROR: Source file >%s< doesn't exist!\n",srce_filename->Contents());
		delete dest_file;
		delete srce_file;
		delete dest_filename;
		delete srce_filename;
		exit(1);
		}

	srce_file->Open();

	if (dest_file->Exists())
		{
		dest_file->Close();
		dest_file->Delete();
		}

	// create our new palette file
	dest_file->Close();
	dest_file->Create();
	dest_file->Open();

	// initialize graphics, clear to black & save original palette
	grInitVesa(gr640x480x256);
	// install system font
	int SYSTEMFONT = grInitFont("\\GAME\\RESOURCE\\SYSTEM.FNT");

	if (SYSTEMFONT<0)
		{
		grExitVesa();
		printf("ERROR: Could not initialise \\GAME\\RESOURCE\\SYSTEM.FNT\n");
		exit(1);
		}

	grSetSystemFont(SYSTEMFONT);

	grPalette* orig_pal = grCreatePalette(256);
	grGetPalette(orig_pal,0,256);

	// display working message
	grDrawString(0,0,"Converting %s to %s",srce_filename->Contents(),dest_filename->Contents());
	grBlitRegion(0,0,grMaxx,10);

	DrawPalette();

	// start translating file formats...

	// create our source palette structure
	int palsize = 256;
	grPalette* workpal = grCreatePalette(palsize);

	// and read it from the disk
	int err = srce_file->Read((char*) &workpal->data[0],palsize*3);
	if (err != file_NoError) DBG(470,"ERROR READING PALETTE DATA %d",err);

	// write 2-row header
	String* linebuf0 = new String("NeoPaint Palette File");
	String* linebuf1 = new String("(C)1992 NeoSoft Corp.");
	dest_file->SeekTOF();
	dest_file->WriteLine(linebuf0->Contents());
	dest_file->WriteLine(linebuf1->Contents());
	delete linebuf1;
	delete linebuf0;

	// write size of palette
	String* linebuf = new String("256");
	dest_file->WriteLine(linebuf->Contents());
	delete linebuf;

	// get each row & set the palette accordingly
	int srce_index = 0;
	linebuf = new String(20);
	for (int pal=0;pal<palsize;pal++)
		{
		long r = workpal->data[srce_index++];
		long g = workpal->data[srce_index++];
		long b = workpal->data[srce_index++];

		grSetPaletteEntry(pal,r,g,b);
		DrawPaletteEntry(pal);

		linebuf->Set(r);
		linebuf->Append(' ');
		linebuf->Append(g);
		linebuf->Append(' ');
		linebuf->Append(b);

		dest_file->WriteLine(linebuf->Contents());
		}
	delete linebuf;

	// and clean up everything
	grDestroyPalette(workpal);

	grDrawString(0,340,"Finished! Press any key to exit.");
	grBlitRegion(0,340,grMaxx,350);

	getch();

	// restore original palette & terminate graphics system...
	grSetPalette(orig_pal,0,256);

	grExitFont();
	grExitVesa();

	delete dest_file;
	delete srce_file;

	delete dest_filename;
	delete srce_filename;

	printf("\nFinished!\n");
	}