//	-------------------------------------------------------------------------
//
//	PAL.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew Carvel Evans
//
//	DESCRIPTION:	This is the Palette Editting Window
//
//	-------------------------------------------------------------------------

#include "pal.hpp"
#include "tooldefn.hpp"

#include "palfile.hpp"
#include "palettes.hpp"
#include "grid.hpp"

#include "graphics.hpp"
#include "fonts.hpp"
#include "eventmgr.hpp"
#include "timer.hpp"
#include "keydefns.hpp"
#include "mouse.hpp"
#include "buttons.hpp"

#include "dos.h"

#define		step_PREC				1000
#define		SIZE_OF_RANGEBAR		600

PaletteWindow::PaletteWindow(String* name)
	: OnScreenHandler(0,0,grMaxx,grMaxy,grBACKGROUND,bs_NONE,bs_NONE,
					&evhno_PaletteWindow,0,1)
	{
	// load in the GUI pics
	palette_GUI_file = new ResourceFile("PICS\\PALGUI.RES");

	for (int i=0;i<palette_GUI_file->NumRecords();i++)
		palette_BITMAP[i] = (grImage*) palette_GUI_file->ReadRecord(i);

	// load in the palette to edit
	palette_name = new String(name->Contents());
	palette_file = new PALFile(palette_name);

	if (!palette_file->Exists())
		{
		palette = grCreatePalette(256);
		grGetPalette(palette,0,255);
		}
	else
		palette = palette_file->LoadPalette();

	grSetPalette(palette,0,255);
	grGetPalette(grPALETTE,0,255);

	// create all the accessories
	CreateGrid();
	CreateRangeBar();
	CreateArrows();
	CreateBaseSelect();
	CreateButtons();

	// make sure the whole things is onscreen
	isOnScreen = true;
	}

PaletteWindow::~PaletteWindow()
	{
	delete base_grid;
	delete range_grid;
	delete main_grid;

	grDestroyPalette(palette);
	delete palette_file;
	delete palette_name;

	for (int i=palette_GUI_file->NumRecords()-1;i>=0;i--)
		free(palette_BITMAP[i]);
	delete palette_GUI_file;
	}

void PaletteWindow::LoadPalette()
	{
	grPalette* pal = palette_file->LoadPalette();

	grSetPalette(pal,0,255);

	grDestroyPalette(pal);

	grGetPalette(grPALETTE,0,255);
	}

void PaletteWindow::SavePalette()
	{
	// if file doesn't exist create it
	if (!palette_file->Exists())
		{
		DBG(460,"Create = %d",palette_file->Create());
		DBG(470,"Open = %d",palette_file->Open());
		}

	int err;
	if ((err = palette_file->SavePalette(grPALETTE)) < 0)
		DBG(460,"SavePalette didn't work #%d",err);
	}

void PaletteWindow::CreateGrid()
	{
	// Select Edit Range Grid
	grPoint	p = { gap, gap },
			n = { 16, 16 },
			s = { 16, 16 };

	main_grid = new Grid(p,n,s,1,grWHITE,1,grWHITE,grBACKGROUND);

	// selected cells

	grid_start_cell.x = 1;
	grid_start_cell.y = 1;

	grid_end_cell.x = 16;
	grid_end_cell.y = 16;
	}

void PaletteWindow::CreateRangeBar()
	{
	// indexes

	start_i = main_grid->CellToIndex(grid_start_cell);
	end_i = main_grid->CellToIndex(grid_end_cell);

	// difference between indexes

	number_i = end_i - start_i + 1;

	// Create Edit Range Bar as a horizontal 1 by n grid

	grPoint	p = { gap, (main_grid->BottomRight()).y+gap },
			n = { number_i, 1 },
			s = { SIZE_OF_RANGEBAR / number_i, 30 };

	range_grid = new Grid(p,n,s,1,grWHITE,0,grWHITE,grBACKGROUND);
	}

void PaletteWindow::CreateArrows()
	{
	// Arrow's sizes
	arrow.x = 20;
	arrow.y = 20;

	// x positions for left buttons and right buttons
	leftend_x = (main_grid->BottomRight()).x + gap;
	leftfast_x = leftend_x + arrow.x;
	left_x = leftfast_x + arrow.x;

	middle_x = left_x + arrow.x;

	right_x = middle_x + gap;
	rightfast_x = right_x + arrow.x;
	rightend_x = rightfast_x + arrow.x;

	// Top Arrows
	red1_y = gap;
	green1_y = red1_y + arrow.y + gap;
	blue1_y = green1_y + arrow.y + gap;

	// Bottom Arrows

	red2_y = (main_grid->BottomRight()).y - 3*arrow.y - 3*gap;
	green2_y = red2_y + arrow.y + gap;
	blue2_y = green2_y + arrow.y + gap;

	// arrow button accessory creation

	AddAccessory(new DblLtArrowButton(leftend_x,red1_y,0,cmd_r1_lll));
	AddAccessory(new DblLtArrowButton(leftfast_x,red1_y,0,cmd_r1_ll));
	AddAccessory(new LtArrowButton(left_x,red1_y,0,cmd_r1_l));
	AddAccessory(new RtArrowButton(right_x,red1_y,0,cmd_r1_r));
	AddAccessory(new DblRtArrowButton(rightfast_x,red1_y,0,cmd_r1_rr));
	AddAccessory(new DblRtArrowButton(rightend_x,red1_y,0,cmd_r1_rrr));

	AddAccessory(new DblLtArrowButton(leftend_x,green1_y,0,cmd_g1_lll));
	AddAccessory(new DblLtArrowButton(leftfast_x,green1_y,0,cmd_g1_ll));
	AddAccessory(new LtArrowButton(left_x,green1_y,0,cmd_g1_l));
	AddAccessory(new RtArrowButton(right_x,green1_y,0,cmd_g1_r));
	AddAccessory(new DblRtArrowButton(rightfast_x,green1_y,0,cmd_g1_rr));
	AddAccessory(new DblRtArrowButton(rightend_x,green1_y,0,cmd_g1_rrr));

	AddAccessory(new DblLtArrowButton(leftend_x,blue1_y,0,cmd_b1_lll));
	AddAccessory(new DblLtArrowButton(leftfast_x,blue1_y,0,cmd_b1_ll));
	AddAccessory(new LtArrowButton(left_x,blue1_y,0,cmd_b1_l));
	AddAccessory(new RtArrowButton(right_x,blue1_y,0,cmd_b1_r));
	AddAccessory(new DblRtArrowButton(rightfast_x,blue1_y,0,cmd_b1_rr));
	AddAccessory(new DblRtArrowButton(rightend_x,blue1_y,0,cmd_b1_rrr));

	AddAccessory(new DblLtArrowButton(leftend_x,red2_y,0,cmd_r2_lll));
	AddAccessory(new DblLtArrowButton(leftfast_x,red2_y,0,cmd_r2_ll));
	AddAccessory(new LtArrowButton(left_x,red2_y,0,cmd_r2_l));
	AddAccessory(new RtArrowButton(right_x,red2_y,0,cmd_r2_r));
	AddAccessory(new DblRtArrowButton(rightfast_x,red2_y,0,cmd_r2_rr));
	AddAccessory(new DblRtArrowButton(rightend_x,red2_y,0,cmd_r2_rrr));

	AddAccessory(new DblLtArrowButton(leftend_x,green2_y,0,cmd_g2_lll));
	AddAccessory(new DblLtArrowButton(leftfast_x,green2_y,0,cmd_g2_ll));
	AddAccessory(new LtArrowButton(left_x,green2_y,0,cmd_g2_l));
	AddAccessory(new RtArrowButton(right_x,green2_y,0,cmd_g2_r));
	AddAccessory(new DblRtArrowButton(rightfast_x,green2_y,0,cmd_g2_rr));
	AddAccessory(new DblRtArrowButton(rightend_x,green2_y,0,cmd_g2_rrr));

	AddAccessory(new DblLtArrowButton(leftend_x,blue2_y,0,cmd_b2_lll));
	AddAccessory(new DblLtArrowButton(leftfast_x,blue2_y,0,cmd_b2_ll));
	AddAccessory(new LtArrowButton(left_x,blue2_y,0,cmd_b2_l));
	AddAccessory(new RtArrowButton(right_x,blue2_y,0,cmd_b2_r));
	AddAccessory(new DblRtArrowButton(rightfast_x,blue2_y,0,cmd_b2_rr));
	AddAccessory(new DblRtArrowButton(rightend_x,blue2_y,0,cmd_b2_rrr));
	}

void PaletteWindow::AssignBaseColour(int i,char* n,uchar r,uchar g,uchar b)
	{
	memcpy(base[i].name,n,SIZE_OF_NAME);
	base[i].red = r;
	base[i].green = g;
	base[i].blue = b;
	}

void PaletteWindow::CreateBaseSelect()
	{
	// Base Select Squares

	grPoint	p = { gap, (range_grid->BottomRight()).y + gap },
			n = { 11,4 },
			s = { 55,22 };

	base_grid = new Grid(p,n,s,2,grWHITE,1,grWHITE,grDARKGRAY);

	// fill up the list of base colours with random entries
	for(int base_i=0; base_i<NUMBER_OF_BASECOLOURS; base_i++)
		{
		int		red = 	random(64),
				green =	random(64),
				blue =	random(64);

		char	name[SIZE_OF_NAME] = "------";

		// and try to apply appropriate names to them
		if ((red>green) && (red>blue)) strcpy(name,"Red--?");
		if ((green>blue) && (green>red)) strcpy(name,"Gre--?");
		if ((blue>red) && (blue>green)) strcpy(name,"Blu--?");

		AssignBaseColour(base_i,name,red,green,blue);
		}

	AssignBaseColour(0,"Black",0,0,0);
	AssignBaseColour(1,"DkGray",15,15,15);
	AssignBaseColour(2,"Gray",31,31,31);
	AssignBaseColour(3,"LtGray",47,47,47);
	AssignBaseColour(4,"White",63,63,63);
	AssignBaseColour(5,"Red",63,0,0);
	AssignBaseColour(6,"DkRed",31,0,0);
	AssignBaseColour(7,"Green",0,63,0);
	AssignBaseColour(8,"DkGree",0,31,0);
	AssignBaseColour(9,"Blue",0,0,63);
	AssignBaseColour(10,"DkBlue",0,0,31);
	AssignBaseColour(11,"Yellow",63,63,0);
	AssignBaseColour(12,"DkYell",31,31,0);
	AssignBaseColour(13,"Cyan",0,63,63);
	AssignBaseColour(14,"DkCyan",0,31,31);
	AssignBaseColour(15,"Magent",63,0,63);
	AssignBaseColour(16,"DkMage",31,0,31);
	AssignBaseColour(17,"DkBrow",20,10,0);
	AssignBaseColour(18,"Brown",40,30,20);
	AssignBaseColour(19,"LtBrow",60,50,40);
	AssignBaseColour(20,"Orange",63,31,0);
	AssignBaseColour(21,"Pink",60,35,35);
	AssignBaseColour(22,"Olive",30,40,25);
	AssignBaseColour(23,"Tree",10,40,15);
	AssignBaseColour(24,"Metal",15,35,45);
	AssignBaseColour(25,"Egg",45,60,50);
	}

void PaletteWindow::CreateButtons()
	{
	int space = gap + grFontHeight();

	// Button accessory creation
	AddAccessory(new PictureButton(grMaxx-6*gap,gap,
					palette_BITMAP[0],palette_BITMAP[1],255,0,cmdLoad));

	AddAccessory(new PictureButton(grMaxx-6*gap,range_grid->TopLeft().y - 6*gap,
					palette_BITMAP[2],palette_BITMAP[3],255,0,cmdSave));

	AddAccessory(new PictureButton(grMaxx-6*gap,range_grid->TopLeft().y - 3*gap,
					palette_BITMAP[4],palette_BITMAP[5],255,0,cmdExitProgram));

//	AddAccessory(new TextButton(grMaxx-3*gap,gap,"Load",0,cmdLoad));
//	AddAccessory(new TextButton(grMaxx-3*gap,range_grid->TopLeft().y - 2*space,"Save",0,cmdSave));
//	AddAccessory(new TextButton(grMaxx-3*gap,range_grid->TopLeft().y - space,"Quit",0,cmdExitProgram));
	}

void PaletteWindow::Draw()
	{
	// draw "special" stuff to double buffer
	grDrawString(gap,5,"Select - Left=Start - Right=End");
	DrawGrid();

	grDrawString((range_grid->TopLeft()).x,(range_grid->TopLeft()).y-15,"Click This to Fill Range");
	DrawRangeBar();

	grDrawString(middle_x,red1_y-15,"Start");
	grDrawString(leftend_x,red1_y-15,"Red");
	grDrawString(leftend_x,green1_y-15,"Green");
	grDrawString(leftend_x,blue1_y-15,"Blue");

	grDrawString(middle_x,red2_y-15,"End");
	grDrawString(leftend_x,red2_y-15,"Red");
	grDrawString(leftend_x,green2_y-15,"Green");
	grDrawString(leftend_x,blue2_y-15,"Blue");

	DrawArrows();

	grDrawString((base_grid->TopLeft()).x,(base_grid->TopLeft()).y-15,"Preset Colours for your Enjoyment - Left=Start - Right=End");
	DrawBaseSelect();

	// draw "normal" stuff to double buffer
	OnScreenHandler::Draw();
	}

void PaletteWindow::DrawGrid()
	{
	int		xx,yy,i;

	// draw the GRID for full range of palette
	main_grid->Draw();

	// put all the colours in it
	i = 0;
	for (yy=1; yy<=main_grid->Height(); yy++)
		for (xx=1; xx<=main_grid->Width(); xx++)
			{
			grPoint c = {xx,yy};
			main_grid->FillCell(c, i);
			i++;
			}

	// select the currently selected cells
	main_grid->SelectCell(grid_start_cell, grLIGHTBLUE);
	main_grid->SelectCell(grid_end_cell, grLIGHTRED);

	grBlitRegion(
				(main_grid->TopLeft()).x,		(main_grid->TopLeft()).y,
				(main_grid->BottomRight()).x,	(main_grid->BottomRight()).y
				);
	}

void PaletteWindow::DrawRangeBar()
	{
	int		cnt,i;

	grDrawFilledRect(
					(range_grid->TopLeft()).x,					(range_grid->TopLeft()).y,
					(range_grid->TopLeft()).x+SIZE_OF_RANGEBAR,	(range_grid->BottomRight()).y,
					grBACKGROUND
					);

	range_grid->Draw();

	i = (start_i<end_i) ? start_i : end_i;

	// put all the colours in it
	for (cnt=1; cnt<=range_grid->Width(); cnt++)
		{
		grPoint c = { cnt, 1 };
		range_grid->FillCell(c,i);
		i++;
		}

	grBlitRegion(
				(range_grid->TopLeft()).x,					(range_grid->TopLeft()).y,
				(range_grid->TopLeft()).x+SIZE_OF_RANGEBAR,	(range_grid->BottomRight()).y
				);
	}

void PaletteWindow::DrawArrows()
	{
	// blank out old numbers
	grDrawFilledRect(middle_x,red1_y, right_x-1,blue2_y+arrow.y,grBACKGROUND);

	// get the start and end red, green and blue values
	grGetPaletteEntry(start_i,&r1,&g1,&b1);
	grGetPaletteEntry(end_i,&r2,&g2,&b2);

	// draw in the new numbers
	grDrawString(middle_x+2,red1_y,itoa(r1,NULL,10));
	grDrawString(middle_x+2,green1_y,itoa(g1,NULL,10));
	grDrawString(middle_x+2,blue1_y,itoa(b1,NULL,10));
	grDrawString(middle_x+2,red2_y,itoa(r2,NULL,10));
	grDrawString(middle_x+2,green2_y,itoa(g2,NULL,10));
	grDrawString(middle_x+2,blue2_y,itoa(b2,NULL,10));

	grBlitRegion(
				middle_x,		red1_y,
				right_x-1,		blue2_y+arrow.y
				);
	}

void PaletteWindow::DrawBaseSelect()
	{
	int		xx,yy, base_i = 0;

	// draw the first BASE grid for selection of base colour for start range
	base_grid->Draw();

	// put all the colours in it
	for (yy=1; yy<=base_grid->Height(); yy++)
		for (xx=1; xx<=base_grid->Width(); xx++)
			{
			grCell	c = { xx, yy };
			grPoint t = base_grid->CellTopLeft(c);

			// put some text for each different base colour
			grDrawString(t.x+4, t.y+5, base[base_i].name);

			base_i++;
			}

	grBlitRegion(
				(base_grid->TopLeft()).x,		(base_grid->TopLeft()).y,
				(base_grid->BottomRight()).x,	(base_grid->BottomRight()).y
				);
	}

void Swap(int* v1, int* v2)
	{
	int temp = *v1;
	*v1 = *v2;
	*v2 = temp;
	}

void grSwap(grPoint* v1, grPoint* v2)
	{
	grPoint temp = { (*v1).x, (*v1).y };
	(*v1).x = (*v2).x;
	(*v1).y = (*v2).y;

	(*v2).x = temp.x;
	(*v2).y = temp.y;
	}

void PaletteWindow::DoMainGrid(grPoint cell, bool left_mouse_button)
	{
	if (left_mouse_button)
		{
		main_grid->DeselectCell(grid_start_cell);

		grid_start_cell.x = cell.x;
		grid_start_cell.y = cell.y;

		main_grid->SelectCell(grid_start_cell, grLIGHTBLUE);
		}
	else
		{
		main_grid->DeselectCell(grid_end_cell);

		grid_end_cell.x = cell.x;
		grid_end_cell.y = cell.y;

		main_grid->SelectCell(grid_end_cell, grLIGHTRED);
		}

	DrawGrid();

	// re-calculate all the index values based on the new info
	start_i = main_grid->CellToIndex(grid_start_cell);
	end_i = main_grid->CellToIndex(grid_end_cell);

	if (start_i > end_i)
		number_i = start_i - end_i + 1;
	else
		number_i = end_i - start_i + 1;

	range_grid->SetWidth(number_i);
	range_grid->SetCellWidth(SIZE_OF_RANGEBAR/number_i);

	DrawRangeBar();
	DrawArrows();
	}

void PaletteWindow::FillRangeBar()
	{
	int 	i;

	uchar	start_red,start_green,start_blue;
	uchar	end_red,end_green,end_blue;

	uchar	red,green,blue;

	float	red_step,green_step,blue_step;

	int		si = (start_i<end_i) ? start_i : end_i,
			ei = (start_i<end_i) ? end_i : start_i;

	// get initial values
	grGetPaletteEntry(si,&start_red,&start_green,&start_blue);
	grGetPaletteEntry(ei,&end_red,&end_green,&end_blue);

	// calculate the gradients
	red_step = (float)(end_red - start_red) / (float)(number_i - 1);
	green_step = (float)(end_green - start_green) / (float)(number_i - 1);
	blue_step = (float)(end_blue - start_blue) / (float)(number_i - 1);

	// scan through each palette entry between start and end exclusive
	for (i=si+1; i<=ei-1; i++)
		{
		red = (uchar)((float)start_red + (float)(red_step* (i-si)));
		green = (uchar)((float)start_green + (float)(green_step* (i-si)));
		blue = (uchar)((float)start_blue + (float)(blue_step* (i-si)));

		grSetPaletteEntry(i,red,green,blue);
		}

	// get adjusted palette into memory
	grGetPalette(grPALETTE,0,255);
	}

void PaletteWindow::DoBaseGrid(grCell cell, bool left_mouse_button)
	{
	int edit_i = (left_mouse_button) ? start_i : end_i;

	int base_i = base_grid->CellToIndex(cell);

	grSetPaletteEntry(edit_i,base[base_i].red,base[base_i].green,base[base_i].blue);

	DrawArrows();
	}

void PaletteWindow::ModifyColour(int index, int entry, int movement)
	{
	uchar	red,
			green,
			blue;

	grGetPaletteEntry(index,&red,&green,&blue);

	int	r = red,
		g = green,
		b = blue;

	switch (entry)
		{
		case 1:
			r += movement;
			break;
		case 2:
			g += movement;
			break;
		case 3:
			b += movement;
			break;
		}

	if (r<0) r = 0;
	if (g<0) g = 0;
	if (b<0) b = 0;
	if (r>63) r = 63;
	if (g>63) g = 63;
	if (b>63) b = 63;

	red = r;
	green = g;
	blue = b;

	grSetPaletteEntry(index,red,green,blue);
	grGetPalette(grPALETTE,0,255);
	DrawArrows();
	}

void PaletteWindow::HandleEvent(Event& e)
	{
	OnScreenHandler::HandleEvent(e);

	bool	handled = false;

	switch (e.what)
		{
		case ev_mouse:
			switch (e.mouse.event)
				{
				// left mouse button for moving start point of range
				// and other things.
				// right mouse button for moving end point of range
				case mb_leftdn:
				case mb_rightdn:
					handled = HandleGrids(e);
					break;

				case mb_leftup:
				case mb_rightup:
				case mb_move:
				default:
					break;
				}
			break;

		case ev_command:
			handled = HandleArrows(e);

			if (!handled) switch (e.command.event)
				{
				// other commands
				case cmdLoad:

					LoadPalette();

					DrawArrows();

					handled = true;
					break;

				case cmdSave:
					SavePalette();

					handled = true;
					break;

				case cmdFill:
					FillRangeBar();
					handled = true;
					break;
				}
			break;

		case ev_keyboard:
			switch (e.keyboard.event)
				{
				case ESC_UP:
					EVENTMGR->GenEvent(0,cmdExitProgram,0);
					handled = true;
					break;
				}
			break;
		}

	// if we handled an event here then clear it HERE!!
	if (handled)
		{
		EVENTMGR->ClearEvent(e);
		}
	}

bool PaletteWindow::HandleArrows(Event& e)
	{
	bool	handled = false;

	switch (e.command.event)
		{
		// NORMAL arrow commands

		case cmd_r1_l:
			ModifyColour(start_i,1,-1);
			handled = true;
			break;
		case cmd_g1_l:
			ModifyColour(start_i,2,-1);
			handled = true;
			break;
		case cmd_b1_l:
			ModifyColour(start_i,3,-1);
			handled = true;
			break;

		case cmd_r2_l:
			ModifyColour(end_i,1,-1);
			handled = true;
			break;
		case cmd_g2_l:
			ModifyColour(end_i,2,-1);
			handled = true;
			break;
		case cmd_b2_l:
			ModifyColour(end_i,3,-1);
			handled = true;
			break;

		case cmd_r1_r:
			ModifyColour(start_i,1,1);
			handled = true;
			break;
		case cmd_g1_r:
			ModifyColour(start_i,2,1);
			handled = true;
			break;
		case cmd_b1_r:
			ModifyColour(start_i,3,1);
			handled = true;
			break;

		case cmd_r2_r:
			ModifyColour(end_i,1,1);
			handled = true;
			break;
		case cmd_g2_r:
			ModifyColour(end_i,2,1);
			handled = true;
			break;
		case cmd_b2_r:
			ModifyColour(end_i,3,1);
			handled = true;
			break;

		// FAST arrow commands

		case cmd_r1_ll:
			ModifyColour(start_i,1,-5);
			handled = true;
			break;
		case cmd_g1_ll:
			ModifyColour(start_i,2,-5);
			handled = true;
			break;
		case cmd_b1_ll:
			ModifyColour(start_i,3,-5);
			handled = true;
			break;

		case cmd_r2_ll:
			ModifyColour(end_i,1,-5);
			handled = true;
			break;
		case cmd_g2_ll:
			ModifyColour(end_i,2,-5);
			handled = true;
			break;
		case cmd_b2_ll:
			ModifyColour(end_i,3,-5);
			handled = true;
			break;

		case cmd_r1_rr:
			ModifyColour(start_i,1,5);
			handled = true;
			break;
		case cmd_g1_rr:
			ModifyColour(start_i,2,5);
			handled = true;
			break;
		case cmd_b1_rr:
			ModifyColour(start_i,3,5);
			handled = true;
			break;

		case cmd_r2_rr:
			ModifyColour(end_i,1,5);
			handled = true;
			break;
		case cmd_g2_rr:
			ModifyColour(end_i,2,5);
			handled = true;
			break;
		case cmd_b2_rr:
			ModifyColour(end_i,3,5);
			handled = true;
			break;

		// END arrow commands

		case cmd_r1_lll:
			ModifyColour(start_i,1,-1000);
			handled = true;
			break;
		case cmd_g1_lll:
			ModifyColour(start_i,2,-1000);
			handled = true;
			break;
		case cmd_b1_lll:
			ModifyColour(start_i,3,-1000);
			handled = true;
			break;

		case cmd_r2_lll:
			ModifyColour(end_i,1,-1000);
			handled = true;
			break;
		case cmd_g2_lll:
			ModifyColour(end_i,2,-1000);
			handled = true;
			break;
		case cmd_b2_lll:
			ModifyColour(end_i,3,-1000);
			handled = true;
			break;

		case cmd_r1_rrr:
			ModifyColour(start_i,1,1000);
			handled = true;
			break;
		case cmd_g1_rrr:
			ModifyColour(start_i,2,1000);
			handled = true;
			break;
		case cmd_b1_rrr:
			ModifyColour(start_i,3,1000);
			handled = true;
			break;

		case cmd_r2_rrr:
			ModifyColour(end_i,1,1000);
			handled = true;
			break;
		case cmd_g2_rrr:
			ModifyColour(end_i,2,1000);
			handled = true;
			break;
		case cmd_b2_rrr:
			ModifyColour(end_i,3,1000);
			handled = true;
			break;
		}

	return handled;
	}

bool PaletteWindow::HandleGrids(Event& e)
	{
	bool	handled = false;
	grCell	cell;

	if (main_grid->IsSelected(e.mouse.where))
		{
		handled = true;
		// get feedback from the main grid
		cell = main_grid->WhichCell(e.mouse.where);
		// if a cell is selected on the main grid
		if ((cell.x > 0) && (cell.y > 0))
			{
			// then do something with the selection depending on right or left
			DoMainGrid(cell,(e.mouse.event == mb_leftdn));
			}
		}

	if ((!handled) && base_grid->IsSelected(e.mouse.where))
		{
		handled = true;
		// get feedback from the base1 grid
		cell = base_grid->WhichCell(e.mouse.where);
		// if a cell is selected on the base1 grid
		if ((cell.x > 0) && (cell.y > 0))
			{
			// then do something with the selection
			DoBaseGrid(cell,(e.mouse.event == mb_leftdn));
			}
		}

	if ((!handled) && range_grid->IsSelected(e.mouse.where))
		{
		handled = true;

		FillRangeBar();
		}

	return handled;
	}

