//	-------------------------------------------------------------------------
//
//	PAL.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew Carvel Evans
//
//	DESCRIPTION:	This is the Palette Editting Window
//
//	-------------------------------------------------------------------------

#ifndef __pal_hpp
#define __pal_hpp

#include "palettes.hpp"
#include "palfile.hpp"

#include "onscreen.hpp"
#include "graphics.hpp"
#include "grid.hpp"
#include "resources.hpp"

#define		NUMBER_OF_BASECOLOURS	80
#define		SIZE_OF_NAME			12

struct BaseColour
	{
	char	name[SIZE_OF_NAME];

	uchar	red,
			green,
			blue;
	};

class PaletteWindow : public OnScreenHandler
	{
	private:
		ResourceFile*	palette_GUI_file;

		grImage*	palette_BITMAP[20];

		String*		palette_name;
		PALFile*	palette_file;
		grPalette*	palette;

		// Select Range Grid
		Grid*		main_grid;

		grPoint		grid_start_cell,		// selected start cell position
					grid_end_cell;			// selected end cell position

		// Edit Range Bar
		Grid*		range_grid;

		int			start_i,				// palette mapping start index
					end_i,					// palette mapping end index
					number_i;				// the difference between these

		// Arrows
		grPoint		arrow;					// physical size of each arrow

		// all arrows x position
		int			leftend_x,
					leftfast_x,
					left_x,

					middle_x,

					right_x,
					rightfast_x,
					rightend_x;

		// top arrows y positions
		int			red1_y,
					green1_y,
					blue1_y,
		// bottom arrows y positions
					blue2_y,
					green2_y,
					red2_y;

		// storage for the numbers that are displayed in between arrows
		uchar		r1,g1,b1;
		uchar		r2,g2,b2;

		// Base Select Square
		Grid*		base_grid;
		BaseColour	base[NUMBER_OF_BASECOLOURS];

		// create functions
		void		CreateGrid();
		void		CreateRangeBar();
		void		CreateArrows();
		void		AssignBaseColour(int i,char* n,uchar r,uchar g,uchar b);
		void		CreateBaseSelect();
		void		CreateButtons();

		// other functions
		void		DoMainGrid(grCell cell, bool left_mouse_button);
		void		FillRangeBar();
		void		DoBaseGrid(grCell cell, bool left_mouse_button);
		void		ModifyColour(int index, int entry, int movement);

		// handle event functions
		bool		HandleArrows(Event& e);
		bool		HandleGrids(Event& e);

	public:
		PaletteWindow(String* name);
		~PaletteWindow();

		// disk functions
		void		LoadPalette();
		void		SavePalette();

		// draw functions
		void		DrawGrid();
		void		DrawRangeBar();
		void		DrawArrows();
		void		DrawBaseSelect();

		// standard functions
		virtual void	Draw();
		virtual void	HandleEvent(Event& e);
	};

#endif

