//	-------------------------------------------------------------------------
//
//	TOOL.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew C. Evans
//
//	DESCRIPTION:	This program is a useful tool to create graphic
//					resources such as palettes, fonts, bitmaps and sprites
//					to be used by the `grGraphics Library'.
//
//	-------------------------------------------------------------------------

// include program definition

#include "tool.hpp"
#include "graphics.hpp"
#include "palettes.hpp"

// here #include all relevant bits and pieces!!!

#include "tooldefn.hpp"		// program-specific definitions

#include "pal.hpp"			// palette editting and display window

//	-------------------------------------------------------------------------
//
//	FUNC:		TOOL::Constructor()
//	DESC:		Allocates memory for map storage & since the program only
//				uses a single palette, sets this also.
//
//	-------------------------------------------------------------------------
TOOL::TOOL(char* s,char* Init_String,int options) : Program("Palette Editor Tool",Init_String,options)
	{
	name = new String(s);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		TOOL::Destructor()
//	DESC:		Cleans up after TOOL::Constructor()
//
//	-------------------------------------------------------------------------
TOOL::~TOOL()
	{
	delete name;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		MainLoop()
//	DESC:       This is the main loop of the program.
//
//	-------------------------------------------------------------------------
void TOOL::MainLoop()
	{
	grPALETTE = grCreatePalette(256);
	grGetPalette(grPALETTE,0,255);

	// set up our timer to work at 100 times a second
	timer_ChangeInterruptRate(timer_100Hz);

	// create all objects used in the program
	// NOTE:: for optimum use of this library, objects should be created
	//		  such that the first handler is the most used, the last the
	//		  least used....

	EVENTMGR->AddEventHandler(new PaletteWindow(name));

	// clear the screen to the default background colour
	grClearScreen(grBACKGROUND);

	// draw all the relevant handlers
	EVENTMGR->DrawAllHandlers();

	// tell the event manager to execute using the handlers!!!
	EVENTMGR->Execute();

	// clear the screen to black
	grClearScreen(grBLACK);

	// delete the objects now that we have finished with them
	EVENTMGR->DelAllEventHandlers();

	grDestroyPalette(grPALETTE);
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		main()
//	DESC:		This is the function called at program execution.
//				At the moment it simply creates the program, executes it,
//				then deletes it, exiting normally
//
//	-------------------------------------------------------------------------
void main(int argc, char* argv[])
	{
	// set up some default file strings
	String* defstr = new String("paledit.pal",255);

	// override with params iff required
	if (argc>1) defstr->Set(argv[1]);

	// convert to uppercase....
	defstr->ToUpper();

	// make sure we have full filename
	if (!defstr->Contains(".PAL")) defstr->Append(".PAL");

	// create our program
	TOOL* theProgram = new TOOL(defstr->Contents(),argv[0],
								prg_InstallSystemFont|prg_InstallGUI);

	// run it's intro, then setup, then mainloop, then finale
	theProgram->Execute();

	// and clear up after ourselves
	delete theProgram;

	delete defstr;
	}
