//	-------------------------------------------------------------------------
//
//	BASEWIN.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew Carvel Evans
//
//	DESCRIPTION:	This is the Base Colour Selection Window
//
//	-------------------------------------------------------------------------

#include "mouse.hpp"
#include "eventmgr.hpp"

#include "basewin.hpp"
// this window talks to these windows
#include "butwin.hpp"

int				evhno_BaseRegion;
BaseRegion*		basewin;

BaseRegion::BaseRegion()
	: OnScreenHandler(gap,gap,gap+SIZEOF_EDITREGION,gap+SIZEOF_EDITREGION,&evhno_BaseRegion,0,1)
	{
	if (screen_mode == GUI_PALETTE)	isOnScreen = true;
	else							isOnScreen = false;

	base_grid = new Grid(r.left,r.top,6,16,55,22,1,grWHITE,1,grWHITE,grDARKGRAY);

	// fill up the list of base colours with entries
	AssignBaseColour(0,"Black",0,0,0);
	AssignBaseColour(1,"DkGray",15,15,15);
	AssignBaseColour(2,"Gray",31,31,31);
	AssignBaseColour(3,"LtGray",47,47,47);
	AssignBaseColour(4,"White",63,63,63);
	AssignBaseColour(5,"Red",63,0,0);
	AssignBaseColour(6,"DkRed",31,0,0);
	AssignBaseColour(7,"Green",0,63,0);
	AssignBaseColour(8,"DkGree",0,31,0);
	AssignBaseColour(9,"Blue",0,0,63);
	AssignBaseColour(10,"DkBlue",0,0,31);
	AssignBaseColour(11,"Yellow",63,63,0);
	AssignBaseColour(12,"DkYell",31,31,0);
	AssignBaseColour(13,"Cyan",0,63,63);
	AssignBaseColour(14,"DkCyan",0,31,31);
	AssignBaseColour(15,"Magent",63,0,63);
	AssignBaseColour(16,"DkMage",31,0,31);
	AssignBaseColour(17,"DkBrow",20,10,0);
	AssignBaseColour(18,"Brown",40,30,20);
	AssignBaseColour(19,"LtBrow",60,50,40);
	AssignBaseColour(20,"Orange",63,31,0);
	AssignBaseColour(21,"Pink",60,35,35);
	AssignBaseColour(22,"Olive",30,40,25);
	AssignBaseColour(23,"Tree",10,40,15);
	AssignBaseColour(24,"Metal",15,35,45);
	AssignBaseColour(25,"Egg",45,60,50);

	// fill the rest randomly
	for (int base_i=26; base_i<NUMBER_OF_BASECOLOURS; base_i++)
		{
		int		red = 	random(64),
				green =	random(64),
				blue =	random(64);

		char	name[SIZE_OF_NAME] = "------";

		// and try to apply appropriate names to them
		if ((red>green) && (red>blue)) strcpy(name,"Red--?");
		if ((green>blue) && (green>red)) strcpy(name,"Gre--?");
		if ((blue>red) && (blue>green)) strcpy(name,"Blu--?");

		AssignBaseColour(base_i,name,red,green,blue);
		}
	}

BaseRegion::~BaseRegion()
	{
	delete base_grid;
	}

void BaseRegion::AssignBaseColour(int i,char* n,uchar r,uchar g,uchar b)
	{
	memcpy(base[i].name,n,SIZE_OF_NAME);
	base[i].red = r;
	base[i].green = g;
	base[i].blue = b;
	}

void BaseRegion::Draw()
	{
	grDrawFilledRect(r.left,r.top,r.right,r.bottom,grBACKGROUND);

	int		xx,yy, base_i = 0;

	// draw the first BASE grid for selection of base colour for start range
	base_grid->Draw();

	// put all the colours in it
	for (yy=0; yy<base_grid->Height(); yy++)
		for (xx=0; xx<base_grid->Width(); xx++)
			{
			grCell	c = { xx, yy };
			grPoint t = base_grid->CellTopLeft(c);

			// put some text for each different base colour
			grDrawString(t.x+4, t.y+5, base[base_i].name);

			base_i++;
			}

DBG(470,"Finished basewin->Draw");
	}

void BaseRegion::HandleEvent(Event& e)
	{
	bool	mouseclicked = false,
			handled = false;

	switch (e.what)
		{
		case ev_mouse:
			switch (e.mouse.event)
				{
				case mb_leftdn:
				case mb_rightdn:
				case mb_leftauto:
				case mb_rightauto:
					mouseisdown = true;
					mouseclicked = true;

					break;

				case mb_leftup:
				case mb_rightup:
					mouseisdown = false;
					mouseclicked = true;
					dragison = false;

					break;
				}
			break;
		}

	if ((mouseclicked == true) && (base_grid->IsSelected(e.mouse.where)))
		{
		handled = true;

		// get feedback from the base grid
		grCell cell = base_grid->WhichCell(e.mouse.where);

		// if the user presses the left button use start index otherwise end index
		int edit_i = ((e.mouse.event == mb_leftdn)||(e.mouse.event == mb_leftup)||(e.mouse.event == mb_leftauto)) ? color1 : color2;
		// work out what colour they want to change it to
		int base_i = base_grid->CellToIndex(cell);

		// set the index to the colour
		grSetPaletteEntry(edit_i,base[base_i].red,base[base_i].green,base[base_i].blue);

		// redraw the button window to update displayed colour values
		palbutwin->Refresh();
		}

	if (handled) EVENTMGR->ClearEvent(e);

	OnScreenHandler::HandleEvent(e);
	}
