//	-------------------------------------------------------------------------
//
//	EDITWIN.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew Carvel Evans
//
//	DESCRIPTION:	This is the Editting Window
//
//	-------------------------------------------------------------------------

#include "mouse.hpp"
#include "eventmgr.hpp"

#include "editwin.hpp"
// this window talks to these windows
#include "tilewin.hpp"
#include "thumbwin.hpp"

int				evhno_EditRegion;
EditRegion*		editwin;

EditRegion::EditRegion()
	: OnScreenHandler(gap,gap,gap+SIZEOF_EDITREGION,gap+SIZEOF_EDITREGION,&evhno_EditRegion,0,1)
	{
	if (screen_mode == GUI_ICONS)	isOnScreen = true;
	else							isOnScreen = false;

	// create a grid to select cells upon
	edit_region = new Grid(r.left,r.top,
		MAX_WIDTH, MAX_HEIGHT, SIZEOF_EDITREGION/MAX_WIDTH-1,SIZEOF_EDITREGION/MAX_HEIGHT-1,1,grDARKGRAY,1,grDARKGRAY,grBACKGROUND);
	}

EditRegion::~EditRegion()
	{
	DelAllAccessories();
	delete edit_region;
	}

//----------------------------------------------------------------------------
//
//		HANDLEDRAWING
//
//----------------------------------------------------------------------------
void EditRegion::HandleDrawing(Event& e)
	{
	int			x,y, x1,y1,x2,y2, oldx1,oldy1,oldx2,oldy2;

	// if left mousebutton use color1
	uchar		color = ((e.mouse.event == mb_leftdn)||(e.mouse.event == mb_leftup)||(e.mouse.event == mb_leftauto))
				? color1 : color2;

	// find the cell currently being pointed at
	grCell		cell = editwin->edit_region->WhichCell(e.mouse.where);

	ChooseSingleOrExtendedImage;

	if (!dragison)
		{
		memcpy(undo,current_image,grImageSize(0,0,current_image->hdr.width-1,current_image->hdr.height-1));
		undoUsed = true;
		}

	// then do something with the selection depending on right or left
	switch (output_draw_mode)
		{
		case ERASE:
			color = grBACKGROUND;
		case DRAW:
			PutPoint(cell,color);
			break;

		case LINE:
		case RECT:
		case ELLIPSE:
			if (!dragison)
				{
				memcpy(working,current_image,grImageSize(0,0,current_image->hdr.width-1,current_image->hdr.height-1));

				first_x = cell.x;
				first_y = cell.y;

				dragison = true;
				}
			else
				{
				memcpy(current_image,working,grImageSize(0,0,working->hdr.width-1,working->hdr.height-1));

				// store the old values for the second control
				old_second_x = second_x;
				old_second_y = second_y;

				// get the new ones
				second_x = cell.x;
				second_y = cell.y;

				// and put it on the grid, image and tiles
				switch (output_draw_mode)
					{
					case LINE:
						PutLine(color);
						break;

					case RECT:
						PutRect(color);
						break;

					case ELLIPSE:
						PutEllipse(color);
						break;
					}
				}
			break;
		}

	// draw the real life image, tile or repeat, etc... actual sized
	tilewin->Refresh();

	// copy the extended image to each of its component images
	if ((input_grid_mode == EXTENDED) || (input_grid_mode == EXTENDED_SINGLE))
		{
		tilewin->UseTiledImage();
//		tilewin->MakeTiledImage();
		thumbwin->Refresh();
		}
	else
		thumbwin->DrawThumb(current_image_index);
	}

//----------------------------------------------------------------------------
//
//		ADJUSTCOORDS
//
//		used to translate from onscreen grid coords to actual image coords
//
//----------------------------------------------------------------------------
void EditRegion::AdjustCoords(int* x, int* y)
	{
	if (tilewin->tile_selected_x != 0) *x = *x+tilewin->tile_selected_x*tilewin->tile_width-EXTRA_SPACE;
	if (tilewin->tile_selected_y != 0) *y = *y+tilewin->tile_selected_y*tilewin->tile_height-EXTRA_SPACE;
	}

//----------------------------------------------------------------------------
//
//		PUTPOINT
//
//----------------------------------------------------------------------------
void EditRegion::PutPoint(grCell cell, uchar color)
	{
	int	x1,y1,x2,y2, minx,miny,maxx,maxy;

	// -------------------------------------------
	// just fill from the cell if fill mode is on solid
	if (output_fill_mode == SOLID)
		{
		AdjustCoords(&cell.x,&cell.y);
		grFloodFill(current_image,cell.x,cell.y,color);
		Refresh();

		return;
		}

	// -------------------------------------------
	// if we're dragging the brush around
	if (dragison)
		{
		// the first point is the old point
		first_x = second_x;
		first_y = second_y;
		// and second point is the new position
		second_x = cell.x;
		second_y = cell.y;

		// and depending on the order translate those to image coordinates
		if (first_x<=second_x)
			{
			x1 = first_x;	y1 = first_y;
			x2 = second_x;	y2 = second_y;
			}
		else
			{
			x1 = second_x;	y1 = second_y;
			x2 = first_x;	y2 = first_y;
			}

		if (input_grid_mode == EXTENDED_SINGLE)
			{
			AdjustCoords(&x1,&y1);
			AdjustCoords(&x2,&y2);
			}

		// and draw a line on the image
		grDrawLineVirtual(current_image,x1,y1,x2,y2,color);

		// find the max bounds made by both new and old
		minx = second_x;
		if (first_x<minx) minx = first_x;

		maxx = second_x;
		if (first_x>maxx) maxx = first_x;

		miny = second_y;
		if (first_y<miny) miny = first_y;

		maxy = second_y;
		if (first_y>maxy) maxy = first_y;

		// and draw the region of grid that has changed
		RefreshRegion(minx,miny,maxx,maxy);

		return;
		}

	// -------------------------------------------
	// if the mouse has only just been pushed
	// put a point on the image
	int index;

	memcpy(working,current_image,grImageSize(0,0,current_image->hdr.width-1,current_image->hdr.height-1));

	index = edit_region->CellToIndex(cell);

	if (input_grid_mode == EXTENDED_SINGLE)
		{
		x1 = cell.x;
		y1 = cell.y;

		AdjustCoords(&x1,&y1);
		index = edit_region->CellToIndex(x1,y1);
		}

	if ((index>=0) && (index<current_image->hdr.width*current_image->hdr.height))
		{
		current_image->data[index] = color;

		// and the grid as well
		edit_region->FillCell(cell,color);
		edit_region->BlitCell(cell);
		}

	// and setup variables in case they want to drag the brush around
	second_x = cell.x;
	second_y = cell.y;

	dragison = true;
	}

//----------------------------------------------------------------------------
//
//		PUTLINE
//
//----------------------------------------------------------------------------
void EditRegion::PutLine(uchar color)
	{
	int	x1,y1,x2,y2, oldx1,oldy1,oldx2,oldy2, minx,miny,maxx,maxy;

	// depending on the order translate ends of line to image coordinates
	if (first_x<=second_x)
		{
		x1 = first_x;	y1 = first_y;
		x2 = second_x;	y2 = second_y;
		}
	else
		{
		x1 = second_x;	y1 = second_y;
		x2 = first_x;	y2 = first_y;
		}

	if (input_grid_mode == EXTENDED_SINGLE)
		{
		AdjustCoords(&x1,&y1);
		AdjustCoords(&x2,&y2);
		}

	// and draw a line on the image
	grDrawLineVirtual(current_image,x1,y1,x2,y2,color);

	// find the max bounds made by both new and old
	minx = second_x;
	if (old_second_x<minx) minx = old_second_x;
	if (first_x<minx) minx = first_x;

	maxx = second_x;
	if (old_second_x>maxx) maxx = old_second_x;
	if (first_x>maxx) maxx = first_x;

	miny = second_y;
	if (old_second_y<miny) miny = old_second_y;
	if (first_y<miny) miny = first_y;

	maxy = second_y;
	if (old_second_y>maxy) maxy = old_second_y;
	if (first_y>maxy) maxy = first_y;

	// and draw the region of grid that has changed
	RefreshRegion(minx,miny,maxx,maxy);
	}

//----------------------------------------------------------------------------
//
//		PUTRECT
//
//----------------------------------------------------------------------------
void EditRegion::PutRect(uchar color)
	{
	int	x1,y1,x2,y2, minx,miny,maxx,maxy;

	// depending on the order translate ends of line to image coordinates
	if (first_x<=second_x)
		{
		x1 = first_x;	y1 = first_y;
		x2 = second_x;	y2 = second_y;
		}
	else
		{
		x1 = second_x;	y1 = second_y;
		x2 = first_x;	y2 = first_y;
		}

	if (input_grid_mode == EXTENDED_SINGLE)
		{
		AdjustCoords(&x1,&y1);
		AdjustCoords(&x2,&y2);
		}

	// and draw a rect/filledrect on the image
	if (output_fill_mode)
		grDrawFilledRectVirtual(current_image,x1,y1,x2,y2,color);
	else
		grDrawRectVirtual(current_image,x1,y1,x2,y2,color);

	// find the max bounds made by both new and old
	minx = second_x;
	if (old_second_x<minx) minx = old_second_x;
	if (first_x<minx) minx = first_x;

	maxx = second_x;
	if (old_second_x>maxx) maxx = old_second_x;
	if (first_x>maxx) maxx = first_x;

	miny = second_y;
	if (old_second_y<miny) miny = old_second_y;
	if (first_y<miny) miny = first_y;

	maxy = second_y;
	if (old_second_y>maxy) maxy = old_second_y;
	if (first_y>maxy) maxy = first_y;

	// and draw the region of grid that has changed
	RefreshRegion(minx,miny,maxx,maxy);
	}

//----------------------------------------------------------------------------
//
//		PUTELLIPSE
//
//----------------------------------------------------------------------------
void EditRegion::PutEllipse(uchar color)
	{
	int	x1,y1,x2,y2, oldx2,oldy2, minx,miny,maxx,maxy;

	// depending on the order translate ends of line to image coordinates
	x1 = first_x;
	y1 = first_y;
	x2 = abs(second_x-first_x);
	y2 = abs(second_y-first_y);

	if (input_grid_mode == EXTENDED_SINGLE)
		AdjustCoords(&x1,&y1);

	// and draw a rect/filledrect on the image
	if (output_fill_mode)
		grDrawFilledEllipseVirtual(current_image,x1,y1,x2,y2,color);
	else
		grDrawEllipseVirtual(current_image,x1,y1,x2,y2,color);

	// work out where the last drag image was
	oldx2 = abs(old_second_x-first_x);
	oldy2 = abs(old_second_y-first_y);

	// find the max bounds made by both new and old
	minx = ((first_x-oldx2) < (first_x-x2)) ? (first_x-oldx2) : (first_x-x2);
	miny = ((first_y-oldy2) < (first_y-y2)) ? (first_y-oldy2) : (first_y-y2);
	maxx = ((first_x+oldx2) > (first_x+x2)) ? (first_x+oldx2) : (first_x+x2);
	maxy = ((first_y+oldy2) > (first_y+y2)) ? (first_y+oldy2) : (first_y+y2);

	// and draw the region of grid that has changed
	RefreshRegion(minx,miny,maxx,maxy);
	}

//----------------------------------------------------------------------------
//
//		RESIZEEDITREGION
//
//----------------------------------------------------------------------------
void EditRegion::ResizeEditRegion()
	{
	int cw,ch;

	if (input_grid_mode == EXTENDED_SINGLE)
		{
		// allow a gap of four on all sides
		edit_region->SetWidth(tilewin->tile_width+EXTRA_SPACE*2);
		edit_region->SetHeight(tilewin->tile_height+EXTRA_SPACE*2);

		// unless we only need extra space on one side
		if ((tilewin->tile_selected_x == 0) || (tilewin->tile_selected_x == tilewin->tiles_across-1))
			edit_region->SetWidth(tilewin->tile_width+EXTRA_SPACE);
		if ((tilewin->tile_selected_y == 0) || (tilewin->tile_selected_y == tilewin->tiles_down-1))
			edit_region->SetHeight(tilewin->tile_height+EXTRA_SPACE);

		// or even none
		if (tilewin->tiles_across==1)
			edit_region->SetWidth(tilewin->tile_width);
		if (tilewin->tiles_down==1)
			edit_region->SetHeight(tilewin->tile_height);
		}
	else if (input_grid_mode == EXTENDED)
		{
		current_image = extended_image;

		// no gaps needed
		edit_region->SetWidth(current_image->hdr.width);
		edit_region->SetHeight(current_image->hdr.height);
		}
	else
		{
		current_image = image[current_image_index];

		// allow a gap of four on all sides
		edit_region->SetWidth(current_image->hdr.width);
		edit_region->SetHeight(current_image->hdr.height);
		}

	if ((edit_region->Width()!=0) && (edit_region->Height()!=0))
		{
		cw = SIZEOF_EDITREGION/edit_region->Width() - 1;
		ch = SIZEOF_EDITREGION/edit_region->Height() - 1;

		if (cw>ch) cw = ch;
		else ch = cw;

		edit_region->SetCellWidth(cw);
		edit_region->SetCellHeight(ch);
		}
	}

//----------------------------------------------------------------------------
//
//		DRAWREGION
//
//----------------------------------------------------------------------------
void EditRegion::DrawRegion(int x1,int y1,int x2,int y2)
	{
	int		x,y, xx,yy, xx1,yy1, xx2,yy2;
	uchar	color;
	grImage*work;

	// resize the main grid so that it fits the current image
	ResizeEditRegion();

	// bounds and order check
	CLIP2(x1,y1,0,0,edit_region->Width()-1,edit_region->Height()-1);
	CLIP2(x2,y2,0,0,edit_region->Width()-1,edit_region->Height()-1);
	if (x2<x1) { int temp = x1; x1 = x2; x2 = temp; }
	if (y2<y1) { int temp = y1; y1 = y2; y2 = temp; }

	// only draw the full grid if the default or max parameters came into function
	bool	drawfullgrid = ((x1==0)&&(y1==0)&&(x2==edit_region->Width()-1)&&(y2==edit_region->Height()-1));

	// blank max area the grid can take
	if (drawfullgrid)
		{
		grDrawFilledRect(edit_x1,edit_y1, max_edit_x2,max_edit_y2, grBACKGROUND);

		// draw the GRID for the editting image
		edit_region->Draw();
		}

	if (input_grid_mode == SINGLE)
		{
		current_image = image[current_image_index];

		xx1 = (edit_region->TopLeft()).x;
		yy1 = (edit_region->TopLeft()).y;
		xx2 = (edit_region->BottomRight()).x;
		yy2 = (edit_region->BottomRight()).y;

		grDrawRect(xx1,yy1,xx2,yy2,grLIGHTRED);

		xx1 = 0;
		yy1 = 0;
		}
	else
		{
		current_image = extended_image;

		if (input_grid_mode == EXTENDED_SINGLE)
			{
			if (tilewin->tile_selected_x == 0)
				{
				xx1 = (edit_region->CellTopLeft(0,0)).x-1;
				if (tilewin->tiles_across>1)
					xx2 = (edit_region->CellTopLeft(edit_region->Width()-EXTRA_SPACE,0)).x-1;
				else
					xx2 = (edit_region->CellTopLeft(edit_region->Width(),0)).x-1;
				}
			else if (tilewin->tile_selected_x == tilewin->tiles_across-1)
				{
				xx1 = (edit_region->CellTopLeft(EXTRA_SPACE,0)).x-1;
				xx2 = (edit_region->CellTopLeft(edit_region->Width(),0)).x-1;
				}
			else
				{
				xx1 = (edit_region->CellTopLeft(EXTRA_SPACE,0)).x-1;
				xx2 = (edit_region->CellTopLeft(edit_region->Width()-EXTRA_SPACE,0)).x-1;
				}

			if (tilewin->tile_selected_y == 0)
				{
				yy1 = (edit_region->CellTopLeft(0,0)).y-1;
				if (tilewin->tiles_down>1)
					yy2 = (edit_region->CellTopLeft(0,edit_region->Height()-EXTRA_SPACE)).y-1;
				else
					yy2 = (edit_region->CellTopLeft(0,edit_region->Height())).y-1;
				}
			else if (tilewin->tile_selected_y == tilewin->tiles_across-1)
				{
				yy1 = (edit_region->CellTopLeft(0,EXTRA_SPACE)).y-1;
				yy2 = (edit_region->CellTopLeft(0,edit_region->Height())).y-1;
				}
			else
				{
				yy1 = (edit_region->CellTopLeft(0,EXTRA_SPACE)).y-1;
				yy2 = (edit_region->CellTopLeft(0,edit_region->Height()-EXTRA_SPACE)).y-1;
				}

			grDrawRect(xx1,yy1,xx2,yy2,grLIGHTRED);

			xx1 = 0;
			yy1 = 0;
			AdjustCoords(&xx1,&yy1);
			}
		else if (input_grid_mode == EXTENDED)
			{
			for (x=0; x<=tilewin->tiles_across; x++)
				{
				xx1 = (edit_region->CellTopLeft(tilewin->tile_width*x,0)).x-1;
				grDrawVLine(xx1,edit_y1+1,edit_y2-1,grLIGHTRED);
				}

			for (y=0; y<=tilewin->tiles_down; y++)
				{
				yy1 = (edit_region->CellTopLeft(0,tilewin->tile_height*y)).y-1;
				grDrawHLine(yy1,edit_x1+1,edit_x2-1,grLIGHTRED);
				}
			xx1 = 0;
			yy1 = 0;
			}
		}

	// put all the colours in it
	for (yy=y1; yy<=y2; yy++)
		for (xx=x1; xx<=x2; xx++)
			{
			color = grBLACK;

			if ((xx1+xx>=0) && (yy1+yy>=0) && (xx1+xx<current_image->hdr.width) && (yy1+yy<current_image->hdr.height))
				color = current_image->data[Index(current_image,xx1+xx,yy1+yy)];

			// then draw it
			edit_region->FillCell(xx,yy, color);
			}
	}

//----------------------------------------------------------------------------
//
//		DRAWMOUSEXYPOS
//
//----------------------------------------------------------------------------
inline
void EditRegion::DrawMouseXYPos(grPoint m)
	{
	String*	xystr = new String();

	xystr->Append("X-->");
	xystr->Append((long)m.x);
	xystr->Append(" Y-->");
	xystr->Append((long)m.y);
	xystr->Append("  ");

	grBlitString(tile_x1,0,grBACKGROUND,xystr->Contents());

	delete xystr;
	}

//----------------------------------------------------------------------------
//
//		DRAWGRIDDETAILS
//
//----------------------------------------------------------------------------
void EditRegion::DrawGridDetails()
	{
	long	w,h, bytes;
	String*	st = new String(256);
	grImage*i;

	if (input_grid_mode == EXTENDED_SINGLE)
		i = image[tilewin->tile[tilewin->tile_selected_x][tilewin->tile_selected_y]-1];
	else if (input_grid_mode == EXTENDED)
		i = extended_image;
	else
		i = image[current_image_index];

	w = i->hdr.width;
	h = i->hdr.height;

	bytes = grImageSize(0,0,w-1,h-1);

	st->Append(" Width "); st->Append(w);
	st->Append(" by Height "); st->Append(h);
	st->Append(" == Bytes "); st->Append(bytes);

	if (input_grid_mode == EXTENDED_SINGLE)
		st->Append(" Extended (Single)   ");
	else if (input_grid_mode == EXTENDED)
		st->Append(" Extended (Whole)    ");
	else
		st->Append(" Single Image        ");

	grBlitString(edit_x1,0,grBACKGROUND,st->Contents());

	delete st;
	}

//----------------------------------------------------------------------------
//
//		DRAW
//
//----------------------------------------------------------------------------
void EditRegion::Draw()
	{
	// draw the full grid
	DrawRegion(0,0,10000,10000);
DBG(470,"Finished editwin->Draw");
	}

//----------------------------------------------------------------------------
//
//		REFRESHREGION
//
//----------------------------------------------------------------------------
void EditRegion::RefreshRegion(int x1,int y1,int x2,int y2)
	{
	DrawRegion(x1,y1,x2,y2);

	// bounds and order check
	CLIP2(x1,y1,0,0,edit_region->Width()-1,edit_region->Height()-1);
	CLIP2(x2,y2,0,0,edit_region->Width()-1,edit_region->Height()-1);
	if (x2<x1) { int temp = x1; x1 = x2; x2 = temp; }
	if (y2<y1) { int temp = y1; y1 = y2; y2 = temp; }

	// only refresh the full grid if the default or max parameters came into function
	bool	drawfullgrid = ((x1==0)&&(y1==0)&&(x2==edit_region->Width()-1)&&(y2==edit_region->Height()-1));

	if (drawfullgrid)
		grBlitRegion(edit_x1,edit_y1,max_edit_x2,max_edit_y2);
	else
		grBlitRegion(
					(edit_region->CellTopLeft(x1,y1)).x,	(edit_region->CellTopLeft(x1,y1)).y,
					(edit_region->CellBottomRight(x2,y2)).x,(edit_region->CellBottomRight(x2,y2)).y
					);
	}

//----------------------------------------------------------------------------
//
//		HANDLEEVENT
//
//----------------------------------------------------------------------------
void EditRegion::HandleEvent(Event& e)
	{
	bool	mouseclicked = false,
			handled = false;

	switch (e.what)
		{
		case ev_mouse:
			switch (e.mouse.event)
				{
				case mb_leftdn:
				case mb_rightdn:
				case mb_leftauto:
				case mb_rightauto:
					mouseisdown = true;
					mouseclicked = true;

					break;

				case mb_move:
					if (edit_region->IsSelected(e.mouse.where))
						DrawMouseXYPos(edit_region->WhichCell(e.mouse.where));

					handled = true;
					break;

				case mb_leftup:
				case mb_rightup:
					mouseisdown = false;
					mouseclicked = true;

					break;
				}
			break;
		}

	if (mouseclicked == true)
		{
		ChooseSingleOrExtendedImage;
		handled = true;

		if (edit_region->IsSelected(e.mouse.where))
			{
			switch (dragging)
				{
				case BRUSH:
					HandleDrawing(e);
					handled = true;
					break;
				}
			}
		}

	if (!mouseisdown) dragison = false;

	if (handled) EVENTMGR->ClearEvent(e);

	OnScreenHandler::HandleEvent(e);
	}
