//	-------------------------------------------------------------------------
//
//	PALWIN.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew Carvel Evans
//
//	DESCRIPTION:	This is the Palette Window
//
//	-------------------------------------------------------------------------

#include "mouse.hpp"
#include "eventmgr.hpp"

#include "palwin.hpp"
// this window talks to these windows
#include "editwin.hpp"
#include "rangewin.hpp"

int				evhno_PaletteRegion;
PaletteRegion*	palwin;

PaletteRegion::PaletteRegion()
	: OnScreenHandler(gap,max_edit_y2,gap+SIZEOF_EDITREGION,grMaxy,&evhno_PaletteRegion,0,1)
	{
	isOnScreen = true;

	// Palette Select Squares
	palette_grid = new Grid(edit_x1,max_edit_y2+gap,
		32,8,10,8,1,grDARKGRAY,1,grDARKGRAY,grBACKGROUND);
	}

PaletteRegion::~PaletteRegion()
	{
	DelAllAccessories();
	delete palette_grid;
	}

void PaletteRegion::Draw()
	{
	grDrawFilledRect(r.left,r.top,r.right,r.bottom,grBACKGROUND);

	int		xx,yy;

	// draw the grid for selection of colour for editting with
	palette_grid->Draw();

	// put all the colours in it
	for (yy=0; yy<palette_grid->Height(); yy++)
		for (xx=0; xx<palette_grid->Width(); xx++)
			{
			palette_grid->FillCell(xx,yy,palette_grid->CellToIndex(xx,yy));
			}

	// select the colours that we are currently working with
	grPoint selectedcell1 = palette_grid->IndexToCell(color1);
	grPoint selectedcell2 = palette_grid->IndexToCell(color2);
	palette_grid->SelectCell(selectedcell1,grLIGHTRED);
	palette_grid->SelectCell(selectedcell2,grRED);

DBG(470,"Finished palwin->Draw");
	}

void PaletteRegion::HandleEvent(Event& e)
	{
	bool	mouseclicked = false,
			handled = false;

	switch (e.what)
		{
		case ev_mouse:
			switch (e.mouse.event)
				{
				case mb_leftdn:
				case mb_rightdn:
				case mb_leftauto:
				case mb_rightauto:
					mouseisdown = true;
					mouseclicked = true;
					break;

				case mb_leftup:
				case mb_rightup:
					mouseisdown = false;
					mouseclicked = true;
					break;
				}
			break;
		}

	if (mouseclicked == true)
	  if (palette_grid->IsSelected(e.mouse.where))
		{
		// get feedback from the base1 grid
		grCell cell = palette_grid->WhichCell(e.mouse.where);

		// then if its the left mouse button
		if ((e.mouse.event == mb_leftdn)||(e.mouse.event == mb_leftup)||(e.mouse.event == mb_leftauto))
			{
			palette_grid->DeselectCell(palette_grid->IndexToCell(color1));
			palette_grid->BlitCell(palette_grid->IndexToCell(color1));

			palette_grid->SelectCell(cell,grLIGHTRED);
			color1 = palette_grid->CellToIndex(cell);
			}
		else // or right mouse button
			{
			palette_grid->DeselectCell(palette_grid->IndexToCell(color2));
			palette_grid->BlitCell(palette_grid->IndexToCell(color2));

			palette_grid->SelectCell(cell,grRED);
			color2 = palette_grid->CellToIndex(cell);
			}

		palette_grid->BlitCell(cell);

		// redraw range bar if we're in palette editting mode
		if (screen_mode == GUI_PALETTE)
			fillrngwin->Refresh();

		handled = true;
		}

	if (handled) EVENTMGR->ClearEvent(e);

	OnScreenHandler::HandleEvent(e);
	}
