//	-------------------------------------------------------------------------
//
//	RANGEWIN.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew Carvel Evans
//
//	DESCRIPTION:	This is the Fill Range Bar Window
//
//	-------------------------------------------------------------------------

#include "mouse.hpp"
#include "eventmgr.hpp"

#include "rangewin.hpp"
// this window talks to these windows

int					evhno_FillRangeRegion;
FillRangeRegion*	fillrngwin;

FillRangeRegion::FillRangeRegion()
	: OnScreenHandler(grMaxx-32-gap,gap,grMaxx,grMaxy,&evhno_FillRangeRegion,0,1)
	{
	if (screen_mode == GUI_PALETTE)	isOnScreen = true;
	else							isOnScreen = false;

	// make sure the range bar has the right dimensions
	if (color1 > color2)
		{
		number_indexes = color1 - color2 + 1;
		}
	else
		{
		number_indexes = color2 - color1 + 1;
		}

	// Create Edit Range Bar as a vertical n by 1 grid
	range_grid = new Grid(r.left,r.top,1,number_indexes,r.right-r.left,(r.bottom-r.top)/number_indexes,1,grWHITE,0,grWHITE,grBACKGROUND);
	}

FillRangeRegion::~FillRangeRegion()
	{
	delete range_grid;
	}

void FillRangeRegion::Draw()
	{
	grDrawFilledRect(r.left,r.top,r.right,r.bottom,grBACKGROUND);

	int i;
	// make sure the range bar has the right dimensions
	if (color1 > color2)
		{
		number_indexes = color1 - color2 + 1;
		i = color2;
		}
	else
		{
		number_indexes = color2 - color1 + 1;
		i = color1;
		}

	range_grid->SetHeight( number_indexes );
	range_grid->SetCellHeight( (r.bottom-r.top)/number_indexes );

	range_grid->Draw();

	// put all the colours in it
	for (int cnt=0; cnt<range_grid->Height(); cnt++)
		{
		range_grid->FillCell(0,cnt,i);
		i++;
		}
	}

void FillRangeRegion::FillRangeBar()
	{
	uchar	start_red,start_green,start_blue;
	uchar	end_red,end_green,end_blue;

	uchar	red,green,blue;

	float	red_step,green_step,blue_step;

	int		si = (color1<color2) ? color1 : color2,
			ei = (color1<color2) ? color2 : color1;

	// get initial values
	grGetPaletteEntry(si,&start_red,&start_green,&start_blue);
	grGetPaletteEntry(ei,&end_red,&end_green,&end_blue);

	// calculate the gradients
	red_step = (float)(end_red - start_red) / (float)(number_indexes - 1);
	green_step = (float)(end_green - start_green) / (float)(number_indexes - 1);
	blue_step = (float)(end_blue - start_blue) / (float)(number_indexes - 1);

	// scan through each palette entry between start and end exclusive
	for (int i=si+1; i<=ei-1; i++)
		{
		red = (uchar)((float)start_red + (float)(red_step* (float)(i-si)));
		green = (uchar)((float)start_green + (float)(green_step* (float)(i-si)));
		blue = (uchar)((float)start_blue + (float)(blue_step* (float)(i-si)));

		grSetPaletteEntry(i,red,green,blue);
		}

	// get adjusted palette into memory
	grGetPalette(palette,0,255);
	}

void FillRangeRegion::HandleEvent(Event& e)
	{
	bool	mouseclicked = false,
			handled = false;

	switch (e.what)
		{
		case ev_mouse:
			switch (e.mouse.event)
				{
				case mb_leftup:
				case mb_rightup:
					mouseisdown = false;
					mouseclicked = true;
					dragison = false;

					break;
				}
			break;
		}

	if ((mouseclicked == true) && (range_grid->IsSelected(e.mouse.where)))
		{
		handled = true;

		FillRangeBar();
		}

	if (handled) EVENTMGR->ClearEvent(e);

	OnScreenHandler::HandleEvent(e);
	}
