//	-------------------------------------------------------------------------
//
//	THUMBWIN.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew Carvel Evans
//
//	DESCRIPTION:	This is the Thumb Window
//
//	-------------------------------------------------------------------------

#include "mouse.hpp"
#include "eventmgr.hpp"
#include "buttons.hpp"

#include "thumbwin.hpp"
// this window talks to these windows
#include "editwin.hpp"
#include "tilewin.hpp"

int				evhno_ThumbRegion;
ThumbRegion*	thumbwin;

ThumbRegion::ThumbRegion()
	: OnScreenHandler(grMaxx-32-gap,gap,grMaxx,grMaxy,&evhno_ThumbRegion,0,1)
	{
	if (screen_mode == GUI_ICONS)	isOnScreen = true;
	else							isOnScreen = false;

	// Create Thumbnail Range Bar
	thumbnails = new Grid(r.left,r.top+GUIbitmap[GUI_TOP]->hdr.height,
		1,12,32,32,1,grDARKGRAY,1,grDARKGRAY,grBACKGROUND);

	// RangeBar Button creation
	AddAccessory(new PictureButton(r.left,r.top,
					GUIbitmap[GUI_TOP],GUIbitmap[GUI_TOP+1],0,cmdGotoFirstImage));

	AddAccessory(new PictureButton(r.left,(thumbnails->BottomRight()).y,
					GUIbitmap[GUI_BOTTOM],GUIbitmap[GUI_BOTTOM+1],0,cmdGotoLastImage));

	thumbnail_start_index = 0;
	}

ThumbRegion::~ThumbRegion()
	{
	DelAllAccessories();
	delete thumbnails;
	}

void ThumbRegion::DrawThumb(int img_index)
	{
	int cnt, i;

	// select the image halfway before the currently selected one
	thumbnail_start_index = current_image_index - thumbnails->Height()/2;

	// and if that means we'll display blanks on the end then move back
	while (thumbnail_start_index>number_images-thumbnails->Height()) thumbnail_start_index--;

	// and if that is before the first one then move along until it isn't
	while (thumbnail_start_index<0) thumbnail_start_index++;

	i = thumbnail_start_index;

	// select the image that we are currently working on
	grPoint selectedcell = { 0,current_image_index-i };
	thumbnails->SelectCell(selectedcell,grLIGHTRED);

	// put all the images in it
	for (cnt=0; cnt<thumbnails->Height(); cnt++)
		{
		if ((i<number_images) && (image[i]) && (i == img_index))
			{
			grPoint p1 = thumbnails->CellTopLeft(0,cnt);
			grPoint p2 = thumbnails->CellBottomRight(0,cnt);

			grPutScaledImage(p1.x,p1.y, p2.x,p2.y, image[i]);

			thumbnails->BlitCell(0,cnt);
			}
		i++;
		}
	}

void ThumbRegion::Draw()
	{
	grDrawFilledRect(r.left,r.top,r.right,r.bottom,grBACKGROUND);

	int cnt, i;

	grBlitString(grMaxx-grFontWidth("WWWWWWWW.WWW"),0,grBACKGROUND,pcx_file[current_image_index]->FileName());

	thumbnails->Draw();

	// select the image 2 image before the currently selected one
	thumbnail_start_index = current_image_index - thumbnails->Height()/2;

	// and if that means we'll display blanks on the end then move back
	while (thumbnail_start_index>number_images-thumbnails->Height()) thumbnail_start_index--;

	// and if that is before the first one then move along until it isn't
	while (thumbnail_start_index<0) thumbnail_start_index++;

	i = thumbnail_start_index;

	// select the image that we are currently working on
	grPoint selectedcell = { 0,current_image_index-i };
	thumbnails->SelectCell(selectedcell,grLIGHTRED);

	// put all the images in it
	for (cnt=0; cnt<thumbnails->Height(); cnt++)
		{
		if (i<number_images)
			if (image[i])
				{
				grPoint p1 = thumbnails->CellTopLeft(0,cnt);
				grPoint p2 = thumbnails->CellBottomRight(0,cnt);

				grPutScaledImage(p1.x,p1.y, p2.x,p2.y, image[i]);
				}
		i++;
		}

DBG(470,"Finished thumbwin->Draw");
	}

void ThumbRegion::HandleEvent(Event& e)
	{
	bool	mouseclicked = false,
			handled = false;

	switch (e.what)
		{
		case ev_mouse:
			switch (e.mouse.event)
				{
				case mb_leftdn:
				case mb_rightdn:
				case mb_leftauto:
				case mb_rightauto:
					mouseisdown = true;
					mouseclicked = true;
					break;

				case mb_leftup:
				case mb_rightup:
					mouseisdown = false;
					mouseclicked = true;
					break;
				}
			break;

		case ev_command:
			switch (e.command.event)
				{
				case cmdGotoFirstImage:
					current_image_index = 0;

					Refresh();
					editwin->Refresh();
					tilewin->Refresh();

					handled = true;
					break;

				case cmdGotoLastImage:
					current_image_index = number_images-1;

					Refresh();
					editwin->Refresh();
					tilewin->Refresh();

					handled = true;
					break;

				}
			break;
		}

	if (mouseclicked == true)
	  if (thumbnails->IsSelected(e.mouse.where))
		{
		// make sure the function doesn't respond to the mouse up event
		// otherwise single click makes a double jump
		if ((e.mouse.event == mb_leftdn)||(e.mouse.event == mb_leftauto)||
				(e.mouse.event == mb_rightdn)||(e.mouse.event == mb_rightauto))
			{
			current_image_index = thumbnail_start_index+thumbnails->CellToIndex(thumbnails->WhichCell(e.mouse.where));

			CLIP1(current_image_index,0,number_images-1);
			Refresh();

			// if we're in a mode that responds to the current image then redraw
			if ((input_grid_mode != EXTENDED) && (input_grid_mode != EXTENDED_SINGLE))
				{
				editwin->Refresh();
				tilewin->Refresh();
				}
			}
		}

	// if we handled an event here then clear it HERE!!
	if (handled) EVENTMGR->ClearEvent(e);

	OnScreenHandler::HandleEvent(e);
	}
