//	-------------------------------------------------------------------------
//
//	TILEWIN.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Andrew Carvel Evans
//
//	DESCRIPTION:	This is the Tile Window
//
//	-------------------------------------------------------------------------

#include "mouse.hpp"
#include "eventmgr.hpp"

#include "tilewin.hpp"
// this window talks to these windows
#include "editwin.hpp"

int				evhno_TileRegion;
TileRegion*		tilewin;

TileRegion::TileRegion()
	: OnScreenHandler(max_edit_x2+gap,gap,max_edit_x2+gap+SIZEOF_TILEAREA,gap+SIZEOF_TILEAREA,&evhno_TileRegion,0,1)
	{
	isOnScreen = true;

	// tile Grid
	tile_grid = new Grid(r.left,r.top,
		MAX_TILES_ACROSS,MAX_TILES_DOWN, MAX_WIDTH-1, MAX_HEIGHT-1, 1,grRED,1,grRED,grBACKGROUND);
	}

TileRegion::~TileRegion()
	{
	DelAllAccessories();
	delete tile_grid;
	}

void TileRegion::ClearTiledImage()
	{
	int x,y;

	for (y=0; y<MAX_TILES_DOWN; y++)
		for (x=0; x<MAX_TILES_ACROSS; x++)
			tile[x][y] = 0;

	tile_selected_x = 0;
	tile_selected_y = 0;

	tiles_across = 0;
	tiles_down = 0;

	tile_width = default_width;
	tile_height = default_height;

	any_tiles_selected = false;

	ResetExtendedImage();
	}

void TileRegion::MakeTiledImage()
	{
	int		i,x,y,x1,y1,x2,y2, mw=0,mh=0;

	tiles_across = 0;
	tiles_down = 0;

	ResetExtendedImage();

	for (y=0;y<MAX_TILES_DOWN;y++)
		for (x=0;x<MAX_TILES_ACROSS;x++)
			{
			if (tile[x][y])
				{
				current_image = image[tile[x][y]-1];
				tile_width = current_image->hdr.width;
				tile_height = current_image->hdr.height;

				if (x>=tiles_across) tiles_across = x+1;
				if (y>=tiles_down) tiles_down = y+1;
				}
			}
	extended_image->hdr.width = tiles_across*tile_width;
	extended_image->hdr.height = tiles_down*tile_height;

	if ((tiles_across==0) || (tiles_down==0))
		any_tiles_selected = false;
	else
		{
		any_tiles_selected = true;

		for (y=0;y<tiles_down;y++)
			for (x=0;x<tiles_across;x++)
				{
				if (tile[x][y])
					{
					current_image = image[tile[x][y]-1];

					x1 = tile_width * x;
					y1 = tile_height * y;

					x2 = x1 + tile_width - 1;
					y2 = y1 + tile_height - 1;

					grPutImageVirtual(extended_image,x1,y1,x2,y2,current_image);
					}
				}
		}
	}

void TileRegion::UseTiledImage()
	{
	if ((input_grid_mode != EXTENDED) && (input_grid_mode != EXTENDED_SINGLE))
		return;

	int x,y, x1,y1,x2,y2, width,height;

	// copy the extended image into its respective tiles
	for (y=0;y<tiles_down;y++)
		for (x=0;x<tiles_across;x++)
			{
			if (tile[x][y])
				{
				grImage* img = image[tile[x][y]-1];
				width = img->hdr.width;
				height = img->hdr.height;

				x1 = width * x;
				y1 = height * y;

				x2 = x1 + width - 1;
				y2 = y1 + height - 1;

				grGetImageVirtual(extended_image,x1,y1,x2,y2,img);
				}
			}
	}

void TileRegion::DrawTileSelect()
	{
	tile_grid->SetCellWidth(tile_width-1);
	tile_grid->SetCellHeight(tile_height-1);

	tile_grid->Draw();
	}

void TileRegion::Draw()
	{
	int		x1,y1,x2,y2, width,height;

	// blank the area
	grDrawFilledRect(r.left,r.top, r.right,r.bottom, grBACKGROUND);

	if (input_grid_mode == SINGLE)
		current_image = image[current_image_index];
	else if ((input_grid_mode == EXTENDED) || (input_grid_mode == EXTENDED_SINGLE))
		{
		current_image = extended_image;
		DrawTileSelect();
		}

	if (output_tile_mode == SINGLE)
		{
		width = current_image->hdr.width;
		height = current_image->hdr.height;

		// put the single/extended image on the screen
		grPutImage(tile_x1,tile_y1,tile_x1+width-1,tile_y1+height-1,current_image);

		// maybe draw a red box around the individual image so we can see it
		if ((input_grid_mode == EXTENDED_SINGLE) && (output_bounds_mode))
			tile_grid->SelectCell(tile_selected_x,tile_selected_y,grLIGHTRED);
		}
	else
		{
		width = current_image->hdr.width;
		height = current_image->hdr.height;

		x1 = tile_x1;
		y1 = tile_y1;

		x2 = x1 + width - 1;
		y2 = y1 + height - 1;

		// while we haven't gone past the maximum bounds
		while (y2<r.bottom)
			{
			while (x2<r.right)
				{
				// keep on putting the single/extended image on the screen
				grPutImage(x1,y1, x2,y2, current_image);

				// move along to the next position across
				x1 = x2 + 1;
				x2 = x1 + width - 1;
				}
			// finished moving across so go back to the left
			x1 = tile_x1; x2 = x1 + width - 1;
			// and go down line
			y1 = y2 + 1; y2 = y1 + height - 1;
			}
		}
DBG(470,"Finished tilewin->Draw");
	}

void TileRegion::Refresh()
	{
	if (output_tile_mode == REPEAT)
		{
		OnScreenHandler::Refresh();
		return;
		}

	Draw();
	grBlitRegion(r.left,r.top,r.left+current_image->hdr.width,r.top+current_image->hdr.height);
	}

void TileRegion::HandleEvent(Event& e)
	{
	bool	mouseclicked = false,
			handled = false;

	switch (e.what)
		{
		case ev_mouse:
			switch (e.mouse.event)
				{
				case mb_leftdn:
				case mb_rightdn:
				case mb_leftauto:
				case mb_rightauto:
					mouseisdown = true;
					mouseclicked = true;
					break;

				case mb_leftup:
				case mb_rightup:
					mouseisdown = false;
					mouseclicked = true;
					break;
				}
			break;
		}

	if (mouseclicked == true)
		{
		// make sure the function doesn't respond to the mouse down events
		// otherwise single click makes a double jump
		if (tile_grid->IsSelected(e.mouse.where) && ((e.mouse.event == mb_leftup)||(e.mouse.event == mb_rightup)) && (input_grid_mode != SINGLE))
			{
			int x,y;
			grCell cell = tile_grid->WhichCell(e.mouse.where);
			bool Ok;

			// for the left mouse button which selects/deselects a single image from an extended image
			if (e.mouse.event == mb_leftup)
				{
				if ((tile_selected_x == cell.x) && (tile_selected_y == cell.y))
					{
					if (input_grid_mode == EXTENDED_SINGLE)
						input_grid_mode = EXTENDED;
					else
						input_grid_mode = EXTENDED_SINGLE;
					}
				else
					input_grid_mode = EXTENDED_SINGLE;
				}
			else
				{
				// show that a tile position has been selected
				tile_grid->SelectCell(cell,grLIGHTRED);
				tile_grid->BlitCell(cell);

				// if there is a tile where we have clicked the right mouse button
				if (tile[cell.x][cell.y])
					{
					// then get rid of it
					tile[cell.x][cell.y] = 0;
					}
				else
					{
					// all images have to be the same size
					if ((!any_tiles_selected) || ((image[current_image_index]->hdr.width==tile_width) && (image[current_image_index]->hdr.height==tile_height)))
						{
						Ok = true;

						// if this tile is somewhere else within the extended image
						for (y=0; y<tiles_down; y++)
							for (x=0; x<tiles_across; x++)
								if (tile[x][y] == current_image_index+1)
									// then can't move it
									Ok = false;

						if (Ok)
							tile[cell.x][cell.y] = current_image_index+1;
						}
					else
						{
//ACE
//						DrawMessage("All tiles must be the same size");
						}
					}
				}

			if (tile[cell.x][cell.y])
				{
				tile_selected_x = cell.x;
				tile_selected_y = cell.y;
				}

			MakeTiledImage();

			Refresh();
			editwin->Refresh();
			}
		handled = true;
		}

	if (handled) EVENTMGR->ClearEvent(e);

	OnScreenHandler::HandleEvent(e);
	}
