//
//	TOOL data definitions
//

#include "tooldefn.hpp"

int		screen_mode;
bool	icons_available;

// -------------------------------------------------------------------

ResourceFile*	GUI_file = NULL;
grImage**		GUIbitmap = NULL;

// -------------------------------------------------------------------

// multiple images
String*		icons_name = NULL;				// definition file details that we
DEFFile*	definition_file = NULL;		// use to select images to edit
String*		definition_str = NULL;			// list of files in definition file

int			number_images;			// number of images in definition file

PCXFile**	pcx_file = NULL;				// for loading the images
grImage**	image = NULL;					// current images

int			current_image_index;

grImage*	current_image = NULL;
grImage*	extended_image = NULL;

// -------------------------------------------------------------------

int			image_selected;

int			default_width,
			default_height;

// variables used while dragging
int			first_x,
			first_y,
			second_x, old_second_x,
			second_y, old_second_y;

// currently selected colors
uchar		color1,			// left mouse button
			color2;			// right mouse button

// -------------------------------------------------------------------

// States

// mouse state for continuous movement
bool		mouseisdown;

// are we dragging domething now?
bool		dragison;

// what we are currently dragging
int			dragging;

// input mode, eg. NORMAL_INPUT,TILE_INPUT
int			input_query_mode;

// the way we edit the images, eg. SINGLE or EXTENDED images
int			input_grid_mode;

// output mode, eg. ERASE,DRAW,LINE,RECT,ELLIPSE
int			output_draw_mode;

// fill shapes
int			output_fill_mode;

// tile mode
int			output_tile_mode;

// whether we draw red lines all over the place
bool		output_bounds_mode;

String*		message = NULL;

// where we display output_draw_mode and output_fill_mode
int			mode_x, mode_y;

// -------------------------------------------------------------------

// for the undo command
grImage*	undo = NULL;
// for working with when resizing, scaling, etc.
grImage*	working = NULL;
// for store and retrieve command
grImage*	store = NULL;
// for copying and pasting parts of an image
grImage*	clipboard = NULL;

bool		undoUsed;
bool		storeUsed;
bool		clipboardUsed;

// -------------------------------------------------------------------

String*		palette_name = NULL;
PALFile*	palette_file = NULL;
grPalette*	palette = NULL;

// all arrows x position
int			leftend_x,leftfast_x,left_x,
			middle_x,
			right_x,rightfast_x,rightend_x;

// top arrows y positions
int			red1_y,green1_y,blue1_y,
// bottom arrows y positions
			blue2_y,green2_y,red2_y;

// storage for the numbers that are displayed in between arrows
uchar		r1,g1,b1;
uchar		r2,g2,b2;

// -------------------------------------------------------------------

inline
void Swap(int* v1, int* v2)
	{
	int temp = *v1;
	*v1 = *v2;
	*v2 = temp;
	}

void DrawMessage(int x1,int y1,int x2,int y2,char* message)
	{
	// clear the whole message region
	grDrawFilledRect(x1,y1,x2,y2,grBACKGROUND);
	grBlitRegion(x1,y1,x2,y2);

	grBlitString(x1,y1,grBACKGROUND,message);
	}

//----------------------------------------------------------------------------
//
//		RESIZE FUNCTIONS
//
//----------------------------------------------------------------------------
void ResetCurrentImage()
	{
	current_image->hdr.width = MAX_WIDTH*3-1;
	current_image->hdr.height = MAX_HEIGHT*3-1;
	}

void ResetExtendedImage()
	{
	extended_image->hdr.width = MAX_WIDTH*3-1;
	extended_image->hdr.height = MAX_HEIGHT*3-1;

	grClearImage(extended_image,0);
	}

void ResetUndo()
	{
	undo->hdr.width = MAX_WIDTH*3-1;
	undo->hdr.height = MAX_HEIGHT*3-1;
	}

void ResetWorking()
	{
	working->hdr.width = MAX_WIDTH*3-1;
	working->hdr.height = MAX_HEIGHT*3-1;
	}

void ResetStore()
	{
	store->hdr.width = MAX_WIDTH*3-1;
	store->hdr.height = MAX_HEIGHT*3-1;
	}

void ResetClipboard()
	{
	clipboard->hdr.width = MAX_WIDTH*3-1;
	clipboard->hdr.height = MAX_HEIGHT*3-1;
	}

void MakeImageDefaultSize(grImage* img)
	{
	img->hdr.width = default_width;
	img->hdr.height = default_width;
	}

void ResizeImageIfNecessary(grImage* img)
	{
	if (img->hdr.width < 1) img->hdr.width = 1;
	if (img->hdr.height < 1) img->hdr.height = 1;

	if (img->hdr.width > MAX_WIDTH) img->hdr.width = MAX_WIDTH;
	if (img->hdr.height > MAX_HEIGHT) img->hdr.height = MAX_HEIGHT;
	}
