//
//	TOOL data definitions
//

#ifndef		__tooldefn_hpp
#define		__tooldefn_hpp

#include "stddefns.hpp"
#include "strings.hpp"
#include "graphics.hpp"
#include "resource.hpp"
#include "pcxfile.hpp"
#include "palfile.hpp"
#include "deffile.hpp"

// -------------------------------------------------------------------
extern	int		screen_mode;
extern	bool	icons_available;

// GUI stuff

extern	ResourceFile*	GUI_file;
extern	grImage**		GUIbitmap;

// -------------------------------------------------------------------

#define		SIZEOF_EDITREGION		380
#define		SIZE_OF_RANGEBAR		600
#define		SIZEOF_TILEAREA			MAX_TILES_ACROSS*MAX_WIDTH

#define		EXTRA_SPACE				4
#define		MAX_WIDTH				64
#define		MAX_HEIGHT				64
#define		MAX_TILES_ACROSS		3
#define		MAX_TILES_DOWN			3

// -------------------------------------------------------------------

// for neatness
#define ChooseSingleOrExtendedImage \
			if (input_grid_mode == SINGLE) \
				current_image = image[current_image_index]; \
			else if ((input_grid_mode == EXTENDED) || (input_grid_mode == EXTENDED_SINGLE)) \
				current_image = extended_image

// -------------------------------------------------------------------
// buttons

// standard button numbers
#define		GUI_BUTTON			0
#define		GUI_UPARROW			2
#define		GUI_DNARROW			4
#define		GUI_LTARROW			6
#define		GUI_RTARROW			8
#define		GUI_TOP				10
#define		GUI_BOTTOM			12
#define		GUI_LEFT			14
#define		GUI_RIGHT			16
#define		GUI_TICK			18
#define		GUI_CROSS			20
#define		GUI_QUESTION		22
#define		GUI_BULLET			24
#define		GUI_QUIT			26			//bh
#define		GUI_LOAD			28			//bh
#define		GUI_SAVE			30			//bh

// button numbers for bitmap controls
#define		GUI_LOADALL			32			//bh
#define		GUI_SAVEALL			34			//bh
#define		GUI_ANIM			36			//b
#define		GUI_SHOWDEF			38			//b

#define		GUI_ERASE			40			//bh
#define		GUI_DRAW			42			//bh
#define		GUI_LINE			44			//bh
#define		GUI_RECT			46			//bh
#define		GUI_ELLIPSE			48			//bh

#define		GUI_FILL			50			//bh
#define		GUI_FULLFILL		52			//bh

#define		GUI_COPY			54			//b
#define		GUI_PASTE			56			//b
#define		GUI_STORE			58			//bh
#define		GUI_GET				60			//bh
#define		GUI_SHOWCLIP		62			//b

#define		GUI_FLIPX			64			//bh
#define		GUI_FLIPY			66			//bh
#define		GUI_ROTATE90		68			//bh
#define		GUI_ROTATE180		70			//bh
#define		GUI_ROTATE270		72			//bh

#define		GUI_UNDO			74

#define		GUI_SINGLE			76
#define		GUI_REPEAT			78
#define		GUI_TILE			80
#define		GUI_CLEARTILE		82

#define		GUI_ICONS			84
#define		GUI_PALETTE			86

// button numbers for palette control
#define		PGUI_BUTTON		0
#define		PGUI_LOAD		2
#define		PGUI_SAVE		4
#define		PGUI_QUIT		6
#define		PGUI_ICONS	    8

// -------------------------------------------------------------------

// the things that we can drag on the screen
#define		NONE			0
#define		BRUSH			1
#define		THUMBNAIL		2

// drawing modes
#define		ERASE			0
#define		DRAW			1
#define		LINE			2
#define		FLATLINE		3
#define		RECT			4
#define		ELLIPSE			5

// input modes
#define		NORMAL_INPUT	0
#define		TILE_INPUT		1

// fill modes
#define		NONE			0
#define		SOLID			1

// output modes
#define		SINGLE			1
#define		REPEAT			2
#define		EXTENDED		3
#define		EXTENDED_SINGLE	4

// -------------------------------------------------------------------

// command-events

// disk
#define	cmdLoad							2001
#define	cmdSave							2002

#define	cmdLoadAll						2003
#define	cmdSaveAll						2004

#define	cmdShowDefinitions				2010
#define cmdShowAnimation				2020

#define cmdGotoFirstImage				2030
#define cmdGotoLastImage				2031

// clipboard
#define cmdStoreImage					2050
#define cmdGetImage						2051
#define cmdCopy							2052
#define cmdPaste						2053
#define cmdShowClipboard				2054

// editting
#define cmdFillImage					2100
#define cmdClearImage					2101
#define cmdFill							2102

#define cmdErase						2104
#define cmdFilledErase					2105
#define cmdDraw							2106
#define cmdFilledDraw					2107
#define cmdLine							2108
#define cmdFlatLine						2109
#define cmdRect							2110
#define cmdFilledRect					2111
#define cmdEllipse						2112
#define cmdFilledEllipse				2113

// manipulation
#define cmdDecreaseHeightBottom			2150
#define cmdDecreaseHeightTop			2151
#define cmdIncreaseHeightBottom			2152
#define cmdIncreaseHeightTop			2153
#define cmdDecreaseWidthRight			2154
#define cmdDecreaseWidthLeft			2155
#define cmdIncreaseWidthRight			2156
#define cmdIncreaseWidthLeft			2157

#define	cmdFlipHorizontal				2160
#define	cmdFlipVertical					2161

#define cmdScrollLeft					2162
#define cmdScrollLeftFive				2163
#define cmdScrollRight					2164
#define cmdScrollRightFive				2165
#define cmdScrollUp						2166
#define cmdScrollUpFive					2167
#define cmdScrollDown					2168
#define cmdScrollDownFive				2169

#define	cmdRotate90						2170
#define	cmdRotate180					2171
#define	cmdRotate270					2172

#define	cmdSingle						2180
#define	cmdRepeat						2181
#define	cmdTile							2182
#define	cmdClearTile					2183

#define	cmdUndo							2190

#define	cmdGotoIcons					2220
#define	cmdGotoPalette					2221

// colour controls
#define cmd_r1_l						5101
#define cmd_g1_l						5102
#define cmd_b1_l						5103
#define cmd_r2_l						5104
#define cmd_g2_l						5105
#define cmd_b2_l						5106

#define cmd_r1_r						5111
#define cmd_g1_r						5112
#define cmd_b1_r						5113
#define cmd_r2_r						5114
#define cmd_g2_r						5115
#define cmd_b2_r						5116

#define cmd_r1_ll						5121
#define cmd_g1_ll						5122
#define cmd_b1_ll						5123
#define cmd_r2_ll						5124
#define cmd_g2_ll						5125
#define cmd_b2_ll						5126

#define cmd_r1_rr						5131
#define cmd_g1_rr						5132
#define cmd_b1_rr						5133
#define cmd_r2_rr						5134
#define cmd_g2_rr						5135
#define cmd_b2_rr						5136

#define cmd_r1_lll						5141
#define cmd_g1_lll						5142
#define cmd_b1_lll						5143
#define cmd_r2_lll						5144
#define cmd_g2_lll						5145
#define cmd_b2_lll						5146

#define cmd_r1_rrr						5151
#define cmd_g1_rrr						5152
#define cmd_b1_rrr						5153
#define cmd_r2_rrr						5154
#define cmd_g2_rrr						5155
#define cmd_b2_rrr						5156

// -------------------------------------------------------------------

// definitions
#define	gap 10

#define		edit_x1		(editwin->edit_region->TopLeft()).x
#define		edit_y1		(editwin->edit_region->TopLeft()).y
#define		edit_x2		(editwin->edit_region->BottomRight()).x
#define		edit_y2		(editwin->edit_region->BottomRight()).y
#define		max_edit_x2	(editwin->edit_region->TopLeft()).x+SIZEOF_EDITREGION
#define		max_edit_y2	(editwin->edit_region->TopLeft()).y+SIZEOF_EDITREGION

#define		thumb_x1	(thumbwin->thumbnails->TopLeft()).x
#define		thumb_y1	(thumbwin->thumbnails->TopLeft()).y
#define		thumb_x2	(thumbwin->thumbnails->BottomRight()).x
#define		thumb_y2	(thumbwin->thumbnails->BottomRight()).y

#define		palette_x1	(palwin->palette->TopLeft()).x
#define		palette_y1	(palwin->palette->TopLeft()).y
#define		palette_x2	(palwin->palette->BottomRight()).x
#define		palette_y2	(palwin->palette->BottomRight()).y

#define		tile_x1		(tilewin->tile_grid->TopLeft()).x
#define		tile_y1		(tilewin->tile_grid->TopLeft()).y
#define		tile_x2		(tilewin->tile_grid->BottomRight()).x
#define		tile_y2		(tilewin->tile_grid->BottomRight()).y
#define		max_tile_x2	(tilewin->tile_grid->TopLeft()).x+SIZEOF_TILEAREA
#define		max_tile_y2	(tilewin->tile_grid->TopLeft()).y+SIZEOF_TILEAREA

// -------------------------------------------------------------------

// multiple images
extern	String*		icons_name;				// definition file details that we
extern	DEFFile*	definition_file;		// use to select images to edit
extern	String*		definition_str;			// list of files in definition file

extern	int			number_images;			// number of images in definition file

extern	PCXFile**	pcx_file;				// for loading the images
extern	grImage**	image;					// current images

extern	int			current_image_index;

extern	grImage*	current_image;
extern	grImage*	extended_image;

// -------------------------------------------------------------------

extern	int			image_selected;

extern	int			default_width,
					default_height;

// variables used while dragging
extern	int			first_x,
					first_y,
					second_x, old_second_x,
					second_y, old_second_y;

// currently selected colors
extern	uchar		color1,			// left mouse button
					color2;			// right mouse button

// -------------------------------------------------------------------

// States

// mouse state for continuous movement
extern	bool		mouseisdown;

// are we dragging domething now?
extern	bool		dragison;

// what we are currently dragging
extern	int			dragging;

// input mode, eg. NORMAL_INPUT,TILE_INPUT
extern	int			input_query_mode;

// the way we edit the images, eg. SINGLE or EXTENDED images
extern	int			input_grid_mode;

// output mode, eg. ERASE,DRAW,LINE,RECT,ELLIPSE
extern	int			output_draw_mode;

// fill shapes
extern	int			output_fill_mode;

// tile mode
extern	int			output_tile_mode;

// whether we draw red lines all over the place
extern	bool		output_bounds_mode;

extern	String*		message;

// where we display output_draw_mode and output_fill_mode
extern	int			mode_x, mode_y;

// -------------------------------------------------------------------

// for the undo command
extern	grImage*	undo;
// for working with when resizing, scaling, etc.
extern	grImage*	working;
// for store and retrieve command
extern	grImage*	store;
// for copying and pasting parts of an image
extern	grImage*	clipboard;

extern	bool		undoUsed;
extern	bool		storeUsed;
extern	bool		clipboardUsed;

// -------------------------------------------------------------------

extern	String*		palette_name;
extern	PALFile*	palette_file;
extern	grPalette*	palette;

// all arrows x position
extern	int	leftend_x,leftfast_x,left_x,
			middle_x,
			right_x,rightfast_x,rightend_x;

// top arrows y positions
extern	int	red1_y,green1_y,blue1_y,
// bottom arrows y positions
			blue2_y,green2_y,red2_y;

// storage for the numbers that are displayed in between arrows
extern	uchar		r1,g1,b1;
extern	uchar		r2,g2,b2;

// -------------------------------------------------------------------

void Swap(int* v1, int* v2);

void DrawMessage(int x1,int y1,int x2,int y2,char* message);

// functions to reset the default/maximum/minimum image sizes
void ResetCurrentImage();
void ResetExtendedImage();
void ResetUndo();
void ResetWorking();
void ResetStore();
void ResetClipboard();
void MakeImageDefaultSize(grImage* img);
void ResizeImageIfNecessary(grImage* img);

#endif
