#include        "textfile.hpp"
#include        "palfile.hpp"
#include        "resource.hpp"

#include        "graphics.hpp"

#include        "palettes.hpp"
#include        "fonts.hpp"

#include        "conio.h"
#include        "dos.h"

char*	defaultpalette = "\\game\\utils\\paledit\\paledit.pal";
int		font;

void StartUp()
	{
	printf("\nViewRes, Copyright 1995 by Andrew Evans. 19th July 1995.");
	printf("\n");
	printf("\nUsage:      VIEWRES [<path>]<filename> [[<path>]<parameter>]");
	printf("\n");
	printf("\nExamples:   VIEWRES c:\\game\\pics\\overlay.res");
	printf("\n            VIEWRES overlay.res railway.pal");
	printf("\n            VIEWRES west30.fnt intro.doc");
	printf("\n");
	printf("\nDescription:");
	printf("\n            Displays a resource using the second parameter as for:");
	printf("\n             - Sprites and Bitmaps have a palette file");
	printf("\n             - Fonts have a text document");
	printf("\n             - Texts have a font");
	printf("\n             - Palettes have nothing as yet");
	printf("\n            Default is used if no second parameter is given, for:");
	printf("\n             - Sprites and Bitmaps use \"%s\"",defaultpalette);
	printf("\n             - Fonts display a full ASCII set");
	printf("\n             - Texts use the system font");
	printf("\n             - Palettes still have nothing");
	printf("\n");
	}

void ViewBitmaps(ResourceFile* resourcefile, String* parameter);
void ViewSprites(ResourceFile* resourcefile, String* parameter);
void ViewPalettes(ResourceFile* resourcefile, String* parameter);
void ViewFonts(ResourceFile* resourcefile, String* parameter);

void main(int argc, char** argv)
	{
	if (argc<2)
		{
		printf("\nError! Not Enough Arguments.");
		StartUp();
		exit(1);
		}

	// create a DirString for the name of the resource file
	ResourceFile* resourcefile = new ResourceFile(argv[1]);

	// make sure file exists
	if (!resourcefile->Exists())
		{
		printf("\nError! Resource file \"%s\" does not exist.",argv[1]);
		delete resourcefile;
		StartUp();
		exit(1);
		}

	String* parameter;
	// create a String for the name of the parameter
	if (argc>2)
		parameter = new String(argv[2]);

	if (grInitVesa(gr640x480x256))
		{
		// load a palette to use as default
		PALFile* palfile = new PALFile(defaultpalette);
		grPALETTE = palfile->LoadPalette();
		grSetPalette(grPALETTE,0,255);

		switch (resourcefile->ResourceType())
			{
			case BITMAP_RESOURCE:
				ViewBitmaps(resourcefile, parameter);
				break;

			case SPRITE_RESOURCE:
				ViewSprites(resourcefile, parameter);
				break;

			case PALETTE_RESOURCE:
				ViewPalettes(resourcefile, parameter);
				break;

			case FONT_RESOURCE:
				// load in test font
				font = grInitFont(argv[1]);
				ViewFonts(resourcefile, parameter);
				grExitFont();
				break;

			case TEXT_RESOURCE:
				break;
			}

		free(grPALETTE);
		delete palfile;

		grExitVesa();
		}

	// if we created a parameter
	if (argc>2)
		delete parameter;

	delete resourcefile;

	StartUp();
	}



void ViewBitmaps(ResourceFile* resourcefile, String* parameter)
	{
	// load in the bitmap resources
	long		number = resourcefile->NumRecords();
	grImage**	data = (grImage**)resourcefile->ReadAllRecords();

	if (data)
		{
		int xx=0,yy=0,maxheight=0;

		// display the data in a tile fashion
		for (int i=0;i<number;i++)
			{
			// get width and height
			int width = data[i]->hdr.width;
			int height = data[i]->hdr.height;

			// update maxheight
			if (height>maxheight) maxheight = height;

			// check for overflow of the screen boundaries
			if (xx+width > grMaxx) { xx=0; yy+=maxheight; maxheight=height; }
			if (yy+height > grMaxy) { getch(); grClearScreen(grBLACK); xx=0; yy=0; maxheight=height; }

			// put the image on the screen
			grPutImage(xx,yy,width+xx-1,height+yy-1,data[i]);
			grBlitRegion(xx,yy,width+xx-1,height+yy-1);

			// calculate how many steps across and down we will plot
			xx += width;
			}
		// free resources and bitmap file
		data = (grImage**) resourcefile->FreeAllRecords((char**)data);
		}

	// display a summary
	grDrawString(0,470,"Number of Bitmaps == %d",number);
	grBlitRegion(0,470,grMaxx,grMaxy);
	getch();
	}



#define         animate_backward        -1
#define         animate_still           0
#define         animate_forward         1

void ViewSprites(ResourceFile* resourcefile, String* parameter)
	{
/*	long		number = resourcefile->NumRecords();
	grSprite**	data = (grSprite**)resourcefile->ReadAllRecords();

	// display a summary
	grDrawString(0,470,"Number of Sprites %d",number);

	if (data)
		{
		unsigned delay_time = 50;

		int fh = grFontHeight();
		// display a background
		grDrawString(130,100+fh*0,    "This key list is not redrawn ever.");
		grDrawString(100,100+fh*2,"Esc Q or q     Quit Program");
		grDrawString(100,100+fh*3,"Space          Skip to next sprite");

		grDrawString(130,100+fh*5,    "On the Number Pad");
		grDrawString(100,100+fh*7," + sets animation forward");
		grDrawString(100,100+fh*8,"CR freezes the animation");
		grDrawString(100,100+fh*9," - sets animation backward");

		grDrawString(100,100+fh*10," / subtracts 10 milliseconds from the delay time");
		grDrawString(100,100+fh*11," * adds 10 milliseconds to the delay time");

		grDrawString(100,100+fh*12," 7 8 9");
		grDrawString(100,100+fh*13," 4   6  moves the sprite that direction");
		grDrawString(100,100+fh*14," 1 2 3");
		grDrawString(100,100+fh*16," 5 freezes animation and movement");

		grBlitScreen();

		bool quit = false;

		for (int i=0; i<number; i++)
			{
			// erase old text
			grDrawFilledRect(0,460,600,469,grBLACK);
			// display which resource
			grDrawString(0,460,"Sprite number %d",i);

			bool finished = false;
			int dx = 0, dy = 0;
			char ch = 'z';

			// assign spr as the i'th of data
			grSprite* spr = data[i];

			// initialise the sprite
			spr->hdr.x = 300;
			spr->hdr.y = 200;
			spr->hdr.current_frame = 0;
			spr->hdr.state = animate_forward;

			// give it something for first time in loop
			grSaveUnderSprite(spr);

			while(!finished)
				{
				// for wraparound animation
				if (spr->hdr.current_frame >= spr->hdr.number_frames)
					spr->hdr.current_frame = 0;

				if (spr->hdr.current_frame < 0)
					spr->hdr.current_frame = spr->hdr.number_frames - 1;

				// erase the previously drawn sprite
				grEraseSprite(spr);

				// move the sprite by delta x and y
				grMoveSprite(spr,dx,dy);

				// erase old text
				grDrawFilledRect(0,0,639,grFontHeight(),grBLACK);
				// display sprite values
				grDrawString(0,0,"Delay %d   Delta X %d Y%d",delay_time,dx,dy);

				// get the background at the new position
				grSaveUnderSprite(spr);

				// draw the sprite in its current animation frame
				grDrawSprite(spr);

				// wait a while so the user can see it
				delay(delay_time);

				// change the current animation frame
				spr->hdr.current_frame += (int)spr->hdr.state;

				// check on the user
				if (kbhit())
					{
					ch = getch();

					switch (ch)
						{
						// check for an escape, quit or continue signal
						case 27 :
						case 'Q':
						case 'q': quit = true;
						case 32 : finished = true; break;

						// check for animation direction changes
						case '+': spr->hdr.state = animate_forward; break;
						case 13 : spr->hdr.state = animate_still; break;
						case '-': spr->hdr.state = animate_backward; break;

						// check for delay changes
						case '/': delay_time -= 10; break;
						case '*': delay_time += 10; break;

						// check for directions
						case '7': dx = -1; dy = -1; break;
						case '8': dx =  0; dy = -1; break;
						case '9': dx =  1; dy = -1; break;
						case '6': dx =  1; dy =  0; break;
						case '3': dx =  1; dy =  1; break;
						case '2': dx =  0; dy =  1; break;
						case '1': dx = -1; dy =  1; break;
						case '4': dx = -1; dy =  0; break;

						// stop moving sprite
						case '5': dx =  0; dy =  0;
						spr->hdr.state = animate_still; break;

						default: break;
						}
					}
				}
			grEraseSprite(spr);

			if (quit) i = number;
			}
		data = (grSprite**) resourcefile->FreeAllRecords((char**)data);
		}
*/	}



void ViewFonts(ResourceFile* resourcefile, String* parameter)
	{
	// get test font's height
	grSetFont(font);
	int height = grFontHeight();

	grClearScreen(grBLACK);

	// if no parameter was passed then print out ascii set
	if (!parameter)
		{
		grSetFont(0);   // 0 is the system font
		int systemheight = grFontHeight();
		grDrawString(0,0,"Height == %u",height);

		grSetFont(font);

		grDrawString(0,systemheight*3+height*4," !\"#$%&'()*+,-./0123456789:;<=>?@");
		grDrawString(0,systemheight*3+height*5,"ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`");
		grDrawString(0,systemheight*3+height*6,"abcdefghijklmnopqrstuvwxyz{|}~");

		grBlitScreen();
		}
	else
		{
		// create a cursor position
		int     cx=0, cy=0;

		// open a text file
		TextFile* document = new TextFile(parameter);

		// malloc a string
		char* doc = (char*) malloc(document->FileSize()+1);

		// seek to the beginning of the doc file
		document->SeekTOF();

		// read the file
		document->Read(doc,document->FileSize());

		char* ch = (char*) malloc(3);

		for (int i=0; i<document->FileSize(); i++)
			{
			// if we go off the bottom of the page
			if (cy > 479-grFontHeight())
				{
				getch();

				// go to a new page
				cx = 0;
				cy = 0;
				grClearScreen(grBLACK);
				}
			// if we go off the side of the page
			if (cx > 639-grFontCharWidth('W'))
				{
				grBlitRegion(0,cy,grMaxx,cy+grFontHeight());

				// go to a new line
				cx = 0;
				cy += grFontHeight();
				}

			ch[0] = doc[i];
			// if this is a displayable character then display it
			if ( (ch[0]>31) && (ch[0]<128) )
				{
				ch[1] = '\0';
				grDrawString(cx, cy, ch);
				cx += grFontCharWidth(ch[0]);
				}
			else
				{
				switch (ch[0])
					{
					case 13:	// a carriage return
						grBlitRegion(0,cy,grMaxx,cy+grFontHeight());
						cx = 0;
						cy += grFontHeight();
						break;
					case 8:     // a tab
						cx += 40;
						break;
					default:
						break;
					}
				}
			}
		grBlitScreen();
		free(ch);
		free(doc);
		delete document;
		}
	getch();
	}



void ViewPalettes(ResourceFile* resourcefile, String* parameter)
	{
	// display a tile to show off palettes
	for (int i=0;i<256;i++)
		{
		// get width and height
		int width = 32;
		int height = 32;

		// calculate how many steps across and down we will plot
		int xx = i % (640 / width);
		int yy = i / (640 / width);

		// if plot doesnt go beyond the bottom of the screen then plot
		grDrawFilledRect(xx*width,yy*height,width+xx*width,height+yy*height,i);
		}

	long number = resourcefile->NumRecords();
	grPalette** data = (grPalette**)resourcefile->ReadAllRecords();

	// display a summary
	grDrawString(0,470,"Number of Palettes == %d",number);
	grBlitScreen();

	if (data)
		{
		for (int j=0; j<number; j++)
			{
			// erase old text
			grDrawFilledRect(0,460,grMaxx,469,grBLACK);
			grDrawString(0,460,"Palette number == %d",j);
			grBlitRegion(0,460,grMaxx,469);

			grSetPalette(data[j],0,data[j]->hdr.numentries-1);
			getch();
			}
		// free resources and palette file
		data = (grPalette**) resourcefile->FreeAllRecords((char**)data);
		}
	}

